{*****************************************************************************}
{                                                                             }
{   This file is under development for the LKAccounts components Library      }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Components to provide financial forecasting for the LKNominalLedger    }
{                                                                             }
{*****************************************************************************}


unit LKAccountsForecast;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, Forms, Dialogs,
  ImgList, StdCtrls, ComCtrls, Contnrs, Variants, Menus, ShellApi,
  ActnList, DB, DBTables, DBClient, SqlExpr, DBXCommon, Provider, Registry,
  DBCtrls, DBGrids, Math,
  LKBaseComponent, LKRecordsComponents, LKDefine, LKAccountsComponents;

type

  TLKForecastEditor = class;
  TLKLedgerNominalProfitAndBalancesForecast = class;

  TLKForecastEditor = class abstract(
      TLKProfitAndBalancesForecastContainedDocuments)
  private
    FTblView: TLKTableView;
    function GettmNominalCode: string;
    function GettmPeriodIndex: Integer;
    function GettmValue: Currency;
    procedure SettmNominalCode(const Value: string);
    procedure SettmPeriodIndex(Value: Integer);
    procedure SettmValue(Value: Currency);
  protected
    procedure CreateSubComponents; override;
    procedure SetContainedComponentNames; override;
    procedure TblViewBuild; virtual; abstract;
  public
    procedure DlgViewDatasetViewExecute;
    procedure TblViewRefresh(Rebuild: boolean);
    property TblView: TLKTableView read FTblView;
    property tmNominalCode: string read GettmNominalCode write SettmNominalCode;
    property tmPeriodIndex: Integer read GettmPeriodIndex write
        SettmPeriodIndex;
    property tmValue: Currency read GettmValue write SettmValue;
  end;

  TLKForecastSales = class(TLKForecastEditor)
  protected
    procedure TblViewBuild; override;
  public
    constructor Create(AOwner: TComponent); override;
  end;

  TLKProfitForecast = class(TLKProfitAndBalancesSection)
  protected
    procedure DoDetailBeforePost; override;
    procedure TblViewBuild; override;
  public
    constructor Create(AOwner: TComponent); override;
  end;

  TLKBalancesForecast = class(TLKProfitAndBalancesSection)
  protected
    procedure TblViewBuild; override;
  public
    constructor Create(AOwner: TComponent); override;
  end;

  TLKCashFlowForecast = class(TLKProfitAndBalancesSection)
  protected
    procedure TblViewBuild; override;
  public
    constructor Create(AOwner: TComponent); override;
  end;

  TLKForecastDates = class(TLKProfitAndBalancesForecastContainedDocuments)
  private
    function GettmPeriodCaption: string;
    function GettmPeriodIndex: Integer;
    procedure SettmPeriodCaption(const Value: string);
    procedure SettmPeriodIndex(Value: Integer);
  public
    constructor Create(AOwner: TComponent); override;
    property tmPeriodCaption: string read GettmPeriodCaption write
        SettmPeriodCaption;
    property tmPeriodIndex: Integer read GettmPeriodIndex write
        SettmPeriodIndex;
  end;

  TLKProfitAndBalancesSection = class(TLKForecastEditor)
  end;

  TLKLedgerNominalProfitAndBalancesForecast = class(TLKRecordsAccounts)
  private
    FBalancesForecast: TLKBalancesForecast;
    FCashFlowForecast: TLKCashFlowForecast;
    FForecastDates: TLKForecastDates;
    FForecastName: string;
    FNominalLedger: TLKLedgerNominal;
    FProfitForecast: TLKProfitForecast;
    FSalesForecast: TLKForecastSales;
    function doInsertNewRecord: TModalResult;
    procedure ForecastDatesRebuild;
    function GettdDescription: string;
    function GettdNominalCode: string;
    function GettmDefaultCreditorsDays: Integer;
    function GettmDefaultDebtorsDays: Integer;
    function GettmForecastPeriods: Integer;
    function GettmForecastStartDate: TDateTime;
    procedure SetForecastName(const Value: string);
    procedure SetNominalLedger(Value: TLKLedgerNominal);
    procedure SettdDescription(const Value: string);
    procedure SettdNominalCode(const Value: string);
    procedure SettmDefaultCreditorsDays(Value: Integer);
    procedure SettmDefaultDebtorsDays(Value: Integer);
    procedure SettmForecastPeriods(Value: Integer);
    procedure SettmForecastStartDate(Value: TDateTime);
  protected
    procedure CreateViewFormRecord; override;
    procedure SetActive(Value: Boolean); override;
    procedure SetContainedComponentNames; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure CreateViewForm; override;
    procedure DlgForecastWizardExecute;
    procedure DlgSalesForecastExecute;
    function DlgViewExecute: TModalResult; override;
    procedure doCreditorsDaysWizExecute(Sender: TObject);
    procedure doDebtorsDaysWizExecute(Sender: TObject);
    procedure ForecastUpdate;
    function LoadFromFile: Boolean; override;
    function RecordNew: TModalResult; override;
    procedure TblViewsRefresh(Rebuild: boolean);
    property ForecastName: string read FForecastName write SetForecastName;
    property NominalLedger: TLKLedgerNominal read FNominalLedger write
        SetNominalLedger;
    property SalesForecast: TLKForecastSales read FSalesForecast;
    property tdDescription: string read GettdDescription write SettdDescription;
    property tdNominalCode: string read GettdNominalCode write SettdNominalCode;
    property tmDefaultCreditorsDays: Integer read GettmDefaultCreditorsDays
        write SettmDefaultCreditorsDays;
    property tmDefaultDebtorsDays: Integer read GettmDefaultDebtorsDays write
        SettmDefaultDebtorsDays;
    property tmForecastPeriods: Integer read GettmForecastPeriods write
        SettmForecastPeriods;
    property tmForecastStartDate: TDateTime read GettmForecastStartDate write
        SettmForecastStartDate;
  published
    property ACGroup;
    property BalancesForecast: TLKBalancesForecast read FBalancesForecast;
    property CashFlowForecast: TLKCashFlowForecast read FCashFlowForecast;
    property ForecastDates: TLKForecastDates read FForecastDates;
    property ProfitForecast: TLKProfitForecast read FProfitForecast;
  end;

  TLKProfitAndBalancesForecastContainedDocuments = class(
      TLKContainedRecordsAccounts)
  private
    function GetContainerComponent: TLKLedgerNominalProfitAndBalancesForecast;
  public
    constructor Create(AOwner: TComponent); override;
    property ContainerComponent: TLKLedgerNominalProfitAndBalancesForecast read
        GetContainerComponent;
  end;


implementation

uses
  LKProfitAndBalancesForecastWizardDlg,
  LKProfitAndBalancesForecastDlg, LKProfitAndBalancesForecastRecordDlg;

{
****************************** TLKForecastEditor *******************************
}
procedure TLKForecastEditor.CreateSubComponents;
begin
  inherited CreateSubComponents;
  FTblView := TLKTableView.Create(Self);
end;

procedure TLKForecastEditor.DlgViewDatasetViewExecute;
begin
  //
end;

function TLKForecastEditor.GettmNominalCode: string;
begin
  Result := TblMaster.FieldByName('NOMINAL_CODE').AsString
end;

function TLKForecastEditor.GettmPeriodIndex: Integer;
begin
  Result := GetInteger(TblMaster.FieldByName('PERIOD_INDEX'));
end;

function TLKForecastEditor.GettmValue: Currency;
begin
  Result := GetCurrency(TblMaster.FieldByName('VALUE'));
end;

procedure TLKForecastEditor.SetContainedComponentNames;
begin
  inherited SetContainedComponentNames;
  FTblView.Name := Name + 'TblView';
end;

procedure TLKForecastEditor.SettmNominalCode(const Value: string);
begin
  TblMaster.FieldByName('NOMINAL_CODE').AsString := Value;
end;

procedure TLKForecastEditor.SettmPeriodIndex(Value: Integer);
begin
  TblMaster.FieldByName('PERIOD_INDEX').AsInteger := Value;
end;

procedure TLKForecastEditor.SettmValue(Value: Currency);
begin
  TblMaster.FieldByName('VALUE').AsCurrency := Value;
end;

procedure TLKForecastEditor.TblViewRefresh(Rebuild: boolean);
var
  Tot: Currency;
  F: TField;
begin
  TblView.EmptyDataset;
  if Rebuild then
  begin
    TblView.Fields.Clear;
    TblViewBuild;
  end;
  MasterComponent := ContainerComponent;
  MasterTable := tbDetail;
  MasterFields := 'NOMINAL_CODE';
  Filter := 'PERIOD_INDEX > 0';
  Filtered := True;
  try
    if ContainerComponent.TblDetail.FindFirst then
      repeat
        if FindFirst then
        begin
          TblView.InsertRecord(
            [ContainerComponent.tdNominalCode,
              ContainerComponent.tdDescription]);
          TblView.Edit;
          Tot := 0;
          repeat
            Tot := Tot + tmValue;
            TblView.Fields[tmPeriodIndex + 1].AsCurrency := tmValue;
          until not FindNext;
          F := TblView.FindField('TOTAL');
          if Assigned(F) then
            F.AsCurrency := Tot;
          TblView.Post;
        end;
      until not ContainerComponent.TblDetail.FindNext;
  finally
    MasterComponent := nil;
    MasterFields := '';
    Filter := '';
    Filtered := False;
  end;
end;

{
******************************* TLKForecastSales *******************************
}
constructor TLKForecastSales.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Name := 'SalesForecast';
  NameFriendly := 'Sales Forecast';
  Filter := 'IS_TURNOVER = True';
end;

procedure TLKForecastSales.TblViewBuild;
var
  s: string;
begin
  inherited TblViewBuild;
  if not Assigned(TblView) then
    Exit;
  TblMaster.CloneCursor(ContainerComponent.ProfitForecast.TblMaster, True);
  s := 'PERIOD_END > ' + chr(39) +
    DateToStr(ContainerComponent.tmForecastStartDate) + chr(39);
  with ContainerComponent.ForecastDates do
  begin
    Filter := s;
    Filtered := True;
  end;
  try
    with TStringField.Create(Self) do
    begin
      Name := 'ViewBalancesNOMINAL_CODE';
      FieldKind := fkData;
      FieldName := 'NOMINAL_CODE';
      Size := 8;
      DataSet := TblView;
      Required := True;
    end; //ID
    with TStringField.Create(Self) do
    begin
      Name := 'ViewBalancesDESCRIPTION';
      FieldKind := fkData;
      FieldName := 'DESCRIPTION';
      Size := 40;
      DataSet := TblView;
    end; //ID
    with ContainerComponent.ForecastDates do
      if TblMaster.FindFirst then
        repeat
          with TFMTBCDField.Create(Self) do
          begin
            Name := 'ViewBalancesPERIOD_' + IntToStr(tmPeriodIndex);
            FieldKind := fkData;
            FieldName := 'PERIOD_' + IntToStr(tmPeriodIndex);
            DisplayLabel := tmPeriodCaption;
            Size := 18;
            Precision := 4;
            Currency := True;
            DataSet := TblView;
          end; //IDwith TFMTBCDField.Create(Self) do
        until not TblMaster.FindNext;
    IndexFieldNames := 'NOMINAL_CODE';
    TblView.CreateDataSet;
  finally
    with ContainerComponent.ForecastDates do
    begin
      Filter := '';
      Filtered := False;
    end;
  end;
end;

{
****************************** TLKProfitForecast *******************************
}
constructor TLKProfitForecast.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  TblMaster.TableName := 'ProfitForecast';
  with TStringField.Create(Self) do
  begin
    Name := 'ProfitForecastNOMINAL_CODE';
    FieldKind := fkData;
    FieldName := 'NOMINAL_CODE';
    Size := 8;
    DataSet := TblMaster;
    Required := True;
  end; //ID
  with TDateField.Create(Self) do
  begin
    Name := 'ProfitForecastPERIOD_END';
    FieldKind := fkData;
    FieldName := 'PERIOD_END';
    DataSet := TblMaster;
    Required := True;
  end; //ID
  with TIntegerField.Create(Self) do
  begin
    Name := 'ProfitForecastPERIOD_INDEX';
    FieldKind := fkData;
    FieldName := 'PERIOD_INDEX';
    DataSet := TblMaster;
  end; //ID
  with TBooleanField.Create(Self) do
  begin
    Name := 'ProfitForecastIS_TURNOVER';
    FieldKind := fkData;
    FieldName := 'IS_TURNOVER';
    DataSet := TblMaster;
  end; //ID
  with TFMTBCDField.Create(Self) do
  begin
    Name := 'ProfitForecastADJUSTED_HISTORIC';
    FieldKind := fkData;
    FieldName := 'ADJUSTED_HISTORIC';
    Size := 18;
    Precision := 4;
    Currency := True;
    DataSet := TblMaster;
  end; //ID
  with TFMTBCDField.Create(Self) do
  begin
    Name := 'ProfitForecastVALUE';
    FieldKind := fkData;
    FieldName := 'VALUE';
    Size := 18;
    Precision := 4;
    Currency := True;
    DataSet := TblMaster;
  end; //ID
  IndexFieldNames := 'NOMINAL_CODE;PERIOD_END';
  //MasterFields  := 'PERIOD_END';
  //TblMaster.CreateDataset;

  TblDetail.TableName := 'ProfitForecastHistory';
  with TStringField.Create(Self) do
  begin
    Name := 'ProfitForecastHistoryNOMINAL_CODE';
    FieldKind := fkData;
    FieldName := 'NOMINAL_CODE';
    Size := 8;
    DataSet := TblDetail;
    Required := True;
  end; //ID
  with TDateField.Create(Self) do
  begin
    Name := 'ProfitForecastHistoryPERIOD_END';
    FieldKind := fkData;
    FieldName := 'PERIOD_END';
    DataSet := TblDetail;
    Required := True;
  end; //ID
  with TDateField.Create(Self) do
  begin
    Name := 'ProfitForecastHistoryHISTORY_PERIOD_END';
    FieldKind := fkData;
    FieldName := 'HISTORY_PERIOD_END';
    DataSet := TblDetail;
    Required := True;
  end; //ID
  with TFMTBCDField.Create(Self) do
  begin
    Name := 'ProfitForecastHistoryHISTORIC_VALUE';
    FieldKind := fkData;
    FieldName := 'HISTORIC_VALUE';
    Size := 18;
    Precision := 4;
    Currency := True;
    DataSet := TblDetail;
  end; //ID
  with TFloatField.Create(Self) do
  begin
    Name := 'ProfitForecastHistoryINFLATION_INDEX';
    FieldKind := fkData;
    FieldName := 'INFLATION_INDEX';
    DataSet := TblDetail;
  end; //ID
  with TFMTBCDField.Create(Self) do
  begin
    Name := 'ProfitForecastHistoryCALCULATED_VALUE';
    FieldKind := fkData;
    FieldName := 'CALCULATED_VALUE';
    Size := 18;
    Precision := 4;
    Currency := True;
    DataSet := TblDetail;
  end; //ID
  TblDetail.IndexFieldNames := 'NOMINAL_CODE;PERIOD_END;HISTORY_PERIOD_END';
  TblDetail.MasterFields := 'NOMINAL_CODE;PERIOD_END';
  //TblDetail.CreateDataSet;
end;

procedure TLKProfitForecast.DoDetailBeforePost;
begin
  inherited DoDetailBeforePost;
  with TblDetail do
    FieldByName('CALCULATED_VALUE').AsCurrency :=
      GetFloat(FieldByName('INFLATION_INDEX')) *
        GetCurrency(FieldByName('HISTORIC_VALUE'));
end;

procedure TLKProfitForecast.TblViewBuild;
var
  S: string;
begin
  inherited TblViewBuild;
  if not Assigned(TblView) then
    Exit;
  S := 'PERIOD_END > ' + chr(39) +
    DateToStr(ContainerComponent.tmForecastStartDate) + chr(39);
  with ContainerComponent.ForecastDates do
  begin
    Filter := S;
    Filtered := True;
  end;
  try
    with TStringField.Create(Self) do
    begin
      Name := 'ViewProfitForecastNOMINAL_CODE';
      FieldKind := fkData;
      FieldName := 'NOMINAL_CODE';
      Size := 8;
      DataSet := TblView;
      Required := True;
    end; //ID
    with TStringField.Create(Self) do
    begin
      Name := 'ViewProfitForecastDESCRIPTION';
      FieldKind := fkData;
      FieldName := 'DESCRIPTION';
      Size := 40;
      DataSet := TblView;
    end; //ID
    with ContainerComponent.ForecastDates do
      if TblMaster.FindFirst then
        repeat
          with TFMTBCDField.Create(Self) do
          begin
            Name := 'ViewProfitForecastPERIOD_' + IntToStr(tmPeriodIndex);
            FieldKind := fkData;
            FieldName := 'PERIOD_' + IntToStr(tmPeriodIndex);
            DisplayLabel := tmPeriodCaption;
            Size := 18;
            Precision := 4;
            Currency := True;
            DataSet := TblView;
          end; //ID
        until not TblMaster.FindNext;
    with TFMTBCDField.Create(Self) do
    begin
      Name := 'ViewProfitForecastTOTAL';
      FieldKind := fkData;
      FieldName := 'TOTAL';
      Size := 18;
      Precision := 4;
      Currency := True;
      DataSet := TblView;
    end; //ID
    IndexFieldNames := 'NOMINAL_CODE';
    TblView.CreateDataSet;
  finally
    with ContainerComponent.ForecastDates do
    begin
      Filter := '';
      Filtered := False;
    end;
  end;
end;

{
***************************** TLKBalancesForecast ******************************
}
constructor TLKBalancesForecast.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  TblMaster.TableName := 'Balances';
  with TStringField.Create(Self) do
  begin
    Name := 'BalancesNOMINAL_CODE';
    FieldKind := fkData;
    FieldName := 'NOMINAL_CODE';
    Size := 8;
    DataSet := TblMaster;
    Required := True;
  end; //ID
  with TDateField.Create(Self) do
  begin
    Name := 'BalancesPERIOD_END';
    FieldKind := fkData;
    FieldName := 'PERIOD_END';
    DataSet := TblMaster;
    Required := True;
  end; //ID
  with TIntegerField.Create(Self) do
  begin
    Name := 'BalancesPERIOD_INDEX';
    FieldKind := fkData;
    FieldName := 'PERIOD_INDEX';
    DataSet := TblMaster;
  end; //ID
  with TFMTBCDField.Create(Self) do
  begin
    Name := 'BalancesVALUE';
    FieldKind := fkData;
    FieldName := 'VALUE';
    Size := 18;
    Precision := 4;
    Currency := True;
    DataSet := TblMaster;
  end; //ID
  IndexFieldNames := 'NOMINAL_CODE;PERIOD_END';
  //TblMaster.CreateDataSet;
end;

procedure TLKBalancesForecast.TblViewBuild;
var
  s: string;
begin
  inherited TblViewBuild;
  if not assigned(TblView) then
    exit;
  s := 'PERIOD_END > ' + chr(39) +
    DateToStr(ContainerComponent.tmForecastStartDate) + chr(39);
  with ContainerComponent.ForecastDates do
  begin
    Filter := s;
    Filtered := True;
  end;
  try
    with TStringField.Create(Self) do
    begin
      Name := 'ViewBalancesNOMINAL_CODE';
      FieldKind := fkData;
      FieldName := 'NOMINAL_CODE';
      Size := 8;
      DataSet := TblView;
      Required := True;
    end; //ID
    with TStringField.Create(Self) do
    begin
      Name := 'ViewBalancesDESCRIPTION';
      FieldKind := fkData;
      FieldName := 'DESCRIPTION';
      Size := 40;
      DataSet := TblView;
    end; //ID
    with ContainerComponent.ForecastDates do
      if TblMaster.FindFirst then
        repeat
          with TFMTBCDField.Create(Self) do
          begin
            Name := 'ViewBalancesPERIOD_' + IntToStr(tmPeriodIndex);
            FieldKind := fkData;
            FieldName := 'PERIOD_' + IntToStr(tmPeriodIndex);
            DisplayLabel := tmPeriodCaption;
            Size := 18;
            Precision := 4;
            Currency := True;
            DataSet := TblView;
          end; //IDwith TFMTBCDField.Create(Self) do
        until not TblMaster.FindNext;
    IndexFieldNames := 'NOMINAL_CODE';
    TblView.CreateDataSet;
  finally
    with ContainerComponent.ForecastDates do
    begin
      Filter := '';
      Filtered := False;
    end;
  end;
end;

{
***************************** TLKCashFlowForecast ******************************
}
constructor TLKCashFlowForecast.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  TblMaster.TableName := 'CashFlow';
  with TStringField.Create(Self) do
  begin
    Name := 'CashFlowNOMINAL_CODE';
    FieldKind := fkData;
    FieldName := 'NOMINAL_CODE';
    Size := 8;
    DataSet := TblMaster;
    Required := True;
  end; //ID
  with TDateField.Create(Self) do
  begin
    Name := 'CashFlowPERIOD_END';
    FieldKind := fkData;
    FieldName := 'PERIOD_END';
    DataSet := TblMaster;
    Required := True;
  end; //ID
  with TIntegerField.Create(Self) do
  begin
    Name := 'CashFlowPERIOD_INDEX';
    FieldKind := fkData;
    FieldName := 'PERIOD_INDEX';
    DataSet := TblMaster;
  end; //ID
  with TFMTBCDField.Create(Self) do
  begin
    Name := 'CashFlowCALCULATED_VALUE';
    FieldKind := fkData;
    FieldName := 'CALCULATED_VALUE';
    Size := 18;
    Precision := 4;
    Currency := True;
    DataSet := TblMaster;
  end; //ID
  with TFMTBCDField.Create(Self) do
  begin
    Name := 'CashFlowVALUE';
    FieldKind := fkData;
    FieldName := 'VALUE';
    Size := 18;
    Precision := 4;
    Currency := True;
    DataSet := TblMaster;
  end; //ID
  IndexFieldNames := 'NOMINAL_CODE;PERIOD_END';
  //TblMaster.CreateDataSet;
end;

procedure TLKCashFlowForecast.TblViewBuild;
var
  S: string;
begin
  inherited TblViewBuild;
  if not Assigned(TblView) then
    Exit;
  S := 'PERIOD_END > ' + chr(39) +
    DateToStr(ContainerComponent.tmForecastStartDate) + chr(39);
  with ContainerComponent.ForecastDates do
  begin
    Filter := S;
    Filtered := True;
  end;
  try
    with TStringField.Create(Self) do
    begin
      Name := 'ViewCashFlowNOMINAL_CODE';
      FieldKind := fkData;
      FieldName := 'NOMINAL_CODE';
      Size := 8;
      DataSet := TblView;
      Required := True;
    end; //ID
    if not Assigned(TblView) then
      Exit;
    with TStringField.Create(Self) do
    begin
      Name := 'ViewCashFlowDESCRIPTION';
      FieldKind := fkData;
      FieldName := 'DESCRIPTION';
      Size := 40;
      DataSet := TblView;
    end; //ID
    with ContainerComponent.ForecastDates do
      if TblMaster.FindFirst then
        repeat
          with TFMTBCDField.Create(Self) do
          begin
            Name := 'ViewCashFlowPERIOD_' + IntToStr(tmPeriodIndex);
            FieldKind := fkData;
            FieldName := 'PERIOD_' + IntToStr(tmPeriodIndex);
            DisplayLabel := tmPeriodCaption;
            Size := 18;
            Precision := 4;
            Currency := True;
            DataSet := TblView;
          end; //ID
        until not TblMaster.FindNext;
    with TFMTBCDField.Create(Self) do
    begin
      Name := 'ViewCashFlowTOTAL';
      FieldKind := fkData;
      FieldName := 'TOTAL';
      Size := 18;
      Precision := 4;
      Currency := True;
      DataSet := TblView;
    end; //ID
    IndexFieldNames := 'NOMINAL_CODE';
    TblView.CreateDataSet;
  finally
    with ContainerComponent.ForecastDates do
    begin
      Filter := '';
      Filtered := False;
    end;
  end;
end;

{
******************************* TLKForecastDates *******************************
}
constructor TLKForecastDates.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  TblMaster.TableName := 'ForecastDates';
  with TDateField.Create(Self) do
  begin
    Name := 'ForecastDatesPERIOD_END';
    FieldKind := fkData;
    FieldName := 'PERIOD_END';
    DataSet := TblMaster;
    Required := True;
  end; //ID
  with TIntegerField.Create(Self) do
  begin
    Name := 'ForecastDatesPERIOD_INDEX';
    FieldKind := fkData;
    FieldName := 'PERIOD_INDEX';
    DataSet := TblMaster;
  end; //ID
  IndexFieldNames := 'PERIOD_END';
  with TStringField.Create(Self) do
  begin
    Name := 'ForecastDatesNOMINAL_CODE';
    FieldKind := fkData;
    FieldName := 'NOMINAL_CODE';
    Size := 11;
    DataSet := TblMaster;
    Required := True;
  end; //ID
  //TblMaster.CreateDataset;
end;

function TLKForecastDates.GettmPeriodCaption: string;
begin
  Result := TblMaster.FieldByName('PERIOD_CAPTION').AsString
end;

function TLKForecastDates.GettmPeriodIndex: Integer;
begin
  Result := GetInteger(TblMaster.FieldByName('PERIOD_INDEX'));
end;

procedure TLKForecastDates.SettmPeriodCaption(const Value: string);
begin
  TblMaster.FieldByName('PERIOD_CAPTION').AsString := Value;
end;

procedure TLKForecastDates.SettmPeriodIndex(Value: Integer);
begin
  TblMaster.FieldByName('PERIOD_INDEX').AsInteger := Value;
end;

{
****************** TLKLedgerNominalProfitAndBalancesForecast *******************
}
constructor TLKLedgerNominalProfitAndBalancesForecast.Create(AOwner:
    TComponent);
begin
  inherited Create(AOwner);
  FSalesForecast := TLKForecastSales.Create(Self);
  FBalancesForecast := TLKBalancesForecast.Create(Self);
  FCashFlowForecast := TLKCashFlowForecast.Create(Self);
  FForecastDates := TLKForecastDates.Create(Self);
  FProfitForecast := TLKProfitForecast.Create(Self);
  SetName(Name);
  TblMaster.TableName := 'ForecastProperties';
  with TStringField.Create(Self) do
  begin
    Name := 'ForecastPropertiesFORECAST_REFERENCE';
    FieldKind := fkData;
    FieldName := 'FORECAST_REFERENCE';
    Size := 8;
    DataSet := TblMaster;
  end; //ID
  with TStringField.Create(Self) do
  begin
    Name := 'ForecastPropertiesFORECAST_NAME';
    FieldKind := fkData;
    FieldName := 'FORECAST_NAME';
    Size := 60;
    DataSet := TblMaster;
  end; //ID
  with TDateField.Create(Self) do
  begin
    Name := 'ForecastPropertiesSTART_DATE';
    FieldKind := fkData;
    FieldName := 'START_DATE';
    DataSet := TblMaster;
  end; //ID
  with TIntegerField.Create(Self) do
  begin
    Name := 'ForecastPropertiesFORECAST_PERIODS';
    FieldKind := fkData;
    FieldName := 'FORECAST_PERIODS';
    DataSet := TblMaster;
  end; //ID
  with TIntegerField.Create(Self) do
  begin
    Name := 'ForecastPropertiesDEFAULT_DEBTORS_DAYS';
    FieldKind := fkData;
    FieldName := 'DEFAULT_DEBTORS_DAYS';
    DataSet := TblMaster;
  end; //ID
  with TIntegerField.Create(Self) do
  begin
    Name := 'ForecastPropertiesDEFAULT_CREDITORS_DAYS';
    FieldKind := fkData;
    FieldName := 'DEFAULT_CREDITORS_DAYS';
    DataSet := TblMaster;
  end; //ID
  IndexFieldNames := 'FORECAST_REFERENCE';
  //TblMaster.CreateDataSet;

  TblDetail.TableName := 'ForecastPropertiesDtl';
  TblDetail.IndexFieldNames := 'FORECAST_REFERENCE;NOMINAL_CODE';
  TblDetail.MasterFields := 'FORECAST_REFERENCE';
  with TStringField.Create(Self) do
  begin
    Name := 'ForecastPropertiesDtlFORECAST_REFERENCE';
    FieldKind := fkData;
    FieldName := 'FORECAST_REFERENCE';
    Size := 8;
    DataSet := TblDetail;
  end; //ID
  with TStringField.Create(Self) do
  begin
    Name := 'ForecastPropertiesDtlNOMINAL_CODE';
    FieldKind := fkData;
    FieldName := 'NOMINAL_CODE';
    Size := 8;
    DataSet := TblDetail;
    Required := True;
  end; //ID
  with TStringField.Create(Self) do
  begin
    Name := 'ForecastPropertiesDtlDESCRIPTION';
    FieldKind := fkData;
    FieldName := 'DESCRIPTION';
    Size := 40;
    DataSet := TblDetail;
  end; //ID
  with TFloatField.Create(Self) do
  begin
    Name := 'ForecastPropertiesDtlVARIABLE_COST_PERCENT';
    FieldKind := fkData;
    FieldName := 'VARIABLE_COST_PERCENT';
    {Size := 18;
    Precision := 4;}
    Currency := True;
    DataSet := TblDetail;
  end; //ID
  with TIntegerField.Create(Self) do
  begin
    Name := 'ForecastPropertiesDtlCREDIT_DAYS';
    FieldKind := fkData;
    FieldName := 'AVE_CREDIT_DAYS';
    DataSet := TblDetail;
  end; //ID
  with TStringField.Create(Self) do
  begin
    Name := 'ForecastPropertiesDtlBALANCESHEET_NOMINAL_CODE';
    FieldKind := fkData;
    FieldName := 'DR_CR_BS_NOMINAL_CODE';
    Size := 8;
    DataSet := TblDetail;
    Required := False;
  end; //ID
  //TblDetail.CreateDataSet;
end;

procedure TLKLedgerNominalProfitAndBalancesForecast.CreateViewForm;
begin
  Application.CreateForm(TFormLKProfitAndBalancesForecastRecord, FFormView);
  TFormLKProfitAndBalancesForecastRecord(FormView).Construct(Self);
end;

procedure TLKLedgerNominalProfitAndBalancesForecast.CreateViewFormRecord;
begin
  Application.CreateForm(TFormLKProfitAndBalancesForecastRecord,
    FFormViewRecord);
  TFormLKProfitAndBalancesForecastRecord(FormViewRecord).Construct(Self);
end;

procedure TLKLedgerNominalProfitAndBalancesForecast.DlgForecastWizardExecute;
begin
  with TFormLKProfitAndBalancesForecastWizard.Create(nil) do
  try
    Construct(Self);
    ShowModal;
  finally Free;
  end;
end;

procedure TLKLedgerNominalProfitAndBalancesForecast.DlgSalesForecastExecute;
begin
  FSalesForecast.TblViewRefresh(False);
  //  to do SalesForecast.DlgTblViewView;
end;

function TLKLedgerNominalProfitAndBalancesForecast.DlgViewExecute: TModalResult;
begin
  if not assigned(FormView) then
  begin
    CreateViewForm;
    TFormLKProfitAndBalancesForecast(FormView).Construct(Self);
  end;
  Result := FormView.ShowModal;
end;

procedure TLKLedgerNominalProfitAndBalancesForecast.doCreditorsDaysWizExecute(
    Sender: TObject);
begin
  Showmessage('Procedure not yet implemented');
end;

procedure TLKLedgerNominalProfitAndBalancesForecast.doDebtorsDaysWizExecute(
    Sender: TObject);
begin
  Showmessage('Procedure not yet implemented');
end;

function TLKLedgerNominalProfitAndBalancesForecast.doInsertNewRecord:
    TModalResult;
var
  docRef: string;
  isTurnover: Boolean;
  NominalCode: string;
  startDate: TDate;
  tmpDate: TDate;
  histDate: TDate;
  crDays: Integer;
  i: Integer;
  bsNomCode: string;
  bsCrNomCode: string;
  varCostPercent: Variant;
  inflationIndex: Double;
  s: string;
  amount: Currency;
  total: Currency;

  function ThisAnnualPeriod(const HistoryDate, CurrentDate: TDate): boolean;
  var
    d: double;
  begin
    d := (CurrentDate - HistoryDate) / 365.24;
    d := (d - trunc(d)) * 365.24;
    Result := (d < 3.5) and (d > -3.5);
  end;

begin
  Result := mrOK;
  if (TblMaster.RecordCount = 0) then
  begin
    case AccountsHub.AccountsInterval of
      aciWeekly: i := 52;
      aciFortnightly: i := 26;
      aciFourWeeks: i := 13;
      aciAnnual: i := 1;
      aciDays: i := trunc(366 / AccountsHub.AccountsIntervalDays);
    else
      i := 12; //aciMonthly
    end;
    InsertRecord(['Default', 'Most Likely basis', MonthStartThis(Date), i, 60,
      30]);
  end;
  docRef := TblMaster.FieldByName('FORECAST_REFERENCE').AsString;
  crDays := GetInteger(TblMaster.FieldByName('AVE_DEBTORS_DAYS'));
  inflationIndex := 1; // for future development

  with NominalLedger do
  begin
    // get default balance sheet Debtors and Creditors codes
    bsNomCode := '';
    bsCrNomCode := '';
    with AccountsHub.Ledgers do
      for i := 0 to Count - 1 do
      begin
        if TComponent(Items[i]) is TLKLedgerSales then
          DecodeNominalAccount(TLKLedgerSales(Items[i]).NominalControlCode,
            bsNomCode, s);
        if TComponent(Items[i]) is TLKLedgerPurchase then
          DecodeNominalAccount(TLKLedgerPurchase(Items[i]).NominalControlCode,
            bsCrNomCode, s);
      end;
    // set the Forecast Start date
    with AccountsPeriods do
      if FindLast then
        startDate := GetDate(TblMaster.FieldByName('PERIOD_END')) + 1
      else
        with TLKViewDialog.Create(Owner) do
        try
          Title := 'Forecast Start date';
          DataType := vDate;
          HeaderCaption := 'Enter Forecast Start date';
          HeaderComments := 'You do not seem to have created any Management Accounts'
            + ' so a blank Forecast will be created.';
          Value := MonthStartThis(Date);
          if not Execute then
            Result := mrCancel;
          startDate := Value;
        finally Free;
        end;
    if (Result = mrCancel) then
      exit;
    screen.cursor := crHourglass;
    try
      Self.TblMaster.Edit;
      tmForecastStartDate := startDate;
      // Clear any existing data
      FProfitForecast.EmptyTables;
      FCashFlowForecast.EmptyTables;
      FBalancesForecast.EmptyTables;
      ForecastDatesRebuild;
      // get the Profit & Loss Nominal codes:
     // the first section of the P&L Accounts is assumed to be Turnover
      varCostPercent := Null; // Turnover section
      isTurnover := True;
    finally screen.cursor := crDefault;
    end;
    if ReportLayouts.Locate('LAYOUT_TYPE', 'A', []) then
    begin
      if ReportLayouts.TblDetail.FindFirst then
        repeat
          if ReportLines.FindFirst then
            repeat
              if ReportLines.TblDetail.FindFirst then
                repeat
                  if ManagementAccounts.FindFirst then
                    repeat
                      NominalCode :=
                        ManagementAccounts.TblMaster.FieldByName('NOMINAL_CODE').AsString;
                      if not Self.TblDetail.Locate('NOMINAL_CODE', NominalCode,
                        []) then
                      begin
                        s :=
                          ManagementAccounts.TblMaster.FieldByName('DESCRIPTION').AsString;
                        Self.TblDetail.InsertRecord([docRef, NominalCode, s,
                          varCostPercent, crDays, bsNomCode]);
                      end;
                      amount := 0;
                      {
                       count down the FForecastDates so that we may have an amount in the case
                       that the forecast month is prior to Management Accounting
                      }
                      if FForecastDates.FindLast then
                        repeat
                          tmpDate :=
                            GetDate(FForecastDates.TblMaster.FieldByName('PERIOD_END'));
                          FProfitForecast.InsertRecord([NominalCode, tmpDate,
                            isTurnover]);
                          //       FCashFlowForecast.InsertRecord([NominalCode,tmpDate]);
                          i := 0;
                          total := 0;
                          if ManagementAccounts.TblDetail.FindFirst then
                            repeat
                              histDate :=
                                GetDate(FForecastDates.TblMaster.FieldByName('PERIOD_END'));
                              if ThisAnnualPeriod(histDate, tmpDate) then
                                //eg the history month = the forecast month
                              begin
                                amount :=
                                  GetCurrency(ManagementAccounts.TblDetail.FieldByName('THIS_MONTH'));
                                FProfitForecast.TblDetail.InsertRecord(
                                  [NominalCode, tmpDate, histDate, amount,
                                    inflationIndex]);
                                total := total +
                                  GetCurrency(FieldByName('CALCULATED_VALUE'));
                                inc(i);
                              end;
                              if (i > 0) and (tmpDate > startDate) then
                                amount := total / i;
                              {
                              the value of amount is:
                                within the forecast period: the average of the indexed historic values
                                prior to the forecast period: the actual historic value from the M Accounts
                                forecast month is prior to the commencement of management accounting:
                                 the first management accounts value
                                no managemennt accounts have been produced: zero
                               //}
                              with FProfitForecast.TblMaster do
                              begin
                                Edit;
                                FieldByName('ADJUSTED_HISTORIC').AsCurrency :=
                                  amount;
                                FieldByName('FORECAST_VALUE').AsCurrency :=
                                  amount;
                                Post;
                              end;
                            until not ManagementAccounts.TblDetail.FindNext;
                        until not FForecastDates.FindPrior;
                    until not ManagementAccounts.FindNext;
                until not ReportLines.TblDetail.FindNext;
            until not ReportLines.FindNext;
          // the sections of the P&L Accounts after the first section are assumed to be Costs
          bsNomCode := bsCrNomCode;
          crDays := GetInteger(TblMaster.FieldByName('DEFAULT_CREDITORS_DAYS'));
          // the sections of the Accounts after the second are assumed to be Fixed Costs
          if (varCostPercent = 100) then
            varCostPercent := 0;
          // the second section of the Accounts is assumed to be Direct Costs
          if (varCostPercent = Null) then
            varCostPercent := 100;
        until not ReportLayouts.TblDetail.FindNext;
    end
    else
    begin
      Showmessage('Problem finding Layout Type "A" Profit and Loss in ' +
        ReportLayouts.NameFriendly);
      ReportLines.View(Null, True, [mbOK]);
      exit;
    end;
  end;
end;

procedure TLKLedgerNominalProfitAndBalancesForecast.ForecastDatesRebuild;
var
  startDate: TDate;
  tmpDate: TDate;
  histDate: TDate;
  i: Integer;
  s: string;

  procedure IncAPDate(var ADate: TDate);
  begin
    case AccountsHub.AccountsInterval of
      aciWeekly: ADate := ADate + 7;
      aciFortnightly: ADate := ADate + 14;
      aciFourWeeks: ADate := ADate + 28;
      aciMonthly: ADate := MonthEndThis(ADate + 1);
      aciAnnual: ADate := MonthEndThis(ADate + 360);
      aciDays: ADate := ADate + AccountsHub.AccountsIntervalDays;
    end;
  end;

  procedure DecAPDate(var ADate: TDate);
  begin
    case AccountsHub.AccountsInterval of
      aciWeekly: ADate := ADate - 7;
      aciFortnightly: ADate := ADate - 14;
      aciFourWeeks: ADate := ADate - 28;
      aciMonthly: ADate := MonthEndThis(ADate - 32);
      aciAnnual: ADate := MonthEndThis(ADate - 366);
      aciDays: ADate := ADate - AccountsHub.AccountsIntervalDays;
    end;
  end;

begin
  s := ShortDateFormat;
  try
    FForecastDates.TblMaster.EmptyDataset;
    startDate := tmForecastStartDate;
    case AccountsHub.AccountsInterval of
      aciMonthly: ShortDateFormat := 'mmm yyyy';
    else
      ShortDateFormat := 'dd mmm yy';
    end;
    i := tmDefaultDebtorsDays;
    if (i < tmDefaultCreditorsDays) then
      i := tmDefaultCreditorsDays;
    histDate := startDate - i - 70;
    tmpDate := startDate - 1; // Last M Accounts date
    i := 0;
    while (tmpDate > histDate) do
    begin
      FForecastDates.TblMaster.InsertRecord([tmpDate, i, DateToStr(tmpDate)]);
      DecAPDate(tmpDate);
      dec(i);
    end;
    tmpDate := startDate - 1; // Last M Accounts date
    IncAPDate(tmpDate); // first forecast period
    for i := 1 to tmForecastPeriods do
    begin
      FForecastDates.TblMaster.InsertRecord([tmpDate, i, DateToStr(tmpDate)]);
      IncAPDate(tmpDate);
    end;
  finally ShortDateFormat := s;
  end;
end;

procedure TLKLedgerNominalProfitAndBalancesForecast.ForecastUpdate;
begin
end;

function TLKLedgerNominalProfitAndBalancesForecast.GettdDescription: string;
begin
  Result := TblDetail.FieldByName('DESCRIPTION').AsString
end;

function TLKLedgerNominalProfitAndBalancesForecast.GettdNominalCode: string;
begin
  Result := TblDetail.FieldByName('NOMINAL_CODE').AsString
end;

function TLKLedgerNominalProfitAndBalancesForecast.GettmDefaultCreditorsDays:
    Integer;
begin
  Result := GetInteger(TblMaster.FieldByName('DEFAULT_CREDITORS_DAYS'));
end;

function TLKLedgerNominalProfitAndBalancesForecast.GettmDefaultDebtorsDays:
    Integer;
begin
  Result := GetInteger(TblMaster.FieldByName('DEFAULT_DEBTORS_DAYS'));
end;

function TLKLedgerNominalProfitAndBalancesForecast.GettmForecastPeriods:
    Integer;
begin
  Result := GetInteger(TblMaster.FieldByName('FORECAST_PERIODS'));
end;

function TLKLedgerNominalProfitAndBalancesForecast.GettmForecastStartDate:
    TDateTime;
begin
  Result := GetDate(TblMaster.FieldByName('START_DATE'));
end;

function TLKLedgerNominalProfitAndBalancesForecast.LoadFromFile: Boolean;
begin
  if (FSalesForecast = nil) then
  begin
    FSalesForecast := TLKForecastSales.Create(Self);
    FSalesForecast.TblMaster.CreateDataSet;
  end;
  Result := inherited LoadFromFile;
end;

function TLKLedgerNominalProfitAndBalancesForecast.RecordNew: TModalResult;
begin
  Result := doInsertNewRecord;
  if (Result = mrOK) then
    DlgForecastWizardExecute;
end;

procedure TLKLedgerNominalProfitAndBalancesForecast.SetActive(Value: Boolean);
begin
  if (Value = Active) then
    exit;
  inherited SetActive(Value);
  if (csLoading in ComponentState) then
    exit;
  if Value then
  begin
    if (TblMaster.RecordCount = 0) then
    begin
      LoadFromFile;
      if (TblMaster.RecordCount = 0) then
        doInsertNewRecord;
    end;
  end;
  //TblViewsRefresh(True);
end;

procedure TLKLedgerNominalProfitAndBalancesForecast.SetContainedComponentNames;
begin
  inherited SetContainedComponentNames;
  FSalesForecast.Name := Name + 'SalesForecast';
  FForecastDates.Name := Name + 'Dates';
  FProfitForecast.Name := Name + 'Profit';
  FCashFlowForecast.Name := Name + 'CashFlow';
  FBalancesForecast.Name := Name + 'Balances';
end;

procedure TLKLedgerNominalProfitAndBalancesForecast.SetForecastName(const
    Value: string);
begin
  TblMaster.FieldByName('FORECAST_NAME').AsString := Value;
end;

procedure TLKLedgerNominalProfitAndBalancesForecast.SetNominalLedger(Value:
    TLKLedgerNominal);
begin
  FNominalLedger := Value;
end;

procedure TLKLedgerNominalProfitAndBalancesForecast.SettdDescription(const
    Value: string);
begin
  TblDetail.FieldByName('DESCRIPTION').AsString := Value;
end;

procedure TLKLedgerNominalProfitAndBalancesForecast.SettdNominalCode(const
    Value: string);
begin
  TblDetail.FieldByName('NOMINAL_CODE').AsString := Value;
end;

procedure TLKLedgerNominalProfitAndBalancesForecast.SettmDefaultCreditorsDays(
    Value: Integer);
begin
  TblMaster.FieldByName('DEFAULT_CREDITORS_DAYS').AsInteger := Value;
end;

procedure TLKLedgerNominalProfitAndBalancesForecast.SettmDefaultDebtorsDays(
    Value: Integer);
begin
  TblMaster.FieldByName('DEFAULT_DEBTORS_DAYS').AsInteger := Value;
end;

procedure TLKLedgerNominalProfitAndBalancesForecast.SettmForecastPeriods(Value:
    Integer);
begin
  TblMaster.FieldByName('FORECAST_PERIODS').AsInteger := Value;
end;

procedure TLKLedgerNominalProfitAndBalancesForecast.SettmForecastStartDate(
    Value: TDateTime);
begin
  TblMaster.FieldByName('START_DATE').AsDateTime := Value;
end;

procedure TLKLedgerNominalProfitAndBalancesForecast.TblViewsRefresh(Rebuild:
    boolean);
begin
  if Assigned(FSalesForecast) then
    SalesForecast.TblViewRefresh(Rebuild);
  FProfitForecast.TblViewRefresh(Rebuild);
  FCashFlowForecast.TblViewRefresh(Rebuild);
  FBalancesForecast.TblViewRefresh(Rebuild);
end;

{
**************** TLKProfitAndBalancesForecastContainedDocuments ****************
}
constructor TLKProfitAndBalancesForecastContainedDocuments.Create(AOwner:
    TComponent);
begin
  if not (AOwner is TLKLedgerAnalysis) then
    raise
      LKError.Create('Program error: '
      + 'LKLedgerNominalProfitAndBalancesForecastContainedDocuments.Owner '
      + 'should be a TLKLedgerNominalProfitAndBalancesForecast');
  inherited Create(AOwner);
end;

function TLKProfitAndBalancesForecastContainedDocuments.GetContainerComponent:
    TLKLedgerNominalProfitAndBalancesForecast;
begin
  Result := TLKLedgerNominalProfitAndBalancesForecast(Owner);
end;



end.
