{*****************************************************************************}
{                                                                             }
{   This file is part of the LKAccounts components Library                    }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for display of a TLKDocumentsCashBook record                      }
{                                                                             }
{*****************************************************************************}

unit LKDocumentsCashBookRecordDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  LKDocumentsSalesTaxRecordDlg, ImgList, Menus, ActnList, Db, Variants,
  ComCtrls, StdCtrls, DBCtrls, Buttons, ToolWin, ExtCtrls, LKUtils,
  LKRecordsComponents, LKAccountsComponents, LKdlgs, StdActns,
  LKDBControls, LKUtilComponents, Grids, DBGrids,
  JvExDBGrids, JvDBGrid, JvDBUltimGrid;

type
  TFormLKDocumentsCashBookRecord = class(TFormLKDocumentsSalesTaxRecord)
    ActionBtnLedgers: TAction;
    GbLedgers: TGroupBox;
    BtnNominalLedger: TButton;
    procedure ActionBtnLedgersExecute(Sender: TObject);
    procedure ActionItemEnterExecute(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ActionItemExitExecute(Sender: TObject);
  private
    { Private declarations }
    FMyAccountsComponent: TLKDocumentsCashBook;
  protected
    ContactLedgerList: TLKLedgerList;
    procedure SetDetailEditsActions; override;
  protected
    DefaultLedgerBtnTag: Integer;
    procedure CustomiseDetailEdits; override;
  public
    { Public declarations }
    procedure Construct(Sender: TLKDocumentsCashBook);
end;

const
  BtnNominalLedgerTag = 0;

var
  FormLKDocumentsCashBookRecord: TFormLKDocumentsCashBookRecord;

implementation

{$R *.DFM}

procedure TFormLKDocumentsCashBookRecord.Construct(Sender:
  TLKDocumentsCashBook);
var
  I, H, T, X: integer;
begin
  FMyAccountsComponent := Sender;
  inherited Construct(Sender);
  ContactLedgerList := TLKLedgerList.Create(False);
  ActionBtnLedgers.Enabled := False;
  DefaultLedgerBtnTag := 0;
  X := 1;
  with FMyAccountsComponent.AccountsHub do
  begin
    T := BtnNominalLedger.Top; // rgLedgers count = nominal ledger
    H := BtnNominalLedger.Height;
    BtnNominalLedger.Tag := BtnNominalLedgerTag;
    for I := 0 to Ledgers.Count - 1 do
      if (Ledgers[I] is TLKLedgerContact) then
      begin
        with TButton.Create(Self) do
        begin
          T := T + H + 4;
          Parent := GbLedgers;
          Height := H;
          Left := BtnNominalLedger.Left;
          Top := T;
          Width := BtnNominalLedger.Width;
          Action := ActionBtnLedgers;
          Tag := X;
          Inc(X);
          Caption := Ledgers[I].NameFriendly;
          if Ledgers[I] = FMyAccountsComponent.DefaultPaymentLedger then
            DefaultLedgerBtnTag := Tag;
        end;
        ContactLedgerList.Add(Ledgers[I]);
      end;
  end;
  with GbLedgers, ContactLedgerList do
  begin
    I := Height + Count * (H + 4);
    Height := I;
    I := I + Top;
    Visible := Count > 0;
  end;
  with Panel1 do
    if (Height < I) then
      Height := I;
end;

procedure TFormLKDocumentsCashBookRecord.CustomiseDetailEdits;
var
  I: Integer;
begin
  inherited;
  if Assigned(DetailEditList) then
  // Make contact ledger fields read only as these need to be done with ContactLedgerPayment procedure
    with DetailEditList do
      for I := 0 to Count - 1 do
        with Items[I] do
          if (pos('LEDGER', FieldName) = 1) then
            ReadOnly := True;
end;

procedure TFormLKDocumentsCashBookRecord.ActionBtnLedgersExecute(
  Sender: TObject);
var
  X: integer;
  TransactionDate: TDateTime;
  MyLedger: TLKLedgerContact;
begin
  ActionBtnLedgers.Enabled := False;
  if not FMyAccountsComponent.TblDetail.FieldByName('GOODS').IsNull then
  begin
    ActionNextItem.Execute;
    Exit;
  end;
  X := TCustomAction(Sender).ActionComponent.Tag;
  if (X = BtnNominalLedgerTag) then // nominal ledger
  begin
    dbeNominal.SetFocus;
    Exit;
  end;
  TransactionDate := dbeDate.Field.asDateTime;
  MyLedger := TLKLedgerContact(ContactLedgerList[X-1]);
  with MyLedger do
    if (LedgerDate < TransactionDate) then
    begin
      Showmessage('Warning: You cannot process entries onto the ' + Namefriendly
        +
        ' for this month until End of Month has been run for ' +
          DateToStr(LedgerDate));
    end;
  FMyAccountsComponent.ContactLedgerPayment(Self, MyLedger,
    TransactionDate);
  DBEDLast.SetFocus;
end;

procedure TFormLKDocumentsCashBookRecord.ActionItemEnterExecute(
  Sender: TObject);
begin
  with dbeDate do
    if Field.IsNull then
    begin
      Showmessage('You must enter a date first');
      SetFocus;
      Exit;
    end;
  inherited;
end;

procedure TFormLKDocumentsCashBookRecord.FormDestroy(
  Sender: TObject);
begin
  ContactLedgerList.Free;
  inherited;
end;

procedure TFormLKDocumentsCashBookRecord.ActionItemExitExecute(
  Sender: TObject);
begin
  inherited;
  with FMyAccountsComponent.TblDetail do
  begin
    if (State = dsInsert) then
    begin
      ActionBtnLedgers.Enabled := True;
      TButton(GbLedgers.Controls[DefaultLedgerBtnTag]).SetFocus;
    end;
  end;
end;

procedure TFormLKDocumentsCashBookRecord.SetDetailEditsActions;
begin
  inherited;
  GbLedgers.Top := dbeCost.Top + 16;
end;

end.

