{*****************************************************************************}
{                                                                             }
{   This file is part of the LKAccounts components Library                    }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for display of a TLKContainedRecords record                       }
{       of a TLKNominalLedger                                                 }
{                                                                             }
{*****************************************************************************}

unit LKDocumentsInputNominalRecordDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, System.UITypes,
  Graphics, Controls, Forms,
  Dialogs, LKRecordsBaseRecordDlg, ImgList, Menus, DB, ActnList, StdActns,
  DBCtrls, Buttons, ExtCtrls, ToolWin, ComCtrls, StdCtrls, Grids, DBGrids, Math,
  LKdlgs, LKDefine, LKRecordsComponents, LKAccountsComponents, LKUtils,
  LKUtilComponents, LKDBControls, JvExDBGrids, JvDBGrid, JvDBUltimGrid;

type
  TFormLKDocumentsInputNominalRecord = class(TFormLKRecordsBaseRecord)
    ActionNextItem: TAction;
    ActionItemEnter: TAction;
    ActionPostEditDetailDB: TAction;
    ActionDetailEditsProperties: TAction;
    ActionItemExit: TAction;
    ActionMasterLastEditExit: TAction;
    ActionNominalExit: TAction;
    ActionCostExit: TAction;
    SplitterDetailEdits: TSplitter;
    SplitterMasterEdits: TSplitter;
    DetailEditList: TLKDcDBEditsPanel;
    PanelNotes: TPanel;
    pmPanelDetailEdits: TPopupMenu;
    DeisplayProperties1: TMenuItem;
    Copy2: TMenuItem;
    Paste2: TMenuItem;
    ActionNextItem1: TMenuItem;
    MiDetailEditsEditBreak: TMenuItem;
    DisplayProperties2: TMenuItem;
    LabelNominalDescription: TLabel;
    LabelCostDescription: TLabel;
    procedure ActionNominalExitExecute(Sender: TObject);
    procedure ActionCostExitExecute(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ActionItemEnterExecute(Sender: TObject);
    procedure ActionNextItemExecute(Sender: TObject);
    procedure ActionPostEditDetailDBExecute(Sender: TObject);
    procedure ActionDetailEditsPropertiesExecute(Sender: TObject);
    procedure ActionItemExitExecute(Sender: TObject);
    procedure ActionMasterLastEditExitExecute(Sender: TObject);
    procedure ActionButtonOKExecute(Sender: TObject);
    procedure ActionMasterEditsPropertiesExecute(Sender: TObject);
    procedure SplitterMasterEditsMoved(Sender: TObject);
    procedure SplitterDetailEditsMoved(Sender: TObject);
  private
    { Private declarations }
    FMyAccountsComponent: TLKContainedRecords;
    procedure DBEDLastKeyPress(Sender: TObject; var Key: Char);
  protected                   
    dbeDLast: TWinControl;
    dbeDocumentNo: TWinControl;
    dbeItemNo: TWinControl;
    dbeMLast: TWinControl;
    dbeNominal, dbeCost: TLKDBDCLookupEdit;
    NominalLedger: TLKLedgerNominal;
    procedure CustomiseDetailEdits; override;
    procedure PlaceEditControls; override;
    procedure SetDetailEditsActions; virtual;
    procedure SetMasterEditsActions; override;
  public
    { Public declarations }
    procedure Construct(Sender: TLKContainedRecords;
      ANominalLedger: TLKLedgerNominal);
  end;

var
  FormLKDocumentsInputNominalRecord: TFormLKDocumentsInputNominalRecord;

implementation

{$R *.dfm}

{ TFormLKDocumentsInputNominalRecord }

procedure TFormLKDocumentsInputNominalRecord.Construct(
  Sender: TLKContainedRecords; ANominalLedger: TLKLedgerNominal);
begin
  FMyAccountsComponent := Sender;
  NominalLedger := ANominalLedger;
  inherited Construct(Sender);
end;

procedure TFormLKDocumentsInputNominalRecord.PlaceEditControls;
var
  MasterEditsLayout: TLKEditsLayout;
  DetailEditsLayout: TLKEditsLayout;
begin
  DetailEditsLayout := aceOneCol;
  with FMyAccountsComponent do
  begin
    if TblDetailUsed then
    begin
      MasterEditsLayout := aceOneCol;
      MasterEditList.Width :=
        RecallInteger('FormViewRecord', 'pMasterEditsWidth', 250, False);
      DetailEditList.Width :=
        RecallInteger('FormViewRecord', 'pDetailEditsWidth', 250, False);
    end
    else
    begin
      MasterEditsLayout := aceTwoCol;
      MasterEditList.Width :=
        RecallInteger('FormViewRecord', 'pMasterEditsWidth', 500, False);
      DetailEditList.Free;
      SplitterMasterEdits.Free;
    end;
    with MasterEditList do
    begin
      ConstructFromDatasource(HideMasterFields);
      CustomiseMasterEdits;
      ReadOnly := not Editing;
      Layout := MasterEditsLayout;
      LoadFormat(FMyAccountsComponent, 'FormViewRecord', 'dbMasterEdits');
      SetMasterEditsActions;
    end;
    if TblDetailUsed then
    begin
      DetailEditList.ConstructFromDatasource(TblDetail.MasterFields);
      CustomiseDetailEdits;
      DetailEditList.Layout := DetailEditsLayout;
      DetailEditList.ReadOnly := not Editing;
      DetailEditList.LoadFormat(
        FMyAccountsComponent, 'FormViewRecord', 'dbDetailEdits');
      SetDetailEditsActions;
    end;
    Resize;
    CreatedbGridDetail;
    with dbGridDetail do
    begin
      ReadOnly := True;
      TabStop := False;
      try
        LoadFormat(FMyAccountsComponent, 'FormViewRecord', 'dbGridDetail');
      except ClearFormat(FMyAccountsComponent, 'FormViewRecord',
        'dbGridDetail');
      end;
    end;
  end;
end;


procedure TFormLKDocumentsInputNominalRecord.CustomiseDetailEdits;
var
  I: integer;
  S: string;
begin
  if Assigned(DetailEditList) then
  with DetailEditList do
    for I := 0 to Count - 1 do
      with Items[I] do
      begin
        if (Pos('IMAGE', FieldName) > 0) then
          ControlType := dbcImageFile;
        with FMyAccountsComponent do
        begin
          S := RecallString(TblDetail.TableName, FieldName, '', true);
          if (S > '') then
          begin
            if (S = 'DATE_') then
              ControlType := dbcDateEdit;
            if (S = 'TIME_') then
              ControlType := dbcTimeEdit;
            if (S = 'COMBOBOX') then
            begin
              ControlType := dbcComboBox;
              TDBComboBox(EditControl).Items.Text :=
                RecallString(TblDetail.TableName, FieldName + 'ComboBox', '',
                  true);
            end;
            if (S = 'FILE') then
              ControlType := dbcFilePath;
            if (S = 'URL') then
              ControlType := dbcURL;
            if (S = 'EMAIL') then
              ControlType := dbcEMail;
            // if (s = '') then ControlType := ;
          end;
        end;
      end;
end;

procedure TFormLKDocumentsInputNominalRecord.DBEDLastKeyPress(Sender: TObject;
  var Key: Char);
begin
  if (Key = Chr(VK_RETURN)) then
    ActionNextItem.OnExecute(Sender);
end;

procedure TFormLKDocumentsInputNominalRecord.FormShow(Sender: TObject);
begin
  inherited;
  LabelNominalDescription.Caption := '';
  LabelCostDescription.Caption := '';
  if Assigned(DetailEditList) then
  begin
    Panel1.Height :=
      4 + Max(MasterEditList.DisplayHeight, DetailEditList.DisplayHeight);
    SplitterMasterEdits.Left := MasterEditList.Width;
  end
  else Panel1.Height := 4 + MasterEditList.DisplayHeight;
end;

procedure TFormLKDocumentsInputNominalRecord.SetMasterEditsActions;
var
  I: Integer;
begin
  with MasterEditList do
    for I := 0 to Count - 1 do
      with Items[I] do
      begin
        if (FieldName = 'NOMINAL_CODE') then
        begin
          ControlType := dbcDCLookup;
          with TLKDBDCLookupEdit(EditControl) do
          begin
            LookupDataComponent := NominalLedger.NominalCodes;
            LookupDataset := tbMaster;
          end;
        end;
        if (FieldName = 'COST_CENTRE') then
        begin
          ControlType := dbcDCLookup;
          with TLKDBDCLookupEdit(EditControl) do
          begin
            LookupDataComponent := NominalLedger.CostLedger.CostCentres;
            LookupDataset := tbMaster;
          end;
        end;
      end;
  inherited;
end;

procedure TFormLKDocumentsInputNominalRecord.SetDetailEditsActions;
var
  I: Integer;
begin
  inherited;
  DBEItemNo := nil;
  DBEDLast := nil;
  if Assigned(DetailEditList) then
    with DetailEditList do
    begin
      for I := 0 to Count - 1 do
        with Items[I] do if Visible then
          if (EditControl is TWinControl) then
          begin
            DBEItemNo := TWinControl(EditControl);
            OnEnter := ActionItemEnter.OnExecute;
            OnExit := ActionItemExit.OnExecute;
            Break;
          end;
      for I := Count - 1 downto 0 do
        with Items[I] do
          if Visible and not ReadOnly then
          if (EditControl is TWinControl) then
          begin
            DBEDLast := TWinControl(EditControl);
            // OnExit := ActionNextItem.OnExecute;
            OnKeyPress := DBEDLastKeyPress;
            Break;
          end;
      for I := 0 to Count - 1 do
        with Items[I] do
        begin
          if (FieldName = 'NOMINAL_CODE') then
          begin
            ControlType := dbcDCLookup;
            dbeNominal := TLKDBDCLookupEdit(EditControl);
            with dbeNominal do
            begin
              LookupDataComponent := NominalLedger.NominalCodes;
              LookupDataset := tbMaster;
            end;
            OnExit := ActionNominalExit.OnExecute;
            LabelNominalDescription.Top := dbeNominal.Top;
          end;
          if (FieldName = 'COST_CENTRE') then
          begin
            ControlType := dbcDCLookup;
            dbeCost := TLKDBDCLookupEdit(EditControl);
            with dbeCost do
            begin
              LookupDataComponent := NominalLedger.CostLedger.CostCentres;
              LookupDataset := tbMaster;
            end;
            OnExit := ActionCostExit.OnExecute;
            LabelCostDescription.Top := dbeCost.Top;
          end;
        end;
    end;
end;

procedure TFormLKDocumentsInputNominalRecord.ActionDetailEditsPropertiesExecute(
  Sender: TObject);
begin
  inherited;
  if Assigned(DetailEditList) then
    if (DetailEditList.DlgEditListPropsExecute(Self) = mrOK) then
    begin
      DetailEditList.SaveFormat(FMyAccountsComponent, 'FormViewRecord',
        'dbDetailEdits');
      SetDetailEditsActions;
      Panel1.Height :=
        4 + Max( MasterEditList.DisplayHeight, DetailEditList.DisplayHeight);
    end;
end;

procedure TFormLKDocumentsInputNominalRecord.ActionItemEnterExecute(
  Sender: TObject);
begin
  inherited; 
  // override if required
end;

procedure TFormLKDocumentsInputNominalRecord.ActionItemExitExecute(
  Sender: TObject);
begin
  inherited;   
  // override if required
end;

procedure TFormLKDocumentsInputNominalRecord.ActionMasterEditsPropertiesExecute(
  Sender: TObject);
begin
  inherited;
  if Assigned(DetailEditList) then
    Panel1.Height :=
      Max(MasterEditList.DisplayHeight, 4 + DetailEditList.DisplayHeight);
end;

procedure TFormLKDocumentsInputNominalRecord.ActionMasterLastEditExitExecute(
  Sender: TObject);
begin
  inherited;
  with FMyAccountsComponent do
    if TblMaster.State in [dsEdit, dsInsert] then
    begin
      Post;
      if TblDetailUsed then
        TblDetail.Append;
    end;
  FocusControl(dbeItemNo);
end;

procedure TFormLKDocumentsInputNominalRecord.ActionNextItemExecute(
  Sender: TObject);
begin
  inherited;
  with FMyAccountsComponent.TblDetail do
    if (State in [dsInsert, dsEdit]) then
      Post;
//  if (ActiveControl = btnOK) then
//    Exit;
  if (MessageDlg('Another item?', mtInformation, [mbYes, mbNo], 0) = mrYes) then
  begin
    FMyAccountsComponent.TblDetail.Append;
    FocusControl(dbeItemNo);
  end
  else
    FocusControl(btnOK);
end;

procedure TFormLKDocumentsInputNominalRecord.ActionNominalExitExecute(
  Sender: TObject);
begin
  if not (FMyAccountsComponent.TblDetail.State in [dsInsert, dsEdit]) then
    Exit;
  if (dbeNominal.Field.AsString > ' ') then
//    if Locate('NOMINAL_CODE', code, []) then
    LabelNominalDescription.Caption := NominalLedger.NominalCodes.tmDescription
  else LabelNominalDescription.Caption := '';
end;

procedure TFormLKDocumentsInputNominalRecord.ActionPostEditDetailDBExecute(
  Sender: TObject);
begin
  inherited;
  try
    with FMyAccountsComponent.TblDetail do
      if State in [dsInsert, dsEdit] then
      begin
        Post;
        Edit;
      end;
  except;
  end;
end;

procedure TFormLKDocumentsInputNominalRecord.ActionButtonOKExecute(
  Sender: TObject);
begin
  with FMyAccountsComponent do
    if (TblDetail.State = dsInsert) then
      if (dbeItemNo is TDBEdit) then
        if TDBEdit(dbeItemNo).Field.IsNull then
          TblDetail.Cancel;
  inherited;
end;

procedure TFormLKDocumentsInputNominalRecord.ActionCostExitExecute(
  Sender: TObject);
begin
  if not (FMyAccountsComponent.TblDetail.State in [dsInsert, dsEdit]) then
    Exit;
  if (dbeCost.Field.AsString > ' ') then
//    if Locate('COST_CENTRE', code, []) then
    LabelCostDescription.Caption :=
      NominalLedger.CostLedger.CostCentres.tmDescription
  else LabelCostDescription.Caption := '';
end;

procedure TFormLKDocumentsInputNominalRecord.SplitterDetailEditsMoved(Sender:
  TObject);
begin
  FMyAccountsComponent.RememberInteger('FormViewRecord', 'pDetailEditsWidth',
    DetailEditList.Width, False);
  // if assigned(DetailEditList) then DetailEditList.Resize;
end;

procedure TFormLKDocumentsInputNominalRecord.SplitterMasterEditsMoved(Sender:
  TObject);
begin
  FMyAccountsComponent.RememberInteger('FormViewRecord', 'pMasterEditsWidth',
    MasterEditList.Width, False);
  // MasterEditList.Resize;
end;

end.

