{*****************************************************************************}
{                                                                             }
{   This file is part of the LKAccounts components Library                    }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for display of a TLKDocumentsSalesTax record                       }
{                                                                             }
{*****************************************************************************}

unit LKDocumentsSalesTaxRecordDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, System.UITypes, System.Actions,
  Graphics, Controls, Forms, Dialogs,
  LKDocumentsNominalRecordDlg, ImgList, Menus, ActnList, Db, ComCtrls, StdCtrls,
  DBCtrls, Buttons, ToolWin, ExtCtrls, Variants, StdActns, Grids, DBGrids,
  JvExDBGrids, JvDBGrid, JvDBUltimGrid, LKRecordsComponents, LKUtils, LKdlgs,
  LKAccountsComponents, LKDBControls, LKUtilComponents;

type
  TFormLKDocumentsSalesTaxRecord = class(TFormLKDocumentsNominalRecord)
    ActionP11dView: TAction;
    ActionViewP11dPopup: TAction;
    pmViewP11d: TPopupMenu;
    sbP11d: TToolButton;
    ActionListViewP11D: TActionList;
    procedure ActionP11dViewExecute(Sender: TObject);
    procedure ActionViewP11dPopupExecute(Sender: TObject);
  private
    FMyAccountsComponent: TLKDocumentsSalesTax;
  protected
    FIsSalesTax: Boolean;
    //    procedure aNominalExitExecute(Sender: TObject); override;
    procedure CustomiseDetailEdits; override;
    procedure CustomiseMasterEdits; override;
  public
    procedure Construct(Sender: TLKDocumentsSalesTax);
  end;

var
  FormLKDocumentsSalesTaxRecord: TFormLKDocumentsSalesTaxRecord;

implementation

{$R *.DFM}
uses LKDefine;

{
************************ TFormLKDocumentsSalesTaxRecord ************************
}
{
procedure TFormLKDocumentsSalesTaxRecord.aNominalExitExecute(Sender: TObject);
begin
  inherited aNominalExitExecute(Sender);
  if not isSalesTax then exit;
  with MyAccountsComponent do
   begin
   if not (TblDetail.State in [dsInsert,dsEdit]) then exit;
   with TblDetail do
    if (FieldByName('NOMINAL_CODE').AsString > ' ') and FieldByName('VAT_CODE').IsNull then
    FieldByName('VAT_CODE').asInteger := NominalLedger.NominalCodes.tmDefaultSalesTaxCode;
   end;
end;
}

procedure TFormLKDocumentsSalesTaxRecord.ActionP11dViewExecute(Sender: TObject);
begin
  FMyAccountsComponent.AccountsHub.P11d.View(null, false, []);
end;

procedure TFormLKDocumentsSalesTaxRecord.ActionViewP11dPopupExecute(Sender: TObject);
begin
  pmViewP11D.Popup(Mouse.CursorPos.X, Mouse.CursorPos.Y);
end;

procedure TFormLKDocumentsSalesTaxRecord.Construct(Sender: TLKDocumentsSalesTax);
var
  NewAction: TAction;
  NewMenuitem: TMenuItem;
  AC: TLKDataComponent;
begin
  FMyAccountsComponent := Sender;
  FIsSalesTax := assigned(FMyAccountsComponent.SalesTaxReturns);
  inherited Construct(Sender);
  with FMyAccountsComponent do
  begin
    ActionListViewP11D.Images := Images;
    if Assigned(AccountsHub) then
      if Assigned(AccountsHub.P11D) then
      begin
        AC := AccountsHub.P11D;
        sbP11D.Visible := assigned(AC);
        if Assigned(AC) then
        begin
          NewAction := TAction.Create(Self);
          with NewAction do
          begin
            ActionList := ActionListViewP11D;
            Caption := '&' + AC.NameFriendly;
            Hint := 'View ' + AC.NameFriendly;
            ImageIndex := AC.ImageIndex;
            OnExecute := ActionP11DViewExecute;
          end;
          NewMenuitem := TMenuitem.Create(miView);
          NewMenuitem.Action := NewAction;
          //  newMenuItem.ImageIndex := AC.ImageIndex;
          miView.Add(NewMenuitem);
          NewMenuitem := TMenuitem.Create(PopupMenu1);
          NewMenuitem.Action := NewAction;
          //  newMenuItem.ImageIndex := AC.ImageIndex;
          PopupMenu1.Items.Add(NewMenuitem);
          sbP11D.Action := NewAction;
        end;
      end;
  end;
end;

procedure TFormLKDocumentsSalesTaxRecord.CustomiseDetailEdits;
var
  I: Integer;
begin
  inherited;
  if Assigned(DetailEditList) then
    with DetailEditList do
      for I := 0 to Count - 1 do
        with Items[I] do
        begin
          if (FieldName = 'VAT_CODE') then
            Visible := FIsSalesTax;
          if (FieldName = 'VAT') then
            Visible := FIsSalesTax;
//          if (FieldName = 'SALES_TAX') then
//            Visible := FIsSalesTax;
          if (FieldName = 'TOTAL') then
            Visible := FIsSalesTax;
        end;
  if Assigned(dbGridDetail) then
    with dbGridDetail do
      for I := 0 to Count - 1 do
        with Items[I] do
        begin
          if (FieldName = 'VAT_CODE') then
            Visible := FIsSalesTax;
          if (FieldName = 'VAT') then
            Visible := FIsSalesTax;
          if (FieldName = 'TOTAL') then
            Visible := FIsSalesTax;
        end;
end;

procedure TFormLKDocumentsSalesTaxRecord.CustomiseMasterEdits;
var
  I: Integer;
begin
  inherited;
  with MasterEditList do
    for I := 0 to Count - 1 do
      with Items[I] do
      begin
        if (FieldName = 'VAT') then
          Visible := FIsSalesTax;
        if (FieldName = 'TOTAL') then
          Visible := FIsSalesTax;
        if (FieldName = 'VAT_PERIOD') then
          if FIsSalesTax then
            ReadOnly := True
          else
            Visible := False;
      end
end;

end.

