{*****************************************************************************}
{                                                                             }
{   This file is part of the LKAccounts components Library                    }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for editing of a TLKHubAccounts settings and properties           }
{                                                                             }
{*****************************************************************************}

unit LKHubAccountsEnterpriseDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, LKHubEnterpriseDlg, LKAccountsComponents, StdCtrls, Mask, JvExMask,
  JvToolEdit, JvMaskEdit, JvCheckedMaskEdit, JvDatePickerEdit, JvBaseDlg,
  JvBrowseFolder, ImgList, ActnList, LKBaseComponent, ExtCtrls, ComCtrls;

type
  TDlgLKHubAccountsEnterprise = class(TDlgLKHubEnterprise)
    StaticText1: TStaticText;
    JvDatePickerEditPeriodStart: TJvDatePickerEdit;
    StaticText2: TStaticText;
    JvDatePickerEditPeriodEnd: TJvDatePickerEdit;
    StaticText3: TStaticText;
    ComboBoxAccountsInterval: TComboBox;
    EditAccountsIntervalDays: TEdit;
    procedure ComboBoxAccountsIntervalChange(Sender: TObject);
    procedure JvDatePickerEditPeriodStartChange(Sender: TObject);
  private
    { Private declarations }
    FHub: TLKHubAccounts;
  protected
    procedure GetEnterpriseInfo; override;
    procedure SaveEnterpriseInfo; override;
  public
    { Public declarations }
    procedure Construct(AHub: TLKHubAccounts);
  end;

var
  DlgLKHubAccountsEnterprise: TDlgLKHubAccountsEnterprise;

implementation

{$R *.dfm}

uses LKDefine;

{ TForm2 }

procedure TDlgLKHubAccountsEnterprise.ComboBoxAccountsIntervalChange(Sender: TObject);
begin
  EditAccountsIntervalDays.Visible :=
    ComboBoxAccountsInterval.ItemIndex = 5;
  btnOK.Enabled := False;
  btnApply.Enabled := True;
end;

procedure TDlgLKHubAccountsEnterprise.Construct(AHub: TLKHubAccounts);
begin
  FHub := AHub;
  inherited Construct(AHub);
end;

procedure TDlgLKHubAccountsEnterprise.GetEnterpriseInfo;
begin
  with FHub do
  begin
    JvDatePickerEditPeriodStart.Date := AccountsPeriodStart;
    JvDatePickerEditPeriodEnd.Date := AccountsPeriodEnd;
    ComboBoxAccountsInterval.ItemIndex := Ord(AccountsInterval) - 1;
    EditAccountsIntervalDays.Text := IntToStr(AccountsIntervalDays);
  end;
  EditAccountsIntervalDays.Visible :=
    ComboBoxAccountsInterval.ItemIndex = 5;
  inherited;
end;

procedure TDlgLKHubAccountsEnterprise.JvDatePickerEditPeriodStartChange(
  Sender: TObject);
begin
  btnOK.Enabled := False;
  btnApply.Enabled := True;
end;

procedure TDlgLKHubAccountsEnterprise.SaveEnterpriseInfo;
begin
  with FHub do
  begin
    AccountsPeriodStart := JvDatePickerEditPeriodStart.Date;
    AccountsPeriodEnd := JvDatePickerEditPeriodEnd.Date;
    AccountsInterval  :=
      TLKAccountsInterval(ComboBoxAccountsInterval.ItemIndex + 1);
    AccountsIntervalDays := StrToInt(EditAccountsIntervalDays.Text);
  end;
  inherited;
end;

end.
