{*****************************************************************************}
{                                                                             }
{   This file is part of the LKAccounts components Library                    }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for Journaling adjustmnts between Accounts in a TLKLedgerNominal  }
{                                                                             }
{*****************************************************************************}

unit LKJournalNomCostDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, Dialogs,
  Buttons, ExtCtrls, DB, DBTables, Grids, DBGrids,
  LKDefine, LKRecordsComponents, ImgList, ActnList, Variants,
  DBClient, LKAccountsComponents, Mask, JvExMask, JvToolEdit, Vcl.DBCtrls;

type
  TDlgNominalAndCostJournal = class(TForm)
    DBNLPostingsGrid: TDBGrid;
    DataSource1: TDataSource;
    ImageList1: TImageList;
    TempJournal: TClientDataSet;
    Panel1: TPanel;
    btnSkeleton: TButton;
    Label8: TLabel;
    EditDescription: TEdit;
    Label6: TLabel;
    Panel2: TPanel;
    BtnCancel: TButton;
    BtnNext: TButton;
    Label7: TLabel;
    EditAuthor: TEdit;
    Label3: TLabel;
    lToAllocate: TLabel;
    TempJournalNOMINAL_CODE: TStringField;
    TempJournalCOST_CENTRE: TStringField;
    TempJournalDESCRIPTION: TStringField;
    TempJournalDEBIT: TCurrencyField;
    TempJournalCREDIT: TCurrencyField;
    EditDate: TJvDateEdit;
    btnNomSelect: TButton;
    btnCostSelect: TButton;
    DBNavigator1: TDBNavigator;
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure EditDescriptionExit(Sender: TObject);
    procedure BtnNextClick(Sender: TObject);
    procedure TempJournalAfterPost(DataSet: TDataSet);
    procedure TempJournalBeforePost(DataSet: TDataSet);
    procedure btnSkeletonClick(Sender: TObject);
    procedure DBNLPostingsGridExit(Sender: TObject);
    procedure TempJournalAfterInsert(DataSet: TDataSet);
    procedure btnNomSelectClick(Sender: TObject);
    procedure btnCostSelectClick(Sender: TObject);
    procedure EditDateExit(Sender: TObject);
    procedure EditAuthorKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
    FMyJournal: TLKJournal;
    FDesc: string;
    FJournalDate: TDateTime;
    function GetNominalEqualsZero: boolean;
    property NominalEqualsZero: boolean read GetNominalEqualsZero;
    procedure ActionUseExecute(Sender: TObject);
    function CheckForFinish: Boolean;
  public
    { Public declarations }
    procedure Construct(Sender: TLKJournal);
  end;

var
  DlgNominalAndCostJournal: TDlgNominalAndCostJournal;

implementation

uses LKutils, LKUtilsDB, LKJournalDlg;

{$R *.DFM}

procedure TDlgNominalAndCostJournal.FormKeyPress(Sender: TObject; var Key:
  Char);
begin
  MoveMyFocus(Self, Key);
end;

function TDlgNominalAndCostJournal.GetNominalEqualsZero: boolean;
var
  Dr, Cr: currency;
  MyBookMark: TBookmark;
begin
  with TempJournal do //if  then
  begin
    Result := (RecordCount > 0);
    if not Result then
      Exit;
    MyBookmark := GetBookmark;
    DisableControls;
    try
      if FindFirst then
      begin
        Dr := 0.0;
        Cr := 0.0;
        repeat
          Dr := Dr + GetCurrency(FieldByName('DEBIT'));
          Cr := Cr + GetCurrency(FieldByName('CREDIT'));
        until not FindNext;
        Result := CurrencyIsZero(Dr - Cr) and (Dr <> 0);
        lToAllocate.Caption := FloatToStrF(Dr - Cr, ffCurrency, 15, 2);
      end;
    finally
      try
        GoToBookmark(myBookmark);
      except;
      end;
      EnableControls;
      FreeBookmark(MyBookmark);
    end;
  end;
end;

procedure TDlgNominalAndCostJournal.TempJournalBeforePost(
  DataSet: TDataSet);
var
  NomCode, CostCentre: string;
begin
  with TLKTable(Dataset) do
  begin
    if FieldByName('NOMINAL_CODE').IsNull then
    begin
      if (FieldByName('DEBIT').IsNull and FieldByName('CREDIT').IsNull) then
        Exit
      else
        raise LKError.Create('You must enter a nominal code');
    end;
    NomCode := FieldByName('NOMINAL_CODE').AsString;
    CostCentre := FieldByName('COST_CENTRE').AsString;
  end;
  with FMyJournal.NominalLedger do
  begin
    if not AccountIsValid(VarArrayOf([NomCode, ''])) then
    begin
      btnNomSelect.Click;
      SysUtils.Abort;
    end;
    if not CostLedger.AccountIsValid(CostCentre) then
    begin
      btnCostSelect.Click;
      SysUtils.Abort;
    end;
  end;
end;

procedure TDlgNominalAndCostJournal.TempJournalAfterPost(DataSet: TDataSet);
begin
  FDesc := Dataset.FieldByName('DESCRIPTION').asString;
  if NominalEqualsZero then
    EditAuthor.SetFocus;
end;

function TDlgNominalAndCostJournal.CheckForFinish: Boolean;
begin
  Result := (EditDate.Text > '')
            and NominalEqualsZero
            and (FDesc > '')
            and (EditAuthor.Text > '');
end;

procedure TDlgNominalAndCostJournal.Construct(Sender: TLKJournal);
begin
  FMyJournal := Sender;
  FDesc := '';
//  while TempJournal.FindLast do
//    TempJournal.Delete;
  FJournalDate := FMyJournal.NominalLedger.LedgerDate;
  if (FJournalDate > Date) then
    FJournalDate := Date;
  EditDate.Text := DateToStr(FJournalDate);
end;

procedure TDlgNominalAndCostJournal.EditDateExit(Sender: TObject);
begin
  if (EditDate.Text > '') then
  begin
    FJournalDate := StrTodate(EditDate.Text);
    EditDate.Text := DateToStr(FJournalDate);
    DBNLPostingsGrid.OnExit(Sender);
  end;
end;

procedure TDlgNominalAndCostJournal.EditDescriptionExit(Sender: TObject);
begin
  FDesc := EditDescription.Text;
  DBNLPostingsGrid.OnExit(Sender);
end;

procedure TDlgNominalAndCostJournal.DBNLPostingsGridExit(Sender: TObject);
begin
  with btnNext do
  begin
    Enabled := CheckForFinish;
    if Enabled then
      SetFocus;
  end;
end;

procedure TDlgNominalAndCostJournal.BtnNextClick(Sender: TObject);
var
  DrValue, CrValue: Currency;
  JnlNo, NominalCode, CostCentre: string;
  Item: integer;
begin
  BtnNext.Enabled := false;
  try
    with TempJournal do
      if State in [dsEdit, dsInsert] then
        Post;
    // enter Journal master entry
    with FMyJournal do
    begin
      JnlNo := IntToStr(SequenceNumberNext);
      InsertRecord([JnlNo, Null, FJournalDate, EditDescription.Text,
        EditAuthor.Text]);
      JnlNo := TblMaster.Fields[0].AsString;
    end;
    Item := 1;
    with TempJournal do
      if FindFirst then
        repeat
          NominalCode := FieldByName('NOMINAL_CODE').asString;
          CostCentre := FieldByName('COST_CENTRE').asString;
          FDesc := FieldByName('DESCRIPTION').asString;
          DrValue := getCurrency(FieldByName('DEBIT'));
          CrValue := getCurrency(FieldByName('CREDIT'));
          if not (CurrencyIsZero(DrValue) and CurrencyIsZero(CrValue)) then
          begin
            FMyJournal.TblDetail.AppendRecord([JnlNo, Item, NominalCode,
              CostCentre, Null,
              Null, FDesc, CurrencyOrNull(DrValue), CurrencyOrNull(CrValue)]);
            Inc(Item);
          end;
        until not FindNext;
    FMyJournal.ApplyUpdates(0);
  except
    FMyJournal.RecordDelete;
    raise;
  end;
  with FMyJournal.Daybook do
  begin
    // BatchDate := LedgerDate;
    ProcessBatch(Self);
  end;
  ModalResult := mrOK;
end;

procedure TDlgNominalAndCostJournal.btnSkeletonClick(Sender: TObject);
begin
  with FMyJournal.SkeletonJournals do
  begin
    Open;
    if not Assigned(FormView) then
      CreateViewForm;
    with TFormLKJournal(FormView).ActionButtonOK do
    begin
      Caption := '&Use';
      OnExecute := ActionUseExecute;
    end;
    View(Null, True, [mbOK, mbCancel]);
    Close;
  end;
end;

procedure TDlgNominalAndCostJournal.ActionUseExecute(Sender: TObject);
begin
  with FMyJournal.SkeletonJournals do
  begin
    with TblMaster do
    begin
      if (recNo < 1) then
        raise LKError.Create('No skeleton selected!');
      with EditDescription do
        if (Text = '') then
          Text := FieldByName('DESCRIPTION').asString;
    end;
    with TempJournal do
      while findFirst do
        delete;
    with TblDetail do
      if findFirst then
        repeat
          TempJournal.append;
          TempJournal.FieldByName('NOMINAL_CODE').asString :=
            FieldByName('NOMINAL_CODE').asString;
          TempJournal.FieldByName('COST_CENTRE').asString :=
            FieldByName('COST_CENTRE').asString;
          TempJournal.FieldByName('DESCRIPTION').asString :=
            FieldByName('DESCRIPTION').asString;
          TempJournal.FieldByName('DEBIT').value := FieldByName('DEBIT').value;
          TempJournal.FieldByName('CREDIT').value :=
            FieldByName('CREDIT').value;
          TempJournal.post;
        until not findNext;
    FormView.ModalResult := mrOK;
  end;
end;

procedure TDlgNominalAndCostJournal.TempJournalAfterInsert(
  DataSet: TDataSet);
begin
  Dataset.FieldByName('DESCRIPTION').asString := FDesc;
end;

procedure TDlgNominalAndCostJournal.EditAuthorKeyUp(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  btnNext.Enabled := CheckForFinish;
end;

procedure TDlgNominalAndCostJournal.btnNomSelectClick(Sender: TObject);
begin
  FMyJournal.NominalLedger.NominalCodes.View(Null, True, [mbOK]);
  with TempJournal do
    if State = dsBrowse then
      if RecordCount = 0 then
        Insert
      else
        Edit;
  with DBNLPostingsGrid do
  begin
    Columns[0].Field.AsString :=
      FMyJournal.NominalLedger.NominalCodes.TblMaster.FieldByName('NOMINAL_CODE').AsString;
    SetFocus;
  end;
end;

procedure TDlgNominalAndCostJournal.btnCostSelectClick(Sender: TObject);
begin
  FMyJournal.NominalLedger.CostLedger.CostCentres.View(Null, True, [mbOK]);
  with TempJournal do
    if State = dsBrowse then
      if RecordCount = 0 then
        Insert
      else
        Edit;
  with DBNLPostingsGrid do
  begin
    Columns[1].Field.AsString :=
      FMyJournal.NominalLedger.CostLedger.CostCentres.TblMaster.FieldByName('COST_CENTRE').AsString;
    SetFocus;
  end;
end;

end.

