{*****************************************************************************}
{                                                                             }
{   This file is part of the LKAccounts components Library                    }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for setting the properties of a TLKLedgerAnalysis                 }
{                                                                             }
{*****************************************************************************}

unit LKLedgerAnalysisSetupWizDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, System.UITypes,
  Graphics, Controls, Forms,
  Dialogs, LKHeaderDlg, ImgList, ActnList, StdCtrls, ExtCtrls,
  LKRecordsComponents, LKAccountsComponents, LKdlgs, StdActns, LKBaseComponent;

type

  TFormLKLedgerAnalysisSetupWiz = class(TFormLKHeaderDlg)
    Panel2: TPanel;
    procedure btnOKClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    AccrualsPanel: TLKWizPanel;
    PanelCount: Integer;
    FLedger: TLKLedgerAnalysis;
    PandLPanel: TLKWizPanel;
    PrePaymentsPanel: TLKWizPanel;
    SuspensePanel: TLKWizPanel;
  public
    procedure Construct(ALedger: TLKLedgerAnalysis);
    procedure DoAccrualsPanelClick(Sender: TObject);
    procedure DoPandLPanelClick(Sender: TObject);
    procedure DoPrePaymentsPanelClick(Sender: TObject);
    procedure DoSuspensePanelClick(Sender: TObject);
  end;

var
  FormLKLedgerAnalysisSetupWiz: TFormLKLedgerAnalysisSetupWiz;

implementation

{$R *.dfm}

uses LKHubSetupWizardDlg, LKWikiURIs;

{
************************ TFormLKLedgerAnalysisSetupWiz ************************
}

procedure TFormLKLedgerAnalysisSetupWiz.btnOKClick(Sender: TObject);
begin
  inherited;
  //
end;

procedure TFormLKLedgerAnalysisSetupWiz.Construct(ALedger:
  TLKLedgerAnalysis);
var
  ActivePanelCount: Integer;
  I: Integer;

  procedure AddHeadingPanel(const ACaption: string);
  begin
    Inc(PanelCount);
    with TPanel.Create(Self) do
    begin
      BorderStyle := bsNone;
      Height := 32;
      Parent := Panel2;
      Top := 60 + PanelCount * 32;
      Align := alTop;
      Alignment := taLeftJustify;
      Caption := ACaption;
      Font.Style := Font.Style + [fsBold];
    end;
  end;

  procedure AddSetupPanel(AComponent: TLKSystemComponent;
    AHelpStr: string = ''; AHelpURI: string = '');
  begin
    if not AComponent.DlgSetUpIsImplemented then
      Exit;
    Inc(PanelCount);
    Inc(ActivePanelCount);
    with TLKACSetupWizPanel.Create(Self) do
    begin
      Parent := Panel2;
      Top := 60 + PanelCount * 32;
      Align := alTop;
      LabelItemNo.Caption := IntToStr(ActivePanelCount) + '.';
      LKComponent := AComponent;
      HelpStr := AHelpStr;
      HelpURI := AHelpURI;
    end;
  end;

  function AddNLCodePanel(ACaption: string; AHelpStr: string = ''): TLKWizPanel;
  begin
    Inc(PanelCount);
    Inc(ActivePanelCount);
    Result := TLKWizPanel.Create(Self);
    with Result do
    begin
      Parent := Panel2;
      Top := 60 + PanelCount * 32;
      Align := alTop;
      LabelItemNo.Caption := IntToStr(ActivePanelCount) + '.';
      ButtonCaption := ACaption;
      HelpStr := AHelpStr;
      HelpURI := URIZoomUserNominalControlAccounts;
    end;
  end;

begin
  FLedger := ALedger;
  Caption := ALedger.NameFriendly;
  LKHeaderPanel1.Caption := ALedger.NameFriendly + ' Setup';
  PanelCount := 0;
  ActivePanelCount := 0;
  AddHeadingPanel('Management Accounts layout and Ledger coding:');
  if ALedger is TLKLedgerNominal then
  begin
    with TLKLedgerNominal(ALedger) do
    begin
      AddSetUpPanel(ALedger.ReportLines,
        'Enter the Nominal Codes and the layouts for Management Accounts',
        URIZoomUserNominalCodesAndLayouts);
      AddHeadingPanel('Set Balance Sheet Nominal Codes for:');
      SuspensePanel := AddNLCodePanel('Suspense postings account',
        'Enter the account for Suspense Postings where transactions will be '
        + 'posted if a valid Nominal account is not found.');
      SuspensePanel.OnButtonClick := DoSuspensePanelClick;
      AccrualsPanel := AddNLCodePanel('Accruals creditor',
        'Enter the account for the total Accruals creditor'
        + ' (reserves for expected future costs)');
      AccrualsPanel.OnButtonClick := DoAccrualsPanelClick;
      PrePaymentsPanel := AddNLCodePanel('Prepayments debtor',
        'Enter the account for the total Prepayments debtor'
        + ' (reserves for future costs paid in advance)');
      PrePaymentsPanel.OnButtonClick := DoPrePaymentsPanelClick;
      PandLPanel := AddNLCodePanel('Profit and Loss account',
        'Enter the account for the Profit and Loss account'
        + ' on the Balance Sheet.');
      PandLPanel.OnButtonClick := DoPandLPanelClick;
      AddHeadingPanel('Opening balances and ongoing reserves:');
      AddSetUpPanel(OpeningBalances,'Enter the Nominal ledger Opening debtor, '
        + 'creditor and asset Balances from the previous accounting period',
        URIZoomUserNominalOpeningBalances);
      AddSetUpPanel(Reserves, 'Enter your Ficed assets, depreciation, Prepaid '
        + 'expenses and Accrued liabilities',URIZoomUserNominalReserves);
      AddSetUpPanel(Budget, 'Enter your Budget for the current and future '
        + 'Accounting periods so you can report on Actual to Budget variances.',
        URIZoomUserNominalForecastsBudgets);
    end;
    with ALedger.AccountsHub.Ledgers do
    begin
    AddHeadingPanel('Set Balance Sheet Control Account Codes for:');
    for I := 0 to Count - 1 do
      If (Items[I] is TLKLedgerNominalControl) then
        AddSetupPanel(Items[I], 'Enter the Ledger Balance Sheet Nominal Code '
         + 'which will hold the net total of the ledger balances',
         URIZoomUserNominalControlAccounts);
    end;
  end else AddSetUpPanel(ALedger.ReportLines);
end;

procedure TFormLKLedgerAnalysisSetupWiz.DoAccrualsPanelClick(Sender: TObject);
begin
  with TLKLedgerNominal(FLedger) do
    CodeAccruals := DlgNominalCodeSelectExecute(
      AccrualsPanel.ButtonCaption, CodeAccruals, AccrualsPanel.HelpStr,
      False);
end;

procedure TFormLKLedgerAnalysisSetupWiz.DoPandLPanelClick(Sender: TObject);
begin
  with TLKLedgerNominal(FLedger) do
    CodeProfitAndLoss := DlgNominalCodeSelectExecute(
      PandLPanel.ButtonCaption, CodeProfitAndLoss, PandLPanel.HelpStr,
      False);
end;

procedure TFormLKLedgerAnalysisSetupWiz.DoPrePaymentsPanelClick(Sender:
  TObject);
begin
  with TLKLedgerNominal(FLedger) do
    CodePrepayments := DlgNominalCodeSelectExecute(
      PrePaymentsPanel.ButtonCaption, CodePrepayments,
        PrePaymentsPanel.HelpStr,
      false);
end;

procedure TFormLKLedgerAnalysisSetupWiz.DoSuspensePanelClick(Sender: TObject);
begin
  with TLKLedgerNominal(FLedger) do
    SuspensePostingsCode := DlgNominalCodeSelectExecute(
      SuspensePanel.ButtonCaption, SuspensePostingsCode,
        SuspensePanel.HelpStr,
      False);
end;

procedure TFormLKLedgerAnalysisSetupWiz.FormCreate(Sender: TObject);
begin
  inherited;
  LKHeaderPanel1.Comments.Add('Procedures to set up your Accounts analysis');
end;

procedure TFormLKLedgerAnalysisSetupWiz.FormShow(Sender: TObject);
begin
  inherited;
  ClientHeight := Panel1.Height + PanelCount * 32;
end;

end.

