{*****************************************************************************}
{                                                                             }
{   This file is part of the LKAccountsC components Library                    }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for display of a TLKLedgerBank                                    }
{                                                                             }
{*****************************************************************************}

unit LKLedgerBankDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  System.Contnrs, System.Actions,
  StdCtrls, ExtCtrls, LKdlgs, ImgList, Menus, ActnList, Db,
  ComCtrls, DBCtrls, Buttons, ToolWin, Variants,
  LKRecordsComponents, LKAccountsComponents, LKLedgerCashDlg, JvExExtCtrls,
  JvNetscapeSplitter, LKDBControls, LKUtilComponents,
  Grids, DBGrids, JvExDBGrids, JvDBGrid, JvDBUltimGrid;

type
  TFormBankLedger = class(TFormCashLedger)
    ActionBankPayment: TAction;
    ActionBankAgreement: TAction;
    sbAgreement: TToolButton;
    procedure ActionBankPaymentExecute(Sender: TObject);
    procedure ActionBankAgreementExecute(Sender: TObject);
  private
    { Private declarations }
    MyAccountsComponent: TLKLedgerBank;
  public
    { Public declarations }
    procedure Construct(Sender: TLKLedgerBank);
  end;

var
  FormBankLedger: TFormBankLedger;

implementation

{$R *.DFM}

procedure TFormBankLedger.Construct(Sender: TLKLedgerBank);
var
  J, I: Integer;
  NewAction: TAction;
  NewMenuitem: TMenuItem;
  MyContractLedger: TLKLedgerContact;
  MyPmtBankBook: TLKDocumentsBankBook;
begin
  MyAccountsComponent := Sender;
  inherited Construct(Sender);
  for I := 0 to pmNewDocPopup.Items.Count - 1 do
  begin
    if
      (TComponent(MyAccountsComponent.ChildComponents[pmNewDocPopup.Items[I].Tag])
      is TLKDocumentsBankBook) then
    begin
      with pmNewDocPopup.Items[I] do
      begin
        Action := nil;
        OnClick := nil;
      end;
      NewAction := TAction.Create(Self);
      with NewAction do
      begin
        ActionList := ActionListAcsComp;
        Caption := 'Nominal Ledger';
        Hint := 'General payment';
        ImageIndex := 14;
        Tag := -1;
        OnExecute := ActionBankPaymentExecute;
      end;
      NewMenuitem := TMenuitem.create(pmNewDocPopup.Items[I]);
      with NewMenuitem do
      begin
        Action := NewAction;
        Tag := -1;
      end;
      NewMenuitem.Action := NewAction;
      pmNewDocPopup.Items[I].Add(NewMenuitem);

      MyPmtBankBook :=
        TLKDocumentsBankBook(MyAccountsComponent.ChildComponents[pmNewDocPopup.Items.Tag]);
      with MyPmtBankBook.AccountsHub.Ledgers do
        for J := 0 to Count - 1 do
          if (TComponent(Items[J]) is TLKLedgerContact) then
          begin
            MyContractLedger := TLKLedgerContact(Items[J]);
            NewAction := TAction.Create(Self);
            with NewAction do
            begin
              ActionList := ActionListAcsComp;
              Caption := MyContractLedger.NameFriendly;
              Hint := 'Enter new payment on ' + MyContractLedger.NameFriendly;
              ImageIndex := 14;
              Tag := J;
              OnExecute := ActionBankPaymentExecute;
            end;
            NewMenuitem := TMenuitem.create(pmNewDocPopup.Items[I]);
            with NewMenuitem do
            begin
              Action := NewAction;
              Tag := J;
            end;
            pmNewDocPopup.Items[I].Add(NewMenuitem);
          end;
    end;
  end;
end;

procedure TFormBankLedger.ActionBankPaymentExecute(Sender: TObject);
var
  MyContractLedger: TLKLedgerContact;
  MyPmtBankBook: TLKDocumentsBankBook;
  MyMenu: TComponent;
  MyCLAccount: variant;
begin
  inherited;
  MyMenu := TAction(Sender).ActionComponent;
  MyPmtBankBook :=
    TLKDocumentsBankBook(MyAccountsComponent.ChildComponents[MyMenu.Owner.Tag]);
  if (MyMenu.Tag < 0) then
    MyPmtBankBook.RecordNew
  else
  begin
    MyContractLedger := TLKLedgerContact(MyPmtBankBook.AccountsHub.Ledgers[MyMenu.Tag]);
    MyCLAccount := null;
    if (MyContractLedger.DlgSelectRecordExecute('Select account', MyCLAccount) = mrOK) then
      MyPmtBankBook.DlgContactLedgerPaymentExecute(self, MyContractLedger);
  end;
end;

procedure TFormBankLedger.ActionBankAgreementExecute(Sender: TObject);
begin
  inherited;
  MyAccountsComponent.DlgBankAgreementExecute;
end;

end.

