{*****************************************************************************}
{                                                                             }
{   This file is part of the LKAccounts components Library                    }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for display of a TLKLedgerBank record                             }
{                                                                             }
{*****************************************************************************}

unit LKLedgerBankRecordDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  System.Contnrs, System.Actions,
  Dialogs, Menus, ImgList, ActnList, DB,  Grids, DBGrids,
  StdCtrls, ComCtrls, DBCtrls, Buttons, ExtCtrls, ToolWin,
  LKRecordsComponents, LKAccountsComponents, LKLedgerNomControlRecordDlg,
  LKdlgs, LKDBControls, LKUtilComponents, JvExDBGrids, JvDBGrid, JvDBUltimGrid;

type
  TFormLKLedgerBankRecord = class(TFormLKLedgerNominalControlRecord)
    ActionBankPayment: TAction;
    ActionBankAgreement: TAction;
    BankReconciliation1: TMenuItem;
    sbReconciliation: TToolButton;
    procedure ActionBankPaymentExecute(Sender: TObject);
    procedure ActionBankAgreementExecute(Sender: TObject);
  private
    { Private declarations }
    MyAccountsComponent: TLKLedgerBank;
  public
    { Public declarations }
    procedure Construct(Sender: TLKLedgerBank);
  end;

var
  FormLKLedgerBankRecord: TFormLKLedgerBankRecord;

implementation

{$R *.dfm}

procedure TFormLKLedgerBankRecord.Construct(Sender: TLKLedgerBank);
var
  x, i: Integer;
  NewAction: TAction;
  NewMenuitem: TMenuItem;
  myCL: TLKLedgerContact;
  myPmt: TLKDocumentsBankBook;
begin
  MyAccountsComponent := Sender;
  inherited Construct(Sender);
  for i := 0 to pmNewDocPopup.Items.Count - 1 do
  begin
    if
      (TComponent(MyAccountsComponent.ChildComponents[pmNewDocPopup.Items[i].Tag])
      is TLKDocumentsBankBook) then
    begin
      with pmNewDocPopup.Items[i] do
      begin
        Action := nil;
        OnClick := nil;
      end;
      NewAction := TAction.Create(Self);
      with NewAction do
      begin
        ActionList := ActionListAcsComp;
        Caption := 'Nominal Ledger';
        Hint := 'General payment';
        ImageIndex := 14;
        Tag := -1;
        OnExecute := ActionBankPaymentExecute;
      end;
      NewMenuitem := TMenuitem.create(pmNewDocPopup.Items[i]);
      with NewMenuitem do
      begin
        Action := NewAction;
        Tag := -1;
      end;
      NewMenuitem.Action := NewAction;
      pmNewDocPopup.Items[i].Add(NewMenuitem);

      myPmt :=
        TLKDocumentsBankBook(MyAccountsComponent.ChildComponents[pmNewDocPopup.Items.Tag]);
      with myPmt.AccountsHub.Ledgers do
        for x := 0 to Count - 1 do
          if (TComponent(Items[x]) is TLKLedgerContact) then
          begin
            myCL := TLKLedgerContact(Items[x]);
            NewAction := TAction.Create(Self);
            with NewAction do
            begin
              ActionList := ActionListAcsComp;
              Caption := myCL.NameFriendly;
              Hint := 'Enter new payment on ' + myCL.NameFriendly;
              ImageIndex := 14;
              Tag := x;
              OnExecute := ActionBankPaymentExecute;
            end;
            NewMenuitem := TMenuitem.create(pmNewDocPopup.Items[i]);
            with NewMenuitem do
            begin
              Action := NewAction;
              Tag := x;
            end;
            pmNewDocPopup.Items[i].Add(NewMenuitem);
          end;
    end;
  end;
end;

procedure TFormLKLedgerBankRecord.ActionBankPaymentExecute(Sender: TObject);
var
  myCL: TLKLedgerContact;
  myPmt: TLKDocumentsBankBook;
  myMenu: TComponent;
  myCLAccount: variant;
begin
  inherited;
  myMenu := TAction(Sender).ActionComponent;
  myPmt :=
    TLKDocumentsBankBook(MyAccountsComponent.ChildComponents[myMenu.Owner.Tag]);
  if (myMenu.Tag < 0) then
    myPmt.RecordNew
  else
  begin
    myCL := TLKLedgerContact(myPmt.AccountsHub.Ledgers[myMenu.Tag]);
    myCLAccount := null;
    if (myCL.DlgSelectRecordExecute('Select account', myCLAccount) = mrOK) then
      myPmt.DlgContactLedgerPaymentExecute(self, myCL);
  end;
end;

procedure TFormLKLedgerBankRecord.ActionBankAgreementExecute(Sender: TObject);
begin
  inherited;
  MyAccountsComponent.DlgBankAgreementExecute;
end;

end.

