{ ***************************************************************************** }
{ }
{ This file is part of the LKAccounts components Library }
{ }
{ See the file COPYING.LGPL.txt included in this distribution, }
{ for details about the Licence and Copyright. }
{ }
{ This code is distributed in the hope that it will be useful, }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. }
{ }
{ Author: Leslie John Kaye :    www.leskaye.com }
{ }
{ Abstract: }
{ Form for reconciling a TLKLedgerBank to bank Statements }
{ }
{ ***************************************************************************** }

unit LKLedgerBankUpOsDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, Dialogs,
  Buttons, ExtCtrls, ComCtrls, Grids, DBGrids, Db, LKBaseComponent,
  Messages, ShellAPI, DBCtrls, Variants, LKRecordsComponents,
  LKAccountsComponents;

type
  TDlgBankUnpresentedOutstanding = class(TForm)
    dsBankAccount: TDataSource;
    dsBankTrans: TDataSource;
    dbgBank: TDBGrid;
    Panel1: TPanel;
    BtnPrint: TButton;
    BtnClose: TButton;
    Panel2: TPanel;
    ckbxViewAll: TCheckBox;
    btnFind: TBitBtn;
    btnUndo: TBitBtn;
    DBText1: TDBText;
    eAgreementDate: TEdit;
    vdDate: TLKViewDialog;
    vdBalance: TLKViewDialog;
    Label1: TLabel;
    dlgFind: TLKViewDialog;
    GroupBox1: TGroupBox;
    dtpPresentedDate: TDateTimePicker;
    btnPlusDate: TButton;
    BitBtnEnterDate: TBitBtn;
    DBNavigator1: TDBNavigator;
    BitBtnEnterWithTransactionDate: TBitBtn;
    procedure ckbxViewAllClick(Sender: TObject);
    procedure UnpresentedBankLedgerDetailBeforePost(DataSet: TDataSet);
    procedure btnUndoClick(Sender: TObject);
    procedure BitBtnEnterDateClick(Sender: TObject);
    procedure btnFindClick(Sender: TObject);
    procedure dtpPresentedDateChange(Sender: TObject);
    procedure btnPlusDateClick(Sender: TObject);
    procedure dbgBankKeyPress(Sender: TObject; var Key: Char);
    procedure BtnPrintClick(Sender: TObject);
    procedure ePrefixKeyPress(Sender: TObject; var Key: Char);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure eAgreementDateExit(Sender: TObject);
    procedure BitBtnEnterWithTransactionDateClick(Sender: TObject);
  private
    { Private declarations }
    myBankLedger: TLKLedgerBank;
    dateCleared, lastDocumentDate: TDateTime;
    wasFiltered: boolean;
    oldFilter, lastDaybookID, lastDocumentNumber: string;
  public
    { Public declarations }
    agreementDate: TDateTime;
    statementBalance, adjustedBalance, BankledgerBalance: currency;
    function Construct(ALedgerBank: TLKLedgerBank): boolean;
  end;

var
  DlgBankUnpresentedOutstanding: TDlgBankUnpresentedOutstanding;

implementation

uses LKutils, LKUtilsDB, prLKBankAgreement;

{$R *.DFM}

function TDlgBankUnpresentedOutstanding.Construct
  (ALedgerBank: TLKLedgerBank): boolean;
begin
  dateCleared := date;
  myBankLedger := ALedgerBank;
  with myBankLedger do
  begin
    with vdDate do
    begin
      Value := LedgerDate;
      result := execute;
      if not result then
        exit;
      agreementDate := Value;
    end;
    eAgreementDate.Text := dateToStr(agreementDate);
    with vdBalance do
    begin
      result := execute;
      if not result then
        exit;
      statementBalance := Value;
    end;
    TblMasterDatasource := dsBankAccount;
    TblDetailDataSource := dsBankTrans;
    // TblDetail.IndexName := '';
    with TblDetail do
    begin
      oldFilter := Filter;
      wasFiltered := filtered;
      Filter := 'CLEARED <> NULL';
      filtered := true;
      if findLast then
        dateCleared := getdate(fieldByName('DATE_'));
      filtered := false;
      Filter := 'CLEARED = NULL';
      filtered := true;
    end;
    OKToEdit := true;
  end;
  dtpPresentedDate.DateTime := dateCleared;
end;

procedure TDlgBankUnpresentedOutstanding.ckbxViewAllClick(Sender: TObject);
begin
  myBankLedger.TblDetail.filtered := not ckbxViewAll.Checked;
end;

procedure TDlgBankUnpresentedOutstanding.UnpresentedBankLedgerDetailBeforePost
  (DataSet: TDataSet);
begin
  with TLKTable(DataSet) do
    if not fieldByName('CLEARED').isNull then
    begin
      lastDocumentDate := getdate(fieldByName('DATE_'));
      lastDaybookID := fieldByName('DAYBOOK_ID').asString;
      lastDocumentNumber := fieldByName('DOCUMENT_NUMBER').asString;
      btnUndo.Enabled := true;
    end;
end;

procedure TDlgBankUnpresentedOutstanding.btnUndoClick(Sender: TObject);
var
  bankAccountRef: string;
begin
  with myBankLedger do
  begin
    bankAccountRef := TblMaster.fields[0].asString;
    with TblDetail do
    begin
      filtered := false;
      if findKey([bankAccountRef, lastDocumentDate, lastDaybookID,
        lastDocumentNumber]) then
      begin
        edit;
        fieldByName('CLEARED').Value := null;
        post;
      end
      else
        showmessage('Could not find last record');
    end;
  end;
  btnUndo.Enabled := false;
  // reset the filter
  ckbxViewAllClick(Self);
end;

procedure TDlgBankUnpresentedOutstanding.BitBtnEnterWithTransactionDateClick(
  Sender: TObject);
begin
  dtpPresentedDate.DateTime :=
    myBankLedger.TblDetail.fieldByName('DATE_').AsDateTime;
  dtpPresentedDateChange(Sender);
  BitBtnEnterDateClick(Sender);
end;

procedure TDlgBankUnpresentedOutstanding.btnFindClick(Sender: TObject);
var
  doc: string;
begin
  with dlgFind do
  begin
    if not execute then
      exit;
    doc := Value;
  end;
  if myBankLedger.TblDetail.Locate('DOCUMENT_NUMBER', doc, []) then
    BitBtnEnterDate.SetFocus
  else
    showmessage(doc + ' not found');
end;

procedure TDlgBankUnpresentedOutstanding.BitBtnEnterDateClick(Sender: TObject);
begin
  with myBankLedger.TblDetail do
  begin
    edit;
    fieldByName('CLEARED').AsDateTime := dateCleared;
    post;
  end;
  dbgBank.SetFocus;
end;

procedure TDlgBankUnpresentedOutstanding.dtpPresentedDateChange
  (Sender: TObject);
begin
  dateCleared := dtpPresentedDate.DateTime;
end;

procedure TDlgBankUnpresentedOutstanding.btnPlusDateClick(Sender: TObject);
begin
  myBankLedger.ApplyUpdates(0);
  dateCleared := dateCleared + 1;
  dtpPresentedDate.DateTime := dateCleared;
end;

procedure TDlgBankUnpresentedOutstanding.dbgBankKeyPress(Sender: TObject;
  var Key: Char);
begin
  if (Key = chr(VK_RETURN)) then
    myBankLedger.TblDetail.Post;
end;

procedure TDlgBankUnpresentedOutstanding.BtnPrintClick(Sender: TObject);
begin
  BtnPrint.Enabled := false;
  try
    myBankLedger.ApplyUpdates(0);
    BankledgerBalance :=
      getCurrency(myBankLedger.TblMaster.fieldByName('ACCOUNT_BALANCE'));
    adjustedBalance := statementBalance;
    with TDlgPrintBankAgreement.Create(Self) do
    begin
      PrBankLedger := myBankLedger;
      frxReport1.ShowReport;
    end;
    if EqualCurrency(adjustedBalance, BankledgerBalance) then
      ModalResult := mrOK
    else
      Showmessage('Please locate the difference');
  finally
    BtnPrint.Enabled := true;
  end;
end;

procedure TDlgBankUnpresentedOutstanding.ePrefixKeyPress(Sender: TObject;
  var Key: Char);
begin
  if (Key = chr(VK_RETURN)) then
    SendMessage(Self.handle, WM_NEXTDLGCTL, 0, 0); // next tab
end;

procedure TDlgBankUnpresentedOutstanding.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  with myBankLedger do
  begin
    ApplyUpdates(0);
    with TblDetail do
    begin
      Filter := oldFilter;
      filtered := wasFiltered;
    end;
    OKToEdit := false;
  end;
  inherited;
end;

procedure TDlgBankUnpresentedOutstanding.eAgreementDateExit(Sender: TObject);
begin
  agreementDate := strToDate(eAgreementDate.Text);
end;

end.
