{*****************************************************************************}
{                                                                             }
{   This file is part of the LKAccounts components Library                    }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for allocating payments against invoice transactions in           }
{        a TLKLedgerContact                                                   }
{                                                                             }
{*****************************************************************************}

unit LKLedgerContactAllocateDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, Variants,
  Buttons, ExtCtrls, DB, DBClient, SqlExpr, Provider, Grids, DBGrids,
    dialogs,
  LKAccountsComponents, LKBaseComponent, LKFormBaseDlg, ImgList, ActnList,
  StdActns;

type
  TLKContactLedgerAllocateDlg = class(TFormLKBase)
    DBGrid1: TDBGrid;
    DataSource1: TDataSource;
    UnallocatedCashDlg: TLKViewDialog;
    Panel1: TPanel;
    btnCopyAcross: TBitBtn;
    lValueDescription: TLabel;
    lTotal: TLabel;
    btnUnallocated: TBitBtn;
    Panel2: TPanel;
    BtnBack: TButton;
    btnOK: TButton;
    btnCancel: TButton;
    lContactName: TLabel;
    btnGetPaid: TButton;
    TempMatchList: TClientDataSet;
    TempMatchListAccounts_reference: TStringField;
    TempMatchListDate_: TDateField;
    TempMatchListDaybookID: TStringField;
    TempMatchListDocument: TStringField;
    TempMatchListDescription: TStringField;
    TempMatchListDebit: TCurrencyField;
    TempMatchListCredit: TCurrencyField;
    TempMatchListUnpaid: TCurrencyField;
    TempMatchListMatch: TCurrencyField;
    TempMatchListURN: TIntegerField;
    BitBtnPost: TBitBtn;
    procedure TempMatchListAfterPost(DataSet: TDataSet);
    procedure btnCopyAcrossClick(Sender: TObject);
    procedure btnUnallocatedClick(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure btnGetPaidClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure BitBtnPostClick(Sender: TObject);
    procedure TempMatchListBeforeEdit(DataSet: TDataSet);
  private
    { Private declarations }
    IsPayDlg, UnmatchedIsDebit: boolean;
    myContactLedger: TLKLedgerContact;
    acRef: string;
    transactionDate: TDateTime;
    procedure GetItemsFromLedger(const ItemsUnpaid: boolean);
  public
    { Public declarations }
    TotalPaid, ValueUnallocated: currency;
    UnallocatedDescription: string;
    procedure Construct(const AContactLedger: TLKLedgerContact;
      ReturnValueMustBeZero, DebitUnmatchedToLedger: boolean; ATransactionDate:
        TDateTime);
  end;

var
  LKContactLedgerAllocateDlg: TLKContactLedgerAllocateDlg;

implementation

uses LKDefine, LKutils, LKUtilsDB;

{$R *.DFM}

procedure TLKContactLedgerAllocateDlg.GetItemsFromLedger(const ItemsUnpaid:
  boolean);
var
  d: TDateTime;
  doc, docNo, desc: string;
  dr, cr, unpaid: currency;
  urn: Integer;
begin
  with myContactledger.TblDetail do
  begin
    DisableControls;
    try
      if findFirst then
        repeat
          unpaid := getCurrency(fieldByName('UNPAID'));
          if ItemsUnpaid xor CurrencyIsZero(unpaid) then
          begin
            urn := GetInteger(FieldByName('URN'));
            d := getDate(fieldByName('DATE_'));
            doc := fieldByName('DAYBOOK_ID').asString;
            docNo := fieldByName('DOCUMENT_NUMBER').asString;
            desc := fieldByName('DESCRIPTION').asString;
            dr := getCurrency(fieldByName('DEBIT'));
            cr := getCurrency(fieldByName('CREDIT'));
            TempMatchList.AppendRecord([urn, acRef, d, doc, docNo, desc,
              CurrencyOrNull(dr), CurrencyOrNull(cr), unpaid]);
          end;
        until not FindNext;
      TempMatchList.FindFirst;
    finally EnableControls;
      first;
    end;
  end;
end;

procedure TLKContactLedgerAllocateDlg.Construct(const AContactLedger:
  TLKLedgerContact;
  ReturnValueMustBeZero, DebitUnmatchedToLedger: boolean; ATransactionDate:
    TDateTime);

begin
  TotalPaid := 0;
  ValueUnallocated := 0;
  UnmatchedIsDebit := DebitUnmatchedToLedger;
  myContactledger := AContactledger;
  transactionDate := ATransactionDate;
  UnallocatedDescription := 'Unallocated';
  with TempMatchList do
  begin
    Close;
    CreateDataSet;
    Open;
  end;
  with myContactledger do
  begin
    caption := NameFriendly;
    lContactName.Caption := tmAccountName;
    acRef := tmAccountReference;
  end;
  GetItemsFromLedger(true);
  IsPayDlg := not ReturnValueMustBeZero;
  if IsPayDlg then
  begin
    btnOK.Caption := '&Next >>';
    btnBack.Visible := true;
    caption := caption + ' payments';
    lValueDescription.Caption := 'Amount to pay';
    btnUnallocated.Visible := true;
  end
  else
    caption := caption + ' matching';
  BitBtnPost.Enabled := False;
end;

procedure TLKContactLedgerAllocateDlg.FormShow(Sender: TObject);
begin
  Top := 0;
  Height := Screen.Height;
end;

procedure TLKContactLedgerAllocateDlg.BitBtnPostClick(Sender: TObject);
begin
  with TempMatchList do
    if (State in [dsInsert, dsEdit]) then
      Post;
end;

procedure TLKContactLedgerAllocateDlg.btnCopyAcrossClick(Sender: TObject);
var
  curItem: TBookmark;
begin
  with TempMatchList do
  begin
    if (DBGrid1.SelectedRows.Count = 0) then
    begin
      Edit;
      FieldByName('MATCH').value := fieldByName('UNPAID').value;
      Post;
    end
    else
    begin
      curItem := GetBookmark;
      disableControls;
      try // if mutiselect
        if findFirst then
          repeat
            if DBGrid1.SelectedRows.CurrentRowSelected then
            begin
              edit;
              fieldByName('MATCH').value := fieldByName('UNPAID').value;
              post;
            end;
          until not findNext;
      finally enableControls;
        goToBookmark(curItem);
        freeBookmark(curItem);
      end;
    end;
  end;
  DBGrid1.SetFocus;
end;

procedure TLKContactLedgerAllocateDlg.TempMatchListAfterPost(
  DataSet: TDataSet);
var
  myBookMark: TBookmark;
begin
  BitBtnPost.Enabled := False;
  TotalPaid := -ValueUnallocated;
  with TempMatchList do
  begin
    myBookmark := getBookmark;
    disableControls;
    try
      if findFirst then
        repeat addFieldToCurr(fieldByName('MATCH'), TotalPaid);
        until not findNext;
    finally goToBookmark(myBookmark);
      enableControls;
      freeBookmark(myBookmark);
    end;
  end;
  lTotal.Caption := FloatToStrF(TotalPaid, ffCurrency, 15, 2);
  btnOK.Enabled := IsPayDlg xor floatIsZero(TotalPaid);
end;

procedure TLKContactLedgerAllocateDlg.TempMatchListBeforeEdit(
  DataSet: TDataSet);
begin
  BitBtnPost.Enabled := True;
end;

procedure TLKContactLedgerAllocateDlg.btnOKClick(Sender: TObject);
var
  urn: integer;
  matched: currency;
begin
  if IsPayDlg then
    exit;
  try
    with TempMatchList do
    begin
      if findFirst then
        repeat
          matched := getCurrency(fieldByName('MATCH'));
          if not CurrencyIsZero(matched) then
          begin
            urn := GetInteger(FieldByName('URN'));
            with myContactledger do
            begin
              OkToEdit := true;
              with TblDetail do
                if Locate('URN', urn, []) then
                begin
                  Edit;
                  AddCurrencyToField(-matched, fieldByName('UNPAID'));
                  if CurrencyIsZero(fieldByName('UNPAID').asCurrency) then
                    FieldByName('CLEARED').asDateTime := transactionDate;
                  Post;
                end;
              OkToEdit := false;
            end;
          end;
        until not FindNext;
    end;
  finally with myContactledger do
    begin
      ApplyUpdates(0);
      OkToEdit := false;
    end;
  end;
end;

procedure TLKContactLedgerAllocateDlg.btnUnallocatedClick(Sender: TObject);
var
  OldValue: currency;
begin
  OldValue := ValueUnallocated;
  with TLKViewDialog.Create(Self) do
  begin
    try
      Title := 'Unallocated';
      HeaderCaption := 'Enter the amount unallocated';
      HeaderComments := 'Unallocated amounts can be matched against transactions later'
        + ' using the Ledger Matching routine';
      Datatype := vCurrency;
      Value := ValueUnallocated;
      if Execute then
        ValueUnallocated := Value;
    finally Free;
    end;
  end;
  if CurrencyIsZero(ValueUnallocated) then
    exit;
  with TLKViewDialog.Create(Self) do
  begin
    try
      Title := 'Description';
      HeaderCaption := 'Description of the unallocated amount';
      HeaderComments := 'Unallocated amounts can be matched against transactions later'
        + ' using the Ledger Matching routine';
      Datatype := vString;
      Value := UnallocatedDescription;
      if Execute then
        UnallocatedDescription := Value;
    finally Free;
    end;
  end;
  if UnmatchedIsDebit then //normally negative value
    if (ValueUnallocated > 0.00) and
      not currencyIsZero(OldValue + ValueUnallocated)
        {// positive value re-entered} then
      ValueUnallocated := -ValueUnallocated;
  TempMatchListAfterPost(TempMatchList);
  btnOK.SetFocus;
end;

procedure TLKContactLedgerAllocateDlg.btnGetPaidClick(Sender: TObject);
begin
  btnGetPaid.Enabled := False;
  GetItemsFromLedger(False);
end;

end.

