{ ***************************************************************************** }
{ }
{ This file is part of the LKAccounts components Library }
{ }
{ See the file COPYING.LGPL.txt included in this distribution, }
{ for details about the Licence and Copyright. }
{ }
{ This code is distributed in the hope that it will be useful, }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. }
{ }
{ Author: Leslie John Kaye :    www.leskaye.com }
{ }
{ Abstract: }
{ Form for Journaling adjustments between Accounts in a TLKLedgerContact }
{ }
{ ***************************************************************************** }

unit LKLedgerContactJnlDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, DB, Dialogs, LKDefine, LKAccountsComponents, Variants,
  Mask, JvExMask, JvToolEdit, LKFormBaseDlg, ImgList, ActnList, StdActns;

type
  TDlgContactLedgerAcToAc = class(TFormLKBase)
    BtnNext: TButton;
    BtnCancel: TButton;
    Label1: TLabel;
    eValue: TEdit;
    Label2: TLabel;
    Label3: TLabel;
    cbxDrLedger: TComboBox;
    cbxCrLedger: TComboBox;
    eDrAccount: TEdit;
    lDrAccount: TLabel;
    lCrAccount: TLabel;
    eCrAccount: TEdit;
    Label6: TLabel;
    eDescription: TEdit;
    Label7: TLabel;
    eAuthor: TEdit;
    Label8: TLabel;
    btnDrLedgerSelect: TButton;
    btnCrLedgerSelect: TButton;
    eDate: TJvDateEdit;
    procedure FormCreate(Sender: TObject);
    procedure cbxDrLedgerChange(Sender: TObject);
    procedure eValueExit(Sender: TObject);
    procedure eDateExit(Sender: TObject);
    procedure eDrAccountChange(Sender: TObject);
    procedure eCrAccountChange(Sender: TObject);
    procedure eDrAccountExit(Sender: TObject);
    procedure eCrAccountExit(Sender: TObject);
    procedure eDescriptionExit(Sender: TObject);
    procedure BtnNextClick(Sender: TObject);
    procedure btnDrLedgerSelectClick(Sender: TObject);
    procedure btnCrLedgerSelectClick(Sender: TObject);
  private
    { Private declarations }
    FValue: double;
    FDrLedgerRef, FCrLedgerRef, FDrLedgerID, FCrLedgerID: string;
    FJournalDate: TDateTime;
    FDebitIsSelected, FCreditIsSelected: boolean;
    FDrLedger, FCrLedger: TLKLedgerContact;
    FMyJournal: TLKJournal;
    function CheckForFinish: Boolean;
  public
    { Public declarations }
    procedure Construct(AJournal: TLKJournal; ALedger: TLKLedgerContact);
  end;

var
  DlgContactLedgerAcToAc: TDlgContactLedgerAcToAc;

implementation

uses LKutils, LKUtilsDB, LKRecordsComponents;

{$R *.DFM}

procedure TDlgContactLedgerAcToAc.FormCreate(Sender: TObject);
begin
  FValue := 0.0;
  FDebitIsSelected := False;
  FCreditIsSelected := False;
end;

function TDlgContactLedgerAcToAc.CheckForFinish: boolean;
begin
  Result := FDebitIsSelected and FCreditIsSelected and (eDescription.Text > '')
    and (eAuthor.Text > '') and not currencyIsZero(FValue);
end;

procedure TDlgContactLedgerAcToAc.Construct(AJournal: TLKJournal;
  ALedger: TLKLedgerContact);
var
  I: integer;
begin
  FMyJournal := AJournal;
  FJournalDate := Date;
  FDrLedger := ALedger;
  FDrLedgerID := ALedger.LedgerID;
  FCrLedger := ALedger;
  FCrLedgerID := ALedger.LedgerID;
  with FDrLedger do
  begin
    Caption := NameFriendly + Caption;
    cbxDrLedger.Text := NameFriendly;
    eDrAccount.Text := TblMaster.fields[0].asString;
    eDrAccountChange(Self);
    if (FJournalDate > LedgerDate) then
      FJournalDate := LedgerDate;
    eDate.Text := dateToStr(FJournalDate);
    with AccountsHub do
    begin
      cbxDrLedger.Items.Clear;
      for I := 0 to Ledgers.Count - 1 do
        if (TLKLedgerBase(Ledgers[I]) is TLKLedgerContact) then
          cbxDrLedger.Items.Add(TLKLedgerContact(Ledgers[I]).NameFriendly);
    end;
  end;
  with cbxCrLedger do
  begin
    Items.Clear;
    Items.Assign(cbxDrLedger.Items);
    Text := FCrLedger.NameFriendly;
  end;
end;

procedure TDlgContactLedgerAcToAc.cbxDrLedgerChange(Sender: TObject);
var
  I: integer;
  MyLedger: TLKLedgerContact;
  MyCBX: TComboBox;
begin
  MyCBX := TComboBox(Sender);
  with FMyJournal.AccountsHub.Ledgers do
    for I := 0 to Count - 1 do
      if (Items[I].NameFriendly = MyCBX.Text) then
      begin
        MyLedger := TLKLedgerContact(Items[I]);
        with MyLedger do
          if (FJournalDate > LedgerDate) then
          begin
            FJournalDate := LedgerDate;
            eDate.Text := dateToStr(FJournalDate);
          end;
        if (MyCBX = cbxDrLedger) then
        begin
          FDrLedger := MyLedger;
          FDrLedgerID := MyLedger.LedgerID
        end
        else
        begin
          FCrLedger := MyLedger;
          FCrLedgerID := MyLedger.LedgerID;
        end;
        with MyLedger do
          if not Active then
            Open;
        Break;
      end;
end;

procedure TDlgContactLedgerAcToAc.eDateExit(Sender: TObject);
var
  NewDate: TDateTime;
begin
  NewDate := StrTodate(eDate.Text);
  with FDrLedger do
    if (NewDate > LedgerDate) then
      NewDate := LedgerDate;
  with FCrLedger do
    if (NewDate > LedgerDate) then
      NewDate := LedgerDate;
  FJournalDate := NewDate;
  eDate.Text := dateToStr(NewDate);
  BtnNext.enabled := CheckForFinish;
end;

procedure TDlgContactLedgerAcToAc.eValueExit(Sender: TObject);
begin
  FValue := strToFloat(eValue.Text);
  BtnNext.enabled := CheckForFinish;
end;

procedure TDlgContactLedgerAcToAc.eDrAccountChange(Sender: TObject);
begin
  with FDrLedger do
  begin
    FDebitIsSelected := Locate(TblMaster.fields[0].Fieldname, eDrAccount.Text,
      [loCaseInsensitive, loPartialKey]);
    if FDebitIsSelected then
    begin
      FDrLedgerRef := tmAccountReference;
      lDrAccount.Caption := tmAccountName;
    end;
  end;
end;

procedure TDlgContactLedgerAcToAc.eCrAccountChange(Sender: TObject);
begin
  with FCrLedger do
  begin
    FCreditIsSelected := Locate(TblMaster.fields[0].Fieldname, eCrAccount.Text,
      [loCaseInsensitive, loPartialKey]);
    if FCreditIsSelected then
    begin
      FCrLedgerRef := tmAccountReference;
      lCrAccount.Caption := tmAccountName;
    end;
  end;
end;

procedure TDlgContactLedgerAcToAc.eDrAccountExit(Sender: TObject);
begin
  BtnNext.enabled := CheckForFinish;
end;

procedure TDlgContactLedgerAcToAc.eCrAccountExit(Sender: TObject);
begin
  BtnNext.enabled := CheckForFinish;
end;

procedure TDlgContactLedgerAcToAc.btnDrLedgerSelectClick(Sender: TObject);
var
  V: variant;
begin
  V := null;
  with FDrLedger do
    if (DlgSelectRecordExecute('Select account', V) = mrOK) then
    begin
      eDrAccount.Text := tmAccountReference;
      eDrAccountChange(Sender);
    end;
end;

procedure TDlgContactLedgerAcToAc.btnCrLedgerSelectClick(Sender: TObject);
var
  V: variant;
begin
  V := null;
  with FCrLedger do
    if (DlgSelectRecordExecute('Select account', V) = mrOK) then
    begin
      eCrAccount.Text := tmAccountReference;
      eCrAccountChange(Sender);
    end;
end;

procedure TDlgContactLedgerAcToAc.eDescriptionExit(Sender: TObject);
begin
  BtnNext.enabled := CheckForFinish;
  if BtnNext.enabled then
    BtnNext.SetFocus;
end;

procedure TDlgContactLedgerAcToAc.BtnNextClick(Sender: TObject);
var
  D, JnlNo, DrNomCtrl, DrCost, CrNomCtrl, CrCost: string;
  NewDate: TDateTime;
begin
  NewDate := StrTodate(eDate.Text);
  if (NewDate < FJournalDate) then
    FJournalDate := NewDate;
  if (FDrLedgerID = FCrLedgerID) and (FDrLedgerRef = FCrLedgerRef) then
    raise LKError.Create('Debit and credit entries are to the same Account!');
  DecodeNominalAccount(FDrLedger.NominalControlCode, DrNomCtrl, DrCost);
  DecodeNominalAccount(FCrLedger.NominalControlCode, CrNomCtrl, CrCost);
  BtnNext.enabled := False;
  with FMyJournal do
  try
    JnlNo := intToStr(SequenceNumberNext);
    D := eDescription.Text;
    InsertRecord([JnlNo, null, FJournalDate, D, eAuthor.Text]);
    JnlNo := TblMaster.fields[0].asString;
    with TblDetail do
    begin
      D := eDrAccount.Text + '/' + eCrAccount.Text + ' ' + D;
      InsertRecord([JnlNo, 1, DrNomCtrl, DrCost, FDrLedgerID, FDrLedgerRef, D,
        FValue, null]);
      AppendRecord([JnlNo, 2, CrNomCtrl, CrCost, FCrLedgerID, FCrLedgerRef, D,
        null, FValue]);
    end;
    ApplyUpdates(0);
    Daybook.ProcessBatch(Self);
  except
    BtnNext.enabled := true;
    raise;
  end;
  ModalResult := mrOK;
end;

end.
