{*****************************************************************************}
{                                                                             }
{   This file is part of the LKAccounts components Library                    }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for entering Diary notes for records in a TLKLedgerNominalControl }
{                                                                             }
{*****************************************************************************}


unit LKLedgerNomAnalysisDiaryDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, DB, DBTables, Grids, DBGrids,
  DBClient, Provider, LKRecordsComponents, LKAccountsComponents;

type
  TLKLedgerNominalAnalysisDiaryDlg = class(TForm)
    OKBtn: TButton;
    DBGrid1: TDBGrid;
    DataSource1: TDataSource;
    TableDiaryDtl: TClientDataSet;
    TableDiaryDtlProvider: TDataSetProvider;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure TableDiaryDtlAfterInsert(DataSet: TDataSet);
    procedure TableDiaryDtlAfterPost(DataSet: TDataSet);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure DBGrid1Exit(Sender: TObject);
  private
    { Private declarations }
    myLedgerID, myLedgerRef: string;
    myLedger: TLKLedgerNominalControl;
    myDiary: TLKHubAccountsDiary;
  public
    { Public declarations }
    procedure Construct(Sender: TLKLedgerNominalControl);
  end;

var
  LKLedgerNominalAnalysisDiaryDlg: TLKLedgerNominalAnalysisDiaryDlg;

implementation

uses LKUtils, LKUtilsDB;

{$R *.dfm}

procedure TLKLedgerNominalAnalysisDiaryDlg.Construct(Sender:
  TLKLedgerNominalControl);
begin
  myLedger := Sender;
  with myLedger do
  begin
    myLedgerID := LedgerID;
    myLedgerRef := TblMaster.Fields[0].AsString;
  end;
  myDiary := myLedger.AccountsHub.Diary;
  TableDiaryDtlProvider.DataSet := myDiary.TblDetail.InternalDataSet;
  with TableDiaryDtl do
  begin
    Filter := '(Ledger_ID = ' + chr(39) + myLedgerID + chr(39)
      + ') AND (Ledger_reference = ' + chr(39) + myLedgerRef + chr(39) + ')';
    Filtered := true;
    Open;
  end;
end;

procedure TLKLedgerNominalAnalysisDiaryDlg.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  TableDiaryDtl.Close;
end;

procedure TLKLedgerNominalAnalysisDiaryDlg.TableDiaryDtlAfterInsert(
  DataSet: TDataSet);
begin
  with Dataset do
  begin
    FieldByName('Ledger_ID').asString := myLedgerID;
    FieldByName('Ledger_reference').AsString := myLedgerRef;
  end;
end;

procedure TLKLedgerNominalAnalysisDiaryDlg.TableDiaryDtlAfterPost(
  DataSet: TDataSet);
var
  myDiaryDate: TDateTime;
begin
  myDiaryDate := Getdate(Dataset.FieldByName('Diary_date'));
  with myDiary do
    if not Locate('Diary_date', myDiaryDate, []) then
      InsertRecord([myDiaryDate]);
end;

procedure TLKLedgerNominalAnalysisDiaryDlg.FormKeyPress(Sender: TObject;
  var Key: Char);
begin
  MoveMyFocus(Self, Key);
end;

procedure TLKLedgerNominalAnalysisDiaryDlg.DBGrid1Exit(Sender: TObject);
begin
  with TableDiaryDtl do
    if (state in [dsEdit, dsInsert]) then
      Post;
end;

end.

