{*****************************************************************************}
{                                                                             }
{   This file is part of the LKAccounts components Library                    }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for Journaling adjustments between Accounts in a                  }
{        TLKLedgerNominalControl                                              }
{                                                                             }
{*****************************************************************************}


unit LKLedgerNomControlJnlDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, DB, Dialogs, LKRecordsComponents, LKDefine,
    LKAccountsComponents, Variants,
  Mask, JvExMask, JvToolEdit;

type
  TDlgNomControlLedgerAcToAc = class(TForm)
    BtnNext: TButton;
    BtnCancel: TButton;
    Label1: TLabel;
    EditValue: TEdit;
    Label2: TLabel;
    Label3: TLabel;
    Label6: TLabel;
    EditDescription: TEdit;
    Label7: TLabel;
    EditAuthor: TEdit;
    Label8: TLabel;
    lDrAccount: TLabel;
    EditDrAccount: TEdit;
    btnDrLedgerSelect: TButton;
    lCrAccount: TLabel;
    EditCrAccount: TEdit;
    btnCrLedgerSelect: TButton;
    EditDate: TJvDateEdit;
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure FormCreate(Sender: TObject);
    procedure EditValueExit(Sender: TObject);
    procedure EditDrAccountChange(Sender: TObject);
    procedure EditCrAccountChange(Sender: TObject);
    procedure EditDrAccountExit(Sender: TObject);
    procedure EditCrAccountExit(Sender: TObject);
    procedure EditDescriptionExit(Sender: TObject);
    procedure BtnNextClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnDrLedgerSelectClick(Sender: TObject);
    procedure btnCrLedgerSelectClick(Sender: TObject);
    procedure EditDateExit(Sender: TObject);
    procedure EditAuthorKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
    Value: Currency;
    DrLedgerRef, CrLedgerRef, MyLedgerID: string;
    JournalDate: TDateTime;
    DebitIsSelected, CreditIsSelected: boolean;
    MyLedger: TLKLedgerNominalControl;
    HoldMasterComponent: TLKDataComponent;
    MyJournal: TLKJournal;
    function CheckForFinish: Boolean;
  public
    { Public declarations }
    procedure Construct(AJournal: TLKJournal; ALedger:
      TLKLedgerNominalControl);
  end;

var
  DlgNomControlLedgerAcToAc: TDlgNomControlLedgerAcToAc;

implementation

uses LKutils, LKUtilsDB;

{$R *.DFM}

procedure TDlgNomControlLedgerAcToAc.FormCreate(Sender: TObject);
begin
  Value := 0.0;
  DebitIsSelected := False;
  CreditIsSelected := False;
end;

procedure TDlgNomControlLedgerAcToAc.FormKeyPress(Sender: TObject; var Key:
  Char);
begin
  MoveMyFocus(self, key);
end;

function TDlgNomControlLedgerAcToAc.CheckForFinish: boolean;
begin
  Result := DebitIsSelected and CreditIsSelected and
    (EditDescription.Text > '') and
    (EditAuthor.Text > '') and not CurrencyIsZero(Value);
end;

procedure TDlgNomControlLedgerAcToAc.Construct(AJournal: TLKJournal; ALedger:
  TLKLedgerNominalControl);
begin
  MyJournal := AJournal;
  JournalDate := Date;
  MyLedger := ALedger;
  MyLedgerID := ALedger.LedgerID;
  with MyLedger do
  begin
    Caption := NameFriendly + Caption;
    HoldMasterComponent := MasterComponent;
    EditDrAccount.Text := tmAccountReference;
    EditDrAccountChange(Self);
    if (JournalDate > LedgerDate) then
      JournalDate := LedgerDate;
    EditDate.Text := dateToStr(JournalDate);
  end;
end;

procedure TDlgNomControlLedgerAcToAc.EditValueExit(Sender: TObject);
begin
  Value := strToFloat(EditValue.Text);
  BtnNext.enabled := CheckForFinish;
end;

procedure TDlgNomControlLedgerAcToAc.EditDrAccountChange(Sender: TObject);
begin
  with MyLedger do
  begin
    DebitIsSelected :=
      locate(TblMaster.fields[0].fieldname, EditDrAccount.Text,
        [loCaseInsensitive, loPartialKey]);
    if DebitIsSelected then
    begin
      DrLedgerRef := tmAccountReference;
      lDrAccount.caption := TmAccountName;
    end;
  end;
end;

procedure TDlgNomControlLedgerAcToAc.EditAuthorKeyUp(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  BtnNext.Enabled := CheckForFinish;
end;

procedure TDlgNomControlLedgerAcToAc.EditCrAccountChange(Sender: TObject);
begin
  with MyLedger do
  begin
    CreditIsSelected :=
      locate(TblMaster.Fields[0].fieldname, EditCrAccount.Text,
        [loCaseInsensitive, loPartialKey]);
    if CreditIsSelected then
    begin
      CrLedgerRef := tmAccountReference;
      lCrAccount.Caption := TmAccountName;
    end;
  end;
end;

procedure TDlgNomControlLedgerAcToAc.EditDrAccountExit(Sender: TObject);
begin
  BtnNext.enabled := CheckForFinish;
end;

procedure TDlgNomControlLedgerAcToAc.EditCrAccountExit(Sender: TObject);
begin
  BtnNext.enabled := CheckForFinish;
end;

procedure TDlgNomControlLedgerAcToAc.EditDescriptionExit(Sender: TObject);
begin
  BtnNext.Enabled := CheckForFinish;
  if BtnNext.Enabled then
    btnNext.SetFocus;
end;

procedure TDlgNomControlLedgerAcToAc.EditDateExit(Sender: TObject);
var
  NewDate: TDateTime;
begin
  NewDate := StrTodate(EditDate.Text);
  with MyLedger do
    if (NewDate > LedgerDate) then
      NewDate := LedgerDate;
  JournalDate := NewDate;
  EditDate.Text := DateToStr(newDate);
  BtnNext.enabled := CheckForFinish;
end;

procedure TDlgNomControlLedgerAcToAc.BtnNextClick(Sender: TObject);
var
  JnlNo, NomCtrl, Cost, D: string;
begin
  if (DrLedgerRef = CrLedgerRef) then
    raise
      LKError.Create('Debit and credit entries are to the same Account!');
  DecodeNominalAccount(MyLedger.NominalControlCode, NomCtrl, Cost);
  BtnNext.Enabled := false;
  with MyJournal do
  begin
    try
      JnlNo := intToStr(SequenceNumberNext);
      InsertRecord(
        [JnlNo, null, JournalDate, EditDescription.Text, EditAuthor.Text]);
      JnlNo := TblMaster.Fields[0].AsString;
      with TblDetail do
      begin
        D := EditDrAccount.Text + '/' + EditCrAccount.Text + ' '
          + EditDescription.Text;
        InsertRecord([JnlNo, 1, NomCtrl, Cost, MyLedgerID, DrLedgerRef, D,
          Value, Null]);
        AppendRecord([JnlNo, 2, NomCtrl, Cost, MyLedgerID, CrLedgerRef, D, Null,
          Value]);
      end;
      Daybook.ProcessBatch(Self);
    except
      MyJournal.RecordDelete;
      BtnNext.Enabled := true;
      raise;
    end;
  end;
  ModalResult := mrOK;
end;

procedure TDlgNomControlLedgerAcToAc.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  MyLedger.MasterComponent := HoldMasterComponent;
end;

procedure TDlgNomControlLedgerAcToAc.btnDrLedgerSelectClick(
  Sender: TObject);
var
  V: Variant;
begin
  V := Null;
  with MyLedger do
    if (DlgSelectRecordExecute('Select account', V) = mrOK) then
    begin
      EditDrAccount.Text := tmAccountReference;
      EditDrAccountChange(Sender);
    end;
end;

procedure TDlgNomControlLedgerAcToAc.btnCrLedgerSelectClick(
  Sender: TObject);
var
  V: Variant;
begin
  V := Null;
  with MyLedger do
    if (DlgSelectRecordExecute('Select account', V) = mrOK) then
    begin
      EditCrAccount.Text := TmAccountReference;
      EditCrAccountChange(Sender);
    end;
end;

end.

