{*****************************************************************************}
{                                                                             }
{   This file is part of the LKAccounts components Library                    }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for Journaling adjustments to Accounts in a                       }
{         TLKLedgerNominalControl against the Nominal Ledger                  }
{                                                                             }
{*****************************************************************************}


unit LKLedgerNomControlNomJnlDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, Dialogs,
  DB, DBTables, ExtCtrls, Grids, DBGrids, Variants, LKHeaderDlg,
  Buttons, LKRecordsComponents, LKAccountsComponents, DBClient, Mask, JvExMask,
  ImgList, ActnList, LKBaseComponent, JvToolEdit;

type
  TDlgNomControlLedgerAcToNom = class(TFormLKHeaderDlg)
    DBNLPostingsGrid: TDBGrid;
    DataSource1: TDataSource;
    TempJournal: TClientDataSet;
    Panel2: TPanel;
    Label8: TLabel;
    Label1: TLabel;
    EditValue: TEdit;
    Label2: TLabel;
    lCAccount: TLabel;
    EditCAccount: TEdit;
    RadioGroupDrCr: TRadioGroup;
    Label3: TLabel;
    Label7: TLabel;
    EditAuthor: TEdit;
    TempJournalNOMINAL_CODE: TStringField;
    TempJournalCOST_CENTRE: TStringField;
    TempJournalDESCRIPTION: TStringField;
    TempJournalDEBIT: TCurrencyField;
    TempJournalCREDIT: TCurrencyField;
    Label6: TLabel;
    EditDescription: TEdit;
    btnDrLedgerSelect: TButton;
    EditDate: TJvDateEdit;
    btnNomSelect: TButton;
    btnCostSelect: TButton;
    procedure FormCreate(Sender: TObject);
    procedure EditValueExit(Sender: TObject);
    procedure EditCAccountChange(Sender: TObject);
    procedure EditCAccountExit(Sender: TObject);
    procedure EditDescriptionExit(Sender: TObject);
    procedure BtnOKClick(Sender: TObject);
    procedure TempJournalAfterPost(DataSet: TDataSet);
    procedure TempJournalBeforePost(DataSet: TDataSet);
    procedure TempJournalAfterInsert(DataSet: TDataSet);
    procedure btnDrLedgerSelectClick(Sender: TObject);
    procedure btnNomSelectClick(Sender: TObject);
    procedure btnCostSelectClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure EditDateExit(Sender: TObject);
    procedure DBNLPostingsGridExit(Sender: TObject);
    procedure EditAuthorKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
    FValue: currency;
    FJournalDate: TDateTime;
    FAcSelected, FNominalEqualsValue: boolean;
    FCLedger: TLKLedgerNominalControl;
    FCLedgerID, FJnlDescription: string;
    FMyJournal: TLKJournal;
    function CheckForFinish: Boolean;
  public
    { Public declarations }
    procedure Construct(AJournal: TLKJournal; ALedger:
      TLKLedgerNominalControl);
  end;

var
  DlgNomControlLedgerAcToNom: TDlgNomControlLedgerAcToNom;

implementation

uses LKutils, LKUtilsDB, LKDefine;

{$R *.DFM}

procedure TDlgNomControlLedgerAcToNom.FormCreate(Sender: TObject);
begin    
  inherited;
  FValue := 0.0;
  FAcSelected := False;
  FNominalEqualsValue := False;
  FJnlDescription := '';
end;

procedure TDlgNomControlLedgerAcToNom.Construct(AJournal: TLKJournal; ALedger:
  TLKLedgerNominalControl);
begin
  FMyJournal := AJournal;
  FJournalDate := Date;
  FCLedger := ALedger;
  FCLedgerID := ALedger.LedgerID;
  with FCLedger do
  begin
    if not Active then
      Open;
    if (FJournalDate > LedgerDate) then
      FJournalDate := LedgerDate;
    Caption := NameFriendly + Caption;
    EditCAccount.Text := tmAccountReference;
    EditCAccountChange(Self);
  end;
  EditDate.Text := DateToStr(FJournalDate);
end;

procedure TDlgNomControlLedgerAcToNom.DBNLPostingsGridExit(Sender: TObject);
begin
  with TempJournal do
  begin
    if (State in [dsInsert, dsEdit]) then
      Post;
    if FieldByName('DEBIT').IsNull and FieldByName('CREDIT').IsNull then
      Delete;
  end;
end;

procedure TDlgNomControlLedgerAcToNom.FormShow(Sender: TObject);
begin
  inherited;
  BtnNomSelect.Top := DBNLPostingsGrid.Top + 3;
  BtnCostSelect.Top := BtnNomSelect.Top;
end;

procedure TDlgNomControlLedgerAcToNom.TempJournalBeforePost(
  DataSet: TDataSet);
var
  NomCode, CostCode: string;
begin
  with DataSet do
  begin
    if FieldByName('NOMINAL_CODE').IsNull then
      if (FieldByName('DEBIT').IsNull and FieldByName('CREDIT').IsNull) then
        Exit
      else
        raise LKError.Create('You must enter a nominal code');
    NomCode := FieldByName('NOMINAL_CODE').AsString;
    CostCode := FieldByName('COST_CENTRE').AsString;
  end;
  FMyJournal.NominalLedger.AccountIsValid(VarArrayOf([NomCode, CostCode]), True);
end;

procedure TDlgNomControlLedgerAcToNom.TempJournalAfterPost(DataSet: TDataSet);
var
  MyValue, Sum: currency;
  MyBookMark: TBookmark;
begin
  case RadioGroupDrCr.ItemIndex of
    0: MyValue := FValue;
  else
    MyValue := -FValue;
  end;
  Sum := 0.0;
  with DataSet do
    if (RecordCount > 0) then
    begin
      FJnlDescription := FieldByName('DESCRIPTION').asString;
      MyBookMark := GetBookmark;
      DisableControls;
      try
        FindFirst;
        repeat
          Sum := Sum + GetCurrency(FieldByName('DEBIT')) -
            GetCurrency(FieldByName('CREDIT'));
        until not FindNext;
      finally
        GoToBookmark(MyBookMark);
        EnableControls;
        FreeBookmark(MyBookMark);
      end;
    end;
  FNominalEqualsValue := CurrencyIsZero(MyValue + Sum);
  if FNominalEqualsValue then
    EditAuthor.SetFocus;
end;

function TDlgNomControlLedgerAcToNom.CheckForFinish: Boolean;
begin
  Result := FNominalEqualsValue
    and FAcSelected
    and (EditDescription.Text > '')
    and (EditAuthor.Text > '')
    and not CurrencyIsZero(FValue);
end;

procedure TDlgNomControlLedgerAcToNom.EditValueExit(Sender: TObject);
begin
  FValue := StrToFloat(EditValue.Text);
  TempJournalAfterPost(TempJournal);
  BtnOK.enabled := checkForFinish;
end;

procedure TDlgNomControlLedgerAcToNom.EditAuthorKeyUp(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  BtnOK.Enabled := CheckForFinish;
end;

procedure TDlgNomControlLedgerAcToNom.EditCAccountChange(Sender: TObject);
begin
  with FCLedger do
  begin
    FAcSelected :=
      Locate(TblMaster.Fields[0].Fieldname, EditCAccount.Text,
        [loCaseInsensitive, loPartialKey]);
    if FAcSelected then
    begin
      lCAccount.Caption := TmAccountName;
    end;
  end;
end;

procedure TDlgNomControlLedgerAcToNom.EditCAccountExit(Sender: TObject);
begin
  BtnOK.enabled := CheckForFinish;
end;

procedure TDlgNomControlLedgerAcToNom.btnDrLedgerSelectClick(
  Sender: TObject);
var
  V: Variant;
begin
  V := Null;
  with FCLedger do
    if (DlgSelectRecordExecute('Select account', V) = mrOK) then
    begin
      EditCAccount.Text := tmAccountReference;
      EditCAccountChange(Sender);
    end;
end;

procedure TDlgNomControlLedgerAcToNom.EditDescriptionExit(Sender: TObject);
begin
  FJnlDescription := EditDescription.Text;
  BtnOK.Enabled := CheckForFinish;
  if BtnOK.Enabled then
    BtnOK.SetFocus;
end;

procedure TDlgNomControlLedgerAcToNom.EditDateExit(Sender: TObject);
begin
  FJournalDate := StrToDate(EditDate.Text);
  BtnOK.Enabled := CheckForFinish;
end;

procedure TDlgNomControlLedgerAcToNom.BtnOKClick(Sender: TObject);
var
  DrValue, CrValue: currency;
  JnlNo, CRef, NominalDescription, NLID, NominalCode, CostCentre: string;
  Item: integer;
begin
  DecodeNominalAccount(FCLedger.NominalControlCode, NominalCode, CostCentre);
  CRef := FCLedger.tmAccountReference;
  case RadioGroupDrCr.ItemIndex of
    0:
      begin
        DrValue := FValue;
        CrValue := 0.0;
      end;
  else
    begin
      DrValue := 0.0;
      CrValue := FValue;
    end;
  end;
  FValue := DrValue - CrValue; // credit is -
  BtnOK.Enabled := False;
  try
    with FMyJournal do
    begin
      NLID := NominalLedger.ledgerID;
      // enter Journal master entry and  detail entry for ContactLedger
      JnlNo := IntToStr(SequenceNumberNext);
      Item := 1;
      InsertRecord(
        [JnlNo, Null, FJournalDate, EditDescription.Text, EditAuthor.Text]);
      JnlNo := TblMaster.Fields[0].AsString;
      TblDetail.InsertRecord([JnlNo, Item, NominalCode, CostCentre, FCLedgerID,
        CRef, EditCAccount.Text + ' ' + EditDescription.Text,
        CurrencyOrNull(DrValue), CurrencyOrNull(CrValue)]);
    end;
    Inc(Item);
    // other  postings
    with TempJournal do if FindFirst then
    repeat
      if not (FieldByName('DEBIT').IsNull
        and FieldByName('CREDIT').IsNull) then
      begin
        NominalCode := FieldByName('NOMINAL_CODE').asString;
        CostCentre := FieldByName('COST_CENTRE').asString;
        NominalDescription := FieldByName('DESCRIPTION').asString;
        DrValue := getFloat(FieldByName('DEBIT'));
        CrValue := getFloat(FieldByName('CREDIT'));
        // journal detail
        FMyJournal.TblDetail.AppendRecord([JnlNo, Item, NominalCode, CostCentre, '', '',
          NominalDescription, CurrencyOrNull(DrValue),
            CurrencyOrNull(CrValue)]);
        Inc(Item);
      end;
    until not FindNext;
    FMyJournal.ApplyUpdates(0);
    FMyJournal.Daybook.ProcessBatch(Self);
  except
    FMyJournal.RecordDelete;
    raise;
  end;
  ModalResult := mrOK;
end;

procedure TDlgNomControlLedgerAcToNom.TempJournalAfterInsert(
  DataSet: TDataSet);
begin
  Dataset.FieldByName('DESCRIPTION').asString := FJnlDescription;
end;

procedure TDlgNomControlLedgerAcToNom.btnNomSelectClick(Sender: TObject);
begin
  FMyJournal.NominalLedger.NominalCodes.View(Null, True, [mbOK]);
  with TempJournal do
    if RecordCount = 0 then
      Insert
    else
      if (State = dsBrowse) then
        Edit;
  with DBNLPostingsGrid do
  begin
    Columns[0].Field.AsString :=
      FMyJournal.NominalLedger.NominalCodes.TblMaster.FieldByName(
        'NOMINAL_CODE').AsString;
    SetFocus;
  end;
end;

procedure TDlgNomControlLedgerAcToNom.btnCostSelectClick(Sender: TObject);
begin
  FMyJournal.NominalLedger.CostLedger.CostCentres.View(null, true, [mbOK]);
  with TempJournal do
    if RecordCount = 0 then
      Insert
    else
      if (State = dsBrowse) then
        Edit;
  with DBNLPostingsGrid do
  begin
    Columns[1].Field.AsString :=
      FMyJournal.NominalLedger.CostLedger.CostCentres.TblMaster.FieldByName(
        'COST_CENTRE').AsString;
    SetFocus;
  end;
end;

end.

