{*****************************************************************************}
{                                                                             }
{   This file is part of the LKAccounts components Library                    }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for display of a TLKLedgerSales record                            }
{                                                                             }
{*****************************************************************************}

unit LKLedgerSalesRecordDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, LKLedgerContactRecordDlg, Menus, ImgList, ActnList, DB, StdActns,
  DBCtrls, Buttons, ExtCtrls, ToolWin, ComCtrls, StdCtrls, Grids, DBGrids,
  JvExDBGrids, JvDBGrid, JvDBUltimGrid, LKUtils, LKDefine, LKRecordsComponents,
  LKdlgs, LKAccountsComponents, LKDBControls, LKUtilComponents;

type
  TFormLKLedgerSalesRecord = class(TFormLKLedgerContactRecord)
    ActionAccountManagerExit: TAction;
    procedure FormShow(Sender: TObject);
    procedure ActionAccountManagerExitExecute(Sender: TObject);
  private
    { Private declarations }
    StaticTextAccountManager: TStaticText;
    dbeAccountManager: TDBEdit;
    FMyAccountsComponent: TLKLedgerSales;
  protected
    procedure SetMasterEditsActions; override;
  public
    { Public declarations }
    procedure Construct(Sender: TLKLedgerSales);
  end;

var
  FormLKLedgerSalesRecord: TFormLKLedgerSalesRecord;

implementation

{$R *.dfm}

{ TFormLKLedgerSalesRecord }

procedure TFormLKLedgerSalesRecord.Construct(Sender: TLKLedgerSales);
begin
  FMyAccountsComponent := Sender;
  inherited Construct(Sender);
end;

procedure TFormLKLedgerSalesRecord.SetMasterEditsActions;
var
  I: integer;
begin
  inherited;
  if Assigned(FMyAccountsComponent.SalesManagers) then
    with MasterEditList do
      for I := 0 to Count - 1 do
        with Items[I] do
        begin
          if (FieldName = 'ACCOUNT_MANAGER') then
          begin
            dbeAccountManager := TDBEdit(EditControl);
            dbeAccountManager.OnExit := ActionAccountManagerExit.OnExecute;
            StaticTextAccountManager := Items[I].CaptionLabel;
          end;
        end;
end;

procedure TFormLKLedgerSalesRecord.FormShow(Sender: TObject);
begin
  inherited;
  if Assigned(StaticTextAccountManager) then
    StaticTextAccountManager.Caption := 'Account Manager';
end;

procedure TFormLKLedgerSalesRecord.ActionAccountManagerExitExecute(
  Sender: TObject);
var
  code: string;
begin
  if not (FMyAccountsComponent.TblMaster.State in [dsInsert, dsEdit]) then
    exit;
  code := dbeAccountManager.field.asString;
  if (code = '') then
    exit;
  with FMyAccountsComponent.SalesManagers do
  begin
    if not Locate('MANAGER_REFERENCE', code, []) then
    begin
      View(Code, true, [mbOK]);
      dbeAccountManager.Field.AsString :=
        TblMaster.FieldByname('MANAGER_REFERENCE').AsString;
    end;
    StaticTextAccountManager.Caption := TblMaster.FieldByname('MANAGER_NAME').AsString;
  end;
end;

end.

