{*****************************************************************************}
{                                                                             }
{   This file is part of the LKAccounts components Library                    }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form to select a valid nominal code from the NominalLedger             }
{                                                                             }
{*****************************************************************************}

unit LKLedgerNominalCodeSelectDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, Dialogs, ExtCtrls, Variants, LKAccountsComponents;

type
  TDlgLedgerNominalCodeSelect = class(TForm)
    OKBtn: TButton;
    CancelBtn: TButton;
    Panel1: TPanel;
    lNomCode: TLabel;
    eNomCode: TEdit;
    btnBrowseNCode: TButton;
    btnHelp: TButton;
    PanelCostCentre: TPanel;
    lCostCentre: TLabel;
    eCostCentre: TEdit;
    btnBrowseCostCentre: TButton;
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure eNomCodeExit(Sender: TObject);
    procedure btnBrowseNCodeClick(Sender: TObject);
    procedure btnBrowseCostCentreClick(Sender: TObject);
    procedure eCostCentreExit(Sender: TObject);
    procedure btnHelpClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    NominalLedger: TLKLedgerNominal;
    HelpText: string;
    procedure CheckForFinish;
  end;

var
  DlgLedgerNominalCodeSelect: TDlgLedgerNominalCodeSelect;

implementation

uses LKutils, LKDefine;

{$R *.DFM}

procedure TDlgLedgerNominalCodeSelect.FormKeyPress(Sender: TObject;
  var Key: Char);
begin
  MoveMyFocus(self, Key);
end;

procedure TDlgLedgerNominalCodeSelect.CheckForFinish;
begin
  OKBtn.Enabled := NominalLedger.AccountIsValid(
    varArrayof([eNomCode.Text, eCostCentre.Text]), false);
end;

procedure TDlgLedgerNominalCodeSelect.eNomCodeExit(Sender: TObject);
begin
  CheckForFinish;
  with NominalLedger.NominalCodes do
    if Locate('NOMINAL_CODE', eNomCode.Text, []) then
      lNomCode.Caption := TblMaster.FieldByName('DESCRIPTION').AsString
    else
      lNomCode.Caption := 'Nominal Code';
end;

procedure TDlgLedgerNominalCodeSelect.eCostCentreExit(Sender: TObject);
begin
  CheckForFinish;
  with NominalLedger.CostLedger.CostCentres do
    if Locate('COST_CENTRE', eCostCentre.Text, []) then
      lCostCentre.Caption := TblMaster.FieldByName('DESCRIPTION').AsString
    else
      lCostCentre.Caption := 'Cost Centre';
end;

procedure TDlgLedgerNominalCodeSelect.btnBrowseNCodeClick(Sender: TObject);
begin
  with NominalLedger.NominalCodes do
  begin
    View(eNomCode.Text, true, [mbOK]);
    try
      eNomCode.Text := TblMaster.FieldByName('NOMINAL_CODE').AsString;
    except;
    end;
  end;
  eNomCodeExit(Sender);
end;

procedure TDlgLedgerNominalCodeSelect.btnBrowseCostCentreClick(
  Sender: TObject);
begin
  with NominalLedger.CostLedger.CostCentres do
  begin
    View(eCostCentre.Text, true, [mbOK]);
    try
      eCostCentre.Text := TblMaster.FieldByName('COST_CENTRE').AsString;
    except;
    end;
  end;
  eCostCentreExit(Sender);
end;

procedure TDlgLedgerNominalCodeSelect.btnHelpClick(Sender: TObject);
begin
  Showmessage(HelpText);
end;

end.

