{ ***************************************************************************** }
{ }
{ This file is part of the LKAccounts components Library }
{ }
{ See the file COPYING.LGPL.txt included in this distribution, }
{ for details about the Licence and Copyright. }
{ }
{ This code is distributed in the hope that it will be useful, }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. }
{ }
{ Author: Leslie John Kaye :    www.leskaye.com }
{ }
{ Abstract: }
{ Form for printing a TLKLedgerBank Reconciliation to Bank Statements }
{ }
{ ***************************************************************************** }

unit prLKBankAgreement;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls, DB, DBTables, Dialogs, sysUtils, DBCtrls,
  LKRecordsComponents, LKAccountsComponents, frxClass, frxDBSet;

type
  TDlgPrintBankAgreement = class(TForm)
    frxReport1: TfrxReport;
    frxDBDataset1: TfrxDBDataset;
    frxDBDataset2: TfrxDBDataset;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure frxReport1AfterPrint(Sender: TfrxReportComponent);
    procedure frxDBDataset1First(Sender: TObject);
    procedure frxDBDataset2First(Sender: TObject);
  private
    { Private declarations }
    FPrBankLedger: TLKLedgerBank;
    procedure SetPrBankLedger(const Value: TLKLedgerBank);
    procedure SetFilter(const Debit:Boolean);
  public
    { Public declarations }
    property PrBankLedger: TLKLedgerBank read FPrBankLedger write SetPrBankLedger;
  end;

var
  DlgPrintBankAgreement: TDlgPrintBankAgreement;

implementation

uses LKUtils, LKUtilsDB, LKLedgerBankUpOsDlg;

{$R *.DFM}

procedure TDlgPrintBankAgreement.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TDlgPrintBankAgreement.SetPrBankLedger(const Value: TLKLedgerBank);
var
  Balance, N: Currency;
begin
  FPrBankLedger := Value;
  if not Assigned(Value) then
    Exit;
  Show;
  frxDBDataset1.DataSet := PrBankLedger.TblDetail;
  frxDBDataset2.DataSet := PrBankLedger.TblDetail;
  TfrxMemoView(frxReport1.FindObject('MemoEnterprise')).Text :=
    PrBankLedger.AccountsHub.EnterpriseName;
  TfrxMemoView(frxReport1.FindObject('MemoTitle')).Text :=
    'Bank Agreement as at ' +
      DateToStr(DlgBankUnpresentedOutstanding.AgreementDate);
  Balance := DlgBankUnpresentedOutstanding.StatementBalance;
  TfrxMemoView(frxReport1.FindObject('MemoBankBalance')).Text :=
    FloatToStrF(Balance, ffCurrency, 15, 2);
  with PrBankLedger.TblDetail do
  begin
    Filtered := False;
    Filter := '([Cleared] = null)';
    Filtered := True;
    if FindFirst then
    repeat
      Balance := Balance + GetCurrency(FieldByName('Debit')) -
        GetCurrency(FieldByName('Credit'));
    until not FindNext;
  end;
  TfrxMemoView(frxReport1.FindObject('MemoAdjustedBalance')).Text :=
    FloatToStrF(Balance, ffCurrency, 15, 2);
  DlgBankUnpresentedOutstanding.AdjustedBalance := Balance;
  N := DlgBankUnpresentedOutstanding.BankLedgerBalance;
  TfrxMemoView(frxReport1.FindObject('MemoCashbook')).Text :=
    FloatToStrF(N, ffCurrency, 15, 2);
  N := N - Balance;
  TfrxMemoView(frxReport1.FindObject('MemoPlsLocate')).Visible :=
    CurrencyIsZero(N);
  with TfrxMemoView(frxReport1.FindObject('MemoPlsLocateAmount')) do
  begin
    Visible := CurrencyIsZero(N);
    Text := FloatToStrF(N, ffCurrency, 15, 2);
  end;
end;

procedure TDlgPrintBankAgreement.SetFilter(const Debit: Boolean);
var AFilter: string;
begin
  if Debit then
    AFilter := '([Cleared] = null) and ([Debit] <> null)'
  else
    AFilter := '([Cleared] = null) and ([Debit] = null)';
  if (PrBankLedger.TblDetail.Filter <> AFilter) then
  begin
    PrBankLedger.TblDetail.Filtered := False;
    PrBankLedger.TblDetail.Filter := AFilter;
    PrBankLedger.TblDetail.Filtered := True;
    PrBankLedger.TblDetail.First;
  end;
end;

procedure TDlgPrintBankAgreement.frxDBDataset1First(Sender: TObject);
begin
  SetFilter(False);
end;

procedure TDlgPrintBankAgreement.frxDBDataset2First(Sender: TObject);
begin
  SetFilter(True);
end;

procedure TDlgPrintBankAgreement.frxReport1AfterPrint(
  Sender: TfrxReportComponent);
begin
  PrBankLedger.TblDetail.Filtered := False;
  PrBankLedger.TblDetail.Filter := '([Cleared] = null)';
  PrBankLedger.TblDetail.Filtered := True;
  Close;
end;

end.
