unit prLKLedgerAnalysislManagementReportBase;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Contnrs, ShellAPI,
  System.Variants, System.Classes,Vcl.Graphics, Vcl.Controls, Vcl.Forms, Math,
  Vcl.Dialogs, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, Generics.Collections,
  prLKPrintBaseDlg, prLKSystemComponentDlg, LKDefine, LKRecordsComponents,
  LKAccountsComponents, LKUtilsDB,
  frxClass, frxExportHTML, frxExportPDF, frxDBSet;

const
  BodyFontSize = 9;
  TotalArrayCount = 15;

type
  TLKManagementBaseReport = class;

  TLKManagementBaseComponent = class(TComponent)
  private
    FDescription: string;
    FNormallyADebit: Boolean;
    FReport: TLKManagementBaseReport;
  protected
    procedure GetManagementData; virtual;
    procedure SetNormallyADebit(Value: Boolean); virtual;
    property Report: TLKManagementBaseReport read FReport;
  public
    constructor Create(AOwner: TComponent); override;
    property Description: string read FDescription write FDescription;
    property NormallyADebit: Boolean read FNormallyADebit write
        SetNormallyADebit;
  end;

  TLKManagementLineBase = class abstract(TLKManagementBaseComponent)
  private
    FIsSubTotal: Boolean;
    FMemos: TObjectList<TfrxMemoView>;
    FNominalCode: string;
    FValuesTotal: Currency;
    FVariableCostPercent: Double;
  protected
    procedure GetData; virtual; abstract;
    procedure GetManagementData; override;
  public
    Values: array[0..TotalArrayCount] of Currency;
    constructor Create(AOwner: TComponent); override;
    procedure FillDataMemos; virtual;
    function HasValues: Boolean; virtual;
    procedure Spreadsheet(CSVList: TStringList; const NormallyADebit: Boolean);
        virtual; abstract;
    procedure ValuesAdd(Source: TLKManagementLineBase); virtual;
    procedure ValuesZero; virtual;
    property IsSubTotal: Boolean read FIsSubTotal write FIsSubTotal;
    property Memos: TObjectList<TfrxMemoView> read FMemos write FMemos;
    property NominalCode: string read FNominalCode write FNominalCode;
    property ValuesTotal: Currency read FValuesTotal write FValuesTotal;
    property VariableCostPercent: Double read FVariableCostPercent write
        FVariableCostPercent;
  end;

  TLKManagementBaseSection = class abstract(TLKManagementBaseComponent)
  private
    FSectionTotal: TLKManagementLineBase;
  protected
    function CreateReportLine: TLKManagementLineBase; virtual; abstract;
    procedure GetManagementData; override;
    procedure SetNormallyADebit(Value: Boolean); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Spreadsheet(CSVList: TStringList; const ReportDetail: Boolean);
        virtual; abstract;
    property SectionTotal: TLKManagementLineBase read FSectionTotal write
        FSectionTotal;
  end;

  TLKManagementHeadBase = class abstract(TLKManagementBaseSection)
  private
    FCurrentLine: Integer;
    FLines: TObjectList<TLKManagementLineBase>;
    FSubTotalNormallyADebit: Boolean;
    FSubTotalTitle: string;
    FZeroSubtotals: Boolean;
  protected
    procedure GetManagementData; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure FillDataMemos;
    procedure Spreadsheet(CSVList: TStringList; const ReportDetail: Boolean);
        override;
    property CurrentLine: Integer read FCurrentLine write FCurrentLine;
    property Lines: TObjectList<TLKManagementLineBase> read FLines write FLines;
    property SubTotalNormallyADebit: Boolean read FSubTotalNormallyADebit write
        FSubTotalNormallyADebit;
    property SubTotalTitle: string read FSubTotalTitle write FSubTotalTitle;
    property ZeroSubtotals: Boolean read FZeroSubtotals write FZeroSubtotals;
  end;

  TLKManagementBaseReport = class abstract(TLKManagementBaseSection)
  private
    FCurrentSection: Integer;
    FCurrLeft: Extended;
    FDataBand: TfrxDataBand;
    FDataMemos: TObjectList<TfrxMemoView>;
    FDateFrom: TDateTime;
    FDateTo: TDateTime;
    FHeader: TfrxHeader;
    FIsProfitAndLoss: Boolean;
    FPrintDetail: Boolean;
    FReportDataComponent: TLKLedgerAnalysisContainedDocuments;
    FReportDatesCount: Integer;
    FSections: TObjectList<TLKManagementHeadBase>;
    FSubtotalMemos: TObjectList<TfrxMemoView>;
    FSubtotals: TfrxFooter;
    FTotalMemos: TObjectList<TfrxMemoView>;
    FTotals: TfrxFooter;
    function GetReport: TLKManagementBaseReport;
  protected
    function CreateReportHead: TLKManagementHeadBase; virtual; abstract;
    procedure GetManagementData; override;
    function NewMemo(const ATitle: string; MWidth: Extended; AFontStyle:
        TFontStyles = []; ABorders: TfrxFrameTypes = []): TfrxMemoView;
    property CurrLeft: Extended read FCurrLeft write FCurrLeft;
    property Report: TLKManagementBaseReport read GetReport;
  public
    ReportDates: array[0..TotalArrayCount] of TDateTime;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function ConstructReport(AReport: TfrxReport): string; virtual;
    procedure DoAfterPrintReport; virtual;
    procedure DoBeforePrint; virtual;
    procedure DoHeadFirst;
    procedure DoHeadNext;
    function DoLinesFirst: Integer;
    procedure DoLinesNext;
    procedure Prepare(ADateFrom, ADateTo: TDateTime);
    procedure Spreadsheet(CSVList: TStringList; const ReportDetail: Boolean);
        override;
    property DataBand: TfrxDataBand read FDataBand;
    property DataMemos: TObjectList<TfrxMemoView> read FDataMemos;
    property DateFrom: TDateTime read FDateFrom write FDateFrom;
    property DateTo: TDateTime read FDateTo write FDateTo;
    property Header: TfrxHeader read FHeader;
    property IsProfitAndLoss: Boolean read FIsProfitAndLoss write
        FIsProfitAndLoss;
    property PrintDetail: Boolean read FPrintDetail write FPrintDetail default
        True;
    property ReportDataComponent: TLKLedgerAnalysisContainedDocuments read
        FReportDataComponent write FReportDataComponent;
    property ReportDatesCount: Integer read FReportDatesCount;
    property Sections: TObjectList<TLKManagementHeadBase> read FSections write
        FSections;
    property SubtotalMemos: TObjectList<TfrxMemoView> read FSubtotalMemos;
    property Subtotals: TfrxFooter read FSubtotals;
    property TotalMemos: TObjectList<TfrxMemoView> read FTotalMemos;
    property Totals: TfrxFooter read FTotals;
  end;

  TDlgLKLedgerAnalysisManagementReportBase = class(TDlgLKPrintSystemComponent)
    cbxDateFrom: TComboBox;
    cbxDateTo: TComboBox;
    frxDBDatasetLayoutHead: TfrxDBDataset;
    frxDBDatasetLayoutReport: TfrxDBDataset;
    frxUserDataSetLines: TfrxUserDataSet;
    InfoLabel: TStaticText;
    Label5: TStaticText;
    procedure cbxDateFromChange(Sender: TObject);
    procedure frxDBDatasetLayoutHeadFirst(Sender: TObject);
    procedure frxDBDatasetLayoutHeadNext(Sender: TObject);
    procedure frxUserDataSetLinesFirst(Sender: TObject);
    procedure frxUserDataSetLinesNext(Sender: TObject);
  private
    FMyPrintingComponent: TLKLedgerAnalysisContainedDocuments;
    FReport: TLKManagementBaseReport;
  protected
    function ConstructReport: string; override;
    procedure DoAfterPrintReport; override;
    procedure DoBeforePrint; override;
    procedure ExecuteBtnSpreadsheet; override;
  public
    procedure Construct(APrintingComponent: TLKDataComponent; const aTitle:
        string); virtual;
    property Report: TLKManagementBaseReport read FReport write FReport;
  end;

var
  DlgLKLedgerAnalysisManagementReportBase: TDlgLKLedgerAnalysisManagementReportBase;

implementation

{$R *.dfm}

{
************************** TLKManagementBaseComponent **************************
}
constructor TLKManagementBaseComponent.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if (AOwner is TLKManagementBaseReport) then
    FReport := TLKManagementBaseReport(AOwner)
  else if (Self is TLKManagementBaseReport) then
    FReport := TLKManagementBaseReport(Self);
end;

procedure TLKManagementBaseComponent.GetManagementData;
begin
  Application.ProcessMessages;
end;

procedure TLKManagementBaseComponent.SetNormallyADebit(Value: Boolean);
begin
  FNormallyADebit := Value;
end;

{
******************* TDlgLKLedgerAnalysisManagementReportBase *******************
}
procedure TDlgLKLedgerAnalysisManagementReportBase.cbxDateFromChange(Sender:
    TObject);
begin
  inherited;
  cbxDateTo.ItemIndex :=
    Max(cbxDateFrom.ItemIndex + 12, cbxDateTo.Items.Count -1);
  ReportNeedsConstruction := True;
end;

procedure TDlgLKLedgerAnalysisManagementReportBase.Construct(
    APrintingComponent: TLKDataComponent; const aTitle: string);
begin
  FMyPrintingComponent := TLKLedgerAnalysisContainedDocuments(APrintingComponent);
  inherited Construct(APrintingComponent, aTitle);
  with FReport do
  begin
    ReportDataComponent := FMyPrintingComponent;
  end;
  with FMyPrintingComponent.ContainerComponent do
  begin
    with AccountsPeriods.TblMaster do
      if FindFirst then
      begin
        with cbxDateFrom do
        begin
          Items.Clear;
          repeat Items.Append(FieldByName('PERIOD_END').AsString);
          until not FindNext;
          if (Items.Count > 12) then
            Text := Items[Items.Count - 12]
          else
            Text := Items[0];
        end;
      end;
    with cbxDateTo do
    begin
      Items.Clear;
      Items.Assign(cbxDateFrom.Items);
      ItemIndex := Items.IndexOf(DateToStr(LedgerDate));
      if (ItemIndex < 0) then
        ItemIndex := Items.Count-1;
    end;
    cbxDateFrom.OnChange(nil);
    frxDBDatasetLayoutReport.DataSet := ReportLayouts.TblMaster;
    frxDBDatasetLayoutHead.DataSet := ReportLayouts.TblDetail;
  end;
end;

function TDlgLKLedgerAnalysisManagementReportBase.ConstructReport: string;
var
  DateFrom, DateTo: TDateTime;
begin
  Result := inherited ConstructReport;
  DateFrom := StrToDate(cbxDateFrom.Text);
  DateTo := StrToDate(cbxDateTo.Text);
  if (Result = 'success') then
  begin
    Report.DateFrom := DateFrom;
    Report.DateTo := DateTo;
    Result := Report.ConstructReport(frxReport1);
  end;
  if (Result = 'success') then
  begin
    FReport.Prepare(DateFrom, DateTo);
    ReportNeedsConstruction := False;
  end;
end;

procedure TDlgLKLedgerAnalysisManagementReportBase.DoAfterPrintReport;
begin
  inherited DoAfterPrintReport;
  FReport.DoAfterPrintReport;
end;

procedure TDlgLKLedgerAnalysisManagementReportBase.DoBeforePrint;
begin
  inherited DoBeforePrint;
  FReport.DoBeforePrint;
end;

procedure TDlgLKLedgerAnalysisManagementReportBase.ExecuteBtnSpreadsheet;
var
  CSVFileName: string;
  CSVList: TStringList;
begin
  FReport.Prepare(StrToDate(cbxDateFrom.Text), StrToDate(cbxDateTo.Text));
  CSVFileName := PrintFileName + '.csv';
  Screen.Cursor := crHourglass;
  CSVList := TStringList.Create;
  try
    FReport.Spreadsheet(CSVList, FReport.PrintDetail);
    CSVList.SaveToFile(CSVFileName);
  finally
    CSVList.Free;
    Screen.Cursor := crDefault;
  end;
  ShellExecute(Application.Handle, 'open', PChar(CSVFileName), nil, nil, 1);
end;

procedure TDlgLKLedgerAnalysisManagementReportBase.frxDBDatasetLayoutHeadFirst(
    Sender: TObject);
begin
  inherited;
  FReport.DoHeadFirst;
end;

procedure TDlgLKLedgerAnalysisManagementReportBase.frxDBDatasetLayoutHeadNext(
    Sender: TObject);
begin
  inherited;
  FReport.DoHeadNext;
end;

procedure TDlgLKLedgerAnalysisManagementReportBase.frxUserDataSetLinesFirst(
    Sender: TObject);
begin
  inherited;
  frxUserDataSetLines.RangeEndCount := Report.DoLinesFirst;
end;

procedure TDlgLKLedgerAnalysisManagementReportBase.frxUserDataSetLinesNext(
    Sender: TObject);
begin
  Report.DoLinesNext;
end;



{
*************************** TLKManagementBaseSection ***************************
}
constructor TLKManagementBaseSection.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Description := '';
  FSectionTotal := CreateReportLine;
  FSectionTotal.IsSubTotal := True;
end;

destructor TLKManagementBaseSection.Destroy;
begin
  SectionTotal.Free;
  inherited;
end;

procedure TLKManagementBaseSection.GetManagementData;
begin
  inherited GetManagementData;
  SectionTotal.ValuesZero;
end;

procedure TLKManagementBaseSection.SetNormallyADebit(Value: Boolean);
begin
  inherited SetNormallyADebit(Value);
  FNormallyADebit := Value;
  FSectionTotal.NormallyADebit := Value;
end;


{
**************************** TLKManagementHeadBase *****************************
}
constructor TLKManagementHeadBase.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FLines := TObjectList<TLKManagementLineBase>.Create(True);
  SectionTotal.Memos := Report.SubtotalMemos;
  FCurrentLine := 0;
end;

destructor TLKManagementHeadBase.Destroy;
begin
  FLines.Free;
  inherited;
end;

procedure TLKManagementHeadBase.FillDataMemos;
begin
  if (FCurrentLine < Lines.Count) then
    TLKManagementLineBase(Lines[FCurrentLine]).FillDataMemos;
  Inc(FCurrentLine);
end;

procedure TLKManagementHeadBase.GetManagementData;
var
  ReportLine: TLKManagementLineBase;

  procedure NewReportLine(const Description: string);
  begin
    ReportLine := CreateReportLine;
    ReportLine.Description := Description;
    ReportLine.NormallyADebit := NormallyADebit;
    ReportLine.Memos := Report.DataMemos;
  end;

begin
  inherited GetManagementData;
  Lines.Clear;
  with Report.ReportDataComponent.ContainerComponent.ReportLines do
    if TblMaster.FindFirst then
    repeat
      if TblDetail.FindFirst then
      begin
        if Report.PrintDetail then
        repeat
          NewReportLine(TblDetail.FieldByName('DESCRIPTION').asString);
          ReportLine.GetManagementData;
          SectionTotal.ValuesAdd(ReportLine);
          if ReportLine.HasValues then
             Lines.Add(ReportLine)
          else ReportLine.Free;
        until not TblDetail.FindNext
        else
        begin
          NewReportLine(FieldByName('DESCRIPTION').asString);
          repeat
            ReportLine.GetManagementData;
          until not TblDetail.FindNext;
          SectionTotal.ValuesAdd(ReportLine);
          if ReportLine.HasValues then
             Lines.Add(ReportLine)
          else ReportLine.Free;
        end;
      end;
    until not TblMaster.FindNext;
end;

procedure TLKManagementHeadBase.Spreadsheet(CSVList: TStringList; const
    ReportDetail: Boolean);
var
  S: string;
  I: Integer;
begin
  CSVList.Add('');
  S := '"'+ Description;
  if NormallyADebit then
    S := S + '","(Normally a Debit)"'
  else
    S := S + '","(Normally a Credit)"';
  CSVList.Add(S);
  for I := 0 to (Lines.Count - 1) do
    TLKManagementLineBase(Lines.Items[I]).Spreadsheet(CSVList, NormallyADebit);
end;

{
**************************** TLKManagementLineBase *****************************
}
constructor TLKManagementLineBase.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  IsSubTotal := False;
  ValuesZero;
end;

procedure TLKManagementLineBase.FillDataMemos;
begin
  Memos[0].Text := Description;
end;

procedure TLKManagementLineBase.GetManagementData;
begin
  inherited GetManagementData;
  with Report.ReportDataComponent.ContainerComponent.ReportLines do
  begin
    if (NominalCode = '') then
    begin
      NominalCode := TblDetail.FieldByName('NOMINAL_CODE').AsString;
      VariableCostPercent :=
        TblDetail.FieldByName('VARIABLE_COST_PERCENT').AsFloat;
    end;
    GetData;
  end;
end;

function TLKManagementLineBase.HasValues: Boolean;
var
  Total: Currency;
  I: Integer;

  function Positive(const Amount: Currency): Currency;
  begin
    if (Amount < 0) then
      Result := -Amount
    else
      Result := Amount;
  end;

begin
  Result := (ValuesTotal <> 0);
  if not Result then
  begin
    Total := 0;
    for I := 0 to TotalArrayCount do
      Total := Total + Positive(Values[I]);
    Result := (Total > 0);
  end;
end;

procedure TLKManagementLineBase.ValuesAdd(Source: TLKManagementLineBase);
var
  I: Integer;
begin
  for I := 0 to Report.ReportDatesCount - 1 do
    Values[I] := Values[I] + Source.Values[I];
  ValuesTotal := ValuesTotal + Source.ValuesTotal;
end;

procedure TLKManagementLineBase.ValuesZero;
var
  I: Integer;
begin
  for I := 0 to TotalArrayCount do
    Values[I] := 0;
  ValuesTotal := 0;
end;

{
*************************** TLKManagementBaseReport ****************************
}
constructor TLKManagementBaseReport.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDateFrom := 0;
  FDateTo := 0;
  FSections := TObjectList<TLKManagementHeadBase>.Create(True);
  FDataMemos := TObjectList<TfrxMemoView>.Create(False);
  FTotalMemos := TObjectList<TfrxMemoView>.Create(False);
  FSubtotalMemos := TObjectList<TfrxMemoView>.Create(False);
  SectionTotal.Memos := FTotalMemos;
end;

destructor TLKManagementBaseReport.Destroy;
begin
  FSections.Free;
  FreeAndNil(FDataMemos);
  FreeAndNil(FTotalMemos);
  FreeAndNil(FSubtotalMemos);
  inherited Destroy;
end;

function TLKManagementBaseReport.ConstructReport(AReport: TfrxReport): string;
function NewDescriptionMemo(AOwner: TfrxBand): TfrxMemoView;
  begin
    Result := TfrxMemoView.Create(AOwner);
    Result.CreateUniqueName;
    Result.SetBounds(0, 0, fr01cm * 40, fr01cm * 4.5);
    Result.HAlign := haLeft;
    Result.LineSpacing := 1;
    Result.Font.Size := 9;
  end;

begin
//  Result := inherited ConstructReport(AReport);
  TfrxReportPage(AReport.FindObject('Page1')).Orientation :=
    TPrinterOrientation.poLandscape;
  TfrxMemoView(AReport.FindObject('MemoDateTo')).Text := DateToStr(DateTo);
  FHeader := TfrxHeader(AReport.FindObject('PageHeader1'));
  if not Assigned(FHeader) then
    raise LKError.Create('PageHeader1 not found');
  FHeader.Clear;
  FDataMemos.Clear;
  FDataBand := TfrxDataBand(AReport.FindObject('DetailData1'));
  if not Assigned(FDataBand) then
    raise LKError.Create('DetailData1 not found');
  FDataBand.Clear;
  FDataMemos.Add(NewDescriptionMemo(FDataBand));
  FSubtotalMemos.Clear;
  FSubtotals := TfrxFooter(AReport.FindObject('FooterSectionTotals'));
  if not Assigned(FSubtotals) then
    raise LKError.Create('FooterSectionTotals not found');
  FSubtotals.Clear;
  FSubtotalMemos.Add(NewDescriptionMemo(FSubtotals));
  FTotalMemos.Clear;
  FTotals := TfrxFooter(AReport.FindObject('FooterReportTotals'));
  if not Assigned(FTotals) then
    raise LKError.Create('FooterReportTotals not found');
  FTotals.Clear;
  FTotalMemos.Add(NewDescriptionMemo(FTotals));
  FReportDatesCount := 0;
  with ReportDataComponent.ContainerComponent.AccountsPeriods.TblMaster do
  begin
    if Locate('PERIOD_END', DateFrom, []) then
    begin
      while ((DateFrom > GetDate(FieldByName('PERIOD_END'))) and
        (FReportDatesCount < TotalArrayCount)) do
        if not FindNext then
          raise
            LKError.Create('Accounts period end not found in Accounts Periods table');
      repeat
        ReportDates[FReportDatesCount] := GetDate(FieldByName('PERIOD_END'));
        Inc(FReportDatesCount);
      until not FindNext or (FieldByName('PERIOD_END').asDateTime > DateTo);
    end
    else
      raise
        LKError.Create('Accounts period start not found in Accounts Periods table');
  end;
end;

procedure TLKManagementBaseReport.DoAfterPrintReport;
begin
  //
end;

procedure TLKManagementBaseReport.DoBeforePrint;
begin
  //
end;

procedure TLKManagementBaseReport.DoHeadFirst;
begin
  FCurrentSection := 0;
  SectionTotal.FillDataMemos;
end;

procedure TLKManagementBaseReport.DoHeadNext;
begin
  Inc(FCurrentSection);
end;

function TLKManagementBaseReport.DoLinesFirst: Integer;
begin
  Result := 0;
  if (FCurrentSection < Sections.Count) then
  begin
    Result := Sections[FCurrentSection].Lines.Count;
    if Result > 0 then
    begin
      Sections[FCurrentSection].CurrentLine := 0;
      Sections[FCurrentSection].FillDataMemos;
      Sections[FCurrentSection].SectionTotal.FillDataMemos;
    end;
  end;
end;

procedure TLKManagementBaseReport.DoLinesNext;
begin
  Sections[FCurrentSection].FillDataMemos;
end;

procedure TLKManagementBaseReport.GetManagementData;
var
  ReportHead: TLKManagementHeadBase;
begin
  inherited GetManagementData;
  FSections.Clear;
  SectionTotal.ValuesZero;
  with ReportDataComponent.ContainerComponent.ReportLayouts do
  begin
    IsProfitAndLoss := (GetInteger(FieldByName('IS_PROFIT_AND_LOSS')) > 0);
    with TblDetail do
      if FindFirst then
      repeat
        ReportHead := CreateReportHead;
        Sections.Add(ReportHead);
        ReportHead.Description := FieldByName('DESCRIPTION').asString;
        ReportHead.NormallyADebit :=
          (GetInteger(FieldByName('NORMALLY_A_DEBIT')) > 0);
        ReportHead.ZeroSubtotals :=
          (GetInteger(FieldByName('SUBTOTAL_PRE_ZERO')) > 0);
        ReportHead.SubTotalTitle := FieldByName('SUBTOTAL_TITLE').asString;
        ReportHead.SubTotalNormallyADebit :=
          (GetInteger(FieldByName('SUBTOTAL_NORMALLY_DEBIT')) > 0);
        ReportHead.GetManagementData;
        if ReportHead.ZeroSubtotals then
          SectionTotal.ValuesZero;
        SectionTotal.ValuesAdd(ReportHead.SectionTotal);
      until not FindNext;
  end;
end;

function TLKManagementBaseReport.GetReport: TLKManagementBaseReport;
begin
  Result := TLKManagementBaseReport(inherited Report);
end;

function TLKManagementBaseReport.NewMemo(const ATitle: string; MWidth: Extended;
    AFontStyle: TFontStyles = []; ABorders: TfrxFrameTypes = []): TfrxMemoView;
  function NewDataMemo(AOwner: TfrxBand): TfrxMemoView;
  begin
    Result := TfrxMemoView.Create(AOwner);
    Result.CreateUniqueName;
    Result.SetBounds(fr01cm * CurrLeft, 0, fr01cm * MWidth, fr01cm * 4.5);
    Result.HAlign := haRight;
    Result.LineSpacing := 1;
    Result.Font.Size := 9;
  end;

begin
    with NewDataMemo(Header) do
    begin
      Text := ATitle;
      Frame.Typ := ABorders + [ftBottom];
    end;
    Result := NewDataMemo(DataBand);
    Result.DisplayFormat.Kind := TfrxFormatKind.fkNumeric;
    Result.Font.Style := AFontStyle;
    Result.Frame.Typ := ABorders;
    DataMemos.Add(Result);
    SubtotalMemos.Add(NewDataMemo(Subtotals));
    with SubtotalMemos[SubtotalMemos.Count - 1] do
    begin
      LineSpacing := 1.5;
      Font.Style := [fsBold];
      Height := fr01cm * 6;
      Frame.Typ := ABorders + [ftTop, ftBottom];
    end;
    TotalMemos.Add(NewDataMemo(Totals));
    with TotalMemos[TotalMemos.Count - 1] do
    begin
      LineSpacing := 1.5;
      Font.Style := [fsBold];
      Height := fr01cm * 6;
      Frame.Typ := ABorders + [ftTop, ftBottom];
    end;
    CurrLeft := CurrLeft + MWidth;
end;

procedure TLKManagementBaseReport.Prepare(ADateFrom, ADateTo: TDateTime);
begin
  ReportDataComponent.ExternalDisableControls;
  try
    GetManagementData;
  finally
    ReportDataComponent.ExternalEnableControls;
  end;
end;

procedure TLKManagementBaseReport.Spreadsheet(CSVList: TStringList; const
    ReportDetail: Boolean);
var
  I: Integer;
begin
  for I := 0 to (Sections.Count - 1) do
    Sections.Items[I].Spreadsheet(CSVList, ReportDetail);
end;


end.
