{*****************************************************************************}
{                                                                             }
{   This file is part of the LKAccounts components Library                    }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for printing a TLKLedgerCostManagementAccounts                    }
{                                                                             }
{*****************************************************************************}

unit prLKLedgerCostManagementAcs;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants,
  System.Classes, Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs,
  Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, Math, Data.DB, Vcl.Grids,Vcl.DBGrids,
  prLKPrintBaseDlg, prLKLedgerAnalysislManagementReportBase, LKDefine,
  LKRecordsComponents, LKAccountsComponents, frxClass, frxExportHTML,
  frxExportPDF, frxDBSet;

const
  TotalArrayCount = 15;

type
  TLKCostAccountsReport = class;
  TLKCostAccountsReportLine = class;

  TLKCostAccountsReport = class(TLKManagementBaseReport)
  private
    function GetReport: TLKCostAccountsReport;
    function GetSectionTotal: TLKCostAccountsReportLine;
    procedure SetSectionTotal(Value: TLKCostAccountsReportLine);
  protected
    function CreateReportHead: TLKManagementHeadBase; override;
    function CreateReportLine: TLKManagementLineBase; override;
    property Report: TLKCostAccountsReport read GetReport;
  public
    function ConstructReport(AReport: TfrxReport): string; override;
    procedure Spreadsheet(CSVList: TStringList; const ReportDetail: Boolean);
        override;
    property SectionTotal: TLKCostAccountsReportLine read GetSectionTotal write
        SetSectionTotal;
  end;

  TLKCostAccountsReportHead = class(TLKManagementHeadBase)
  private
    function GetReport: TLKCostAccountsReport;
    function GetSectionTotal: TLKCostAccountsReportLine;
    procedure SetSectionTotal(Value: TLKCostAccountsReportLine);
  protected
    function CreateReportLine: TLKManagementLineBase; override;
    property Report: TLKCostAccountsReport read GetReport;
  public
    property SectionTotal: TLKCostAccountsReportLine read GetSectionTotal write
        SetSectionTotal;
  end;

  TLKCostAccountsReportLine = class(TLKManagementLineBase)
  private
    FYearToDate: Currency;
    function GetReport: TLKCostAccountsReport;
  protected
    procedure GetData; override;
    property Report: TLKCostAccountsReport read GetReport;
  public
    procedure FillDataMemos; override;
    procedure Spreadsheet(CSVList: TStringList; const NormallyADebit: Boolean);
        override;
    procedure ValuesAdd(Source: TLKManagementLineBase); override;
    procedure ValuesZero; override;
    property YearToDate: Currency read FYearToDate write FYearToDate;
  end;

  TDlgLKLedgerCostManagementAcsPrint = class(
      TDlgLKLedgerAnalysisManagementReportBase)
    dbgCategories: TDBGrid;
    dbgCostCentres: TDBGrid;
    dbgTypes: TDBGrid;
    dsCategories: TDataSource;
    dsCostCentres: TDataSource;
    dsTypes: TDataSource;
    Label6: TStaticText;
    rgReportDetail: TRadioGroup;
    procedure dsTypesDataChange(Sender: TObject; Field: TField);
    procedure FormCreate(Sender: TObject);
  private
    FMyPrintingComponent: TLKLedgerCostManagementAccounts;
    function GetReport: TLKCostAccountsReport;
    procedure SetReport(Value: TLKCostAccountsReport);
  public
    procedure Construct(APrintingComponent: TLKDataComponent; const ATitle:
        string); override;
    property Report: TLKCostAccountsReport read GetReport write SetReport;
  end;

var
  DlgLKLedgerCostManagementAcsPrint: TDlgLKLedgerCostManagementAcsPrint;

implementation

uses LKUtils, LKUtilsDB;

{$R *.DFM}


{
**************************** TLKCostAccountsReport *****************************
}
function TLKCostAccountsReport.ConstructReport(AReport: TfrxReport): string;
var
  I: Integer;
  S: string;
begin
  Result := inherited ConstructReport(AReport);
  CurrLeft := 40;
  try
    if (ReportDatesCount > 7) then
      TfrxReportPage(AReport.FindObject('Page1')).Orientation :=
        TPrinterOrientation.poLandscape;
    NewMemo('Year to Date', 18, [fsBold], [ftLeft, ftRight]).Name := 'MemoYTD';
    for I := 0 to ReportDatesCount - 1 do
    begin
      S := DateToStr(ReportDates[I]);
      NewMemo(Copy(S, 4, Length(S) - 3), 16);
    end;
    NewMemo('Total ', 18, [fsBold], [ftLeft, ftRight]).Name := 'MemoTotal';
    DataMemos.Delete(0);
  except on E: Exception do
    Result := E.Message;
  end;
end;

function TLKCostAccountsReport.CreateReportHead: TLKManagementHeadBase;
begin
  Result := TLKCostAccountsReportHead.Create(Report);
end;

function TLKCostAccountsReport.CreateReportLine: TLKManagementLineBase;
begin
  Result := TLKCostAccountsReportLine.Create(Report);
end;

function TLKCostAccountsReport.GetReport: TLKCostAccountsReport;
begin
  Result := TLKCostAccountsReport(inherited Report);
end;

function TLKCostAccountsReport.GetSectionTotal: TLKCostAccountsReportLine;
begin
  Result := TLKCostAccountsReportLine(inherited SectionTotal);
end;

procedure TLKCostAccountsReport.SetSectionTotal(Value:
    TLKCostAccountsReportLine);
begin
  inherited SectionTotal := Value;
end;

procedure TLKCostAccountsReport.Spreadsheet(CSVList: TStringList; const
    ReportDetail: Boolean);
var
  Headings: string;
  I: Integer;
begin
  // write column headings
  Headings := ',"Year to Date"';
  for I := 0 to ReportDatesCount - 1 do
    Headings := Headings + ',' + DateToStr(ReportDates[I]);
  Headings := Headings + ',' + '"Total"';
  CSVList.Add(Headings);
  // write the data in inherited
  inherited Spreadsheet(CSVList, ReportDetail);
end;


{
************************** TLKCostAccountsReportHead ***************************
}
function TLKCostAccountsReportHead.CreateReportLine: TLKManagementLineBase;
begin
  Result := TLKCostAccountsReportLine.Create(Report);
end;

function TLKCostAccountsReportHead.GetReport: TLKCostAccountsReport;
begin
  Result := TLKCostAccountsReport(inherited Report);
end;

function TLKCostAccountsReportHead.GetSectionTotal: TLKCostAccountsReportLine;
begin
  Result := TLKCostAccountsReportLine(inherited SectionTotal);
end;

procedure TLKCostAccountsReportHead.SetSectionTotal(Value:
    TLKCostAccountsReportLine);
begin
  inherited SectionTotal := Value;
end;



{
************************** TLKCostAccountsReportLine ***************************
}
procedure TLKCostAccountsReportLine.FillDataMemos;
var
  PrintValues: Array[0..TotalArrayCount] of Currency;
  PrintYearToDate: Currency;
  PrintValuesTotal: Currency;
  I: Integer;
//  OKToPrint: Boolean;
begin
  inherited FillDataMemos; // Memos[0].Text = Description
  if NormallyADebit then
  begin
    PrintYearToDate := YearToDate;
    for I := 0 to Report.ReportDatesCount - 1 do
      PrintValues[I] := Values[I];
    PrintValuesTotal := ValuesTotal;
  end
  else
  begin
    PrintYearToDate := -YearToDate;
    for I := 0 to Report.ReportDatesCount - 1 do
      PrintValues[I] := -Values[I];
    PrintValuesTotal := -ValuesTotal;
  end;
  with Report do
  begin
    Memos[1].Text := FloatToStrFOrBlank(PrintYearToDate, ffNumber, 0);
    I := 1;
    while I < DataMemos.Count - 2 do
    begin
      DataMemos[I+1].Text := FloatToStrFOrBlank(PrintValues[I], ffNumber, 0);
      Inc(I);
    end;
    DataMemos[I+1].Text := FloatToStrFOrBlank(PrintValuesTotal, ffNumber, 0);
  end;
end;

procedure TLKCostAccountsReportLine.GetData;
var
  I: Integer;
  L: Currency;
begin
  with Report, ReportDataComponent do
  begin
    YearToDate := YearToDate +
      GetCurrency(TblMaster.FieldByName('PROFIT_AND_LOSS'))
      + GetCurrency(TblMaster.FieldByName('BALANCE_SHEET'));
    for I := 0 to ReportDatesCount - 1 do
    begin
      with TblDetail do
        if Locate('PERIOD_END', ReportDates[I], []) then
        begin
          L := GetCurrency(FieldByName('THIS_MONTH'));
          Values[I] := Values[I] + L;
          ValuesTotal := ValuesTotal + L;
        end;
    end;
  end;
end;

function TLKCostAccountsReportLine.GetReport: TLKCostAccountsReport;
begin
  Result := TLKCostAccountsReport(inherited Report);
end;

procedure TLKCostAccountsReportLine.Spreadsheet(CSVList: TStringList; const
    NormallyADebit: Boolean);
var
  PrintValues: array[0..totalArrayCount] of currency;
  PrintYearToDate: Currency;
  PrintValuesTotal: Currency;
  I: Integer;
  OkToPrint: Boolean;
  S: string;
begin
  OkToPrint := (YearToDate <> 0);
  for I := 0 to Report.ReportDatesCount - 1 do
    if Values[I] <> 0 then
      OkToPrint := True;
  if not OkToPrint then
    Exit;
  if NormallyADebit then
  begin
    PrintYearToDate := YearToDate;
    for I := 0 to Report.ReportDatesCount - 1 do
      PrintValues[I] := Values[I];
    PrintValuesTotal := ValuesTotal;
  end
  else
  begin
    PrintYearToDate := -YearToDate;
    for I := 0 to Report.ReportDatesCount - 1 do
      PrintValues[I] := -Values[I];
    PrintValuesTotal := -ValuesTotal;
  end;
  S := Description + ',';
  S := S + '"' + CurrencyToStrFOrBlank(PrintYearToDate, 2);
  for I := 0 to Report.ReportDatesCount - 1 do
    S := S + '","' + CurrencyToStrFOrBlank(PrintValues[I], 2);
  S := S + '","' + CurrencyToStrFOrBlank(PrintValuesTotal, 2) + '"';
  CSVList.Add(S);
end;

procedure TLKCostAccountsReportLine.ValuesAdd(Source: TLKManagementLineBase);
begin
  inherited ValuesAdd(Source);
  YearToDate := YearToDate + TLKCostAccountsReportLine(Source).YearToDate;
end;

procedure TLKCostAccountsReportLine.ValuesZero;
begin
  inherited ValuesZero;
  YearToDate := 0;
end;


{
********************** TDlgLKLedgerCostManagementAcsPrint **********************
}
procedure TDlgLKLedgerCostManagementAcsPrint.Construct(APrintingComponent:
    TLKDataComponent; const ATitle: string);
begin
  FMyPrintingComponent := TLKLedgerCostManagementAccounts(APrintingComponent);
  inherited Construct(APrintingComponent, ATitle);
  with FMyPrintingComponent do
  begin
    ContainerComponent.ReportLayouts.TblMasterDataSource := dsTypes;
    ContainerComponent.ReportLines.TblMasterDataSource := dsCategories;
    ContainerComponent.ReportLines.TblDetailDataSource := dsCostCentres;
  end;
end;

procedure TDlgLKLedgerCostManagementAcsPrint.dsTypesDataChange(Sender: TObject;
    Field: TField);
begin
  inherited;
  if Assigned(dsTypes.DataSet) then
    eReportTitle.Text := dsTypes.DataSet.FieldByName('DESCRIPTION').AsString;
end;

procedure TDlgLKLedgerCostManagementAcsPrint.FormCreate(Sender: TObject);
begin
  inherited;
  Report := TLKCostAccountsReport.Create(Self);
end;

function TDlgLKLedgerCostManagementAcsPrint.GetReport: TLKCostAccountsReport;
begin
  Result := TLKCostAccountsReport(inherited Report);
end;

procedure TDlgLKLedgerCostManagementAcsPrint.SetReport(Value:
    TLKCostAccountsReport);
begin
  inherited Report := Value;
end;

end.
