{*****************************************************************************}
{                                                                             }
{   This file is part of the LKAccounts components Library                    }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for printing a TLKLedgerNominalManagementAccounts                 }
{                                                                             }
{*****************************************************************************}

unit prLKLedgerNominalBudget;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants,
  System.Classes, Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs,
  Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, Math, Data.DB, Vcl.Grids,Vcl.DBGrids,
  prLKLedgerAnalysislManagementReportBase,prLKLedgerNominalManagementReportBase,
  LKDefine, LKRecordsComponents, LKAccountsComponents, frxClass, frxExportHTML,
  frxExportPDF, frxDBSet;

type
  TLKBudgetReport = class;
  TLKBudgetReportLine = class;

  TLKBudgetReport = class(TLKManagementBaseReport)
    function GetReport: TLKBudgetReport;
    function GetSectionTotal: TLKBudgetReportLine;
    procedure SetSectionTotal(Value: TLKBudgetReportLine);
  protected
    function CreateReportHead: TLKManagementHeadBase; override;
    function CreateReportLine: TLKManagementLineBase; override;
    property Report: TLKBudgetReport read GetReport;
  public
    function ConstructReport(AReport: TfrxReport): string; override;
    procedure Spreadsheet(CSVList: TStringList; const ReportDetail: Boolean);
        override;
    property SectionTotal: TLKBudgetReportLine read GetSectionTotal write
        SetSectionTotal;
  end;

  TLKBudgetReportHead = class(TLKManagementHeadBase)
    function GetReport: TLKBudgetReport;
    function GetSectionTotal: TLKBudgetReportLine;
    procedure SetSectionTotal(Value: TLKBudgetReportLine);
  protected
    function CreateReportLine: TLKManagementLineBase; override;
    property Report: TLKBudgetReport read GetReport;
  public
    property SectionTotal: TLKBudgetReportLine read GetSectionTotal write
        SetSectionTotal;
  end;

  TLKBudgetReportLine = class(TLKManagementLineBase)
    function GetReport: TLKBudgetReport;
  protected
    procedure GetData; override;
    property Report: TLKBudgetReport read GetReport;
  public
    procedure FillDataMemos; override;
    procedure Spreadsheet(CSVList: TStringList; const NormallyADebit: Boolean);
        override;
  end;

  TDlgLKLedgerNominalBudgetPrint = class(
      TDlgLKLedgerNominalManagementReportBase)
    procedure btnRememberClick(Sender: TObject);
    procedure cbxReportNameChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    function GetReport: TLKBudgetReport;
    procedure SetReport(const Value: TLKBudgetReport);
  private
    FMyPrintingComponent: TLKLedgerNominalBudget;
  public
    procedure Construct(APrintingComponent: TLKDataComponent; const aTitle:
        string); override;
    property Report: TLKBudgetReport read GetReport write SetReport;
  end;

var
  DlgLKLedgerNominalBudgetPrint: TDlgLKLedgerNominalBudgetPrint;

implementation

uses LKUtils, LKUtilsDB;

{$R *.DFM}



{
******************************* TLKBudgetReport ********************************
}
function TLKBudgetReport.ConstructReport(AReport: TfrxReport): string;
var
  I: Integer;
  S: string;
begin
  Result := inherited ConstructReport(AReport);
    TfrxMemoView(AReport.FindObject('MemoDescription')).Left := fr01cm * 20;
  with TfrxMemoView.Create(DataBand) do
  begin
    CreateUniqueName;
    DataSet := DataBand.DataSet;
    DataField := 'NOMINAL_CODE';
    LineSpacing := 1;
    Font.Size := 9;
    SetBounds(0, 0, fr01cm * 20, fr01cm * 4.5);
  end;
  CurrLeft := 60;
  try
    if (ReportDatesCount > 7) then
      TfrxReportPage(AReport.FindObject('Page1')).Orientation :=
        TPrinterOrientation.poLandscape;
    NewMemo('Year to Date', 18, [fsBold], [ftLeft, ftRight]).Name := 'MemoYTD';
    for I := 0 to ReportDatesCount - 1 do
    begin
      S := DateToStr(ReportDates[I]);
      NewMemo(Copy(S, 4, Length(S) - 3), 16);
    end;
    NewMemo('Total ', 18, [fsBold], [ftLeft, ftRight]).Name := 'MemoTotal';
    DataMemos.Delete(0);
  except on E: Exception do
    Result := E.Message;
  end;
end;

function TLKBudgetReport.CreateReportHead: TLKManagementHeadBase;
begin
  Result := TLKBudgetReportHead.Create(Report);
end;

function TLKBudgetReport.CreateReportLine: TLKManagementLineBase;
begin
  Result := TLKBudgetReportLine.Create(Report);
end;

function TLKBudgetReport.GetReport: TLKBudgetReport;
begin
  Result := TLKBudgetReport(inherited Report);
end;

function TLKBudgetReport.GetSectionTotal: TLKBudgetReportLine;
begin
  Result := TLKBudgetReportLine(inherited SectionTotal);
end;

procedure TLKBudgetReport.SetSectionTotal(Value: TLKBudgetReportLine);
begin
  inherited SectionTotal := Value;
end;

procedure TLKBudgetReport.Spreadsheet(CSVList: TStringList; const ReportDetail:
    Boolean);
var
  Headings: string;
  I: Integer;
begin
  // write column headings
  Headings := '"Nominal","Description"';
  for I := 0 to ReportDatesCount - 1 do
    Headings := Headings + ',' + DateToStr(ReportDates[I]);
  Headings := Headings + ',"Total"';
  CSVList.Add(Headings);
  // write the data in inherited
  inherited Spreadsheet(CSVList, ReportDetail);
end;

{
***************************** TLKBudgetReportHead ******************************
}
function TLKBudgetReportHead.CreateReportLine: TLKManagementLineBase;
begin
  Result := TLKBudgetReportLine.Create(Report);
end;

function TLKBudgetReportHead.GetReport: TLKBudgetReport;
begin
  Result := TLKBudgetReport(inherited Report);
end;

function TLKBudgetReportHead.GetSectionTotal: TLKBudgetReportLine;
begin
  Result := TLKBudgetReportLine(inherited SectionTotal);
end;

procedure TLKBudgetReportHead.SetSectionTotal(Value: TLKBudgetReportLine);
begin
  inherited SectionTotal := Value;
end;


{
***************************** TLKBudgetReportLine ******************************
}
procedure TLKBudgetReportLine.FillDataMemos;
var
  PrintValues: array[0..totalArrayCount] of Currency;
  PrintValuesTotal: Currency;
  I: Integer;
begin
  inherited FillDataMemos; // Memo[0].Text = Description
  for I := 0 to TotalArrayCount do
    PrintValues[I] := 0;
  if NormallyADebit then
  begin
    PrintValuesTotal := ValuesTotal;
    for I := 0 to Report.ReportDatesCount - 1 do
      PrintValues[I] := Values[I];
  end
  else
  begin
    PrintValuesTotal := -ValuesTotal;
    for I := 0 to Report.ReportDatesCount - 1 do
      PrintValues[I] := -Values[I];
  end;
  I := 0;
  while I < Memos.Count - 2 do
  begin
    Memos[I+1].Text := FloatToStrFOrBlank(PrintValues[I], ffNumber, 0);
    Inc(I);
  end;
  Memos[I+1].Text := FloatToStrFOrBlank(PrintValuesTotal, ffNumber, 0);
end;

procedure TLKBudgetReportLine.GetData;
var
  I: Integer;
  L: Currency;
begin
  with Report, ReportDataComponent.TblDetail do
    for I := 0 to ReportDatesCount - 1 do
      if Locate('PERIOD_END', ReportDates[I], []) then
      begin
        L :=  GetCurrency(FieldByName('BUDGET'));
          Values[I] := Values[I] + L;
        ValuesTotal := ValuesTotal + L;
      end;
end;

function TLKBudgetReportLine.GetReport: TLKBudgetReport;
begin
  Result := TLKBudgetReport(inherited Report);
end;

procedure TLKBudgetReportLine.Spreadsheet(CSVList: TStringList; const
    NormallyADebit: Boolean);
var
  PrintValues: array[0..totalArrayCount] of Currency;
  PrintValuesTotal: Currency;
  I: Integer;
  S: string;
begin
  for I := 0 to TotalArrayCount do
    PrintValues[I] := 0;
  if NormallyADebit then
  begin
    PrintValuesTotal := ValuesTotal;
    for I := 0 to Report.ReportDatesCount - 1 do
      PrintValues[I] := Values[I];
  end
  else
  begin
    PrintValuesTotal := -ValuesTotal;
    for I := 0 to Report.ReportDatesCount - 1 do
      PrintValues[I] := -Values[I];
  end;
  S := '"'+ NominalCode +'","'+Description;
  for I := 0 to Report.ReportDatesCount - 1 do
    S := S + '","' + CurrencyToStrFOrBlank(PrintValues[I], 2);
  S := S + '","' + CurrencyToStrFOrBlank(PrintValuesTotal, 2) + '"';
  CSVList.Add(S);
end;



{
************************ TDlgLKLedgerNominalBudgetPrint ************************
}
procedure TDlgLKLedgerNominalBudgetPrint.btnRememberClick(Sender: TObject);
begin
  FMyPrintingComponent.RememberInteger('PrintBudget', 'ReportDetail',
    rgReportDetail.ItemIndex , False);
  ShowMessage('Settings saved!');
end;

procedure TDlgLKLedgerNominalBudgetPrint.cbxReportNameChange(Sender: TObject);
begin
  inherited;
  //
end;

procedure TDlgLKLedgerNominalBudgetPrint.Construct(APrintingComponent:
    TLKDataComponent; const aTitle: string);
begin
  FMyPrintingComponent := TLKLedgerNominalBudget(APrintingComponent);
  inherited Construct(APrintingComponent, aTitle);
  rgReportDetail.ItemIndex :=
    FMyPrintingComponent.RecallInteger('PrintBudget', 'ReportDetail', 2 , False);
end;

procedure TDlgLKLedgerNominalBudgetPrint.FormCreate(Sender: TObject);
begin
  inherited;
  Report := TLKBudgetReport.Create(Self);
end;

function TDlgLKLedgerNominalBudgetPrint.GetReport: TLKBudgetReport;
begin
  Result := TLKBudgetReport(inherited Report);
end;

procedure TDlgLKLedgerNominalBudgetPrint.SetReport(const Value:
    TLKBudgetReport);
begin
  inherited Report := Value;
end;

end.


