{*****************************************************************************}
{                                                                             }
{   This file is part of the LKAccounts components Library                    }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for printing a TLKLedgerNominalManagementAccounts                 }
{                                                                             }
{*****************************************************************************}

unit prLKLedgerNominalManagementAcs;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants,
  System.Classes, Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs,
  Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, Math, Data.DB, Vcl.Grids,Vcl.DBGrids,
  prLKLedgerAnalysislManagementReportBase,prLKLedgerNominalManagementReportBase,
  LKDefine, LKRecordsComponents, LKAccountsComponents,
  frxClass, frxExportHTML, frxExportPDF, frxDBSet;

type
  TManagementReportType =
    (mrt12colma, mrtFinancial, mrt4qtABV, mrt4qtAVC, mrtPBAV);

  TLKManagementAccountsReport = class;
  TLKManagementAccountsReportLine = class;

  TLKManagementAccountsReport = class(TLKManagementBaseReport)
    function GetReport: TLKManagementAccountsReport;
    function GetSalesBudget: Currency;
    function GetSectionTotal: TLKManagementLineBase;
    procedure SetSectionTotal(Value: TLKManagementLineBase);
  private
    FReportType: TManagementReportType;
    FSalesActual: Currency;
    FSalesBudget: Currency;
  protected
    function CreateReportHead: TLKManagementHeadBase; override;
    function CreateReportLine: TLKManagementLineBase; override;
    procedure GetManagementData; override;
    property Report: TLKManagementAccountsReport read GetReport;
  public
    constructor Create(AOwner: TComponent); override;
    function ConstructReport(AReport: TfrxReport): string; override;
    procedure Spreadsheet(CSVList: TStringList; const ReportDetail: Boolean);
        override;
    property ReportType: TManagementReportType read FReportType write
        FReportType;
    property SalesActual: Currency read FSalesActual write FSalesActual;
    property SalesBudget: Currency read GetSalesBudget write FSalesBudget;
    property SectionTotal: TLKManagementLineBase read GetSectionTotal write
        SetSectionTotal;
  end;

  TLKManagementAccountsReportHead = class(TLKManagementHeadBase)
    function GetReport: TLKManagementAccountsReport;
    function GetSectionTotal: TLKManagementLineBase;
    procedure SetSectionTotal(Value: TLKManagementLineBase);
  protected
    function CreateReportLine: TLKManagementLineBase; override;
    property Report: TLKManagementAccountsReport read GetReport;
  public
    property SectionTotal: TLKManagementLineBase read GetSectionTotal write
        SetSectionTotal;
  end;

  TLKManagementAccountsReportLine = class(TLKManagementLineBase)
    function GetReport: TLKManagementAccountsReport;
  private
    FBudgetTotal: Currency;
    FPreviousYear: Currency;
    FYearToDate: Currency;
  protected
    procedure GetData; override;
    property Report: TLKManagementAccountsReport read GetReport;
  public
    Budget: array[0..TotalArrayCount] of Currency;
    procedure FillDataMemos; override;
    function HasValues: Boolean; override;
    procedure Spreadsheet(CSVList: TStringList; const NormallyADebit: Boolean);
        override;
    procedure ValuesAdd(Source: TLKManagementLineBase); override;
    procedure ValuesZero; override;
    property BudgetTotal: Currency read FBudgetTotal write FBudgetTotal;
    property PreviousYear: Currency read FPreviousYear write FPreviousYear;
    property YearToDate: Currency read FYearToDate write FYearToDate;
  end;

  TDlgLKLedgerNominalManagementAcsPrint = class(
      TDlgLKLedgerNominalManagementReportBase)
    procedure btnRememberClick(Sender: TObject);
    procedure cbxReportNameChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    function GetReport: TLKManagementAccountsReport;
    procedure SetReport(const Value: TLKManagementAccountsReport);
  private
    FMyPrintingComponent: TLKLedgerNominalManagementAccounts;
  public
    procedure Construct(APrintingComponent: TLKDataComponent; const aTitle:
        string); override;
    property Report: TLKManagementAccountsReport read GetReport write SetReport;
  end;

var
  DlgLKLedgerNominalManagementAcsPrint: TDlgLKLedgerNominalManagementAcsPrint;

implementation

uses LKUtils, LKUtilsDB;

{$R *.DFM}


{
************************* TLKManagementAccountsReport **************************
}
constructor TLKManagementAccountsReport.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
end;

function TLKManagementAccountsReport.ConstructReport(AReport: TfrxReport):
    string;
var
  I, J: Integer;
  S: string;

begin
  Result := inherited ConstructReport(AReport);
  CurrLeft := 40;
  try
    case ReportType of //
      mrt12colma: // total, monthCols, total
      begin
        if (ReportDatesCount > 7) then
          TfrxReportPage(AReport.FindObject('Page1')).Orientation :=
            TPrinterOrientation.poLandscape;
        NewMemo('Year to Date',18,[fsBold],[ftLeft, ftRight]).Name := 'MemoYTD';
        for I := 0 to ReportDatesCount - 1 do
        begin
          S := DateToStr(ReportDates[I]);
          NewMemo(Copy(S, 4, Length(S) - 3), 16);
        end;
        NewMemo('Total ', 18, [fsBold], [ftLeft, ftRight]).Name := 'MemoTotal';
      end;
      mrtFinancial, mrtPBAV: // 4 cols
      begin
        CurrLeft := 60;
        DataMemos[0].Width := fr01cm * CurrLeft;
        SubtotalMemos[0].Width := fr01cm * CurrLeft;
        TotalMemos[0].Width := fr01cm * CurrLeft;
        case ReportType of
          mrtFinancial:
          begin
            NewMemo('', 25);
            NewMemo('This Year', 25);
            NewMemo('', 25);
            NewMemo('Previous Year', 25);
          end;
          mrtPBAV: // 4 cols
          begin
            NewMemo('Previous', 25);
            NewMemo('Budget', 25);
            NewMemo('Actual', 25);
            NewMemo('Variance', 25);
          end;
        end;
      end;
      mrt4qtABV, mrt4qtAVC: // Actual, Budget, monthCols
      begin
        if (ReportDatesCount > 7) then
          TfrxReportPage(AReport.FindObject('Page1')).Orientation :=
            TPrinterOrientation.poLandscape;
        NewMemo('Actual', 18, [fsBold], [ftLeft, ftRight]).Name := 'MemoActual';
        NewMemo('Budget', 18, [fsBold], [ftRight]).Name := 'MemoBudget';
        for I := 0 to Trunc((ReportDatesCount-1)/3) do
        begin
          J := Min((3*I) + 2, ReportDatesCount-1);
          S := DateToStr(ReportDates[J]);
          NewMemo(Copy(S, 4, length(S) - 3), 17);
          case ReportType of
            mrt4qtABV:
            begin
              NewMemo('Budget', 17);
              NewMemo('Variance', 14, [], [ftRight]);
            end;
            mrt4qtAVC:
            begin
              NewMemo('Volume', 17);
              NewMemo('Cost', 14, [], [ftRight]);
            end;
          end;
        end;
      end;
    end;
  except on E: Exception do
    Result := E.Message;
  end;
end;

function TLKManagementAccountsReport.CreateReportHead: TLKManagementHeadBase;
begin
  Result := TLKManagementAccountsReportHead.Create(Report);
end;

function TLKManagementAccountsReport.CreateReportLine: TLKManagementLineBase;
begin
  Result := TLKManagementAccountsReportLine.Create(Report);
end;

procedure TLKManagementAccountsReport.GetManagementData;
var I: Integer;
begin
  FSalesActual := 0;
  FSalesBudget := 0;
  inherited GetManagementData;
  if not Report.IsProfitAndLoss then
    Exit;
  for I := 0 to Sections.Count -1 do
  if not TLKManagementAccountsReportHead(Sections[I]).NormallyADebit then
  begin
    FSalesActual := Sections[I].SectionTotal.ValuesTotal;
    FSalesBudget :=
      TLKManagementAccountsReportLine(Sections[I].SectionTotal).BudgetTotal;
    Break;
  end;
end;

function TLKManagementAccountsReport.GetReport: TLKManagementAccountsReport;
begin
  Result := TLKManagementAccountsReport(inherited Report);
end;

function TLKManagementAccountsReport.GetSalesBudget: Currency;
begin
  if (FSalesBudget = 0) then
    Result := FSalesActual
  else Result := FSalesBudget;
end;

function TLKManagementAccountsReport.GetSectionTotal: TLKManagementLineBase;
begin
  Result := TLKManagementLineBase(inherited SectionTotal);
end;

procedure TLKManagementAccountsReport.SetSectionTotal(Value:
    TLKManagementLineBase);
begin
  inherited SectionTotal := Value;
end;

procedure TLKManagementAccountsReport.Spreadsheet(CSVList: TStringList; const
    ReportDetail: Boolean);
var
  Headings: string;
  I, J: Integer;
begin
  // write column headings

  Headings := '"Nominal","Description"';
  case ReportType of
    mrt12colma: // total, monthCols, total
    begin
      for I := 0 to ReportDatesCount - 1 do
        Headings := Headings + ',' + DateToStr(ReportDates[I]);
      Headings := Headings + ',' + '"Total"';
    end;
    mrtFinancial:
    begin
      Headings := Headings + ',,"This Year"';
      Headings := Headings + ',,"Previous Year"';
    end;
    mrtPBAV: // 4 cols
    begin
      Headings := Headings + ',"Previous","Budget"';
      Headings := Headings + ',"Actual","Variance"';
    end;
    mrt4qtABV, mrt4qtAVC: // Actual, Budget, monthCols
    begin
      Headings := Headings + ',"Actual","Budget"';
      for I := 0 to Trunc((ReportDatesCount-1)/3) do
      begin
        J := Min((3*I) + 2, ReportDatesCount-1);
        Headings := Headings + ',"' + DateToStr(ReportDates[J]) + ',"';
        case ReportType of
          mrt4qtABV: Headings := Headings + ',"Budget","Variance"';
          mrt4qtAVC: Headings := Headings + ',"Volume","Cost"';
        end;
      end;
    end;
  end;
  CSVList.Add(Headings);
  // write the data in inherited
  inherited Spreadsheet(CSVList, ReportDetail);
end;


{
*********************** TLKManagementAccountsReportHead ************************
}
function TLKManagementAccountsReportHead.CreateReportLine:
    TLKManagementLineBase;
begin
  Result := TLKManagementAccountsReportLine.Create(Report);
end;

function TLKManagementAccountsReportHead.GetReport: TLKManagementAccountsReport;
begin
  Result := TLKManagementAccountsReport(inherited Report);
end;

function TLKManagementAccountsReportHead.GetSectionTotal: TLKManagementLineBase;
begin
  Result := TLKManagementLineBase(inherited SectionTotal);
end;

procedure TLKManagementAccountsReportHead.SetSectionTotal(Value:
    TLKManagementLineBase);
begin
  inherited SectionTotal := Value;
end;


{
*********************** TLKManagementAccountsReportLine ************************
}
procedure TLKManagementAccountsReportLine.FillDataMemos;
var
  PrintValues: array[0..TotalArrayCount] of Currency;
  PrintYearToDate: Currency;
  PrintValuesTotal: Currency;
  PrintPreviousYear: Currency;
  PrintBudget: Currency;
  VolumeVariance: Currency;
  I, J: Integer;

begin
  inherited FillDataMemos; // Memo[0].Text = Description
  for I := 0 to TotalArrayCount do
    PrintValues[I] := 0;
  PrintYearToDate := YearToDate;
  PrintValuesTotal := ValuesTotal;
  PrintPreviousYear := PreviousYear;
  PrintBudget := 0;
  for I := 0 to Report.ReportDatesCount - 1 do
   PrintBudget := PrintBudget + Budget[I];
  case Report.ReportType of
    mrt12colma: // total, monthCols, total
      for I := 0 to Report.ReportDatesCount - 1 do
        PrintValues[I] := Values[I];
    mrtFinancial:;
    mrtPBAV: // 4 cols
    begin
      for I := 0 to Report.ReportDatesCount - 1 do
        PrintValues[0] := PrintValues[0] + Values[I];
      PrintValues[1] := PrintValues[0] - PrintBudget;
    end;
    mrt4qtABV, mrt4qtAVC: // Actual, Budget, monthCols
    begin
      J := 0;
      for I := 0 to Trunc((Report.ReportDatesCount-1)/3) do
      begin
        PrintValues[J] := Values[J] + Values[J+1] + Values[J+2];
        PrintValues[J+1] :=  Budget[J] + Budget[J+1] + Budget[J+2];
        case Report.ReportType of
          mrt4qtABV:         // 'Budget' + 'Variance');
            PrintValues[J+2] := PrintValues[J] - PrintValues[J+1];
          mrt4qtAVC:
          begin // 'Volume'  + 'Cost');
            if Report.SalesBudget = 0 then
              VolumeVariance := 0
            else
            begin
              VolumeVariance :=
                (PrintValues[J+1] * Report.SalesActual/Report.SalesBudget)
                  - PrintValues[J+1];
              VolumeVariance := VolumeVariance * VariableCostPercent / 100;
            end;
            PrintValues[J+2] :=
              PrintValues[J] - VolumeVariance - PrintValues[J+1];
            PrintValues[J+1] := VolumeVariance;
          end;
        end;
        Inc(J,3);
      end;
    end;
  end; // case
  if not NormallyADebit then
  begin
    PrintYearToDate := -PrintYearToDate;
    PrintValuesTotal := -PrintValuesTotal;
    PrintPreviousYear := -PrintPreviousYear;
    PrintBudget := -PrintBudget;
    for I := 0 to TotalArrayCount do
      PrintValues[I] := -PrintValues[I];
  end;
  case Report.ReportType of
    mrt12colma: // total, monthCols, total
    begin
      Memos[1].Text := FloatToStrFOrBlank(PrintYearToDate, ffNumber, 0);
      I := 0;
      while (I < Memos.Count - 3) do
      begin
        Memos[I+2].Text := FloatToStrFOrBlank(PrintValues[I], ffNumber, 0);
        Inc(I);
      end;
      Memos[I+2].Text := FloatToStrFOrBlank(PrintValuesTotal, ffNumber, 0);
    end;
    mrtFinancial:
      if IsSubTotal then
      begin
        Memos[1].Text := '';
        Memos[2].Text := FloatToStrFOrBlank(PrintYearToDate, ffNumber, 0);
        Memos[3].Text := '';
        Memos[4].Text := FloatToStrFOrBlank(PrintPreviousYear, ffNumber, 0);
      end else
      begin
        Memos[1].Text := FloatToStrFOrBlank(PrintYearToDate, ffNumber, 0);
        Memos[2].Text := '';
        Memos[3].Text := FloatToStrFOrBlank(PrintPreviousYear, ffNumber, 0);
        Memos[4].Text := '';
      end;
    mrtPBAV: // 4 cols
    begin
      Memos[1].Text := FloatToStrFOrBlank(PrintPreviousYear, ffNumber, 0);
      Memos[2].Text := FloatToStrFOrBlank(PrintBudget, ffNumber, 0);
      Memos[3].Text := FloatToStrFOrBlank(PrintValuesTotal, ffNumber, 0);
      Memos[4].Text :=
        FloatToStrFOrBlank(PrintValuesTotal-PrintBudget,ffNumber,0);
    end;
    mrt4qtABV, mrt4qtAVC: // Actual, Budget, monthCols
    begin
      Memos[1].Text := FloatToStrFOrBlank(PrintValuesTotal, ffNumber, 0);
      Memos[2].Text := FloatToStrFOrBlank(PrintBudget, ffNumber, 0);
      for I := 0 to -2 + 3*Trunc((Report.ReportDatesCount)/3) do
        Memos[1 + I * 2].Text := FloatToStrFOrBlank(PrintValues[I], ffNumber,0);
    end;
  end;
end;

procedure TLKManagementAccountsReportLine.GetData;
var
  I: Integer;
  L: Currency;
begin
  with Report, ReportDataComponent do
  begin
    with TblMaster do
    begin
      YearToDate := YearToDate
        + GetCurrency(FieldByName('PROFIT_AND_LOSS'))
        + GetCurrency(FieldByName('BALANCE_SHEET'));
      PreviousYear := PreviousYear +
        GetCurrency(FieldByName('PREVIOUS_PERIOD'))
    end;
    for I := 0 to ReportDatesCount - 1 do
      with TblDetail do
        if Locate('PERIOD_END', ReportDates[I], []) then
        begin
          L := GetCurrency(FieldByName('THIS_MONTH'));
          Values[I] := Values[I] + L;
          ValuesTotal := ValuesTotal + L;
        end;
    if (ContainerComponent is TLKLedgerNominal) then
      with TLKLedgerNominal(ContainerComponent).Budget.TblDetail do
        for I := 0 to ReportDatesCount - 1 do
          if Locate('PERIOD_END', ReportDates[I], []) then
          begin
            L := GetCurrency(FieldByName('BUDGET'));
            Budget[I] := Budget[I] + L;
            BudgetTotal := BudgetTotal + L;
          end;
  end;
end;

function TLKManagementAccountsReportLine.GetReport: TLKManagementAccountsReport;
begin
  Result := TLKManagementAccountsReport(inherited Report);
end;

function TLKManagementAccountsReportLine.HasValues: Boolean;
var
  Total: Currency;
  I: Integer;

  function Positive(const Amount: Currency): Currency;
  begin
    if (Amount < 0) then
      Result := -Amount
    else
      Result := Amount;
  end;

begin
  Result := inherited HasValues;
  if not Result then
  case Report.ReportType of
    mrtPBAV,
    mrt4qtABV, mrt4qtAVC:
    begin
      Total := 0;
      for I := 0 to Report.ReportDatesCount - 1  do
        Total := Total + Positive(Budget[I]);
      Result := (Total > 0);
    end;
  end;
end;

procedure TLKManagementAccountsReportLine.Spreadsheet(CSVList: TStringList;
    const NormallyADebit: Boolean);
var
  PrintValues: array[0..totalArrayCount] of Currency;
  PrintYearToDate: Currency;
  PrintValuesTotal: Currency;
  PrintPreviousYear: Currency;
  PrintBudget: Currency;
  VolumeVariance: Currency;
  I, J: Integer;
  OkToPrint: Boolean;
  S: string;
begin
  OkToPrint := (YearToDate <> 0);
  for I := 0 to Report.ReportDatesCount - 1 do
    if Values[I] <> 0 then
      OkToPrint := True;
  if not OkToPrint then
    Exit;
  for I := 0 to TotalArrayCount do
    PrintValues[I] := 0;
  PrintYearToDate := YearToDate;
  PrintValuesTotal := ValuesTotal;
  PrintPreviousYear := PreviousYear;
  PrintBudget := 0;
  for I := 0 to Report.ReportDatesCount - 1 do
   PrintBudget := PrintBudget + Budget[I];
  case Report.ReportType of
    mrt12colma: // total, monthCols, total
      for I := 0 to Report.ReportDatesCount - 1 do
        PrintValues[I] := Values[I];
    mrtFinancial:;
    mrtPBAV: // 4 cols
    begin
      for I := 0 to Report.ReportDatesCount - 1 do
      begin
        PrintValues[0] := PrintValues[0] + Values[I];
      end;
      PrintValues[1] := PrintValues[0] - PrintBudget;
    end;
    mrt4qtABV, mrt4qtAVC: // Actual, Budget, monthCols
    begin
      J := 0;
      for I := 0 to Trunc((Report.ReportDatesCount-1)/3) do
      begin
        PrintValues[J] := Values[J] + Values[J+1] + Values[J+2];
        PrintValues[J+1] :=  Budget[J] + Budget[J+1] + Budget[J+2];
        case Report.ReportType of
          mrt4qtABV:
          begin // 'Budget' + 'Variance');
            PrintValues[J+2] := PrintValues[J] - PrintValues[J+1];
          end;
          mrt4qtAVC:
          begin // 'Volume'  + 'Cost');
            if Report.SalesBudget = 0 then
              VolumeVariance := 0
            else
              VolumeVariance :=
                (PrintValues[J+1] * Report.SalesActual/Report.SalesBudget)
                  - PrintValues[J+1];
            PrintValues[J+2] :=
              PrintValues[J] - VolumeVariance - PrintValues[J+1];
            PrintValues[J+1] := VolumeVariance;
          end;
        end;
        Inc(J,3);
      end;
    end;
  end; // case
  if not NormallyADebit then
  begin
    PrintYearToDate := -PrintYearToDate;
    PrintValuesTotal := -PrintValuesTotal;
    PrintPreviousYear := -PrintPreviousYear;
    PrintBudget := -PrintBudget;
    for I := 0 to TotalArrayCount do
      PrintValues[I] := -PrintValues[I];
  end;
  //Report.SetReportTabs(0);
  S := '"'+ NominalCode +'","'+Description;
  case Report.ReportType of
    mrt12colma: // total, monthCols, total
    begin
      for I := 0 to Report.ReportDatesCount - 1 do
        S := S + '","' + CurrencyToStrFOrBlank(PrintValues[I], 2);
      S := S + '","' + CurrencyToStrFOrBlank(PrintValuesTotal, 2) + '"';
    end;
    mrtFinancial:
    begin
      if IsSubTotal then
        S := S + '","';
      S := S + '","' + CurrencyToStrFOrBlank(PrintYearToDate, 2) + '","';
      S := S + '","'+ CurrencyToStrFOrBlank(PrintPreviousYear, 2) + '"';
      if not IsSubTotal then
        S := S + ',""';
    end;
    mrtPBAV: // 4 cols
    begin
      S := S + '","' + CurrencyToStrFOrBlank(PrintPreviousYear, 2);
      S := S + '","' + CurrencyToStrFOrBlank(PrintBudget, 2);
      S := S + '","' + CurrencyToStrFOrBlank(PrintYearToDate, 2);
      S := S + '","' + CurrencyToStrFOrBlank(PrintYearToDate-PrintBudget, 2)
             + '"';
    end;
    mrt4qtABV, mrt4qtAVC: // Actual, Budget, monthCols
    begin
      S := S + '","' + CurrencyToStrFOrBlank(PrintYearToDate, 2);
      S := S + '","' + CurrencyToStrFOrBlank(PrintBudget, 2);
      for I := 0 to 3*Trunc((Report.ReportDatesCount-1)/3) do
        S := S + '","' + CurrencyToStrFOrBlank(PrintValues[I], 2);
      end;
    end;
  CSVList.Add(S);
end;

procedure TLKManagementAccountsReportLine.ValuesAdd(Source:
    TLKManagementLineBase);
var
  I: Integer;
  S: TLKManagementAccountsReportLine;
begin
  inherited ValuesAdd(Source);
  if (Source is TLKManagementAccountsReportLine) then
    S := TLKManagementAccountsReportLine(Source)
  else Exit;
  for I := 0 to Report.ReportDatesCount - 1 do
    Budget[I] := Budget[I] + S.Budget[I];
  BudgetTotal := BudgetTotal + S.BudgetTotal;
  YearToDate := YearToDate + S.YearToDate;
  PreviousYear := PreviousYear + S.PreviousYear;
end;

procedure TLKManagementAccountsReportLine.ValuesZero;
var
  I: Integer;
begin
  inherited ValuesZero;
  for I := 0 to TotalArrayCount do
    Budget[I] := 0;
  YearToDate := 0;
  PreviousYear := 0;
  BudgetTotal := 0;
end;



{
******************** TDlgLKLedgerNominalManagementAcsPrint *********************
}
procedure TDlgLKLedgerNominalManagementAcsPrint.btnRememberClick(Sender:
    TObject);
begin
  FMyPrintingComponent.RememberInteger('PrintAcs', 'ReportDetail',
    rgReportDetail.ItemIndex , False);
  FMyPrintingComponent.RememberInteger('PrintAcs', 'ReportType',
    cbxReportName.ItemIndex , False);
  ShowMessage('Settings saved!');
end;

procedure TDlgLKLedgerNominalManagementAcsPrint.cbxReportNameChange(Sender:
    TObject);
begin
  Report.ReportType := TManagementReportType(cbxReportName.ItemIndex);
end;

procedure TDlgLKLedgerNominalManagementAcsPrint.Construct(APrintingComponent:
    TLKDataComponent; const aTitle: string);
begin
  FMyPrintingComponent := TLKLedgerNominalManagementAccounts(APrintingComponent);
  inherited Construct(APrintingComponent, aTitle);
  rgReportDetail.ItemIndex :=
    FMyPrintingComponent.RecallInteger('PrintAcs', 'ReportDetail', 2 , False);
  cbxReportName.ItemIndex :=
    FMyPrintingComponent.RecallInteger('PrintAcs', 'ReportType', 0 , False);
  cbxReportNameChange(nil);
end;

procedure TDlgLKLedgerNominalManagementAcsPrint.FormCreate(Sender: TObject);
begin
  inherited;
  Report := TLKManagementAccountsReport.Create(Self);
  with cbxReportName.Items do
  begin
    Clear;
    Add('12 Column Management Accounts');
    Add('Financial, Current, Previous');
    Add('4 Quarter Actual, Budget, Variance');
    Add('4 Quarter Actual, Volume, Cost');
    Add('Previous, Budget, Actual, Variance');
  end;
end;

function TDlgLKLedgerNominalManagementAcsPrint.GetReport:
    TLKManagementAccountsReport;
begin
  Result := TLKManagementAccountsReport(inherited Report);
end;

procedure TDlgLKLedgerNominalManagementAcsPrint.SetReport(const Value:
    TLKManagementAccountsReport);
begin
  inherited Report := Value;
end;

end.
