{*****************************************************************************}
{                                                                             }
{   This file is part of the LKAccounts components Library                    }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for printing a TLKLedgerNominalManagementAccounts                 }
{                                                                             }
{*****************************************************************************}

unit prLKLedgerNominalManagementReportBase;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants,
  System.Classes, Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs,
  Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, Math, Vcl.Grids, Vcl.DBGrids,Data.DB,
  prLKLedgerAnalysislManagementReportBase, LKDefine, LKRecordsComponents,
  LKAccountsComponents, frxClass, frxExportHTML, frxExportPDF, frxDBSet;

type
  TDlgLKLedgerNominalManagementReportBase = class(
      TDlgLKLedgerAnalysisManagementReportBase)
    dbg: TDBGrid;
    dsReports: TDataSource;
    rgReportDetail: TRadioGroup;
    StaticTextReport: TStaticText;
    procedure dsReportsDataChange(Sender: TObject; Field: TField);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormDestroy(Sender: TObject);
    procedure rgReportDetailClick(Sender: TObject);
  private
    FMyPrintingComponent: TLKLedgerNominalManagementAccounts;
  public
    procedure Construct(APrintingComponent: TLKDataComponent; const aTitle:
        string); override;
  end;

var
  DlgLKLedgerNominalManagementReportBase: TDlgLKLedgerNominalManagementReportBase;

implementation

uses LKUtils, LKUtilsDB;

{$R *.DFM}

{
******************* TDlgLKLedgerNominalManagementReportBase ********************
}
procedure TDlgLKLedgerNominalManagementReportBase.Construct(APrintingComponent:
    TLKDataComponent; const aTitle: string);
begin
  FMyPrintingComponent := TLKLedgerNominalManagementAccounts(APrintingComponent);
  inherited Construct(APrintingComponent, aTitle);
  rgReportDetail.ItemIndex :=
    FMyPrintingComponent.RecallInteger('PrintAcs', 'ReportDetail', 1 , False);
  rgReportDetailClick(nil);
  with FMyPrintingComponent do
  begin
    ContainerComponent.ReportLayouts.TblMasterDataSource := dsReports;
    MasterComponent := ContainerComponent.ReportLines;
    MasterTable := tbDetail;
    MasterFields := 'NOMINAL_CODE';
  end;
end;

procedure TDlgLKLedgerNominalManagementReportBase.dsReportsDataChange(Sender:
    TObject; Field: TField);
begin
  inherited;
  if Assigned(dsReports.DataSet) then
    eReportTitle.Text := dsReports.DataSet.FieldByName('TITLE').AsString;
end;

procedure TDlgLKLedgerNominalManagementReportBase.FormClose(Sender: TObject;
    var Action: TCloseAction);
begin
  with FMyPrintingComponent do
  begin
    RememberInteger('PrintAcs', 'ReportDetail', rgReportDetail.ItemIndex,False);
    Close;
    MasterComponent := nil;
    MasterFields := '';
    Open;
  end;
  inherited;
end;

procedure TDlgLKLedgerNominalManagementReportBase.FormDestroy(Sender: TObject);
begin
  FMyPrintingComponent.ContainerComponent.ReportLayouts.TblMasterDataSource:=nil;
  inherited;
end;

procedure TDlgLKLedgerNominalManagementReportBase.rgReportDetailClick(Sender:
    TObject);
begin
  inherited;
  if Report.PrintDetail <> Boolean(rgReportDetail.ItemIndex) then
  begin
    Report.PrintDetail := Boolean(rgReportDetail.ItemIndex);
    ReportNeedsConstruction := True;
  end;
end;

end.
