{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Base Form for the display of errors found before processing            }
{                                                                             }
{*****************************************************************************}

unit LKBatchErrorsBaseDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, ComCtrls, Buttons, StdCtrls, Grids, Menus;

type
  TDlgLKBatchErrorsBase = class(TForm)
    Panel2: TPanel;
    btnPrint: TBitBtn;
    btnCorrectAll: TBitBtn;
    btnAbort: TBitBtn;
    pErrors: TPanel;
    sgErrors: TStringGrid;
    Panel3: TPanel;
    pWarnings: TPanel;
    Panel5: TPanel;
    sgWarnings: TStringGrid;
    procedure btnPrintClick(Sender: TObject);
    procedure btnCorrectAllClick(Sender: TObject);
  private
    { Private declarations }
  public
    AreErrors, AreWarnings: Boolean;
    ProblemDaybook: TComponent;
    procedure AddError(const ErrorInfo: array of string);
    procedure AddWarning(const WarningInfo: array of string);
    procedure ClearMessages;
  end;

var
  DlgLKBatchErrorsBase: TDlgLKBatchErrorsBase;

implementation

uses prLKStringGrid, LKRecordsComponents;

{$R *.DFM}

procedure TDlgLKBatchErrorsBase.AddError(const ErrorInfo: array of string);
var
 I: integer;
begin
  // ErrCodeInteger,DocRef,DocItem,Description,CorrectionAction,
  with sgErrors do
  begin
    // is already a blank row for first item else append a row
    if AreErrors then
      RowCount := RowCount + 1;
    AreErrors := true;
    for I := 0 to High(ErrorInfo) do
      Cells[I, RowCount - 1] := ErrorInfo[I];
  end;
  if errorInfo[0] = '999' then // abort problem
    btnCorrectAll.Enabled := false;
end;

procedure TDlgLKBatchErrorsBase.AddWarning(
  const WarningInfo: array of string);
var
  I: integer;
begin
  with sgWarnings do
  begin
    // is already a blank row for first item else append a row
    if AreWarnings then
      RowCount := RowCount + 1;
    AreWarnings := True;
    for I := 0 to High(warningInfo) do
      Cells[I, RowCount - 1] := WarningInfo[I];
    Cells[ColCount - 1, RowCount - 1] := 'Y'
  end;
end;

procedure TDlgLKBatchErrorsBase.btnPrintClick(Sender: TObject);
begin
  with TDlgLKStringGridPrinter.create(self) do
  try
    if AreErrors then
      PrintStringGrid(TLKDataComponent(ProblemDaybook).NameFriendly +
        ' errors ', sgErrors, false);
    if areWarnings then
      PrintStringGrid(TLKDataComponent(ProblemDaybook).NameFriendly +
        ' warnings ', sgWarnings, false);
  finally free;
  end;
end;

procedure TDlgLKBatchErrorsBase.ClearMessages;
begin
  btnCorrectAll.Enabled := true;
  AreErrors := False;
  with sgErrors do
  begin
    RowCount := 2;
    FixedRows := 1;
  end;
  AreWarnings := False;
  with sgWarnings do
  begin
    RowCount := 2;
    FixedRows := 1;
  end;
end;

procedure TDlgLKBatchErrorsBase.btnCorrectAllClick(Sender: TObject);
begin
  // override here
end;

end.

