{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form relating to TLKDBEditsPanel and LKDBGridController which          }
{      provides image viewing of TDataComponent scanned documents etc.        }
{                                                                             }
{*****************************************************************************}

unit LKDBImageEditDlg;

interface

uses
  Windows, Messages, SysUtils, System.UITypes, Variants, Classes,
  Graphics, Controls, Forms,
  Dialogs, ExtDlgs, ExtCtrls, Buttons, ToolWin, ComCtrls,
  StdCtrls, ImgList, ActnList, ExtActns, StdActns, LKDefine, DB, jpeg,
  System.Actions;

type
  TDlgLKDBImageEdit = class(TForm)
    ToolBar1: TToolBar;
    ActionListAcsComp: TActionList;
    ActionPrint: TAction;
    ActionStayOnTop: TAction;
    ImageListAC: TImageList;
    ContainImage: TScrollBox;
    ActionAcquire: TAction;
    Panel2: TPanel;
    eFileName: TEdit;
    ActionOpenPicture: TAction;
    OpenPictureDialog1: TOpenPictureDialog;
    ScrollBox1: TScrollBox;
    Image: TImage;
    ToolButton1: TToolButton;
    ToolButton3: TToolButton;
    ToolButton2: TToolButton;
    procedure eFileNameExit(Sender: TObject);
    procedure ActionAcquireExecute(Sender: TObject);
    procedure aPasteExecute(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ActionOpenPictureExecute(Sender: TObject);
    procedure ActionCloseExecute(Sender: TObject);
    procedure ActionStayOnTopExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    FFilePath, FFileName: string;
    FMyField: TField;
    FSaveImage: boolean;
  public
    { Public declarations }
    property SaveImage: boolean read FSaveImage write FSaveImage;
    procedure Construct(const AFilePath: string; AField: TField);
    procedure LoadFieldImage;
  end;

var
  DlgLKDBImageEdit: TDlgLKDBImageEdit;

implementation

{$R *.dfm}

uses Clipbrd;

{ TDlgLKDBImageEdit }

procedure TDlgLKDBImageEdit.Construct(const AFilePath: string; AField: TField);
begin
  FMyField := AField;
  FFilePath := AFilePath;
  OpenPictureDialog1.InitialDir := FFilePath;
end;

procedure TDlgLKDBImageEdit.LoadFieldImage;
begin
  eFileName.Text := FMyField.AsString;
  eFileNameExit(self);
  if FileExists(FFilePath + PathDelim + FFileName) then
    Image.Picture.LoadFromFile(FFilePath + PathDelim + FFileName);
  SaveImage := false;
end;

procedure TDlgLKDBImageEdit.eFileNameExit(Sender: TObject);
begin
  inherited;
  if (FFileName = eFileName.Text) then
    Exit;
  SaveImage := True;
  FFileName := eFileName.Text;
  OpenPictureDialog1.FileName := FFilePath + PathDelim + FFileName;
end;

procedure TDlgLKDBImageEdit.ActionAcquireExecute(Sender: TObject);
begin
  Showmessage('Sorry: Acquire not yet implemented');
end;

procedure TDlgLKDBImageEdit.aPasteExecute(Sender: TObject);
begin
  Image.Picture.LoadFromClipboardFormat(cf_BitMap,
    ClipBoard.GetAsHandle(cf_Bitmap), 0);
  SaveImage := true;
end;

procedure TDlgLKDBImageEdit.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  inherited;
  if SaveImage then
  begin
    if (ModalResult <> mrOK) then
      if (MessageDlg('Do you want to save your changes?'
        , mtInformation, [mbYes, mbNo], 0) = mrYes) then
        ModalResult := mrOK
      else
        exit;
    if (FFileName = '') then
      LKError.Create('You must enter a file name');
    Image.Picture.SaveToFile(FFilePath + PathDelim + FFileName);
  end;
end;

procedure TDlgLKDBImageEdit.FormCreate(Sender: TObject);
begin
  ActionStayOnTopExecute(nil);
end;

procedure TDlgLKDBImageEdit.ActionOpenPictureExecute(Sender: TObject);
var
  JPG: TJPEGImage;
begin
  inherited;
  if not OpenPictureDialog1.Execute then
    exit;
  eFileName.Text := ExtractFileName(OpenPictureDialog1.FileName);
  SaveImage := false;
  JPG := TJPEGImage.Create;
  try
    JPG.LoadFromFile(OpenPictureDialog1.FileName);
    Image.Picture.Assign(JPG);
  finally JPG.Free;
  end;
end;

procedure TDlgLKDBImageEdit.ActionStayOnTopExecute(Sender: TObject);
begin
  if ActionStayOnTop.Checked then
    FormStyle := fsStayOnTop
  else
    FormStyle := fsNormal;
end;

procedure TDlgLKDBImageEdit.ActionCloseExecute(Sender: TObject);
begin
  inherited;
  if (eFileName.Text <> FMyField.AsString) and (eFileName.Text > '') then
  begin
    FMyField.Dataset.Edit;
    FMyField.AsString := eFileName.Text;
  end;
  Close;
end;

end.

