{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Base Form for the display of TLKDataComponent without controls         }
{                                                                             }
{*****************************************************************************}

unit LKDataCompBaseDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, System.UITypes,
  Graphics, Controls, Forms, Dialogs,
  Grids, DBGrids, StdCtrls, Mask, DBCtrls, ExtCtrls, Menus, Variants,
  Buttons, ComCtrls, ToolWin, LKDlgs, ActnList, ImgList, DB,LKRecordsComponents,
  LKComponentDlg, LKDefine, LKBaseComponent, StdActns, ClipBrd, Math;

type

  TFormLKDataComponentBase = class(TFormLKComponent)
    ActionFind: TAction;
    DSShowMaster: TDataSource;
    ActionBlockDelete: TAction;
    ActionChangeIndex: TAction;
    ActionDrillDown: TAction;
    ActionImport: TAction;
    ActionExport: TAction;
    ActionChildComponentView: TAction;
    PopupMenu1: TPopupMenu;
    Cut1: TMenuItem;
    Copy1: TMenuItem;
    Paste1: TMenuItem;
    Drilldown2: TMenuItem;
    StayonTop1: TMenuItem;
    PopupBreak1: TMenuItem;
    ActionUndo: TAction;
    ActionRefresh: TAction;
    ActionChangeIndexFieldValue: TAction;
    ActionReverseIndex: TAction;
    ActionCurrIdxAsDefault: TAction;
    ActionViewAllDetail: TAction;
    ActionFetchAll: TAction;
    ActionListViewChildAC: TActionList;
    ActionDeleteRecord: TAction;
    Deleterecord1: TMenuItem;
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ActionBlockDeleteExecute(Sender: TObject);
    procedure ActionPrintExecute(Sender: TObject);
    procedure ActionImportExecute(Sender: TObject);
    procedure ActionExportExecute(Sender: TObject);
    procedure ActionUndoExecute(Sender: TObject);
    procedure ActionChangeIndexFieldValueExecute(Sender: TObject);
    procedure ActionChangeIndexExecute(Sender: TObject);
    procedure ActionDrillDownExecute(Sender: TObject);
    procedure ActionFindExecute(Sender: TObject);
    procedure ActionChildComponentViewExecute(Sender: TObject);
    procedure ActionCurrIdxAsDefaultExecute(Sender: TObject);
    procedure ActionViewAllDetailExecute(Sender: TObject);
    procedure ActionFetchAllExecute(Sender: TObject);
    procedure ActionRefreshExecute(Sender: TObject);
    procedure ActionDeleteRecordExecute(Sender: TObject);
    procedure ActionReverseIndexExecute(Sender: TObject);
  private
    FMyDataComponent: TLKDataComponent;
    function DimsToStr: string;
  protected
    DOldFilter: string;
    DWasFiltered: Boolean;
    MOldFilter: string;
    MWasFiltered: Boolean;
    HideMasterFields: string;
    procedure PlaceEditControls; virtual;
    procedure SetDataSourceDataSets; virtual;
    procedure SetHideFields; virtual;
  public
    procedure Construct(Sender: TLKDataComponent);
    procedure SetFolderStoreDocumentImages(const Value: string); virtual;
    procedure SetKeyFieldsEditing; virtual;
    function ShowForm(const ShowAsModal: boolean; Buttons: TMsgDlgButtons):
      TModalResult; virtual;
    //   function ShowForm(const showAsModal : boolean; Buttons: TMsgDlgButtons): TModalResult; virtual;
  end;

var
  FormLKDataComponentBase: TFormLKDataComponentBase;

implementation

uses LKUtils;

{$R *.DFM}

procedure TFormLKDataComponentBase.SetDataSourceDataSets;
begin
  FMyDataComponent.TblMasterDataSource := DSShowMaster;
end;

procedure TFormLKDataComponentBase.SetFolderStoreDocumentImages(
  const Value: string);
begin
  // Set Images folder on ImageEdits
  //Showmessage('Function not implemented');
  // Application.ProcessMessages;
end;

procedure TFormLKDataComponentBase.SetHideFields;
begin
  HideMasterFields := FMyDataComponent.MasterFields;
end;

procedure TFormLKDataComponentBase.PlaceEditControls;
begin
  // override in decendants
end;      

function TFormLKDataComponentBase.DimsToStr: string;
begin
  Result := IntToStr(Top)+ '|' + IntToStr(Left)+ '|'
          + IntToStr(Width)+ '|' + IntToStr(Height);
end;

procedure TFormLKDataComponentBase.Construct(Sender: TLKDataComponent);
begin
  FMyDataComponent := Sender;
  inherited Construct(Sender);
  //WindowMenu := MyAccountsComponent.WindowMenu;
  //if assigned(WindowMenuItem) then
  // WindowMenuItem.ImageIndex := MyAccountsComponent.ImageIndex;
  if (FMyDataComponent.HelpContext > 0) then
    HelpContext := FMyDataComponent.HelpContext;
  SetHideFields;
  with FMyDataComponent do
  begin
    ActionListViewChildAC.Images := Images;
  end;
  SetDataSourceDataSets;
  PlaceEditControls;
  //SetFolderStoreDocumentImages(MyAccountsComponent.FolderStoreDocumentImages);
end;

procedure TFormLKDataComponentBase.FormShow(Sender: TObject);
begin
  inherited;
  SetKeyFieldsEditing;
  with FMyDataComponent do
  begin
    ActionBlockDelete.Visible := Editing;
    ActionBlockDelete.Enabled := Editing;
    ActionCut.Visible := Editing;
    ActionCut.Enabled := Editing;
    ActionPaste.Visible := Editing;
    ActionPaste.Enabled := Editing;
    ActionUndo.Enabled := Editing and (ChangeCount > 0);
  end;
end;

procedure TFormLKDataComponentBase.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  with FMyDataComponent do
  begin
    Post;
    ApplyUpdates(0);
    if (WindowState = wsNormal) then
      RememberString('FormDims', Self.Name, DimsToStr, False);
  end;
  inherited;
end;

{
function TFormLKDataComponentBase.ShowForm(const showAsModal : boolean; Buttons: TMsgDlgButtons):
        TModalResult;
begin
  aClose.Visible := not showAsModal;
  result := mrOK;
  if not visible then if showAsModal then  result := ShowModal else Show;
end;
}

function TFormLKDataComponentBase.ShowForm(const ShowAsModal: boolean;
  Buttons: TMsgDlgButtons): TModalResult;
var
  S: string;

  function ExtractNext: Integer;
  var
    I: Integer;
  begin
    try
      I := Pos('|',S);
      if (I = 0) then
        Result := StrToInt(S)
      else
      begin
        Result := StrToInt(Copy(S,1,I-1));
        Delete(S,1,I);
      end;
    except
      Result := 200;
    end;
  end;

begin
  Result := mrOK;
  S:= FMyDataComponent.RecallString('FormDims', Self.Name, '', False);
  if (S > '') then
  begin
    Top   := ExtractNext;
    Left  := ExtractNext;
    Width := Min(ExtractNext, Screen.Width);
    Height:= Min(ExtractNext, Screen.Height);
  end;
  ActionClose.Visible := not ShowAsModal;
  ActionFetchAll.Enabled := (FMyDataComponent.TblMaster.WhereClause < '');
  if Visible then
    Close;
  if ShowAsModal then
    Result := ShowModal
  else
    Show;
end;

procedure TFormLKDataComponentBase.ActionBlockDeleteExecute(Sender: TObject);
var
  MySelect: TLKACTableSelect;
begin
  with FMyDataComponent do
  begin
    MySelect := tbsMaster;
    if TblDetailUsed then
      MySelect := tbsBoth;
    DlgBlockDeleteExecute(Self, MySelect, Filter, TblDetail.Filter);
  end;
end;

procedure TFormLKDataComponentBase.ActionImportExecute(Sender: TObject);
begin
  FMyDataComponent.DlgImportExportExecute(Self, False, False);
end;

procedure TFormLKDataComponentBase.ActionPrintExecute(Sender: TObject);
begin
  with FMyDataComponent do
    DlgPrintExecute(NameFriendly, 'Default', Null, False);
end;

procedure TFormLKDataComponentBase.ActionExportExecute(Sender: TObject);
begin
  FMyDataComponent.DlgImportExportExecute(Self, True, False);
end;

procedure TFormLKDataComponentBase.ActionChangeIndexFieldValueExecute(
  Sender: TObject);
begin
  FMyDataComponent.DlgChangeKeyFieldValueExecute;
end;

procedure TFormLKDataComponentBase.ActionUndoExecute(Sender: TObject);
begin
  if not (FMyDataComponent.UndoLastChange(true)) then
    ShowMessage('No more changes to undo!');
end;

procedure TFormLKDataComponentBase.ActionChangeIndexExecute(
  Sender: TObject);
begin
  // override in decendants
  ShowMessage('ChangeIndex Not implemented');
end;

procedure TFormLKDataComponentBase.ActionDrillDownExecute(Sender: TObject);
begin
  // override in decendants
  ShowMessage('DrillDown Not implemented');
end;

procedure TFormLKDataComponentBase.ActionFindExecute(Sender: TObject);
begin
  FMyDataComponent.DlgFindExecute('');
end;

procedure TFormLKDataComponentBase.ActionChildComponentViewExecute(
  Sender: TObject);
var
  Cr: TLKContainedRecords;
begin
  Cr :=
    TLKContainedRecords(FMyDataComponent.Components[TComponent(Sender).Tag]);
  if not Assigned(cr) then
    Exit;
  Cr.View(null, false, []);
end;

procedure TFormLKDataComponentBase.SetKeyFieldsEditing;
begin
  // override in decendants
 // showmessage('SetKeyFieldsEditing Not implemented');
end;

procedure TFormLKDataComponentBase.ActionCurrIdxAsDefaultExecute(
  Sender: TObject);
begin
  Showmessage('Current index has been saved for when you restart the program');
end;

procedure TFormLKDataComponentBase.ActionViewAllDetailExecute(Sender: TObject);
begin
  with FMyDataComponent do
    ViewAllDetail(nameFriendly + ' Detail', [mbOK]);
end;

procedure TFormLKDataComponentBase.ActionFetchAllExecute(Sender: TObject);
begin
  ActionFetchAll.Enabled := False;
  FMyDataComponent.Close;
  FMyDataComponent.TblMaster.WhereClause := '';
  FMyDataComponent.Open;
end;

procedure TFormLKDataComponentBase.ActionRefreshExecute(Sender: TObject);
begin
  FMyDataComponent.Refresh;
end;

procedure TFormLKDataComponentBase.ActionReverseIndexExecute(Sender: TObject);
begin
  //FMyAccountsComponent.TblMaster.
end;

procedure TFormLKDataComponentBase.ActionDeleteRecordExecute(Sender: TObject);
begin
  if (MessageDlg('Confirm: delete this record?', mtInformation, [mbOK,
    mbCancel], 0) = mrOK) then
    FMyDataComponent.TblMaster.Delete;
end;

end.

