{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for block deleting TLKDataComponent records                       }
{                                                                             }
{*****************************************************************************}

unit LKDataCompBlockDelDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, System.UITypes,
  Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ImgList, Menus, ActnList, Db, Buttons, DBCtrls, ToolWin,
  ComCtrls, LKDataComponentDlg, LKRecordsComponents, LKDefine, LKUtils,
  LKUtilComponents, 
  LKdlgs, JvExExtCtrls, JvNetscapeSplitter, StdActns, Grids, DBGrids,
  JvExDBGrids, JvDBGrid, JvDBUltimGrid, LKDBControls;

type
  TFormLKDataCompBlockDelete = class(TFormLKDataComponent)
    Panel5: TPanel;
    lProgress: TLabel;
    pbDeleteProgress: TProgressBar;
    aDeleteData: TAction;
    DSShowDetail: TDataSource;
    rgTables: TRadioGroup;
    sbDelete: TToolButton;
    Splitter1: TSplitter;
    GridDelDetail: TJvDBUltimGrid;
    procedure FormDestroy(Sender: TObject);
    procedure aDeleteDataExecute(Sender: TObject);
  private
    { Private declarations }
    FMyDataComponent: TLKDataComponent;
  protected
    dbGridDetail: TLKDcDBGridController;
    procedure CreatedbGridDetail;
    procedure PlaceEditControls; override;
    procedure SetDataSourceDataSets; override;
  public
    { Public declarations }
    procedure Construct(Sender: TLKDataComponent);
    procedure Execute(selection: TLKACTableSelect;
      masterDeleteFilter, detailDeleteFilter: string);
  end;

var
  FormLKDataCompBlockDelete: TFormLKDataCompBlockDelete;

implementation

{$R *.DFM}

procedure TFormLKDataCompBlockDelete.Construct(Sender: TLKDataComponent);
begin
  FMyDataComponent := Sender;
  inherited Construct(Sender);
end;

procedure TFormLKDataCompBlockDelete.SetDataSourceDataSets;
begin
  inherited;
  with FMyDataComponent do
    if (TblDetailUsed) then
      TblDetailDataSource := DSShowDetail;
end;

procedure TFormLKDataCompBlockDelete.CreatedbGridDetail;
begin
  dbGridDetail := TLKDcDBGridController.Create(self);
  dbGridDetail.Grid := GridDelDetail;
  dbGridDetail.ConstructFromDatasource(FMyDataComponent.TblDetail.MasterFields);
end;

procedure TFormLKDataCompBlockDelete.PlaceEditControls;
begin
  inherited;
  if FMyDataComponent.TblDetailUsed then
    CreatedbGridDetail
  else
    with rgTables do
    begin
      itemIndex := 1 { master table};
      visible := false;
    end;
end;

procedure TFormLKDataCompBlockDelete.Execute(selection: TLKACTableSelect;
  masterDeleteFilter, detailDeleteFilter: string);
begin
  // Construct(Sender);
  with rgTables do
    case selection of
      tbsBoth: itemIndex := 0;
      tbsMaster: itemIndex := 1;
      tbsDetail: itemIndex := 2;
    end;
  ShowForm(True, [mbOK]);
end;

procedure TFormLKDataCompBlockDelete.FormDestroy(Sender: TObject);
begin
  inherited;
  dbGridDetail.Free;
end;

procedure TFormLKDataCompBlockDelete.aDeleteDataExecute(Sender: TObject);
var
  totalMasterRecs, thisMasterRec: integer;
begin
  inherited;
  if (messageDlg('Do you really want to delete all the data shown???',
    mtConfirmation, [mbYes, mbCancel], 0)
    = mrCancel) then
    exit;
  screen.cursor := crHourGlass;
  thisMasterRec := 0;
  lProgress.Caption := 'Counting records';
  application.ProcessMessages;
  with FMyDataComponent do
  begin
    // if TblDetailUsed then TblDetail.disableControls;
    try
      TblMasterDataSource := nil;
      if (TblDetailUsed) then
        TblDetailDataSource := nil;
      totalMasterRecs := TblMaster.RecordCount;
      // with TblMaster do if findFirst then
      //   repeat inc(totalMasterRecs) until not findNext;
      lProgress.Caption := 'Deleteing records';
      case rgTables.itemIndex of
        0: while TblMaster.FindLast do
          begin // delete only detail table records
            inc(thismasterRec);
            pbDeleteProgress.Position := trunc(thisMasterRec * 100 /
              totalMasterRecs);
            application.ProcessMessages;
            try
              with TblDetail do
                while FindLast do
                  delete;
            except;
            end;
            TblMaster.Delete;
          end;
        1: with TblMaster do
          begin
            //       disableControls; //disconnect to avoid deletion of detail table
            //       try
            while findFirst do
            begin
              inc(thismasterRec);
              pbDeleteProgress.Position := trunc(thisMasterRec * 100 /
                totalMasterRecs);
              application.ProcessMessages;
              delete;
            end;
            //       finally enableControls; end;
          end;
        2: if TblMaster.findFirst then
            repeat // delete only detail table records
              inc(thismasterRec);
              pbDeleteProgress.Position := trunc(thisMasterRec * 100 /
                totalMasterRecs);
              application.ProcessMessages;
              with TblDetail do
                while findFirst do
                  delete;
            until not TblMaster.findNext;
      end;
    finally
      with FMyDataComponent do
      begin
        TblMasterDataSource := DSShowMaster;
        if (TblDetailUsed) then
          TblDetailDataSource := DSShowDetail;
      end;
      pbDeleteProgress.Position := 0;
      screen.cursor := crDefault;
      // TblDetail.enableControls;
      lProgress.Caption := 'Delete completed';
    end;
  end;
end;


end.

