{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for editing key fields in a master detail data heirachy           }
{                                                                             }
{*****************************************************************************}

unit LKDataCompChangeKeyFieldValueDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, DB, DBTables, LKRecordsComponents, dialogs, Variants,
  Grids;

type
  TDlgLKChangeKeyFieldValue = class(TForm)
    OKBtn: TButton;
    CancelBtn: TButton;
    StringGrid1: TStringGrid;
    Label5: TLabel;
    Memo1: TMemo;
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure StringGrid1KeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    MyDocuments: TLKDataComponent;
  public
    { Public declarations }
    ChangeFieldNames: TStringList;
    FieldDataTypes: array[0..500] of TFieldType;
    FieldOldValues: array[0..500] of variant;
    FieldNewValues: array[0..500] of variant;
    function Execute(ADocuments: TLKDataComponent): TModalResult;
  end;

var
  DlgLKChangeKeyFieldValue: TDlgLKChangeKeyFieldValue;

implementation

uses LKUtils, LKDefine;

{$R *.DFM}

function TDlgLKChangeKeyFieldValue.Execute(ADocuments: TLKDataComponent):
  TModalResult;
var
  i, j: integer;
begin
  MyDocuments := ADocuments;
  with myDocuments do // save master table info then disconnect
  begin
    caption := 'Change Index Field value ' + NameFriendly;
    with StringGrid1 do
    begin
      Cells[0, 0] := 'Field name';
      Cells[1, 0] := 'Old value';
      Cells[2, 0] := 'New value';
      with TblMaster do
      begin
        i := 0;
        for j := 0 to IndexFieldCount - 1 do
        begin
          if (i > 0) then
            RowCount := RowCount + 1;
          Cells[0, i + 1] := IndexFields[i].FieldName;
          Cells[1, i + 1] := FieldByName(Cells[0, i + 1]).asString;
          Cells[2, i + 1] := Cells[1, i + 1];
          ChangeFieldNames.Add(Cells[0, i + 1]);
          FieldDataTypes[i] := IndexFields[i].DataType;
          FieldOldValues[i] := FieldByName(Cells[0, i + 1]).Value;
          inc(i);
        end;
        if (IndexFieldCount = 0) then
        begin
          Showmessage('There is no index on this table');
          result := mrCancel;
        end
        else
          result := ShowModal;
      end;
    end;
  end;
end;

procedure TDlgLKChangeKeyFieldValue.FormKeyPress(Sender: TObject;
  var Key: Char);
begin
  MoveMyFocus(self, key);
end;

procedure TDlgLKChangeKeyFieldValue.FormCreate(Sender: TObject);
begin
  ChangeFieldNames := TStringList.Create;
end;

procedure TDlgLKChangeKeyFieldValue.FormDestroy(Sender: TObject);
begin
  ChangeFieldNames.Free;
end;

procedure TDlgLKChangeKeyFieldValue.OKBtnClick(Sender: TObject);
var
  i: integer;
  mr: TModalResult;
begin
  mr := mrCancel;
  for i := 0 to ChangeFieldNames.Count - 1 do
  begin
    case FieldDataTypes[i] of
      ftAutoInc: FieldNewValues[i] := null;
      ftTime: FieldNewValues[i] := StrToTime(StringGrid1.Cells[2, i + 1]);
      ftDate,
        ftDateTime: FieldNewValues[i] := StrToDate(StringGrid1.Cells[2, i + 1]);
      ftFloat: FieldNewValues[i] := StrToFloat(StringGrid1.Cells[2, i + 1]);
      ftCurrency: FieldNewValues[i] := StrToCurr(StringGrid1.Cells[2, i + 1]);
      ftInteger,
        ftSmallint,
        ftLargeInt: FieldNewValues[i] := StrToInt(StringGrid1.Cells[2, i + 1]);
      ftBoolean: FieldNewValues[i] := boolean(uppercase(StringGrid1.Cells[2, i +
        1]) = 'TRUE');
    else
      FieldNewValues[i] := StringGrid1.Cells[2, i + 1];
    end;
    // check that a field value has been changed
    if (FieldNewValues[i] <> FieldOldValues[i]) then
      mr := mrOK;
  end;
  ModalResult := mr;
end;

procedure TDlgLKChangeKeyFieldValue.StringGrid1KeyPress(Sender: TObject;
  var Key: Char);
begin
  OKBtn.Enabled := true;
end;

end.

