{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for the display of TLKDataComponent with common controls to       }
{       display both records as a list and an individual record               }
{                                                                             }
{*****************************************************************************}

unit LKDataCompCommonDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, System.UITypes, System.Actions,
  Graphics, Controls, Forms, Dialogs,
  Grids, DBGrids, StdCtrls, Mask, DBCtrls, ExtCtrls, Menus, Variants,
  Buttons, ComCtrls, ToolWin, LKdlgs, ActnList, ImgList, DB, Contnrs,
  LKDefine, LKRecordsComponents, LKDataCompBaseDlg, StdActns;

type

  TFormLKDataComponentCommon = class(TFormLKDataComponentBase)
    ActionTotals: TAction;
    MainMenuAC: TMainMenu;
    miActivities: TMenuItem;
    miClose: TMenuItem;
    miCopy: TMenuItem;
    miCut: TMenuItem;
    miEdit: TMenuItem;
    miFile: TMenuItem;
    miFind: TMenuItem;
    miHelp: TMenuItem;
    miHelpIndex: TMenuItem;
    miPaste: TMenuItem;
    miPrint: TMenuItem;
    miTools: TMenuItem;
    miTotals: TMenuItem;
    miView: TMenuItem;
    miFileBreak2: TMenuItem;
    pModalButtons: TPanel;
    BlockDelete1: TMenuItem;
    miImport: TMenuItem;
    miExport: TMenuItem;
    miFileBreak1: TMenuItem;
    ChangeMasterIndex1: TMenuItem;
    myStatusBar: TStatusBar;
    miHelpBreak1: TMenuItem;
    btnOK: TButton;
    btnCancel: TButton;
    btnYes: TButton;
    btnNo: TButton;
    btnAbort: TButton;
    btnRetry: TButton;
    Drilldown1: TMenuItem;
    ActionSaveColumnSettings: TAction;
    miSettings: TMenuItem;
    SaveGridsettings1: TMenuItem;
    miExit: TMenuItem;
    aChangeIndexFieldValue1: TMenuItem;
    Window1: TMenuItem;
    StayonTop3: TMenuItem;
    ActionClearColumnSettings: TAction;
    miClipbreak1: TMenuItem;
    Undolastchange1: TMenuItem;
    miViewBreak2: TMenuItem;
    Refreshfromdatabase1: TMenuItem;
    Viewalldetailrecords1: TMenuItem;
    ToolBar1: TToolBar;
    ActionCalendar: TAction;
    ActionButtonOK: TAction;
    miHelpWindow: TMenuItem;
    SaveColumnsettings1: TMenuItem;
    ClearColumnsettings1: TMenuItem;
    aDeleteRecord1: TMenuItem;
    ToolbarStandard: TControlBar;
    ToolBar4: TToolBar;
    ToolBar5: TToolBar;
    ToolBar2: TToolBar;
    ToolBar3: TToolBar;
    SBClose: TToolButton;
    sbDrillDown: TToolButton;
    sbFetchAll: TToolButton;
    ToolButton3: TToolButton;
    sbFind: TToolButton;
    sbSum: TToolButton;
    sbIndexM: TToolButton;
    sbCut: TToolButton;
    sbCopy: TToolButton;
    sbPaste: TToolButton;
    sbUndo: TToolButton;
    sbPrint: TToolButton;
    DBNavigator1: TDBNavigator;
    FilterPanel: TLKFilterPanel;
    sbCalendar: TToolButton;
    procedure FormShow(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure ActionTotalsExecute(Sender: TObject);
    procedure ActionSaveColumnSettingsExecute(Sender: TObject);
    procedure miExitClick(Sender: TObject);
    procedure ActionClearColumnSettingsExecute(Sender: TObject);
    procedure ActionCalendarExecute(Sender: TObject);
    procedure ActionButtonOKExecute(Sender: TObject);
  private
    FMyDataComponent: TLKDataComponent;
  protected
  public
    procedure Construct(Sender: TLKDataComponent);
    function ShowForm(const ShowAsModal: Boolean; Buttons: TMsgDlgButtons): TModalResult; override;
  end;

implementation

uses LKUtils;

{$R *.DFM}

{
************************** TFormAccountsComponentBase **************************
}


procedure TFormLKDataComponentCommon.Construct(Sender: TLKDataComponent);
var X, I: Integer;
  NewAction: TAction;
  NewMenuitem: TMenuItem;
  DC: TLKDataComponent;
begin
  FMyDataComponent := Sender;
  inherited Construct(Sender);
  SetFolderStoreDocumentImages(FMyDataComponent.FolderStoreDocumentImages);
  FilterPanel.DataComponent := FMyDataComponent;
  with FMyDataComponent do
  begin
    ActionTotals.Enabled := (TotalMasterFieldNames.Count + TotalDetailFieldNames.Count > 0);
    for I := 0 to ComponentCount - 1 do if Components[I] is TLKContainedRecords then
      begin
        DC := TLKContainedRecords(Components[I]);
        if DC.TreeNodeVisible then
        begin
          NewAction := TAction.Create(Self);
          with NewAction do
          begin
            ActionList := ActionListViewChildAC;
            Caption := '&' + DC.NameFriendly;
            Hint := 'View ' + DC.NameFriendly;
            ImageIndex := DC.ImageIndex;
            Tag := I;
            OnExecute := ActionChildComponentView.OnExecute;
          end;
          NewMenuitem := TMenuItem.create(miView);
          NewMenuitem.Action := NewAction;
          miView.Add(NewMenuitem);
          with ToolBar1.Buttons[ToolBar1.ButtonCount - 1] do X := Left + Width + 1;
          with TToolButton.Create(Self) do //created button setup
          begin
            Left := X;
            Parent := Toolbar1;
            Action := NewAction;
          end;
        end;
      end;
  end;
end;

procedure TFormLKDataComponentCommon.FormShow(Sender: TObject);
begin
  inherited;
  if Assigned(ToolBar5) then
    FilterPanel.FilterReload;
  if (ToolBar1.ButtonCount > 1) then    
    ToolBar1.Visible := True;
  if (ToolBar1.Visible) then ToolBar1.Top := 36;
  FormResize(Sender);
end;

function TFormLKDataComponentCommon.ShowForm(const ShowAsModal: Boolean;
  Buttons: TMsgDlgButtons): TModalResult;
begin
  pModalButtons.Visible := ShowAsModal;
  if ShowAsModal then
  begin
    SBClose.Visible := False;
    btnOK.Visible := (mbOK in Buttons);
    btnCancel.Visible := (mbCancel in Buttons);
    btnYes.Visible := (mbYes in Buttons);
    btnNo.Visible := (mbNo in Buttons);
    btnAbort.Visible := (mbAbort in Buttons);
    btnRetry.Visible := (mbRetry in Buttons);
    if not btnNo.Visible then btnNo.Visible := (mbIgnore in Buttons);
    if not btnYes.Visible then btnYes.Visible := (mbAll in Buttons);
    if not btnNo.Visible then btnNo.Visible := (mbNoToAll in Buttons);
    if not btnYes.Visible then btnYes.Visible := (mbYesToAll in Buttons);
  end
  else ToolBar1.Visible := True;
  Result := inherited ShowForm(ShowAsModal, Buttons);
end;


procedure TFormLKDataComponentCommon.FormResize(Sender: TObject);
var VisibleBtnList: TObjectList;
  I, L: integer;
begin
  if pModalbuttons.visible then
  begin
    VisibleBtnList := TObjectList.Create(False);
    try
      if btnOK.Visible then VisibleBtnList.Add(btnOK);
      if btnCancel.Visible then VisibleBtnList.Add(btnCancel);
      if btnYes.Visible then VisibleBtnList.Add(btnYes);
      if btnNo.Visible then VisibleBtnList.Add(btnNo);
      if btnAbort.Visible then VisibleBtnList.Add(btnAbort);
      if btnRetry.Visible then VisibleBtnList.Add(btnRetry);
      with VisibleBtnList do
      begin
        I := Count * 83; // 75 + 8 space
        L := pmodalbuttons.Width - I;
        L := Trunc(L / 2) - 4;
        for I := 0 to Count - 1 do
        begin
          TButton(items[I]).Left := L;
          L := L + 83;
        end;
      end;
    finally VisibleBtnList.Free; end;
  end;
end;

procedure TFormLKDataComponentCommon.ActionTotalsExecute(Sender: TObject);
begin
  with FMyDataComponent do DlgViewTotalsExecute(self, nameFriendly + ' totals', [mbOK]);
end;

procedure TFormLKDataComponentCommon.ActionSaveColumnSettingsExecute(
  Sender: TObject);
begin
// override here
  ShowMessage('Grid column width and position settings have been saved');
end;

procedure TFormLKDataComponentCommon.miExitClick(Sender: TObject);
begin
  if (MessageDlg('Confirm that you want to close this program?', mtWarning,
    [mbOK, mbCancel], 0) = mrOK) then Application.Terminate;
end;

procedure TFormLKDataComponentCommon.ActionClearColumnSettingsExecute(
  Sender: TObject);
begin
 // override in decendants
  ShowMessage('aClearColumnSettings Not implemented');
end;

procedure TFormLKDataComponentCommon.ActionCalendarExecute(
  Sender: TObject);
begin
  FMyDataComponent.AccountsHub.Diary.View(null, false, []);
end;

procedure TFormLKDataComponentCommon.ActionButtonOKExecute(Sender: TObject);
begin
  inherited;
//
end;

end.

