{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for setting fields for TLKDataComponent CSV import / export       }
{                                                                             }
{*****************************************************************************}

unit LKDataCompImpExpSetDlg;

interface

uses Windows, SysUtils, Classes, System.UITypes,
  Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, IniFiles, dialogs, db, LKDefine, LKdlgs,
  LKRecordsComponents, ComCtrls;

type
  TDlgLKDataComponentImportExport = class(TForm)
    SelectFolderDlg: TOpenDialog;
    PageControl1: TPageControl;
    stSettings: TTabSheet;
    tsFields: TTabSheet;
    lTargetFolder: TLabel;
    eImpExpFolder: TEdit;
    BtnBrowse: TButton;
    Label8: TLabel;
    lDetailCSV: TLabel;
    eMasterCSV: TEdit;
    eDetailCSV: TEdit;
    ckbxFixedField: TCheckBox;
    Label1: TLabel;
    cbFieldSep: TComboBox;
    Label2: TLabel;
    cbTextID: TComboBox;
    Label5: TLabel;
    cbDateSep: TComboBox;
    Label4: TLabel;
    cbDateID: TComboBox;
    Label7: TLabel;
    cbFillNo: TComboBox;
    Label6: TLabel;
    cbFillText: TComboBox;
    gbMaster: TGroupBox;
    btnMasterclear: TCheckBox;
    gbDetail: TGroupBox;
    btnDetailclear: TCheckBox;
    Panel1: TPanel;
    ckbxRemember: TCheckBox;
    btnImpExp: TBitBtn;
    BtnApply: TButton;
    BtnCancel: TButton;
    pFilters: TPanel;
    cbImpExpMaster: TCheckBox;
    cbImpExpDetail: TCheckBox;
    LKFilterPanel1: TLKFilterPanel;
    LKFilterPanel2: TLKFilterPanel;
    procedure BtnBrowseClick(Sender: TObject);
    procedure BtnApplyClick(Sender: TObject);
    procedure eImpExpFolderChange(Sender: TObject);
    procedure btnImpExpClick(Sender: TObject);
    procedure ckbxRememberClick(Sender: TObject);
    procedure btnMasterclearClick(Sender: TObject);
  private
    { Private declarations }
    ckbxM, ckbxD: array[0..200] of TCheckBox;
    myRecord: TBookmark;

  public
    { Public declarations }
    ImpExpDataComponent: TLKDataComponent;
    procedure Execute(Sender: TLKDataComponent; AsExport, AutoExecute:
      Boolean);
  end;

var
  DlgLKDataComponentImportExport: TDlgLKDataComponentImportExport;

implementation

uses LKutils;

{$R *.DFM}

procedure TDlgLKDataComponentImportExport.Execute(Sender: TLKDataComponent;
  AsExport, Autoexecute: Boolean);
var
  I, hm, hd, hg: Integer;
  S: string;
begin
  ImpExpDataComponent := sender;
  LKFilterPanel1.DataComponent := ImpExpDataComponent;
  LKFilterPanel2.DataComponent := ImpExpDataComponent;
  pFilters.Visible := asExport;
  if asExport then
  begin
    Caption := 'Export - ';
    lTargetFolder.Caption := 'Export folder';
    btnImpExp.caption := '&Export';
  end
  else
  begin
    Caption := 'Import - ';
    lTargetFolder.Caption := 'Import folder';
    btnImpExp.caption := '&Import';
  end;
  with ImpExpDataComponent do
  begin
    Caption := Caption + NameFriendly;
    S := InstallFolder;
    eImpExpFolder.Text := RecallString('ImpExp', 'ImportExportFolder', S, True);
    eMasterCSV.Text := RecallString('ImpExp', 'MasterFileName', TblMaster.TableName
      + '.csv', true);
    eDetailCSV.Text := RecallString('ImpExp', 'DetailFileName', TblDetail.TableName
      + '.csv', true);
    if not TblDetailUsed then
    begin
      eDetailCSV.hide;
      lDetailCSV.hide;
    end;
    ASCIIFieldSeparator := RecallString('ImpExp', 'ASCIIFieldSeparator', ',',
      true);
    ASCIIStringIdentifier := RecallString('ImpExp', 'ASCIIStringIdentifier',
      '"', true);
    ASCIIDateIdentifier := RecallString('ImpExp', 'ASCIIDateIdentifier', '"',
      true);
    ASCIIDateSeparator := RecallString('ImpExp', 'ASCIIDateSeparator', '/',
      true);
    ASCIIFillCharString := RecallString('ImpExp', 'ASCIIFillCharString', ' ',
      true);
    ASCIIFillCharNumber := RecallString('ImpExp', 'ASCIIFillCharNumber', '0',
      true);
    ASCIIFixedFieldSize := RecallBool('ImpExp', 'ASCIIFixedFieldSize', false,
      true);
    cbImpExpMaster.Checked := RecallBool('ImpExp', 'ImpExpMaster', true, true);
    cbImpExpDetail.Checked := RecallBool('ImpExp', 'ImpExpDetail', true, true);
    cbFieldSep.Text := ASCIIFieldSeparator;
    cbTextID.Text := ASCIIStringIdentifier;
    cbDateID.Text := ASCIIDateIdentifier;
    cbDateSep.Text := ASCIIDateSeparator;
    cbFillText.Text := ASCIIFillCharString;
    cbFillNo.Text := ASCIIFillCharNumber;
    ckbxFixedField.checked := ASCIIFixedFieldSize;
    hm := 0;
    hd := 0;
    with TblMaster do
    begin
      myRecord := getBookmark;
      fieldDefs.update;
      for I := 0 to FieldCount - 1 do
        with Fields[I] do
        begin
          inc(hm);
          ckbxM[I] := TCheckbox.create(gbMaster);
          with ckbxM[I] do
          begin
            Parent := gbMaster;
            OnClick := eImpExpFolderChange;
            checked := true;
            caption := DisplayName;
            height := 16;
            Width := 130;
            top := 16 * hm;
            left := 8;
          end;
        end;
    end;
    if TblDetailUsed then
      with TblDetail do
      begin
        FieldDefs.update;
        for I := 0 to FieldCount - 1 do
          with fields[I] do
          begin
            inc(hd);
            ckbxD[I] := TCheckbox.create(gbDetail);
            with ckbxD[I] do
            begin
              parent := gbDetail;
              OnClick := eImpExpFolderChange;
              checked := true;
              caption := DisplayName;
              height := 16;
              Width := 130;
              top := 16 * hd;
              left := 8;
            end;
          end;
      end
    else
      LKFilterPanel2.Visible := false;
    for I := 0 to TblMaster.FieldCount - 1 do
      ckbxM[I].checked := (RecallString('ImpExpFields', 'Master' + intToStr(I),
        'Yes', true) = 'Yes');
    for I := 0 to TblDetail.FieldCount - 1 do
      ckbxD[I].checked := (RecallString('ImpExpFields', 'Detail' + intToStr(I),
        'Yes', true) = 'Yes');
  end;
  hg := 12;
  if hg < hm then
    hg := hm;
  if hg < hd then
    hg := hd;
  height := 105 + hg * 16;
  gbMaster.height := 25 + hm * 16;
  gbDetail.height := 25 + hd * 16;
  if (height < 375) then
    height := 375;
  btnApply.Enabled := false;
  ckbxRemember.enabled := false;
  if autoexecute then
  begin
    show;
    btnImpExpClick(self);
  end
  else
    showModal;
end;

procedure TDlgLKDataComponentImportExport.BtnApplyClick(Sender: TObject);
begin
  btnApply.Enabled := false;
  if (eImpExpFolder.text = '') then
    raise LKError.create(lTargetFolder.Caption + ' must be assigned.');
  if not directoryexists(eImpExpFolder.text) then
    //warn if dir specified doesn't exist
  begin
    if (messageDlg('Folder does not exist!' + chr(10) + 'Create directory ' +
      eImpExpFolder.text + ' now?', mtWarning, [mbYes, mbNo], 0) <> mrYes) then
      Showmessage('Folder name invalid')
    else
      forceDirectories(eImpExpFolder.text);
  end;
  with ImpExpDataComponent do
  begin
    ASCIIFieldSeparator := cbFieldSep.Text;
    ASCIIStringIdentifier := cbTextID.Text;
    ASCIIDateIdentifier := cbDateID.Text;
    ASCIIDateSeparator := cbDateSep.Text;
    ASCIIFillCharString := cbFillText.Text;
    ASCIIFillCharNumber := cbFillNo.Text;
    ASCIIFixedFieldSize := ckbxFixedField.checked;
  end;
  ckbxRememberClick(self); //remember if checked
  btnImpExp.Enabled := true;
  //btnCancel.Enabled := false;
end;

procedure TDlgLKDataComponentImportExport.BtnBrowseClick(Sender: TObject);
var
  fn: string;
begin
  if not SelectFolderDlg.Execute then
    exit;
  btnApply.Enabled := true;
  ckbxRemember.enabled := true;
  fn := ExtractFilePath(SelectFolderDlg.FileName);
  Delete(fn, length(fn), 1); //strip final '/'
  eImpExpFolder.text := fn;
  eMasterCSV.Text := ExtractFileName(SelectFolderDlg.FileName);
end;

procedure TDlgLKDataComponentImportExport.eImpExpFolderChange(
  Sender: TObject);
begin
  btnApply.Enabled := true;
  ckbxRemember.Enabled := true;
end;

procedure TDlgLKDataComponentImportExport.btnImpExpClick(Sender: TObject);
var
  mfn, dfn: string;
  success: boolean;
  mf, df: TLKFieldNumbers;
  i: integer;
begin
  btnImpExp.Enabled := false;
  if cbImpExpMaster.Checked then
    mfn := eMasterCSV.Text
  else
    mfn := '';
  if (mfn > '') then
    mfn := eImpExpFolder.Text + '\' + mfn;
  if cbImpExpDetail.Checked then
    dfn := eDetailCSV.Text
  else
    dfn := '';
  if (dfn > '') then
    dfn := eImpExpFolder.Text + '\' + dfn;
  mf := [];
  df := [];
  with ImpExpDataComponent do
  begin
    for i := 0 to TblMaster.FieldCount - 1 do
      if ckbxM[i].checked then
        mf := mf + [i];
    if TblDetailUsed then
      for i := 0 to TblDetail.FieldCount - 1 do
        if ckbxD[i].checked then
          df := df + [i];
    if btnImpExp.caption = '&Export' then
      success := ExportData(mfn, dfn, mf, df)
    else
      success := ImportData(mfn, dfn, mf, df);
  end;
  if success then
    Close
  else
    Showmessage('An error occurred');
end;

procedure TDlgLKDataComponentImportExport.ckbxRememberClick(Sender: TObject);
var
  x: integer;
begin
  if not ckbxRemember.checked then
    Exit;
  with ImpExpDataComponent do
  begin
    RememberString('ImpExp', 'ImportExportFolder', eImpExpFolder.Text, true);
    RememberString('ImpExp', 'MasterFileName', eMasterCSV.Text, true);
    RememberString('ImpExp', 'DetailFileName', eDetailCSV.Text, true);
    RememberString('ImpExp', 'ASCIIFieldSeparator', cbFieldSep.Text, true);
    RememberString('ImpExp', 'ASCIIStringIdentifier', cbTextID.Text, true);
    RememberString('ImpExp', 'ASCIIDateIdentifier', cbDateID.Text, true);
    RememberString('ImpExp', 'ASCIIDateSeparator', cbDateSep.Text, true);
    RememberString('ImpExp', 'ASCIIFillCharString', cbFillText.Text, true);
    RememberString('ImpExp', 'ASCIIFillCharNumber', cbFillNo.Text, true);
    RememberBool('ImpExp', 'ASCIIFixedFieldSize', ckbxFixedField.Checked, true);
    RememberBool('ImpExp', 'ImpExpMaster', cbImpExpMaster.Checked, true);
    RememberBool('ImpExp', 'ImpExpDetail', cbImpExpDetail.Checked, true);
    Forget('ImpExpFields', true);
    for x := 0 to TblMaster.FieldCount - 1 do
      RememberString('ImpExpFields', 'Master' + intToStr(x),
        booleanToYesNo(ckbxM[x].checked), true);
    for x := 0 to TblDetail.FieldCount - 1 do
      RememberString('ImpExpFields', 'Detail' + intToStr(x),
        booleanToYesNo(ckbxD[x].checked), true);
  end;
end;

procedure TDlgLKDataComponentImportExport.btnMasterclearClick(
  Sender: TObject);
var
  i: integer;
  mainCkbx: TCheckBox;

begin //check/uncheck all checkboxes in groupbox
  mainCkbx := (sender as TCheckBox);
  if mainCkbx.checked then //change hints for clear/check buttons
    hint := 'Check all boxes'
  else
    hint := 'Uncheck all boxes';
  with mainCkbx.Parent do //find groupbox belonging to clicked checkbox
    for i := 1 to ControlCount - 1 do //loop through controls in groupbox
      if (Controls[i] is TCheckBox) then //to find all checkboxes
        TCheckBox(Controls[i]).Checked := mainCkbx.Checked;
end;

end.

