{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for the selecting a record in a TLKDataComponent                  }
{                                                                             }
{*****************************************************************************}

unit LKDataCompSelectRecordWiz;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, Grids, DBGrids, Db, dialogs, Menus, DBClient,
  LKRecordsComponents, LKUtils, LKUtilComponents, LKFormBaseDlg, ImgList,
  ActnList, StdActns, JvExDBGrids, JvDBGrid, JvDBUltimGrid, LKDBControls;

type
  TLKDataCompSelectRecordDlg = class(TFormLKBase)
    ACMasterDS: TDataSource;
    PopupMenu1: TPopupMenu;
    SaveColumnsettings1: TMenuItem;
    Panel1: TPanel;
    BtnBack: TButton;
    BtnNext: TButton;
    BtnCancel: TButton;
    Panel2: TPanel;
    llocate: TLabel;
    eLocate: TEdit;
    GridSelectMaster: TJvDBUltimGrid;
    dbGridMaster: TLKDcDBGridController;
    procedure eLocateChange(Sender: TObject);
    procedure SaveColumnsettings1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    locateChar: word;
    locateFieldName: string;
    FMyDataComponent: TLKDataComponent;
  public
    { Public declarations }
    property MyDataComponent: TLKDataComponent read FMyDataComponent;
    procedure Construct(const AComponent: TLKDataComponent);
  end;

var
  LKDataCompSelectRecordDlg: TLKDataCompSelectRecordDlg;

implementation

{$R *.DFM}

procedure TLKDataCompSelectRecordDlg.Construct(const AComponent:
  TLKDataComponent);
var
  index1Names: string;
  i: integer;
  myField: TField;
begin
  locateChar := 32;
  FMyDataComponent := AComponent;
  with FMyDataComponent do
  begin
    if not Active then
      Open;
    index1Names := TblMaster.CurrentIndexFieldNames;
    i := pos(';', index1Names);
    if (i > 0) then
      System.Delete(index1Names, i, 1 + length(index1Names) - i);
    myField := TblMaster.FindField(index1Names);
    if assigned(myField) then
    begin
      locateFieldName := myField.FieldName;
      lLocate.Caption := '&' + myField.DisplayName;
      locateChar := word(myField.DisplayName[1]);
    end;
    TblMasterDataSource := ACMasterDS;
  end;
  with dbGridMaster do
  begin
    ConstructFromDatasource('');
    try
      LoadFormat(MyDataComponent, 'SelectDlg', 'GridColumns');
    except ClearFormat(MyDataComponent, 'SelectDlg', 'GridColumns');
    end;
    ReadOnly := false;
  end;
end;

procedure TLKDataCompSelectRecordDlg.eLocateChange(Sender: TObject);
begin
  inherited;
  if (eLocate.Text = '') then
    Exit;
  ACMasterDS.Dataset.Locate(locateFieldName, eLocate.Text, [loCaseInsensitive,
    loPartialKey]);
end;

procedure TLKDataCompSelectRecordDlg.FormShow(Sender: TObject);
begin
  inherited;
  Height := Screen.Height - 250;
end;

procedure TLKDataCompSelectRecordDlg.SaveColumnsettings1Click(
  Sender: TObject);
begin
  dbGridMaster.SaveFormat(FMyDataComponent, 'SelectDlg', 'GridColumns');
end;

end.

