{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for showing the total of SumMDPairFieldNames of a TLKDataComponent}
{                                                                             }
{*****************************************************************************}

unit LKDataCompTotalsDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, DB, dialogs, Contnrs,
  LKRecordsComponents;

const
  ATotalPanelHeight = 24;

type
  TLKDCTotalPanel = class(TPanel)
  private
    FTotal: Double;
  protected
    procedure Resize; override;
  public
    LabelTotal: TLabel;
    constructor Create(AOwner: TComponent); override;
    procedure ShowTotal;
    property Total: Double read FTotal write FTotal;
  end;

  TDlgLKDataCompTotals = class(TForm)
    btnOK: TButton;
    gbDetail: TGroupBox;
    gbMaster: TGroupBox;
    Panel1: TPanel;
    Splitter1: TSplitter;
    procedure FormResize(Sender: TObject);
  public
    function Construct(DataComponent: TLKDataComponent): Boolean;
  end;

var
  DlgLKDataCompTotals: TDlgLKDataCompTotals;

implementation

uses LKUtils, LKUtilsDB;

{$R *.DFM}

{
******************************* TLKDCTotalPanel *******************************
}

constructor TLKDCTotalPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Height := ATotalPanelHeight;
  Align := alTop;
  Alignment := taLeftJustify;
  ParentFont := False;
  LabelTotal := TLabel.Create(AOwner);
  with LabelTotal do
  begin
    Top := 6;
    Font.Style := [fsBold];
    Parent := Self;
  end;
  Total := 0;
  Resize;
end;

procedure TLKDCTotalPanel.Resize;
begin
  inherited Resize;
  LabelTotal.Left := Trunc(Width * 0.6);
end;

procedure TLKDCTotalPanel.ShowTotal;
begin
  LabelTotal.Caption := FloatToStrF(Total, ffFixed, 19, 2);
end;

{
******************************* TLKDCTotalsDlg ********************************
}

function TDlgLKDataCompTotals.Construct(DataComponent: TLKDataComponent):
  Boolean;
var
  myBookMark: TBookmark;
  i: Integer;
begin
  Result := True;
  if (DataComponent.TotalMasterFieldNames.Count = 0) and
    (DataComponent.TotalDetailFieldNames.Count = 0) then
  begin
    Showmessage('There are no total fields identified for '
      + DataComponent.NameFriendly);
    Result := False;
    Exit;
  end;
  Screen.Cursor := crHourglass;
  try
    Caption := 'Totals for ' + DataComponent.NameFriendly;
    with DataComponent do
    begin
      i := TotalMasterFieldnames.Count;
      if (TotalDetailFieldnames.Count > i) then
        i := TotalDetailFieldnames.Count;
      Self.Height := 104 + i * ATotalPanelHeight;
      for i := 0 to (TotalMasterFieldnames.Count - 1) do
      begin
        with TLKDCTotalPanel.Create(Self) do
        begin
          Parent := gbMaster;
          Caption := TotalMasterFieldnames[i];
        end;
      end;
      for i := 0 to (TotalDetailFieldnames.Count - 1) do
      begin
        with TLKDCTotalPanel.Create(Self) do
        begin
          Parent := gbDetail;
          Caption := TotalDetailFieldnames[i];
        end;
      end;
      if TblDetailUsed then
        with TblDetail do
        begin
          if assigned(TblDetail.Clone) then
            gbDetail.Caption := TblDetail.Clone.TableName
          else
            gbDetail.Caption := TblDetail.TableName;
          if FindFirst then
            with gbDetail do
              repeat
                for i := 0 to (ControlCount - 1) do
                  with TLKDCTotalPanel(Controls[i]) do
                    Total := Total + GetFloat(FieldByName(Caption));
              until not FindNext;
          with gbDetail do
            for i := 0 to (ControlCount - 1) do
              TLKDCTotalPanel(Controls[i]).ShowTotal;
        end
      else
      begin
        FreeAndNil(gbDetail);
        Splitter1.Free;
        Width := Trunc(Width / 2);
        gbMaster.Align := alClient;
      end;
      gbMaster.Caption := TblMaster.TableName;
      with TblMaster do
      begin
        myBookmark := GetBookmark;
        DisableControls;
        try
          if FindFirst then
            with gbMaster do
              repeat
                for i := 0 to (ControlCount - 1) do
                  with TLKDCTotalPanel(Controls[i]) do
                    Total := Total + GetFloat(FieldByName(Caption));
              until not FindNext;
        finally
          if Assigned(myBookmark) then
          begin
            if BookmarkValid(myBookmark) then
              GotoBookmark(myBookmark);
            FreeBookmark(myBookmark);
          end;
          EnableControls;
        end;
      end;
    end;
    with gbMaster do
      for i := 0 to (ControlCount - 1) do
        TLKDCTotalPanel(Controls[i]).ShowTotal;
  finally Screen.cursor := crDefault;
  end;
end;

procedure TDlgLKDataCompTotals.FormResize(Sender: TObject);
begin
  btnOk.Left := Trunc((Width - BtnOK.Width) / 2)
end;

end.

