{*****************************************************************************}
{                                                                             }
{   This file is part of the LKAccounts components Library                    }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for display of a TLKLedgerAnalysisManagementAccounts              }
{                                                                             }
{*****************************************************************************}

unit LKDataComponentDetailDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, System.UITypes,
  Graphics, Controls, Forms,
  Dialogs, Menus, ImgList, ActnList, DB, StdCtrls, DBCtrls, StdActns,
  ExtCtrls, ComCtrls, Buttons, ToolWin, Grids, DBGrids,
  JvExExtCtrls, JvNetscapeSplitter, JvExDBGrids, JvDBGrid, JvDBUltimGrid,
  LKDBControls, LKUtilComponents, LKRecordsComponents, LKDefine,
  LKdlgs, LKDataComponentDlg;

type
  TFormLKDataComponentDetail = class(TFormLKDataComponent)
    PrintManagementAccounts1: TMenuItem;
    miActiviitesBreak1: TMenuItem;
    CalculateManagementAccounts1: TMenuItem;
    Splitter1: TSplitter;
    DSShowDetail: TDataSource;
    PopupMenu2: TPopupMenu;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItem5: TMenuItem;
    MenuItem6: TMenuItem;
    GridDetail: TJvDBUltimGrid;
    dbGridDetail: TLKDcDBGridController;
    ActionDrillDownDetail: TAction;
    ActionDetailDelete: TAction;
    ToolBarDetail: TToolBar;
    DBNavigator2: TDBNavigator;
    FilterPanelDetail: TLKFilterPanel;
    procedure DSShowMasterDataChange(Sender: TObject; Field: TField);
    procedure ActionDrillDownDetailExecute(Sender: TObject);
    procedure ActionDetailDeleteExecute(Sender: TObject);
  private
    { Private declarations }
    FMyDataComponent: TLKDataComponent;
  protected
    HideDetailFields: string;
    procedure CreatedbGridDetail;
    procedure PlaceEditControls; override;
    procedure SetDataSourceDataSets; override;
    procedure SetHideFields; override;
  public
    { Public declarations }
    procedure Construct(Sender: TLKDataComponent);
  end;

var
  FormLKDataComponentDetail: TFormLKDataComponentDetail;

implementation

{$R *.dfm}

procedure TFormLKDataComponentDetail.ActionDetailDeleteExecute(Sender: TObject);
begin
  if (MessageDlg('Confirm: delete this detail record?', mtInformation, [mbOK,
    mbCancel], 0) = mrOK) then
    FMyDataComponent.TblDetail.Delete;
end;

procedure TFormLKDataComponentDetail.ActionDrillDownDetailExecute(
  Sender: TObject);
begin
  FMyDataComponent.DrillDownRecord;
end;

procedure TFormLKDataComponentDetail.Construct(Sender: TLKDataComponent);
begin
  FMyDataComponent := Sender;
  inherited Construct(Sender);
  FilterPanelDetail.DataComponent := FMyDataComponent;
  FilterPanelDetail.FilterDataset := tbDetail;
end;

procedure TFormLKDataComponentDetail.CreatedbGridDetail;
begin
  dbGridDetail.ConstructFromDatasource(HideDetailFields);
  dbGridDetail.ReadOnly := not FMyDataComponent.Editing;
  with GridDetail do
  begin
    OnDblClick := ActionDrillDown.OnExecute;
  end;
end;

procedure TFormLKDataComponentDetail.DSShowMasterDataChange(
  Sender: TObject; Field: TField);
begin
  inherited;
  FMyDataComponent.TblDetail.Last;
end;

procedure TFormLKDataComponentDetail.PlaceEditControls;
begin
  inherited;
  if FMyDataComponent.TblDetailUsed then
  begin
    CreatedbGridDetail;
    try
      ActionClearColumnSettings.Enabled := dbGridDetail.LoadFormat(
        FMyDataComponent, 'FormViewRecord', 'dbGridDetail');
    except ActionClearColumnSettings.Execute;
    end;
  end
  else
  begin
    GridDetail.Visible := False;
    GridMaster.Align := alClient;
  end;
end;

procedure TFormLKDataComponentDetail.SetDataSourceDataSets;
begin
  inherited;
  with FMyDataComponent do
  begin
    if TblDetailUsed then
      TblDetailDataSource := DSShowDetail;
  end;
end;

procedure TFormLKDataComponentDetail.SetHideFields;
begin
  inherited;
  HideDetailFields := FMyDataComponent.TblDetail.MasterFields;
end;

end.

