{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for the display of TLKDataComponent                               }
{                                                                             }
{*****************************************************************************}

unit LKDataComponentDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, System.UITypes,
  Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, ImgList, Menus, ActnList, Db, DBClient,
    Variants,
  DBCtrls, DBGrids, Buttons, ExtCtrls, ToolWin, Grids, Contnrs,
  JvExExtCtrls, JvNetscapeSplitter,
  LKDlgs, LKUtils, LKUtilComponents, LKDefine, LKRecordsComponents,
  LKDataCompCommonDlg, LKDBControls,
  StdActns, JvExDBGrids, JvDBGrid, JvDBUltimGrid;

type
  TFormLKDataComponent = class(TFormLKDataComponentCommon)
    ActionNew: TAction;
    ActionCheckAllAccountTotals: TAction;
    CheckallAccountTotals1: TMenuItem;
    New1: TMenuItem;
    PopupBreak2: TMenuItem;
    New2: TMenuItem;
    miClipbreak2: TMenuItem;
    pMasterDocs: TPanel;
    pLocate: TPanel;
    llocate: TLabel;
    eLocate: TEdit;
    SplitterMDPanel: TJvNetscapeSplitter;
    ActionViewAllRecords: TAction;
    ActionViewMasterPanel: TAction;
    miViewMasterPanel: TMenuItem;
    Panel1Data: TPanel;
    GridMaster: TJvDBUltimGrid;
    dbGridMaster: TLKDcDBGridController;
    ToolButtonNew: TToolButton;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormDestroy(Sender: TObject);
    procedure SplitterMDPanelMoved(Sender: TObject);
    procedure pMasterDocsResize(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormActivate(Sender: TObject);
    procedure eLocateChange(Sender: TObject);
    procedure ActionViewMasterPanelExecute(Sender: TObject);
    procedure ActionViewAllRecordsExecute(Sender: TObject);
    procedure Panel1Exit(Sender: TObject);
    procedure ActionChangeIndexExecute(Sender: TObject);
    procedure ActionNewExecute(Sender: TObject);
    procedure ActionDrillDownExecute(Sender: TObject);
    procedure ActionFindExecute(Sender: TObject);
    procedure ActionSaveColumnSettingsExecute(Sender: TObject);
    procedure ActionCheckAllAccountTotalsExecute(Sender: TObject);
    procedure ActionClearColumnSettingsExecute(Sender: TObject);
    procedure ActionReverseIndexExecute(Sender: TObject);
    procedure GridMasterKeyDown(Sender: TObject; var Key: Word; Shift:
      TShiftState);
  private
    { Private declarations }
    FMyDataComponent: TLKDataComponent;
    FMyMasterComponent: TLKDataComponent;
  protected
    LocateChar: Word;
    LocateDocsDS: TDatasource;
    LocateFieldName: string;
    MasterPanelList: TLKDcMasterPanelList;
    miViewAllDocs: TMenuItem;
    procedure ApplyMasterComponentInfo;
    procedure CreatedbGridMaster; virtual;
    procedure CustomisedbGridMaster; virtual;
    procedure PlaceEditControls; override;
  public
    { Public declarations }
    procedure Construct(Sender: TLKDataComponent);
    procedure SetFolderStoreDocumentImages(const Value: string); override;
    procedure SetKeyFieldsEditing; override;
  end;

var
  FormLKDataComponent: TFormLKDataComponent;

implementation

{$R *.DFM}
uses LKDataComponentRecordDlg;

procedure TFormLKDataComponent.ApplyMasterComponentInfo;
var
  I, J, T, NewTop: Integer;
  MyPanel: TLKDcMasterPanel;
  MyDataComp, MyDataCompMaster: TLKDataComponent;
  NextGrid: TLKDcDBGridController;
  ShortcutList: TStringList;
  MasterComponentList: TComponentList;
  S: string;
  HasMasterComponent: Boolean;
  F: TField;
begin
  with FMyDataComponent do
  begin
    HasMasterComponent := Assigned(MasterComponent);
    pMasterDocs.Visible := HasMasterComponent;
    SplitterMDPanel.Visible := HasMasterComponent;
    ActionViewMasterPanel.Visible := HasMasterComponent;
    ActionViewAllRecords.Visible := HasMasterComponent;
    if (MasterComponent = FMyMasterComponent) then
      Exit;
    with MasterPanelList do
    begin
      for I := 0 to Count - 1 do
        Items[I].Free;
      Clear;
    end;
    MasterComponentList := TComponentList.Create(False);
    NewTop := pLocate.Height + 1;
    try
      MyDataCompMaster := FMyDataComponent.MasterComponent;
      while Assigned(MyDataCompMaster) do
      begin
        MasterComponentList.Add(MyDataCompMaster);
        MyDataComp := MyDataCompMaster;
        MyDataCompMaster := MyDataComp.MasterComponent;
      end;
      for I := MasterComponentList.Count - 1 downto 0 do
      begin
        MyDataCompMaster := TLKDataComponent(MasterComponentList[I]);
        if (I = 0) then
          MyDataComp := FMyDataComponent
        else
          MyDataComp := TLKDataComponent(MasterComponentList[I - 1]);
        //
        MyPanel := TLKDcMasterPanel.Create(Self);
        with MyPanel do
        begin
          Top := NewTop;
          Parent := pMasterDocs;
          Images := ImageListAC;
          OwnerComponent := FMyDataComponent;
          GridComponent := MyDataCompMaster;
          GridTable := tbMaster;
//          Grid.Name := 'ACMGrid' + myAcsCompMaster.TblMaster.TableName;
          Construct;
          NewTop := Splitter.Top + 3;
        end;
        MasterPanelList.Add(MyPanel);
        if (MyDataComp.MasterTable = tbDetail) then
        begin
          MyPanel := TLKDcMasterPanel.Create(Self);
          with MyPanel do
          begin
            Top := NewTop;
            Parent := pMasterDocs;
            Images := ImageListAC;
            OwnerComponent := FMyDataComponent;
            GridComponent := MyDataCompMaster;
            GridTable := tbDetail;        
            Construct;
  //          Grid.Name := 'Grid' + myAcsCompMaster.TblDetail.TableName;
            NewTop := Splitter.Top + 3;
          end;
          MasterPanelList.Add(MyPanel);
        end;
      end; // masterComponentList
    finally MasterComponentList.Free;
    end;
    ShortcutList := TStringList.Create;
    try
      with ShortcutList do
      begin
        Add('O');
        Add('L');
        Add('N');
        Add(' ');
      end;
      // set up the Locate edit
      MyPanel := TLKDcMasterPanel(MasterPanelList[0]);
      with MyPanel do // top panel
      begin
        //  TabOrder := 1;
        LocateDocsDS := GridDatasource;
        with GridComponent.TblMaster do
        begin
          if (IndexFieldNames > '') then
          begin
            S := Trim(Copy(IndexFieldNames,1,Pos(';',IndexFieldNames)-1));
            if (S = '')  then
              S := IndexFieldNames;
            F := FieldByName(S);
          end
          else
          try
            F := IndexFields[0];
          except
            F := nil;
          end;
          if Assigned(F) then
          begin
            LocateFieldName := F.FieldName;
            S := F.DisplayName;
            I := 0; // one less than current character position
            while (I < Length(S))
              and (ShortcutList.IndexOf(Uppercase(S[I + 1])) > 0)
              do
              Inc(I);
            ShortcutList.Add(Uppercase(S[I + 1]));
            LocateChar := Word(S[I + 1]);
            LLocate.Caption := Copy(S, 1, I) + '&' + Copy(S,I+1,Length(S) - I);
          end;
        end;
      end;
      T := MasterPanelList.Count;
      NextGrid := dbGridMaster;
      with MasterPanelList do
        for I := Count - 1 downto 0 do
          with TLKDcMasterPanel(Items[I]) do
          begin
            if (I = Count - 1) then
            begin
              Splitter.Free;
              Align := alClient;
            end;
            ChildMPGrid := NextGrid.Grid;
            NextGrid := GridController;
            TabOrder := T;
            Dec(T);
            S := GridComponent.NameFriendly;
            J := 0; // one less than current character position
            while (J < length(S)) and (ShortcutList.IndexOf(uppercase(S[J + 1]))
              > 0) do
              inc(J);
            ShortcutList.Add(S[J + 1]);
            ShortcutChar := Word(S[J + 1]);
            GridLabel.Caption := ' ' + copy(S, 1, J) + '&' + copy(S, J + 1,
              length(S) - J);
          end;
      FMyMasterComponent := FMyDataComponent.MasterComponent;
    finally ShortcutList.Free;
    end;
  end; // while
  pMasterDocsResize(Self);
end;

procedure TFormLKDataComponent.GridMasterKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Shift = [ssAlt]) then
  begin
    if (key = word('V')) then
    begin
      key := 0;
      ActionDrillDown.Execute;
    end;
  end;
end;

procedure TFormLKDataComponent.PlaceEditControls;
begin
  inherited;
  with FMyDataComponent do
  begin
    CreatedbGridMaster;
    ActiveControl := GridMaster;
    CustomisedbGridMaster;
    pMasterDocs.Width := RecallInteger('FormView', 'pMasterDocsWidth', 200,
      false);
    try
      ActionClearColumnSettings.Enabled := dbGridMaster.LoadFormat(
        FMyDataComponent, 'FormView', 'dbGridMaster');
    except ActionClearColumnSettingsExecute(Self);
    end;
  end;
  ActiveControl := dbGridMaster.Grid;
end;

procedure TFormLKDataComponent.pMasterDocsResize(Sender: TObject);
var
  i, Y, newY, t: Integer;
begin
  inherited;
  if (masterPanelList = nil) then
    Exit;
  i := masterPanelList.Count;
  if (i = 0) then
    exit;
  Y := Trunc((pMasterDocs.Height * (i - 1) / i) / i);
  with masterPanelList do
    for i := 0 to Count - 2 do
    begin
      newY := FMyDataComponent.RecallInteger('FormView',
        TLKDcMasterPanel(Items[i]).Name, 0, false);
      if (newY = 0) then
        newY := Y;
      t := newY + 1;
      with TLKDcMasterPanel(Items[i]) do
      begin
        while (Height < newY) do
        begin
          Height := Height + 2;
        end;
        while (Height > t) do
        begin
          Height := Height - 1;
        end;
      end;
    end;
end;

procedure TFormLKDataComponent.Construct(Sender: TLKDataComponent);
var
  i, x: Integer;
  newMenuitem: TMenuItem;
begin
  FMyDataComponent := Sender;
  inherited Construct(Sender);
  FilterPanel.FilterDataset := tbMaster;
  with FMyDataComponent do
  begin
    ActionCheckAllAccountTotals.Visible := (MSumDPairFieldNames.Count > 0);
    ActionNew.Visible := Editing;
    with TblMaster do
    begin
      //  MOldFilter := filter;
      //  MWasFiltered := filtered;
      IndexDefs.Update;
      ActionChangeIndex.Enabled := (IndexDefs.Count > 2);
    end;
    {
   with TblDetail do
    begin
    DOldFilter := filter;
    DWasFiltered := filtered;
    end;
    }
   //create menu items for each action
    with ActionList do
      for i := 0 to ActionCount - 1 do
        if (Actions[i].Category <> 'DetailTable') then
        begin
          newMenuitem := TMenuitem.Create(Self);
          newMenuitem.Action := Actions[i];
          miActivities.Add(newMenuitem);
            //add new menu item to 'Activities' men
          with ToolBar1.Buttons[ToolBar1.ButtonCount - 1] do
            x := Left + Width + 1;
          with TToolButton.Create(Self) do //created button setup
          begin
            Left := x;
            Parent := Toolbar1;
            Action := Actions[i];
          end;
        end; //for...
  end;
  LocateChar := 32;
end;

procedure TFormLKDataComponent.CreatedbGridMaster;
begin
  dbGridMaster.ConstructFromDatasource(HideMasterFields);
  dbGridMaster.ReadOnly := not FMyDataComponent.Editing;
  with dbGridMaster.Grid do
  begin
    PopupMenu := PopupMenu1;
    //Options := Options + [dgAlwaysShowSelection];
    OnKeyDown := GridMasterKeyDown;
    OnDblClick := ActionDrillDown.OnExecute;
  end;
end;

procedure TFormLKDataComponent.CustomisedbGridMaster;
var
  I: integer;
  S: string;
begin
  with dbGridMaster do
    for I := 0 to Count - 1 do
      with Items[I] do
      begin
        if (Pos('IMAGE', FieldName) > 0) then
          ControlType := dbcImageFile;
        with FMyDataComponent do
        begin
          S := RecallString(NameFriendly+'TblMaster', FieldName, '', true);
          if (S > '') then
          begin
            if (S = 'Date') then
              ControlType := dbcDateEdit;
            if (S = 'Time') then
              ControlType := dbcTimeEdit;
            if (S = 'ComboBox') then
            begin
              ControlType := dbcComboBox;
              TDBComboBox(EditControl).Items.Text :=
                RecallString(NameFriendly+'TblMaster', FieldName + 'ComboBox', '',
                  true);
            end;
            if (S = 'File') then
              ControlType := dbcFilePath;
            if (S = 'URL') then
              ControlType := dbcURL;
            if (S = 'EMail') then
              ControlType := dbcEMail;
            // if (s = '') then ControlType := ;
          end;
          if (MSumDPairFieldNames.IndexOf(FieldName) > -1) then
            ReadOnly := true;
        end;
      end;
end;

procedure TFormLKDataComponent.eLocateChange(Sender: TObject);
begin
  if (eLocate.Text = '') then
    exit;
  LocateDocsDS.Dataset.Locate(locateFieldName, eLocate.Text, [loCaseInsensitive,
    loPartialKey]);
end;

procedure TFormLKDataComponent.SetFolderStoreDocumentImages(
  const Value: string);
var
  I: integer;
begin
  //  inherited;
  with dbGridMaster do
    for I := 0 to Count - 1 do
      with Items[I] do
        if (ControlType = dbcImageFile) then
          TLKDBImageEdit(EditControl).ImageFilePath := Value;
end;

procedure TFormLKDataComponent.FormShow(Sender: TObject);
begin
  inherited;
  with FMyDataComponent do
  begin
    if Filtered then
    begin
      Filtered := False;
      Filtered := True;
    end;
    ActionNew.Enabled := Editing;
    ActionNew.Visible := Editing;
    ActionDrillDown.Enabled := not FMyDataComponent.TblMaster.IsEmpty;
    myStatusBar.Panels[0].Text := 'Index: ' + TblMaster.IndexName;
  end;
  ApplyMasterComponentInfo;
end;

procedure TFormLKDataComponent.Panel1Exit(Sender: TObject);
begin
  inherited;
  FMyDataComponent.Post;
end;

procedure TFormLKDataComponent.ActionChangeIndexExecute(Sender: TObject);
begin
  with FMyDataComponent do
  begin
//    TblMaster.IndexNext;
    myStatusBar.Panels[0].Text := 'Index: ' + IndexName;
  end;
end;

procedure TFormLKDataComponent.ActionNewExecute(Sender: TObject);
begin
  FMyDataComponent.RecordNew;
end;

procedure TFormLKDataComponent.ActionDrillDownExecute(Sender: TObject);
begin
  FMyDataComponent.DrillDown;
end;

procedure TFormLKDataComponent.FormActivate(Sender: TObject);
begin
  inherited;
  if Assigned(pMasterDocs) then
    if pMasterDocs.visible then
    begin
      ActiveControl := eLocate;
      eLocateChange(self);
    end;
end;

procedure TFormLKDataComponent.FormClose(Sender: TObject;
  var Action: TCloseAction);
var
  I : Integer;
//var myBookMark : TBookMark;
begin
  dbGridMaster.DoOnFormClose;
  with MasterPanelList do
    for I := 0 to Count - 1 do
      Items[I].DoOnFormClose;
  FMyDataComponent.Post;
  inherited;
end;

procedure TFormLKDataComponent.FormCreate(Sender: TObject);
begin
  inherited;
  MasterPanelList := TLKDcMasterPanelList.Create(False);
  FMyMasterComponent := nil;
end;

procedure TFormLKDataComponent.FormDestroy(Sender: TObject);
begin
  MasterPanelList.Free;
  inherited;
end;

procedure TFormLKDataComponent.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  I: Integer;
begin
  if (Shift = [ssAlt]) then
  begin
    if (Key = LocateChar) then
    begin
      Key := 0;
      eLocate.SetFocus;
    end
    else
      with MasterPanelList do
        for I := 0 to count - 1 do
          with TLKDcMasterPanel(items[I]) do
            if (Key = ShortcutChar) then
            begin
              Key := 0;
              Grid.SetFocus;
            end;
  end;
  inherited;
end;

procedure TFormLKDataComponent.ActionFindExecute(Sender: TObject);
var
  fn: string;
begin
  fn := '';
  with GridMaster do
    if assigned(SelectedField) then
      fn := SelectedField.FieldName;
  FMyDataComponent.DlgFindExecute(fn);
end;

procedure TFormLKDataComponent.ActionSaveColumnSettingsExecute(
  Sender: TObject);
begin
  dbGridMaster.SaveFormat(FMyDataComponent, 'FormView', 'dbGridMaster');
  ActionClearColumnSettings.Enabled := true;
  inherited;
end;

procedure TFormLKDataComponent.ActionViewAllRecordsExecute(Sender: TObject);
begin
  FMyDataComponent.ViewAll(null, [mbOK]);
end;

procedure TFormLKDataComponent.ActionViewMasterPanelExecute(Sender: TObject);
begin
  inherited;
  ActionViewMasterPanel.Checked := not ActionViewMasterPanel.Checked;
  pMasterDocs.Visible := ActionViewMasterPanel.checked;
end;

procedure TFormLKDataComponent.ActionCheckAllAccountTotalsExecute(
  Sender: TObject);
var
  hasMaster, wasFiltered: boolean;
  oldFilter: string;
begin
  inherited;
  with FMyDataComponent do
  begin
    hasMaster := assigned(MasterComponent);
    with TblDetail do
    begin
      wasFiltered := Filtered;
      oldFilter := Filter;
      if Filtered and (Filter > '') then
        if MessageDlg('Remove Filter on Detail records?',
          mtInformation, [mbYes, mbNo], 0) = mrYes then
        begin
          Filter := '';
          Filtered := false;
        end;
    end;
    try
      if hasMaster then
        MasterComponent.FindFirst;
      repeat if TblMaster.FindFirst then
          repeat
            if not AccountCheckTotals(false, false, false) then
              if
                (messageDlg('Do you wish to correct the Balance to equal the sum of the records?',
                mtInformation, [mbYes, mbNo], 0) = mrYes) then
                AccountCheckTotals(false, true, false);
          until not TblMaster.FindNext;
        if not hasMaster then
          break;
      until not MasterComponent.FindNext;
      Showmessage('Check totals complete');
    finally
      with TblDetail do
      begin
        Filtered := WasFiltered;
        Filter := oldFilter;
      end;
    end;
  end;
end;

procedure TFormLKDataComponent.SetKeyFieldsEditing;
var
  I: Integer;
  ixfn: TStringList;
begin
  // inherited not called;
  ixfn := TStringList.Create;
  try
    with FMyDataComponent do
      if Editing then
      begin
        with TblMaster do
          for I := 0 to IndexFieldCount - 1 do
            ixfn.Add(IndexFields[I].FieldName);
        with dbGridMaster do
          for i := 0 to Count - 1 do
              if (ixfn.IndexOf(Items[i].FieldName) > -1) then
                Items[i].ReadOnly := True;
      end;
  finally ixfn.Free;
  end;
end;

procedure TFormLKDataComponent.SplitterMDPanelMoved(Sender: TObject);
begin
  inherited;
  FMyDataComponent.RememberInteger('FormView', 'pMasterDocsWidth',
    pMasterDocs.Width, false);
end;

procedure TFormLKDataComponent.ActionClearColumnSettingsExecute(
  Sender: TObject);
begin
  //inherited;
  dbGridMaster.ClearFormat(FMyDataComponent, 'FormView', 'dbGridMaster');
end;

procedure TFormLKDataComponent.ActionReverseIndexExecute(Sender: TObject);
var
  id: TIndexDef;
begin
  inherited;
  with FMyDataComponent do
    id := IndexDefs.Find(IndexName);
  if assigned(id) then
    if (ixDescending in id.Options) then
      id.Options := id.Options - [ixDescending]
    else
      id.Options := id.Options + [ixDescending];
end;


end.

