{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for the display of TLKDataComponent record                        }
{                                                                             }
{*****************************************************************************}

unit LKDataComponentRecordDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, System.UITypes,
  Graphics, Controls, Forms, Dialogs,
  ImgList, Menus, ActnList, Db, ToolWin, Variants, Math,
  ComCtrls, StdCtrls, DBCtrls, Buttons, ExtCtrls, Grids, DBGrids,
  LKdlgs, LKUtils, LKDefine, LKRecordsComponents, LKDataCompCommonDlg,
  LKUtilComponents, LKDBControls,
  StdActns, JvExDBGrids, JvDBGrid, JvDBUltimGrid;

type
  TFormLKDataComponentRecord = class(TFormLKDataComponentCommon)
    Panel1: TPanel;
    DSShowDetail: TDataSource;
    miViewBreak1: TMenuItem;
    ActionCheckTotals: TAction;
    CheckAccountTotals1: TMenuItem;
    ActionMasterEditsProperties: TAction;
    pmPanelMasterEdits: TPopupMenu;
    DisplayProperties1: TMenuItem;
    MasterEditList: TLKDcDBEditsPanel;
    GridDetail: TJvDBUltimGrid;
    dbGridDetail: TLKDcDBGridController;
    procedure FormShow(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ActionCheckTotalsExecute(Sender: TObject);
    procedure ActionMasterEditsPropertiesExecute(Sender: TObject);
    procedure ActionSaveColumnSettingsExecute(Sender: TObject);
    procedure ActionClearColumnSettingsExecute(Sender: TObject);
    procedure ActionFindExecute(Sender: TObject);
    procedure ActionChangeIndexExecute(Sender: TObject);
    procedure ActionDrillDownExecute(Sender: TObject);
    procedure ActionCloseExecute(Sender: TObject);
    procedure ActionDeleteRecordExecute(Sender: TObject);
    procedure ActionPrintExecute(Sender: TObject);
    procedure ActionReverseIndexExecute(Sender: TObject);
  private
    { Private declarations }
    FMyDataComponent: TLKDataComponent;
  protected
    HideDetailFields: string;
    procedure CreatedbGridDetail;
    procedure PlaceEditControls; override;
    procedure SetDataSourceDataSets; override;
    procedure SetHideFields; override;
    procedure CustomiseDetailEdits; virtual;
    procedure CustomiseDetailGrid; virtual;
    procedure CustomiseMasterEdits; virtual;
    procedure SetMasterEditsActions; virtual;
  public
    { Public declarations }
//    DlgDtlDataComponent : TLKDataComponent;
    procedure Construct(Sender: TLKDataComponent);
    procedure SetFolderStoreDocumentImages(const Value: string); override;
    procedure SetKeyFieldsEditing; override;
  end;

var
  FormLKDataComponentRecord: TFormLKDataComponentRecord;

implementation

{$R *.DFM}
uses LKUtilsEditListPropsDlg;

procedure TFormLKDataComponentRecord.SetDataSourceDataSets;
begin
  with FMyDataComponent do
  begin
    TblMasterDataSource := DSShowMaster;
    if TblDetailUsed then
      TblDetailDataSource := DSShowDetail;
  end;
end;

procedure TFormLKDataComponentRecord.SetHideFields;
begin
  inherited;
  HideDetailFields := FMyDataComponent.TblDetail.MasterFields;
end;

procedure TFormLKDataComponentRecord.PlaceEditControls;
var
  I, X: integer;
  MasterEditsLayout: TLKEditsLayout;
begin
  inherited;
  with FMyDataComponent do
  begin
    // count up fields
    X := 0;
    with TblMaster do
      for I := 0 to FieldCount - 1 do
        with Fields[I] do
          if Visible and (Pos(FieldName, TblMaster.MasterFields) = 0) then
            Inc(X);
    // calculate space available
    // i = height divisor
    I := 1;
    if TblDetailUsed then
      Inc(I);
    // calculate 16 pixel lines available
    I := Trunc((Screen.Height - 20) / (I * 16));
    MasterEditsLayout := aceTwoCol;
    // make 3 columns if not enough space available for two columns
    if (X > (I / 2)) then
      MasterEditsLayout := aceThreeCol;
    MasterEditList.ConstructFromDatasource(HideMasterFields);
    CustomiseMasterEdits;
    MasterEditList.Layout := MasterEditsLayout;
    MasterEditList.ReadOnly := not Editing;
    MasterEditList.LoadFormat(FMyDataComponent, 'FormViewRecord',
      'dbMasterEdits');
    SetMasterEditsActions;
    if TblDetailUsed then
    begin
      CreatedbGridDetail;
      CustomiseDetailGrid;
      try
        ActionClearColumnSettings.Enabled := dbGridDetail.LoadFormat(
          FMyDataComponent, 'FormViewRecord', 'dbGridDetail');
      except ActionClearColumnSettings.Execute;
      end;
    end
    else
    begin
      GridDetail.Visible := False;
      Panel1.Align := alClient;
    end;
  end;
end;

procedure TFormLKDataComponentRecord.Construct(Sender: TLKDataComponent);
var
  I, X: Integer;
  NewMenuitem: TMenuItem;
begin
  FMyDataComponent := Sender;
  inherited Construct(Sender);
  with FMyDataComponent do
  begin
    ActionCheckTotals.Visible := (MSumDPairFieldNames.Count > 0);
    if TblDetailUsed then
      with TblDetail do
      begin
        IndexDefs.Update;
        ActionChangeIndex.Enabled := (IndexDefs.Count > 2);
        FilterPanel.FilterDataset := tbDetail;
      end
    else
    begin
      ToolBar5.Free;
      with ToolBar4 do
      begin
        Top := 2;
        Left := 11;
      end;
      ToolbarStandard.Width := 280;
      ActionFind.Enabled := False;
      ActionChangeIndex.Enabled := False;
    end;
  end;
  with FMyDataComponent.ActionList do
    for I := 0 to ActionCount - 1 do
      if (Actions[I].Category <> 'MasterTable') then
      begin
        NewMenuitem := TMenuitem.Create(self);
        NewMenuitem.Action := Actions[I];
        miActivities.Add(NewMenuitem); //add new menu item to 'Activities' men
        //create speedbutton for each action
        with ToolBar1.Buttons[ToolBar1.ButtonCount - 1] do
          X := Left + Width + 1;
        with TToolButton.Create(Self) do //created button setup
        begin
          Left := X;
          Parent := Toolbar1;
          Action := Actions[I];
        end;
      end; //for...
end;

procedure TFormLKDataComponentRecord.FormShow(Sender: TObject);
var
  I: Integer;
begin
  inherited;
  SetKeyFieldsEditing;
  with MasterEditList do
  begin
    for I := 0 to Count - 1 do
      if (Items[I].Visible and (Items[I].EditControl is TWinControl)
       and not Items[I].ReadOnly) then
      begin
        ActiveControl := TWinControl(Items[I].EditControl);
        Break;
      end;
    Resize;
  end;
  Panel1.Height := 4 + Max(Panel1.Height, MasterEditList.DisplayHeight);
  myStatusBar.Panels[0].Text := 'Index: ' +
    FMyDataComponent.TblDetail.IndexName;
  I := 90 + ToolbarStandard.Height + Panel1.Height + pModalButtons.Height;
  if ToolBar1.Visible then
    I := I + ToolBar1.Height;
  if GridDetail.Visible then
    I := I + 120;
  if (I > Screen.Height)  then
    I := Screen.Height;
  Height := Max(I, Height);
end;

procedure TFormLKDataComponentRecord.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  FMyDataComponent.Post;
  if FMyDataComponent.TblDetailUsed then
    dbGridDetail.DoOnFormClose;
  MasterEditList.DoOnFormClose;
  inherited;
end;

procedure TFormLKDataComponentRecord.ActionCheckTotalsExecute(
  Sender: TObject);
begin
  inherited;
  with FMyDataComponent do
    if not AccountCheckTotals(True, False, False) then
      if (MessageDlg(
        'Do you wish to correct the Balance to equal the sum of the records?',
        mtInformation, [mbYes, mbNo], 0) = mrYes) then
        AccountCheckTotals(False, True, False);
end;

procedure TFormLKDataComponentRecord.FormResize(Sender: TObject);
var
  X: Integer;
begin
  inherited;
  // set minimum height according to viisible features
  X := ToolbarStandard.Height + Panel1.Height + myStatusBar.Height + 48;
  if ToolBar1.Visible then
    X := X + ToolBar1.Height;
  if pModalButtons.Visible then
    X := X + pModalButtons.Height;
  if GridDetail.Visible then
    X := X + 50;
  if (X > Screen.Height) then
    X := Screen.Height;
  if (Height < X) then
    Height := X;
end;

procedure TFormLKDataComponentRecord.SetKeyFieldsEditing;
var
  I: Integer;
  IndexFieldNameList, MasterFieldList: TStringlist;
begin
//  if not Assigned(MasterEditList) then
//    Exit;
  with FMyDataComponent do
    if Editing then
    begin
      MasterEditList.UpdateFields;
      IndexFieldNameList := TStringlist.Create;
      MasterFieldList := TStringlist.Create;
      try
        DelimitedStringToList(MasterFields, ';', MasterFieldList);
        with TblMaster do
          for I := 0 to IndexFieldCount - 1 do
            IndexFieldNameList.Add(IndexFields[I].FieldName);
        with MasterEditList do
          for I := 0 to Count - 1 do
            with Items[I] do
              if (IndexFieldNameList.IndexOf(FieldName) > -1) then
              begin
                if (MasterFieldList.IndexOf(FieldName) > -1) then
                  ReadOnly := True
                else
                  ReadOnly := not IsRecordNew;
              end;
      finally
        begin
          IndexFieldNameList.Free;
          MasterFieldList.Free
        end;
      end;
    end;
end;

procedure TFormLKDataComponentRecord.ActionChangeIndexExecute(
  Sender: TObject);
begin
  MyStatusBar.Panels[0].Text := 'Index: ' +FMyDataComponent.TblDetail.IndexName;
end;

procedure TFormLKDataComponentRecord.ActionClearColumnSettingsExecute(
  Sender: TObject);
begin
  //  inherited;
  ActionClearColumnSettings.Enabled := False;
  if not FMyDataComponent.TblDetailUsed then
    Exit;
  dbGridDetail.ClearFormat(FMyDataComponent, 'FormViewRecord', 'dbGridDetail');
  CreatedbGridDetail;
end;

procedure TFormLKDataComponentRecord.ActionCloseExecute(Sender: TObject);
begin
  if fsModal in FormState then
    ModalResult := mrOK
  else
    Close;
end;

procedure TFormLKDataComponentRecord.ActionDeleteRecordExecute(
  Sender: TObject);
begin
  if (MessageDlg('Confirm: delete this detail record?', mtInformation, [mbOK,
    mbCancel], 0) = mrOK) then
    FMyDataComponent.TblDetail.Delete;
end;

procedure TFormLKDataComponentRecord.ActionDrillDownExecute(
  Sender: TObject);
begin
  FMyDataComponent.DrillDownRecord;
end;

procedure TFormLKDataComponentRecord.ActionFindExecute(Sender: TObject);
var
  Fn: string;
begin
  Fn := '';
  if FMyDataComponent.TblDetailUsed then
    with dbGridDetail do
      if Assigned(SelectedField) then
        Fn := SelectedField.FieldName;
  FMyDataComponent.DlgFindDetailExecute(Self, Fn);
end;

procedure TFormLKDataComponentRecord.ActionMasterEditsPropertiesExecute(
  Sender: TObject);
begin
  inherited;
  if (MasterEditList.DlgEditListPropsExecute(Self) = mrOK) then
  begin
    MasterEditList.SaveFormat(FMyDataComponent, 'FormViewRecord',
      'dbMasterEdits');
    SetMasterEditsActions;
  end;
end;

procedure TFormLKDataComponentRecord.ActionPrintExecute(Sender: TObject);
begin
  with FMyDataComponent do
    DlgPrintExecute(NameFriendly, 'Single record', null, False);
end;

procedure TFormLKDataComponentRecord.ActionReverseIndexExecute(
  Sender: TObject);
var
  ID: TIndexDef;
begin
  inherited;
  with FMyDataComponent do
    ID := TblDetail.IndexDefs.Find(TblDetail.IndexName);
  if Assigned(ID) then
    if (ixDescending in ID.Options) then
      ID.Options := ID.Options - [ixDescending]
    else
      ID.Options := ID.Options + [ixDescending];
end;

procedure TFormLKDataComponentRecord.ActionSaveColumnSettingsExecute(
  Sender: TObject);
begin
  if not FMyDataComponent.TblDetailUsed then
    Exit;
  dbGridDetail.SaveFormat(FMyDataComponent, 'FormViewRecord',
    'dbGridDetail');
  ActionClearColumnSettings.Enabled := True;
  inherited;
end;

procedure TFormLKDataComponentRecord.CreatedbGridDetail;
begin
  dbGridDetail.ConstructFromDatasource(HideDetailFields);
  dbGridDetail.ReadOnly := not FMyDataComponent.Editing;
  GridDetail.OnDblClick := ActionDrillDown.OnExecute;
end;

procedure TFormLKDataComponentRecord.SetMasterEditsActions;
begin
  //override in Child forms
end;

procedure TFormLKDataComponentRecord.CustomiseDetailEdits;
begin
  // override as required
end;

procedure TFormLKDataComponentRecord.CustomiseDetailGrid;
begin
  // override as required
end;

procedure TFormLKDataComponentRecord.CustomiseMasterEdits;
begin
  // override as required
end;

procedure TFormLKDataComponentRecord.SetFolderStoreDocumentImages(
  const Value: string);
var
  I: Integer;
begin
  with MasterEditList do
    for I := 0 to Count - 1 do
      with Items[I] do
        if (ControlType = dbcImageFile) then
          TLKDBImageEdit(EditControl).ImageFilePath := Value;
end;

end.

