{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      TLKGroup component editor dialog                                       }
{                                                                             }
{*****************************************************************************}

unit LKGroupEditorDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls, XPStyleActnCtrls, ActnList,
  ActnMan, ToolWin, ActnCtrls, LKRecordsComponents, StdActnMenus, Buttons,
  Menus, ImgList, JvExControls, JvComponent, JvOutlookBar, LKFormBaseDlg,
  StdActns;

type
  TDlgLKGroupEditor = class(TFormLKBase)
    ActionList1: TActionList;
    aOffScreen: TAction;
    aOnScreen: TAction;
    pShow: TPanel;
    JvOutlookBar1: TJvOutlookBar;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure aCloseExecute(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure aOffScreenExecute(Sender: TObject);
    procedure aOnScreenExecute(Sender: TObject);
    procedure pShowClick(Sender: TObject);
    procedure JvOutlookBar1ButtonClick(Sender: TObject; Index: Integer);
  private
    { Private declarations }
    FActiveForm: TForm;
    FMyGroup: TLKGroup;
  public
    { Public declarations }
    property MyGroup: TLKGroup read FMyGroup write FMyGroup;
    procedure Construct(Sender: TLKGroup); virtual;
  end;

var
  DlgLKGroupEditor: TDlgLKGroupEditor;

implementation

{$R *.dfm}

procedure TDlgLKGroupEditor.Construct(Sender: TLKGroup);
var
  i: integer;
  olbfdr: TJvOutlookBarPage;
begin
  FMyGroup := Sender;
  with FMyGroup do
  begin
    Caption := NameFriendly;
    JvOutlookBar1.PageImages := Images;
    AddToOutlookBar(JvOutlookBar1);
    olbfdr := OutlookBarPage;
    with GroupComponents do
      for i := 0 to count - 1 do
        with TLKGroupComponent(Items[i]) do
          if TreeNodeVisible then
          begin
            AddToOutLookBarPage(olbfdr);
            OutlookBarButton.Tag := i;
          end;
  end;
end;

procedure TDlgLKGroupEditor.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  if Assigned(FActiveForm) then
    FActiveForm.Close;
end;

procedure TDlgLKGroupEditor.aCloseExecute(Sender: TObject);
begin
  Close;
end;

procedure TDlgLKGroupEditor.FormShow(Sender: TObject);
begin
  aOnScreen.Execute;
end;

procedure TDlgLKGroupEditor.aOffScreenExecute(Sender: TObject);
var
  i: integer;
begin
  pShow.Enabled := false;
  Top := 0;
  Height := Screen.Height;
  for i := 0 downto 10 - Width do
    Left := i;
  pShow.Caption := '>';
  pShow.Enabled := true;
end;

procedure TDlgLKGroupEditor.aOnScreenExecute(Sender: TObject);
var
  i: integer;
begin
  pShow.Enabled := false;
  Top := 0;
  Height := Screen.Height;
  for i := 10 - Width to 0 do
    Left := i;
  pShow.Caption := '<';
  pShow.Enabled := true;
end;

procedure TDlgLKGroupEditor.pShowClick(Sender: TObject);
begin
  if (pShow.Caption = '>') then
    aOnScreenExecute(Sender)
  else
    aOffScreenExecute(Sender);
end;

procedure TDlgLKGroupEditor.JvOutlookBar1ButtonClick(Sender: TObject;
  Index: Integer);
var
  i: integer;
begin
  if assigned(FActiveForm) then
    FActiveForm.Close;
  with JvOutlookBar1 do
    i := Pages[ActivePageIndex].DownButton.Tag;
  with TLKGroupComponent(FMyGroup.GroupComponents[i]) do
  begin
    DlgViewExecute;
    FActiveForm := FormView;
  end;
end;

end.

