{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for setting LKHub database properties                             }
{                                                                             }
{*****************************************************************************}

unit LKHubDatabaseDlg;

interface

uses Windows, SysUtils, Classes, System.UITypes,
  Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, registry, dialogs, LKdlgs, LKRecordsComponents, Grids,
  LKDefine, ComCtrls, ShellAPI,
  JvBrowseFolder, ImgList, ActnList, StdActns, LKBaseComponent, JvBaseDlg,
  JvPageList, JvExControls;

type
  TDlgHubDatabase = class(TForm)
    SelectFolderDlg: TJvBrowseForFolderDialog;
    LKHeaderPanel2: TLKHeaderPanel;
    Panel1: TPanel;
    ButtonBack: TButton;
    ButtonNext: TButton;
    ButtonCancel: TButton;
    LKHeaderPanel1: TLKHeaderPanel;
    LKHeaderPanelTestConnection: TLKHeaderPanel;
    BitBtnTestConnection: TBitBtn;
    Panel2: TPanel;
    Panel3: TPanel;
    STEnterpriseNick: TStaticText;
    EditEnterpriseCode: TEdit;
    SbAddEnterpriseCode: TBitBtn;
    ListBoxEnterpriseCodes: TListBox;
    Label5: TLabel;
    SbDelEnterprise: TBitBtn;
    sbDelYear: TBitBtn;
    Label7: TLabel;
    ListBoxEnterpriseYears: TListBox;
    SbAddEnterpriseYear: TBitBtn;
    EditEnterpriseYear: TEdit;
    STFinYear: TStaticText;
    Label4: TLabel;
    LabelDBFileName: TLabel;
    CbEnterpriseYearDefault: TCheckBox;
    LKHeaderPanel4: TLKHeaderPanel;
    Panel4: TPanel;
    Panel5: TPanel;
    STDBHostServer: TStaticText;
    EditDatabaseHost: TEdit;
    EditDBUserName: TEdit;
    STDBUserName: TStaticText;
    STDBPasswd: TStaticText;
    EditDatabasePassword: TEdit;
    LKHeaderPaneDatabaseNotes: TLKHeaderPanel;
    JvPageList1: TJvPageList;
    JvStandardPage1: TJvStandardPage;
    JvStandardPage2: TJvStandardPage;
    JvStandardPage3: TJvStandardPage;
    MemoCurrentSettings: TMemo;
    StaticText1: TStaticText;
    ComboBoxDBMS: TComboBox;
    procedure SbDelEnterpriseClick(Sender: TObject);
    procedure SbDelYearClick(Sender: TObject);
    procedure SbAddEnterpriseCodeClick(Sender: TObject);
    procedure ImageOpenFirebirdClick(Sender: TObject);
    procedure ButtonBackClick(Sender: TObject);
    procedure ButtonNextClick(Sender: TObject);
    procedure EditEnterpriseCodeChange(Sender: TObject);
    procedure EditEnterpriseYearChange(Sender: TObject);
    procedure ListBoxEnterpriseCodesClick(Sender: TObject);
    procedure ListBoxEnterpriseYearsClick(Sender: TObject);
    procedure SbAddEnterpriseYearClick(Sender: TObject);
    procedure BitBtnTestConnectionClick(Sender: TObject);
    procedure JvPageList1Change(Sender: TObject);
    procedure EditDatabaseHostChange(Sender: TObject);
    procedure ComboBoxDBMSChange(Sender: TObject);
  private
    { Private declarations }
    MyHub: TLKHubRecords;
    procedure UpdateUI;
    function GetSelectedEnterprise: string;
    function GetSelectedEnterpriseYear: string;
    function GetSelectedDBMSExtension: string;
    property SelectedDBMSExtension: string read GetSelectedDBMSExtension;
    property SelectedEnterprise: string read GetSelectedEnterprise;
    property SelectedEnterpriseYear: string read GetSelectedEnterpriseYear;
  public
    { Public declarations }
    procedure Construct(AHub: TLKHubRecords); virtual;
  end;

var
  DlgHubDatabase: TDlgHubDatabase;

implementation

uses LKutils;

{$R *.DFM}             

procedure TDlgHubDatabase.EditDatabaseHostChange(Sender: TObject);
begin
  UpdateUI;
end;

procedure TDlgHubDatabase.ComboBoxDBMSChange(Sender: TObject);

  procedure CheckFiles(const AVendorLib, ADBXDriver: string);
  begin
    if not FileExists(ProgramFolder + PathDelim + AVendorLib) then
      ShowMessage('To use '+ ComboBoxDBMS.Text +', please save a copy of the'
        +' Vendor library "'+AVendorLib+'" in the folder '+ ProgramFolder);
    if not FileExists(ProgramFolder + PathDelim + ADBXDriver) then
      ShowMessage('To use '+ ComboBoxDBMS.Text +', please save a copy of the'
        +' dtiver library "'+ADBXDriver+'" in the folder '+ ProgramFolder);
  end;

begin
  case ComboBoxDBMS.ItemIndex of
    0:
    begin
      EditDatabaseHost.Text := 'Localhost';
      CheckFiles('fbembed.dll', 'dbxfb.dll');
    end;
    1: CheckFiles('fbclient.dll', 'dbxfb.dll');
    2: CheckFiles('sqlncli10.dll', 'dbxmss.dll');
    3: CheckFiles('sqlncli.dll', 'dbxmss9.dll');
  end;
  UpdateUI;
end;

procedure TDlgHubDatabase.Construct(AHub: TLKHubRecords);
begin
// TLKSupportedDatabaseType = (sdbUnknown, sdbLocalFirebird, sdbDB2, sdbFirebird,
//    sdbInformix, sdbInterbase, sdbMSSQL, sdbMSSQL9, sdbMySQL, sdbOracle);
  MyHub := AHub;
  with MyHub do
  begin
    Close;
    case DBMS of
      sdbUnknown,
      sdbLocalFirebird : ComboBoxDBMS.ItemIndex := 0;
      sdbFirebird      : ComboBoxDBMS.ItemIndex := 1;
      sdbMSSQL         : ComboBoxDBMS.ItemIndex := 2;
      sdbMSSQL9        : ComboBoxDBMS.ItemIndex := 3;
    end;
    EditDatabaseHost.Text := DatabaseHost;
    EditDBUsername.Text := DBUserName;
    EditDatabasePassword.Text := DBUserPassword;
    with ListBoxEnterpriseCodes do
    begin
      Items.Assign(Enterprises);
      ItemIndex := Items.IndexOf(EnterpriseCurrent);
    end;
  end;
  JvPageList1.ActivePageIndex := 0;
  with ListBoxEnterpriseYears do
  begin
    Items.Assign(MyHub.EnterpriseYears);
    ItemIndex := Count -1;
  end;
  ListBoxEnterpriseCodesClick(nil);
end;

function TDlgHubDatabase.GetSelectedDBMSExtension: string;
begin
  case ComboBoxDBMS.ItemIndex of
    0,1: Result := '.FDB';
    else Result := '.MDF';
  end;
end;

function TDlgHubDatabase.GetSelectedEnterprise: string;
begin
  Result := '';
  with ListBoxEnterpriseCodes do
    if (ItemIndex > -1) then
      Result := Items[ItemIndex];
end;

function TDlgHubDatabase.GetSelectedEnterpriseYear: string;
begin
  Result := '';
  with ListBoxEnterpriseYears do
    if (ItemIndex > -1) then
      Result := Items[ItemIndex];
end;

procedure TDlgHubDatabase.ImageOpenFirebirdClick(Sender: TObject);
begin
  ShellExecute(Application.Handle,'open','http://www.firebirdsql.org/','','',1);
end;

procedure TDlgHubDatabase.ListBoxEnterpriseCodesClick(Sender: TObject);
begin
  if (MyHub.EnterpriseCurrent <> SelectedEnterprise) then
    MyHub.EnterpriseCurrent := SelectedEnterprise;
  ListBoxEnterpriseYearsClick(Sender);
end;

procedure TDlgHubDatabase.ListBoxEnterpriseYearsClick(Sender: TObject);
begin
  MyHub.EnterpriseAccountsYearCurrent := SelectedEnterpriseYear;
  UpdateUI;
end;

procedure TDlgHubDatabase.JvPageList1Change(Sender: TObject);
var
  H, DF : string;

  procedure AddServerConnectionComment;
  begin
    LKHeaderPanelTestConnection.Comments.Append('Please ensure that the'
      + ' database Server is running on '+ H + '.');
    LKHeaderPanelTestConnection.Comments.Append(' ');
    LKHeaderPanelTestConnection.Comments.Append('Check that you have entered a'
      + ' line like');
    LKHeaderPanelTestConnection.Comments.Append('  '+DF+'=/(full path to data)/'
       + DF + SelectedDBMSExtension);
    LKHeaderPanelTestConnection.Comments.Append('in the aliases.conf file found'
        + ' in the Firebird program folder.');
  end;

begin    
  case JvPageList1.ActivePageIndex of
    0:
    begin
      ButtonBack.Visible := False;
      ButtonNext.Caption := 'Next >>';
      ActiveControl := ComboBoxDBMS;
    end;
    1:
    begin
      case ComboBoxDBMS.ItemIndex of
        0: MyHub.DBMS := sdbLocalFirebird;
        1: MyHub.DBMS := sdbFirebird;
        2: MyHub.DBMS := sdbMSSQL;
        3: MyHub.DBMS := sdbMSSQL9;
      end;
      MyHub.DatabaseHost := EditDatabaseHost.Text;
      MyHub.DBUserName := EditDBUsername.Text;
      MyHub.DBUserPassword := EditDatabasePassword.Text;
      ButtonBack.Visible := True;
      ButtonNext.Caption := 'Next >>';
      ActiveControl := EditEnterpriseCode;
    end;
    2:
    begin
      ButtonBack.Visible := True;
      ButtonNext.Caption := 'Finish';
      with MemoCurrentSettings.Lines do
      begin
        Clear;
        Add('Params = ');
        AddStrings(MyHub.SQLConnection.Params);
        Add('Drivername = '+MyHub.SQLConnection.DriverName);
        Add('LibraryName = '+MyHub.SQLConnection.LibraryName);
        Add('VendorLib = '+MyHub.SQLConnection.VendorLib);
      end;
      if CbEnterpriseYearDefault.Checked then
      begin
        MyHub.EnterpriseDefault := SelectedEnterprise;
        MyHub.EnterpriseAccountsYearDefault := SelectedEnterpriseYear;
      end;
      DF := SelectedEnterprise + SelectedEnterpriseYear;
      LKHeaderPanelTestConnection.Comments.Text :=
        'Please check or create your '+ DF + SelectedDBMSExtension +
        ' database file. You can copy and rename the SAMPLE'
        + SelectedDBMSExtension + ' file found in your '+ DataFolder
        + ' folder';
      LKHeaderPanelTestConnection.Comments.Append(' ');
      case ComboBoxDBMS.ItemIndex of
      0:
        begin
          H := 'this computer';
          AddServerConnectionComment;
        end;
      1, 2, 3:
        begin
          H := ComboBoxDBMS.Text;
          AddServerConnectionComment;
        end;
      else
        LKHeaderPanelTestConnection.Comments.Text := 'The database options are'
        + ' not set. Please click [<< Back] twice to return to the settings.';
      end;
      if not MyHub.Active then
        ActiveControl := BitBtnTestConnection;
    end;
  end;
  UpdateUI;
end;

procedure TDlgHubDatabase.SbDelEnterpriseClick(Sender: TObject);
begin
  if (ListBoxEnterpriseCodes.ItemIndex < 0) then
    Exit;
  if (MessageDlg('Confirm that you want to remove ' + SelectedEnterprise
      + '? (does not delete data)', mtWarning, [mbYes, mbNo], 0) = mrYes) then
  begin
    with ListBoxEnterpriseCodes do
    begin
      Items.Delete(ItemIndex);
      if Count > 0 then
        ItemIndex := 0;   
      MyHub.Enterprises := Items;
    end;
    ListBoxEnterpriseCodesClick(Sender);
  end;
end;

procedure TDlgHubDatabase.SbAddEnterpriseCodeClick(Sender: TObject);
var
  S: string;
begin
  SbAddEnterpriseCode.Enabled := False;
  S := Uppercase(Trim(EditEnterpriseCode.Text));
  EditEnterpriseCode.Text := '';
  if (Length(S) <> 4) then
  begin
    EditEnterpriseCode.Text := S;
    Showmessage('Nickname should be 4 characters long');
  end
  else with ListBoxEnterpriseCodes do
  begin
    if (Items.IndexOf(S) < 0) then
    begin
      AddItem(S,nil);
      MyHub.Enterprises := Items;
    end;
    ItemIndex := Items.IndexOf(S);
  end;
  ListBoxEnterpriseCodesClick(Sender);
end;

procedure TDlgHubDatabase.SbAddEnterpriseYearClick(Sender: TObject);
var
  S: string;
begin
  SbAddEnterpriseYear.Enabled := False;
  S := Trim(EditEnterpriseYear.Text);
  EditEnterpriseYear.Text := '';
  if (Length(S) <> 4) then
  begin
    EditEnterpriseYear.Text := S;
    Showmessage('Accounts Year should be 4 digits long');
  end
  else with ListBoxEnterpriseYears do
  begin
    if (Items.IndexOf(S) < 0) then
    begin
      AddItem(S, nil);
      MyHub.EnterpriseYears := Items;
    end;
    ItemIndex := Items.IndexOf(S);
  end;
  ListBoxEnterpriseYearsClick(Sender);
end;

procedure TDlgHubDatabase.SbDelYearClick(Sender: TObject);
begin
  if (ListBoxEnterpriseYears.ItemIndex < 0) then
    Exit;
  if (MessageDlg('Confirm that you want to remove Accounts year ' + SelectedEnterpriseYear
      + '? (does not delete data)', mtWarning, [mbYes, mbNo], 0) = mrYes) then
  begin
    with ListBoxEnterpriseYears do
    begin
      Items.Delete(ItemIndex);
      if Count > 0 then
        ItemIndex := 0;     
      MyHub.EnterpriseYears := Items;
    end;
    ListBoxEnterpriseYearsClick(Sender);
  end;
end;

procedure TDlgHubDatabase.UpdateUI;
const
  UP = 'Enter the database server User name and Password as provided'
    + ' by your database administrator. (See the database documentation)';
begin
  case JvPageList1.ActivePageIndex of
    0:
    begin
      case ComboBoxDBMS.ItemIndex of
        0: LKHeaderPaneDatabaseNotes.Comments.Text := 'You do not need to enter'
          + ' a Username or Password when using Firebird Embedded';
        1,2,
        3: LKHeaderPaneDatabaseNotes.Comments.Text :=
          'Enter the Database Server Name (as it appears in Windows Explorer)'
          + ' as host server. If you are unable to connect you may need to'
          + ' make an entry for it using Notepad in your C:\WINDOWS\system32\'
          + 'drivers\etc\hosts file. Alternatively you can enter the server'
          + ' network address (for example 192.168.1.2 or 10.0.0.4). '+ UP;
        else
          LKHeaderPaneDatabaseNotes.Comments.Text :=
            'Please select a database option from the drop-down list below.';
      end;
      case ComboBoxDBMS.ItemIndex of
        0:
        begin
          EditDatabaseHost.Text := 'Localhost';
          EditDatabaseHost.Enabled := False;
          ButtonNext.Enabled := True;
        end;
        else
        begin
          EditDatabaseHost.Enabled := True;
          if (EditDatabaseHost.Text = 'Localhost') then
            EditDatabaseHost.Text := '';
          ButtonNext.Enabled := (EditDatabaseHost.Text > '') and
           (EditDBUserName.Text > '') and (EditDatabasePassword.Text > '');
        end;
      end;
    end;
    1:
    begin
      ButtonNext.Enabled := (ListBoxEnterpriseCodes.ItemIndex > -1)
        and (ListBoxEnterpriseYears.ItemIndex > -1);
      SbAddEnterpriseCode.Enabled := (Length(EditEnterpriseCode.Text)=4);
      SbAddEnterpriseYear.Enabled := (Length(EditEnterpriseYear.Text)=4);
      if ButtonNext.Enabled then
        LabelDBFileName.Caption := SelectedEnterprise
          + SelectedEnterpriseYear + SelectedDBMSExtension
      else LabelDBFileName.Caption := '';
    end;
    2:
    begin
      ButtonNext.Enabled := MyHub.Active;
      if ButtonNext.Enabled then
        BitBtnTestConnection.Caption := 'Connected okay!'
      else
        BitBtnTestConnection.Caption := 'Test connection';
    end;
  end;
end;

procedure TDlgHubDatabase.BitBtnTestConnectionClick(Sender: TObject);
begin
  try
    MyHub.Open;
  except
    on E: Exception do
      Showmessage('There is a problem connecting: please check that your'
        + ' settings are correct. The error was: '+ E.Message);
  end;
   if MyHub.Active then
     ShowMessage('Success!! You have connected to your data.');
   UpdateUI;
end;

procedure TDlgHubDatabase.ButtonBackClick(Sender: TObject);
begin
  JvPageList1.ActivePageIndex := JvPageList1.ActivePageIndex - 1;
  JvPageList1Change(Sender);
  UpdateUI;
end;

procedure TDlgHubDatabase.ButtonNextClick(Sender: TObject);
begin
  if JvPageList1.ActivePageIndex = 2 then
    ModalResult := mrOK
  else
  begin
    JvPageList1.ActivePageIndex := JvPageList1.ActivePageIndex + 1;
    JvPageList1Change(Sender);
    UpdateUI;
  end;
end;

procedure TDlgHubDatabase.EditEnterpriseCodeChange(Sender: TObject);
begin
  if (Length(Trim(EditEnterpriseCode.Text)) = 4) then
    UpdateUI;
end;

procedure TDlgHubDatabase.EditEnterpriseYearChange(Sender: TObject);
var
  S: string;
begin
  S := Trim(EditEnterpriseYear.Text);
  if (Length(S) = 4) then
  begin
    StrToInt(S);
    UpdateUI;
  end;
end;

end.

