{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for showing a LKHubDiary                                          }
{                                                                             }
{*****************************************************************************}

unit LKHubDiaryDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, DBGrids, Grids, Calendar, StdCtrls, ComCtrls, Menus,
  DB, DBCtrls, ActnList, LKDefine, LKDlgs,
  LKFormBaseDlg, LKRecordsComponents, ImgList, LKDataCompBaseDlg,
  Buttons, ToolWin, StdActns;

const
  calendarStartday = 1;

type
  TFormLKHubDiary = class(TFormLKDataComponentBase)
    Panel2: TPanel;
    gb2: TGroupBox;
    Memo2: TMemo;
    gb1: TGroupBox;
    DBMemo1: TDBMemo;
    Splitter2: TSplitter;
    Splitter3: TSplitter;
    MonthCalendar1: TMonthCalendar;
    ActionDrillDownDiary: TAction;
    ActionShowWeekNumbers: TAction;
    ActionShowThingsToDo: TAction;
    ShowThingstodo1: TMenuItem;
    ShowThingstodo2: TMenuItem;
    procedure Memo2Enter(Sender: TObject);
    procedure ActionDrillDownDiaryExecute(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ActionShowWeekNumbersExecute(Sender: TObject);
    procedure MonthCalendar1Click(Sender: TObject);
    procedure ActionShowThingsToDoExecute(Sender: TObject);
    procedure DBMemo1Exit(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    MyAccountsComponent: TLKHubRecordsDiary;
  protected
    procedure GetSettings;
    procedure SaveSettings;
  public
    SelectedDate: TDateTime;
    procedure Construct(Sender: TLKHubRecordsDiary);
    procedure UpdateMemos;
  end;

var
  FormLKHubDiary: TFormLKHubDiary;

implementation

uses LKUtils;

{$R *.dfm}

procedure TFormLKHubDiary.Memo2Enter(Sender: TObject);
begin
  MonthCalendar1.Date := SelectedDate + 1;
  UpdateMemos;
end;

procedure TFormLKHubDiary.ActionDrillDownDiaryExecute(Sender: TObject);
begin
  //DataModuleLK.Hub.DrillDownDiary(Self);
end;

procedure TFormLKHubDiary.MonthCalendar1Click(Sender: TObject);
begin
  UpdateMemos;
end;

procedure TFormLKHubDiary.UpdateMemos;
begin
  SelectedDate := Trunc(MonthCalendar1.Date);
  gb1.Caption := 'Notes: ' + FormatDateTime('ddd, dd/mm/yy', SelectedDate);
  gb2.Caption := FormatDateTime('ddd, dd/mm/yy', SelectedDate + 1);
  Memo2.Lines.clear;
  with MyAccountsComponent do
    //Post;
    if Locate('DIARY_DATE',SelectedDate,[]) then
      Memo2.lines.Assign(TblMaster.FieldByName('THINGS_TO_DO'))
    else
    begin
      InsertRecord([SelectedDate]);
      ApplyUpdates(0);
    end;
end;

procedure TFormLKHubDiary.Construct(Sender: TLKHubRecordsDiary);
begin
  MyAccountsComponent := TLKHubRecordsDiary(Sender);
  inherited Construct(Sender);
  MonthCalendar1.Date := Date;
end;

procedure TFormLKHubDiary.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  SaveSettings;
  inherited;
end;

procedure TFormLKHubDiary.FormShow(Sender: TObject);
begin
  inherited;
  GetSettings;
  UpdateMemos;
end;

procedure TFormLKHubDiary.GetSettings;
var
  S: string;
  I: integer;

  function NextValue: string;
  var
    P: integer;
  begin
    result := '';
    for P := I to Length(S) do
      if S[P] = '|' then
        break
      else
        result := result + S[P];
    I := P + 1;
  end;

begin
  S := '40|494|424|290|336|98|Yes|Yes|Yes';
  S := MyAccountsComponent.RecallString('Settings', 'FormView', S);
  I := 1;
  Top := StrToInt(NextValue);
  Left := StrToInt(NextValue);
  Width := StrToInt(NextValue);
  Height := StrToInt(NextValue);
  gb1.Width := StrToInt(NextValue);
  Panel2.Height := StrToInt(NextValue);
  ActionStayOnTop.Checked := (NextValue = 'No');
    // allow for Action.AutoCheck on Execute;
  ActionStayOnTop.Execute;
  ActionShowWeekNumbers.Checked := (NextValue = 'No');
  ActionShowWeekNumbers.Execute;
  ActionShowThingsToDo.Checked := (NextValue = 'No');
  ActionShowThingsToDo.Execute;
end;

procedure TFormLKHubDiary.SaveSettings;
var
  S: string;
begin
  S := IntToStr(Top) + '|' + IntToStr(Left) + '|' + IntToStr(Width) + '|' +
    IntToStr(Height) + '|'
    + IntToStr(gb1.Width) + '|' + IntToStr(Panel2.Height) + '|'
    + BooleanToYesNo(ActionStayOnTop.Checked) + '|' +
      BooleanToYesNo(MonthCalendar1.WeekNumbers)
    + '|' + BooleanToYesNo(ActionShowThingsToDo.Checked);
  MyAccountsComponent.RememberString('Settings', 'FormView', S);
end;

procedure TFormLKHubDiary.ActionShowWeekNumbersExecute(Sender: TObject);
begin
  inherited;
  MonthCalendar1.WeekNumbers := ActionShowWeekNumbers.Checked;
end;

procedure TFormLKHubDiary.ActionShowThingsToDoExecute(Sender: TObject);
begin
  inherited;
  if ActionShowThingsToDo.Checked then
  begin
    with MonthCalendar1 do
    begin
      Align := alTop;
      Height := 153;
    end;
    if (Height < 290) then
      Height := 290;
    Panel2.Visible := true;
    Splitter2.Visible := true;
    MonthCalendar1.Align := alClient;
  end
  else
  begin
    Panel2.Visible := false;
    Splitter2.Visible := false;
  end
end;

procedure TFormLKHubDiary.DBMemo1Exit(Sender: TObject);
begin
  inherited;
  with MyAccountsComponent.TblMaster do
    if State in [dsEdit, dsInsert] then
    begin
      Post;
      ApplyUpdates(-1);
    end;
end;

end.

