{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for setting LKHub runtime users and passwords                     }
{                                                                             }
{*****************************************************************************}


unit LKHubEditUsersDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, System.UITypes,
  Graphics, Controls, Forms,
  Dialogs, LKFormBaseDlg, StdCtrls, ExtCtrls, ImgList, ActnList, ShellApi,
  LKRecordsComponents, StdActns, LKBaseComponent;

type
  TDlgLKHubEditUser = class(TFormLKBase)
    Panel1: TPanel;
    btnAdd: TButton;
    btnPasswd: TButton;
    btnDelete: TButton;
    btnClose: TButton;
    lbUserNames: TListBox;
    btnAdmin: TButton;
    LKHeaderPanel1: TLKHeaderPanel;
    procedure btnAddClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure btnPasswdClick(Sender: TObject);
    procedure btnAdminClick(Sender: TObject);
  private
    { Private declarations }
    FHub: TLKHubRecords;
  public
    { Public declarations }
    procedure Construct(AHub: TLKHubRecords);
  end;

var
  DlgLKHubEditUser: TDlgLKHubEditUser;

implementation

{$R *.dfm}

uses LKHubEnterPasswdDlg;

{ TDlgMtMACHubEditUser }

procedure TDlgLKHubEditUser.Construct(AHub: TLKHubRecords);
begin
  FHub := AHub;
  with lbUserNames do
  begin
    Items.AddStrings(FHub.Users);
    Sorted := true;
    ItemIndex := Count - 1;
  end;
end;

procedure TDlgLKHubEditUser.btnAddClick(Sender: TObject);
var
  Usr, Pass: string;
begin
  // allow default + Licence count users)
  {
  if lbUserNames.Count > FHub.License.UsersLicenced then
  begin
    Showmessage('You already have your licenced number of users. Please upgrade your licence!');
    ShellExecute(Self.Handle, 'open', 'file:///C:/MtMbis/bin/MtMRegister.htm',
      nil, nil, 1);
    exit;
  end;
  }
  with TFormLKHubEnterPasswd.Create(Application) do
  try
    if (ShowModal = mrOK) then
    begin
      Usr := Uppercase(eUserName.Text);
      Pass := ePasswd.Text;
      if (lbUserNames.Items.IndexOf(Usr) < 0) then
      begin
        FHub.UserAdd(Usr, Pass);
        lbUserNames.Items.Add(Usr);
      end
      else
        Showmessage('User name ' + Usr + ' already exists!');
    end;
  finally Free;
  end;
end;

procedure TDlgLKHubEditUser.btnDeleteClick(Sender: TObject);
var
  Usr: string;
begin
  with lbUserNames do
    if (ItemIndex < 0) then
      exit
    else
      Usr := Items[ItemIndex];
  if (MessageDlg('Confirm that you wish to delete ' + Usr + ' ?', mtInformation,
    [mbYes, mbNo], 0) = mrYes) then
    FHub.UserDelete(Usr);
end;

procedure TDlgLKHubEditUser.btnPasswdClick(Sender: TObject);
var
  Usr, Pass: string;
begin
  with lbUserNames do
    if (ItemIndex < 0) then
      exit
    else
      Usr := Items[ItemIndex];
  with TFormLKHubEnterPasswd.Create(Application) do
  try
    with eUserName do
    begin
      Text := Usr;
      ParentColor := true; ReadOnly := true;
    end;
    if (ShowModal = mrOK) then
    begin
      Pass := ePasswd.Text;
      with FHub.SystemSettings do
        RememberString('Users', Usr, Crypt(Pass, 25, True), True);
    end;
  finally Free;
  end;
end;

procedure TDlgLKHubEditUser.btnAdminClick(Sender: TObject);
begin
  inherited;
  FHub.DlgAdminPasswordSetExecute;
end;

end.

