{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for showing a LKHub end of year wizard                            }
{                                                                             }
{*****************************************************************************}

unit LKHubEndOfYearWizDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, LKHeaderDlg, ImgList, ActnList, StdCtrls, ExtCtrls,
  JvExControls, JvComponent,
  LKBaseComponent,LKRecordsComponents, LKDlgs, ComCtrls;

type
  TLKEndOfYearPanel = class(TLKComponentWizPanel)
  protected
    procedure DoButtonClick(Sender: TObject); override;
    procedure SetLKComponent(Value: TLKSystemComponent); override;
  end;

  TFormLKHubEndOfYearWizard = class(TFormLKHeaderDlg)
    Panel2: TPanel;
    StatusBar1: TStatusBar;
    procedure FormCreate(Sender: TObject);
  public
    procedure Construct(AHub : TLKHubRecords);
  end;

var
  FormLKHubEndOfYearWizard: TFormLKHubEndOfYearWizard;

implementation

{$R *.dfm}

{
****************************** TLKEndOfYearPanel ******************************
}
procedure TLKEndOfYearPanel.DoButtonClick(Sender: TObject);
begin
  inherited DoButtonClick(Sender);
  LKComponent.EndOfYear(StatusText);
end;

procedure TLKEndOfYearPanel.SetLKComponent(Value: TLKSystemComponent);
begin
  inherited SetLKComponent(Value);
end;

{
*************************** TFormLKEndOfYearWizard ****************************
}
procedure TFormLKHubEndOfYearWizard.Construct(AHub : TLKHubRecords);
var
  A: Integer;
  I: Integer;
  J: Integer;
  K: Integer;

  procedure AddEOYPanel(AComponent : TLKSystemComponent);
  begin
    if not (AComponent.HasPeriodEndProcedure
      or Assigned(AComponent.OnEndOfYear)) then
      Exit;
    Inc(A);
    with TLKEndOfYearPanel.Create(Self) do
    begin
      Parent := Panel2;
      Top := 60 + A*32;
      Align := alTop;
      LabelItemNo.Caption := IntToStr(A) + '.';
      LKComponent := AComponent;
      StatusText := StatusBar1.SimpleText;
    end;
  end;

begin
  LKHeaderPanel1.Caption := AHub.EnterpriseCurrent + ' '
   + AHub.EnterpriseAccountsYearCurrent;
  A := 0;
  with AHub.Groups do
    for I := 0 to Count-1 do
    with Items[I].GroupComponents do
    for J := 0 to Count-1 do
    begin
      with Items[J] do
        for K := 0 to ComponentCount-1 do
        if (Components[K] is TLKSystemComponent) then
          AddEOYPanel(TLKSystemComponent(Components[K]));
      AddEOYPanel(TLKSystemComponent(Items[J]));
    end;
end;

procedure TFormLKHubEndOfYearWizard.FormCreate(Sender: TObject);
begin
  inherited;
  Caption := 'Accounts End of Year Procedures';
  LKHeaderPanel1.Caption := 'Year end...';
  LKHeaderPanel1.Comments.Text :=
   'Procedures to remove old year data and audit trail. Once your new year'
    + ' database is cleaned up you can refer to the old data by opening'
    + ' the prior year database';
end;

end.
