{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for editing of a TLKHubRecords settings and properties            }
{                                                                             }
{*****************************************************************************}


unit LKHubEnterpriseDlg;

interface

uses Windows, SysUtils, Classes, System.UITypes,
  Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, registry, dialogs, LKDlgs, LKRecordsComponents, Grids,
  LKDefine, ComCtrls, LKHeaderDlg, JvComponent, JvBaseDlg,
  JvBrowseFolder, ImgList, ActnList, StdActns, LKBaseComponent, Mask, JvExMask,
  JvToolEdit, JvMaskEdit, JvCheckedMaskEdit, JvDatePickerEdit;

type
  TDlgLKHubEnterprise = class(TFormLKHeaderDlg)
    PageControl1: TPageControl;
    tsGeneral: TTabSheet;
    tsSequence: TTabSheet;
    Label5: TLabel;
    Label1: TLabel;
    Label4: TLabel;
    eAddress1: TEdit;
    eAddress2: TEdit;
    eAddress3: TEdit;
    eAddress4: TEdit;
    Label7: TLabel;
    ePostcode: TEdit;
    Label8: TLabel;
    eTelephone: TEdit;
    Label9: TLabel;
    eFax: TEdit;
    Label10: TLabel;
    eEmail: TEdit;
    eEnterpriseName: TEdit;
    tsFolders: TTabSheet;
    ckbxHardDiskBackup: TCheckBox;
    eBackupFolder: TEdit;
    BtnBrowseHDBackup: TButton;
    ckbxRemovable: TCheckBox;
    eBackupRemovable: TEdit;
    BtnBrowseRDBackup: TButton;
    tsOptions: TTabSheet;
    ckbxSharePrintLayouts: TCheckBox;
    SelectFolderDlg: TJvBrowseForFolderDialog;
    eFolderStoreDocumentImages: TEdit;
    BtnBrowseImageFolder: TButton;
    Label15: TLabel;
    procedure BtnApplyClick(Sender: TObject);
    procedure eAddress1Change(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ckbxHardDiskBackupClick(Sender: TObject);
    procedure ckbxRemovableClick(Sender: TObject);
    procedure BtnBrowseHDBackupClick(Sender: TObject);
    procedure BtnBrowseRDBackupClick(Sender: TObject);
    procedure BtnBrowseImageFolderClick(Sender: TObject);
  private
    { Private declarations }
    FSeqEdits: array[0..100] of TEdit;
    FSeqCheckBox: array[0..100] of TCheckBox;
    FSeqCount: integer;
    FHub: TLKHubRecords;
  protected
    FHoldHDPath, FHoldRDPath: string;
    function GetTheFolder: string;
    procedure GetEnterpriseInfo; virtual;
    procedure SaveEnterpriseInfo; virtual;
  public
    { Public declarations }
    procedure Construct(AHub: TLKHubRecords);
  end;

var
  DlgLKHubEnterprise: TDlgLKHubEnterprise;

implementation

uses LKutils;

{$R *.DFM}

procedure TDlgLKHubEnterprise.FormCreate(Sender: TObject);
begin
  FSeqCount := 0;
end;

procedure TDlgLKHubEnterprise.Construct(AHub: TLKHubRecords);
begin
  FHub := AHub;
  tsFolders.Visible := (FHub.DBMS = sdbLocalFirebird);
  GetEnterpriseInfo;
  btnApply.Enabled := False;
  btnOK.Enabled := False;
end;

procedure TDlgLKHubEnterprise.GetEnterpriseInfo;
var
  I: Integer;
begin
  with FHub do
  begin
    LKHeaderPanel1.Caption :=  'Database: ' + EnterpriseCurrent +
        EnterpriseAccountsYearCurrent;
    for I := 0 to FSeqCount - 1 do
    begin
      FSeqEdits[I].Free;
      FSeqCheckBox[I].Free;
    end;
    eEnterpriseName.Text := EnterpriseName;
    eAddress1.Text := EnterpriseAddress1;
    eAddress2.Text := EnterpriseAddress2;
    eAddress3.Text := EnterpriseAddress3;
    eAddress4.Text := EnterpriseAddress4;
    ePostcode.Text := EnterpriseAddressPostcode;
    eTelephone.Text := EnterpriseAccountsTelephone;
    eFax.Text := EnterpriseAccountsFax;
    eEmail.Text := EnterpriseAccountsEmail;
    FSeqCount := Records.Count;
    if (DBMS = sdbLocalFirebird) then
    begin
      ckbxHardDiskBackup.Checked := DoBackup;
      ckbxHardDiskBackupClick(self);
      ckbxRemovable.checked := DoBackupRemovable;
      ckbxRemovableClick(self);
      eBackupFolder.Text := FolderBackup;
      eBackupRemovable.Text := FolderBackupRemovable;
    end;
    eFolderStoreDocumentImages.Text := FolderStoreDocumentImages;
    for I := 0 to FSeqCount - 1 do
      with Records[I] do
      begin
        FSeqEdits[I] := TEdit.Create(self);
        with FSeqEdits[I] do
        begin
          parent := tsSequence;
          //caption := NameFriendly;
          left := 208;
          width := 48;
          borderstyle := bsNone;
          height := 15;
          top := 56 + I * 16;
          Text := intToStr(FHub.SequenceGet(NameFriendly));
          OnChange := eAddress1Change;
        end;
        FSeqCheckBox[I] := TCheckBox.Create(self);
        with FSeqCheckBox[I] do
        begin
          parent := tsSequence;
          height := 15;
          width := 16;
          left := 184;
          top := 56 + I * 16;
          Checked := SequenceActive;
          OnClick := eAddress1Change;
        end;
        with TLabel.Create(self) do
        begin
          parent := tsSequence;
          caption := NameFriendly;
          left := 24;
          top := 56 + I * 16;
          tag := 1;
          focusControl := FSeqEdits[I];
        end;
      end;
    ckbxSharePrintLayouts.Checked := PrintShareLayouts;
  end;
end;

procedure TDlgLKHubEnterprise.SaveEnterpriseInfo;
var
  I: integer;

  function CheckFolder(const Value: string): string;
  begin
    if Value = '' then
      raise LKError.Create('All folders must be assigned before closing');
    if not DirectoryExists(value) then
    begin
      if (messageDlg('Directory does not exist!' + chr(10) + 'Create directory '
        +
        value + ' now?', mtWarning, [mbYes, mbNo], 0) = mrYes) then
        ForceDirectories(value)
      else
        Showmessage('Folder name ' + value + ' is invalid');
    end;
    result := value;
  end;

begin
  with FHub do
  begin
    if (DBMS = sdbLocalFirebird) then
    begin
      DoBackup := ckbxHardDiskBackup.Checked;
      DoBackupRemovable := ckbxRemovable.Checked;
      FolderBackup := CheckFolder(eBackupFolder.Text);
      FolderBackupRemovable := eBackupRemovable.Text;
    end;
    FolderStoreDocumentImages := CheckFolder(eFolderStoreDocumentImages.Text);
    EnterpriseName := eEnterpriseName.Text;
    EnterpriseAddress1 := eAddress1.Text;
    EnterpriseAddress2 := eAddress2.Text;
    EnterpriseAddress3 := eAddress3.Text;
    EnterpriseAddress4 := eAddress4.Text;
    EnterpriseAddressPostcode := ePostcode.Text;
    EnterpriseAccountsTelephone := eTelephone.Text;
    EnterpriseAccountsFax := eFax.Text;
    EnterpriseAccountsEmail := eEmail.Text;
    // write the sequence numbers
    for I := 0 to FSeqCount - 1 do
      with Records[I] do
      begin
        SequenceSet(NameFriendly, strToInt(FSeqEdits[I].Text));
        SequenceActive := FSeqCheckBox[I].Checked;
      end;
    PrintShareLayouts := ckbxSharePrintLayouts.Checked;
  end;
end;

procedure TDlgLKHubEnterprise.BtnApplyClick(Sender: TObject);
begin
  BtnApply.Enabled := False;
  SaveEnterpriseInfo;
  btnOK.Enabled := true;
  btnCancel.Enabled := False;
end;

procedure TDlgLKHubEnterprise.eAddress1Change(Sender: TObject);
begin
  btnOK.Enabled := False;
  btnApply.Enabled := True;
end;

procedure TDlgLKHubEnterprise.ckbxHardDiskBackupClick(Sender: TObject);
begin
  btnApply.Enabled := true;
  with eBackupFolder do
    if ckbxHardDiskBackup.checked then
    begin
      Color := clWindow;
      Text := FHoldHDPath;
    end
    else
    begin
      Color := clInactiveCaptionText;
      Text := '';
    end;
end;

procedure TDlgLKHubEnterprise.ckbxRemovableClick(Sender: TObject);
begin
  btnApply.Enabled := true;
  with eBackupRemovable do
    if ckbxRemovable.checked then
    begin
      Color := clWindow;
      Text := FHoldRDPath;
    end
    else
    begin
      Color := clInactiveCaptionText;
      Text := '';
    end;
end;

procedure TDlgLKHubEnterprise.BtnBrowseHDBackupClick(Sender: TObject);
begin
  eBackupFolder.Text := GetTheFolder;
end;

procedure TDlgLKHubEnterprise.BtnBrowseRDBackupClick(Sender: TObject);
begin
  eBackupRemovable.Text := GetTheFolder;
end;

procedure TDlgLKHubEnterprise.BtnBrowseImageFolderClick(Sender: TObject);
begin
  inherited;
  eFolderStoreDocumentImages.Text := GetTheFolder;
end;

function TDlgLKHubEnterprise.GetTheFolder: string;
begin
  Result := InstallFolder;
  if SelectFolderDlg.Execute then
    Result := SelectFolderDlg.Directory;
  if (Result[Length(Result)] = PathDelim) then
    System.Delete(Result, Length(Result), 1);
end;

end.

