{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      ImageIndex component editor dialog                                       }
{                                                                             }
{*****************************************************************************}


unit LKImageIndexEditor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ComCtrls, ImgList, StdCtrls, ExtCtrls, Buttons;

type
  TDlgLKImageIndexEditor = class(TForm)
    Bevel1: TBevel;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    ComboBox1: TComboBox;
    Image1: TImage;
    ImageList1: TImageList;
    ListView1: TListView;
    Panel1: TPanel;
    procedure BitBtn2Click(Sender: TObject);
    procedure ComboBox1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ListView1DblClick(Sender: TObject);
    procedure ListView1SelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
  private
    FFormCaptionName: string;
    FImageIndex: integer;
    procedure SetFormCaptionName(const Value: string);
    procedure SetImageIndex(const Value: integer);
    { Private declarations }
  public
    { Public declarations }
    property FormCaptionName: string read FFormCaptionName write SetFormCaptionName;
    property ImageIndex: integer read FImageIndex write SetImageIndex;
  end;

var
  DlgLKImageIndexEditor: TDlgLKImageIndexEditor;

implementation

{$R *.DFM}

{ TImageIndexForm }

procedure TDlgLKImageIndexEditor.BitBtn2Click(Sender: TObject);
begin
  FImageIndex := -1;
  ModalResult := mrOK;
end;

procedure TDlgLKImageIndexEditor.ComboBox1Click(Sender: TObject);
begin
  with ComboBox1 do
    ListView1.ViewStyle := TViewStyle(Items.Objects[ItemIndex]);
end;

procedure TDlgLKImageIndexEditor.FormCreate(Sender: TObject);
begin
  FImageIndex := -1;
end;

procedure TDlgLKImageIndexEditor.FormShow(Sender: TObject);
var
  I: integer;
  ListItem: TListItem;
begin
  // Create a ListView item for each image in the ImageList
  with ListView1 do
  begin
    SmallImages := ImageList1;
    LargeImages := ImageList1;
    for I := 0 to ImageList1.Count - 1 do
    begin
      ListItem := Items.Add;
      Listitem.Caption := 'Image ' + IntToStr(I);
      ListItem.ImageIndex := I;
    end;
    // Add View styles and constants to the Combo Box
    ComboBox1.Items.AddObject('vsIcon', TObject(vsIcon));
    ComboBox1.Items.AddObject('vsList', TObject(vsList));
    ComboBox1.Items.AddObject('vsReport', TObject(vsReport));
    ComboBox1.Items.AddObject('vsSmallIcon', TObject(vsSmallIcon));
    // Display first item in the Combo Box
    ComboBox1.ItemIndex := 0;
    //Focus image in list
    if (fImageIndex < -1) or (fImageIndex > ImageList1.Count-1) then
      begin
        ListView1.Selected := nil;
        fImageIndex := -1;
      end
     else ListView1.Selected :=
            ListView1.Items.Item[fImageIndex];
  end;
end;

procedure TDlgLKImageIndexEditor.ListView1DblClick(Sender: TObject);
begin
  ModalResult := mrOK;
end;

procedure TDlgLKImageIndexEditor.ListView1SelectItem(Sender: TObject;
  Item: TListItem; Selected: Boolean);
begin
  if Selected then
    begin
      BitBtn1.Caption := 'Use '+ Item.Caption;
      BitBtn1.Enabled := true;
      ImageList1.GetIcon(Item.ImageIndex, Image1.Picture.Icon );
      FImageIndex := Item.ImageIndex;
    end;
end;

procedure TDlgLKImageIndexEditor.SetFormCaptionName(const Value: string);
begin
  FFormCaptionName := Value;
  Caption := Value+'.ImageIndex Property Selector';
end;

procedure TDlgLKImageIndexEditor.SetImageIndex(const Value: integer);
begin
  FImageIndex := Value;
end;

end.
