{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for showing a TLKLedgerBase record                                }
{                                                                             }
{*****************************************************************************}

unit LKLedgerBaseRecordDlg;

interface

uses
  Windows, Messages, SysUtils, Variants,
  Classes, System.Contnrs, System.Actions,
  Graphics, Controls, Forms,
  Dialogs, LKDataComponentRecordDlg, ImgList, Menus, ActnList, DB, StdCtrls,
  ComCtrls, DBCtrls, Buttons, ExtCtrls, ToolWin, LKRecordsComponents, LKdlgs,
  StdActns, LKDBControls, LKUtilComponents, Grids, DBGrids,
  JvExDBGrids, JvDBGrid, JvDBUltimGrid;

type
  TFormLKLedgerBaseRecord = class(TFormLKDataComponentRecord)
    ActionNewInputDocument: TAction;
    pmNewDocPopup: TPopupMenu;
    ActionNewInputDocPopup: TAction;
    sbNewDocPopup: TToolButton;
    procedure ActionNewInputDocumentExecute(Sender: TObject);
    procedure ActionNewInputDocPopupExecute(Sender: TObject);
  private
    { Private declarations }
    FMyLedger: TLKLedgerBase;
  protected
    procedure SetHideFields; override;
  public
    { Public declarations }
    procedure Construct(Sender: TLKLedgerBase); 
  end;

var
  FormLKLedgerBaseRecord: TFormLKLedgerBaseRecord;

implementation

{$R *.dfm}

procedure TFormLKLedgerBaseRecord.Construct(Sender: TLKLedgerBase);
var
  I, X, L: Integer;
  S: string;
  NewAction: TAction;
  NewMenuitem: TMenuItem;
begin
  FMyLedger := Sender;
  inherited Construct(Sender);
  with FMyLedger do
  begin
    with ChildComponents do
      if (Count > 0) then
      begin
        ToolBar1.Visible := True;
        I := 0;
        for X := 0 to Count - 1 do
          if (TComponent(ChildComponents[X]) is TLKRecordsPosting)
            then
            with TLKRecordsPosting(ChildComponents[X]) do
              if Assigned(Daybook) then
                Inc(I);
        sbNewDocPopup.Visible := (I > 1); // Add to popup speed button if many
        for X := 0 to Count - 1 do
          if (TComponent(ChildComponents[X]) is TLKRecordsPosting)
            then
            with TLKRecordsPosting(ChildComponents[X]) do
            begin
              NewAction := TAction.Create(Self);
              with NewAction do
              begin
                ActionList := ActionListAcsComp;
                S := NameFriendly;
                if S[length(S)] = 's' then
                  System.Delete(S, length(S), 1);
                Caption := S;
                Hint := 'Enter new document';
                ImageIndex := 14;
                Tag := X;
                OnExecute := ActionNewInputDocumentExecute;
              end;
              NewMenuitem := TMenuitem.create(miActivities);
              with NewMenuitem do
              begin
                Action := NewAction;
                Tag := X;
              end;
              miActivities.Add(NewMenuitem);
              NewMenuitem := TMenuItem.Create(PopupMenu1);
              with NewMenuitem do
              begin
                Action := NewAction;
                Tag := X;
              end;
              PopupMenu1.Items.Add(NewMenuitem);
              if sbNewDocPopup.Visible then
              begin
                NewMenuitem := TMenuitem.create(pmNewDocPopup);
                with NewMenuitem do
                begin
                  Action := NewAction;
                  Tag := X;
                end;
                pmNewDocPopup.Items.Add(NewMenuitem);
              end
              else
              begin
                with ToolBar1.Buttons[ToolBar1.ButtonCount - 1] do
                  L := Left + Width + 1;
                with TToolButton.Create(Self) do
                begin
                  Left := L;
                  Action := NewAction;
                  Parent := toolbar1;
                end;
              end
            end;
      end;
  end;
end;

procedure TFormLKLedgerBaseRecord.ActionNewInputDocumentExecute(
  Sender: TObject);
begin
  inherited;
  TLKDaybookBase(
    FMyLedger.ChildComponents[TComponent(Sender).Tag]).RecordNew;
end;

procedure TFormLKLedgerBaseRecord.ActionNewInputDocPopupExecute(
  Sender: TObject);
begin
  inherited;
  pmNewDocPopup.Popup(Mouse.CursorPos.X, Mouse.CursorPos.Y);
end;

procedure TFormLKLedgerBaseRecord.SetHideFields;
begin
  inherited;
  HideDetailFields := HideDetailFields + ';URN';
end;

end.

