{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for showing a TLKRecordsASCII                                     }
{                                                                             }
{*****************************************************************************}

unit LKRecordsASCIIDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, System.UITypes,
  Graphics, Controls, Forms, Dialogs,
  LKRecordsBaseDlg, DBCtrls, LKdlgs, ImgList, Menus, ActnList, Db, DBTables,
  ComCtrls, Grids, DBGrids, StdCtrls, Buttons, ExtCtrls, ToolWin, StdActns,
  JvExExtCtrls, JvNetscapeSplitter, JvExDBGrids, JvDBGrid, JvDBUltimGrid,
  LKDefine, LKDBControls, LKBaseComponent, LKRecordsComponents, LKUtils,
  LKUtilComponents;

type
  TFormLKRecordsASCII = class(TFormLKRecordsBase)
    TextFileOpenDialog: TOpenDialog;
    ImportTextfile1: TMenuItem;
    ExportTextfile1: TMenuItem;
    SetProcessedvalues1: TMenuItem;
    aImportText: TAction;
    aExportText: TAction;
    aSetProcessed: TAction;
    ViewDialogInteger: TLKViewDialog;
    procedure aImportTextExecute(Sender: TObject);
    procedure aExportTextExecute(Sender: TObject);
    procedure aSetProcessedExecute(Sender: TObject);
  private
    { Private declarations }
    FMyAccountsComponent: TLKRecordsASCII;
  public
    { Public declarations }
    procedure Construct(Sender: TLKRecordsASCII);
  end;

implementation

{$R *.DFM}

procedure TFormLKRecordsASCII.Construct(Sender: TLKRecordsASCII);
begin
  FMyAccountsComponent := Sender;
  inherited Construct(Sender);
end;

procedure TFormLKRecordsASCII.aImportTextExecute(Sender: TObject);
var
  MasterTextFileName, DetailTextFileName: string;
  Mf, Df: TLKFieldNumbers;
  X: integer;
begin
  MasterTextFileName := '';
  DetailTextFileName := '';
  with FMyAccountsComponent do
  begin
    with TextFileOpenDialog do
    begin
      Title := 'Select Master Table import file';
      if Execute then
        MasterTextFileName := FileName;
    end;
    if TblDetailUsed then
      with TextFileOpenDialog do
      begin
        Title := 'Select Detail Table import file';
        if execute then
          detailTextFileName := FileName;
      end;
    if TblMaster.findFirst then
      if (MessageDlg('Do you want to empty the Tables before import?',
        mtinformation,
        [mbYes, mbNo], 0) = mrYes) then
      begin
        with TblMaster do
        begin
          Close;
          EmptyTable;
          Open;
        end;
        if TblDetailUsed then
          with TblDetail do
          begin
            Close;
            EmptyTable;
            Open;
          end;
      end;
    Mf := [];
    Df := [];
    with TblMaster do
      for X := 0 to FieldCount - 1 do
        if Fields[X].visible then
          Mf := Mf + [X];
    if TblDetailUsed then
      with TblDetail do
        for X := 0 to FieldCount - 1 do
          if Fields[X].visible then
            Df := Df + [X];
    ImportData(MasterTextFileName, DetailTextFileName, Mf, Df);
  end;
end;

procedure TFormLKRecordsASCII.aExportTextExecute(Sender: TObject);
var
  MasterTextFileName, DetailTextFileName: string;
begin
  inherited;
  MasterTextFileName := '';
  DetailTextFileName := '';
  with FMyAccountsComponent do
  begin
    if not TblMaster.findFirst then
      raise LKError.Create('No records found to export!');
    {
     with ViewDialog do
      begin
      Title := 'Enter full Master Table export file name';
      if execute then masterTextFileName := Value;
      end;
     if TblDetailUsed then with ViewDialog do
      begin
      Title := 'Enter full Detail Table export file name';
      if execute then detailTextFileName := Value;
      end;
     mf := [];
     df := [];
     with TblMaster do
      for x := 0 to fieldcount -1 do if fields[x].visible then mf := mf + [x];
     if TblDetailUsed then with TblDetail do
      for x := 0 to fieldcount -1 do if fields[x].visible then df := df + [x];
     exportData(masterTextFileName,DetailTextFileName,mf,df);

     }
  end;
end;

procedure TFormLKRecordsASCII.aSetProcessedExecute(Sender: TObject);
var
  Setting: integer;
begin
  inherited;
  with FMyAccountsComponent.TblMaster do
  begin
    if (FindField('Batch') = nil) then
      raise LKError.Create('Cannot find field name Batch');
    if not FindFirst then
      Exit;
    with ViewDialogInteger do
    begin
      if not Execute then
        Exit;
      Setting := Value;
    end;
    screen.Cursor := crHourglass;
    try
      if (Pos('[BATCH]', filter) > 0) and filtered then
      begin
        while FindFirst do
        begin
          if (fieldByName('Batch').asInteger = Setting) then
            raise LKError.Create('Batch value for record is already set!');
          Edit;
          FieldByName('BATCH').asInteger := Setting;
          Post;
          Application.ProcessMessages;
        end;
      end
      else if FindFirst then // not filtered on Batch field
        repeat
          Edit;
          FieldByName('BATCH').asInteger := Setting;
          Post;
          Application.ProcessMessages;
        until not FindNext;
    finally
      Screen.Cursor := crDefault;
    end;
  end;
end;

end.

