{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for showing a TLKRecordsInput record                              }
{                                                                             }
{*****************************************************************************}

unit LKRecordsInputRecordDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, System.UITypes,
  Graphics, Controls, Forms,
  Dialogs, Menus, ImgList, ActnList, DB, StdCtrls, ComCtrls, DBCtrls, Buttons,
  ExtCtrls, ToolWin, Grids, DBGrids, JvExDBGrids, JvDBGrid, JvDBUltimGrid, Math,
  LKDefine, LKUtils, LKUtilComponents, LKRecordsComponents, LKdlgs,
  LKDBControls, LKRecordsBaseRecordDlg, StdActns;

type
  TFormLKRecordsInputRecord = class(TFormLKRecordsBaseRecord)
    ActionDetailEditsProperties: TAction;
    ActionItemEnter: TAction;
    ActionItemExit: TAction;
    ActionMasterLastEditExit: TAction;
    ActionNextItem: TAction;
    ActionPostEditDetailDB: TAction;
    Copy2: TMenuItem;
    DeisplayProperties1: TMenuItem;
    DisplayProperties2: TMenuItem;
    Paste2: TMenuItem;
    pmPanelDetailEdits: TPopupMenu;
    SplitterDetailEdits: TSplitter;
    SplitterMasterEdits: TSplitter;
    DetailEditList: TLKDcDBEditsPanel;
    PanelNotes: TPanel;
    ActionNextItem1: TMenuItem;
    MiDetailEditsEditBreak: TMenuItem;
    procedure ActionDetailEditsPropertiesExecute(Sender: TObject);
    procedure ActionItemEnterExecute(Sender: TObject);
    procedure ActionItemExitExecute(Sender: TObject);
    procedure ActionMasterEditsPropertiesExecute(Sender: TObject);
    procedure ActionMasterLastEditExitExecute(Sender: TObject);
    procedure ActionNextItemExecute(Sender: TObject);
    procedure ActionPostEditDetailDBExecute(Sender: TObject);
    procedure DBNavigator1Click(Sender: TObject; Button: TNavigateBtn);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
    procedure SplitterDetailEditsMoved(Sender: TObject);
    procedure SplitterMasterEditsMoved(Sender: TObject);
    procedure ActionButtonOKExecute(Sender: TObject);
  private
    FMyRecordsInput: TLKRecordsInput;
    procedure DBEDLastKeyPress(Sender: TObject; var Key: Char);
  protected
    DBEDLast: TWinControl;
    DBEDocumentNo: TWinControl;
    DBEItemNo: TWinControl;
    DBEMLast: TWinControl;
    procedure CustomiseCommonEdits(
      EditList: TLKDcDBEditsPanel; Data: TLKTable);
    procedure CustomiseMasterEdits; override;
    procedure CustomiseDetailEdits; override;
    procedure PlaceEditControls; override;
    procedure SetDetailEditsActions; virtual;
    procedure SetHideFields; override;
    procedure SetMasterEditsActions; override;
  public
    procedure Construct(Sender: TLKRecordsInput);
    procedure SetFolderStoreDocumentImages(const Value: string); override;
  end;

var
  FormLKRecordsInputRecord: TFormLKRecordsInputRecord;

implementation

{$R *.dfm}

{
************************* TFormLKDocumentsInputRecord *************************
}

procedure TFormLKRecordsInputRecord.ActionButtonOKExecute(Sender: TObject);
begin
  with FMyRecordsInput do
    if (TblDetail.State = dsInsert) then
      if (DBEItemNo is TDBEdit) then
        if TDBEdit(DBEItemNo).Field.IsNull then
          TblDetail.Cancel;
  inherited;
end;

procedure TFormLKRecordsInputRecord.ActionDetailEditsPropertiesExecute(Sender:
  TObject);
begin
  inherited;
  if Assigned(DetailEditList) then
    if (DetailEditList.DlgEditListPropsExecute(Self) = mrOK) then
    begin
      DetailEditList.SaveFormat(FMyRecordsInput, 'FormViewRecord',
        'dbDetailEdits');
      SetDetailEditsActions;
      Panel1.Height :=
        4 + Max(DetailEditList.DisplayHeight, MasterEditList.DisplayHeight);
    end;
end;

procedure TFormLKRecordsInputRecord.ActionItemEnterExecute(Sender: TObject);
begin
  // override if required
end;

procedure TFormLKRecordsInputRecord.ActionItemExitExecute(Sender: TObject);
begin
  //override as required
end;

procedure TFormLKRecordsInputRecord.ActionMasterEditsPropertiesExecute(Sender:
  TObject);
begin
  inherited;
  if Assigned(DetailEditList) then
    Panel1.Height :=
     4 + Max(MasterEditList.DisplayHeight, DetailEditList.DisplayHeight)
  else
    Panel1.Height := 4 + MasterEditList.DisplayHeight;
end;

procedure TFormLKRecordsInputRecord.ActionMasterLastEditExitExecute(Sender:
  TObject);
begin
  with FMyRecordsInput do
    if TblMaster.State in [dsEdit, dsInsert] then
    begin
      Post;
      if TblDetailUsed then
      begin
        TblDetail.Append;
        FocusControl(DBEItemNo);
      end;
    end;
end;

procedure TFormLKRecordsInputRecord.ActionNextItemExecute(Sender: TObject);
begin
  inherited;
  with FMyRecordsInput.TblDetail do
    if (State in [dsInsert, dsEdit]) then
      Post;
//  if (ActiveControl = btnOK) then
//    Exit;
  if (MessageDlg('Another item?', mtInformation, [mbYes, mbNo], 0) = mrYes) then
  begin
    FMyRecordsInput.TblDetail.Append;
    FocusControl(DBEItemNo);
  end
  else
    FocusControl(btnOK);
end;

procedure TFormLKRecordsInputRecord.ActionPostEditDetailDBExecute(Sender:
  TObject);
begin
  try
    with FMyRecordsInput.TblDetail do
      if State in [dsInsert, dsEdit] then
      begin
        Post;
        Edit;
      end;
  except;
  end;
end;

procedure TFormLKRecordsInputRecord.Construct(Sender: TLKRecordsInput);
begin
  FMyRecordsInput := Sender;
  inherited Construct(Sender);
end;

procedure TFormLKRecordsInputRecord.CustomiseCommonEdits(
  EditList: TLKDcDBEditsPanel; Data: TLKTable);
var
  I: Integer;
  S: string;
begin
  with EditList do
    for I := 0 to Count - 1 do
      with Items[I] do
      begin
        if (Pos('IMAGE', FieldName) > 0) then
          ControlType := dbcImageFile;
        with FMyRecordsInput do
        begin
          S := RecallString(Data.TableName, FieldName, '', True);
          if (S > '') then
          begin
            if (S = 'DATE_') then
              ControlType := dbcDateEdit;
            if (S = 'TIME_') then
              ControlType := dbcTimeEdit;
            {
            if (S = 'COMBOBOX') then
            begin
              ControlType := dbcComboBox;
              TDBComboBox(EditControl).Items.Text :=
                RecallString(Data.TableName, FieldName + 'ComboBox', '',
                  True);
            end;
            }
            if (S = 'FILE') then
              ControlType := dbcFilePath;
            if (S = 'URL') then
              ControlType := dbcURL;
            if (S = 'EMAIL') then
              ControlType := dbcEMail;
          end;
        end;
      end;
end;

procedure TFormLKRecordsInputRecord.CustomiseDetailEdits;
begin
  inherited;
  if Assigned(DetailEditList) then
    CustomiseCommonEdits(DetailEditList, FMyRecordsInput.TblDetail);
end;

procedure TFormLKRecordsInputRecord.CustomiseMasterEdits;
begin
  inherited;
  CustomiseCommonEdits(MasterEditList, FMyRecordsInput.TblMaster);
end;

procedure TFormLKRecordsInputRecord.DBNavigator1Click(Sender: TObject;
  Button: TNavigateBtn);
begin
  if (Button = nbInsert) then
    DBEItemNo.SetFocus;
end;

procedure TFormLKRecordsInputRecord.FormClose(Sender: TObject; var Action:
  TCloseAction);
begin
  if Assigned(DetailEditList) then
    DetailEditList.DoOnFormClose;
  inherited;
end;

procedure TFormLKRecordsInputRecord.DBEDLastKeyPress(Sender: TObject;
  var Key: Char);
begin
  if (Key = Chr(VK_RETURN)) then
    ActionNextItem.OnExecute(Sender);
end;

procedure TFormLKRecordsInputRecord.FormShow(Sender: TObject);
begin
  inherited;
  SplitterDetailEdits.Left := PanelNotes.Left;
  if Assigned(DetailEditList) then
  begin
    Panel1.Height :=
      4 + Max(MasterEditList.DisplayHeight, DetailEditList.DisplayHeight);
    SplitterMasterEdits.Left := MasterEditList.Width;
  end
  else Panel1.Height := 4 + MasterEditList.DisplayHeight;
  DBEDocumentNo.SetFocus;
end;

procedure TFormLKRecordsInputRecord.PlaceEditControls;
var
  MasterEditsLayout: TLKEditsLayout;
  DetailEditsLayout: TLKEditsLayout;
begin
  DetailEditsLayout := aceOneCol;
  with FMyRecordsInput do
  begin
    if TblDetailUsed then
    begin
      MasterEditsLayout := aceOneCol;
      MasterEditList.Width :=
        RecallInteger('FormViewRecord', 'pMasterEditsWidth', 250, False);
      DetailEditList.Width :=
        RecallInteger('FormViewRecord', 'pDetailEditsWidth', 250, False);
    end
    else
    begin
      MasterEditsLayout := aceTwoCol;
      MasterEditList.Width :=
        RecallInteger('FormViewRecord', 'pMasterEditsWidth', 500, False);
      DetailEditList.Free;
      SplitterMasterEdits.Free;
    end;
    with MasterEditList do
    begin
      ConstructFromDatasource(HideMasterFields);
      CustomiseMasterEdits;
      ReadOnly := not Editing;
      Layout := MasterEditsLayout;
      LoadFormat(FMyRecordsInput, 'FormViewRecord', 'dbMasterEdits');
      SetMasterEditsActions;
    end;
    if TblDetailUsed then
    begin
      DetailEditList.ConstructFromDatasource(TblDetail.MasterFields);
      CustomiseDetailEdits;
      DetailEditList.Layout := DetailEditsLayout;
      DetailEditList.ReadOnly := not Editing;
      DetailEditList.LoadFormat(
        FMyRecordsInput, 'FormViewRecord', 'dbDetailEdits');
      SetDetailEditsActions;
    end;
    Resize;
    CreatedbGridDetail;
    with dbGridDetail do
    begin
      ReadOnly := True;
      TabStop := False;
      try
        LoadFormat(FMyRecordsInput, 'FormViewRecord', 'dbGridDetail');
      except ClearFormat(FMyRecordsInput, 'FormViewRecord',
        'dbGridDetail');
      end;
    end;
  end;
end;

procedure TFormLKRecordsInputRecord.SetDetailEditsActions;
var
  I: Integer;
begin
  DBEItemNo := nil;
  DBEDLast := nil;
  if Assigned(DetailEditList) then
    with DetailEditList do
    begin
      for I := 0 to Count - 1 do
        with Items[I] do if Visible then
          if (EditControl is TWinControl) then
          begin
            DBEItemNo := TWinControl(EditControl);
            OnEnter := ActionItemEnter.OnExecute;
            OnExit := ActionItemExit.OnExecute;
            Break;
          end;
      for I := Count - 1 downto 0 do
        with Items[I] do
          if Visible and not ReadOnly then
          if (EditControl is TWinControl) then
          begin
            DBEDLast := TWinControl(EditControl);
            // OnExit := ActionNextItem.OnExecute;
            OnKeyPress := DBEDLastKeyPress;
            Break;
          end;
    end;
end;

procedure TFormLKRecordsInputRecord.SetFolderStoreDocumentImages(const Value
  : string);
var
  I: Integer;
begin
  inherited;
  if Assigned(DetailEditList) then
  with DetailEditList do
    for I := 0 to Count - 1 do
      with Items[I] do
        if (ControlType = dbcImageFile) then
          TLKDBImageEdit(Items[I].EditControl).ImageFilePath := Value;
end;

procedure TFormLKRecordsInputRecord.SetHideFields;
begin
  inherited;
  if (HideMasterFields > ' ') then
    HideMasterFields := HideMasterFields + ';';
  HideMasterFields := HideMasterFields + 'BATCH';
end;

procedure TFormLKRecordsInputRecord.SetMasterEditsActions;
var
  I: Integer;
begin
  inherited; // does nothing but just in case
  with FMyRecordsInput do
    if Editing then
      with MasterEditList do
        for I := 0 to Count - 1 do
          with Items[I] do
            if (MSumDPairFieldNames.IndexOf(FieldName) >= 0) then
              ReadOnly := True;
  DBEDocumentNo := nil;
  with MasterEditList do
  begin
    for I := 0 to Count - 1 do
      with Items[I] do
        if Visible and (EditControl is TWinControl) then
        begin
          DBEDocumentNo := TWinControl(EditControl);
          OnExit := ActionPostEditMasterDB.OnExecute;
          Break;
        end;
    DBEMLast := DBEDocumentNo;
    for I := Count - 1 downto 0 do
      with Items[I] do
        if Visible and not ReadOnly and (EditControl is TWinControl) then
        begin
          DBEMLast := TWinControl(EditControl);
          OnExit := ActionMasterLastEditExit.OnExecute;
          Break;
        end;
  end;
end;

procedure TFormLKRecordsInputRecord.SplitterDetailEditsMoved(Sender:
  TObject);
begin
  inherited;
  if assigned(DetailEditList) then
  begin
    FMyRecordsInput.RememberInteger('FormViewRecord', 'pDetailEditsWidth',
      DetailEditList.Width, False);
   // DetailEditList.Resize;
  end;
end;

procedure TFormLKRecordsInputRecord.SplitterMasterEditsMoved(Sender:
  TObject);
begin
  inherited;
  FMyRecordsInput.RememberInteger('FormViewRecord', 'pMasterEditsWidth',
    MasterEditList.Width, False);
  // MasterEditList.Resize;
end;

end.

