{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Wizard to set the Filter of a TLKTableBase                             }
{                                                                             }
{*****************************************************************************}


unit LKTblFilterDg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, LKDefine, Mask;

type
  TFormLKTableFilter = class(TForm)
    btnOK: TButton;
    CancelBtn: TButton;
    GroupBox1: TGroupBox;
    SpeedButton1: TSpeedButton;
    mLink: TMemo;
    mFieldName: TMemo;
    mTest: TMemo;
    mData: TMemo;
    GroupBox2: TGroupBox;
    sbPlus: TSpeedButton;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    lData: TLabel;
    Label5: TLabel;
    cbxFieldName: TComboBox;
    cbxTest: TComboBox;
    eData: TMaskEdit;
    cbxLink: TComboBox;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure cbxFieldNameChange(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure sbPlusClick(Sender: TObject);
    procedure cbxTestChange(Sender: TObject);
  private
    { Private declarations }
    FFieldDataType: string;   
    FStringFields: TStringList;
  public
    { Public declarations }
    property StringFields: TStringList read FStringFields;
  end;

var
  FormLKTableFilter: TFormLKTableFilter;

implementation

{$R *.DFM}

procedure TFormLKTableFilter.FormCreate(Sender: TObject);
begin
  FStringFields := TStringList.Create;
end;

procedure TFormLKTableFilter.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  FStringFields.Free;
  Action := caFree;
end;

procedure TFormLKTableFilter.cbxFieldNameChange(Sender: TObject);
var
  n: integer;
begin
  btnOK.enabled := false;
  FFieldDataType := 'unknown';
  with cbxFieldName do
    for n := 0 to items.Count - 1 do
      if text = items[n] then
        FFieldDataType := FStringFields[n];
  if FFieldDataType = 'unknown' then
    raise LKError.Create('Field name not recognised');
  lData.Caption := FFieldDataType;
  if (FFieldDataType = 'Date') then
    eData.EditMask := '!99/99/00;1;_'
  else if (FFieldDataType = 'Time') then
    eData.EditMask := '!90:00;1;_'
  else
    eData.EditMask := '';
end;

procedure TFormLKTableFilter.SpeedButton1Click(Sender: TObject);
begin
  btnOK.enabled := false;
  mLink.Lines.Clear;
  mFieldName.Lines.Clear;
  mTest.Lines.Clear;
  mData.Lines.Clear;
end;

procedure TFormLKTableFilter.sbPlusClick(Sender: TObject);
var
  Data, Test: string;
label
  continue;
begin
  Test := cbxTest.text;
  if (Test = 'is blank') then
  begin
    Test := '=';
    Data := 'NULL';
    goto continue;
  end;
  if (Test = 'not blank') then
  begin
    Test := '<>';
    Data := 'NULL';
    goto continue;
  end;
  Data := eData.Text;
  if Data = '' then
    Data := 'NULL';
  if Uppercase(Data) = 'NULL' then
    goto continue;
  if (FFieldDataType = 'Text') then
    Data := chr(39) + Data + chr(39);
  if (FFieldDataType = 'Date') then
  begin
    StrToDate(Data);
    Data := chr(39) + Data + chr(39);
  end;
  if (FFieldDataType = 'Number') then
    StrToFloat(Data);
  if (FFieldDataType = 'TRUE or FALSE') then
  begin
    Data := UpperCase(Data);
    if (Test <> '<>') then
      Test := '=';
    if (Data <> 'TRUE') and (Data <> 'FALSE') then
      raise LKError.Create('Data must be TRUE or FALSE');
  end;
  continue:
  if mLink.lines.count = 0 then
    mLink.Lines.add(' ')
  else
    mLink.Lines.add(cbxLink.text);
  mFieldName.lines.Add(cbxFieldName.text);
  mTest.lines.add(Test);
  mData.lines.add(Data);
  cbxTest.text := '=';
  eData.Text := '';
  btnOK.enabled := true;
end;

procedure TFormLKTableFilter.cbxTestChange(Sender: TObject);
begin
  btnOK.enabled := false;
end;

end.
 
