{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      various utility components and controls used by TLKDataComponent etc.  }
{                                                                             }
{*****************************************************************************}

unit LKUtilComponents;

interface

uses
  SysUtils, Classes, Dialogs, Controls, System.Types,
  Forms, Clipbrd, StdCtrls, Variants,
  DBCtrls, DBGrids, WinTypes, Messages, ShellAPI, graphics, ComCtrls, Buttons,
  ExtCtrls, DB, DateUtils, ActnList, Contnrs, Mask, Menus,
  JvExMask, JvToolEdit, JvDBControls, JvExDBGrids, JvDBGrid, JvDBUltimGrid,
  LKBaseComponent, LKDefine, LKRecordsComponents, LKDBControls;

type
  TLKHTML = class(TLKComponent)
  private
    FAutoExecute: Boolean;
    FBodyText: TStrings;
    FHTML: TStrings;
    FTitle: string;
    function GetDocument: string;
    function GetTitle: string;
    procedure SetBodyText(Value: TStrings);
    procedure SetHTML(Value: TStrings);
  protected
    procedure GetBody; virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure CreateViewForm; override;
    property Document: string read GetDocument;
    property HTML: TStrings read FHTML write SetHTML;
  published
    property AutoExecute: Boolean read FAutoExecute write FAutoExecute;
    property BodyText: TStrings read FBodyText write SetBodyText;
    property Title: string read GetTitle write FTitle;
  end;

  TLKDcBasePanel = class(TPanel)
  private
    FGridController: TLKDcDBGridController;
    FSplitter: TSplitter;
  protected
    FGridComponent: TLKDataComponent;
    FGridTable: TLKACTable;
    FImages: TImageList;
    FOwnerComponent: TLKDataComponent;
    procedure MPGridKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure MPKeyPress(Sender: TObject; var Key: Char);
    procedure MPPopupClearColumnsClick(Sender: TObject);
    procedure MPPopupSaveColumnsClick(Sender: TObject);
    procedure OnMPSplitterMoved(Sender: TObject);
    procedure OnSbNewClick(Sender: TObject);
    procedure OnSbViewClick(Sender: TObject);
    procedure OnSbViewRecordClick(Sender: TObject);
    procedure SetGridTable(Value: TLKACTable); virtual;
  public
    ChildMPGrid: TJvDBUltimGrid;
    Grid: TJvDBUltimGrid;
    GridDatasource: TDataSource;
    GridLabel: TLabel;
    GridPopUp: TPopupMenu;
    GridPopUpItemClearGrid: TMenuItem;
    GridTableName: string;
    sbNew: TSpeedButton;
    sbView: TSpeedButton;
    sbViewRecord: TSpeedButton;
    ShortcutChar: Word;
    Toolbar: TToolbar;
    constructor Create(AOwner: TComponent); override;
    procedure Construct; virtual;
    procedure DoOnFormClose;
    property GridController: TLKDcDBGridController read FGridController;
    property Splitter: TSplitter read FSplitter;
  published
    property GridComponent: TLKDataComponent read FGridComponent write
        FGridComponent;
    property GridTable: TLKACTable read FGridTable write SetGridTable;
    property Images: TImageList read FImages write FImages;
    property OwnerComponent: TLKDataComponent read FOwnerComponent write
        FOwnerComponent;
  end;

  TLKDcMasterPanel = class(TLKDcBasePanel)
  protected
    procedure SetGridTable(Value: TLKACTable); override;
  end;

  TLKDcMasterPanelList = class(TComponentList)
  protected
    function GetItems(Index: Integer): TLKDcMasterPanel;
    procedure SetItems(Index: Integer; AComponent: TLKDcMasterPanel);
  public
    function Add(AComponent: TLKDcMasterPanel): Integer;
    function Extract(Item: TLKDcMasterPanel): TLKDcMasterPanel;
    function First: TLKDcMasterPanel;
    function IndexOf(AComponent: TLKDcMasterPanel): Integer;
    procedure Insert(Index: Integer; AComponent: TLKDcMasterPanel);
    function Last: TLKDcMasterPanel;
    property Items[Index: Integer]: TLKDcMasterPanel read GetItems write
        SetItems; default;
  end;

  TLKDcLookupPanel = class(TLKDcBasePanel)
  private
    FFieldPairsToLookup: TStrings;
    procedure SetFieldPairsToLookup(Value: TStrings);
  protected
    FLookupTable: TLKACTable;
    procedure OnSbUseClick(Sender: TObject);
    procedure SetGridTable(Value: TLKACTable); override;
  public
    sbUse: TSpeedButton;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Construct; override;
  published
    property FieldPairsToLookup: TStrings read FFieldPairsToLookup write
        SetFieldPairsToLookup;
    property LookupTable: TLKACTable read FLookupTable write FLookupTable
        default tbMaster;
  end;

  TLKProcedureStepBase = class(TCollectionItem)
  private
    FDisplayName: string;
    FEnabled: Boolean;
  protected
    function GetDisplayName: string; override;
    function GetEnabled: Boolean; virtual;
    procedure SetDisplayName(const Value: string); override;
    procedure SetEnabled(Value: Boolean); virtual;
  public
    property Enabled: Boolean read GetEnabled write SetEnabled;
  published
    property DisplayName: string read GetDisplayName write SetDisplayName;
  end;

  TLKProcedureStepAction = class(TLKProcedureStepBase)
  private
    FAction: TCustomAction;
    procedure SetAction(Value: TCustomAction);
  protected
    function GetEnabled: Boolean; override;
    procedure SetEnabled(Value: Boolean); override;
  published
    property Action: TCustomAction read FAction write SetAction;
  end;

  TLKProcedureStepActions = class(TOwnedCollection)
  private
    FEnabled: Boolean;
    procedure SetEnabled(Value: Boolean);
  public
    property Enabled: Boolean read FEnabled write SetEnabled;
  end;

  TLKProcedureStep = class(TLKProcedureStepBase)
  private
    FActions: TLKProcedureStepActions;
    FBackstepActions: TLKProcedureStepActions;
    procedure SetActions(Value: TLKProcedureStepActions);
    procedure SetBackstepActions(Value: TLKProcedureStepActions);
  protected
    procedure SetEnabled(Value: Boolean); override;
  public
    constructor Create(Collection: TCollection); override;
    function DlgBackstepExecute: TModalResult;
  published
    property Actions: TLKProcedureStepActions read FActions write SetActions;
    property BackstepActions: TLKProcedureStepActions read FBackstepActions
        write SetBackstepActions;
  end;

  TLKProcedureSteps = class(TOwnedCollection)
  end;

  TLKProcedure = class(TComponent)
  private
    FButtonBackStep: TButton;
    FButtonNextStep: TButton;
    FCurrentStep: Integer;
    FStepNameLabel: TLabel;
    FSteps: TLKProcedureSteps;
    procedure SetButtonBackStep(Value: TButton);
    procedure SetButtonNextStep(Value: TButton);
    procedure SetCurrentStep(Value: Integer);
    procedure SetStepNameLabel(Value: TLabel);
    procedure SetSteps(Value: TLKProcedureSteps);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation);
        override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure BtnBackStepClick(Sender: TObject);
    procedure BtnNextStepClick(Sender: TObject);
  published
    property ButtonBackStep: TButton read FButtonBackStep write
        SetButtonBackStep;
    property ButtonNextStep: TButton read FButtonNextStep write
        SetButtonNextStep;
    property CurrentStep: Integer read FCurrentStep write SetCurrentStep
        default -1;
    property StepNameLabel: TLabel read FStepNameLabel write SetStepNameLabel;
    property Steps: TLKProcedureSteps read FSteps write SetSteps;
  end;

procedure Register;

implementation

uses
  Printers, LKDlgs, LKUtils, LKUtilsDB,
  LKApProcedureBackstepDlg, LKHMTLDlg;

procedure Register;
begin
  // RegisterComponents('LKUtils', [TLKDBDcLookupEdit]);
  RegisterComponents('LKUtils', [TLKProcedure]);
end;

{
*********************************** TLKHTML ************************************
}
constructor TLKHTML.Create(AOwner: TComponent);
begin
  inherited;
  FBodyText := TStringList.Create;
  FHTML := TStringList.Create;
  FTitle := '';
  FAutoExecute := True;
end;

destructor TLKHTML.Destroy;
begin
  FBodyText.Free;
  FHTML.Free;
  inherited Destroy;
end;

procedure TLKHTML.CreateViewForm;
begin
  FFormView := TDlgLKHTML.Create(Self);
end;

procedure TLKHTML.GetBody;
begin
  HTML.AddStrings(FBodyText);
end;

function TLKHTML.GetDocument: string;
begin
  FHTML.Clear;

  FHTML.Add('<!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">');
  FHTML.Add('<html>');

  FHTML.Add('<head>');
  FHTML.Add('<meta content="text/html;charset=ISO-8859-1" http-equiv="Content-Type">');
  FHTML.Add('<title>' + Title + '</title>');
  FHTML.Add('</head>');

  FHTML.Add('<body bgcolor="#ffffff" text="#000000">');

  GetBody;

  FHTML.Add('<!-- ************************************************************************** -->');
  FHTML.Add('</body>');
  FHTML.Add('</html>');
  Result := FHTML.Text;
end;

function TLKHTML.GetTitle: string;
begin
  Result := FTitle;
end;

procedure TLKHTML.SetBodyText(Value: TStrings);
begin
  FBodyText.Assign(Value);
end;

procedure TLKHTML.SetHTML(Value: TStrings);
begin
  FHTML.Assign(Value);
end;

{
******************************** TLKDcBasePanel ********************************
}
constructor TLKDcBasePanel.Create(AOwner: TComponent);
begin
  inherited;
  Height := 40; // make small enough to take correct position on panel
  Align := alTop;
  Toolbar := TToolbar.Create(AOwner);
  sbNew := TSpeedButton.Create(AOwner);
  sbViewRecord := TSpeedButton.Create(AOwner);
  sbView := TSpeedButton.Create(AOwner);
  GridDatasource := TDataSource.Create(AOwner);
  GridPopUp := TPopUpMenu.Create(AOwner);
  Grid := TJvDBUltimGrid.Create(AOwner);
  FGridController := TLKDcDBGridController.Create(AOwner);
  GridLabel := TLabel.Create(AOwner);
  FSplitter := TSplitter.Create(AOwner);
  FGridTable := tbMaster;
end;

procedure TLKDcBasePanel.Construct;
var
  S: string;
  GridPopUpItem: TMenuItem;
begin
  with Toolbar do
  begin
    Parent := Self;
    ButtonHeight := 25;
  end;
  with sbNew do
  begin
    Parent := Toolbar;
    Flat := true;
    ShowHint := True;
    OnClick := OnSbNewClick;
  end;
  with sbViewRecord do
  begin
    Parent := Toolbar;
    Flat := True;
    ShowHint := True;
    OnClick := OnSbViewRecordClick;
  end;
  with sbView do
  begin
    Parent := Toolbar;
    Flat := true;
    ShowHint := True;
    OnClick := OnSbViewClick;
  end;
  GridPopUpItem := TMenuItem.Create(Self);
  with GridPopUpItem do
  begin
    Caption := 'View';
    OnClick := OnSbViewRecordClick;
  end;
  GridPopUp.Items.Add(GridPopUpItem);
  GridPopUpItem := TMenuItem.Create(Self);
  with GridPopUpItem do
  begin
    Caption := 'New';
    OnClick := OnSbNewClick;
  end;
  GridPopUp.Items.Add(GridPopUpItem);
  GridPopUpItem := TMenuItem.Create(Self);
  with GridPopUpItem do
  begin
    Caption := 'Save Column settings';
    OnClick := MPPopupSaveColumnsClick;
  end;
  GridPopUp.Items.Add(GridPopUpItem);
  GridPopUpItemClearGrid := TMenuItem.Create(Self);
  with GridPopUpItemClearGrid do
  begin
    Caption := 'Clear Column settings';
    OnClick := MPPopupClearColumnsClick;
  end;
  GridPopUp.Items.Add(GridPopUpItemClearGrid);
  if Assigned(FImages) then
    with FImages do
    begin
      GetBitmap(14, sbNew.Glyph);
      sbNew.Glyph.TransparentColor := clSilver;
      GetBitmap(9, sbViewRecord.Glyph);
      sbViewRecord.Glyph.TransparentColor := clSilver;
      GetBitmap(42, sbView.Glyph);
      sbView.Glyph.TransparentColor := clSilver;
    end;
  with GridPopUp do
  begin
    Images := FImages;
    Items[0].ImageIndex := 9;
    Items[1].ImageIndex := 14;
    Items[2].ImageIndex := 18;
  end;
  GridController.Grid := Grid;
  with Grid do
  begin
    Parent := Self;
    Datasource := GridDatasource;
    Align := alClient;
    AlternateRowColor := $FFFCFC;//cl3dLight;
    PopUpMenu := GridPopUp;
    Options := Options + [dgRowSelect];
    OnKeyDown := MPGridKeyDown;
    OnKeyPress := MPKeyPress;
    OnDblClick := OnSbViewRecordClick;
  end;
  with GridLabel do
  begin
    Layout := tlCenter;
    Height := 25;
    Left := 100;
    Parent := Toolbar;
    FocusControl := Grid;
  end;
  FSplitter.Top := Top + Height + 1;
  with FSplitter do
  begin
    Height := 3;
    Align := alTop;
    Parent := Self.Parent;
    Autosnap := False;
    ResizeStyle := rsLine;
    MinSize := 40;
    OnMoved := OnMPSplitterMoved;
  end;
  S := '';
  if not Assigned(FGridComponent) then Exit;
  with FGridComponent do case FGridTable of
    tbMaster:
      begin
        S := MasterFields;
        TblMasterDataSource := GridDatasource;
      end;
    tbDetail:
      begin
        S := TblDetail.MasterFields;
        TblDetailDataSource := GridDatasource;
      end;
  end;
  if Assigned(GridDatasource.Dataset) then
  begin
    Grid.Name := FGridComponent.Name +'Grid' + GridDatasource.Dataset.Name;
    Self.Name :=  Grid.Name + 'Panel';
  end;
  GridController.ConstructFromDatasource(S);
  Grid.ReadOnly := True;
    //  MPGrid.Columns.Clear;
  GridTableName := TLKTable(GridDatasource.Dataset).TableName;
  S := GridTableName;
  if (pos('.', S) > 0) then
    System.Delete(S, pos('.', S), 3);
  try
    GridPopUpItemClearGrid.Enabled := GridController.LoadFormat(FOwnerComponent,
      'FormView', S + 'Grid')
  except MPPopupClearColumnsClick(Self);
  end;
  S := FGridComponent.NameFriendly;
  if (S[length(S)] = 'S') then
    system.Delete(S, length(S), 1);
  sbNew.Hint := 'New ' + S;
  sbView.Hint := 'View ' + S;
end;

procedure TLKDcBasePanel.DoOnFormClose;
begin
  FGridController.DoOnFormClose;
end;

procedure TLKDcBasePanel.MPGridKeyDown(Sender: TObject; var Key: Word; Shift:
    TShiftState);

  const
    myKeys = 'VN';

begin
  if (Shift = [ssAlt]) then
  begin
    if (key = word('V')) then
    begin
      key := 0;
      OnSbViewRecordClick(Self);
    end;
    if (key = word('N')) then
    begin
      key := 0;
      OnSbNewClick(Self);
    end;
  end;
end;

procedure TLKDcBasePanel.MPKeyPress(Sender: TObject; var Key: Char);
begin
  if (Key = chr(VK_RETURN)) then
  begin
    Key := chr(0);
    if Assigned(ChildMPGrid) then
      ChildMPGrid.SetFocus;
  end;
end;

procedure TLKDcBasePanel.MPPopupClearColumnsClick(Sender: TObject);
begin
  GridPopUpItemClearGrid.Enabled := false;
  GridController.ClearFormat(FOwnerComponent, 'FormView', GridTableName + 'Grid');
end;

procedure TLKDcBasePanel.MPPopupSaveColumnsClick(Sender: TObject);
begin
  GridController.SaveFormat(FOwnerComponent, 'FormView', GridTableName + 'Grid');
  GridPopUpItemClearGrid.Enabled := True;
  Showmessage('Grid column width and position settings have been saved');
end;

procedure TLKDcBasePanel.OnMPSplitterMoved(Sender: TObject);
begin
  FOwnerComponent.RememberInteger('FormView', Name, Height, false);
end;

procedure TLKDcBasePanel.OnSbNewClick(Sender: TObject);
begin
  FGridComponent.RecordNew;
end;

procedure TLKDcBasePanel.OnSbViewClick(Sender: TObject);
begin
  FGridComponent.View(null, true, [mbOK]);
end;

procedure TLKDcBasePanel.OnSbViewRecordClick(Sender: TObject);
begin
  FGridComponent.ViewRecord(null, true, [mbOK]);
end;

procedure TLKDcBasePanel.SetGridTable(Value: TLKACTable);
begin
  FGridTable := Value;
  sbViewRecord.Visible := (FGridTable = tbMaster);
  sbView.Visible := (FGridTable = tbMaster);
  sbNew.Visible := (FGridTable = tbMaster);
end;

{
******************************* TLKDcMasterPanel *******************************
}
procedure TLKDcMasterPanel.SetGridTable(Value: TLKACTable);
var
  s: string;
begin
  inherited SetGridTable(Value);
  if Assigned(FGridComponent) then
  begin
    case FGridTable of
      tbMaster: s := 'M';
    else
      s := 'D';
    end;
    Name := FGridComponent.Name + s + 'Panel';
  end;
end;

{
***************************** TLKDcMasterPanelList *****************************
}
function TLKDcMasterPanelList.Add(AComponent: TLKDcMasterPanel): Integer;
begin
  Result := inherited Add(AComponent);
end;

function TLKDcMasterPanelList.Extract(Item: TLKDcMasterPanel): TLKDcMasterPanel;
begin
  Result := TLKDcMasterPanel(inherited Extract(Item));
end;

function TLKDcMasterPanelList.First: TLKDcMasterPanel;
begin
  Result := TLKDcMasterPanel(inherited First);
end;

function TLKDcMasterPanelList.GetItems(Index: Integer): TLKDcMasterPanel;
begin
  Result := TLKDcMasterPanel(inherited Items[Index]);
end;

function TLKDcMasterPanelList.IndexOf(AComponent: TLKDcMasterPanel): Integer;
begin
  Result := inherited IndexOf(AComponent);
end;

procedure TLKDcMasterPanelList.Insert(Index: Integer; AComponent:
    TLKDcMasterPanel);
begin
  inherited Insert(Index, AComponent);
end;

function TLKDcMasterPanelList.Last: TLKDcMasterPanel;
begin
  Result := TLKDcMasterPanel(inherited Last);
end;

procedure TLKDcMasterPanelList.SetItems(Index: Integer; AComponent:
    TLKDcMasterPanel);
begin
  inherited Items[Index] := AComponent;
end;

{
******************************* TLKDcLookupPanel *******************************
}
constructor TLKDcLookupPanel.Create(AOwner: TComponent);
var
  GridPopUpItem: TMenuItem;
begin
  inherited;
  sbUse := TSpeedButton.Create(ToolBar);
  with sbUse do
  begin
    Parent := Toolbar;
    Flat := true;
    ShowHint := True;
    OnClick := OnSbUseClick;
  end;
  GridPopUpItem := TMenuItem.Create(Self);
  with GridPopUpItem do
  begin
    Caption := 'Use this item';
    OnClick := OnSbUseClick;
  end;
  GridPopUp.Items.Add(GridPopUpItem);
  GridLabel.Left := 125;
  FFieldPairsToLookup := TStringList.Create;
end;

destructor TLKDcLookupPanel.Destroy;
begin
  FFieldPairsToLookup.Free;
  inherited;
end;

procedure TLKDcLookupPanel.Construct;
begin
  inherited Construct;
  if Assigned(FImages) then
    with FImages do
    begin
      GetBitmap(28, sbUse.glyph);
      sbUse.Glyph.TransparentColor := clSilver;
      GridPopup.Items[3].ImageIndex := 28;
    end;
end;

procedure TLKDcLookupPanel.OnSbUseClick(Sender: TObject);
var
  OwnerTable: TLKTable;
  I: Integer;
begin
  case FLookupTable of
    tbMaster: OwnerTable := OwnerComponent.TblMaster;
  else
    OwnerTable := OwnerComponent.TblDetail;
  end;
  OwnerTable.Edit;
  i := 0;
  while (i < FieldPairsToLookup.Count - 2) do
  begin
    OwnerTable.FieldByName(FieldPairsToLookup[i]).Value :=
      GridDatasource.Dataset.FieldByName(FieldPairsToLookup[i + 1]).Value;
    Inc(i, 2);
  end;
  OwnerTable.Post;
end;

procedure TLKDcLookupPanel.SetFieldPairsToLookup(Value: TStrings);
begin
  FFieldPairsToLookup.Assign(Value);
end;

procedure TLKDcLookupPanel.SetGridTable(Value: TLKACTable);
var
  S: string;
begin
  inherited SetGridTable(Value);
  if Assigned(FGridComponent) then
  begin
    case FGridTable of
      tbMaster: S := 'M';
    else
      S := 'D';
    end;
    Name := FGridComponent.Name + S + 'Lookup';
  end;
end;

{
***************************** TLKProcedureStepBase *****************************
}
function TLKProcedureStepBase.GetDisplayName: string;
begin
  Result := FDisplayName;
end;

function TLKProcedureStepBase.GetEnabled: Boolean;
begin
  Result := FEnabled;
end;

procedure TLKProcedureStepBase.SetDisplayName(const Value: string);
begin
  FDisplayName := Value;
end;

procedure TLKProcedureStepBase.SetEnabled(Value: Boolean);
begin
  FEnabled := Value;
end;

{
**************************** TLKProcedureStepAction ****************************
}
function TLKProcedureStepAction.GetEnabled: Boolean;
begin
  if Assigned(FAction) then
    Result := FAction.Enabled
  else
    Result := false;
end;

procedure TLKProcedureStepAction.SetAction(Value: TCustomAction);
begin
  FAction := Value;
  Enabled := false;
  if Assigned(Value) then
    DisplayName := Value.Caption;
end;

procedure TLKProcedureStepAction.SetEnabled(Value: Boolean);
begin
  if Assigned(FAction) then
    FAction.Enabled := Value;
end;

{
*************************** TLKProcedureStepActions ****************************
}
procedure TLKProcedureStepActions.SetEnabled(Value: Boolean);
var
  I: Integer;
begin
  FEnabled := Value;
  for i := 0 to Count - 1 do
    TLKProcedureStepAction(Items[i]).Enabled := Value;
end;

{
******************************* TLKProcedureStep *******************************
}
constructor TLKProcedureStep.Create(Collection: TCollection);
begin
  inherited Create(Collection);
  FActions := TLKProcedureStepActions.Create(Self, TLKProcedureStepAction);
  FBackstepActions := TLKProcedureStepActions.Create(Self,
    TLKProcedureStepAction);
end;

function TLKProcedureStep.DlgBackstepExecute: TModalResult;
begin
  Result := mrOK;
  if BackstepActions.Count = 0 then
    exit;
  with TDlgLKApProcedureBackstep.Create(TLKProcedure(Collection.Owner)) do
  begin
    try
      Construct(BackstepActions);
      Result := Showmodal;
    finally Free;
    end;
  end
end;

procedure TLKProcedureStep.SetActions(Value: TLKProcedureStepActions);
begin
  FActions.Assign(Value);
end;

procedure TLKProcedureStep.SetBackstepActions(Value: TLKProcedureStepActions);
begin
  FBackstepActions.Assign(Value);
end;

procedure TLKProcedureStep.SetEnabled(Value: Boolean);
begin
  Actions.Enabled := Value;
  BackstepActions.Enabled := Value;
  inherited SetEnabled(Value);
end;

{
********************************* TLKProcedure *********************************
}
constructor TLKProcedure.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCurrentStep := -1;
  FSteps := TLKProcedureSteps.Create(Self, TLKProcedureStep);
end;

destructor TLKProcedure.Destroy;
begin
  FSteps.Free;
  inherited Destroy;
end;

procedure TLKProcedure.BtnBackStepClick(Sender: TObject);
begin
  if Assigned(FSteps.Items[CurrentStep]) then
    if (
      TLKProcedureStep(FSteps.Items[CurrentStep]).DlgBackstepExecute = mrOK)
        then
      CurrentStep := CurrentStep - 1;
end;

procedure TLKProcedure.BtnNextStepClick(Sender: TObject);
begin
  CurrentStep := CurrentStep + 1;
end;

procedure TLKProcedure.Notification(AComponent: TComponent; Operation:
    TOperation);
var
  I, j: Integer;
begin
  if (Operation = opRemove) then
  begin
    if (AComponent = FButtonBackStep) then
      FButtonBackStep := nil;
    if (AComponent = FButtonNextStep) then
      FButtonNextStep := nil;
    if (AComponent = FStepNameLabel) then
      FStepNameLabel := nil;
    if (AComponent is TCustomAction) then
      with Steps do
        for i := 0 to count - 1 do
          with TLKProcedureStep(Items[i]) do
          begin
            with Actions do
              for j := 0 to Count - 1 do
                with TLKProcedureStepAction(Items[j]) do
                  if (Action = AComponent) then
                    Action := nil;
            with BackstepActions do
              for j := 0 to Count - 1 do
                with TLKProcedureStepAction(Items[j]) do
                  if (Action = AComponent) then
                    Action := nil;
          end;
  end;
  inherited Notification(AComponent, Operation);
end;

procedure TLKProcedure.SetButtonBackStep(Value: TButton);
begin
  if Assigned(Value) then
    Value.OnClick := BtnBackStepClick;
  FButtonBackStep := Value;
end;

procedure TLKProcedure.SetButtonNextStep(Value: TButton);
begin
  FButtonNextStep := Value;
  if Assigned(Value) then
    Value.OnClick := BtnNextStepClick;
end;

procedure TLKProcedure.SetCurrentStep(Value: Integer);
begin
  if (Value < 0) or (csLoading in ComponentState) then
    exit;
  if (FCurrentStep > -1) then
    TLKProcedureStep(Steps.Items[FCurrentStep]).Enabled := false;
  FCurrentStep := Value;
  if (FCurrentStep < Steps.Count) then
    TLKProcedureStep(Steps.Items[FCurrentStep]).Enabled := true;
  if Assigned(ButtonBackStep) then
    ButtonBackStep.Enabled := (FCurrentStep > 0);
  if Assigned(ButtonNextStep) then
    ButtonNextStep.Enabled := (FCurrentStep < Steps.Count - 1);
  if Assigned(StepNameLabel) then
    StepNameLabel.Caption :=
      Steps.Items[FCurrentStep].DisplayName;
  // TLKProcedureStep(Steps.Items[FCurrentStep]).DisplayName;
end;

procedure TLKProcedure.SetStepNameLabel(Value: TLabel);
begin
  FStepNameLabel := Value;
  if Assigned(Value) then
    Value.FreeNotification(Self);
end;

procedure TLKProcedure.SetSteps(Value: TLKProcedureSteps);
begin
  FSteps.Assign(Value)
end;

end.
