{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for user manipulation of a TLKDBEditsPanel                        }
{                                                                             }
{*****************************************************************************}

unit LKUtilsEditListPropsDlg;

interface

uses Windows, SysUtils, Classes, System.Types,
  Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, Contnrs, LKUtils, LKDBControls, LKUtilComponents;

type
  TDBControlPanel = class(TPanel)
  private
    eCaption: TEdit;
    eHeight: TEdit;
    myCkbx: TCheckbox;
    myDBControl: TLKDcDBEditControl;
    myLabel: TLabel;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Construct(ADBControl: TLKDcDBEditControl);
    procedure UpdateDBControl;
  end;

  TDBControlPanelList = class(TComponentList)
  protected
    function GetItems(Index: Integer): TDBControlPanel;
    procedure SetItems(Index: Integer; AComponent: TDBControlPanel);
  public
    function Add(AComponent: TDBControlPanel): Integer;
    function Extract(Item: TDBControlPanel): TDBControlPanel;
    function First: TDBControlPanel;
    function IndexOf(AComponent: TDBControlPanel): Integer;
    procedure Insert(Index: Integer; AComponent: TDBControlPanel);
    function Last: TDBControlPanel;
    property Items[Index: Integer]: TDBControlPanel read GetItems write
    SetItems; default;
  end;

  TDlgLKUtilsEditListProps = class(TForm)
    CancelBtn: TButton;
    eColWidth0: TEdit;
    eColWidth1: TEdit;
    eColWidth2: TEdit;
    gbFields: TGroupBox;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    OKBtn: TButton;
    Panel1: TPanel;
    rgColumns: TRadioGroup;
    procedure eColWidth0Exit(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure OKBtnClick(Sender: TObject);
    procedure rgColumnsClick(Sender: TObject);
  private
    DBControlPanelList: TDBControlPanelList;
    myEditList: TLKDcDBEditsPanel;
  public
    procedure Construct(AEditList: TLKDcDBEditsPanel);
  end;

var
  DlgLKUtilsEditListProps: TDlgLKUtilsEditListProps;

implementation

{$R *.dfm}

uses LKDefine;

{
******************************* TDBControlPanel ********************************
}

constructor TDBControlPanel.Create(AOwner: TComponent);
begin
  inherited;
  BevelOuter := bvNone;
  Height := 16;
  Width := 334;
  Caption := ' ';
  myLabel := TLabel.Create(AOwner);
  with myLabel do
  begin
    Left := 0;
    Top := 0;
    Parent := Self;
  end;
  eCaption := TEdit.Create(AOwner);
  with eCaption do
  begin
    BorderStyle := bsNone;
    Left := 112;
    Top := 0;
    Height := 15;
    Width := 137;
    Parent := Self;
  end;
  myCkbx := TCheckbox.Create(AOwner);
  with myCkbx do
  begin
    Width := 16;
    Left := 264;
    Top := 0;
    Parent := Self;
  end;
  eHeight := TEdit.Create(AOwner);
  with eHeight do
  begin
    BorderStyle := bsNone;
    Left := 316;
    Top := 0;
    Height := 15;
    Width := 16;
    Parent := Self;
  end;
end;

procedure TDBControlPanel.Construct(ADBControl: TLKDcDBEditControl);
begin
  inherited;
  myDBControl := ADBControl;
  myLabel.Caption := myDBControl.FieldName;
  eCaption.Text := myDBControl.Caption;
  myCkbx.Checked := myDBControl.Visible;
  eHeight.Text := FloatToStr(myDBControl.LinesHeight);
end;

procedure TDBControlPanel.UpdateDBControl;
begin
  with myDBControl do
  begin
    Caption := eCaption.Text;
    Visible := myCkbx.Checked;
    if (StrToFloat(eHeight.Text) = 0) then
      eHeight.Text := FloatToStr(LinesHeight)
    else
      LinesHeight := StrToFloat(eHeight.Text);
  end;
end;

{
***************************** TDBControlPanelList ******************************
}

function TDBControlPanelList.Add(AComponent: TDBControlPanel): Integer;
begin
  Result := inherited Add(AComponent);
end;

function TDBControlPanelList.Extract(Item: TDBControlPanel): TDBControlPanel;
begin
  Result := TDBControlPanel(inherited Extract(Item));
end;

function TDBControlPanelList.First: TDBControlPanel;
begin
  Result := TDBControlPanel(inherited First);
end;

function TDBControlPanelList.GetItems(Index: Integer): TDBControlPanel;
begin
  Result := TDBControlPanel(inherited Items[Index]);
end;

function TDBControlPanelList.IndexOf(AComponent: TDBControlPanel): Integer;
begin
  Result := inherited IndexOf(AComponent);
end;

procedure TDBControlPanelList.Insert(Index: Integer; AComponent:
  TDBControlPanel);
begin
  inherited Insert(Index, AComponent);
end;

function TDBControlPanelList.Last: TDBControlPanel;
begin
  Result := TDBControlPanel(inherited Last);
end;

procedure TDBControlPanelList.SetItems(Index: Integer; AComponent:
  TDBControlPanel);
begin
  inherited Items[Index] := AComponent;
end;

{
************************** TDlgLKUtilsEditListProps ***************************
}

procedure TDlgLKUtilsEditListProps.Construct(AEditList: TLKDcDBEditsPanel);
var
  i: Integer;
  myDBControlPanel: TDBControlPanel;
begin
  myEditList := AEditList;
  rgColumns.ItemIndex := Ord(myEditList.Layout);
  rgColumnsClick(Self);
  for i := 0 to myEditList.EditControls.Count - 1 do
  begin
    myDBControlPanel := TDBControlPanel.Create(self);
    with myDBControlPanel do
    begin
      Top := 32 + i * 16;
      Left := 8;
      Parent := gbFields;
      Construct(myEditList.EditControls.Items[i]);
    end;
    DBControlPanelList.Add(myDBControlPanel);
  end;
  i := 68 + myEditList.EditControls.Count * 16;
  if (Height < i) then
    Height := i;
end;

procedure TDlgLKUtilsEditListProps.eColWidth0Exit(Sender: TObject);
var
  a, b: Integer;
begin
  a := strToInt(eColWidth0.Text);
  b := strToInt(eColWidth1.Text);
  case rgColumns.ItemIndex of
    0, 1:
      begin
        eColWidth0.Text := '100';
        eColWidth1.Text := '0';
        eColWidth2.Text := '0';
      end;
    2:
      begin
        eColWidth1.Text := intToStr(100 - a);
        eColWidth2.Text := '0';
      end;
    3:
      begin
        eColWidth2.Text := intToStr(100 - a - b);
      end;
  end;
end;

procedure TDlgLKUtilsEditListProps.FormCreate(Sender: TObject);
begin
  DBControlPanelList := TDBControlPanelList.Create;
  DBControlPanelList.OwnsObjects := false;
end;

procedure TDlgLKUtilsEditListProps.FormDestroy(Sender: TObject);
begin
  DBControlPanelList.Free;
end;

procedure TDlgLKUtilsEditListProps.FormKeyPress(Sender: TObject; var Key:
  Char);
begin
  MoveMyFocus(Self, Key);
end;

procedure TDlgLKUtilsEditListProps.OKBtnClick(Sender: TObject);
var
  i: Integer;
begin
  with DBControlPanelList do
    for i := 0 to Count - 1 do
      Items[i].UpdateDBControl;
  with myEditList do
  begin
    Layout := TLKEditsLayout(rgColumns.ItemIndex);
    ColWidth0 := StrToInt(eColWidth0.Text);
    ColWidth1 := StrToInt(eColWidth1.Text);
    ColWidth2 := StrToInt(eColWidth2.Text);
    Resize;
  end;
end;

procedure TDlgLKUtilsEditListProps.rgColumnsClick(Sender: TObject);
begin
  case rgColumns.ItemIndex of
    0, 1:
      begin
        with eColWidth0 do
        begin
          Text := '100';
          Color := clBtnFace;
        end;
        eColWidth1.Color := clBtnFace;
        eColWidth0Exit(Sender);
      end;
    2:
      begin
        with eColWidth0 do
        begin
          Text := '55';
          Color := clWindow;
        end;
        eColWidth1.Color := clBtnFace;
        eColWidth0Exit(Sender);
      end;
    3:
      begin
        with eColWidth0 do
        begin
          Text := '40';
          Color := clWindow;
        end;
        with eColWidth1 do
        begin
          Text := '30';
          Color := clWindow;
        end;
        eColWidth0Exit(Sender);
      end;
  end;
end;

end.

