{ ***************************************************************************** }
{ }
{ This file is part of the LKRecords components Library }
{ }
{ See the file COPYING.LGPL.txt included in this distribution, }
{ for details about the Licence and Copyright. }
{ }
{ This code is distributed in the hope that it will be useful, }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. }
{ }
{ Author: Leslie John Kaye :    www.leskaye.com }
{ }
{ Abstract: }
{ Form for printing a postal envelope/label }
{ }
{ ***************************************************************************** }

unit prLKEnvelopeDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, prLKPrintBaseDlg, ExtCtrls, StdCtrls, Buttons,
  LKRecordsComponents, LKBaseComponent, LKDefine,
  frxClass, frxDBSet, frxExportHTML, frxExportPDF;

type
  TLKPrintEnvelopeDlg = class(TDlgLKPrintBase)
    Edit0: TEdit;
    Edit1: TEdit;
    Edit2: TEdit;
    Edit3: TEdit;
    Edit4: TEdit;
    Edit5: TEdit;
    Panel2: TPanel;
    procedure FormCreate(Sender: TObject);
  private
    FMyPrintingComponent: TLKDataComponent;
  protected
    function ConstructReport: string; override;
    procedure ExecuteBtnLayout; override;
    procedure ExecuteBtnRemember; override;
  public
    procedure Construct(APrntingComponent: TLKDataComponent;
      const ATitle: string);
  end;

var
  LKPrintEnvelopeDlg: TLKPrintEnvelopeDlg;

implementation

{$R *.dfm}

{
  **************************** TLKPrintEnvelopeDlg ****************************
}
procedure TLKPrintEnvelopeDlg.Construct(APrntingComponent: TLKDataComponent;
  const ATitle: string);
begin
  FMyPrintingComponent := APrntingComponent;
  ReportName := 'PrintEnvelope';
end;

function TLKPrintEnvelopeDlg.ConstructReport: string;
var
  MyPage: TfrxReportPage;
  AddressMemo: TfrxMemoView;
begin
  Result := inherited;
  MyPage := TfrxReportPage(frxReport1.FindObject('Page1'));
  case cbxReportName.ItemIndex of
    0: MyPage.Orientation := TPrinterOrientation.poLandscape;
    1: MyPage.Orientation := TPrinterOrientation.poPortrait;
  end;
  AddressMemo := TfrxMemoView(frxReport1.FindObject('MemoAddress'));
  AddressMemo.Top := (MyPage.Height / 2);
  AddressMemo.Left := (MyPage.Width / 2);
  AddressMemo.Clear;
  AddressMemo.Lines.Add(Edit0.text);
  AddressMemo.Lines.Add(Edit1.text);
  AddressMemo.Lines.Add(Edit2.text);
  AddressMemo.Lines.Add(Edit3.text);
  AddressMemo.Lines.Add(Edit4.text);
  AddressMemo.Lines.Add(Edit5.text);
end;

procedure TLKPrintEnvelopeDlg.ExecuteBtnLayout;
begin
  with TLKViewDialog.Create(Self) do
    try
      DataType := vBoolean;
      Title := 'Envelope orientation';
      HeaderCaption := 'Set the printer orientation';
      HeaderComments :=
        'be careful not to contaminate the printer mechanism with glue from the envelope';
      ChoiceItems.Add('Landscape');
      ChoiceItems.Add('Portrait');
      Value := true;
      if Execute then
      begin
        if Value then
          cbxReportName.ItemIndex := 0
        else
          cbxReportName.ItemIndex := 1;
        btnRememberClick(Self);
      end;
    finally
      Free;
    end;
end;

procedure TLKPrintEnvelopeDlg.ExecuteBtnRemember;
begin
  inherited ExecuteBtnRemember;
  with FMyPrintingComponent do
  begin
    RememberInteger(ReportName, 'Layout', cbxReportName.ItemIndex,
      PrintShareLayouts);
  end;
end;

procedure TLKPrintEnvelopeDlg.FormCreate(Sender: TObject);
begin
  inherited;
  ckbxReportTitle.Visible := False;
  eReportTitle.Visible := False;
end;


end.
