{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Base Form for printing stuff                                           }
{                                                                             }
{*****************************************************************************}

unit prLKPrintBaseDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, System.Types,
  Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, Printers, LKDefine, LKUtils, StdActns,
  System.Contnrs, frxClass, frxExportHTML, frxExportPDF, frxDBSet;

type
  TDlgLKPrintBase = class(TForm)
    btnClose: TBitBtn;
    btnEmail: TBitBtn;
    BtnHTML: TBitBtn;
    btnLayout: TBitBtn;
    BtnPDF: TBitBtn;
    btnPrint: TBitBtn;
    btnRemember: TButton;
    btnSpreadsheet: TBitBtn;
    cbxReportName: TComboBox;
    ckbxReportTitle: TCheckBox;
    eReportTitle: TEdit;
    frxHTMLExport1: TfrxHTMLExport;
    frxPDFExport1: TfrxPDFExport;
    frxReport1: TfrxReport;
    Label1: TLabel;
    Panel1: TPanel;
    Panel3: TPanel;
    reportStatus: TLabel;
    procedure btnCloseClick(Sender: TObject);
    procedure btnEmailClick(Sender: TObject);
    procedure BtnHTMLClick(Sender: TObject);
    procedure btnLayoutClick(Sender: TObject);
    procedure BtnPDFClick(Sender: TObject);
    procedure btnPrintClick(Sender: TObject);
    procedure btnRememberClick(Sender: TObject);
    procedure btnSpreadsheetClick(Sender: TObject);
    procedure cbxReportNameChange(Sender: TObject);
    procedure ckbxReportTitleClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure frxReport1AfterPrintReport(Sender: TObject);
    procedure frxReport1BeforePrint(Sender: TfrxReportComponent);
    procedure frxReport1GetValue(const VarName: string; var Value: Variant);
    procedure frxReport1NewGetValue(Sender: TObject; const VarName: string; var
        Value: Variant);
  private
    FReportName: string;
    FReportNeedsConstruction: Boolean;
    function CheckReportConstructed: Boolean;
  protected
    function ConstructReport: string; virtual;
    procedure DoAfterPrintReport; virtual;
    procedure DoBeforePrint; virtual;
    procedure DoGetValue(const VarName: string; var Value: Variant); virtual;
    procedure DoNewGetValue(Sender: TObject; const VarName: string; var Value:
        Variant); virtual;
    procedure ExecuteBtnEmail; virtual;
    procedure ExecuteBtnHTML; virtual;
    procedure ExecuteBtnLayout; virtual;
    procedure ExecuteBtnPDF; virtual;
    procedure ExecuteBtnRemember; virtual;
    procedure ExecuteBtnSpreadsheet; virtual;
    function PrintFileName: string; virtual;
    procedure SetReportName(const Value: string); virtual;
    property ReportNeedsConstruction: Boolean read FReportNeedsConstruction
        write FReportNeedsConstruction;
  public
    procedure Execute(const AReportName: string; Autoexecute: Boolean);
    property ReportName: string read FReportName write SetReportName;
  end;

var
  DlgLKPrintBase: TDlgLKPrintBase;

implementation

{$R *.dfm}

{
******************************* TDlgLKPrintBase ********************************
}
procedure TDlgLKPrintBase.btnCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TDlgLKPrintBase.btnEmailClick(Sender: TObject);
begin
  btnEmail.Enabled := False;
  try
    ExecuteBtnEmail;
  finally
    btnEmail.Enabled := True;
  end;
end;

procedure TDlgLKPrintBase.BtnHTMLClick(Sender: TObject);
begin
  BtnPDF.Enabled := False;
  try
    ExecuteBtnHTML;
  finally
    BtnPDF.Enabled := True;
  end;
end;

procedure TDlgLKPrintBase.btnLayoutClick(Sender: TObject);
begin
  btnLayout.Enabled := False;
  try
    ExecuteBtnLayout;
    FReportNeedsConstruction := True;
  finally
    btnLayout.Enabled := True;
  end;
end;

procedure TDlgLKPrintBase.BtnPDFClick(Sender: TObject);
begin
  BtnPDF.Enabled := False;
  try
    ExecuteBtnPDF;
  finally
    BtnPDF.Enabled := True;
  end;
end;

procedure TDlgLKPrintBase.btnPrintClick(Sender: TObject);
begin
  btnPrint.Enabled := False;
  try
    if CheckReportConstructed then
      frxReport1.ShowReport(False);
  finally
    btnPrint.Enabled := True;
  end;
end;

procedure TDlgLKPrintBase.btnRememberClick(Sender: TObject);
begin
  btnRemember.Enabled := False;
  Screen.Cursor := crHourGlass;
  try
    ExecuteBtnRemember;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TDlgLKPrintBase.btnSpreadsheetClick(Sender: TObject);
begin
  if (btnSpreadsheet.Caption = 'Abort') then
    Exit
  else
  begin
    btnSpreadsheet.Caption := 'Abort';
    try
      ExecuteBtnSpreadsheet;
    finally
      btnSpreadsheet.Caption := '&Spreadsheet';
    end;
  end;
end;

procedure TDlgLKPrintBase.cbxReportNameChange(Sender: TObject);
begin
  inherited;
  with cbxReportName do
    if (Items.IndexOf(Text) > -1) then
      ReportName := Text;
end;

function TDlgLKPrintBase.CheckReportConstructed: Boolean;
var
  Msg: string;
begin
  if FReportNeedsConstruction then
  begin
    Msg := ConstructReport;
    Result := (Msg = 'success');
    if Result then
      FReportNeedsConstruction := False
    else
      ShowMessage(Msg);
  end else
    Result := True;
end;

procedure TDlgLKPrintBase.ckbxReportTitleClick(Sender: TObject);
begin
  TfrxReportTitle(frxReport1.FindObject('ReportTitle1')).Visible :=
    ckbxReportTitle.Checked;
end;

function TDlgLKPrintBase.ConstructReport: string;
begin
  TfrxReportPage(frxReport1.FindObject('Page1')).SetDefaults;
  TfrxMemoView(frxReport1.FindObject('MemoReportTitle')).Memo.Text :=
    eReportTitle.Text;
  Result := 'success';
end;

procedure TDlgLKPrintBase.DoAfterPrintReport;
begin
  //
end;

procedure TDlgLKPrintBase.DoBeforePrint;
begin
  //
end;

procedure TDlgLKPrintBase.DoGetValue(const VarName: string; var Value: Variant);
begin
  //
end;

procedure TDlgLKPrintBase.DoNewGetValue(Sender: TObject; const VarName: string;
    var Value: Variant);
begin
  //
end;

procedure TDlgLKPrintBase.Execute(const AReportName: string; Autoexecute:
    Boolean);
begin
  ReportName := AReportName;
  if Autoexecute then
  begin
    Show;
    btnPrint.OnClick(btnPrint);
  end
  else
    ShowModal;
end;

procedure TDlgLKPrintBase.ExecuteBtnEmail;
begin
  Showmessage('Sorry - Email not implemented for this report');
end;

procedure TDlgLKPrintBase.ExecuteBtnHTML;
begin
  if CheckReportConstructed then
  begin
    frxReport1.Export(frxHTMLExport1);
    frxHTMLExport1.FileName := PrintFileName + '.html';
    frxHTMLExport1.ShowModal;
  end;
end;

procedure TDlgLKPrintBase.ExecuteBtnLayout;
begin
  Showmessage('Sorry - Layout not implemented for this report');
end;

procedure TDlgLKPrintBase.ExecuteBtnPDF;
begin
  if CheckReportConstructed then
  begin
    frxReport1.Export(frxPDFExport1);
    frxPDFExport1.FileName := PrintFileName + '.pdf';
    frxPDFExport1.Start;
  end;
end;

procedure TDlgLKPrintBase.ExecuteBtnRemember;
begin
  Showmessage('Sorry - Remember not implemented for this report');
end;

procedure TDlgLKPrintBase.ExecuteBtnSpreadsheet;
begin
  Showmessage('Sorry - Spreadsheet not implemented for this report');
end;

procedure TDlgLKPrintBase.FormCreate(Sender: TObject);
begin
  inherited;
  FReportName := 'Default';
  eReportTitle.Text := '';
  FReportNeedsConstruction := True;
end;

procedure TDlgLKPrintBase.frxReport1AfterPrintReport(Sender: TObject);
begin
  DoAfterPrintReport;
end;

procedure TDlgLKPrintBase.frxReport1BeforePrint(Sender: TfrxReportComponent);
begin
  DoBeforePrint;
end;

procedure TDlgLKPrintBase.frxReport1GetValue(const VarName: string; var Value:
    Variant);
begin
  DoGetValue(VarName, Value);
end;

procedure TDlgLKPrintBase.frxReport1NewGetValue(Sender: TObject; const VarName:
    string; var Value: Variant);
begin
  DoNewGetValue(Sender, VarName, Value);
end;

function TDlgLKPrintBase.PrintFileName: string;
begin
  Result := InstallFolder + PathDelim + FReportName;
end;

procedure TDlgLKPrintBase.SetReportName(const Value: string);
begin
  FReportName := Value;
  FReportNeedsConstruction := True;
  if cbxReportName.Items.IndexOf(FReportName) < 0 then
    cbxReportName.Items.Add(FReportName);
  cbxReportName.ItemIndex := cbxReportName.Items.IndexOf(FReportName);
end;

end.
