{ ***************************************************************************** }
{ }
{ This file is part of the LKRecords components Library }
{ }
{ See the file COPYING.LGPL.txt included in this distribution, }
{ for details about the Licence and Copyright. }
{ }
{ This code is distributed in the hope that it will be useful, }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. }
{ }
{ Author: Leslie John Kaye :    www.leskaye.com }
{ }
{ Abstract: }
{ Form implementing printing a TLKDataComponent }
{ }
{ ***************************************************************************** }

unit prLKRecordDlg;

interface

uses WinTypes, WinProcs, Classes, Graphics, System.UITypes,
  Forms, Controls, Buttons,
  StdCtrls, ExtCtrls, DB, DBTables, prLKSystemComponentDlg, Dialogs, ImgList,
  ActnList, sysUtils, DBCtrls, Printers, ComCtrls, ShellAPI,
  LKDlgs, prLKRecordFieldsDlg, LKDefine, LKBaseComponent, LKRecordsComponents,
  frxExportHTML, frxClass, frxExportPDF, frxDBSet;

type
  TDlgLKPrintRecord = class(TDlgLKPrintSystemComponent)
    procedure FormDestroy(Sender: TObject);
  private
    FDetailFilter: string;
    FDetailWasFiltered: Boolean;
    FMasterFilter: string;
    FMasterWasFiltered: Boolean;
  protected
    FDlgPrintRecordFields: TDlgLKPrintRecordFields;
    FMyPrintingComponent: TLKDataComponent;
    function ConstructReport: string; override;
    procedure DoBeforePrint; override;
    procedure ExecuteBtnEmail; override;
    procedure ExecuteBtnLayout; override;
    procedure ExecuteBtnRemember; override;
    procedure ExecuteBtnSpreadsheet; override;
    procedure SetReportName(const Value: string); override;
  public
    procedure Construct(APrintingComponent: TLKDataComponent; const aTitle:
        string); virtual;
    property MyPrintingComponent: TLKDataComponent read FMyPrintingComponent;
  end;

var
  DlgLKPrintRecord: TDlgLKPrintRecord;

implementation

uses LKutils;

{$R *.dfm}

{
  ****************************** TDlgLKPrintRecord *******************************
}
{
****************************** TDlgLKPrintRecord *******************************
}
procedure TDlgLKPrintRecord.Construct(APrintingComponent: TLKDataComponent;
    const aTitle: string);
begin
  FMyPrintingComponent := TLKDataComponent(APrintingComponent);
  inherited Construct(APrintingComponent, aTitle);
  btnRemember.Enabled := False;
  with FMyPrintingComponent do
  begin
    FMasterWasFiltered := Filtered;
    FMasterFilter := Filter;
    FDetailWasFiltered := TblDetail.Filtered;
    FDetailFilter := TblDetail.Filter;
  end;
  Application.CreateForm(TDlgLKPrintRecordFields, FDlgPrintRecordFields);
  FDlgPrintRecordFields.btnRemember := btnRemember;
  FDlgPrintRecordFields.Construct(MyPrintingComponent);
  with cbxReportName.Items do
    if (Count = 1) then
      Add('Single record');
  ReportName := 'Default';
end;

function TDlgLKPrintRecord.ConstructReport: string;
begin
  Result := inherited ConstructReport;
  if (Result = 'success') then
    Result :=
      FDlgPrintRecordFields.PrintSpecScroller.ConstructReport(frxReport1);
end;

procedure TDlgLKPrintRecord.DoBeforePrint;
begin
  inherited;
  // FDlgPrintRecordFields.PrintSpecScroller.PrintBefore(frxReport1);
end;

procedure TDlgLKPrintRecord.ExecuteBtnEmail;
// var
// Content: string;
// Recipients: TStringList;
begin
  inherited;
  // Recipients := TStringList.Create;
  // try
  // with FMyPrintingComponent.TblMaster do
  // if Assigned(FindField('EMAIL')) then
  // Recipients.Add(FieldByName('EMAIL').AsString);
  // Content := FDlgPrintRecordFields.PrintSpecPages.HTML;
  // {
  // EMail(const Originator, Subject, Content : string;
  // Recipients, AttachmentFiles : TStrings);
  // }
  // EMail('', FMyPrintingComponent.NameFriendly, Content, Recipients, nil);
  // finally Recipients.Free;
  // end;
end;

procedure TDlgLKPrintRecord.ExecuteBtnLayout;
var
  Again: Boolean;
  I: Integer;
  NewReportName: string;
begin
  with TLKViewDialog.Create(Self) do
    try
      DataType := vRadioGroup;
      Title := 'Report Layout wizard';
      HeaderCaption := 'Select an option';
      HeaderComments := 'The Default report should be modified as you wish and'
        + ' other layouts added for frequently used reports';
      ChoiceItems.Add('Modify ' + ReportName + ' layout');
      ChoiceItems.Add('Delete ' + ReportName + ' layout');
      ChoiceItems.Add('Create a new layout');
      Value := 0;
      Again := Execute;
      I := Value;
    finally
      Free;
    end;
  if Again then
    case I of
      0:
        ;
      1:
        begin
          if (MessageDlg('Confirm you want to Delete settings for ' +
            ReportName, mtWarning, [mbYes, mbAbort], 0) <> mrYes) then
            exit;
          FDlgPrintRecordFields.ForgetPrintPositions(ReportName);
          with cbxReportName do
          begin
            if (ReportName <> 'Default') then
              with Items do
                Delete(IndexOf(ReportName));
            Text := 'Default';
            OnChange(Self);
          end;
          exit;
        end; // 1
      2:
        begin
          with TLKViewDialog.Create(Self) do
            try
              Title := 'New report';
              HeaderCaption := 'Enter a name for the new report';
              HeaderComments :=
                'The Default report should be modified as you wish and' +
                ' other layouts added for frequently used reports';
              DataType := vString;
              Again := Execute;
              NewReportName := Value;
            finally
              Free;
            end;
          if not Again then
            exit;
          with cbxReportName do
          begin
            if Items.IndexOf(NewReportName) = -1 then
            begin
              Items.Add(NewReportName);
              FMyPrintingComponent.RememberSection('PrintReportNames',
                Items, True);
            end
            else
            begin
              Again := (MessageDlg(NewReportName +
                ' already exists : do you want to modify this report?',
                mtInformation, [mbYes, mbNo], 0) = mrYes);
              if not Again then
                exit;
            end;
            Text := NewReportName;
            OnChange(Self);
            btnRemember.Enabled := True;
          end; // cbxReportName
        end; // 2
    else
      exit;
    end; // case
  if not Again then
    exit;
  {
    if FMyPrintingComponent.PrintUseLayoutwizard then
    begin
    with TLKViewDialog.Create(Self) do
    try
    DataType := vBoolean;
    Title := 'Report Layout wizard';
    HeaderCaption := 'Enter a name for the new report';
    HeaderComments:= 'The Default report should be modified as you wish and'
    + ' other layouts added for frequently used reports';
    ChoiceItems.Add('Run Layout wizard');
    ChoiceItems.Add('Go to Layout editor');
    Value := True;
    OptionCaption := 'Never run Wizard';
    if not Execute then SysUtils.Abort;
    continue := Value;
    if OptionChecked then
    begin
    FMyPrintingComponent.PrintUseLayoutwizard := False;
    Showmessage('You can re-enable the Layout wizard in the Program options');
    end;
    finally Free; end;
    if continue then
    continue := (myDlgPrintRecordFields.LayoutwizardExecute = mrOK)
    else continue := True;
    end;
  }
  if Again then
    FDlgPrintRecordFields.ShowModal;
end;

procedure TDlgLKPrintRecord.ExecuteBtnRemember;
begin
  inherited ExecuteBtnRemember;
  FMyPrintingComponent.RememberSection('PrintReportNames',
    cbxReportName.Items, True);
  FDlgPrintRecordFields.RememberPrintPositions;
end;

procedure TDlgLKPrintRecord.ExecuteBtnSpreadsheet;
var
  CSVFileName: string;
  CSVFile: TextFile;
begin
  // myDlgPrintRecordFields.AccountsComponentPrinting.FindFirst;
  // FMyPrintingComponent.FindFirst;
  CSVFileName := PrintFileName + '.csv';
  AssignFile(CSVFile, CSVFileName);
  Rewrite(CSVFile);
  Screen.Cursor := crHourglass;
  try
    FDlgPrintRecordFields.PrintSpecScroller.Spreadsheet(CSVFile);
  finally
    CloseFile(CSVFile);
    Screen.Cursor := crDefault;
  end;
  ShellExecute(Application.Handle, 'open', PChar(CSVFileName), nil, nil, 1);
end;

procedure TDlgLKPrintRecord.FormDestroy(Sender: TObject);
begin
  FDlgPrintRecordFields.Free;
  inherited;
end;

procedure TDlgLKPrintRecord.SetReportName(const Value: string);
begin
  if (Value = '') then
    Exit;
  inherited SetReportName(Value);
  if Assigned(FDlgPrintRecordFields) then
    FDlgPrintRecordFields.ReportName := Value;
end;

end.
