{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form to persist field layouts for printing a TLKDataComponent          }
{                                                                             }
{*****************************************************************************}

unit prLKRecordFieldsDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, DB, DBTables, ComCtrls,
  Contnrs, ActnList, ExtDlgs, Dialogs, jpeg, ImgList,
  LKFormBaseDlg, prLKTableBasePrintRecordFieldsDlg, LKRecordsComponents,
    LKDefine, LKDlgs, LKBaseComponent, frxClass, frxDBSet;

type
  TLKDCDataBandPrintSpecPanel = class(TLKDataBandPrintSpecPanel)
  private
    function GetAccountsComponentPrinting: TLKDataComponent;
  public
    procedure ForgetPrintPositions(AReportName: string);
    function RecallPrintPositions(AReportName: string): Boolean;
    procedure RememberPrintPositions(AReportName: string);
    property AccountsComponentPrinting: TLKDataComponent read
        GetAccountsComponentPrinting;
  end;

  TLKDCPrintSpecificationScroller = class(TLKPrintSpecificationScroller)
  private
    FAccountsComponentPrinting: TLKDataComponent;
    procedure SetAccountsComponentPrinting(const Value: TLKDataComponent);
  protected
    procedure SetReportName(const Value: string); override;
  public
    procedure ForgetPrintPositions(AReportName: string);
    procedure RecallPrintPositions;
    procedure RememberPrintPositions;
    property AccountsComponentPrinting: TLKDataComponent read
        FAccountsComponentPrinting write SetAccountsComponentPrinting;
  end;

  TDlgLKPrintRecordFields = class(TDlgLKTableBasePrintRecordFields)
  protected
    procedure SetReportName(const Value: string); override;
  public
    AccountsComponentPrinting: TLKDataComponent;
    btnRemember: TButton;
    procedure Construct(AAccountsComponentPrinting: TLKDataComponent);
        reintroduce; overload;
    procedure ForgetPrintPositions(AReportName: string);
    procedure RememberPrintPositions;
    procedure SettingsChanged; override;
    procedure SettingsRecordChanged; override;
  end;

var
  DlgLKPrintRecordFields: TDlgLKPrintRecordFields;

implementation

uses LKutils, prLKRecordDlg;

{$R *.dfm}


// TODO:
//begin
//if IsPrintingComponentPage then
//begin
//  with TblMasterPanel do
//  begin
//    ckbxPrintAll.Checked := not (AReportName = 'Single record');
//    if ckbxPrintAll.Checked then
//      rgStyle.ItemIndex := 1 // List
//    else
//      rgStyle.ItemIndex := 2; // Fields
//  end;
//  with TblDetailPanel do
//  begin
//    rgStyle.ItemIndex := 1; // List
//    ckbxPrintAll.Checked := True;
//  end;
//end
//else
//begin
//  with TblMasterPanel do
//  begin
//    ckbxPrintAll.Checked := False;
//    rgStyle.ItemIndex := 0; // none
//  end;
//  with TblDetailPanel do
//  begin
//    ckbxPrintAll.Checked := False;
//    rgStyle.ItemIndex := 0; // none
//  end;
//end;
//// override if previously Remembered
//Result := TblMasterPanel.RecallPrintPositions(AReportName) and
//  TblDetailPanel.RecallPrintPositions(AReportName);
//end;
//end;



{
************************* TLKDCDataBandPrintSpecPanel **************************
}
procedure TLKDCDataBandPrintSpecPanel.ForgetPrintPositions(AReportName: string);
var
  IniSection: string;
begin
  if (TblName = '') or not Assigned(AccountsComponentPrinting) then
    Exit; // no tablename
  IniSection := AReportName + AccountsComponentPrinting.Name + TblName;
  AccountsComponentPrinting.Forget(IniSection, True);
end;

function TLKDCDataBandPrintSpecPanel.GetAccountsComponentPrinting:
    TLKDataComponent;
begin
  Result := TDlgLKPrintRecordFields(Owner).AccountsComponentPrinting;
end;

function TLKDCDataBandPrintSpecPanel.RecallPrintPositions(AReportName: string):
    Boolean;
var
  I: Integer;
  IniSection: string;
begin
  Result := True;
  if (TblName = '') or not Assigned(AccountsComponentPrinting) then
    exit; // no tablename
  IniSection := AReportName + AccountsComponentPrinting.Name + TblName;
  with AccountsComponentPrinting do
  begin
    try
      I := RecallInteger(IniSection, 'Style', -1, True);
      Result := (I > -1);
      if Result then
      begin
        rgStyle.ItemIndex := I;
        for I := 0 to FieldPrintSpecs.Count - 1 do
          with FieldPrintSpecs[I] do
            DecodeFieldSpecStr(RecallString(IniSection, psField.FieldName, '',
              True));
        ckbxPrintAll.Checked := RecallBool(IniSection, 'PrintAll',
          ckbxPrintAll.Checked, True);
        ckbxNewPage.Checked := RecallBool(IniSection, 'NewPage',
          ckbxNewPage.Checked, True);
        ListIndent := RecallFloat(IniSection, 'ListIndent', ListIndent, True);
        ckbxPrintListCaptions.Checked :=
          RecallBool(IniSection, 'ListPrintHead', ckbxPrintListCaptions.Checked,
            True);
        ckbxAllowSplit.Checked := RecallBool(IniSection, 'AllowSplit',
          ckbxAllowSplit.Checked, True);
        {
        if FckbxPrintAll.Checked then
         begin
         FilterPanel.Filter := RecallString(IniSection,'Filter',prTable.Filter,True);
         FilterPanel.Filtered := RecallBool(IniSection,'Filtered',prTable.Filtered,True);
         end;
         }
      end;
    except Forget(IniSection, True);
      Result := False;
    end;
  end;
end;

procedure TLKDCDataBandPrintSpecPanel.RememberPrintPositions(AReportName:
    string);
var
  I: Integer;
  IniSection: string;
begin
  if (TblName = '') or not Assigned(AccountsComponentPrinting) then
    Exit; // no tablename
  IniSection := AReportName + AccountsComponentPrinting.Name + TblName;
  with AccountsComponentPrinting do
  begin
    Forget(IniSection, true);
    for I := 0 to FieldPrintSpecs.Count - 1 do
      with TLKFieldPrintSpec(FieldPrintSpecs[I]) do
        RememberString(IniSection, psField.FieldName, EncodeFieldSpecStr, true);
    RememberInteger(IniSection, 'Style', rgStyle.ItemIndex, true);
    RememberBool(IniSection, 'PrintAll', ckbxPrintAll.Checked, true);
    RememberBool(IniSection, 'NewPage', ckbxNewPage.Checked, true);
    RememberFloat(IniSection, 'ListIndent', ListIndent, true);
    RememberBool(IniSection, 'ListPrintHead', ckbxPrintListCaptions.Checked,
      true);
    RememberBool(iniSection,'AllowSplot',ckbxAllowSplit.Checked);
    //RememberString(iniSection,'Filter',FilterPanel.Filter,true);
  end;
end;

{
*********************** TLKDCPrintSpecificationScroller ************************
}
procedure TLKDCPrintSpecificationScroller.ForgetPrintPositions(AReportName:
    string);
var
  I: Integer;
begin
  for I := 0 to DataBandPrintSpecPanels.Count - 1 do
    TLKDCDataBandPrintSpecPanel(
      DataBandPrintSpecPanels[I]).ForgetPrintPositions(AReportName);
end;

procedure TLKDCPrintSpecificationScroller.RecallPrintPositions;
var
  I: Integer;
  PrintPositionsRemembered: Boolean;
begin
  ListIndent := AccountsComponentPrinting.RecallInteger(ReportName,
    'ListIndent', 1, True);
  PrintPositionsRemembered := False;
  for I := 0 to DataBandPrintSpecPanels.Count - 1 do
    if TLKDCDataBandPrintSpecPanel(
      DataBandPrintSpecPanels[I]).RecallPrintPositions(ReportName) then
        PrintPositionsRemembered := True;
  if not PrintPositionsRemembered then
    CalculatePrintPositions;
end;

procedure TLKDCPrintSpecificationScroller.RememberPrintPositions;
var
  I: Integer;
begin
  AccountsComponentPrinting.RememberInteger(ReportName, 'ListIndent',
    ListIndent, True);
  for I := 0 to DataBandPrintSpecPanels.Count - 1 do
    TLKDCDataBandPrintSpecPanel(
      DataBandPrintSpecPanels[I]).RememberPrintPositions(ReportName);
    // to do
    // AccountsComponentPrinting to remenber FDataBandPrintSpecPanels[I].PrintPositions(ReportName);
end;

procedure TLKDCPrintSpecificationScroller.SetAccountsComponentPrinting(const
    Value: TLKDataComponent);
var
  CurrDataComponent: TLKDataComponent;
  CurrPanel: TLKDataBandPrintSpecPanel;
  IncludeDetail: Boolean;
  I: Integer;

  procedure AddPanel(ADataTable: TLKTable; const ATotalFieldNames: TStrings);
  var
    NewPanel: TLKDCDataBandPrintSpecPanel;
  begin
    NewPanel:= TLKDCDataBandPrintSpecPanel.Create(Owner);
    DataBandPrintSpecPanels.Insert(0, NewPanel);
    NewPanel.Parent := Self;
    NewPanel.Align := alBottom;
    NewPanel.ChildPrintSpecPanel := CurrPanel;
    NewPanel.Construct(ADataTable, ATotalFieldNames);
    CurrPanel := NewPanel;
  end;

  procedure AddCurrDataComponentPanels;
  begin
    with CurrDataComponent do
    begin
      if TblDetailUsed and IncludeDetail then
        AddPanel(TblDetail, TotalDetailFieldNames);
      AddPanel(TblMaster, TotalMasterFieldNames);
    end;
  end;

begin
  if (FAccountsComponentPrinting = Value) then
    Exit;
  FAccountsComponentPrinting := Value;
  CurrPanel := nil;
  if FDataBandPrintSpecPanels.Count > 0 then
    FDataBandPrintSpecPanels.Clear;
  CurrDataComponent := Value;
  IncludeDetail := True;
  while Assigned(CurrDataComponent) do
  begin
    AddCurrDataComponentPanels;
    IncludeDetail := (CurrDataComponent.MasterTable = tbDetail);
    CurrDataComponent := CurrDataComponent.MasterComponent;
  end;
  ListIndent := ListIndent;
  for I := 0 to DataBandPrintSpecPanels.Count - 1 do
    DataBandPrintSpecPanels[I].Align := alTop;
end;

procedure TLKDCPrintSpecificationScroller.SetReportName(const Value: string);
begin
  inherited SetReportName(Value);
  RecallPrintPositions;
end;

{
*************************** TDlgLKPrintRecordFields ****************************
}
procedure TDlgLKPrintRecordFields.Construct(AAccountsComponentPrinting:
    TLKDataComponent);
begin
  AccountsComponentPrinting := AAccountsComponentPrinting;
  FPrintSpecScroller := TLKDCPrintSpecificationScroller.Create(Self);
  with TLKDCPrintSpecificationScroller(FPrintSpecScroller) do
  begin
    Parent := Self;
    Align := alClient;
    AccountsComponentPrinting := AAccountsComponentPrinting;
  end;
  TLKDCPrintSpecificationScroller(FPrintSpecScroller).RecallPrintPositions;
end;

procedure TDlgLKPrintRecordFields.ForgetPrintPositions(AReportName: string);
begin
  Screen.Cursor := crHourGlass;
  try
    AccountsComponentPrinting.Forget(AReportName, True);
    TLKDCPrintSpecificationScroller(
      PrintSpecScroller).ForgetPrintPositions(AReportName);
  finally Screen.Cursor := crDefault;
  end;
end;

procedure TDlgLKPrintRecordFields.RememberPrintPositions;
begin
  Screen.Cursor := crHourGlass;
  try
    TLKDCPrintSpecificationScroller(PrintSpecScroller).RememberPrintPositions;
    AccountsComponentPrinting.RememberString(ReportName, 'Image',
      eImageTop.Text + ',' + eImageLeft.Text + ',' + PrintImageFileName + ',',
        True);
  finally Screen.Cursor := crDefault;
  end;
end;

procedure TDlgLKPrintRecordFields.SetReportName(const Value: string);
var
  s: string;

  function SGetNextValue: string;
  begin
    Result := Copy(s, 1, Pos(',', s) - 1);
    Delete(s, 1, Pos(',', s));
  end;

begin
  inherited SetReportName(Value);
  s := AccountsComponentPrinting.RecallString(ReportName, 'Image', '0,0,,',
    True);
  eImageTop.Text := SGetNextValue;
  eImageLeft.Text := SGetNextValue;
  PrintImageFileName := SGetNextValue;
end;

procedure TDlgLKPrintRecordFields.SettingsChanged;
begin
  inherited SettingsChanged;
  btnRemember.Enabled := True;
end;

procedure TDlgLKPrintRecordFields.SettingsRecordChanged;
begin
  inherited SettingsRecordChanged;
  btnRemember.Enabled := True;
end;

end.


