{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form to set field print properties for printing a TLKDataComponent     }
{                                                                             }
{*****************************************************************************}

unit prLKRecordPrintSpecDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, prLKTableBasePrintRecordFieldsDlg;

type
  TDlgLKRecordPrintSpecs = class(TForm)
    Bevel1: TBevel;
    Bevel2: TBevel;
    CancelBtn: TButton;
    dckbxPrCaption: TCheckBox;
    deCaptionX: TEdit;
    deCaptionY: TEdit;
    deCFontSize: TEdit;
    deColWidthR: TEdit;
    deFFontSize: TEdit;
    deFieldX: TEdit;
    deFieldY: TEdit;
    deJustify: TEdit;
    Label1: TLabel;
    Label10: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    lFieldName: TLabel;
    OKBtn: TButton;
    procedure dckbxPrCaptionClick(Sender: TObject);
    procedure deCaptionXExit(Sender: TObject);
    procedure deJustifyExit(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  public
    function Execute(AFieldPrintSpec : TLKFieldPrintSpec): TModalResult;
  end;

var
  DlgLKRecordPrintSpecs: TDlgLKRecordPrintSpecs;

implementation

uses LKUtils, LKDefine, frxClass;

{$R *.DFM}

{
**************************** TDlgLKRecordPrintSpecs ****************************
}
procedure TDlgLKRecordPrintSpecs.dckbxPrCaptionClick(Sender: TObject);
begin
  deCaptionX.Enabled := dckbxPrCaption.checked;
  deCaptionY.Enabled := dckbxPrCaption.checked;
end;

procedure TDlgLKRecordPrintSpecs.deCaptionXExit(Sender: TObject);
begin
  StrToInt(TEdit(Sender).text);
end;

procedure TDlgLKRecordPrintSpecs.deJustifyExit(Sender: TObject);
var s : string;
begin
  s := Uppercase(deJustify.Text);
  if (s = '') then s := 'L';
  case s[1] of
    'L','C','R' : exit;
  else
    raise LKError.Create('Justification specification must be L,C or R');
  end;
  deJustify.Text := s;
end;

function TDlgLKRecordPrintSpecs.Execute(AFieldPrintSpec : TLKFieldPrintSpec):
    TModalResult;
begin
  with AFieldPrintSpec do
  begin
    lFieldName.Caption := HeadingR;
    dckbxPrCaption.Checked := printCaption;
    deCaptionX.Text := IntToStr(CaptionX);
    deCaptionY.Text := IntToStr(CaptionY);
    deCFontSize.Text := IntToStr(CFontSize);
    deFieldX.Text := IntToStr(FieldX);
    deFieldY.Text := IntToStr(FieldY);
    deFFontSize.Text := IntToStr(FontSize);
    case Align of
      haRight : deJustify.Text := 'R';
      haCenter : deJustify.Text := 'C';
    else deJustify.Text := 'L';
    end;
    deColWidthR.Text := FloatToStr(ColWidthR);
  end;
  dckbxPrCaptionClick(self);
  Result := ShowModal;
  if (Result = mrOK) then with AFieldPrintSpec do
  begin
    printCaption := dckbxPrCaption.Checked;
    CaptionX := StrToInt(deCaptionX.Text);
    CaptionY := StrToInt(deCaptionY.Text);
    CFontSize := StrToInt(deCFontSize.Text);
    FieldX := StrToInt(deFieldX.Text);
    FieldY := StrToInt(deFieldY.Text);
    FontSize := StrToInt(deFFontSize.Text);
    case deJustify.Text[1] of
      'C' : Align := haCenter;
      'R' : Align := haRight;
    else Align := haLeft;
    end;
    ColWidthR := StrToInt(deColWidthR.Text);
  end;
end;

procedure TDlgLKRecordPrintSpecs.FormKeyPress(Sender: TObject; var Key: Char);
begin
  MoveMyFocus(self,key);
end;

end.
