{ ***************************************************************************** }
{ }
{ This file is part of the LKRecords components Library }
{ }
{ See the file COPYING.LGPL.txt included in this distribution, }
{ for details about the Licence and Copyright. }
{ }
{ This code is distributed in the hope that it will be useful, }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. }
{ }
{ Author: Leslie John Kaye :    www.leskaye.com }
{ }
{ Abstract: }
{ Form for printing a TStringGrid }
{ }
{ ***************************************************************************** }

unit prLKStringGrid;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Contnrs, System.Math,
  Winapi.ShellAPI, System.Variants, System.Classes, System.UITypes,Vcl.Graphics,
  Vcl.Controls, Vcl.Forms,  Vcl.Dialogs, Vcl.StdCtrls, Vcl.Buttons, Vcl.Grids,
  Vcl.ExtCtrls, Generics.Collections,
  prLKPrintBaseDlg, frxClass, frxExportHTML, frxExportPDF;

type
  TDlgLKStringGridPrinter = class(TDlgLKPrintBase)
    frxUserDataSet1: TfrxUserDataSet;
    procedure frxUserDataSet1Next(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    rfStringGrid: TStringGrid;
    FMemos: TObjectList<TfrxMemoView>;
  protected
    function ConstructReport: string; override;
  public
    procedure PrintStringGrid(const Title: string;
      AStringGrid: TStringGrid; AutoExecute: boolean);
  end;

var
  DlgLKStringGridPrinter: TDlgLKStringGridPrinter;

implementation

{$R *.DFM}

procedure TDlgLKStringGridPrinter.FormCreate(Sender: TObject);
begin
  inherited;
  FMemos := TObjectList<TfrxMemoView>.Create(False);
end;

procedure TDlgLKStringGridPrinter.FormDestroy(Sender: TObject);
begin
  inherited;
  FMemos.Free;
end;

procedure TDlgLKStringGridPrinter.frxUserDataSet1Next(Sender: TObject);
var
  X, Y: integer;
begin
  Y := frxUserDataSet1.RecNo;
  case Y of
    0: for X := 0 to rfStringGrid.ColCount - 1 do
         FMemos[X].Font.Style := [fsBold];
    1: for X := 0 to rfStringGrid.ColCount - 1 do
         FMemos[X].Font.Style := [];
  end;
  for X := 0 to rfStringGrid.ColCount - 1 do
//          if colwidths[Y] > 0 then // ...if column is visible...
    FMemos[X].Text := rfStringGrid.Cells[Y, X];
end;

procedure TDlgLKStringGridPrinter.PrintStringGrid(const Title: string;
      AStringGrid: TStringGrid; AutoExecute: boolean);
begin
  rfStringGrid := aStringGrid;
  ReportNeedsConstruction := True;
  Execute(Title, AutoExecute);
end;


function TDlgLKStringGridPrinter.ConstructReport: string;
var
  I: Integer;
  TotalWidth, ColWidth, ColWidthFactor, CurrLeft: Extended;
  DataBand: TfrxMasterData;
  Memo: TfrxMemoView;
  Page: TfrxReportPage;
begin
  Result := inherited;
  Page := TfrxReportPage(frxReport1.FindObject('Page1'));
  FMemos.Clear;
  DataBand := TfrxMasterData(frxReport1.FindObject('MasterBand1'));
  DataBand.Clear;
  TotalWidth := 0;
  for I := 0 to rfStringGrid.ColCount - 1 do
    TotalWidth := TotalWidth + rfStringGrid.ColWidths[I];
  if (TotalWidth > DataBand.Width) then
    Page.Orientation := TPrinterOrientation.poLandscape;
  ColWidthFactor := TotalWidth/DataBand.Width;
  CurrLeft := 0;
  for I := 0 to rfStringGrid.ColCount - 1 do
  begin
    Memo := TfrxMemoView.Create(DataBand);
    FMemos.Add(Memo);
    ColWidth := rfStringGrid.ColWidths[I]*ColWidthFactor;
    Memo.SetBounds(CurrLeft,0,ColWidth,20);
    CurrLeft := CurrLeft + ColWidth;
  end;
end;

end.
