{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Base Form for printing a TLKSystemComponent                              }
{                                                                             }
{*****************************************************************************}

unit prLKSystemComponentDlg;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons, ActnList,
  StdCtrls, ExtCtrls, DB, DBTables, prLKPrintBaseDlg, Dialogs, ImgList,
  SysUtils, DBCtrls, LKDefine, Printers, ComCtrls,
  LKRecordsComponents, LKdlgs, StdActns, frxExportHTML, frxClass, frxExportPDF,
  frxDBSet;

type
  TDlgLKPrintSystemComponent = class(TDlgLKPrintBase)
    procedure frxReport1AfterPrint(Sender: TfrxReportComponent);
  private
    FMyPrintingComponent: TLKSystemComponent;
  protected
    procedure Construct(APrintingComponent: TLKSystemComponent; const aTitle:
        string);
    procedure DoBeforePrint; override;
    procedure ExecuteBtnRemember; override;
    function PrintFileName: string; override;
    procedure SetReportName(const Value: string); override;
  end;

var
  DlgLKPrintSystemComponent: TDlgLKPrintSystemComponent;

implementation

uses LKutils;

{$R *.DFM}


{
************************** TDlgLKPrintSystemComponent **************************
}
procedure TDlgLKPrintSystemComponent.Construct(APrintingComponent:
    TLKSystemComponent; const aTitle: string);
begin
  FMyPrintingComponent := APrintingComponent;
  Caption := 'Print ' + FMyPrintingComponent.NameFriendly;
  if Assigned(FMyPrintingComponent.AccountsHub) then
    TfrxMemoView(frxReport1.FindObject('MemoEnterpriseName')).Text :=
      FMyPrintingComponent.AccountsHub.EnterpriseName;
  FMyPrintingComponent.RecallSection('PrintReportNames', cbxReportName.Items,
    True);
  with cbxReportName.Items do
    if (Count = 0) then
      Add('Default');
end;

procedure TDlgLKPrintSystemComponent.DoBeforePrint;
begin
  inherited;
  with FMyPrintingComponent do
  begin
    frxReport1.PrintOptions.Printer := RecallString(ReportName,'PrinterName',
      'Default', False);
    frxReport1.PrintOptions.Copies := RecallInteger(ReportName,'Copies',1,True);
  end;
end;

procedure TDlgLKPrintSystemComponent.ExecuteBtnRemember;
begin
  with FMyPrintingComponent do
  begin
    RememberString(ReportName, 'ReportTitle', eReportTitle.Text, True);
    RememberString(ReportName, 'PrintReportTitle',
      BooleanToYesNo(ckbxReportTitle.Checked), True);
  end;
end;

procedure TDlgLKPrintSystemComponent.frxReport1AfterPrint(Sender:
    TfrxReportComponent);
begin
  inherited;
  with FMyPrintingComponent do
  begin
    RememberInteger(ReportName, 'Copies', frxReport1.PrintOptions.Copies, True);
    RememberString(ReportName, 'PrinterName',
      Printer.Printers[Printer.PrinterIndex], PrintShareLayouts);
  end;
end;

function TDlgLKPrintSystemComponent.PrintFileName: string;
begin
  Result := IntToStr(Trunc(Now * 1000));
  System.Delete(Result, 1, Length(Result) - 7);
  Result := InstallFolder + PathDelim + FMyPrintingComponent.NameFriendly +
    Result;
end;

procedure TDlgLKPrintSystemComponent.SetReportName(const Value: string);
begin
  inherited SetReportName(Value);
  with FMyPrintingComponent do
  begin
    eReportTitle.Text := RecallString(ReportName, 'ReportTitle', NameFriendly,
      PrintShareLayouts);
    ckbxReportTitle.Checked := (RecallString(ReportName, 'PrintReportTitle',
      'Yes', PrintShareLayouts) = 'Yes');
  end;
end;

end.
