{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for printing a TLKTableBase                                       }
{                                                                             }
{*****************************************************************************}

unit prLKTableBasePrintRecordDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, prLKPrintBaseDlg, ImgList,
  ActnList, StdCtrls, Buttons, ExtCtrls, ShellAPI,
  LKUtils, LKDefine, LKRecordsComponents, prLKTableBasePrintRecordFieldsDlg,
  frxExportHTML, frxClass, frxExportPDF;

type
  TDlgLKTableBasePrintRecord = class(TDlgLKPrintBase)
    procedure rpPrinterAfterPrint(Sender: TObject);
  private
    FMyDlgPrintTableFields: TDlgLKTableBasePrintRecordFields;
    FMyPrintingTable: TLKTable;
  protected
    procedure ExecuteBtnEmail; override;
    procedure ExecuteBtnLayout; override;
    procedure ExecuteBtnRemember; override;
    procedure ExecuteBtnSpreadsheet; override;
    procedure SetReportName(const Value: string); override;
    property MyDlgPrintTableFields: TDlgLKTableBasePrintRecordFields
      read FMyDlgPrintTableFields;
  public
    procedure Construct(APrintingTable: TLKTable; const aTitle :
            string);
  end;

var
  DlgLKTableBasePrintRecord: TDlgLKTableBasePrintRecord;

implementation

{$R *.dfm}

{ TDlgLKTableBasePrintRecord }


{
************************* TDlgLKTableBasePrintRecord **************************
}
procedure TDlgLKTableBasePrintRecord.Construct(APrintingTable:
        TLKTable; const aTitle : string);
begin
  FMyPrintingTable := APrintingTable;
  Caption := 'Print ' + FMyPrintingTable.TableName;
  Application.CreateForm(TDlgLKTableBasePrintRecordFields,
    FMyDlgPrintTableFields);
  FMyDlgPrintTableFields.Construct(APrintingTable);
  with cbxReportName.Items do
    if (Count = 0) then
      Add('Default');
  ReportName := 'Default';
end;

procedure TDlgLKTableBasePrintRecord.ExecuteBtnEmail;
//var
//  Content: string;
//  Recipients: TStringList;
begin
  inherited;
//  Recipients := TStringList.Create;
//  try
//    with FMyPrintingTable do
//      if Assigned(FindField('EMAIL')) then
//        Recipients.Add(FieldByName('EMAIL').AsString);
//    Content := FMyDlgPrintTableFields.PrintSpecsPanel.HTML;
    {
    EMail(const Originator, Subject, Content : string;
     Recipients, AttachmentFiles : TStrings);
     }
//    EMail('', FMyPrintingTable.TableName, Content, Recipients, nil);
//  finally Recipients.Free;
//  end;
end;

procedure TDlgLKTableBasePrintRecord.ExecuteBtnLayout;
begin
  FMyDlgPrintTableFields.ShowModal;
end;

procedure TDlgLKTableBasePrintRecord.ExecuteBtnRemember;
begin
  inherited ExecuteBtnRemember;
  //
end;

procedure TDlgLKTableBasePrintRecord.ExecuteBtnSpreadsheet;
var
  CSVFileName: string;
  F: TextFile;
begin
  //myDlgPrintRecordFields.AccountsComponentPrinting.FindFirst;
  //prAccountsComponent.FindFirst;
  CSVFileName := PrintFileName + '.csv';
  AssignFile(F, CSVFileName);
  Rewrite(F);
  Screen.Cursor := crHourglass;
  try
    FMyDlgPrintTableFields.PrintSpecScroller.Spreadsheet(F);
  finally CloseFile(F);
    Screen.Cursor := crDefault;
  end;
  ShellExecute(Application.Handle, 'open', PChar(CSVFileName), nil, nil, 1);
end;

procedure TDlgLKTableBasePrintRecord.rpPrinterAfterPrint(Sender: TObject);
begin
  inherited;
  Screen.Cursor := crDefault;
  Close; //close print dialog after printing
end;

procedure TDlgLKTableBasePrintRecord.SetReportName(const Value: string);
begin
  if (Value = '') then
    Exit;
  inherited SetReportName(Value);
  FMyDlgPrintTableFields.ReportName := Value;
end;

end.

