{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form for printing a TLKTableBase record                                }
{                                                                             }
{*****************************************************************************}

unit prLKTableBasePrintRecordFieldsDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, System.Types,
  Graphics, Controls, Forms, Generics.Collections,
  Dialogs, LKFormBaseDlg, DB, DBTables, jpeg, pngImage, ImgList, ActnList, StdCtrls,
  ExtCtrls, Contnrs, LKRecordsComponents, LKDefine, LKDlgs, LKUtils, LKUtilsDB,
  StdActns, LKBaseComponent, WideStrings, Math,
  frxExportHTML, frxClass, frxExportPDF, frxDBSet;

type
  TLKDataBandLevel = (dblUndefined,dblMasterData,dblDetailData,dblSubdetailData,
    dblDataLevel4, dblDataLevel5, dblDataLevel6, dblTooManyLevels);
  TLKPrintSpecificationScroller = class;

  TLKFieldPrintSpec = class(TPanel)
  private
    ckbxCaptionR: TCheckBox;
    ckbxPrintThisL: TCheckBox;
    ckbxPrintThisR: TCheckBox;
    ckbxT: TCheckBox;
    eCaptionX: TEdit;
    eCaptionY: TEdit;
    eCFontSize: TEdit;
    eColWidthL: TEdit;
    eColWidthR: TEdit;
    eFAlign: TEdit;
    eFieldX: TEdit;
    eFieldY: TEdit;
    eFontSizeF: TEdit;
    eHeadingL: TEdit;
    eHeadingR: TEdit;
    FAdjColWidthL: Extended;
    FPSField: TField;
    procedure doPrintSpecCkbxCaptionRClick;
    procedure doPrintSpecCkbxPrintThisLClick;
    procedure doPrintSpecCkbxPrintThisRClick;
    procedure doPrintSpecFDblClick;
    procedure doPrintSpecPanelExit;
    function GetAlign: TfrxHAlign;
    function GetCaptionX: Integer;
    function GetCaptionY: Integer;
    function GetCFontSize: Integer;
    function GetColWidthL: Extended;
    function GetColWidthR: Extended;
    function GetFieldX: Integer;
    function GetFieldY: Integer;
    function GetFontSize: Integer;
    function GetHeadingL: string;
    function GetHeadingR: string;
    function GetprintCaption: Boolean;
    function GetprintThisL: Boolean;
    function GetprintThisR: Boolean;
    function GetprintTotal: Boolean;
    procedure OnPrintSpecCkbxCaptionRClick(Sender: TObject);
    procedure OnPrintSpecCkbxPrintThisLClick(Sender: TObject);
    procedure OnPrintSpecCkbxPrintThisRClick(Sender: TObject);
    procedure OnPrintSpecFDblClick(Sender: TObject);
    procedure OnPrintSpecPanelExit(Sender: TObject);
    procedure OnSettingsChanged(Sender: TObject);
    procedure OnSettingsRecordChanged(Sender: TObject);
    procedure SetAlign(const Value: TfrxHAlign);
    procedure SetCaptionX(Value: Integer);
    procedure SetCaptionY(Value: Integer);
    procedure SetCFontSize(Value: Integer);
    procedure SetColWidthL(Value: Extended);
    procedure SetColWidthR(Value: Extended);
    procedure SetFieldX(Value: Integer);
    procedure SetFieldY(Value: Integer);
    procedure SetFontSize(Value: Integer);
    procedure SetHeadingL(const Value: string);
    procedure SetHeadingR(const Value: string);
    procedure SetprintCaption(Value: Boolean);
    procedure SetprintThisL(Value: Boolean);
    procedure SetprintThisR(Value: Boolean);
    procedure SetprintTotal(Value: Boolean);
    procedure SetPSField(const Value: TField);
  protected
    procedure AssignFieldPropertiesTo(AFieldPrintSpec: TLKFieldPrintSpec);
        dynamic;
    procedure Resize; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure AssignFieldProperties(AFieldPrintSpec: TLKFieldPrintSpec);
    procedure Construct(AField: TField);
    procedure DecodeFieldSpecStr(const FieldSpecStr: string);
    function EncodeFieldSpecStr: string;
    function HasDataToPrint: Boolean;
    procedure ShowAsForList;
    procedure ShowAsForListWithTotals;
    procedure ShowAsForNone;
    procedure ShowAsForRecord;
    procedure ShowAsForTotalsOnly;
    property AdjColWidthL: Extended read FAdjColWidthL write FAdjColWidthL;
    property Align: TfrxHAlign read GetAlign write SetAlign;
    property CaptionX: Integer read GetCaptionX write SetCaptionX;
    property CaptionY: Integer read GetCaptionY write SetCaptionY;
    property CFontSize: Integer read GetCFontSize write SetCFontSize;
    property ColWidthL: Extended read GetColWidthL write SetColWidthL;
    property ColWidthR: Extended read GetColWidthR write SetColWidthR;
    property FieldX: Integer read GetFieldX write SetFieldX;
    property FieldY: Integer read GetFieldY write SetFieldY;
    property FontSize: Integer read GetFontSize write SetFontSize;
    property HeadingL: string read GetHeadingL write SetHeadingL;
    property HeadingR: string read GetHeadingR write SetHeadingR;
    property printCaption: Boolean read GetprintCaption write SetprintCaption;
    property printThisL: Boolean read GetprintThisL write SetprintThisL;
    property printThisR: Boolean read GetprintThisR write SetprintThisR;
    property printTotal: Boolean read GetprintTotal write SetprintTotal;
    property PSField: TField read FPSField write SetPSField;
  end;

  TLKDataBandPrintSpecPanel = class(TPanel)
  private
    FBandDataSet: TfrxDBDataset;
    FChildPrintSpecPanel: TLKDataBandPrintSpecPanel;
    FckbxAllowSplit: TCheckBox;
    FckbxCheckAllField: TCheckBox;
    FckbxCheckAllTotal: TCheckBox;
    FckbxNewPage: TCheckBox;
    FckbxPrintAll: TCheckBox;
    FckbxPrintListCaptions: TCheckBox;
    FDataBand: TfrxDataBand;
    FDataBandLevel: TLKDataBandLevel;
    FFullHeight: Integer;
    FgbFieldCkbx: TGroupBox;
    FHeaderBand: TfrxHeader;
    FIsPrintingComponentPanel: Boolean;
    FIsTotalsPage: Boolean;
    FListIndent: Double;
    FReport: TfrxReport;
    FrgStyle: TRadioGroup;
    FTotalsBand: TfrxFooter;
    HTMLWidth: Integer;
    l1: TLabel;
    l2: TLabel;
    ListHeading1: string;
    ListHeading2: string;
    procedure ckbxCheckAllClick(Sender: TObject);
    function GetTblName: string;
    function GetThereAreTotals: Boolean;
    procedure rgStyleClick(Sender: TObject);
  protected
    procedure AssignTablePropertiesTo(APrintSpecPanel:
        TLKDataBandPrintSpecPanel); dynamic;
    property IsPrintingComponentPanel: Boolean read FIsPrintingComponentPanel
        write FIsPrintingComponentPanel;
  public
    DoChanged: TNotifyEvent;
    FieldPrintSpecs: TObjectList<TLKFieldPrintSpec>;
    TotalFieldNames: TStrings;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure AssignTableProperties(APrintSpecPanel: TLKDataBandPrintSpecPanel);
    function CalculatePrintPositions(const NextTop: integer): Integer;
    function ChecktListWidthOkay(AReport: TfrxReport): string;
    procedure Construct(ADataTable: TLKTable; const ATotalFieldNames: TStrings);
    function ConstructReport(AReport: TfrxReport): string;
    function CreateReportBands(var NextTop: Extended): string;
    function HasDataToPrint: Boolean;
    procedure ShowAsForList;
    procedure ShowAsForNone;
    procedure ShowAsForRecord;
    procedure ShowAsForTotalsOnly;
    procedure Spreadsheet(CSVList: TStringList; var Column, Line: integer);
    function SpreadsheetHeadings: string;
    property BandDataSet: TfrxDBDataset read FBandDataSet;
    property ChildPrintSpecPanel: TLKDataBandPrintSpecPanel read
        FChildPrintSpecPanel write FChildPrintSpecPanel;
    property ckbxAllowSplit: TCheckBox read FckbxAllowSplit write
        FckbxAllowSplit;
    property ckbxCheckAllField: TCheckBox read FckbxCheckAllField write
        FckbxCheckAllField;
    property ckbxCheckAllTotal: TCheckBox read FckbxCheckAllTotal write
        FckbxCheckAllTotal;
    property ckbxNewPage: TCheckBox read FckbxNewPage write FckbxNewPage;
    property ckbxPrintAll: TCheckBox read FckbxPrintAll write FckbxPrintAll;
    property ckbxPrintListCaptions: TCheckBox read FckbxPrintListCaptions write
        FckbxPrintListCaptions;
    property DataBand: TfrxDataBand read FDataBand;
    property DataBandLevel: TLKDataBandLevel read FDataBandLevel write
        FDataBandLevel;
    property FullHeight: Integer read FFullHeight write FFullHeight;
    property gbFieldCkbx: TGroupBox read FgbFieldCkbx write FgbFieldCkbx;
    property HeaderBand: TfrxHeader read FHeaderBand;
    property IsTotalsPage: Boolean read FIsTotalsPage write FIsTotalsPage;
    property ListIndent: Double read FListIndent write FListIndent;
    property Report: TfrxReport read FReport write FReport;
    property rgStyle: TRadioGroup read FrgStyle write FrgStyle;
    property TblName: string read GetTblName;
    property ThereAreTotals: Boolean read GetThereAreTotals;
    property TotalsBand: TfrxFooter read FTotalsBand;
  end;

  TLKPrintSpecificationScroller = class(TScrollBox)
  private
    FListIndent: Integer;
    FMyReport: TfrxReport;
    FPageReportTotals: TLKDataBandPrintSpecPanel;
    FPrintImageFileName: string;
    FReportName: string;
    procedure SetListIndent(Value: Integer);
    procedure SetPageReportTotals(Value: TLKDataBandPrintSpecPanel);
  protected
    FDataBandPrintSpecPanels: TObjectList<TLKDataBandPrintSpecPanel>;
    procedure SetReportName(const Value: string); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function CalculatePrintPositions: Integer;
    function ConstructReport(AReport: TfrxReport): string;
    procedure Spreadsheet(const ACSVFile: TextFile);
    function SpreadsheetHeadings: string;
    property DataBandPrintSpecPanels: TObjectList<TLKDataBandPrintSpecPanel>
        read FDataBandPrintSpecPanels write FDataBandPrintSpecPanels;
    property ListIndent: Integer read FListIndent write SetListIndent default 0;
    property PageReportTotals: TLKDataBandPrintSpecPanel read FPageReportTotals
        write SetPageReportTotals;
    property PrintImageFileName: string read FPrintImageFileName write
        FPrintImageFileName;
    property ReportName: string read FReportName write SetReportName;
  end;

  TDlgLKTableBasePrintRecordFields = class(TForm)
    btnLoadImage: TButton;
    btnOK: TButton;
    btnRecalc: TButton;
    eImageLeft: TEdit;
    eImageTop: TEdit;
    Image1: TImage;
    Label1: TLabel;
    Label2: TLabel;
    LKHeaderPanel1: TLKHeaderPanel;
    OpenDialog1: TOpenDialog;
    Panel1: TPanel;
    procedure btnLoadImageClick(Sender: TObject);
    procedure btnRecalcClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    function GetPrintImageFileName: string;
    procedure SetPrintImageFileName(const Value: string);
  protected
    FPrintSpecScroller: TLKPrintSpecificationScroller;
    FReportName: string;
    procedure LoadImage;
    procedure SetReportName(const Value: string); virtual;
  public
    TotalFieldNames: TStrings;
    procedure Construct(ATablePrinting: TLKTable); overload; virtual;
    procedure SettingsChanged; virtual;
    procedure SettingsRecordChanged; virtual;
    property PrintImageFileName: string read GetPrintImageFileName write
        SetPrintImageFileName;
    property PrintSpecScroller: TLKPrintSpecificationScroller read
        FPrintSpecScroller;
    property ReportName: string read FReportName write SetReportName;
  end;

var
  DlgLKTableBasePrintRecordFields: TDlgLKTableBasePrintRecordFields;

implementation

{$R *.dfm}
uses prLKRecordPrintSpecDlg;


{
****************************** TLKFieldPrintSpec *******************************
}
constructor TLKFieldPrintSpec.Create(AOwner: TComponent);
begin
  inherited;
  Width := 312;
  Left := 4;
  BevelOuter := bvNone;
  Caption := '';
  Height := 16;
  //record
  eHeadingR := TEdit.Create(AOwner);
  with eHeadingR do
  begin
    Cursor := crArrow;
    Ctl3d := False;
    BorderStyle := bsNone;
    Left := 4;
    Height := 15;
    Width := 104;
    Top := 0;
    Parent := Self;
    Text := '';
  end;
  ckbxPrintThisR := TCheckBox.Create(AOwner);
  with ckbxPrintThisR do
  begin
    Caption := '';
    Left := 112;
    Width := 12;
    Height := 15;
    Top := 0;
    Parent := Self;
    TabStop := False;
    Checked := True;
  end;
  ckbxCaptionR := TCheckBox.Create(AOwner);
  with ckbxCaptionR do
  begin
    Caption := '';
    Left := 128;
    Width := 12;
    Height := 15;
    Top := 0;
    Parent := Self;
    TabStop := False;
    Checked := True;
  end;
  eCaptionX := TEdit.Create(AOwner);
  with eCaptionX do
  begin
    Cursor := crArrow;
    ctl3d := False;
    BorderStyle := bsNone;
    Left := 140;
    height := 15;
    Width := 23;
    Top := 0;
    Parent := Self;
    Text := '0';
  end;
  eCaptionY := TEdit.Create(AOwner);
  with eCaptionY do
  begin
    Cursor := crArrow;
    ctl3d := False;
    BorderStyle := bsNone;
    Left := 164;
    height := 15;
    Width := 23;
    Top := 0;
    Parent := Self;
    Text := '0';
  end;
  eCFontSize := TEdit.Create(AOwner);
  with eCFontSize do
  begin
    Cursor := crArrow;
    ctl3d := False;
    BorderStyle := bsNone;
    Left := 188;
    height := 15;
    Width := 15;
    Top := 0;
    Parent := Self;
    Text := '10';
  end;
  eFieldX := TEdit.Create(AOwner);
  with eFieldX do
  begin
    Cursor := crArrow;
    ctl3d := False;
    BorderStyle := bsNone;
    Left := 208;
    height := 15;
    Width := 23;
    Top := 0;
    Parent := Self;
    Text := '50';
  end;
  eFieldY := TEdit.Create(AOwner);
  with eFieldY do
  begin
    Cursor := crArrow;
    ctl3d := False;
    BorderStyle := bsNone;
    Left := 232;
    height := 15;
    Width := 23;
    Top := 0;
    Parent := Self;
    Text := '0';
  end;
  eFontSizeF := TEdit.Create(AOwner);
  with eFontSizeF do
  begin
    Cursor := crArrow;
    ctl3d := False;
    BorderStyle := bsNone;
    Left := 256;
    height := 15;
    Width := 15;
    Top := 0;
    Parent := Self;
    Text := '10';
  end;
  eFAlign := TEdit.Create(AOwner);
  with eFAlign do
  begin
    Cursor := crArrow;
    ctl3d := False;
    BorderStyle := bsNone;
    MaxLength := 1;
    Left := 272;
    height := 15;
    Width := 11;
    Top := 0;
    Parent := Self;
    Text := 'L';
  end;
  eColWidthR := TEdit.Create(AOwner);
  with eColWidthR do
  begin
    Cursor := crArrow;
    Hint := 'Print data width in millimeters';
    Showhint := True;
    ctl3d := False;
    BorderStyle := bsNone;
    left := 284;
    height := 15;
    Width := 23;
    Top := 0;
    Parent := Self;
    Text := '20';
  end;
  //list
  eHeadingL := TEdit.Create(AOwner);
  with eHeadingL do
  begin
    Cursor := crArrow;
    Ctl3d := False;
    BorderStyle := bsNone;
    Left := 4;
    Height := 15;
    Width := 104;
    Top := 0;
    Parent := Self;
    Text := '';
  end;
  ckbxPrintThisL := TCheckBox.Create(AOwner);
  with ckbxPrintThisL do
  begin
    Caption := '';
    Left := 112;
    height := 16;
    Width := 12;
    Top := 0;
    Parent := Self;
    TabStop := False;
    Checked := True;
  end;
  eColWidthL := TEdit.Create(AOwner);
  with eColWidthL do
  begin
    Cursor := crArrow;
    Hint := 'Print column width in millimeters';
    Showhint := True;
    ctl3d := False;
    BorderStyle := bsNone;
    left := 128;
    height := 15;
    Width := 23;
    Top := 0;
    Parent := Self;
    Text := '20';
  end;
  ckbxT := TCheckbox.Create(AOwner);
  with ckbxT do
  begin
    Parent := Self;
    Caption := '';
    top := 0;
    left := 208;
    height := 15;
    Width := 12;
    TabStop := False;
    Checked := False;
  end;
end;

procedure TLKFieldPrintSpec.AssignFieldProperties(AFieldPrintSpec:
    TLKFieldPrintSpec);
begin
  AFieldPrintSpec.AssignFieldPropertiesTo(Self);
end;

procedure TLKFieldPrintSpec.AssignFieldPropertiesTo(AFieldPrintSpec:
    TLKFieldPrintSpec);
begin
  //if (TblName = '') then exit; // no tablename
  AFieldPrintSpec.psField := FPSField;
  AFieldPrintSpec.HeadingR := HeadingR;
  AFieldPrintSpec.printThisR := printThisR;
  AFieldPrintSpec.printCaption := printCaption;
  AFieldPrintSpec.CaptionX := CaptionX;
  AFieldPrintSpec.CaptionY := CaptionY;
  AFieldPrintSpec.CFontSize := CFontSize;
  AFieldPrintSpec.FieldX := FieldX;
  AFieldPrintSpec.FieldY := FieldY;
  AFieldPrintSpec.FontSize := FontSize;
  AFieldPrintSpec.Align := Align;
  AFieldPrintSpec.ColWidthR := ColWidthR;
  AFieldPrintSpec.HeadingL := HeadingL;
  AFieldPrintSpec.printThisL := printThisL;
  AFieldPrintSpec.ColWidthL := ColWidthL;
  AFieldPrintSpec.printTotal := printTotal;
end;

procedure TLKFieldPrintSpec.Construct(AField: TField);
begin
  PSField := AField;
  OnDblClick := OnPrintSpecFDblClick;
  OnExit := OnPrintSpecPanelExit;
  //record
  eHeadingR.OnDblClick := OnPrintSpecFDblClick;
  with ckbxPrintThisR do
  begin
    OnClick := OnPrintSpecCkbxPrintThisRClick;
    OnDblClick := OnPrintSpecFDblClick;
  end;
  with ckbxCaptionR do
  begin
    OnClick := OnPrintSpecCkbxCaptionRClick;
    OnDblClick := OnPrintSpecFDblClick;
  end;
  with eCaptionX do
  begin
    OnDblClick := OnPrintSpecFDblClick;
    OnChange := OnSettingsChanged;
  end;
  with eCaptionY do
  begin
    OnDblClick := OnPrintSpecFDblClick;
    OnChange := OnSettingsChanged;
  end;
  with eCFontSize do
  begin
    OnDblClick := OnPrintSpecFDblClick;
    OnChange := OnSettingsChanged;
  end;
  with eFieldX do
  begin
    OnDblClick := OnPrintSpecFDblClick;
    OnChange := OnSettingsRecordChanged;
  end;
  with eFieldY do
  begin
    OnDblClick := OnPrintSpecFDblClick;
    OnChange := OnSettingsRecordChanged;
  end;
  with eFontSizeF do
  begin
    OnDblClick := OnPrintSpecFDblClick;
    OnChange := OnSettingsChanged;
  end;
  with eFAlign do
  begin
    OnDblClick := OnPrintSpecFDblClick;
    OnChange := OnSettingsChanged;
  end;
  with eColWidthR do
  begin
    OnDblClick := OnPrintSpecFDblClick;
    OnChange := OnSettingsChanged;
  end;
  //list
  with eHeadingL do
  begin
    OnDblClick := OnPrintSpecFDblClick;
    OnChange := OnSettingsChanged;
  end;
  with ckbxPrintThisL do
  begin
    OnClick := OnPrintSpecCkbxPrintThisLClick;
    OnDblClick := OnPrintSpecFDblClick;
  end;
  with eColWidthL do
  begin
    OnDblClick := OnPrintSpecFDblClick;
    OnChange := OnSettingsChanged;
  end;
  if Assigned(ckbxT) then
    with ckbxT do
    begin
      OnDblClick := OnPrintSpecFDblClick;
      OnClick := OnSettingsChanged;
    end;
end;

procedure TLKFieldPrintSpec.DecodeFieldSpecStr(const FieldSpecStr: string);
var
  S: string;

  function SGetNextValue: string;
  begin
    Result := Copy(S, 1, Pos(',', S) - 1);
    Delete(S, 1, Pos(',', S));
  end;

begin
  S := FieldSpecStr;
  if (S > '') then
  begin
    HeadingR := SGetNextValue;
    printThisR := (SGetNextValue = 'Y');
    printCaption := (SGetNextValue = 'Y');
    CaptionX := StrToInt(SGetNextValue);
    CaptionY := StrToInt(SGetNextValue);
    CFontSize := StrToInt(SGetNextValue);
    FieldX := StrToInt(SGetNextValue);
    FieldY := StrToInt(SGetNextValue);
    FontSize := StrToInt(SGetNextValue);
    case S[1] of
      'C': Align := haCenter;
      'R': Align := haRight;
    else
      Align := haLeft;
    end;
    SGetNextValue;
    ColWidthR := StrToInt(SGetNextValue);
    HeadingL := SGetNextValue;
    printThisL := (SGetNextValue = 'Y');
    ColWidthL := StrToInt(SGetNextValue);
    printTotal := (SGetNextValue = 'Y');
  end;
end;

procedure TLKFieldPrintSpec.doPrintSpecCkbxCaptionRClick;
var
  isEditing: Boolean;
begin
  isEditing := ckbxCaptionR.Checked and ckbxPrintThisR.Checked;
  EditSetStyle(eHeadingR, isEditing);
  EditSetStyle(eCaptionX, isEditing);
  EditSetStyle(eCaptionY, isEditing);
  EditSetStyle(eCFontSize, isEditing);
end;

procedure TLKFieldPrintSpec.doPrintSpecCkbxPrintThisLClick;
var
  isEditing: Boolean;
begin
  isEditing := ckbxPrintThisL.Checked;
  EditSetStyle(eHeadingL, isEditing);
  EditSetStyle(eColWidthL, isEditing);
end;

procedure TLKFieldPrintSpec.doPrintSpecCkbxPrintThisRClick;
var
  isEditing: Boolean;
begin
  isEditing := ckbxPrintThisR.Checked;
  CkbxCaptionR.Enabled := isEditing;
  doPrintSpecCkbxCaptionRClick;
  EditSetStyle(eFieldX, isEditing);
  EditSetStyle(eFieldY, isEditing);
  EditSetStyle(eFontSizeF, isEditing);
  EditSetStyle(eFAlign, isEditing);
  EditSetStyle(eColWidthR, isEditing);
end;

procedure TLKFieldPrintSpec.doPrintSpecFDblClick;
begin
  with TDlgLKRecordPrintSpecs.Create(Self) do
  begin
    try
      lFieldName.Caption := HeadingR;
      if (execute(Self) = mrOK) then // refresh the edits
        DecodeFieldSpecStr('');
    finally free;
    end;
  end;
end;

procedure TLKFieldPrintSpec.doPrintSpecPanelExit;
begin
  //update for any modifications
  EncodeFieldSpecStr;
end;

function TLKFieldPrintSpec.EncodeFieldSpecStr: string;

  function YN(Value: boolean): string;
  begin
    if Value then
      Result := 'Y'
    else
      Result := 'N';
  end;

begin
  Result := HeadingR + ','
    + YN(printThisR) + ','
    + YN(printCaption) + ','
    + IntToStr(CaptionX) + ','
    + IntToStr(CaptionY) + ','
    + IntToStr(CFontSize) + ','
    + IntToStr(FieldX) + ','
    + IntToStr(FieldY) + ','
    + IntToStr(FontSize) + ','
    + eFAlign.Text + ','
    + FloatToStr(ColWidthR) + ','
    + HeadingL + ','
    + YN(printThisL) + ','
    + FloatToStr(ColWidthL) + ','
    + YN(printTotal) + ',';
end;

function TLKFieldPrintSpec.GetAlign: TfrxHAlign;
begin
  case eFAlign.Text[1] of
    'C': Result := haCenter;
    'R': Result := haRight;
  else
    Result := haLeft;
  end;
end;

function TLKFieldPrintSpec.GetCaptionX: Integer;
begin
  Result := StrToInt(eCaptionX.Text);
end;

function TLKFieldPrintSpec.GetCaptionY: Integer;
begin
  Result := StrToInt(eCaptionY.Text);
end;

function TLKFieldPrintSpec.GetCFontSize: Integer;
begin
  Result := StrToInt(eCFontSize.Text);
end;

function TLKFieldPrintSpec.GetColWidthL: Extended;
begin
  Result := StrToFloat(eColWidthL.Text);
end;

function TLKFieldPrintSpec.GetColWidthR: Extended;
begin
  Result := StrToFloat(eColWidthR.Text);
end;

function TLKFieldPrintSpec.GetFieldX: Integer;
begin
  Result := StrToInt(eFieldX.Text);
end;

function TLKFieldPrintSpec.GetFieldY: Integer;
begin
  Result := StrToInt(eFieldY.Text);
end;

function TLKFieldPrintSpec.GetFontSize: Integer;
begin
  Result := StrToInt(eFontSizeF.Text);
end;

function TLKFieldPrintSpec.GetHeadingL: string;
begin
  Result := eHeadingL.Text;
end;

function TLKFieldPrintSpec.GetHeadingR: string;
begin
  Result := eHeadingR.Text;
end;

function TLKFieldPrintSpec.GetprintCaption: Boolean;
begin
  Result := ckbxCaptionR.Checked;
end;

function TLKFieldPrintSpec.GetprintThisL: Boolean;
begin
  Result := ckbxPrintThisL.Checked;
end;

function TLKFieldPrintSpec.GetprintThisR: Boolean;
begin
  Result := ckbxPrintThisR.Checked;
end;

function TLKFieldPrintSpec.GetprintTotal: Boolean;
begin
  if Assigned(ckbxT) then
    Result := ckbxT.Checked
  else
    Result := False;
end;

function TLKFieldPrintSpec.HasDataToPrint: Boolean;
begin
  if ckbxPrintThisL.Visible then
    Result := ckbxPrintThisL.Checked
  else if ckbxPrintThisR.Visible then
    Result := ckbxPrintThisR.Checked
  else Result := False;
end;

procedure TLKFieldPrintSpec.OnPrintSpecCkbxCaptionRClick(Sender: TObject);
begin
  DoPrintSpecCkbxCaptionRClick;
  OnSettingsRecordChanged(Sender);
end;

procedure TLKFieldPrintSpec.OnPrintSpecCkbxPrintThisLClick(Sender: TObject);
begin
  DoPrintSpecCkbxPrintThisLClick;
  OnSettingsChanged(Sender);
end;

procedure TLKFieldPrintSpec.OnPrintSpecCkbxPrintThisRClick(Sender: TObject);
begin
  DoPrintSpecCkbxPrintThisRClick;
  //OnPrintSpecCkbxCaptionRClick(Sender);
  OnSettingsRecordChanged(Sender);
end;

procedure TLKFieldPrintSpec.OnPrintSpecFDblClick(Sender: TObject);
begin
  DoPrintSpecFDblClick;
  OnSettingsChanged(Sender);
end;

procedure TLKFieldPrintSpec.OnPrintSpecPanelExit(Sender: TObject);
begin
  DoPrintSpecPanelExit;
end;

procedure TLKFieldPrintSpec.OnSettingsChanged(Sender: TObject);
begin
  with TDlgLKTableBasePrintRecordFields(Owner) do
    SettingsChanged;
end;

procedure TLKFieldPrintSpec.OnSettingsRecordChanged(Sender: TObject);
begin
  with TDlgLKTableBasePrintRecordFields(Owner) do
    SettingsRecordChanged;
end;

procedure TLKFieldPrintSpec.Resize;
var
  z: Double;
begin
  inherited Resize;
  z := Parent.Width / 318;
  Width := Trunc(z * 312);
  eHeadingR.Width := Trunc(z * 104);
  ckbxPrintThisR.Left := Trunc(z * 112);
  ckbxCaptionR.Left := Trunc(z * 128);
  eCaptionX.Left := Trunc(z * 140);
  eCaptionY.Left := Trunc(z * 164);
  eCFontSize.Left := Trunc(z * 188);
  eFieldX.Left := Trunc(z * 208);
  eFieldY.Left := Trunc(z * 232);
  eFontSizeF.Left := Trunc(z * 256);
  eFAlign.Left := Trunc(z * 272);
  eColWidthR.left := Trunc(z * 284);
  eHeadingL.Width := Trunc(z * 104);
  ckbxPrintThisL.Left := Trunc(z * 112);
  eColWidthL.left := Trunc(z * 128);
  if Assigned(ckbxT) then
    ckbxT.left := Trunc(z * 208);
end;

procedure TLKFieldPrintSpec.SetAlign(const Value: TfrxHAlign);
begin
  case Value of
    haRight: eFAlign.Text := 'R';
    haCenter: eFAlign.Text := 'C';
  else
    eFAlign.Text := 'L';
  end;
end;

procedure TLKFieldPrintSpec.SetCaptionX(Value: Integer);
begin
  eCaptionX.Text := IntToStr(Value);
end;

procedure TLKFieldPrintSpec.SetCaptionY(Value: Integer);
begin
  eCaptionY.Text := IntToStr(Value);
end;

procedure TLKFieldPrintSpec.SetCFontSize(Value: Integer);
begin
  eFontSizeF.Text := IntToStr(Value);
end;

procedure TLKFieldPrintSpec.SetColWidthL(Value: Extended);
begin
  eColWidthL.Text := FloatToStr(Value);
end;

procedure TLKFieldPrintSpec.SetColWidthR(Value: Extended);
begin
  eColWidthR.Text := FloatToStr(Value);
end;

procedure TLKFieldPrintSpec.SetFieldX(Value: Integer);
begin
  eFieldX.Text := IntToStr(Value);
end;

procedure TLKFieldPrintSpec.SetFieldY(Value: Integer);
begin
  eFieldY.Text := IntToStr(Value);
end;

procedure TLKFieldPrintSpec.SetFontSize(Value: Integer);
begin
  eCFontSize.Text := IntToStr(Value);
end;

procedure TLKFieldPrintSpec.SetHeadingL(const Value: string);
begin
  eHeadingL.Text := Value;
end;

procedure TLKFieldPrintSpec.SetHeadingR(const Value: string);
begin
  eHeadingR.Text := Value;
end;

procedure TLKFieldPrintSpec.SetprintCaption(Value: Boolean);
begin
  ckbxCaptionR.Checked := Value;
end;

procedure TLKFieldPrintSpec.SetprintThisL(Value: Boolean);
begin
  ckbxPrintThisL.Checked := Value;
end;

procedure TLKFieldPrintSpec.SetprintThisR(Value: Boolean);
begin
  ckbxPrintThisR.Checked := Value;
end;

procedure TLKFieldPrintSpec.SetprintTotal(Value: Boolean);
begin
  if Assigned(ckbxT) then
    ckbxT.Checked := Value;
end;

procedure TLKFieldPrintSpec.SetPSField(const Value: TField);
begin
  FPSField := Value;
  if Assigned(FPSField) then
    with FPSField do
    begin
      HeadingR := DisplayName;
      HeadingL := HeadingR;
      ColWidthR := Trunc(2 * (DisplayWidth + 1));
      case DataType of
        ftSmallInt, ftInteger: Align := haCenter;
        ftDate, ftTime, ftFloat: Align := haRight;
        ftCurrency, ftFMTBcd:
          begin
            ColWidthL := 22;
            Align := haRight;
          end;
      else
        ColWidthL := Trunc(2 * (displayWidth + 1));
      end; // case
      if not (DataType in [ftInteger, ftFloat, ftCurrency, ftBcd, ftFMTBcd])
        then
        FreeAndNil(ckbxT);
    end;
end;

procedure TLKFieldPrintSpec.ShowAsForList;
begin
  eHeadingR.Visible := False;
  ckbxPrintThisR.Visible := False;
  ckbxCaptionR.Visible := False;
  eCaptionX.Visible := False;
  eCaptionY.Visible := False;
  eCFontSize.Visible := False;
  eFieldX.Visible := False;
  eFieldY.Visible := False;
  eFontSizeF.Visible := False;
  eFAlign.Visible := False;
  eColWidthR.Visible := False;
  eHeadingL.Visible := True;
  ckbxPrintThisL.Visible := True;
  DoPrintSpecCkbxPrintThisLClick;
  eColWidthL.Visible := True;
  if Assigned(ckbxT) then
    ckbxT.Visible := True;
end;

procedure TLKFieldPrintSpec.ShowAsForListWithTotals;
begin
  ShowAsForList;
  if Assigned(ckbxT) then
    ckbxT.Visible := True;
end;

procedure TLKFieldPrintSpec.ShowAsForNone;
begin
  eHeadingR.Visible := False;
  ckbxPrintThisR.Visible := False;
  ckbxCaptionR.Visible := False;
  eCaptionX.Visible := False;
  eCaptionY.Visible := False;
  eCFontSize.Visible := False;
  eFieldX.Visible := False;
  eFieldY.Visible := False;
  eFontSizeF.Visible := False;
  eFAlign.Visible := False;
  eColWidthR.Visible := False;
  eHeadingL.Visible := False;
  ckbxPrintThisL.Visible := False;
  doPrintSpecCkbxPrintThisLClick;
  eColWidthL.Visible := False;
  if Assigned(ckbxT) then
    ckbxT.Visible := False;
end;

procedure TLKFieldPrintSpec.ShowAsForRecord;
begin
  eHeadingR.Visible := True;
  ckbxPrintThisR.Visible := True;
  DoPrintSpecCkbxPrintThisRClick;
  ckbxCaptionR.Visible := True;
  eCaptionX.Visible := True;
  eCaptionY.Visible := True;
  eCFontSize.Visible := True;
  eFieldX.Visible := True;
  eFieldY.Visible := True;
  eFontSizeF.Visible := True;
  eFAlign.Visible := True;
  eColWidthR.Visible := True;
  eHeadingL.Visible := False;
  ckbxPrintThisL.Visible := False;
  eColWidthL.Visible := False;
  if Assigned(ckbxT) then
    ckbxT.Visible := False;
end;

procedure TLKFieldPrintSpec.ShowAsForTotalsOnly;
var
  ShowThis: Boolean;
begin
  showThis := Assigned(ckbxT);
  eHeadingR.Visible := False;
  ckbxPrintThisR.Visible := False;
  ckbxCaptionR.Visible := False;
  eCaptionX.Visible := False;
  eCaptionY.Visible := False;
  eCFontSize.Visible := False;
  eFieldX.Visible := False;
  eFieldY.Visible := False;
  eFontSizeF.Visible := False;
  eFAlign.Visible := False;
  eColWidthR.Visible := False;
  eHeadingL.Visible := ShowThis;
  ckbxPrintThisL.Visible := ShowThis;
  DoPrintSpecCkbxPrintThisLClick;
  eColWidthL.Visible := ShowThis;
  if ShowThis then
    ckbxT.Visible := True;
end;





{
************************** TLKDataBandPrintSpecPanel ***************************
}
constructor TLKDataBandPrintSpecPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FBandDataSet := TfrxDBDataset.Create(AOwner);
  FieldPrintSpecs := TObjectList<TLKFieldPrintSpec>.Create(False);
  Width := 790;
  Constraints.MinWidth := 790;
  ListIndent := 0;
  ListHeading1 := '';
  ListHeading2 := '';
  HTMLWidth := 0;
  IsPrintingComponentPanel := True;
  {FilterPanel := TLKFilterPanel.Create(AOwner);
  with FilterPanel do
   begin
   Parent := self;
   Top := 0; Left := 8;
   end;}
  rgStyle := TRadioGroup.Create(AOwner);
  with rgStyle do
  begin
    Parent := self;
    left := 4;
    top := 8;
    height := 81;
    width := 318;
    OnClick := rgStyleClick;
  end;
  ckbxPrintAll := TCheckbox.Create(AOwner);
  with ckbxPrintAll do
  begin
    Parent := Self;
    Checked := False;
    Caption := 'Print all records';
    top := 44;
    left := 160;
    height := 16;
    Width := 100;
  end;
  FckbxAllowSplit := TCheckbox.Create(AOwner);
  with FckbxAllowSplit do
  begin
    Parent := Self;
    Checked := False;
    Caption := 'Allow split to new page';
    top := 100;
    left := 8;
    height := 16;
    Width := 180;
  end;
  FckbxNewPage := TCheckbox.Create(AOwner);
  with FckbxNewPage do
  begin
    Parent := Self;
    Checked := False;
    Caption := 'Start New page';
    top := 100;
    left := 200;
    height := 16;
    Width := 120;
  end;
  FgbFieldCkbx := TGroupBox.Create(AOwner);
  with FgbFieldCkbx do
  begin
    Caption := ' Data Fields ';
    Parent := self;
    left := 4;
    top := 120;
    width := 318;
  end;
  ckbxPrintListCaptions := TCheckbox.Create(AOwner);
  with ckbxPrintListCaptions do
  begin
    Parent := Self;
    Checked := True;
    Caption := 'Print List Captions';
    top := 68;
    left := 160;
    height := 16;
    Width := 100;
  end;
  l1 := TLabel.Create(AOwner);
  with l1 do
  begin
    Top := 16;
    Left := 132;
    Parent := FgbFieldCkbx;
  end;
  l2 := TLabel.Create(AOwner);
  with l2 do
  begin
    Top := 16;
    Left := 212;
    Parent := FgbFieldCkbx;
  end;
  FckbxCheckAllField := TCheckbox.Create(AOwner);
  with FckbxCheckAllField do
  begin
    Parent := FgbFieldCkbx;
    OnClick := ckbxCheckAllClick;
    Caption := '';
    top := 1;
    left := 116;
    height := 16;
    Width := 12;
  end;
end;

destructor TLKDataBandPrintSpecPanel.Destroy;
begin
  FieldPrintSpecs.Free;
// FBandDataSet.Free;
//FDataBand.Free;
//FHeaderBand.Free;
//FTotalsBand.Free;
  inherited;
end;

procedure TLKDataBandPrintSpecPanel.AssignTableProperties(APrintSpecPanel:
    TLKDataBandPrintSpecPanel);
begin
  APrintSpecPanel.AssignTablePropertiesTo(Self);
end;

procedure TLKDataBandPrintSpecPanel.AssignTablePropertiesTo(APrintSpecPanel:
    TLKDataBandPrintSpecPanel);
var
  i: Integer;
begin
  //if (TblName = '') then exit; // no tablename
  APrintSpecPanel.rgStyle.ItemIndex := rgStyle.ItemIndex;
  APrintSpecPanel.ckbxPrintAll.Checked := ckbxPrintAll.Checked;
  APrintSpecPanel.ckbxNewPage.Checked := ckbxNewPage.Checked;
  APrintSpecPanel.ListIndent := ListIndent;
  APrintSpecPanel.ckbxPrintListCaptions.Checked :=
    ckbxPrintListCaptions.Checked;
  for i := 0 to FieldPrintSpecs.Count - 1 do
    APrintSpecPanel.FieldPrintSpecs[i].AssignFieldProperties(FieldPrintSpecs[i]);
end;

function TLKDataBandPrintSpecPanel.CalculatePrintPositions(const NextTop:
    integer): Integer;
var
  I, X, Y, PrintCount, ThisPrintField: Integer;
begin
  // in case of nothing to do
  Result := NextTop;
  if (TblName = '') then
    exit; // no table
  if not (rgStyle.ItemIndex = 2) then
    Exit;
  // count up fields to print
  PrintCount := 0;
  for I := 0 to (FieldPrintSpecs.Count - 1) do
    if FieldPrintSpecs[I].PrintThisR then
      inc(PrintCount);
  // calc new print Positions
  X := 15;
  Y := NextTop;
  ThisPrintField := 0;
  for I := 0 to (FieldPrintSpecs.Count - 1) do
    if FieldPrintSpecs[I].PrintThisR then
    begin
      //restart at top, column 2 if half way
      if (ThisPrintField = Trunc(PrintCount / 2)) then
      begin
        X := 110;
        Y := NextTop;
      end;
      FieldPrintSpecs[I].CaptionX := X;
      FieldPrintSpecs[I].CaptionY := Y;
      FieldPrintSpecs[I].FieldY := Y;
      // 35mm allowed for field Caption
      if FieldPrintSpecs[I].PrintCaption then
        FieldPrintSpecs[I].FieldX := X + 35
      else
        FieldPrintSpecs[I].FieldX := X;
      // write back to edits
      FieldPrintSpecs[I].DecodeFieldSpecStr('');
      Inc(Y, 4);
      Inc(ThisPrintField);
      if (Y > Result) then
        Result := Y;
    end;
end;

function TLKDataBandPrintSpecPanel.ChecktListWidthOkay(AReport: TfrxReport):
    string;
var
  I: Integer;
  WidthRequired, AlphasWidth: Extended;
  MyPage: TfrxReportPage;

  function CheckWidth(const MaxSquashFactor: Double): Boolean;
  var
    PrintWidth, ExcessWidth, ResizeFraction: Double;
    I: Integer;
  begin
    PrintWidth := MyPage.Width/fr01cm-MyPage.RightMargin-MyPage.LeftMargin;
    Result := (WidthRequired <= PrintWidth);
    if Result then
      Exit;
    ExcessWidth := WidthRequired - PrintWidth;
    Result :=  (AlphasWidth > MaxSquashFactor * ExcessWidth);
    if Result then //squash up to MaxSquashFactor
    begin
      ResizeFraction := (AlphasWidth - ExcessWidth) / AlphasWidth;
      for I := 0 to FieldPrintSpecs.Count - 1 do
        if  FieldPrintSpecs[I].PrintThisL
        and (FieldPrintSpecs[I].PSField.DataType = ftString) then
        begin
          FieldPrintSpecs[I].AdjColWidthL :=
            FieldPrintSpecs[I].ColWidthL * ResizeFraction;
          FieldPrintSpecs[I].FontSize :=
            Max(Trunc( FieldPrintSpecs[I].FontSize * ResizeFraction), 7);
        end;
    end;
  end;

begin
  Result := 'success';
  if (TblName = '') then
    Exit;
  case rgStyle.ItemIndex of
    0,2: Exit;
  end;
  MyPage := TfrxReportPage(FReport.FindObject('Page1'));
  WidthRequired := ListIndent;
  AlphasWidth := 0;
  for I := 0 to FieldPrintSpecs.Count - 1 do
    if FieldPrintSpecs[I].PrintThisL then
    begin
      WidthRequired := WidthRequired +  FieldPrintSpecs[I].ColWidthL;
      if (FieldPrintSpecs[I].PSField.DataType = ftString) then
        AlphasWidth := AlphasWidth +  FieldPrintSpecs[I].ColWidthL;
      FieldPrintSpecs[I].AdjColWidthL :=  FieldPrintSpecs[I].ColWidthL;
    end;
  if (MyPage.Orientation =TPrinterOrientation.poPortrait) and CheckWidth(4) then
    Exit;
  MyPage.Orientation := TPrinterOrientation.poLandscape;
  if not CheckWidth(3) then
    Result := 'Requested printout too wide for page!' + #10
      + 'Reduce number of fields to print.' + #10 + TblName;
end;

procedure TLKDataBandPrintSpecPanel.ckbxCheckAllClick(Sender: TObject);
var
  I: Integer;
  ckbx: TCheckBox;
begin
  //check/uncheck all checkboxes in groupbox
  if (sender is TCheckBox) then
    ckbx := TCheckBox(sender)
  else
    exit;
  with (ckbx.parent as TGroupbox) do
    //find groupbox belonging to clicked checkbox
    for I := 0 to (controlcount - 1) do //loop through controls in groupbox
      if (controls[I] is TLKFieldPrintSpec) then
        with TLKFieldPrintSpec(Controls[I]) do
          if ckbxPrintThisR.Visible then
            ckbxPrintThisR.Checked := ckbx.Checked
          else if ckbxPrintThisL.Visible then
            ckbxPrintThisL.Checked := ckbx.Checked;
end;

procedure TLKDataBandPrintSpecPanel.Construct(ADataTable: TLKTable; const
    ATotalFieldNames: TStrings);
var
  I, H: Integer;
  MyFieldPrintSpec: TLKFieldPrintSpec;
  CheckThis: Boolean;

begin
  FBandDataSet.DataSet := ADataTable;
  FBandDataSet.Name := 'frxDB'+ TblName;
  //ImageIndex := ADataTable.ImageIndex;
  //IsPrintingComponentPage := (ADataTable = AccountsComponentPrinting);
  //Caption := ADataTable.TableName;
  //  Width := 313;
  TotalFieldNames := ATotalFieldNames;
  with rgStyle do
  begin
    Items.Add('Print nothing');
    Items.Add('List (one line)');
    Items.Add('Individual fields');
    Items.Add('Totals (line) only');
    ItemIndex := 0;
  end;
   //FilterPanel.AccountsComponent := AccountsComponent;
  FrgStyle.Caption := TblName;
  FckbxCheckAllField.Checked := IsPrintingComponentPanel;
   {insert controls in group boxes from table field definitions}
  //with FieldPrintSpecs do begin for I := 0 to Count - 1 do Items[I].Free; Clear; end;
   //fields.Update;
  H := 1;
  for I := 0 to ADataTable.FieldCount - 1 do
  begin
    Inc(H);
    MyFieldPrintSpec := TLKFieldPrintSpec.Create(Owner);
    FieldPrintSpecs.Add(MyFieldPrintSpec);
    CheckThis := IsPrintingComponentPanel and (Pos(ADataTable.Fields[I].FieldName,
      ADataTable.MasterFields) = 0);
    with MyFieldPrintSpec do
    begin
      Parent := gbFieldCkbx;
      Top := 16 * H;
      Construct(ADataTable.Fields[I]);
      PrintThisR := CheckThis;
      PrintThisL := CheckThis;
      PrintTotal := Assigned(TotalFieldNames);
      if PrintTotal then
        PrintTotal := (TotalFieldNames.IndexOf(ADataTable.Fields[I].FieldName) >= 0)
          and IsPrintingComponentPanel;
      Visible := ADataTable.Fields[I].Visible;
    end; // with
  end; // for
  H := 24 + H * 16;
  FgbFieldCkbx.height := H;
  FullHeight := H + 140;
  Height := FullHeight;
end;

function TLKDataBandPrintSpecPanel.ConstructReport(AReport: TfrxReport): string;
const
  MemoHeight = 18.9;

  procedure ConstructForNone;
  begin
    FreeAndNil(FDataBand);
    FreeAndNil(FHeaderBand);
    FreeAndNil(FTotalsBand);
  end;

  procedure ConstructListTotals;
  var
    I: Integer;
    CurrLeft: Extended;
    MyMemo: TfrxMemoView;
  begin
    if not Assigned(FTotalsBand) then
      Exit;
    CurrLeft := ListIndent;
    for I := 0 to FieldPrintSpecs.Count - 1 do
    begin
      if FieldPrintSpecs[I].PrintThisL then
      begin
        if FieldPrintSpecs[I].PrintTotal then
        begin
          MyMemo := TfrxMemoView.Create(FTotalsBand);
          MyMemo.CreateUniqueName;
          MyMemo.Text := '[SUM(<'+ BandDataSet.Name + '."' +
            FieldPrintSpecs[I].PSField.FieldName + '">,' + FDataBand.Name +')]';
          MyMemo.Font.Size := FieldPrintSpecs[I].FontSize;
          MyMemo.HAlign := FieldPrintSpecs[I].Align;
          //MyMemo.DisplayFormat.Kind := TfrxFormatKind.fkNumeric;
          MyMemo.LineSpacing := 1.25;
          MyMemo.SetBounds(fr01cm*CurrLeft, 0,
            fr01cm*FieldPrintSpecs[I].AdjColWidthL,MemoHeight);
          MyMemo.Frame.Typ := [ftTop,ftBottom];
        end;
        CurrLeft := CurrLeft + FieldPrintSpecs[I].AdjColWidthL;
      end;
    end;
  end;

  procedure ConstructForList;
  var
    I: Integer;
    CurrLeft: Double;
    MyMemo: TfrxMemoView;
  begin
    CurrLeft := ListIndent;
    for I := 0 to FieldPrintSpecs.Count - 1 do
        if FieldPrintSpecs[I].PrintThisL then
        begin
          if Assigned(FHeaderBand) then
          begin
            MyMemo := TfrxMemoView.Create(FHeaderBand);
            MyMemo.CreateUniqueName;
            MyMemo.Font.Style := [fsBold];
            MyMemo.SetBounds(fr01cm*CurrLeft, 0,
              fr01cm*FieldPrintSpecs[I].AdjColWidthL, MemoHeight);
            MyMemo.Text := FieldPrintSpecs[I].HeadingL;
            MyMemo.HAlign := FieldPrintSpecs[I].Align;
            MyMemo.Frame.Typ := [ftLeft,ftRight,ftTop,ftBottom];
          end;
          MyMemo := TfrxMemoView.Create(FDataBand);
          MyMemo.CreateUniqueName;
          MyMemo.DataSet := BandDataSet;
          MyMemo.DataField := FieldPrintSpecs[I].PSField.FieldName;
          MyMemo.Font.Size := FieldPrintSpecs[I].FontSize;
          case FieldPrintSpecs[I].psField.DataType of
            ftFloat,
            ftCurrency, ftFMTBCD, ftBCD:
            begin
              MyMemo.HAlign := haRight;
//              MyMemo.DisplayFormat.Kind := TfrxFormatKind.fkNumeric;
            end;
            ftDate, ftDateTime:
              MyMemo.DisplayFormat.Kind := TfrxFormatKind.fkDateTime;
            ftBoolean, ftSmallint:
            begin
              MyMemo.HAlign := haCenter;
              MyMemo.DisplayFormat.Kind := TfrxFormatKind.fkBoolean;
            end;
          end; //case
          MyMemo.LineSpacing := 1;
          if FieldPrintSpecs[I].PrintTotal then
            MyMemo.Font.Style := [fsBold];
//  if printTotal then AddFieldToFloat(psField,ColumnTotal);
          MyMemo.SetBounds(fr01cm*CurrLeft, 0,
            fr01cm*FieldPrintSpecs[I].AdjColWidthL, MemoHeight);
          CurrLeft := CurrLeft + FieldPrintSpecs[I].AdjColWidthL;
        end;
    ConstructListTotals;
  end;

  procedure ConstructForRecord;
  var
    I: Integer;
    MyMemo: TfrxMemoView;
  begin
    for I := 0 to FieldPrintSpecs.Count - 1 do
      if FieldPrintSpecs[I].PrintThisR then
      begin
        FDataBand.Height :=
          Max(FDataBand.Height, fr01cm*(FieldPrintSpecs[I].FieldY+MemoHeight));
        if FieldPrintSpecs[I].PrintCaption then
          begin
            MyMemo := TfrxMemoView.Create(FDataBand);
            MyMemo.CreateUniqueName;
            MyMemo.AutoWidth := True;
            Mymemo.Font.Size := FieldPrintSpecs[I].CFontSize;
            MyMemo.SetBounds(fr01cm*(ListIndent+FieldPrintSpecs[I].CaptionX),
              fr01cm*FieldPrintSpecs[I].CaptionY, fr01cm*100, MemoHeight);
            MyMemo.Text := FieldPrintSpecs[I].HeadingR;
          end;
        MyMemo := TfrxMemoView.Create(FDataBand);
        MyMemo.CreateUniqueName;
        MyMemo.DataSet := BandDataSet;
        MyMemo.DataField := FieldPrintSpecs[I].PSField.FieldName;
        Mymemo.Font.Size := FieldPrintSpecs[I].FontSize;
// TODO:
//        case psField.DataType of
//          ftFloat:;
//          ftCurrency, ftFMTBCD, ftBCD: ;
//          ftBoolean: ;
//          ftSmallint: ;
//        end; //case
        if FieldPrintSpecs[I].PrintTotal then
          MyMemo.Font.Style := [fsBold];
//  if printTotal then AddFieldToFloat(psField,ColumnTotal);
        MyMemo.SetBounds(fr01cm*(ListIndent+FieldPrintSpecs[I].FieldX),
          fr01cm*FieldPrintSpecs[I].FieldY,
          fr01cm*FieldPrintSpecs[I].ColWidthR, MemoHeight);
        MyMemo.HAlign := FieldPrintSpecs[I].Align;
      end;
  end;

begin
  Result := 'success';
  try
    case rgStyle.ItemIndex of
      0: ConstructForNone;
      1: ConstructForList;
      2: ConstructForRecord;
      3: ConstructListTotals;
    end;
  except on E:Exception do
    Result := E.Message;
  end;
end;

function TLKDataBandPrintSpecPanel.CreateReportBands(var NextTop: Extended):
    string;
var
  MyPage: TfrxReportPage;

const
  BandHeight = 4.5;

  procedure CreateHeader;
  begin
    FHeaderBand := TfrxHeader.Create(MyPage);
    FHeaderBand.Name := 'HeaderBand' + TblName;
    FHeaderBand.Top := NextTop*fr01cm;
    FHeaderBand.Height := BandHeight*fr01cm;
    NextTop := NextTop + BandHeight;
  end;

begin
  FreeAndNil(FDataBand);
  FreeAndNil(FHeaderBand);
  FreeAndNil(FTotalsBand);
  FReport.DataSets.Add(BandDataSet);
  MyPage:= TfrxReportPage(FReport.FindObject('Page1'));
  Result := 'success';
  if HasDataToPrint then
    case rgStyle.ItemIndex of
    3: if ThereAreTotals then
         CreateHeader;
    1: CreateHeader;
    end;
  case FDataBandLevel of
    dblUndefined: Exit;
    dblMasterData:    FDataBand := TfrxMasterData.Create(MyPage);
    dblDetailData:    FDataBand := TfrxDetailData.Create(MyPage);
    dblSubdetailData: FDataBand := TfrxSubdetailData.Create(MyPage);
    dblDataLevel4:    FDataBand := TfrxDataBand4.Create(MyPage);
    dblDataLevel5:    FDataBand := TfrxDataBand5.Create(MyPage);
    dblDataLevel6:    FDataBand := TfrxDataBand6.Create(MyPage);
    else
    begin
      Result := 'Too many data levels to print';
      Exit;
    end;
  end;
  FDataBand.Name := 'DataBand' + TblName;
  FDataBand.Dataset := BandDataSet;
  if ckbxPrintAll.Checked then
  begin
    FBandDataSet.RangeBegin := rbFirst;
    FBandDataSet.RangeEnd := reLast;
  end else
  begin
    FBandDataSet.RangeBegin := rbCurrent;
    FBandDataSet.RangeEnd := reCurrent;
  end;
  FDataBand.AllowSplit := FckbxAllowSplit.Checked;
  FDataBand.Top := NextTop*fr01cm;
  FDataBand.Height := BandHeight*fr01cm;
  NextTop := NextTop + BandHeight;
  FDataBand.StartNewPage := FckbxNewPage.Checked;
  FDataBand.Visible := HasDataToPrint;
  if ThereAreTotals and HasDataToPrint then
  begin
    FTotalsBand := TfrxFooter.Create(MyPage);
    FTotalsBand.Name := 'TotalsBand' + TblName;
    FTotalsBand.Top := NextTop*fr01cm;
    FTotalsBand.Height := BandHeight*fr01cm;
    NextTop := NextTop + BandHeight;
  end;
end;

function TLKDataBandPrintSpecPanel.GetTblName: string;
begin
  Result := '';
  if Assigned(FBandDataSet.DataSet) then
    Result := TLKTable(FBandDataSet.DataSet).TableName;
end;

function TLKDataBandPrintSpecPanel.GetThereAreTotals: Boolean;
var
  I: Integer;
begin
  Result := False;
  case rgStyle.ItemIndex of
    1, 3: for I := 0 to FieldPrintSpecs.Count - 1 do
      with FieldPrintSpecs[I] do
        if PrintThisL and PrintTotal then
        begin
          Result := True;
          Break;
        end;
  end;
end;

function TLKDataBandPrintSpecPanel.HasDataToPrint: Boolean;
var I: Integer;
begin
  Result := False;
  for I := 0 to FieldPrintSpecs.Count -1 do
    if FieldPrintSpecs.Items[I].HasDataToPrint then
    begin
      Result := True;
      Break;
    end;
end;

procedure TLKDataBandPrintSpecPanel.rgStyleClick(Sender: TObject);
begin
  case rgStyle.ItemIndex of
    0: ShowAsForNone;
    1: ShowAsForList;
    2: ShowAsForRecord;
    3: ShowAsForTotalsOnly;
  end;
  ;
end;

procedure TLKDataBandPrintSpecPanel.ShowAsForList;
var
  I: Integer;
begin
  ckbxCheckAllField.Visible := True;
  ckbxPrintListCaptions.Visible := True;
  ckbxPrintAll.Visible := True;
  for I := 0 to FieldPrintSpecs.Count - 1 do
    FieldPrintSpecs[I].ShowAsForList;
  L1.Caption := 'Width';
  L2.Caption := 'Total';
  Visible := (TblName > '');
end;

procedure TLKDataBandPrintSpecPanel.ShowAsForNone;
var
  I: Integer;
begin
  ckbxCheckAllField.Visible := False;
  ckbxPrintAll.Visible := False;
  ckbxPrintListCaptions.Visible := False;
  for I := 0 to FieldPrintSpecs.Count - 1 do
    FieldPrintSpecs[I].ShowAsForNone;
end;

procedure TLKDataBandPrintSpecPanel.ShowAsForRecord;
var
  I: Integer;
begin
  ckbxCheckAllField.Visible := True;
  ckbxPrintListCaptions.Visible := False;
  ckbxPrintAll.Visible := True;
  for I := 0 to FieldPrintSpecs.Count - 1 do
    FieldPrintSpecs[I].ShowAsForRecord;
  L1.Caption := 'Caption';
  L2.Caption := 'Data';
  Visible := (TblName > '');
end;

procedure TLKDataBandPrintSpecPanel.ShowAsForTotalsOnly;
var
  I: Integer;
begin
  ckbxCheckAllField.Visible := False;
  ckbxPrintListCaptions.Visible := True;
  ckbxPrintAll.Visible := True;
  for I := 0 to FieldPrintSpecs.Count - 1 do
    FieldPrintSpecs[I].ShowAsForTotalsOnly;
  L1.Caption := '';
  L2.Caption := 'Total';
end;

procedure TLKDataBandPrintSpecPanel.Spreadsheet(CSVList: TStringList; var
    Column, Line: integer);
var
  MoreLines: Boolean;
  MyColumn, MyDetailColumn: Integer;

  procedure SpreadSheetAppend(var Column, Line: integer);
  var
    BlankPrefix: string;
    j: Integer;
    CurrColumn: Integer;
    CurrLine: Integer;

    procedure SpreadSheetListLine;
    var
      I: Integer;
      TextRecord: string;
      S: string;
    begin
      while (CSVList.Count <= CurrLine) do
        CSVList.Add(BlankPrefix);
      TextRecord := CSVList.Strings[CurrLine];
      for I := 0 to FieldPrintSpecs.Count - 1 do
        with FieldPrintSpecs[I] do
          if printThisL then
          begin
            case psField.DataType of
              ftCurrency: S := CurrencyToStrFOrBlank(GetCurrency(psField), 2);
            else
              S := psField.AsString;
            end;
            if (psField.DataType in
              [ftString, ftWideString, ftDate, ftDateTime, ftTime, ftCurrency,
                ftFMTBCD, ftBCD]) then
              S := '"' + S + '"';
            TextRecord := TextRecord + S + ',';
            Inc(CurrColumn); // for child/detail panel start
          end;
      CSVList.Strings[CurrLine] := TextRecord;
      inc(CurrLine);
    end;

    procedure SpreadSheetRecord;
    var
      I: integer;
      TextRecord: string;
      S: string;
    begin
      for I := 0 to FieldPrintSpecs.Count - 1 do
        with FieldPrintSpecs[I] do
          if printThisR then
          begin
            while (CurrLine >= CSVList.Count) do
              CSVList.Add(BlankPrefix);
            TextRecord := CSVList.Strings[CurrLine];
            case psField.DataType of
              ftCurrency: S := CurrencyToStrFOrBlank(GetCurrency(psField), 2);
            else
              S := psField.AsString;
            end;
            if (psField.DataType in
              [ftString, ftWideString, ftDate, ftDateTime, ftTime, ftCurrency,
                ftFMTBCD, ftBCD]) then
              S := '"' + S + '"';
            TextRecord := TextRecord + '"' + psField.DisplayName + '",' + S + ',';
            CSVList.Strings[CurrLine] := TextRecord;
            Inc(CurrLine);
          end;
      Inc(CurrColumn, 2); // for child/detail panel start
    end;

  begin
    if (TblName = '') then
      exit; // should not arise
    CurrColumn := Column;
    CurrLine := Line;
    try
      BlankPrefix := '';
      // construct a blank entry prefix for added lines
      for j := 0 to CurrColumn - 1 do
        BlankPrefix := BlankPrefix + ',';
      Application.ProcessMessages;
      case rgStyle.ItemIndex of
        1: SpreadSheetListLine;
        2: SpreadSheetRecord;
      end;
    finally
      Column := CurrColumn;
      Line := CurrLine;
    end;
  end;

begin
  MyColumn := Column;
  MoreLines := ckbxPrintAll.Checked;
  // if ckbxPrintAll.Checked then if not DataTable.FindFirst then exit;
  if MoreLines then
    if not FBandDataSet.DataSet.FindFirst then
      Exit;
  repeat if TLKTable(FBandDataSet.DataSet).HasValidData then
    begin
      Application.ProcessMessages;
      MyColumn := Column;
      SpreadSheetAppend(MyColumn, Line);
      MyDetailColumn := MyColumn;
      if Assigned(ChildPrintSpecPanel) then
        ChildPrintSpecPanel.SpreadSheet(CSVList, MyDetailColumn, Line);
      if ckbxPrintAll.Checked then
        MoreLines := FBandDataSet.DataSet.FindNext;
    end;
  until not MoreLines;
  Column := MyColumn;
end;

function TLKDataBandPrintSpecPanel.SpreadsheetHeadings: string;
var
  I: Integer;
begin
  if (TblName = '') then
    Exit; // no tablename
  // write headings
  case rgStyle.ItemIndex of
    1: for I := 0 to FieldPrintSpecs.Count - 1 do
        with FieldPrintSpecs[I] do
          if PrintThisL then
            Result := Result + '"' + HeadingL + '"' + ',';
    2: Result := Result + ',,';
  end;
end;

{
************************ TLKPrintSpecificationScroller *************************
}
constructor TLKPrintSpecificationScroller.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDataBandPrintSpecPanels := TObjectList<TLKDataBandPrintSpecPanel>.Create(False);
end;

destructor TLKPrintSpecificationScroller.Destroy;
begin
  FDataBandPrintSpecPanels.Free;
  inherited Destroy;
end;

function TLKPrintSpecificationScroller.CalculatePrintPositions: Integer;
var
  I: Integer;
begin
  Result := 0;
  for I := 0 to FDataBandPrintSpecPanels.Count - 1 do
    Result := FDataBandPrintSpecPanels[I].CalculatePrintPositions(Result);
end;

function TLKPrintSpecificationScroller.ConstructReport(AReport: TfrxReport):
    string;
var
  MyJpg: TJPEGImage;
  MyPng: TPNGImage;
  MyPage: TfrxReportPage;
  I, X: Integer;
  NextTop: Extended;

begin
  FMyReport := AReport;
  MyPage:= TfrxReportPage(FMyReport.FindObject('Page1'));
  if (PrintImageFileName > ' ') then
    if FileExists(PrintImageFileName) then
    try
      if (Pos('.bm', Lowercase(PrintImageFileName)) > 0) then
        MyPage.BackPicture.LoadFromFile(PrintImageFileName)
      else
      if (Pos('.jp', Lowercase(PrintImageFileName)) > 0) then
      begin
        MyJpg := TJPEGImage.Create;
        try
          MyJpg.LoadFromFile(PrintImageFileName);
          MyPage.BackPicture.Assign(MyJpg);
        finally MyJpg.Free;
        end;
      end
      else
      if (Pos('.png', Lowercase(PrintImageFileName)) > 0) then
      begin
        MyPng := TPNGImage.Create;
        try
          MyPng.LoadFromFile(PrintImageFileName);
          MyPage.BackPicture.Assign(MyPng);
        finally MyPng.Free;
        end;
      end;
    except on E: Exception do
      begin
        Result := E.Message;
        Exit;
      end;
    end;
  for I := 0 to FDataBandPrintSpecPanels.Count - 1 do
  begin
    FDataBandPrintSpecPanels[I].Report := AReport;
    if FDataBandPrintSpecPanels[I].HasDataToPrint then
    begin
      Result := FDataBandPrintSpecPanels[I].ChecktListWidthOkay(AReport);
      if (Result <> 'success') then
        Exit;
    end;
  end;
  X := 1;
  NextTop := fr01cm * 2.0;
  for I := 0 to FDataBandPrintSpecPanels.Count - 1 do
  begin
    FDataBandPrintSpecPanels[I].DataBandLevel := TLKDataBandLevel(X);
    Inc(X);
    Result := FDataBandPrintSpecPanels[I].CreateReportBands(NextTop);
    if (Result <> 'success') then
      Exit;
  end;
  for I := 0 to FDataBandPrintSpecPanels.Count - 1 do
  begin
    Result := FDataBandPrintSpecPanels[I].ConstructReport(AReport);
    if (Result <> 'success') then
      Exit;
  end;
end;

procedure TLKPrintSpecificationScroller.SetListIndent(Value: Integer);
var
  I, X: Integer;
begin
  X := 0;
  for I := 0 to FDataBandPrintSpecPanels.Count - 1 do
    with FDataBandPrintSpecPanels[I] do
    begin
      ListIndent := X;
      Inc(X, Value);
    end;
  FListIndent := Value;
end;

procedure TLKPrintSpecificationScroller.SetPageReportTotals(Value:
    TLKDataBandPrintSpecPanel);
var
  I: Integer;
begin
  if Assigned(Value) then
    for I := 0 to FDataBandPrintSpecPanels.Count - 1 do
      FDataBandPrintSpecPanels[I].IsTotalsPage := (FDataBandPrintSpecPanels[I] = Value);
  FPageReportTotals := Value;
end;

procedure TLKPrintSpecificationScroller.SetReportName(const Value: string);
begin
  FReportName := Value;
end;

procedure TLKPrintSpecificationScroller.Spreadsheet(const ACSVFile: TextFile);
var
  MyRecord: TStringList;
  L, C: Integer;
  I: Integer;
begin
  MyRecord := TStringList.Create;
  try
    // write column headings
    WriteLn(ACSVFile, SpreadsheetHeadings);
    // write the data
    L := 0;
    C := 0;
    FDataBandPrintSpecPanels[0].Spreadsheet(MyRecord, L, C);
    with MyRecord do
      for I := 0 to Count - 1 do
        WriteLn(ACSVFile, strings[I]);
  finally MyRecord.Free;
  end;
end;

function TLKPrintSpecificationScroller.SpreadsheetHeadings: string;
begin
  Result := FDataBandPrintSpecPanels[0].SpreadsheetHeadings;
end;

{
*********************** TDlgLKTableBasePrintRecordFields ***********************
}
procedure TDlgLKTableBasePrintRecordFields.btnLoadImageClick(Sender: TObject);
begin
  if OpenDialog1.Execute then
  begin
    PrintImageFileName := OpenDialog1.FileName;
    LoadImage;
  end
  else
    PrintImageFileName := '';
end;

procedure TDlgLKTableBasePrintRecordFields.btnRecalcClick(Sender: TObject);
begin
  btnRecalc.Enabled := False;
  FPrintSpecScroller.CalculatePrintPositions;
end;

procedure TDlgLKTableBasePrintRecordFields.Construct(ATablePrinting: TLKTable);

var
  NewPanel: TLKDataBandPrintSpecPanel;
begin
  FPrintSpecScroller := TLKPrintSpecificationScroller.Create(Self);
  NewPanel:= TLKDataBandPrintSpecPanel.Create(Self);
  with FPrintSpecScroller do
  begin
    Parent := Self;
    Align := alClient;
    DataBandPrintSpecPanels.Add(NewPanel);
  end;
  NewPanel.Parent := FPrintSpecScroller;
  NewPanel.Align := alTop;
  NewPanel.Construct(ATablePrinting, TotalFieldNames);
  FPrintSpecScroller.CalculatePrintPositions;
end;

procedure TDlgLKTableBasePrintRecordFields.FormShow(Sender: TObject);
//var
//ht: Integer;
//i: Integer;
begin
  btnRecalc.Enabled := False;
// TODO:
//ht := 0;
//with FPrintSpecScroller do
//begin
//  for i := 0 to PageCount - 1 do
//    with Pages[i] do
//    begin
//      if (ht < TblMasterPanel.FullHeight) then
//        ht := TblMasterPanel.FullHeight;
//      if (ht < TblDetailPanel.FullHeight) then
//        ht := TblDetailPanel.FullHeight;
//    end;
//  // Height := ht+ 20;
//end;
//ht := ht + 108 + LKHeaderPanel1.Height;
//if (ht > Screen.Height) then
//  ht := Screen.Height;
//if (Height < ht) then
//  Height := ht;
//Width := 752;
end;

function TDlgLKTableBasePrintRecordFields.GetPrintImageFileName: string;
begin
  Result := FPrintSpecScroller.PrintImageFileName;
end;

procedure TDlgLKTableBasePrintRecordFields.LoadImage;
var
  jp: TJPEGImage;
begin
  try
    if (Pos('.jp', Lowercase(PrintImageFileName)) = 0) then
      Image1.Picture.LoadFromFile(PrintImageFileName)
    else
    begin
      jp := TJPEGImage.Create;
      try
        jp.LoadFromFile(PrintImageFileName);
        Image1.Picture.Assign(jp);
      finally jp.Free;
      end;
    end;
  except PrintImageFileName := '';
  end;
end;

procedure TDlgLKTableBasePrintRecordFields.SetPrintImageFileName(const Value:
    string);
begin
  FPrintSpecScroller.PrintImageFileName := Value;
end;

procedure TDlgLKTableBasePrintRecordFields.SetReportName(const Value: string);
begin
  if (Value = '') then
    Exit;
  FPrintSpecScroller.ReportName := Value;
  //  s := AccountsComponentPrinting.RecallString(ReportName,'Image','0,0,,',True);
  //  eImageTop.Text := SGetNextValue;
  //  eImageLeft.Text := SGetNextValue;
  //  PrintImageFileName := SGetNextValue;
  FReportName := Value;
end;

procedure TDlgLKTableBasePrintRecordFields.SettingsChanged;
begin
  //
end;

procedure TDlgLKTableBasePrintRecordFields.SettingsRecordChanged;
begin
  btnRecalc.Enabled := True;
end;

end.




