{*****************************************************************************}
{                                                                             }
{   This file is part of the LKUtilities components Library                   }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      TLKComponent and utility components                                }
{                                                                             }
{*****************************************************************************}

unit LKBaseComponent;

{$ZEROBASEDSTRINGS OFF}

interface

uses
  Windows, Messages, System.UITypes, System.SysUtils, System.Variants,
  System.Classes, System.Actions, System.Types, Graphics, Controls, Forms,
  Dialogs, Contnrs, ComCtrls, StdCtrls, ExtCtrls, Buttons, ShellAPI,
  Menus, ImgList, IniFiles, ActnList,
  JvTypes, JvExControls, JvComponent, JvOutlookBar, JvGradientHeaderPanel,
  xmldom, XMLIntf, msxmldom, XMLDoc, ShlObj,
  LKFormBaseDlg, LKDefine, LKProgressDlg;

const

  TD_ICON_BLANK = 100;
  TD_ICON_WARNING = 101;
  TD_ICON_QUESTION = 102;
  TD_ICON_ERROR = 103;
  TD_ICON_INFORMATION = 104;
  TD_ICON_BLANK_AGAIN = 105;
  TD_ICON_SHIELD = 106;

  TD_OK = 1;
  TD_YES = 2;
  TD_NO = 4;
  TD_CANCEL = 8;
  TD_RETRY = 16;
  TD_CLOSE = 32;

  DLGRES_OK = 1;
  DLGRES_CANCEL = 2;
  DLGRES_RETRY = 4;
  DLGRES_YES = 6;
  DLGRES_NO = 7;
  DLGRES_CLOSE = 8;

type
  TLKHeaderPanel = class(TCustomPanel)
  private
    FHeaderMemo: TMemo;
    FHeaderPanel: TJvGradientHeaderPanel;
    FHelpURI: string;
    FLabelHelpURI: TLabel;
    procedure DoLabelHelpURIClick(Sender: TObject);
    function GetCaption: TCaption;
    function GetComments: TStrings;
    function GetInheritedPopupMenu: TPopupMenu;
    procedure SetCaption(const Value: TCaption);
    procedure SetComments(Value: TStrings);
    procedure SetHelpURI(const Value: string);
    procedure SetPopupMenu(Value: TPopupMenu);
  protected
    procedure Loaded; override;
    property HeaderMemo: TMemo read FHeaderMemo write FHeaderMemo;
    property HeaderPanel: TJvGradientHeaderPanel read FHeaderPanel write
        FHeaderPanel;
  public
    constructor Create(AOwner: TComponent); override;
    property DockManager;
  published
    property Align;
    property Alignment;
    property Anchors;
    property AutoSize;
    property BevelEdges;
    property BevelInner;
    property BevelKind;
    property BevelOuter;
    property BevelWidth;
    property BiDiMode;
    property BorderStyle;
    property BorderWidth;
    property Caption read GetCaption write SetCaption;
    property Color;
    property Comments: TStrings read GetComments write SetComments;
    property Constraints;
    property Ctl3D;
    property DockSite;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property FullRepaint;
    property HelpURI: string read FHelpURI write SetHelpURI;
    property Locked;
    property OnAlignInsertBefore;
    property OnAlignPosition;
    property OnCanResize;
    property OnClick;
    property OnConstrainedResize;
    property OnContextPopup;
    property OnDblClick;
    property OnDockDrop;
    property OnDockOver;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnGetSiteInfo;
    property OnMouseActivate;
    property OnMouseDown;
    property OnMouseEnter;
    property OnMouseLeave;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnStartDock;
    property OnStartDrag;
    property OnUnDock;
    property Padding;
    property ParentBackground;
    property ParentBiDiMode;
    property ParentColor;
    property ParentCtl3D;
    property ParentShowHint;
    property PopupMenu read GetInheritedPopupMenu write SetPopupMenu;
    property ShowHint;
    property UseDockManager default True;
    property VerticalAlignment;
    property Visible;
  end;

  TLKProgressDialog = class(TComponent)
  private
    FDlgProgress: TDlgLKProgress;
    FOnClose: TCloseEvent;
    FOnShow: TNotifyEvent;
    FMax: Integer;
    FMin: Integer;
    FStatusPanel0Width: Integer;
    FTitle: string;
    FProgressLabelCaption: string;
    FStep: Integer;
    FStatusPanel0Text: string;
    FPosition: Integer;
    FStatusPanel1Text: string;
    function GetAborted: Boolean;
    procedure SetMax(Value: Integer);
    procedure SetMin(Value: Integer);
    procedure SetPosition(Value: Integer);
    procedure SetProgressLabelCaption(const Value: string);
    procedure SetStatusPanel0Text(const Value: string);
    procedure SetStatusPanel0Width(Value: Integer);
    procedure SetStatusPanel1Text(const Value: string);
    procedure SetStep(Value: Integer);
    procedure SetTitle(const Value: string);
  protected
    procedure DoOnClose(Sender: TObject; var Action: TCloseAction);
    procedure DoOnShow;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Close;
    procedure Execute;
    procedure Show(const ATitle: string);
    procedure StepIt;
    property Aborted: Boolean read GetAborted;
    property Max: Integer read FMax write SetMax;
    property Min: Integer read FMin write SetMin;
    property Position: Integer read FPosition write SetPosition;
    property Step: Integer read FStep write SetStep;
  published
    property ProgressLabelCaption: string read FProgressLabelCaption write
        SetProgressLabelCaption;
    property StatusPanel0Text: string read FStatusPanel0Text write
        SetStatusPanel0Text;
    property StatusPanel0Width: Integer read FStatusPanel0Width write
        SetStatusPanel0Width;
    property StatusPanel1Text: string read FStatusPanel1Text write
        SetStatusPanel1Text;
    property Title: string read FTitle write SetTitle;
    property OnClose: TCloseEvent read FOnClose write FOnClose;
    property OnShow: TNotifyEvent read FOnShow write FOnShow;
  end;

  TLKViewDialog = class(TComponent)
  private
    FChoiceItems: TStrings;
    FDataType: TLKVDDataType;
    FHeaderCaption: string;
    FHeaderComments: string;
    FHint: string;
    FOnClose: TNotifyEvent;
    FOnShow: TNotifyEvent;
    FOptionCaption: string;
    FOptionChecked: Boolean;
    FTitle: string;
    FValue: Variant;
    procedure SetChoiceItems(NewValue: TStrings);
    procedure SetDataType(const AValue: TLKVDDataType);
    procedure SetTitle(const Value: string);
  protected
    procedure DoOnClose;
    procedure DoOnShow;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Execute: Boolean;
    function Show(ATitle, AHint: string; AValue: variant; ADataType:
        TLKVDDataType): Boolean;
    property ChoiceItems: TStrings read FChoiceItems write SetChoiceItems;
  published
    property DataType: TLKVDDataType read FDataType write SetDataType nodefault;
    property HeaderCaption: string read FHeaderCaption write FHeaderCaption;
    property HeaderComments: string read FHeaderComments write FHeaderComments;
    property Hint: string read FHint write FHint;
    property OptionCaption: string read FOptionCaption write FOptionCaption;
    property OptionChecked: Boolean read FOptionChecked write FOptionChecked;
    property Title: string read FTitle write SetTitle;
    property Value: Variant read FValue write FValue;
    property OnClose: TNotifyEvent read FOnClose write FOnClose;
    property OnShow: TNotifyEvent read FOnShow write FOnShow;
  end;

  TLKZipper = class(TComponent)
  private
    FDlgUnZipTitle: string;
    FDlgZipTitle: string;
    FPassword: string;
    FShowMessages: Boolean;
  public
    constructor Create(AOwner: TComponent); override;
    function UnZip(TargetFolder, RestoreFile: string): Boolean;
    function Zip(Folder, ZipFileName, ZipFolder: string; ShowMessages,
        AskForDisk, Subfolders: boolean): Boolean;
    function ZipFiles(FileNames: TStrings; BaseFolder, ZipFileName, ZipFolder:
        string; AskForDisk: boolean): Boolean;
    property Password: string read FPassword write FPassword;
  published
    property DlgUnZipTitle: string read FDlgUnZipTitle write FDlgUnZipTitle;
    property DlgZipTitle: string read FDlgZipTitle write FDlgZipTitle;
    property ShowMessages: Boolean read FShowMessages write FShowMessages;
  end;

  TLKComponent = class(TComponent)
  private
    FApplicationDataPath: string;
    FFolderLocalSettings: string;
    FFolderSharedSettings: string;
    FHelpContext: THelpContext;
    FHelpContextSetup: THelpContext;
    FImageIndex: System.UITypes.TImageIndex;
    FIniFileName: string;
    FOutlookBarButton: TJvOutlookBarButton;
    FOutlookBarPage: TJvOutlookBarPage;
    FTreeNode: TTreeNode;
    function GetApplicationDataPath: string;
    function GetFolderLocalSettings: string;
    function GetFolderSharedSettings: string;
  protected
    FDlgSetupIsImplemented: Boolean;
    FFormView: TFormLKBase;
    FImages: TCustomImageList;
    FNameFriendly: string;
    FTreeNodeVisible: Boolean;
    procedure AssignTo(Dest: TPersistent); override;
    procedure DefaultExecuteAction(Sender: TObject); virtual;
    function DoSetup(const ForceIfSettingsExist: Boolean): TModalResult;
        virtual;
    function GetNameFriendly: string; virtual;
    function GetSettings: TModalResult; virtual;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation);
        override;
    procedure SaveProperties; virtual;
    procedure SetFolderLocalSettings(const Value: string); virtual;
    procedure SetFolderSharedSettings(const Value: string); virtual;
    procedure SetImageIndex(Value: System.UITypes.TImageIndex); virtual;
    procedure SetImages(Value: TCustomImageList); virtual;
    procedure SetName(const NewName: TComponentName); override;
    procedure SetNameFriendly(const Value: string); virtual;
    property ApplicationDataPath: string read GetApplicationDataPath write
        FApplicationDataPath;
  public
    constructor Create(AOwner: TComponent); override;
    procedure AddToActionList(AActionList: TCustomActionList); virtual;
    procedure AddToOutlookBar(AnOutlookBar: TJvOutlookBar); virtual;
    procedure AddToOutlookBarPage(AnOutlookBarPage: TJvOutlookBarPage); virtual;
    procedure AddToTreeView(ATreeView: TTreeView; ATreeNode: TTreeNode);
        virtual;
    function AddToXML(ANode: XMLIntf.IXMLNode; const NodeName: WideString):
        XMLIntf.IXMLNode; virtual;
    procedure CreateViewForm; virtual;
    class function Crypt(const Value: string; EncryptLength: integer; Decrypt :
        boolean): string;
    procedure DlgSetupExecute;
    function DlgViewExecute: TModalResult; virtual;
    procedure ForgetIni(const AnIniSection: string; IsSharedSetting: Boolean =
        True);
    function IniFileName(IsShared: Boolean): string;
    procedure ReadFromXML(ANode: XMLIntf.IXMLNode); virtual;
    function RecallBoolIni(const Section, Ident: string; Default: Boolean;
        IsSharedSetting: Boolean = True): Boolean;
    function RecallCryptIni(const Section, Ident, PlainDefault: string; const
        EncryptLength: Integer; IsSharedSetting: Boolean = True): string;
    function RecallDateIni(const Section, Ident: string; Default: TDateTime;
        IsSharedSetting: Boolean = True): TDateTime;
    function RecallFloatIni(const Section, Ident: string; Default: double;
        IsSharedSetting: Boolean = True): Double;
    function RecallIntegerIni(const Section, Ident: string; Default: Longint;
        IsSharedSetting: Boolean = True): LongInt;
    procedure RecallSectionIni(const Section: string; Strings: TStrings;
        IsSharedSetting: Boolean = True);
    function RecallStringIni(const Section, Ident, Default: string;
        IsSharedSetting: Boolean = True): string;
    procedure RememberBoolIni(const Section, Ident: string; Value: Boolean;
        IsSharedSetting: Boolean = True);
    function RememberCryptIni(const Section, Ident, PlainValue: string; const
        EncryptLength: Integer; IsSharedSetting: Boolean = True): string;
    procedure RememberDateIni(const Section, Ident: string; Value: TDateTime;
        IsSharedSetting: Boolean = True);
    procedure RememberFloatIni(const Section, Ident: string; Value: double;
        IsSharedSetting: Boolean = True);
    procedure RememberIntegerIni(const Section, Ident: string; Value: Longint;
        IsSharedSetting: Boolean = True);
    procedure RememberSectionIni(const Section: string; Strings: TStrings;
        IsSharedSetting: Boolean = True);
    procedure RememberStringIni(const Section, Ident, Value: string;
        IsSharedSetting: Boolean = True);
    property DlgSetupIsImplemented: Boolean read FDlgSetupIsImplemented;
    property FolderLocalSettings: string read GetFolderLocalSettings write
        SetFolderLocalSettings;
    property FolderSharedSettings: string read GetFolderSharedSettings write
        SetFolderSharedSettings;
    property FormView: TFormLKBase read FFormView;
    property ImageIndex: System.UITypes.TImageIndex read FImageIndex
      write SetImageIndex default -1;
    property Images: TCustomImageList read FImages write SetImages;
    property NameFriendly: string read GetNameFriendly write SetNameFriendly;
    property OutlookBarButton: TJvOutlookBarButton read FOutlookBarButton write
        FOutlookBarButton;
    property OutlookBarPage: TJvOutlookBarPage read FOutlookBarPage write
        FOutlookBarPage;
    property TreeNode: TTreeNode read FTreeNode write FTreeNode;
  published
    property HelpContext: THelpContext read FHelpContext write FHelpContext
        default 0;
    property HelpContextSetup: THelpContext read FHelpContextSetup write
        FHelpContextSetup default 0;
    property TreeNodeVisible: Boolean read FTreeNodeVisible write
        FTreeNodeVisible default True;
  end;

  TLKComponentList = class(TComponentList)
  private
    FSorted: Boolean;
    procedure SetSorted(Value: Boolean);
  protected
    function GetItems(Index: Integer): TLKComponent;
    procedure SetItems(Index: Integer; AComponent: TLKComponent);
  public
    function Add(AComponent: TLKComponent): Integer;
    procedure AddToStrings(AList: TStrings); virtual;
    function ComponentByNameFriendly(const Value: string): TLKComponent;
    procedure DlgChangeListOrderExecute;
    function Extract(Item: TLKComponent): TLKComponent;
    function First: TLKComponent;
    function IndexOf(AComponent: TLKComponent): Integer;
    procedure Insert(Index: Integer; AComponent: TLKComponent);
    function Last: TLKComponent;
    property Items[Index: Integer]: TLKComponent read GetItems write SetItems;
        default;
    property Sorted: Boolean read FSorted write SetSorted;
  end;

  // Windows Vista
function TaskDialog(AForm: TCustomForm; ATitle, ADescription, AContent: string;
  Buttons, Icon: integer): integer;

procedure Register;

implementation

uses
  LKZipProgressDlg, LKWrZipDlg, LKViewTxDg, LKUtils,
  LKComponentDlg, LKCompomentListSortDlg;

procedure Register;
begin
  RegisterComponents('LKUtils', [TLKHeaderPanel]);
  RegisterComponents('LKUtils', [TLKProgressDialog]);
  RegisterComponents('LKUtils', [TLKViewDialog]);
  RegisterComponents('LKUtils', [TLKZipper]);
end;

{
******************************** TLKHeaderPanel ********************************
}
constructor TLKHeaderPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ParentColor := False;
  Color := clInfoBk;
  Height := 138;
  FHeaderPanel := TJvGradientHeaderPanel.Create(Self);
  with FHeaderPanel do
  begin
    Parent := Self;
    Align := alTop;
    BorderStyle := bsNone;
    GradientStartColor := clMoneyGreen;
    GradientEndColor := clInfoBk;
    GradientStyle := grVertical;
    Height := 40;
    LabelFont.Color := clGreen;
    LabelFont.Name := 'Tahoma';
    LabelFont.Size := 14;
  end;
  FHeaderMemo := TMemo.Create(Self);
  with FHeaderMemo do
  begin
    Parent := Self;
    Anchors := [akLeft,akTop,akRight,akBottom];
    BorderStyle := bsNone;
    Left := 24;
    Height := Self.Height - 60;
    Width := Self.Width - 48;
    Top := 48;
    Font.Color := clGreen;
    Font.Name := 'Tahoma';
    Font.Size := 10;
    ParentColor := True;
  //  ReadOnly := True;
    TabStop := False;
  end;
  FLabelHelpURI := TLabel.Create(Self);
  with FLabelHelpURI do
  begin
    Parent := Self;
    Align := alBottom;
    Caption := 'See On-line Help...';
    Font.Color := clblue;
    Font.Style := [fsUnderline];
    Font.Size := 14;
    Visible := False;
    OnClick := DoLabelHelpURIClick;
  end;
end;

procedure TLKHeaderPanel.DoLabelHelpURIClick(Sender: TObject);
begin
  if (FHelpURI > '') then
    ShellExecute(Application.Handle,'open', PWideChar(FHelpURI),nil,nil,0);
end;

function TLKHeaderPanel.GetCaption: TCaption;
begin
  Result := FHeaderPanel.LabelCaption;
end;

function TLKHeaderPanel.GetComments: TStrings;
begin
  Result := FHeaderMemo.Lines;
end;

function TLKHeaderPanel.GetInheritedPopupMenu: TPopupMenu;
begin
  Result := inherited PopupMenu;
end;

procedure TLKHeaderPanel.Loaded;
begin
  inherited Loaded;
  SetHelpURI(FHelpURI);
  with FHeaderMemo do
  begin
    Left := 24;
    Width := Self.Width - 48;
    Top := 48;
    Font.Color := clGreen;
    Font.Name := 'Tahoma';
    Font.Size := 10;
    ParentColor := True;
  //  ReadOnly := True;
    TabStop := False;
  end;
end;

procedure TLKHeaderPanel.SetCaption(const Value: TCaption);
begin
  inherited Caption := ' ';
  FHeaderPanel.LabelCaption := Value;
end;

procedure TLKHeaderPanel.SetComments(Value: TStrings);
begin
  if Assigned(Value) and (Align in [alTop,alBottom]) then
    Height := 65 + Value.Count * 16;
  FHeaderMemo.Lines := Value;
end;

procedure TLKHeaderPanel.SetHelpURI(const Value: string);
begin
  FHelpURI := Value;
  FLabelHelpURI.Visible := (Value > '');
  if FLabelHelpURI.Visible then
    FHeaderMemo.Height := Self.Height - 60 - FLabelHelpURI.Height
  else FHeaderMemo.Height := Self.Height - 60;

end;

procedure TLKHeaderPanel.SetPopupMenu(Value: TPopupMenu);
begin
  inherited PopupMenu := Value;
  FHeaderPanel.PopupMenu := Value;
  FHeaderMemo.PopupMenu := Value;
end;


{
****************************** TLKProgressDialog *******************************
}

constructor TLKProgressDialog.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDlgProgress := TDlgLKProgress.Create(Self);
  FMax := 100;
  FMin := 0;
  FStatusPanel0Width := 150;
//  FTitle: string;
//  FProgressLabelCaption: string;
//  FStep: Integer;
//  FStatusPanel0Text: string;
//  FPosition: Integer;
//  FStatusPanel1Text: string;
  FDlgProgress.OnClose := DoOnClose;
end;

procedure TLKProgressDialog.Close;
begin
  FDlgProgress.Close;
end;

procedure TLKProgressDialog.DoOnClose(Sender: TObject; var Action:
    TCloseAction);
begin
  if Assigned(FOnClose) then
    FOnClose(Sender, Action);
end;

procedure TLKProgressDialog.DoOnShow;
begin
  if Assigned(FOnShow) then
    FOnShow(self);
end;

procedure TLKProgressDialog.Execute;
begin
  if Owner is TCustomForm then
    FDlgProgress.PopupParent := TCustomForm(Owner);
  with FDlgProgress do
  begin
    lProgress.Caption := FProgressLabelCaption;
    StatusBar1.Panels[0].Text := FStatusPanel0Text;
    StatusBar1.Panels[0].Width := FStatusPanel0Width;
    StatusBar1.Panels[1].Text := FStatusPanel1Text;
    Caption := FTitle;
    Aborted := False;
    DoOnShow;
    Show;
  end;
end;

function TLKProgressDialog.GetAborted: Boolean;
begin
  Result := FDlgProgress.Aborted;
end;

procedure TLKProgressDialog.SetMax(Value: Integer);
begin
  FMax := Value;
  FDlgProgress.ProgressBar1.Max := Value;
end;

procedure TLKProgressDialog.SetMin(Value: Integer);
begin
  FMin := Value;
  FDlgProgress.ProgressBar1.Min := Value;
end;

procedure TLKProgressDialog.SetPosition(Value: Integer);
begin
  FPosition := Value;
  FDlgProgress.ProgressBar1.Position := Value;
end;

procedure TLKProgressDialog.SetProgressLabelCaption(const Value: string);
begin
  FProgressLabelCaption := Value;
  if not (csDesigning in ComponentState) then
    FDlgProgress.lProgress.Caption := FProgressLabelCaption;
end;

procedure TLKProgressDialog.SetStatusPanel0Text(const Value: string);
begin
  FStatusPanel0Text := Value;
  if not (csDesigning in ComponentState) then
    FDlgProgress.StatusBar1.Panels[0].Text := Value;
end;

procedure TLKProgressDialog.SetStatusPanel0Width(Value: Integer);
begin
  FStatusPanel0Width := Value;
  if not (csDesigning in ComponentState) then
    FDlgProgress.StatusBar1.Panels[0].Width := Value;
end;

procedure TLKProgressDialog.SetStatusPanel1Text(const Value: string);
begin
  FStatusPanel1Text := Value;
  if not (csDesigning in ComponentState) then
    FDlgProgress.StatusBar1.Panels[1].Text := Value;
end;

procedure TLKProgressDialog.SetStep(Value: Integer);
begin
  FStep := Value;
  FDlgProgress.ProgressBar1.Step := Value;
end;

procedure TLKProgressDialog.SetTitle(const Value: string);
begin
  FTitle := Value;
  if not (csDesigning in ComponentState) then
    FDlgProgress.Caption := Value;
end;

procedure TLKProgressDialog.Show(const ATitle: string);
begin
  Title := ATitle;
  Execute;
end;

procedure TLKProgressDialog.StepIt;
begin
  FDlgProgress.ProgressBar1.StepIt;
end;


{
******************************** TLKViewDialog *********************************
}
constructor TLKViewDialog.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FHeaderCaption := '';
  FHeaderComments := '';
  FChoiceItems := TStringList.Create;
  FValue := False;
  FOptionCaption := '';
  FOptionChecked := False;
end;

destructor TLKViewDialog.Destroy;
begin
  FChoiceItems.Free;
  inherited Destroy;
end;

procedure TLKViewDialog.DoOnClose;
begin
  if assigned(FOnClose) then
    FOnClose(self);
end;

procedure TLKViewDialog.DoOnShow;
begin
  if assigned(FOnShow) then
    FOnShow(self);
end;

function TLKViewDialog.Execute: Boolean;
begin
  Result := Show(Title, Hint, FValue, FDataType);
end;

procedure TLKViewDialog.SetChoiceItems(NewValue: TStrings);
begin
  ChoiceItems.Assign(NewValue);
end;

procedure TLKViewDialog.SetDataType(const AValue: TLKVDDataType);
begin
  FDataType := AValue;
  if csloading in ComponentState then
    exit;
  case FDataType of
    vBoolean: Value := False;
    vInteger: Value := 0;
    vNumber, vCurrency: Value := 0.00;
    vDate: Value := Date;
    vStringCombo, vRadioGroup: Value := 0;
    vString, vPassword: Value := '';
  end;
end;

procedure TLKViewDialog.SetTitle(const Value: string);
begin
  if (FHeaderCaption = '') then
    FHeaderCaption := Value;
  FTitle := Value;
end;

function TLKViewDialog.Show(ATitle, AHint: string; AValue: variant; ADataType:
    TLKVDDataType): Boolean;
var
  S: string;
  BoRes: Boolean;
  W: Integer;
begin
  doOnShow;
  with TLKViewDlg.Create(Owner) do
  begin
    try
      if Owner is TCustomForm then
        PopupParent := TCustomForm(Owner);
      Hint := AHint;
      btnHint.Visible := (AHint > '');
      Caption := ATitle;
      HeaderPanel.Caption := HeaderCaption;
      HeaderPanel.Comments.Add(HeaderComments);
      Height := HeaderPanel.Height + 170;
      validateType := FDataType;
      Edit1.visible := False;
      ComboBox1.visible := False;
      rgBoolean.visible := False;
      case FDataType of
        vStringCombo: activeEdit := ComboBox1;
        vRadioGroup, vBoolean: activeEdit := rgBoolean;
      else
        activeEdit := Edit1;
      end;
      activeEdit.Visible := True;
      case FDataType of
        vDate:
          begin
            Edit1.EditMask := '!99/99/00;1;_';
            Edit1.Text := AValue;
          end;
        vStringCombo: with ComboBox1 do
          begin
            Items.Assign(ChoiceItems);
            ItemIndex := Items.IndexOf(AValue);
          end;
        vRadioGroup:
          begin
            W := 25 + (16 * (Round((ChoiceItems.Count + 0.9) / 2)));
            Height := HeaderPanel.Height + W + 105;
            ckbxOption.Top := W + 15;
            with rgBoolean do
            begin
              Caption := 'Select';
              Height := W;
              Width := 350;
              Items.Assign(ChoiceItems);
              ItemIndex := AValue;
            end;
          end;
        vBoolean: with rgBoolean do
          begin
            if ChoiceItems.Count = 2 then
              Items.Assign(ChoiceItems);
            W := Length(Items[0]);
            if (W < Length(Items[1])) then
              W := Length(Items[1]);
            width := 40 + W * 16;
            if (AValue = True) then
              ItemIndex := 0
            else
              ItemIndex := 1;
          end;
      else
        Edit1.Text := AValue;
      end;
      if (FDataType = vPassword) then
        Edit1.PasswordChar := '*'
      else
        Edit1.passwordChar := #0;
      with ckbxOption do
      begin
        Caption := OptionCaption;
        Checked := OptionChecked;
        Visible := (OptionCaption > '');
      end;
      Result := (ShowModal = mrOK);
      if Result then
      begin
        S := Edit1.text;
        case ADataType of
          vString, vPassword: FValue := S;
          vBoolean:
            begin
              BoRes := (rgBoolean.itemIndex = 0);
              FValue := BoRes;
            end;
          vDate: FValue := StrToDate(S);
          vInteger: FValue := StrToInt(S);
          vNumber: FValue := StrToFloat(S);
          vCurrency:
            begin
              if not CharInSet(S[1], (['0'..'9', '-'])) then
                delete(S, 1, 1);
              Value := StrToCurr(S);
            end;
          vStringCombo: FValue := ComboBox1.Text;
          vRadioGroup: FValue := rgBoolean.ItemIndex;
        end;
        OptionChecked := ckbxOption.Checked;
      end;
      doOnClose;
    finally Free;
    end;
  end;
end;


{
********************************** TLKZipper ***********************************
}
constructor TLKZipper.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FPassword := '';
  FDlgZipTitle := 'Compressing files to archive';
  fDlgUnZipTitle := 'Extracting files';
  FShowMessages := True;
end;

function TLKZipper.UnZip(TargetFolder, RestoreFile: string): Boolean;
begin
  with TFormLKZipProgress.Create(Owner) do
  try
    if Owner is TCustomForm then
      PopupParent := TCustomForm(Owner);
    StaticTextTitle.Caption := FDlgUnZipTitle;
    AbZipKit1.Password := AnsiString(FPassword);
    Caption := 'Preparing files - please wait';
    Show;
    Result := UnZip(TargetFolder, RestoreFile);
    if FShowMessages then
      if Result then
        Showmessage('Unpack operation complete')
      else
        Showmessage('An error occured in the Unpack operation');
    Close;
  finally Free;
  end;
end;

function TLKZipper.Zip(Folder, ZipFileName, ZipFolder: string; ShowMessages,
    AskForDisk, Subfolders: boolean): Boolean;
begin
  with TFormLKZipProgress.Create(Self) do
  try
    if Owner is TCustomForm then
      PopupParent := TCustomForm(Owner);
    StaticTextTitle.Caption := FDlgZipTitle;
    AbZipKit1.Password := AnsiString(FPassword);
    diskDrive := zipFolder;
    caption := 'Saving to ' + ZipFileName + '...';
    if FShowMessages then Show;
    if AskForDisk then
      if not AskForFirstDisk then
      begin
        Messagedlg('Saving to ' + Zipfolder + ' cancelled.', mtInformation,
          [mbOk], 0);
        Result := False;
        Abort;
      end;
    Result := Zip(Folder, ZipFileName, Zipfolder, False);
    if FShowMessages then
      if Result then
        Messagedlg('Saving to ' + Zipfolder + '\' + ZipFileName + ' complete.',
          mtInformation, [mbOk], 0)
      else
        Messagedlg('An error occured in the backup to ' + Zipfolder, mtError,
        [mbOk], 0);
  finally
    Free;
  end;
end;

function TLKZipper.ZipFiles(FileNames: TStrings; BaseFolder, ZipFileName,
    ZipFolder: string; AskForDisk: boolean): Boolean;
begin
  with TFormLKZipProgress.Create(Self) do
  try
    if Owner is TCustomForm then
      PopupParent := TCustomForm(Owner);
    StaticTextTitle.Caption := FDlgZipTitle;
    AbZipKit1.Password := AnsiString(FPassword);
    diskDrive := Zipfolder;
    caption := 'Saving to ' + ZipFileName + '...';
    if FShowMessages then
      Show;
    if AskForDisk then
      if not AskForFirstDisk then
      begin
        Messagedlg('Saving to ' + Zipfolder + ' cancelled.', mtInformation,
          [mbOk], 0);
        Result := False;
        abort;
      end;
    Result := ZipFiles(FileNames, BaseFolder, ZipFileName, Zipfolder);
    if FShowMessages then
      if Result then
        Messagedlg('Saving to ' + Zipfolder + '\' + ZipFileName + ' complete.',
          mtInformation, [mbOk], 0)
      else
        Messagedlg('An error occured in the backup to ' + Zipfolder, mtError,
          [mbOk], 0);
  finally
    Free;
  end;
end;


{
********************************* TLKComponent *********************************
}
constructor TLKComponent.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FApplicationDataPath := '';
  FIniFileName := '';
  FTreeNodeVisible := True;
  FDlgSetupIsImplemented := False;
  FImages := nil;
  FImageIndex := -1;
  FHelpContext := -1;
  FHelpContextSetup := -1;
  FFormView := nil;
end;

procedure TLKComponent.AddToActionList(AActionList: TCustomActionList);
var
  MyAction: TAction;
begin
  if TreeNodeVisible then
  begin
    MyAction := TAction.Create(AActionList.Owner);
    MyAction.ImageIndex := ImageIndex;
    MyAction.Name := 'a' + Name;
    MyAction.Caption := '&' + NameFriendly;
    MyAction.OnExecute := DefaultExecuteAction;
    MyAction.ActionList := AActionList;
  end;
end;

procedure TLKComponent.AddToOutlookBar(AnOutlookBar: TJvOutlookBar);
begin
  FOutlookBarPage := AnOutlookBar.Pages.Add;
  FOutlookBarPage.Caption := NameFriendly;
  FOutlookBarPage.ImageIndex := ImageIndex;
end;

procedure TLKComponent.AddToOutlookBarPage(AnOutlookBarPage: TJvOutlookBarPage);
begin
  FOutlookBarButton := AnOutlookBarPage.Buttons.Add;
  FOutlookBarButton.Caption := NameFriendly;
  FOutlookBarButton.ImageIndex := ImageIndex;
end;

procedure TLKComponent.AddToTreeView(ATreeView: TTreeView; ATreeNode:
    TTreeNode);
begin
  FTreeNode := ATreeView.Items.AddChildObject(ATreeNode, NameFriendly, Self);
  FTreeNode.ImageIndex := ImageIndex;
  FTreeNode.SelectedIndex := ImageIndex;
end;

function TLKComponent.AddToXML(ANode: XMLIntf.IXMLNode; const NodeName:
    WideString): XMLIntf.IXMLNode;
begin
  Result := ANode.AddChild(NodeName);
  with Result.AddChild('N') do
    Text := NameFriendly;
  with Result.AddChild('II') do
    Text := IntToStr(FImageIndex);
end;

procedure TLKComponent.AssignTo(Dest: TPersistent);
var
  DestACB: TLKComponent;
begin
  //inherited;  no - else inherits 'Dest.AssignError(Self);' from TPersistent !!
  DestACB := TLKComponent(Dest);
  DestACB.ImageIndex := FImageIndex;
  DestACB.Images := FImages;
  DestACB.NameFriendly := FNameFriendly;
  DestACB.TreeNodeVisible := FTreeNodeVisible;
end;

procedure TLKComponent.CreateViewForm;
begin
  Application.CreateForm(TFormLKComponent, FFormView);
  TFormLKComponent(FFormView).Construct(Self);
end;

class function TLKComponent.Crypt(const Value: string; EncryptLength: integer;
    Decrypt : boolean): string;
var
  S, Crypt, Plain: string;
  I, J: Integer;
begin
  Plain :=
    ' ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789!"$%^&*()_+=-~#|\;,./:'; // ?
  Crypt :=
    'V456W%^&*Def9!Ex$lna7LMqYX8Zyz01vstCk,?./S-F:Jr)G2gPQiBKuA TIRwbcdm"opj(~#|\;_+=NOUH3'; // h?
  Result := '';
  if Decrypt then
  begin
    if (Value = '') then
      Exit;
    if (Pos('a1', Value) <> 1) then
      raise LKError.Create('LK Encryption version not recognised');
    J := pos('h', Value);
    if (J = 0) then
      J := Length(Value) + 1;
    S := Copy(Value, 3, J - 3);
    for I := length(S) downto 1 do
    begin
      J := pos(S[I], Crypt);
      if (J = 0) then
        Result := 'h' + Result
      else
        Result := Result + Plain[J];
      Crypt := Crypt[length(Crypt)] + copy(Crypt, 1, length(Crypt) - 1);
    end;
  end
  else
  begin
    for I := 1 to Length(Value) do
    begin
      J := pos(Value[I], Plain);
      //build encripted Result reversed from Crypt string
      if (J = 0) then
        Result := '?' + Result
      else
        Result := Crypt[J] + Result;
      // shift the Crypt string 1 to left to avoid a same letter having same Result
      Crypt := Crypt[length(Crypt)] + copy(Crypt, 1, length(Crypt) - 1);
    end;
    // add cript id
    Result := 'a1' + Result;
    // pad out the length to the minimum EncryptLength if necessary
    I := EncryptLength - Length(Result);
    if (I > 0) then
    begin
      Result := Result + 'h';
      Dec(I);
      while (I > 0) do
      begin
        Result := Result + Copy(Crypt, I, 1);
        Dec(I);
      end;
    end;
  end;
end;

procedure TLKComponent.DefaultExecuteAction(Sender: TObject);
begin
  DlgViewExecute;
end;

procedure TLKComponent.DlgSetupExecute;
begin
  DoSetup(True);
end;

function TLKComponent.DlgViewExecute: TModalResult;
begin
  if not assigned(FFormView) then
   begin
     CreateViewForm;
//     FormView.FreeNotification(Self);
   end;
  Result := FFormView.ShowModal;
end;

function TLKComponent.DoSetup(const ForceIfSettingsExist: Boolean):
    TModalResult;
begin
  //Showmessage('Set-up not implemented for '+ NameFriendly);
  Result := mrOK; // nothing to do
end;

procedure TLKComponent.ForgetIni(const AnIniSection: string; IsSharedSetting:
    Boolean = True);
begin
  with TIniFile.Create(IniFileName(IsSharedSetting)) do
  try
    EraseSection(AnIniSection);
  finally free;
  end;
end;

function TLKComponent.GetApplicationDataPath: string;
var
  S: string;
begin
  if (FApplicationDataPath = '') then
  begin
    S := ExtractFileName(Application.ExeName);
    FApplicationDataPath := GetShellFoldername(CSIDL_APPDATA) + PathDelim
      + 'LesKaye' + PathDelim +  Copy(S, 1, Pos('.', S)-1);
  end;
  ForceDirectories(FApplicationDataPath);
  Result := FApplicationDataPath;
end;

function TLKComponent.GetFolderLocalSettings: string;
begin
  if (FFolderLocalSettings = '') then
    FolderLocalSettings := ApplicationDataPath;
  Result := FFolderLocalSettings;
end;

function TLKComponent.GetFolderSharedSettings: string;
begin
  if (FFolderLocalSettings = '') then
  begin
    FFolderSharedSettings := ExtractFilePath(Application.ExeName);
    System.Delete(FFolderSharedSettings, Length(FFolderSharedSettings), 1);
  end;
  Result := FFolderSharedSettings;
end;

function TLKComponent.GetNameFriendly: string;
begin
  if (FNameFriendly = '') then
    FNameFriendly := Name;
  Result := FNameFriendly;
end;

function TLKComponent.GetSettings: TModalResult;
begin
  Result := mrOK; // nothing to get
end;

function TLKComponent.IniFileName(IsShared: Boolean): string;
var
  S: string;
begin
  if (FIniFileName = '') then
  begin
    S := ChangeFileExt(ExtractFileName(Application.ExeName), '.ini');
    if IsShared then
      FIniFileName := FolderSharedSettings + PathDelim + S
    else
      FIniFileName := FolderLocalSettings + PathDelim + S;
  end;
  Result := FIniFileName;
end;

procedure TLKComponent.Loaded;
begin
  inherited Loaded;
  //loadingTmpName := '';
  SetName(Name);
end;

procedure TLKComponent.Notification(AComponent: TComponent; Operation:
    TOperation);
begin
  if (Operation = opRemove) then
  begin
    try
  //    if (AComponent = FFormView) then
  //      FFormView := nil;
      if (AComponent = FImages) then
        FImages := nil;
    except on E: exception do
        Showmessage(NameFriendly + ' Notification Base ' + E.Message);
    end;
  end;
  inherited Notification(AComponent, Operation);
end;

procedure TLKComponent.ReadFromXML(ANode: XMLIntf.IXMLNode);
begin
  NameFriendly := ANode.ChildNodes['N'].Text;
  FImageIndex := StrToInt(ANode.ChildNodes['II'].Text);
end;

function TLKComponent.RecallBoolIni(const Section, Ident: string; Default:
    Boolean; IsSharedSetting: Boolean = True): Boolean;
begin
  Result := Default;
  if (csLoading in ComponentState) then
    Exit;
  with TIniFile.Create(IniFileName(IsSharedSetting)) do
  try
    Result := ReadBool(Section, Ident, Default);
  finally Free;
  end;
end;

function TLKComponent.RecallCryptIni(const Section, Ident, PlainDefault: string;
    const EncryptLength: Integer; IsSharedSetting: Boolean = True): string;
begin
  Result := RecallStringIni(Section, Ident, '', IsSharedSetting);
  if (Result = '') then
    Result := PlainDefault
  else
    Result := Crypt(Result, EncryptLength, True);
end;

function TLKComponent.RecallDateIni(const Section, Ident: string; Default:
    TDateTime; IsSharedSetting: Boolean = True): TDateTime;
var
  D: Double;
begin
  D := RecallIntegerIni(Section, Ident, Round(Default), IsSharedSetting);
  Result := D;
end;

function TLKComponent.RecallFloatIni(const Section, Ident: string; Default:
    double; IsSharedSetting: Boolean = True): Double;
begin
  Result := StrToFloat(RecallStringIni(Section, Ident, FloatToStr(Default),
    IsSharedSetting));
end;

function TLKComponent.RecallIntegerIni(const Section, Ident: string; Default:
    Longint; IsSharedSetting: Boolean = True): LongInt;
begin
  Result := Default;
  if (csLoading in ComponentState) then
    Exit;
  with TIniFile.Create(IniFileName(IsSharedSetting)) do
  try
    Result := ReadInteger(Section, Ident, Default);
  finally Free;
  end;
end;

procedure TLKComponent.RecallSectionIni(const Section: string; Strings:
    TStrings; IsSharedSetting: Boolean = True);
begin
  if (csLoading in ComponentState) then
    Exit;
  with TIniFile.Create(IniFileName(IsSharedSetting)) do
  try
    ReadSection(Section, Strings);
  finally Free;
  end;
end;

function TLKComponent.RecallStringIni(const Section, Ident, Default: string;
    IsSharedSetting: Boolean = True): string;
begin
  Result := Default;
  if (csLoading in ComponentState) then
    Exit;
  with TIniFile.Create(IniFileName(IsSharedSetting)) do
  try
    Result := ReadString(Section, Ident, Default);
  finally Free;
  end;
end;

procedure TLKComponent.RememberBoolIni(const Section, Ident: string; Value:
    Boolean; IsSharedSetting: Boolean = True);
begin
  if (csLoading in ComponentState) then
    Exit;
  with TIniFile.Create(IniFileName(IsSharedSetting)) do
  try
    WriteBool(Section, Ident, Value);
  finally free;
  end;
end;

function TLKComponent.RememberCryptIni(const Section, Ident, PlainValue: string;
    const EncryptLength: Integer; IsSharedSetting: Boolean = True): string;
begin
  RememberStringIni(Section, Ident, Crypt(PlainValue, EncryptLength, False),
    IsSharedSetting);
end;

procedure TLKComponent.RememberDateIni(const Section, Ident: string; Value:
    TDateTime; IsSharedSetting: Boolean = True);
begin
  RememberIntegerIni(Section, Ident, Round(Value), IsSharedSetting);
end;

procedure TLKComponent.RememberFloatIni(const Section, Ident: string; Value:
    double; IsSharedSetting: Boolean = True);
begin
  if (csLoading in ComponentState) then
    Exit;
  with TIniFile.Create(IniFileName(IsSharedSetting)) do
  try
    WriteString(Section, Ident, FloatToStr(Value));
  finally free;
  end;
end;

procedure TLKComponent.RememberIntegerIni(const Section, Ident: string; Value:
    Longint; IsSharedSetting: Boolean = True);
begin
  if (csLoading in ComponentState) then
    Exit;
  with TIniFile.Create(IniFileName(IsSharedSetting)) do
  try
    WriteInteger(Section, Ident, Value);
  finally free;
  end;
end;

procedure TLKComponent.RememberSectionIni(const Section: string; Strings:
    TStrings; IsSharedSetting: Boolean = True);
var
  I: Integer;
begin
  if (csLoading in ComponentState) then
    Exit;
  with TIniFile.Create(IniFileName(IsSharedSetting)) do
  try
    EraseSection(Section);
    with Strings do
      for i := 0 to Count - 1 do
        WriteString(Section, Strings[i], '');
  finally free;
  end;
end;

procedure TLKComponent.RememberStringIni(const Section, Ident, Value: string;
    IsSharedSetting: Boolean = True);
begin
  if (csLoading in ComponentState) then
    exit;
  with TIniFile.Create(IniFileName(IsSharedSetting)) do
  try
    WriteString(Section, Ident, Value);
  finally free;
  end;
end;

procedure TLKComponent.SaveProperties;
begin
  //
end;

procedure TLKComponent.SetFolderLocalSettings(const Value: string);
begin
  FFolderLocalSettings := Value;
  if (FFolderLocalSettings[Length(FFolderLocalSettings)] = PathDelim) then
    System.Delete(FFolderLocalSettings, Length(FFolderLocalSettings), 1);
end;

procedure TLKComponent.SetFolderSharedSettings(const Value: string);
begin
  FFolderSharedSettings := Value;
  if (FFolderSharedSettings[Length(FFolderSharedSettings)] = PathDelim) then
    System.Delete(FFolderSharedSettings, Length(FFolderSharedSettings), 1);
end;

procedure TLKComponent.SetImageIndex(Value: System.UITypes.TImageIndex);
begin
  FImageIndex := Value;
  if assigned(TreeNode) then
  begin
    TreeNode.ImageIndex := Value;
    TreeNode.SelectedIndex := ImageIndex;
  end;
end;

procedure TLKComponent.SetImages(Value: TCustomImageList);
begin
  FImages := Value;
  if Assigned(Value) then
    Value.FreeNotification(Self);
end;

procedure TLKComponent.SetName(const NewName: TComponentName);
begin
  if (NameFriendly = Name) then
    NameFriendly := NewName;
  inherited SetName(NewName);
end;

procedure TLKComponent.SetNameFriendly(const Value: string);
begin
  FNameFriendly := Value;
  if assigned(TreeNode) then
    TreeNode.Text := Value;
end;


{
******************************* TLKComponentList *******************************
}
function TLKComponentList.Add(AComponent: TLKComponent): Integer;
begin
  Result := inherited Add(AComponent);
end;

procedure TLKComponentList.AddToStrings(AList: TStrings);
var
  i: Integer;
begin
  for i := 0 to Count - 1 do
    AList.AddObject(Items[i].NameFriendly, Items[i]);
end;

function TLKComponentList.ComponentByNameFriendly(const Value: string):
    TLKComponent;
var
  i: Integer;
begin
  Result := nil;
  for i := 0 to Count - 1 do
    if (Items[i].NameFriendly = Value) then
    begin
      Result := Items[i];
      break;
    end;
end;

procedure TLKComponentList.DlgChangeListOrderExecute;
begin
  with TDlgLKCompomentListSort.Create(nil) do
  try
    ListToSort := Self;
    Showmodal;
  finally Free;
  end;
end;

function TLKComponentList.Extract(Item: TLKComponent): TLKComponent;
begin
  Result := TLKComponent(inherited Extract(Item));
end;

function TLKComponentList.First: TLKComponent;
begin
  Result := TLKComponent(inherited First);
end;

function TLKComponentList.GetItems(Index: Integer): TLKComponent;
begin
  Result := TLKComponent(inherited Items[Index]);
end;

function TLKComponentList.IndexOf(AComponent: TLKComponent): Integer;
begin
  Result := inherited IndexOf(AComponent);
end;

procedure TLKComponentList.Insert(Index: Integer; AComponent: TLKComponent);
begin
  inherited Insert(Index, AComponent);
end;

function TLKComponentList.Last: TLKComponent;
begin
  Result := TLKComponent(inherited Last);
end;

procedure TLKComponentList.SetItems(Index: Integer; AComponent: TLKComponent);
begin
  inherited Items[Index] := AComponent;
end;

procedure TLKComponentList.SetSorted(Value: Boolean);
var
  i: Integer;
  j: Integer;
begin
  if Value and not FSorted then
    for i := 1 to Count - 1 do
      if (Items[i].NameFriendly < Items[i - 1].NameFriendly) then
      begin
        j := i;
        while (Items[j].NameFriendly < Items[j - 1].NameFriendly) do
        begin
          Exchange(j, j - 1);
          dec(j);
          if (j = 0) then
            break;
        end;
      end;
  FSorted := Value;
end;



function TaskDialog(AForm: TCustomForm; ATitle, ADescription, AContent: string;
  Buttons, Icon: integer): integer;
var
  VerInfo: TOSVersioninfo;
  DLLHandle: THandle;
  res: integer;
  wTitle, wDescription, wContent: array[0..1024] of widechar;
  Btns: TMsgDlgButtons;
  DlgType: TMsgDlgType;
  TaskDialogProc: function(HWND: THandle; hInstance: THandle; cTitle,
    cDescription, cContent: pwidechar; Buttons: Integer; Icon: integer;
    ResButton: pinteger): integer; cdecl stdcall;

begin
  Result := 0;

  VerInfo.dwOSVersionInfoSize := SizeOf(TOSVersionInfo);
  GetVersionEx(verinfo);

  if (verinfo.dwMajorVersion >= 6) then
  begin
    DLLHandle := LoadLibrary('comctl32.dll');
    if DLLHandle >= 32 then
    begin
      @TaskDialogProc := GetProcAddress(DLLHandle, 'TaskDialog');

      if Assigned(TaskDialogProc) then
      begin
        StringToWideChar(ATitle, wTitle, sizeof(wTitle));
        StringToWideChar(ADescription, wDescription, sizeof(wDescription));
        StringToWideChar(AContent, wContent, sizeof(wContent));
        TaskDialogProc(AForm.Handle, 0, wTitle, wDescription, wContent, Buttons,
          Icon, @res);

        Result := mrOK;

        case res of
          DLGRES_CANCEL: Result := mrCancel;
          DLGRES_RETRY: Result := mrRetry;
          DLGRES_YES: Result := mrYes;
          DLGRES_NO: Result := mrNo;
          DLGRES_CLOSE: Result := mrAbort;
        end;
      end;
      FreeLibrary(DLLHandle);
    end;
  end
  else
  begin
    Btns := [];
    if Buttons and TD_OK = TD_OK then
      Btns := Btns + [MBOK];

    if Buttons and TD_YES = TD_YES then
      Btns := Btns + [MBYES];

    if Buttons and TD_NO = TD_NO then
      Btns := Btns + [MBNO];

    if Buttons and TD_CANCEL = TD_CANCEL then
      Btns := Btns + [MBCANCEL];

    if Buttons and TD_RETRY = TD_RETRY then
      Btns := Btns + [MBRETRY];

    if Buttons and TD_CLOSE = TD_CLOSE then
      Btns := Btns + [MBABORT];

    DlgType := mtCustom;

    case Icon of
      TD_ICON_WARNING: DlgType := mtWarning;
      TD_ICON_QUESTION: DlgType := mtConfirmation;
      TD_ICON_ERROR: DlgType := mtError;
      TD_ICON_INFORMATION: DlgType := mtInformation;
    end;

    Result := MessageDlg(AContent, DlgType, Btns, 0);
  end;
end;

end.
