{*****************************************************************************}
{                                                                             }
{   This file is part of the LKUtilities components Library                   }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Various Defines                                                        }
{                                                                             }
{*****************************************************************************}

unit LKDefine;

interface

uses SysUtils, Classes, Forms;

const
//{$IFDEF LINUX}
//  LKSysRoot := '';
//  LKDirRoot = '/LesKaye'
//  LKFloppyDrive = '/media/floppy';
//{$ELSE}
//  LKSysRoot = 'C:';
//  LesKDirRoot = 'C:\LesKaye';
//  LKDirRoot = 'C:\LesKaye';
//  LKFloppyDrive = 'A:';
//{$ENDIF}
  LKBackupExe = 'LKDataBackup';
  LKFileUtilsExe = 'LKDataFileUtils';

  LKSQLStrDelimiter = Chr(39);
  LKSQLObjDelimiter = '';

resourcestring

  StrData = 'data';
  StrProgram = 'program';
  //Email
  StrEmailInitError = 'Error initialising email';
  StrEmailSendError = 'Error sending mail';

type

  LKError = class(Exception)
  end;

  TLKAccountsInterval = (aciUnknown, aciWeekly, aciFortnightly, aciFourWeeks,
    aciMonthly, aciAnnual, aciDays);
  TLKACDaybookEntryType = (detDebits, detCredits);
  TLKACStatus = (acsUnknown, acsNotSetUp, acsPartSetUp, acsNotUsed, acsBasic,
    acsAdvanced);
  TLKACTable = (tbMaster, tbDetail);
  TLKACTableSelect = (tbsMaster, tbsDetail, tbsBoth);
  TLKFieldNumbers = set of 0..255;
  TLKFixedAssetDepreciationStyle = (fadNone, fadStraightLine,
    fadReducingBalance);
  TLKLedgerEmployeeType = (letAccruing, letReduceToZero);
  TLKPostNominalTransactionDetail = (ptdDocumentItems, ptdDocument, ptdBatch);
  TLKReportStyle = (rsNone, rsList, rsRecord, rsAllRecords, rsTotalsOnly,
    rsDefault);
  TLKSalesProspectState = (spUndefined, spEarlyPlanning, spContractorAppointed,
    spTenderPreparation);
  TLKSalesEnquiryState = (seUndefined, seReceived, seAllocated, seQuoted,
    seNotWon,
    seOrdered, seWorkStarted, seWorkComplete, seInvoicedComplete);
  TLKSupportedDatabaseType = (sdbUnknown, sdbLocalFirebird, sdbDB2,
    sdbFirebird, sdbInformix, sdbInterbase, sdbMySQL, sdbOracle,
    sdbMSSQL, sdbMSSQL9);
  TLKTriggerEventType = (teUnknown, teBeforeInsert, teBeforePost,
    teBeforeDelete, teAfterInsert, teAfterPost, teAfterDelete);
  TLKVDDataType = (vBoolean, vInteger, vNumber, vCurrency, vDate, vPassword,
    vString,
    vStringCombo, vRadioGroup {,vNationalInsurance,vTelephone});

procedure LKSupportedDatabaseAddStrings(AStrings: TStrings);
function LKItemIndexToSupportedDatabaseType(Value: integer):
  TLKSupportedDatabaseType;
///<summary>Returns folder under "/program/(execurable).exe" without trailing PathDelim</summary>
function InstallFolder: string;
function DataFolder: string;
function ProgramFolder: string;

implementation

var
  FProgramFolder: string;
  FInstallFolder: string;

function InstallFolder: string;
var
  I: Integer;
begin
  if (FInstallFolder < ' ') then
  begin
    FInstallFolder := ProgramFolder;
    I := Pos(PathDelim + StrProgram, FInstallFolder);
    if (I > 3) then
      Delete(FInstallFolder, I, Length(StrProgram) + 1);
  end;
  Result := FInstallFolder;
end;

function DataFolder: string;
begin
  Result := InstallFolder + PathDelim + StrData;
end;

function ProgramFolder: string;
begin
  if (FProgramFolder < ' ') then
  begin
    FProgramFolder := ExtractFilePath(Application.ExeName);
    System.Delete(FProgramFolder, Length(FProgramFolder), 1);
  end;
  Result := FProgramFolder;
end;


procedure LKSupportedDatabaseAddStrings(AStrings: TStrings);
begin
  with AStrings do
  begin
    Clear;
    Add('Local Firebird');
    Add('Firebird');
    Add('Microsoft SQL');
//    Add('DB2');;
//    Add('Informix');
//    Add('Interbase');
//    Add('MySQL');
//    Add('Oracle');
  end;
end;

function LKItemIndexToSupportedDatabaseType(Value: integer):
  TLKSupportedDatabaseType;
begin
  if (Value > 0) and (Value < 9) then
    Result := TLKSupportedDatabaseType(Value + 1)
  else
    Result := sdbUnknown;
end;

end.

