{*****************************************************************************}
{                                                                             }
{   This file is part of the LKUtilities components Library                   }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Base form with clipboard and move focus on enter key.                  }
{                                                                             }
{*****************************************************************************}


unit LKFormBaseDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, DBGrids, StdCtrls, Mask, DBCtrls, ExtCtrls, Menus, Variants,
  Buttons, ComCtrls, ActnList, ImgList, StdActns;

type

  TFormLKBase = class(TForm)
    ActionClose: TAction;
    ActionListAcsComp: TActionList;
    ActionPrint: TAction;
    ActionStayOnTop: TAction;
    ImageListAC: TImageList;
    ActionHelpIndex: TAction;
    ActionHelpContents: TAction;
    ActionCut: TAction;
    ActionCopy: TAction;
    ActionPaste: TAction;
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure ActionCloseExecute(Sender: TObject);
    procedure ActionStayOnTopExecute(Sender: TObject);
    procedure ActionHelpIndexExecute(Sender: TObject);
    procedure ActionPrintExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ActionHelpContentsExecute(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure miWindowClick(Sender: TObject);
    procedure ActionCutExecute(Sender: TObject);
    procedure ActionCopyExecute(Sender: TObject);
    procedure ActionPasteExecute(Sender: TObject);
  private
  protected
    //   WindowMenuItem: TMenuItem;
    procedure UpdateUI; virtual;
  public
  end;

var
  FormLKBase: TFormLKBase;

implementation

{$R *.DFM}

uses LKUtils;

procedure TFormLKBase.FormCreate(Sender: TObject);
begin
  HelpFile := Application.HelpFile;
  if (HelpContext > 0) then
    BorderIcons := BorderIcons + [biHelp]
  else
    ActionHelpIndex.Visible := false;
  Font.Size := 9;
  SetVistaFonts(Self);
end;

procedure TFormLKBase.ActionCloseExecute(Sender: TObject);
begin
  Close;
end;

procedure TFormLKBase.ActionCopyExecute(Sender: TObject);
begin
  ClipCopy(Self);
end;

procedure TFormLKBase.ActionCutExecute(Sender: TObject);
begin
  ClipCut(Self);
end;

procedure TFormLKBase.ActionPasteExecute(Sender: TObject);
begin
  ClipPaste(Self);
end;

procedure TFormLKBase.FormKeyPress(Sender: TObject; var Key: Char);
begin
  MoveMyFocus(Self, Key);
end;

procedure TFormLKBase.ActionStayOnTopExecute(Sender: TObject);
begin
  if ActionStayOnTop.Checked then
    FormStyle := fsStayOnTop
  else
    FormStyle := fsNormal;
end;

procedure TFormLKBase.ActionHelpIndexExecute(Sender: TObject);
begin
  if (HelpContext > -1) then
    Application.HelpCommand(HELP_CONTEXT, HelpContext);
end;

procedure TFormLKBase.ActionPrintExecute(Sender: TObject);
begin
  Showmessage('Print not implemented here');
end;

procedure TFormLKBase.ActionHelpContentsExecute(Sender: TObject);
begin
  Application.HelpCommand(HELP_FINDER, 0);
end;

procedure TFormLKBase.FormShow(Sender: TObject);
begin
  {
  if Assigned(WindowMenu) then
   begin
   WindowMenuItem := TMenuItem.Create(WindowMenu.Owner);
   WindowMenuItem.Caption := Caption;
   WindowMenuItem.OnClick := miWindowClick;
   WindowMenu.Add(WindowMenuItem);
   end;
   }
  UpdateUI;
end;

procedure TFormLKBase.FormHide(Sender: TObject);
begin
  //WindowMenuItem.Free
end;

procedure TFormLKBase.miWindowClick(Sender: TObject);
begin
  Show;
end;

procedure TFormLKBase.UpdateUI;
begin
  // override to set Actions according to application state
end;

end.

