{*****************************************************************************}
{                                                                             }
{   This file is part of the LKUtilities components Library                   }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Frequently used methods and utilities.                                 }
{                                                                             }
{*****************************************************************************}


unit LKUtils;

interface

uses System.SysUtils, System.Classes, Vcl.Dialogs, Vcl.Controls, Vcl.Forms,
  Vcl.Clipbrd, Vcl.StdCtrls, Vcl.Mask,
  System.StrUtils, System.UITypes,
  dbCtrls, dbGrids, WinTypes, Messages, ShellAPI, graphics, ComCtrls, Buttons,
  extctrls, DB,
  System.Variants, System.DateUtils, ActnList, System.Contnrs,
  JvDBGrid, ActiveX, ShlObj, LKDefine;

const
  VistaFont = 'Segoe UI';
  XPFont = 'Tahoma';
function IsWindowsVista: Boolean;
procedure SetVistaFonts(const AForm: TCustomForm);
function BitmapSize(BMP: TBitmap): LongInt;
function BooleanToYesNo(const Value: Boolean): string;
//function ChopText(Sender: TBaseReport; longText: string; textLength: double):
//  string;
procedure ClipCut(Sender: TForm);
procedure ClipCopy(Sender: TForm);
procedure ClipPaste(Sender: TForm);
procedure DelimitedStringToList(const AString, Delimiter: string; AList:
  TStrings);
function ListToDelimitedString(const Delimiter: string; AList: TStrings):
  string;
function FileCopy(const SourceFileName, DestFileName: string; overWrite:
  Boolean = True): Int64;
function DirectoryCopy(const SourceDirName, DestDirName: string; overWrite:
  Boolean = True): Int64;
procedure EMail(const Originator, Subject, Content: string; Recipients,
  AttachmentFiles: TStrings);
///<summary>Visibility SW_HIDE SW_SHOWNORMAL SW_SHOWMINIMIZED SW_MAXIMIZE
/// SW_SHOWNOACTIVATE SW_SHOW SW_MINIMIZE SW_SHOWMINNOACTIVE
/// SW_SHOWNA SW_RESTORE SW_SHOWDEFAULT SW_MAX
///</summary>
function Execute(const CommandLine, DefaultDir: string; Visibility: Integer;
  WaitonExit: Cardinal): Boolean;
function FloatIsZero(const Value: double): Boolean;
function FloatNotZero(const Value: double): Boolean;
function FloatToStrFOrBlank(Value: extended; Format: TFloatFormat; digits:
  integer): string;
function GetDefaultPrinterName: string;
function GetCurrentPrinterName: string;
function GetShellFoldername(const FolderID: Integer): string;
function SetCurrentPrinterName(const APrinterName: string): string;
function MonthEndLast(const ADate: TDateTime): TDateTime;
function MonthEndThis(const ADate: TDateTime): TDateTime;
function MonthStartThis(const ADate: TDateTime): TDateTime;
function MonthEndNext(const ADate: TDateTime): TDateTime;
procedure MoveMyFocus(Sender: TForm; var Key: Char);
function NewButton(AOwner, AParent: TWinControl; const L, T, W, H: integer;
  aCaption: string): TButton;
function NewGroupBox(AOwner, AParent: TWinControl; const L, T, W, H: integer;
  aCaption: string): TGroupBox;
function NewLabel(AOwner, AParent: TWinControl; const L, T, W, H: integer;
  aCaption: string): TLabel;
function NewCheckbox(AOwner, AParent: TWinControl; const L, T, W, H: integer;
  aCaption: string): TCheckbox;
function NewEdit(AOwner, AParent: TWinControl; const L, T, W, H: integer):
  TEdit;
function NewCombo(AOwner, AParent: TWinControl; const L, T, W, H: integer):
  TComboBox;
procedure GetDiskDrives(DriveList: TStrings);
function CleanSQLName(AName: string): string;
function DigitToString(ADigit: char): string;
procedure TrashFolder(const Path: string);

implementation

uses printers, Mapi, SHFolder;

procedure SetVistaFonts(const AForm: TCustomForm);
begin
  if (IsWindowsVista) // or not CheckOSVerForFonts)
  and not SameText(AForm.Font.Name, VistaFont)
    and (Screen.Fonts.IndexOf(VistaFont) >= 0) then
  begin
   // AForm.Font.Size := AForm.Font.Size + 1;
    AForm.Font.Name := VistaFont;
  end;
end;

function IsWindowsVista: Boolean;
var
  VerInfo: TOSVersioninfo;
begin
  VerInfo.dwOSVersionInfoSize := SizeOf(TOSVersionInfo);
  GetVersionEx(VerInfo);
  Result := VerInfo.dwMajorVersion >= 6;
end;

function BitmapSize(BMP: TBitmap): LongInt;
var
  P: longint;
begin
  with BMP do
  begin
    case PixelFormat of
      pf1bit: P := 1;
      pf4bit: P := 4;
      pf8bit: P := 8;
      pf15bit: P := 15;
      pf16bit: P := 16;
      pf24bit: P := 24;
      pf32bit: P := 32;
    else
      P := 40;
    end;
    Result := Trunc(P * Height * Width / 8);
  end;
end;

function BooleanToYesNo(const Value: Boolean): string;
begin
  if Value then
    Result := 'Yes'
  else
    Result := 'No';
end;

//function ChopText(Sender: TBaseReport; longText: string; textLength: double):
//  string;
//var
//  head, tail, curChar: string;
//  n, L, s: integer;
//  firstChar: Boolean;
//begin
//  Result := '';
//  head := '';
//  tail := longText;
//  if Sender.textWidth(tail) <= textLength then
//    Result := tail
//  else
//  begin
//    firstChar := true;
//    s := 0;
//    L := 0;
//    for n := 1 to length(tail) do
//    begin
//      curChar := copy(tail, 1, 1);
//      if not ((firstChar) and (curChar = ' ')) then
//      begin
//        firstChar := false;
//        head := head + curChar;
//        L := L + 1; // current position in head;
//        if (curChar = ' ') then
//          s := L; // found a space
//        if (Sender.TextWidth(head) > textLength) then
//          break;
//      end;
//      delete(tail, 1, 1);
//    end;
//    if (s = 0) then
//      s := L; //if no spaces
//    Result := copy(head, 1, s - 1);
//    //then return 1 char short to allow hyphen or erase space
//  end;
//end;

procedure ClipCut(Sender: TForm);
begin
  if (Sender.ActiveControl is TCustomEdit) then
  begin
    TCustomEdit(Sender.ActiveControl).CutToClipboard;
    Exit;
  end;
  if (Sender.ActiveControl is TDBGrid) then
  begin
    Clipboard.AsText := TDBGrid(Sender.ActiveControl).SelectedField.AsString;
    TDBGrid(Sender.ActiveControl).DataSource.Edit;
    TDBGrid(Sender.ActiveControl).SelectedField.Value := Null;
    Exit;
  end;
  if (Sender.ActiveControl is TCustomComboBox) then
  begin
    Clipboard.AsText := TCustomComboBox(Sender.ActiveControl).SelText;
    TCustomComboBox(Sender.ActiveControl).SelText := '';
    Exit;
  end;   
  if (Sender.ActiveControl is TDBImage) then
  begin
    TDBImage(Sender.ActiveControl).CutToClipboard;
    Exit;
  end;
end;

procedure ClipCopy(Sender: TForm);
begin
  if (Sender.ActiveControl is TCustomEdit) then
  begin
    TCustomEdit(Sender.ActiveControl).CopyToClipboard;  
    Exit;
  end;
  if (Sender.ActiveControl is TCustomComboBox) then
  begin
    Clipboard.AsText := TCustomComboBox(Sender.ActiveControl).SelText;   
    Exit;
  end;
  if (Sender.ActiveControl is TCustomDBGrid) then
  begin
    Clipboard.AsText :=
      TCustomDBGrid(Sender.ActiveControl).SelectedField.AsString;  
    Exit;
  end;     
  if (Sender.ActiveControl is TDBImage) then
  begin
    TDBImage(Sender.ActiveControl).CopyToClipboard;      
    Exit;
  end;
end;

procedure ClipPaste(Sender: TForm);
begin
  if (Sender.ActiveControl is TCustomEdit) then
  begin
    TCustomEdit(Sender.ActiveControl).PasteFromClipboard;
    Exit;
  end;
  if (Sender.ActiveControl is TCustomComboBox) then
  begin
    TCustomComboBox(Sender.ActiveControl).SelText := Clipboard.AsText;
    Exit;
  end;
  if (Sender.ActiveControl is TCustomDBGrid) then
  begin              
    TCustomDBGrid(Sender.ActiveControl).SelectedField.DataSet.Edit;
    TCustomDBGrid(Sender.ActiveControl).SelectedField.AsString := Clipboard.AsText;
  end;  
  if (Sender.ActiveControl is TDBImage) then
  begin
    TDBImage(Sender.ActiveControl).PasteFromClipboard;
    Exit;
  end;
end;

procedure DelimitedStringToList(const AString, Delimiter: string; AList:
  TStrings);
var
  start, finish: integer;
  s, t: string;
begin
  AList.Clear;
  s := AString;
  start := 1;
  while (start <= Length(s)) do
  begin
    ;
    finish := PosEx(Delimiter, s, start);
    if (finish = 0) then
      finish := Length(s) + 1;
    t := System.Copy(s, start, finish - start);
    Trim(t);
    if (t > '') then
      AList.Add(t);
    start := finish + Length(Delimiter)
  end;
end;

function ListToDelimitedString(const Delimiter: string; AList: TStrings):
  string;
var
  i: integer;
begin
  Result := '';
  with AList do
    for i := 0 to Count - 1 do
      Result := Result + Strings[i] + Delimiter;
end;

function FileCopy(const SourceFileName, DestFileName: string; OverWrite:
  Boolean = True): Int64;
var
  Source, Dest: TFileStream;
begin
  if (SourceFileName = DestFileName) then
    raise LKError.Create('Error : ' + chr(13) + SourceFileName +
      ' Cannot copy the file on itself');
  if FileExists(DestFileName) then
    if overwrite then
      DeleteFile(PWideChar(DestFileName))
    else
      raise LKError.Create('The file ' + DestFileName + chr(13) +
        ' already exists. Aborting copy');
  Source := TFileStream.Create(SourceFileName, fmOpenRead);
  try
    Dest := TFileStream.Create(DestFileName, fmCreate);
    try
      Result := Dest.CopyFrom(Source, 0);
    finally FreeAndNil(Dest);
    end;
  finally FreeAndNil(Source);
  end;
end;


function DirectoryCopy(const SourceDirName, DestDirName: string; OverWrite:
  Boolean = True): Int64;
var
  SearchRec: TSearchRec;
  P, S: string;
begin
  Result := 0;
  if OverWrite then
    TrashFolder(DestDirName);
  if DirectoryExists(DestDirName) then
  begin
    ShowMessage('Cannot copy - directory already exists: ' + DestDirName);
    Exit;
  end
  else
    ForceDirectories(DestDirName);
  S := SourceDirName + PathDelim;
  P := DestDirName + PathDelim;
  try
    if (FindFirst(S + '*', faAnyFile, SearchRec) = 0) then
    repeat
      if ((SearchRec.Attr and faDirectory) <> 0) then
      begin
      if (Pos('.', SearchRec.Name) <> 1) then
        DirectoryCopy(S + SearchRec.Name, P + SearchRec.Name, OverWrite);
      end
      else
        Result := Result + FileCopy(S + SearchRec.Name,
          P + SearchRec.Name, OverWrite);
      Application.ProcessMessages;
    until not (FindNext(SearchRec) = 0);
  finally System.SysUtils.FindClose(SearchRec);
  end;
end;


function Execute(const CommandLine, DefaultDir: string; Visibility: Integer;
  WaitonExit: Cardinal): Boolean;
var
  zAppName, zCurDir: array[0..255] of Char;
  startUpInfo: TStartupInfo;
  processInfo: TProcessInformation;
  lp: Cardinal;
begin
  StrPCopy(zAppName, CommandLine);
  StrPCopy(zCurDir, DefaultDir);
  fillChar(StartupInfo, sizeOf(StartupInfo), #0);
  with StartupInfo do
  begin
    cb := sizeof(StartupInfo);
    dwFlags := STARTF_USESHOWWINDOW;
    wShowWindow := Visibility;
  end;
  Result := CreateProcess(nil, zAppName, nil, nil, false, CREATE_NEW_CONSOLE or
    NORMAL_PRIORITY_CLASS, nil, nil, StartUpInfo, ProcessInfo);
  if (Result) then
  begin
    lp := WaitForSingleObject(ProcessInfo.hProcess, WaitOnExit);
    Result := GetExitCodeProcess(ProcessInfo.hProcess, lp);
  end;
  if not Result then
    MessageDlg('Error opening ' + CommandLine
      + ' CreateProcess code ' + IntToStr(lp), mtError, [mbOK], 0);
end;

function FloatIsZero(const Value: double): Boolean;
begin
  Result := ((Value < 0.001) and (Value > -0.001));
end;

function FloatNotZero(const Value: double): Boolean;
begin
  Result := floatNotZero(Value);
end;

function FloatToStrFOrBlank(Value: extended; Format: TFloatFormat; digits:
  integer): string;
begin
  if floatIsZero(Value) then
    Result := ''
  else
    Result := FloatToStrF(Value, Format, 15, digits);
end;

function GetDefaultPrinterName: string;
begin
  with printer do
    if Printers.Count = 0 then
      Result := ''
    else
    begin
      printerindex := -1; //select default printer
      Result := Printers[printer.printerindex];
    end;
end;

function GetCurrentPrinterName: string;
begin
  with printer do
    if (Printers.Count = 0) then
      Result := ''
    else
      Result := Printers[printer.printerindex];
end;

function SetCurrentPrinterName(const APrinterName: string): string;
begin
  with printer do
    if (Printers.Count = 0) then
      Result := 'no printers available'
    else
    begin
      PrinterIndex := Printers.IndexOf(APrinterName);
      // = -1 if not found - sets default printer
      Result := Printers[printerindex];
    end;
end;

function MonthEndLast(const ADate: TDateTime): TDateTime;
var
  d, m, y: word;
begin
  DecodeDate(ADate, y, m, d);
  m := m - 1;
  if (m = 0) then
  begin
    m := 12;
    y := y - 1;
  end;
  case m of
    2: if ((y / 4 - trunc(y / 4)) < 0.1) then
        d := 29
      else
        d := 28;
    4, 6, 9, 11: d := 30;
  else
    d := 31;
  end;
  Result := Encodedate(y, m, d);
end;

function MonthEndThis(const ADate: TDateTime): TDateTime;
var
  d, m, y: word;
begin
  DecodeDate(ADate, y, m, d);
  case m of
    2: if ((y / 4 - trunc(y / 4)) < 0.1) then
        d := 29
      else
        d := 28;
    4, 6, 9, 11: d := 30;
  else
    d := 31;
  end;
  Result := Encodedate(y, m, d);
end;

function MonthStartThis(const ADate: TDateTime): TDateTime;
var
  d, m, y: word;
begin
  DecodeDate(ADate, y, m, d);
  Result := Encodedate(y, m, 1);
end;

function MonthEndNext(const ADate: TDateTime): TDateTime;
var
  d, m, y: word;
begin
  DecodeDate(ADate, y, m, d);
  inc(m);
  if (m = 13) then
  begin
    m := 1;
    inc(y);
  end;
  case m of
    2: if ((y / 4 - trunc(y / 4)) < 0.1) then
        d := 29
      else
        d := 28;
    4, 6, 9, 11: d := 30;
  else
    d := 31;
  end;
  Result := Encodedate(y, m, d);
end;

procedure MoveMyFocus(Sender: TForm; var Key: Char);
var
  I: Integer;
begin
  if not (Key = Chr(VK_RETURN)) then
    Exit;
  with Sender do
  begin
    if (ActiveControl is TCustomMemo) then
      Exit;
    if (ActiveControl is TRichEdit) then
      Exit;
    if (ActiveControl is TDBGrid) then
      with TDBGrid(ActiveControl) do
      begin
        if dgRowSelect in Options then
          SendMessage(Sender.handle, WM_NEXTDLGCTL, 0, 0)
        else
        begin
          I := SelectedIndex;
          repeat
            Inc(I);
            if ValidFieldIndex(I) then
              if (Columns[I].ReadOnly = False) and (Columns[I].Visible) then
              begin
                SelectedIndex := I;
                Exit;
              end;
          until (i = Columns.Count);
          I := -1;
          repeat
            Inc(I);
            if ValidFieldIndex(i) then
              if (Columns[I].ReadOnly = False) then
                Break;
          until (I = Columns.Count);
          SelectedIndex := I;
          with Datasource.Dataset do
          begin
            if State in [dsEdit, dsInsert] then
              Post;
            if not FindNext then
              Append;
          end;
        end;
      end
      else
      begin
        if Assigned(ActiveControl) then
        begin
          if (ActiveControl is TDBEdit)
          and Assigned(TDBEdit(ActiveControl).OnKeyPress) then
            TDBEdit(ActiveControl).OnKeyPress(ActiveControl, Key);
          if (ActiveControl is TEdit)
          and Assigned(TEdit(ActiveControl).OnKeyPress) then
            TEdit(ActiveControl).OnKeyPress(ActiveControl, Key);
          if (ActiveControl is TDBListBox)
          and Assigned(TDBListBox(ActiveControl).OnKeyPress) then
            TDBListBox(ActiveControl).OnKeyPress(ActiveControl, Key);
          if (ActiveControl is TDBComboBox)
          and Assigned(TDBComboBox(ActiveControl).OnKeyPress) then
            TDBComboBox(ActiveControl).OnKeyPress(ActiveControl, Key);
          if (ActiveControl is TDBCheckBox)
          and Assigned(TDBCheckBox(ActiveControl).OnKeyPress) then
            TDBCheckBox(ActiveControl).OnKeyPress(ActiveControl, Key);
          if (ActiveControl is TDBLookUpListBox)
          and Assigned(TDBLookUpListBox(ActiveControl).OnKeyPress) then
            TDBLookUpListBox(ActiveControl).OnKeyPress(ActiveControl, Key);
          if (ActiveControl is TDBLookUpComboBox)
          and Assigned(TDBLookUpComboBox(ActiveControl).OnKeyPress) then
            TDBLookUpComboBox(ActiveControl).OnKeyPress(ActiveControl, Key);
        end;
        if not (ActiveControl is TButton) then
          SendMessage(Sender.Handle, WM_NEXTDLGCTL, 0, 0); // next tab
      end;
  end;
  Key := Chr(0);
end;

function NewButton(AOwner, AParent: TWinControl; const L, T, W, H: integer;
  ACaption: string): TButton;
begin
  Result := TButton.Create(AOwner);
  with Result do
  begin
    Parent := AParent;
    Left := L;
    Top := T;
    Width := W;
    Height := H;
    Caption := ACaption
  end;
end;

function NewGroupBox(AOwner, AParent: TWinControl; const L, T, W, H: integer;
  aCaption: string): TGroupBox;
begin
  Result := TGroupBox.Create(AOwner);
  with Result do
  begin
    Parent := AParent;
    Left := L;
    Top := T;
    Width := W;
    Height := H;
    Caption := aCaption
  end;
end;

function NewLabel(AOwner, AParent: TWinControl; const L, T, W, H: integer;
  aCaption: string): TLabel;
begin
  Result := TLabel.Create(AOwner);
  with Result do
  begin
    Parent := AParent;
    Left := L;
    Top := T;
    Width := W;
    Height := H;
    Caption := aCaption
  end;
end;

function NewCheckbox(AOwner, AParent: TWinControl; const L, T, W, H: integer;
  aCaption: string): TCheckbox;
begin
  Result := TCheckbox.Create(AOwner);
  with Result do
  begin
    Parent := AParent;
    Left := L;
    Top := T;
    Width := W;
    Height := H;
    Caption := aCaption
  end;
end;

function NewEdit(AOwner, AParent: TWinControl; const L, T, W, H: integer):
  TEdit;
begin
  Result := TEdit.Create(AOwner);
  with Result do
  begin
    Parent := AParent;
    Left := L;
    Top := T;
    Width := W;
    Height := H;
  end;
end;

function NewCombo(AOwner, AParent: TWinControl; const L, T, W, H: integer):
  TComboBox;
begin
  Result := TComboBox.Create(AParent);
  with Result do
  begin
    Parent := AParent;
    Left := L;
    Top := T;
    Width := W;
    Height := H;
  end;
end;

procedure EMail(const Originator, Subject, Content: string;
  Recipients, AttachmentFiles: TStrings);
var
  MapiMessage: TMapiMessage;
  dwRet: Cardinal;
  MAPI_Session: Cardinal;
  MError: Cardinal;
  WndList: Pointer;
  A: AnsiString;
  S: string;
  I, FilesCount, RecipsCount: integer;
  Orig: TMapiRecipDesc;
  RecipNames: array[0..1000] of AnsiString;
  RecipAddress: array[0..1000] of AnsiString;
  Recips: array[0..1000] of TMapiRecipDesc;
  FilePaths: array[0..1000] of AnsiString;
  FileNames: array[0..1000] of AnsiString;
  Files: array[0..1000] of TMapiFileDesc;

begin
  dwRet := MapiLogon(Application.Handle, '', '', MAPI_LOGON_UI or
    MAPI_NEW_SESSION, 0, @MAPI_Session);
  if (dwRet <> SUCCESS_SUCCESS) then
  begin
    MessageDlg(StrEmailInitError + ' ' + intToStr(dwRet), mtError, [mbOK], 0);
    Abort;
  end;
  try
    RecipsCount := 0;
    if Assigned(Recipients) then
      for I := 0 to Recipients.Count - 1 do
      begin
        S := Trim(Recipients.Names[I]);
        A := AnsiString(S);
        if (S > '') then
        try
          RecipNames[RecipsCount] := A;
          RecipAddress[RecipsCount] := 'SMTP: ' + A;
          with Recips[RecipsCount] do
          begin
            ulReserved := 0;
            S := Uppercase(Recipients.Values[S]);
            if S = 'TO' then
              ulRecipClass := MAPI_TO;
            if S = 'CC' then
              ulRecipClass := MAPI_CC;
            if S = 'BCC' then
              ulRecipClass := MAPI_BCC;
            lpszName := PAnsiChar(RecipNames[RecipsCount]);
            lpszAddress := PAnsiChar(RecipAddress[RecipsCount]);
            ulEIDSize := 0;
          end;
          Inc(RecipsCount);
        except
        end;
      end;

    // attachments
    FilesCount := 0;
    if Assigned(AttachmentFiles) then
      for I := 0 to AttachmentFiles.Count - 1 do
      begin
        S := Trim(AttachmentFiles.Names[I]);
        if (S > '') then
        try
          FilePaths[FilesCount] := AnsiString(ExtractFilePath(S));
          FileNames[FilesCount] := AnsiString(ExtractFileName(S));
          with Files[RecipsCount] do
          begin
            ulReserved := 0;
            flFlags := 0;
            nPosition := ULONG($FFFFFFFF);
            lpszPathName := PAnsiChar(FilePaths[FilesCount]);
            lpszFileName := PAnsiChar(FileNames[FilesCount]);
            lpFileType := nil;
          end;
          Inc(FilesCount);
        except
        end;
      end;

    with MapiMessage do
    begin
      ulReserved := 0;
      lpszSubject := PAnsiChar(AnsiString(Subject));
      lpszNoteText := PAnsiChar(AnsiString(Content));
      lpszMessageType := nil;
      lpszDateReceived := nil;
      lpszConversationID := nil;
      flFlags := 0;
      A := AnsiString(Trim(Originator));
      Orig.lpszName := PAnsiChar(A);
      Orig.ulRecipClass := MAPI_ORIG;
      Orig.lpszAddress := '';
      lpOriginator := @Orig;
      nRecipCount := RecipsCount;
      lpRecips := @Recips;
      nFileCount := FilesCount;
      lpFiles := @Files;
    end;

    WndList := DisableTaskWindows(0);
    try
      MError := MapiSendMail(MAPI_Session, Application.Handle, MapiMessage,
        MAPI_DIALOG or MAPI_LOGON_UI or MAPI_NEW_SESSION, 0);
      if MError <> 0 then
        MessageDlg(StrEmailSendError, mtError, [mbOK], 0);
    finally EnableTaskWindows(WndList);
    end;
  finally MapiLogOff(MAPI_Session, Application.Handle, 0, 0);
  end;
end;

{
function MyDocumentsFolder: string;
var
  P: PChar;
begin
  P := nil;
  try
    P := AllocMem(MAX_PATH);
    if SHGetFolderPath(0, CSIDL_PERSONAL, 0, 0, P) = S_OK then
      Result := P
    else
      Result := GetCurrentDir;
  finally
    FreeMem(P);
  end;
end;
}

procedure GetDiskDrives(DriveList: TStrings);
const
  DriveLetters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
var
  i: integer;
begin
  DriveList.Clear;
  DriveList.Append(DriveLetters[1] + ':');
  for i := 2 to 26 do
    if DirectoryExists(DriveLetters[i] + ':\') then
      DriveList.Append(DriveLetters[i] + ':');
end;

function CleanSQLName(AName: string): string;
var
  I: integer;
  C: char;
const
  illegals = '()&.%' + chr(39);
begin
  Result := Uppercase(StringReplace(Trim(AName), ' ', '_', [rfReplaceAll]));
  for I := 1 to length(illegals) do
    while (pos(illegals[I], Result) > 0) do
      System.Delete(Result, pos(illegals[I], Result), 1);
  if (Result = '') then
    exit;
  if (pos(Result, 'ACTION CURRENT DATE SORT TYPE VALUE FILE') > 0) then
    Result := Result + '_';
  if (pos(Result, 'ADMIN CHARACTER DATABASE COUNT DESC DAY') > 0) then
    Result := Result + '_';
  if (pos(Result, 'DEFAULT DOMAIN EVENT END USER LENGTH GROUP') > 0) then
    Result := Result + '_';
  if (pos(Result, 'KEY MANUAL LOG GRANT HELP INDEX CHECK END') > 0) then
    Result := Result + '_';
  if (pos(Result, 'OPTION MESSAGE PASSWORD MONTH YEAR ORDER') > 0) then
    Result := Result + '_';
  if (pos(Result, 'STARTS TO STATEMENT TABLE TIME VERSION SIZE') > 0) then
    Result := Result + '_';
  if (pos(Result, 'STARTING SUM UNION WORK TRANSLATION POSITION') > 0) then
    Result := Result + '_';
  C := Result[1];
  if CharInSet(C, ['0'..'9']) then
  begin
    System.Delete(Result, 1, 1);
    Result := DigitToString(C) + Result;
  end;
  Result := Copy(Result, 1, 31);
end;

function DigitToString(ADigit: char): string;
begin
  case ADigit of
    '1': Result := 'ONE';
    '2': Result := 'TWO';
    '3': Result := 'THREE';
    '4': Result := 'FOUR';
    '5': Result := 'FIVE';
    '6': Result := 'SIX';
    '7': Result := 'SEVEN';
    '8': Result := 'EIGHT';
    '9': Result := 'NINE';
  else
    Result := 'ZERO';
  end;
end;

function GetShellFoldername(const FolderID: Integer): string;
var
  ShellMalloc: IMalloc;
  ppidl: PItemIdList;
begin
  { CSIDL_DESKTOP
    CSIDL_INTERNET
    CSIDL_PROGRAMS
    CSIDL_CONTROLS
    CSIDL_PRINTERS
    CSIDL_PERSONAL
    CSIDL_FAVORITES
    CSIDL_STARTUP
    CSIDL_RECENT
    CSIDL_SENDTO
    CSIDL_BITBUCKET
    CSIDL_STARTMENU
    CSIDL_DESKTOPDIRECTORY
    CSIDL_DRIVES
    CSIDL_NETWORK
    CSIDL_NETHOOD
    CSIDL_FONTS
    CSIDL_TEMPLATES
    CSIDL_COMMON_STARTMENU
    CSIDL_COMMON_PROGRAMS
    CSIDL_COMMON_STARTUP
    CSIDL_COMMON_DESKTOPDIRECTORY
    CSIDL_APPDATA
    CSIDL_PRINTHOOD
    CSIDL_ALTSTARTUP
    CSIDL_COMMON_ALTSTARTUP
    CSIDL_COMMON_FAVORITES
    CSIDL_INTERNET_CACHE
    CSIDL_COOKIES
    CSIDL_HISTORY
    CSIDL_COMMON_APPDATA
  }
  try
    ppidl := nil;
    try
      if SHGetMalloc(ShellMalloc) = NOERROR then
      begin
        SHGetSpecialFolderLocation(Application.Handle, folderID, ppidl);
        //  ShGetFolderLocation.(Form1.Handle, folderID, ppidl);
        SetLength(Result, MAX_PATH);
        if not SHGetPathFromIDList(ppidl, PChar(Result)) then
          raise exception.create('SHGetPathFromIDList failed : invalid pidl');
        SetLength(Result, lStrLen(PChar(Result)));
      end;
    finally
      if ppidl <> nil then
        CoTaskMemFree(ppidl);
    end;
  except Result := '';
  end;
end;

procedure TrashFolder(const Path: string);
var
  SearchRec: TSearchRec;
  P: string;
begin
  P := Path;
  if not DirectoryExists(P) then
    Exit;
  P := P + PathDelim;
  try
    if (FindFirst(P + '*', faAnyFile, SearchRec) = 0) then
      repeat
        if ((SearchRec.Attr and faDirectory) > 0) then
        begin
          if (pos('.', SearchRec.Name) <> 1) then
            TrashFolder(P + SearchRec.Name);
        end
        else
          DeleteFile(PWideChar(P + SearchRec.Name));
      until not (FindNext(searchRec) = 0);
  finally System.SysUtils.FindClose(SearchRec);
  end;
  RemoveDir(Path);
end;

end.

