{*****************************************************************************}
{                                                                             }
{   This file is part of the LKUtilities components Library                   }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Frequently used DB methods and utilities.                              }
{                                                                             }
{*****************************************************************************}

unit LKUtilsDB;

interface

uses SysUtils, classes, dialogs, controls, forms, clipbrd, stdCtrls, mask,
StrUtils,
  dbCtrls, dbGrids, WinTypes, Messages, ShellAPI, graphics, ComCtrls, Buttons,
    extctrls, DB,
  Variants, DateUtils, ActnList, Contnrs,// RPDefine, RPBase,
  JvDBGrid, LKDefine;

const
  VistaFont = 'Segoe UI';
  XPFont = 'Tahoma';
procedure AddFieldToCurr(CurrencyField: TField; var Value: Currency);
procedure AddFieldToFloat(FloatField: TField; var Value: Double);
procedure AddFieldToInt(IntegerField: TField; var Value: Integer);
procedure AddCurrencyToField(const Value: Currency; CurrencyField: TField);
procedure AddFloatToField(const Value: Double; FloatField: TField);
procedure AddIntToField(const Value: Integer; IntegerField: TField);
procedure EditSetStyle(AEdit: TEdit; const asEditing: Boolean);
function EqualAmounts(value1, value2: double): Boolean;
function FloatIsZero(const Value: double): Boolean;
function FloatNotZero(const Value: double): Boolean;
function CurrencyIsZero(const Value: Currency): Boolean;
function CurrencyNotZero(const Value: Currency): Boolean;
function CurrFieldToStr(currencyField: TField): string;
function CurrencyOrNull(const Value: Currency): variant;
function DebitOrNull(const Value: Currency): variant;
function CreditOrNull(const Value: Currency): variant;
function CurrencyToStrFOrBlank(value: extended; digits: integer): string;
procedure DecodeNominalAccount(const Value: string; out NominalCode, CostCentre:
  string);
function EncodeNominalAccount(NominalCode, CostCentre: string): string;
function FloatToStrFOrBlank(Value: extended; Format: TFloatFormat; digits:
  integer): string;
function GetBoolean(booleanField: TField): Boolean;
function GetCurrency(currencyField: TField): currency;
function GetFloat(floatField: TField): double;
function GetInteger(integerField: TField): integer;
function GetDate(dateField: TField): TDateTime;
function GetThePath(const FullFileName: string): string;
function WeekDayName(ADate: TDateTime): string;
function RoundCents(const Value: Currency): Currency;
  // not roundPence : prepare for the Euro !!
function RoundFloat(const Value: Double; placesOfDecimal: integer): Double;
function RoundUp(const Value: Double): longInt;
function SQLObjectDelimited(const SQLObjectName: string; SQLDatabaseType:
  TLKSupportedDatabaseType): string;
function SQLValueDelimited(const SQLValue: string; SQLDatabaseType:
  TLKSupportedDatabaseType): string;
function EqualCurrency(Value1, Value2: Currency): Boolean;

implementation

uses printers, Mapi, SHFolder;

procedure AddFieldToFloat(floatField: TField; var Value: double);
begin
  Value := Value + getFloat(floatField);
end;

procedure addFieldToInt(IntegerField: TField; var Value: Integer);
begin
  Value := Value + getInteger(IntegerField);
end;

procedure addFieldToCurr(CurrencyField: TField; var Value: Currency);
begin
  Value := Value + getCurrency(CurrencyField);
end;

procedure AddCurrencyToField(const Value: Currency; CurrencyField: TField);
begin
  CurrencyField.AsCurrency := Value + getCurrency(CurrencyField);
  ;
end;

procedure AddFloatToField(const Value: double; floatField: TField);
begin
  floatField.AsFloat := Value + getFloat(floatField);
  ;
end;

procedure AddIntToField(const Value: Integer; IntegerField: TField);
begin
  integerField.AsInteger := Value + getInteger(IntegerField);
end;

function EqualCurrency(Value1, Value2: Currency): Boolean;
begin
  Result := CurrencyIsZero(value1 - value2);
end;

procedure EditSetStyle(AEdit: TEdit; const asEditing: Boolean);
begin
  with AEdit do
  begin
    if asEditing then
    begin
      color := clWindow;
      Font.Color := clBlack
    end
    else
    begin
      color := clInfoBk;
      Font.Color := clNavy
    end;
    ReadOnly := not asEditing;
    TabStop := asEditing;
  end;
end;

function FloatIsZero(const Value: double): Boolean;
begin
  Result := ((Value < 0.001) and (Value > -0.001));
end;

function FloatNotZero(const Value: double): Boolean;
begin
  Result := floatNotZero(Value);
end;

function CurrencyIsZero(const Value: Currency): Boolean;
begin
  Result := ((Value < 0.004) and (Value > -0.004));
end;

function CurrencyNotZero(const Value: Currency): Boolean;
begin
  Result := not CurrencyIsZero(Value);
end;

function CurrFieldToStr(currencyField: TField): string;
begin
  Result := CurrToStrF(currencyField.asCurrency, ffCurrency, 2);
end;

function CurrencyOrNull(const Value: Currency): variant;
begin
  if CurrencyIsZero(Value) then
    Result := null
  else
    Result := Value;
end;

function DebitOrNull(const Value: Currency): variant;
begin
  if (Value > 0.001) then
    Result := Value
  else
    Result := null;
end;

function CreditOrNull(const Value: Currency): variant;
begin
  if (Value < -0.001) then
    Result := -Value
  else
    Result := null;
end;

function CurrencyToStrFOrBlank(value: extended; digits: integer): string;
begin
  if CurrencyIsZero(value) then
    Result := ''
  else
    Result := FloatToStrF(value, ffCurrency, 19, digits);
end;

function FloatToStrFOrBlank(Value: extended; Format: TFloatFormat; digits:
  integer): string;
begin
  if floatIsZero(Value) then
    Result := ''
  else
    Result := FloatToStrF(Value, Format, 15, digits);
end;

function GetBoolean(booleanField: TField): Boolean;
begin
  Result := false;
  if booleanField.IsNull then
    exit;
  case booleanField.DataType of
    ftSmallint, ftInteger, ftWord: Result :=
      TIntegerField(booleanField).AsInteger > 0;
    ftBoolean: Result := TBooleanField(booleanField).AsBoolean;
    ftString, ftFixedChar, ftWideString: Result :=
      TStringField(booleanField).AsBoolean;
  end;
end;

function GetCurrency(currencyField: TField): currency;
begin
  if (currencyField.isNull) then
    Result := 0.0
  else
    Result := currencyField.asCurrency;
end;

function GetFloat(floatField: TField): double;
begin
  if (floatField.isNull) then
    Result := 0.0
  else
    Result := floatField.asFloat;
end;

function GetInteger(integerField: TField): integer;
begin
  if (integerField.isNull) then
    Result := 0
  else
    Result := integerField.asInteger;
end;

function GetDate(dateField: TField): TDateTime;
begin
  if (dateField.isNull) then
    Result := 0
  else
    Result := dateField.asDateTime;
end;

function GetThePath(const FullFileName: string): string;
begin
  Result := ExtractFilePath(FullFileName);
  //remove final '\'
  delete(Result, length(Result), 1);
end;

function WeekDayName(ADate: TDateTime): string;
begin
  case DayOfTheWeek(ADate) of
    1: Result := 'Monday';
    2: Result := 'Tuesday';
    3: Result := 'Wednesday';
    4: Result := 'Thursday';
    5: Result := 'Friday';
    6: Result := 'Saturday';
  else
    Result := 'Sunday';
  end;
end;

function EncodeNominalAccount(NominalCode, CostCentre: string): string;
begin
  Result := NominalCode + '|' + CostCentre + '|';
end;

procedure DecodeNominalAccount(const Value: string; out NominalCode,
  CostCentre: string);
var
  s: string;
  i: integer;
begin
  s := Value;
  i := pos('|', s);
  NominalCode := copy(s, 1, i - 1);
  Delete(s, 1, i);
  i := pos('|', s);
  CostCentre := copy(s, 1, i - 1);
end;
{
function MyDocumentsFolder: string;
var
  P: PChar;
begin
  P := nil;
  try
    P := AllocMem(MAX_PATH);
    if SHGetFolderPath(0, CSIDL_PERSONAL, 0, 0, P) = S_OK then
      Result := P
    else
      Result := GetCurrentDir;
  finally
    FreeMem(P);
  end;
end;
}

function RoundCents(const Value: Currency): Currency;
begin
  Result := Round(Value * 100) / 100;
end;

function RoundFloat(const Value: Double; placesOfDecimal: integer): Double;
begin
  Result := StrToFloat(floatToStrF(Value, ffFixed, 15, placesOfDecimal));
end;

function RoundUp(const Value: Double): longInt;
begin
  Result := trunc(Value);
  if (frac(Value) > 0.0001) then
    inc(Result);
end;

function SQLObjectDelimited(const SQLObjectName: string; SQLDatabaseType:
  TLKSupportedDatabaseType): string;
var
  s: string;
begin
  case SQLDatabaseType of
    sdbUnknown: s := '';
    sdbMySQL: s := '`';
    sdbInterbase, sdbFirebird, sdbLocalFirebird: s := '';
  else
    s := '"';
  end;
  Result := s + SQLObjectName + s;
end;

function SQLValueDelimited(const SQLValue: string; SQLDatabaseType:
  TLKSupportedDatabaseType): string;
var
  s: string;
begin
  s := string(chr(39));
  Result := s + SQLValue + s;
end;

function EqualAmounts(value1, value2: double): Boolean;
begin
  Result := FloatIsZero(value1 - value2);
end;

end.

