{*****************************************************************************}
{                                                                             }
{   This file is part of the LKUtilities components Library                   }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form to show a TLKViewDialog for single item input.                    }
{                                                                             }
{*****************************************************************************}

unit LKViewTxDg;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls, SysUtils, Dialogs, Mask, LKDefine, LKBaseComponent;

type
  TLKViewDlg = class(TForm)
    OKBtn: TButton;
    CancelBtn: TButton;
    rgBoolean: TRadioGroup;
    btnHint: TButton;
    ComboBox1: TComboBox;
    ckbxOption: TCheckBox;
    Edit1: TMaskEdit;
    HeaderPanel: TLKHeaderPanel;
    procedure OKBtnClick(Sender: TObject);
    procedure btnHintClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    FValidateType: TLKVDDataType;
    FActiveEdit: TWinControl;
    procedure CheckValid;
  public
    { Public declarations }
    property ValidateType: TLKVDDataType read FValidateType write FValidateType;
    property ActiveEdit: TWinControl read FActiveEdit write FActiveEdit;
//    myHint: string;
  end;

var
  LKViewDlg: TLKViewDlg;

implementation

{$R *.DFM}

procedure TLKViewDlg.CheckValid;
var
  S: string;
begin
  if (FValidateType = vStringCombo) then
    S := ComboBox1.Text
  else
    S := Edit1.Text;
  case FValidateType of
    vBoolean: ;
    vDate: StrToDate(S);
    vInteger: StrToInt(S);
    vNumber: StrToFloat(S);
    vCurrency:
      begin
        if not CharInSet(S[1], (['0'..'9', '-'])) then
          Delete(S, 1, 1);
        StrToFloat(S);
        Edit1.Text := S;
      end;
    vStringCombo: with ComboBox1 do
        if (Items.IndexOf(Text) < 0) then
          raise LKError.Create('Not a valid selection');
  end;
end;

procedure TLKViewDlg.OKBtnClick(Sender: TObject);
begin
  try
    CheckValid;
    ModalResult := mrOK;
  except FActiveEdit.SetFocus;
    raise;
  end;
end;

procedure TLKViewDlg.btnHintClick(Sender: TObject);
begin
  ShowMessage(Hint);
end;

procedure TLKViewDlg.FormShow(Sender: TObject);
begin
  ActiveControl := FActiveEdit;
end;

end.

