{*****************************************************************************}
{                                                                             }
{   This file is part of the LKUtilities components Library                   }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form to show a TLKZipper zip progress.                                 }
{                                                                             }
{*****************************************************************************}

unit LKZipProgressDlg;

interface

uses
  Windows, Messages, System.SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Gauges,
  AbUtils ,AbConst, AbArcTyp, AbZBrows, AbZipper, AbZipKit, AbMeter,
  AbBase, AbBrowse;

type
  EZipError = class(Exception);

type
  TFormLKZipProgress = class(TForm)
    LabelFilename: TLabel;
    Label2: TLabel;
    BtnCancel: TButton;
    AbZipKit1: TAbZipKit;
    LabelStatus: TLabel;
    AbMeterSProg: TAbMeter;
    AbMeterIProg: TAbMeter;
    AbMeterAProg: TAbMeter;
    Label1: TLabel;
    StaticTextTitle: TStaticText;
    procedure AbZipKit1ArchiveItemProgress(Sender: TObject;
      Item: TAbArchiveItem; Progress: Byte; var Abort: Boolean);
    procedure AbZipKit1ArchiveProgress(Sender: TObject; Progress: Byte;
      var Abort: Boolean);
    procedure BtnCancelClick(Sender: TObject);
    procedure AbZipKit1RequestBlankDisk(Sender: TObject;
      var Abort: Boolean);
    procedure AbZipKit1ProcessItemFailure(Sender: TObject;
      Item: TAbArchiveItem; const ProcessType: TAbProcessType;
      ErrorClass: TAbErrorClass; ErrorCode: Integer);
    procedure AbZipKit1Save(Sender: TObject);
    procedure AbZipKit1Load(Sender: TObject);
    procedure AbZipKit1NeedPassword(Sender: TObject;
      var NewPassword: AnsiString);
  private
    { Private declarations }
    AbortZip: boolean;
  public
    { Public declarations }
    diskDrive: string;
    function AskForFirstDisk: boolean;
    function Zip(folder: string; backupFile, zipFolder: string; subfolders:
      boolean): boolean;
    function ZipFiles(AddFileNames: TStrings; BaseFolder, BackupFile, zipFolder:
      string): boolean;
    function UnZip(targetFolder, restoreFile: string): boolean;
  end;

var
  FormLKZipProgress: TFormLKZipProgress;

  {function SHFormatDrive(hWind : HWND; Drive : Word; fmtID : Word; Options : Word)
   : Longint stdCall;external 'Shell32.dll' name 'SHFormatDrive'; }

implementation

uses LKWrZipDlg;

{$R *.DFM}

function TFormLKZipProgress.askForFirstDisk: boolean;
var
  cancelled: boolean;
begin
  cancelled := false;
  AbZipKit1RequestBlankDisk(self, cancelled);
  result := not cancelled;
end;

function TFormLKZipProgress.Zip(folder: string; backupFile, zipFolder: string;
  subfolders: boolean): boolean;
begin
  AbortZip := false;
  //GaugeArchive.progress := 0;
  try
    application.ProcessMessages;
    with AbZipKit1 do
    begin
      FileName := zipFolder + '\' + backupFile + '.zip';
      if subfolders and not (soRecurse in storeOptions) then
        storeOptions := storeOptions + [soRecurse];
      if (soRecurse in storeOptions) and (not subfolders) then
        storeOptions := storeOptions - [soRecurse];
      if (count > 0) then
      begin
        caption := 'Emptying existing archive';
        application.ProcessMessages;
        deleteFiles('*.*');
      end;
      BaseDirectory := folder;
      Caption := 'Compressing files ...';
      Application.ProcessMessages;
      AddFiles('*.*', 0);
      Caption := 'Writing ' + backupFile + '.zip';
      Application.ProcessMessages;
      Save;
      CloseArchive;
      Filename := '';
      BaseDirectory := 'C:\';
    end;
  finally result := not AbortZip;
  end;
end;

function TFormLKZipProgress.ZipFiles(AddFileNames: TStrings; BaseFolder,
  BackupFile,
  zipFolder: string): boolean;
var
  I: integer;
begin
  AbortZip := false;
  //GaugeArchive.progress := 0;
  try
    application.ProcessMessages;
    with AbZipKit1 do
    begin
      FileName := zipFolder + '\' + backupFile + '.zip';
      if (soRecurse in storeOptions) then
        storeOptions := storeOptions - [soRecurse];
      if (count > 0) then
      begin
        caption := 'Emptying existing archive';
        application.ProcessMessages;
        DeleteFiles('*.*');
      end;
      baseDirectory := BaseFolder;
      caption := 'Compressing files ...';
      application.ProcessMessages;
      for I := 0 to AddFileNames.Count - 1 do
        AddFiles(AddFileNames[I], 0);
      caption := 'Writing ' + backupFile + '.zip';
      application.ProcessMessages;
      Save;
      CloseArchive;
      Filename := '';
      BaseDirectory := 'C:\';
    end;
  finally result := not AbortZip;
  end;
end;

function TFormLKZipProgress.UnZip(targetfolder, restoreFile: string): boolean;
begin
  AbortZip := false;
  //GaugeArchive.progress := 0;
  try
    AbZipKit1.BaseDirectory := targetFolder;
    caption := 'Decompressing ' + restoreFile;
    application.ProcessMessages;
    if not fileExists(restoreFile) then
      raise EZipError.create('Could not find ' + restoreFile);
    with AbZipKit1 do
    begin
      FileName := restoreFile;
      ExtractFiles('*.*');
      CloseArchive;
    end;
  finally result := not abortZip;
  end;
end;

procedure TFormLKZipProgress.AbZipKit1ArchiveItemProgress(Sender: TObject;
  Item: TAbArchiveItem; Progress: Byte; var Abort: Boolean);
begin
  abort := AbortZip;
  LabelFilename.caption := item.filename;
  case Item.action of //aaFailed, aaNone, aaAdd, aaDelete, aaFreshen, aaMove
    aaAdd: LabelStatus.caption := 'adding...';
    aaFreshen: LabelStatus.caption := 'updating...';
    aaFailed: LabelStatus.caption := 'error...';
    aaDelete: LabelStatus.caption := 'deleteing...';
  else
    LabelStatus.caption := '';
  end;
  Application.processMessages;
end;

procedure TFormLKZipProgress.AbZipKit1ArchiveProgress(Sender: TObject;
  Progress: Byte; var Abort: Boolean);
begin
  abort := AbortZip;
  //GaugeArchive.progress := progress;
  Application.processMessages;
end;

procedure TFormLKZipProgress.BtnCancelClick(Sender: TObject);
begin
  AbortZip := true;
  Application.ProcessMessages;
end;

procedure TFormLKZipProgress.AbZipKit1RequestBlankDisk(Sender: TObject;
  var Abort: Boolean);
var
  clearFiles: boolean;
  searchRec: TSearchRec;
  drive: byte;
  holdCaption: string;
begin
  drive := ord(diskdrive[1]) - 64; // A = 1, B = 2 etc.
  with TLKWriteZIPDlg.create(self) do
  begin
    try
      diskDriveLabel.caption := diskDrive;
      cbxClearFiles.checked := (drive < 3);
      if (showModal = mrCancel) then
      begin
        abort := true;
        exit;
      end;
      clearFiles := cbxClearFiles.checked;
    finally free;
    end;
  end;
  if clearFiles and not abort then
  begin
    holdCaption := caption;
    caption := 'Deleting files on removableDisk disk';
    while (findFirst(diskDrive + '\*.*', 0, searchRec) = 0) do
      deleteFile(diskDrive + '\' + searchRec.name);
    findClose(searchRec);
    caption := holdCaption;
  end;
end;

procedure TFormLKZipProgress.AbZipKit1ProcessItemFailure(Sender: TObject;
  Item: TAbArchiveItem; const ProcessType: TAbProcessType;
  ErrorClass: TAbErrorClass; ErrorCode: Integer);
begin
  {TAbErrorClass = (ecAbbrevia, ecInOutError, ecFilerError, ecFileCreateError,
                   ecFileOpenError, ecOther);}
  case ErrorClass of
    ecAbbrevia: ShowMessage(AbStrRes(ErrorCode));
    ecInOutError: ShowMessage('Input/output error : filename ' + item.Filename);
    ecFilerError: ShowMessage('Filer error : filename ' + item.Filename);
    ecFileCreateError: ShowMessage('File create error : filename ' +
      item.Filename);
    // ecFileOpenError : ShowMessage('File open error : filename ' + item.Filename);
  else
    ShowMessage('Unknown error : filename ' + item.Filename);
  end;
end;

procedure TFormLKZipProgress.AbZipKit1Save(Sender: TObject);
begin
  LabelStatus.caption := 'Saving archive ...';
end;

procedure TFormLKZipProgress.AbZipKit1Load(Sender: TObject);
begin
  LabelStatus.caption := 'Loading archive ...';
end;

procedure TFormLKZipProgress.AbZipKit1NeedPassword(Sender: TObject;
  var NewPassword: AnsiString);
begin
  //with EnterPasswordDialog do if execute then NewPassword := value;
end;

end.

