{*****************************************************************************}
{                                                                             }
{   This file is part of the LKCRMBase components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      LKCRMBaseComponents common to most Customer Relations Management       }
{      systems                                                                }
{                                                                             }
{*****************************************************************************}

unit LKCRMBaseComponents;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  DB, Dialogs,
  LKRecordsComponents, LKAccountsComponents;

type
  TLKRecordsOrderEntry = class(TLKRecordsPosting)
  private
    FSalesInvoices: TLKDocumentsSalesInvoices;
    procedure SetSalesInvoices(Value: TLKDocumentsSalesInvoices);
  protected
    procedure AssignTo(Dest: TPersistent); override;
    procedure CreateViewFormRecord; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation);
        override;
  public
    procedure CreateViewForm; override;
    procedure DlgNewInvoiceWizardExecute; virtual;
    procedure DrillDownRecord; override;
  published
    property SalesInvoices: TLKDocumentsSalesInvoices read FSalesInvoices write
        SetSalesInvoices;
  end;

  TLKPricesBase = class(TLKRecordsInput)
  protected
    procedure CreateViewFormRecord; override;
    procedure SetActive(Value: Boolean); override;
  public
    procedure DlgCopyPriceItemsFromExecute(ParentForm: TForm); virtual;
  end;

  TLKPriceLists = class(TLKPricesBase)
  end;

  TLKDocumentsSalesInvoicesPriced = class(TLKDocumentsSalesInvoices)
  private
    FPriceLists: TLKPriceLists;
    procedure SetPriceLists(Value: TLKPriceLists);
  protected
    procedure CreateViewFormRecord; override;
    procedure DoDetailAfterInsert; override;
    procedure DoDetailBeforePost; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation);
        override;
  published
    property PriceLists: TLKPriceLists read FPriceLists write SetPriceLists;
  end;


procedure Register;

implementation

uses
  LKUtils, LKUtilsDB, LKRecordsInputCopyFromDlg,
  LKDocumentsOrderEntryDlg, LKDocumentsOrderEntryRecordDlg,
  LKPricesBaseInputRecordDlg, LKSalesInvoicesPricedRecordDlg;

procedure Register;
begin
//  RegisterComponents('LKCRMBase', [TLKPriceLists]);
  RegisterComponents('LKCRMBase', [TLKDocumentsSalesInvoicesPriced]);
  RegisterComponents('LKCRMBase', [TLKPriceLists]);
end;

{
***************************** TLKRecordsOrderEntry *****************************
}
procedure TLKRecordsOrderEntry.AssignTo(Dest: TPersistent);
begin
  inherited;
  TLKRecordsOrderEntry(Dest).SalesInvoices := SalesInvoices;
end;

procedure TLKRecordsOrderEntry.CreateViewForm;
begin
  Application.CreateForm(TFormLKDocumentsOrderEntry, FFormView);
  TFormLKDocumentsOrderEntry(FormView).Construct(Self);
end;

procedure TLKRecordsOrderEntry.CreateViewFormRecord;
begin
  Application.CreateForm(TFormLKDocumentsOrderEntryRecord, FFormViewRecord);
  TFormLKDocumentsOrderEntryRecord(FormViewRecord).Construct(Self);
end;

procedure TLKRecordsOrderEntry.DlgNewInvoiceWizardExecute;
begin
  if not SalesInvoices.Active then
    SalesInvoices.ACGroup.Open;
end;

procedure TLKRecordsOrderEntry.DrillDownRecord;
begin
  DlgNewInvoiceWizardExecute;
end;

procedure TLKRecordsOrderEntry.Notification(AComponent: TComponent; Operation:
    TOperation);
begin
  if (Operation = opRemove) then
  begin
    if (AComponent = FSalesInvoices) then
      FSalesInvoices := nil;
  end;
  inherited Notification(AComponent, Operation);
end;

procedure TLKRecordsOrderEntry.SetSalesInvoices(Value:
    TLKDocumentsSalesInvoices);
begin
  if FSalesInvoices <> Value then
  begin
    if assigned(Value) then
      Value.FreeNotification(Self);
  FSalesInvoices := Value;
  end;
end;

{
******************************** TLKPricesBase *********************************
}
procedure TLKPricesBase.CreateViewFormRecord;
begin
  Application.CreateForm(TFormLKPricesBaseInputRecord, FFormViewRecord);
  TFormLKPricesBaseInputRecord(FormViewRecord).Construct(Self);
end;

procedure TLKPricesBase.DlgCopyPriceItemsFromExecute(ParentForm: TForm);
var
  I: Integer;
  J: Integer;
  K: Integer;
  BM: TBookmark;
  DetailArray: Variant;
begin
  //check Quotes detail is empty
  if TblDetail.FindFirst then
    raise
      Exception.Create('You cannot copy to this Price list because it already'
        +' has priced items');
  BM := TblMaster.GetBookmark;
  DlgLKRecordsInputCopyFrom := TDlgLKRecordsInputCopyFrom.Create(nil);
  try
    DlgLKRecordsInputCopyFrom.PopUpParent := ParentForm;
    DlgLKRecordsInputCopyFrom.Construct(Self);
    if (DlgLKRecordsInputCopyFrom.ShowModal = mrOK) then
    begin
      J := 0;
      with TblDetail do if FindFirst then
      begin
        repeat
          Inc(J);
        until not FindNext;
        DetailArray :=
          VarArrayCreate([1,FieldCount-1,1,J],varVariant);
        FindFirst;
        for K := 1 to J do
        begin
          for I := 1 to FieldCount - 1 do
            DetailArray[I,K] := Fields[I].Value;
          FindNext;
        end;
      end;
        TblMaster.GotoBookmark(BM);
      RefreshMSumDPairs := False;
      try
        with TblDetail do
        begin
          for K := 1 to J do
          begin
            Append;
            for I := 1 to FieldCount - 1 do
            if Fields[I].IsNull and not VarIsNull(DetailArray[I,K]) then
              Fields[I].Value := DetailArray[I,K];
            Post;
          end;
          ApplyUpdates(0);
        end;
      finally
        RefreshMSumDPairs := True;
      end;
    end;
  finally
    FreeAndNil(DlgLKRecordsInputCopyFrom);
    TblMaster.FreeBookmark(BM);
  end;
end;

procedure TLKPricesBase.SetActive(Value: Boolean);
begin
  inherited SetActive(Value);
  if Active then
    TblMaster.FindLast;
end;

{
*********************** TLKDocumentsSalesInvoicesPriced ************************
}
procedure TLKDocumentsSalesInvoicesPriced.CreateViewFormRecord;
begin
  Application.CreateForm(TFormLKSalesInvoicesPricedRecord, FFormViewRecord);
  TFormLKSalesInvoicesPricedRecord(FormViewRecord).Construct(Self);
end;

procedure TLKDocumentsSalesInvoicesPriced.DoDetailAfterInsert;
begin
  inherited DoDetailAfterInsert;
  if assigned(PriceLists) then
    with PriceLists.TblMaster do
      if (RecordCount > 0) then
        TblDetail.FieldByName('PRICE_LIST').AsString :=
          FieldByName('PRICE_LIST').AsString;
end;

procedure TLKDocumentsSalesInvoicesPriced.DoDetailBeforePost;
var
  pc, n, d: string;
  r: Currency;
  vc: Integer;
begin
  with TblDetail do
  begin
    pc := FieldByName('PRICE_CODE').asString;
    if PriceLists.TblDetail.Locate('PRICE_CODE', pc, []) then
    begin
      with PriceLists.TblDetail do
      begin
        r := GetCurrency(FieldByName('UNIT_PRICE'));
        n := FieldByName('NOMINAL_CODE').asString;
        d := FieldByName('DESCRIPTION').asString;
        vc := GetInteger(FieldByName('VAT_CODE'));
      end;
      FieldByName('NOMINAL_CODE').asString := n;
      if (FieldByName('DESCRIPTION').asString = '') then
        FieldByName('DESCRIPTION').asString := d;
      FieldByName('VAT_CODE').AsInteger := vc;
      FieldByName('RATE').AsCurrency := r;
    end;
  end;
  inherited DoDetailBeforePost;
end;

procedure TLKDocumentsSalesInvoicesPriced.Notification(AComponent: TComponent;
    Operation: TOperation);
begin
  if (Operation = opRemove) then
  begin
    if (AComponent = FPriceLists) then
      FPriceLists := nil;
  end;
  inherited Notification(AComponent, Operation);
end;

procedure TLKDocumentsSalesInvoicesPriced.SetPriceLists(Value: TLKPriceLists);
begin
  if FPriceLists <> Value then
  begin
    if assigned(Value) then
      Value.FreeNotification(Self);
  FPriceLists := Value;
  end;
end;


initialization


  RegisterClasses([TLKPriceLists, TLKDocumentsSalesInvoicesPriced]);
end.
