{*****************************************************************************}
{                                                                             }
{   This file is part of the LKCRMBase components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form to show a TLKDocumentsSalesInvoicesPriced record                  }
{                                                                             }
{*****************************************************************************}

unit LKSalesInvoicesPricedRecordDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, ImgList, StdActns, Grids, DBGrids, ActnList, DB, StdCtrls,
  DBCtrls, ExtCtrls, ComCtrls, Buttons, ToolWin,
  JvExDBGrids, JvDBGrid, JvDBUltimGrid, LKDBControls, LKCRMBaseComponents,
  LKDocumentsSalesTaxRecordDlg, LKUtils, LKUtilComponents, LKDlgs, LKDefine;

type
  TFormLKSalesInvoicesPricedRecord = class(TFormLKDocumentsSalesTaxRecord)
    ActionViewPriceLists: TAction;
    ActioncbPriceListChange: TAction;
    Pricelists1: TMenuItem;
    sbPriceLists: TToolButton;
    procedure ActionViewPriceListsExecute(Sender: TObject);
    procedure ActioncbPriceListChangeExecute(Sender: TObject);
  private
    { Private declarations }
    FMyAccountsComponent: TLKDocumentsSalesInvoicesPriced;
  protected
    cbPriceList: TDBComboBox;
    procedure RefreshPricesComboItems;
    procedure SetDetailEditsActions; override;
    procedure CustomiseDetailEdits; override;
  public
    { Public declarations }
    procedure Construct(Sender: TLKDocumentsSalesInvoicesPriced);
  end;

var
  FormLKSalesInvoicesPricedRecord: TFormLKSalesInvoicesPricedRecord;

implementation

{$R *.dfm}

procedure TFormLKSalesInvoicesPricedRecord.Construct(
  Sender: TLKDocumentsSalesInvoicesPriced);
begin
  FMyAccountsComponent := Sender;
  inherited Construct(Sender);
end;

procedure TFormLKSalesInvoicesPricedRecord.CustomiseDetailEdits;
var
  i: integer;
begin
  inherited;
  if Assigned(DetailEditList) then
  with DetailEditList do
    for i := 0 to Count - 1 do
      with Items[i] do
      begin
        if (FieldName = 'PRICE_LIST') then
        begin
          ControlType := dbcComboBox;
          cbPriceList := TDBComboBox(EditControl);
          RefreshPricesComboItems;
        end;
        if (FieldName = 'PRICE_CODE') then
        begin
          ControlType := dbcDCLookup;
          with TLKDBDCLookupEdit(EditControl) do
          begin
            LookupDataComponent := FMyAccountsComponent.PriceLists;
            LookupDataset := tbDetail;
          end;
        end;
      end;
end;

procedure TFormLKSalesInvoicesPricedRecord.SetDetailEditsActions;
var
  i: integer;
begin
  inherited;
  if assigned(cbPriceList) then
    cbPriceList.OnChange := ActioncbPriceListChange.OnExecute;
  if Assigned(DetailEditList) then
  with DetailEditList do
    for i := 0 to Count - 1 do
      with Items[i] do
      begin
        if (FieldName = 'PRICE_CODE') then
          OnExit := ActionPostEditDetailDB.OnExecute;
      end;
end;

procedure TFormLKSalesInvoicesPricedRecord.ActionViewPriceListsExecute(
  Sender: TObject);
begin
  inherited;
  FMyAccountsComponent.PriceLists.View(null, true, [mbOK]);
end;

procedure TFormLKSalesInvoicesPricedRecord.RefreshPricesComboItems;
begin
  with FMyAccountsComponent.PriceLists.TblMaster do
  begin
    cbPriceList.Items.Clear;
    if FindFirst then
      repeat
        cbPriceList.Items.Add(FieldByName('PRICE_LIST').AsString);
      until not FindNext;
  end;
end;

procedure TFormLKSalesInvoicesPricedRecord.ActioncbPriceListChangeExecute(
  Sender: TObject);
var
  s: string;
begin
  inherited;
  s := cbPriceList.Field.AsString;
  if (s = '') then
    exit
  else
    with FMyAccountsComponent.PriceLists do
      while not Locate('PRICE_LIST', s, []) do
      begin
        if (View(null, true, [mbOK, mbCancel]) = mrOK) then
          cbPriceList.Field.AsString :=
            TblMaster.FieldByname('PRICE_LIST').AsString
        else
        begin
          cbPriceList.Field.AsString := '';
          Break;
        end;
      end;
end;

end.

