{*****************************************************************************}
{                                                                             }
{   This file is part of the LKGliding components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      LKGliding components                                                   }
{                                                                             }
{*****************************************************************************}

unit LKGlidingComponents;

interface

uses
  Windows, Messages, SysUtils, Classes, ExtCtrls, Graphics, Controls, Forms,
    Dialogs,
  LKDefine, DB, StdCtrls, DBCtrls, DBGrids, ActnList, ImgList, IniFiles,
    ComCtrls, Variants,
  LKBaseComponent, LKRecordsComponents, LKAccountsComponents, LKCRMBaseComponents;

type

  TLKRecordsGlidingLog = class;

  TLKClubMembers = class(TLKRecords)
  private
    FMembersAccounts: TLKLedgerSales;
    procedure SetMembersAccounts(Value: TLKLedgerSales);
  protected
    procedure CreateViewFormRecord; override;
    procedure DoMasterBeforePost; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation);
        override;
  public
    procedure CheckMembersAccount;
  published
    property MembersAccounts: TLKLedgerSales read FMembersAccounts write
        SetMembersAccounts;
  end;

  TLKLedgerLogFlight = class(TLKLedgerBase)
  protected
    procedure DeleteInputDocument; override;
  public
    function EndOfYear(ProgressText: string): TModalResult; override;
  end;

  TLKLedgerLogAircraft = class(TLKLedgerLogFlight)
  end;

  TLKLedgerLogGlider = class(TLKLedgerLogAircraft)
  end;

  TLKLedgerLogTug = class(TLKLedgerLogAircraft)
  end;

  TLKLedgerLogPilot = class(TLKLedgerLogFlight)
  private
    FClubMembers: TLKClubMembers;
    procedure SetClubMembers(Value: TLKClubMembers);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation);
        override;
  published
    property ClubMembers: TLKClubMembers read FClubMembers write SetClubMembers;
  end;

  TLKLedgerLogPilotGlider = class(TLKLedgerLogPilot)
  end;

  TLKLedgerLogPilotTug = class(TLKLedgerLogPilot)
  end;

  TLKDaybookGlidingLog = class(TLKDaybookBase)
  private
    FLogGliderPilots: TLKLedgerLogPilotGlider;
    function GetInputDocuments: TLKRecordsGlidingLog;
    function GetLogGliders: TLKLedgerLogGlider;
    procedure SetInputDocuments(Value: TLKRecordsGlidingLog);
    procedure SetLogGliderPilots(Value: TLKLedgerLogPilotGlider);
    procedure SetLogGliders(Value: TLKLedgerLogGlider);
  protected
    procedure AssignTo(Dest: TPersistent); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation);
        override;
    procedure PbBeforeProcessBatch; override;
    procedure PbCheckBatchItemDetailIsValid; override;
    procedure PbPostDocumentToLedger; override;
    procedure PbPostLogs; virtual;
    procedure PbProcessedComplete; override;
    function PbSetFilter: Boolean; override;
    procedure PbTablesApplyUpdates; override;
    procedure PbTablesCancelUpdates; override;
  public
    property InputDocuments: TLKRecordsGlidingLog read GetInputDocuments write
        SetInputDocuments;
  published
    property LogGliderPilots: TLKLedgerLogPilotGlider read FLogGliderPilots
        write SetLogGliderPilots;
    property LogGliders: TLKLedgerLogGlider read GetLogGliders write
        SetLogGliders;
  end;

  TLKDaybookGlidingLogAerotow = class(TLKDaybookGlidingLog)
  private
    FLogTugPilots: TLKLedgerLogPilotTug;
    FLogTugs: TLKLedgerLogTug;
    procedure SetLogTugPilots(Value: TLKLedgerLogPilotTug);
    procedure SetLogTugs(Value: TLKLedgerLogTug);
  protected
    procedure AssignTo(Dest: TPersistent); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation);
        override;
    procedure PbBeforeProcessBatch; override;
    procedure PbCheckBatchItemDetailIsValid; override;
    procedure PbPostLogs; override;
    procedure PbProcessedComplete; override;
    procedure PbTablesApplyUpdates; override;
    procedure PbTablesCancelUpdates; override;
  published
    property LogTugPilots: TLKLedgerLogPilotTug read FLogTugPilots write
        SetLogTugPilots;
    property LogTugs: TLKLedgerLogTug read FLogTugs write SetLogTugs;
  end;

  TLKDaybookGlidingLogWinch = class(TLKDaybookGlidingLog)
  end;

  TLKRecordsGlidingLog = class(TLKRecordsOrderEntry)
  private
    FClubMembers: TLKClubMembers;
    FGliders: TLKLedgerLogGlider;
    FOnInsertLogItemSalesInvoiceItems: TNotifyEvent;
    FOnJournalAnalysisAdjustments: TNotifyEvent;
    function GetDaybook: TLKDaybookGlidingLog;
    procedure InsertLogItemSalesInvoiceItems;
    procedure SetClubMembers(Value: TLKClubMembers);
    procedure SetDaybook(Value: TLKDaybookGlidingLog);
    procedure SetPricingTypeItems(const Value: TStrings);
  protected
    FPricingTypeItems: TStrings;
    procedure CreateViewFormRecord; override;
    procedure DoDetailAfterInsert; override;
    procedure DoDetailBeforePost; override;
    procedure DoMasterBeforePost; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation);
        override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure CheckInvoiced(const LogRef: string; LogItem: Integer);
    procedure DlgNewInvoiceWizardExecute; override;
    property Daybook: TLKDaybookGlidingLog read GetDaybook write SetDaybook;
    property Gliders: TLKLedgerLogGlider read FGliders write FGliders;
  published
    property ClubMembers: TLKClubMembers read FClubMembers write SetClubMembers;
    property PricingTypeItems: TStrings read FPricingTypeItems write
        SetPricingTypeItems;
    property OnInsertLogItemSalesInvoiceItems: TNotifyEvent read
        FOnInsertLogItemSalesInvoiceItems write
        FOnInsertLogItemSalesInvoiceItems;
    property OnJournalAnalysisAdjustments: TNotifyEvent read
        FOnJournalAnalysisAdjustments write FOnJournalAnalysisAdjustments;
  end;

  TLKRecordsGlidingLogAerotow = class(TLKRecordsGlidingLog)
  private
    FTugCurrent: string;
    FTugPilotCurrent: string;
    FTugs: TLKLedgerLogTug;
    function GetDaybook: TLKDaybookGlidingLogAerotow;
    procedure SetDaybook(Value: TLKDaybookGlidingLogAerotow);
  protected
    procedure CreateViewFormRecord; override;
    procedure DoDetailAfterInsert; override;
    procedure DoDetailAfterPost; override;
    procedure DoDetailBeforePost; override;
    procedure DoMasterBeforePost; override;
    property TugCurrent: string read FTugCurrent write FTugCurrent;
    property TugPilotCurrent: string read FTugPilotCurrent write
        FTugPilotCurrent;
  public
    constructor Create(AOwner: TComponent); override;
    procedure CreateViewForm; override;
    procedure DlgImportExportExecute(AOwner: TComponent; const AsExportDlg,
        Autoexecute: boolean); override;
    property Tugs: TLKLedgerLogTug read FTugs write FTugs;
  published
    property Daybook: TLKDaybookGlidingLogAerotow read GetDaybook write
        SetDaybook;
  end;

  TLKRecordsGlidingLogWinch = class(TLKRecordsGlidingLog)
  private
    function GetDaybook: TLKDaybookGlidingLogWinch;
    procedure SetDaybook(Value: TLKDaybookGlidingLogWinch);
  published
    property Daybook: TLKDaybookGlidingLogWinch read GetDaybook write
        SetDaybook;
  end;

procedure Register;

implementation

uses LKUtils, LKUtilsDB, LKdlgs, LKMembersRecordDlg,
//LKDaybookGlidingLogDlg,
  LKGlidingLogInputRecordDlg, LKGlidingLogAerotowInputRecordDlg,
  LKGlidingLogAerotowInputDlg;

procedure Register;
begin
end;


{
******************************** TLKClubMembers ********************************
}
procedure TLKClubMembers.CheckMembersAccount;
var
  CustomerRef, n, a1, a2, a3, a4, p, t, e: string;
begin
  with TblMaster do
  begin
    if State in [dsInsert, dsEdit] then
      Post;
    CustomerRef := FieldByName('MEMBER_REFERENCE').AsString;
    if MembersAccounts.AccountIsValid(CustomerRef, False) then
      exit;
    n := FieldByName('DISPLAY_NAME').AsString;
    a1 := FieldByName('ADDRESS1').AsString;
    a2 := FieldByName('ADDRESS2').AsString;
    a3 := FieldByName('ADDRESS3').AsString;
    a4 := FieldByName('ADDRESS4').AsString;
    p := FieldByName('POSTCODE').AsString;
    t := FieldByName('TELEPHONE').AsString;
    e := FieldByName('EMAIL').AsString;
  end;
  with MembersAccounts do
  begin
    InsertRecord([CustomerRef, n, a1, a2, a3, a4, p, n, t, nil, e]);
    ApplyUpdates(0);
  end;
end;

procedure TLKClubMembers.CreateViewFormRecord;
begin
  Application.CreateForm(TFormLKMembersRecord, FFormViewRecord);
  TFormLKMembersRecord(FormViewRecord).Construct(Self);
end;

procedure TLKClubMembers.DoMasterBeforePost;
begin
  with TblMaster do
    FieldByName('DISPLAY_NAME').AsString :=
      FieldByName('FORENAME').AsString + ' ' + FieldByName('SURNAME').AsString;
  inherited doMasterBeforePost;
end;

procedure TLKClubMembers.Notification(AComponent: TComponent; Operation:
    TOperation);
begin
  if (Operation = opRemove) then
  begin
    if (AComponent = FMembersAccounts) then
      FMembersAccounts := nil;
  end;
  inherited Notification(AComponent, Operation);
end;

procedure TLKClubMembers.SetMembersAccounts(Value: TLKLedgerSales);
begin
  FMembersAccounts := Value;
end;


{
****************************** TLKLedgerLogFlight ******************************
}
procedure TLKLedgerLogFlight.DeleteInputDocument;
var
  DbkID: string;
  DocRef: string;
  D: TLKDaybookBase;
begin
  with TblDetail do
  begin
    DbkID := 'LOG';
    DocRef := FieldByName('DOCUMENT_NUMBER').asString;
    D := Daybooks.ComponentByID(DbkID);
    if Assigned(D) then
      if D.Locate(TblMaster.Fields[0].FieldName, DocRef, []) then
        Delete;
  end;
end;

function TLKLedgerLogFlight.EndOfYear(ProgressText: string): TModalResult;
var
  I: Integer;
  MonthsToRetain: Integer;
begin
  Result := inherited EndOfYear(ProgressText);
  if (Result <> mrOK) then
    Exit;
  // get the number of months transactions to retain
  MonthsToRetain := RecallInteger('EndOfYear', 'MonthsToRetain', 24, True);
  if (MonthsToRetain < 12) or (MonthsToRetain > 240) then
    MonthsToRetain := 24;
  with TLKViewDialog.Create(Owner) do
  try
    DataType := vInteger;
    Title := nameFriendly + ' Year End';
    HeaderCaption := 'Delete audit trail for completed items in the ' + nameFriendly;
    HeaderComments := 'Enter the number of months prior to which matched'
      + ' documents and transactions will be removed.';
    Value := MonthsToRetain;
    Result := mrRetry;
    repeat
      if Execute then
      begin
        if (MonthsToRetain < 12) or (MonthsToRetain > 240) then
          ShowMessage('Months should be between 12 and 240')
        else
        begin
          Result := mrOK;
          if (Value <> MonthsToRetain) then
          begin
            RememberInteger('EndOfYear', 'MonthsToRetain', Value, True);
            MonthsToRetain := Value;
          end;
        end;
      end
      else
      begin
        Result := mrCancel;
        Showmessage('Old transactions for ' + nameFriendly + ' retained.');
      end;
    until not (Result = mrRetry);
  finally Free;
  end;
  if (Result <> mrOK) then
    exit;
  // ensure all transactions and documents are visible
  with Daybooks do
    for I := 0 to Count - 1 do
      with Items[I] do
      begin
        Close;
        TblMaster.WhereClause := '';
        if not ACGroup.Active then
          ACGroup.Open;
        Open;
        with InputDocuments do
        begin
          Filter := '';
          Filtered := False;
          TblDetail.Filter := '';
          TblDetail.Filtered := False;
        end;
        Filter := '';
        Filtered := False;
        TblDetail.Filter := '';
        TblDetail.Filtered := False;
      end;
  ProgressText := 'Clearing old transactions from ' + NameFriendly;
  Application.ProcessMessages;
  Filter := '';
  Filtered := False;
  // Delete the transactions and documents
  TblDetail.Filter := EndOfYearFilter(MonthsToRetain);
  TblDetail.Filtered := True;
  OKToEdit := True;
  try
    if FindFirst then
      repeat
        while TblDetail.FindFirst do
        begin
          DeleteInputDocument;
          TblDetail.Delete;
        end;
      until not FindNext;
    // Apply the updates
    with Daybooks do
      for I := 0 to Count - 1 do
        with Items[I] do
        begin
          ApplyUpdates(0);
          InputDocuments.ApplyUpdates(0);
        end;
    ApplyUpdates(0);
  finally
    OKToEdit := False;
    TblDetail.Filter := '';
    TblDetail.Filtered := False;
  end;
  ProgressText := 'Old transactions removed okay from ' + NameFriendly;
  Application.ProcessMessages;
end;


{
****************************** TLKLedgerLogPilot *******************************
}
procedure TLKLedgerLogPilot.Notification(AComponent: TComponent; Operation:
    TOperation);
begin
  if (Operation = opRemove) then
  begin
    if (AComponent = FClubMembers) then
      FClubMembers := nil;
  end;
  inherited Notification(AComponent, Operation);
end;

procedure TLKLedgerLogPilot.SetClubMembers(Value: TLKClubMembers);
begin
  if FClubMembers <> Value then
  begin
    if Assigned(Value) then
      Value.FreeNotification(Self);
    FClubMembers := Value;
  end;
end;


{
***************************** TLKDaybookGlidingLog *****************************
}
procedure TLKDaybookGlidingLog.AssignTo(Dest: TPersistent);
var
  DestDB: TLKDaybookGlidingLog;
begin
  inherited AssignTo(Dest);
  DestDB := TLKDaybookGlidingLog(Dest);
  DestDB.LogGliders := LogGliders;
  DestDB.LogGliderPilots := LogGliderPilots;
end;

function TLKDaybookGlidingLog.GetInputDocuments: TLKRecordsGlidingLog;
begin
  Result := TLKRecordsGlidingLog(inherited InputDocuments);
end;

function TLKDaybookGlidingLog.GetLogGliders: TLKLedgerLogGlider;
begin
  Result := TLKLedgerLogGlider(inherited Ledger);
end;

procedure TLKDaybookGlidingLog.Notification(AComponent: TComponent; Operation:
    TOperation);
begin
  if (Operation = opRemove) then
  begin
    if (AComponent = FLogGliderPilots) then
      FLogGliderPilots := nil;
  end;
  inherited Notification(AComponent, Operation);
end;

procedure TLKDaybookGlidingLog.PbBeforeProcessBatch;
begin
  inherited PbBeforeProcessBatch;
  if Assigned(LogGliderPilots) then
    LogGliderPilots.TblDetail.Filtered := False;
  if Assigned(LogGliders) then
    LogGliders.TblDetail.Filtered := False;
end;

procedure TLKDaybookGlidingLog.PbCheckBatchItemDetailIsValid;
var
  I: Integer;
  DocRef: string;
  Item: string;
  S: string;
begin
  inherited PbCheckBatchItemDetailIsValid;
  with TblDetail, DlgDaybookErrors do
  begin
    DocRef := Fields[0].AsString;
    Item := Fields[1].AsString;
    S := FieldByName('CUSTOMER_ACCOUNT').AsString;
    if (S = '') then
    begin
      for i := 3 to FieldCount - 1 do
        if not Fields[i].IsNull then
        begin
          AddWarning(['201', DocRef, Item, 'Customer code is blank']);
          Exit;
        end;
      AddError(['200', DocRef, Item, 'Blank Item', 'Delete']);
      Exit;
    end
    else if not LogGliderPilots.AccountIsValid(S, False) then
      AddError(['201', DocRef, Item, 'Incorrect customer ' + S,
        'Display & correct']);
    S := FieldByName('INSTRUCTOR').AsString;
    if (S > ' ') and not LogGliderPilots.AccountIsValid(S, False) then
      AddError(['201', DocRef, Item, 'Incorrect instructor ' + S,
        'Display & correct']);
    S := FieldByName('GLIDER_CALLSIGN').AsString;
    if (S > ' ') then
      if not LogGliders.AccountIsValid(S, False) then
        AddError(['201', DocRef, Item, 'Incorrect glider ' + S,
          'Display & correct']);
  end;
end;

procedure TLKDaybookGlidingLog.PbPostDocumentToLedger;
begin
  with TblDetail do
    if FindFirst then
      repeat
        InputDocuments.CheckInvoiced(TblMDocumentReference, TblDDocumentItemNumber);
        PbPostLogs;
        Dec(LedgerTempURN)
      until not FindNext;
end;

procedure TLKDaybookGlidingLog.PbPostLogs;
var
  I: Integer;
  D: TDateTime;
  Launch: TDateTime;
  Land: TDateTime;
  Duration: TDateTime;
  DocRef: string;
  ImgL: string;
  Pilot: string;
  PilotName: string;
  Inst: string;
  InstName: string;
  Glider: string;
  Notes: string;
  Crew: string;
  H: Integer;
  Kms: Integer;
begin
  D := TblMDocumentDate;
  DocRef := TblMDocumentReference;
  I := TblDDocumentItemNumber;
  ImgL := TblMaster.FieldByName('LOG_IMAGE').AsString;
  with TblDetail do
  begin
    Pilot := Trim(FieldByName('CUSTOMER_ACCOUNT').AsString);
    PilotName := FieldByName('PILOT_NAME').AsString;
    Inst := Trim(FieldByName('INSTRUCTOR').AsString);
    InstName := FieldByName('INSTRUCTOR_NAME').AsString;
    Glider := Trim(FieldByName('GLIDER_CALLSIGN').AsString);
    Launch := GetDate(FieldByName('LAUNCH'));
    Land := GetDate(FieldByName('LAND'));
    Duration := GetDate(FieldByName('DURATION'));
    H := GetInteger(FieldByName('RELEASE_HEIGHT'));
    Kms := GetInteger(FieldByName('KMS_FLOWN'));
    Notes := FieldByName('NOTES').AsString;
  end;
  if (Glider > '') then
    LogGliders.AccountPost([LedgerTempURN, Glider, D, DocRef, I, Pilot,
      PilotName, Inst, Launch, Land, Duration, H, Kms, Notes, ImgL]);
  if (Inst > '') then
    Crew := 'DUAL'
  else
    Crew := 'SOLO';
  if (Pilot > '') then
    LogGliderPilots.AccountPost([LedgerTempURN, Pilot, D, DocRef, I, PilotName,
      Glider, Crew, Inst, InstName, Launch, Land, Duration, H, Kms, Notes, ImgL]);
  if (Inst > '') and (Inst <> Pilot) then
  begin
    Dec(LedgerTempURN);
    Crew := 'INST';
    LogGliderPilots.AccountPost([LedgerTempURN, Inst, D, DocRef, I, InstName,
      Glider, Crew, Pilot, PilotName, Launch, Land, Duration, H, Kms, Notes, ImgL]);
  end;
end;

procedure TLKDaybookGlidingLog.PbProcessedComplete;
begin
  FLogGliderPilots.Refresh;
  inherited PbProcessedComplete;
end;

function TLKDaybookGlidingLog.PbSetFilter: Boolean;
begin
  with TLKViewDialog.Create(Owner) do
  begin
    try
      Title := 'Post Logs to (date)';
      DataType := vDate;
      Value := BatchDate;
      Result := Execute;
      if Result then
        BatchDate := Value;
    finally free;
    end;
  end;
  if not Result then
    Exit;
  Filter := 'BATCH = NULL AND DATE_ <= ' + Chr(39) + DateToStr(BatchDate) +
    Chr(39);
  Filtered := True;
end;

procedure TLKDaybookGlidingLog.PbTablesApplyUpdates;
begin
  inherited PbTablesApplyUpdates;
  FLogGliderPilots.ApplyUpdates(0);
end;

procedure TLKDaybookGlidingLog.PbTablesCancelUpdates;
begin
  FLogGliderPilots.CancelUpdates;
  inherited PbTablesCancelUpdates;
end;

procedure TLKDaybookGlidingLog.SetInputDocuments(Value: TLKRecordsGlidingLog);
begin
  inherited InputDocuments := Value;
end;

procedure TLKDaybookGlidingLog.SetLogGliderPilots(Value:
    TLKLedgerLogPilotGlider);
begin
  if (FLogGliderPilots = Value) then
    Exit;
  if Assigned(FLogGliderPilots) then
  begin
    FLogGliderPilots.RemoveFreeNotification(Self);
    FLogGliderPilots.DaybookRemove(Self);
  end;
  FLogGliderPilots := Value;
  if Assigned(Value) then
  begin
    Value.FreeNotification(Self);
    Value.DaybookAdd(Self);
  end;
end;

procedure TLKDaybookGlidingLog.SetLogGliders(Value: TLKLedgerLogGlider);
begin
  inherited Ledger := Value;
end;


{
************************* TLKDaybookGlidingLogAerotow **************************
}
procedure TLKDaybookGlidingLogAerotow.AssignTo(Dest: TPersistent);
var
  DestDB: TLKDaybookGlidingLogAerotow;
begin
  inherited AssignTo(Dest);
  DestDB := TLKDaybookGlidingLogAerotow(Dest);
  DestDB.LogTugs := LogTugs;
  DestDB.LogTugPilots := LogTugPilots;
end;

procedure TLKDaybookGlidingLogAerotow.Notification(AComponent: TComponent;
    Operation: TOperation);
begin
  if (Operation = opRemove) then
  begin
    if (AComponent = FLogTugPilots) then
      FLogTugPilots := nil;
    if (AComponent = FLogTugs) then
      FLogTugs := nil;
  end;
  inherited Notification(AComponent, Operation);
end;

procedure TLKDaybookGlidingLogAerotow.PbBeforeProcessBatch;
begin
  inherited PbBeforeProcessBatch;
  if Assigned(LogTugPilots) then
    LogTugPilots.TblDetail.Filtered := False;
  if Assigned(LogTugs) then
    LogTugs.TblDetail.Filtered := False;
end;

procedure TLKDaybookGlidingLogAerotow.PbCheckBatchItemDetailIsValid;
var
  DocRef: string;
  Item: string;
  S: string;
begin
  inherited PbCheckBatchItemDetailIsValid;
  with TblDetail, DlgDaybookErrors do
  begin
    DocRef := Fields[0].asString;
    Item := Fields[1].asString;
    S := FieldByName('TUG_PILOT').AsString;
    if (S < 'A') then
      AddWarning(['N', DocRef, Item, 'No tug pilot entered'])
    else if not LogTugPilots.AccountIsValid(S, False) then
      AddError(['201', DocRef, Item, 'Incorrect tug pilot ' + S,
        'Display & correct']);
    S := FieldByName('TUG').AsString;
    if (S < 'A') then
      AddWarning(['N', DocRef, Item, 'No tug '])
    else if not LogTugs.AccountIsValid(S, False) then
      AddError(['201', DocRef, Item, 'Incorrect tug ' + S,
        'Display & correct']);
  end;
end;

procedure TLKDaybookGlidingLogAerotow.PbPostLogs;
var
  DocRef: string;
  D: TDateTime;
  I: Integer;
  ImgT: string;
  Glider: string;
  Tug: string;
  Launch: TDateTime;
  H: Integer;
  Tuggie: string;
  TugTime: TDateTime;
begin
  inherited pbPostLogs;
  D := TblMDocumentDate;
  DocRef := TblMDocumentReference;
  I := TblDDocumentItemNumber;
  ImgT := TblMaster.FieldByName('TUG_LOG_IMAGE').AsString;
  with TblDetail do
  begin
    Glider := Trim(FieldByName('GLIDER_CALLSIGN').AsString);
    Tug := Trim(FieldByName('Tug').AsString);
    Launch := GetDate(FieldByName('Launch'));
    H := GetInteger(FieldByName('RELEASE_HEIGHT'));
    Tuggie := Trim(FieldByName('TUG_PILOT').AsString);
    TugTime := GetDate(FieldByName('TUG_TIME'));
  end;
  if (Tug > '') then
  begin
    LogTugs.AccountPost([LedgerTempURN, Tug, D, DocRef, I, Glider, Launch,
      TugTime, H, Tuggie, ImgT]);
    if (Tuggie > '') then
      LogTugPilots.AccountPost([LedgerTempURN, Tuggie, D, DocRef, I, Tug,
        Launch, TugTime, H, ImgT]);
  end;
end;

procedure TLKDaybookGlidingLogAerotow.PbProcessedComplete;
begin
  FLogTugs.Refresh;
  FLogTugPilots.Refresh;
  inherited PbProcessedComplete;
end;

procedure TLKDaybookGlidingLogAerotow.PbTablesApplyUpdates;
begin
  inherited PbTablesApplyUpdates;
  FLogTugs.ApplyUpdates(0);
  Refresh;
  FLogTugPilots.ApplyUpdates(0);
  Refresh;
end;

procedure TLKDaybookGlidingLogAerotow.PbTablesCancelUpdates;
begin
  FLogTugs.CancelUpdates;
  FLogTugPilots.CancelUpdates;
  inherited PbTablesCancelUpdates;
end;

procedure TLKDaybookGlidingLogAerotow.SetLogTugPilots(Value:
    TLKLedgerLogPilotTug);
begin
  if (FLogTugPilots = Value) then
    Exit;
  if Assigned(FLogTugPilots) then
  begin
    FLogTugPilots.RemoveFreeNotification(Self);
    FLogTugPilots.DaybookRemove(Self);
  end;
  FLogTugPilots := Value;
  if Assigned(Value) then
  begin
    Value.FreeNotification(Self);
    Value.DaybookAdd(Self);
  end;
end;

procedure TLKDaybookGlidingLogAerotow.SetLogTugs(Value: TLKLedgerLogTug);
begin
  if (FLogTugs = Value) then
    Exit;
  if Assigned(FLogTugs) then
  begin
    FLogTugs.RemoveFreeNotification(Self);
    FLogTugs.DaybookRemove(Self);
  end;
  FLogTugs := Value;
  if Assigned(Value) then
  begin
    Value.FreeNotification(Self);
    Value.DaybookAdd(Self);
  end;
end;


{
***************************** TLKRecordsGlidingLog *****************************
}
constructor TLKRecordsGlidingLog.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  DocumentRefLength := 10;
  FPricingTypeItems := TStringList.Create;
end;

destructor TLKRecordsGlidingLog.Destroy;
begin
  FPricingTypeItems.Free;
  inherited Destroy;
end;

procedure TLKRecordsGlidingLog.CheckInvoiced(const LogRef: string; LogItem:
    Integer);
var
  CustomerRef: string;
  D: TDateTime;
begin
  if not TblMaster.Locate('LOG_REFERENCE', LogRef, []) then
    Exit;
  with TblDetail do
  begin
    if Locate('ITEM_NUMBER', LogItem, []) then
      CustomerRef := FieldByName('CUSTOMER_ACCOUNT').AsString
    else
      Exit;
    if (FieldByName('PRICING').AsString = 'No charge') then
      Exit;
  end;
  D := tmDocumentDate;
  if not SalesInvoices.Active then
    SalesInvoices.ACGroup.Open;
  with SalesInvoices do
    if MasterComponent.TblMaster.Locate('CUSTOMER_REFERENCE', CustomerRef, [])
      then
      if TblMaster.Locate('DATE_;LOG_ITEM', VarArrayOf([D, LogItem]), []) then
        Exit;
  DlgNewInvoiceWizardExecute;
end;

procedure TLKRecordsGlidingLog.CreateViewFormRecord;
begin
  Application.CreateForm(TFormLKGlidingLogInputRecord, FFormViewRecord);
  TFormLKGlidingLogInputRecord(FormViewRecord).Construct(Self);
end;

procedure TLKRecordsGlidingLog.DlgNewInvoiceWizardExecute;
var
  CustomerRef, CustomerName: string;
  LogItem: Integer;
  LogDate: TDateTime;
  Desc: string;

  label
    masterWizAgain, detailWizAgain;

  procedure AddToDesc(const S: string);
  begin
    if (S > '') then
      Desc := Desc + ' ' + S;
  end;

begin
  inherited DlgNewInvoiceWizardExecute;
  LogDate := tmDocumentDate;
  with TblDetail do
  begin
    if State in [dsInsert, dsEdit] then
      Post;
    Desc := 'Flight';
    LogItem := GetInteger(FieldByName('ITEM_NUMBER'));
    CustomerRef := FieldByName('CUSTOMER_ACCOUNT').AsString;
    AddToDesc(FieldByName('GLIDER_CALLSIGN').AsString);
    AddToDesc(FieldByName('GLIDER_TYPE').AsString);
    AddToDesc(FieldByName('PILOT_NAME').AsString);
    AddToDesc(FieldByName('VOUCHER_NO').AsString);
    AddToDesc(FieldByName('NOTES').AsString);
    if (CustomerRef = '') then
    begin
      if Assigned(FormViewRecord) then
        if not FormViewRecord.Visible then
          ViewRecord(VarArrayOf([LogDate, LogItem]), False, [])
        else
          ViewRecord(VarArrayOf([LogDate, LogItem]), False, []);
      with TLKViewDialog.Create(Self.Owner) do
      begin
        try
          Title := 'Enter customer account number';
          DataType := vString;
          if Execute then
            CustomerRef := Value;
        finally Free;
        end;
      end;
      if (CustomerRef = '') then
        exit;
    end;
    while not ClubMembers.Locate('MEMBER_REFERENCE', CustomerRef, []) do
    begin
      Showmessage('Member ' + CustomerRef + ' not found');
      if not (ClubMembers.View(null, True, [mbOK, mbAbort]) = mrOK) then
        Exit;
      CustomerRef :=
        ClubMembers.TblMaster.FieldByName('MEMBER_REFERENCE').AsString;
      Edit;
      FieldByName('CUSTOMER_ACCOUNT').AsString := CustomerRef;
      Post;
    end;
    ClubMembers.CheckMembersAccount;
    CustomerName := ClubMembers.TblMaster.FieldByName('DISPLAY_NAME').AsString;
  end;
  with SalesInvoices do
  begin
    if not TblMaster.Locate('DATE_;LOG_ITEM', VarArrayOf([LogDate, LogItem]), [])
      then
      begin
        Insert;
        Post;
        TblMaster.Edit;
        TblMaster.SetFields([nil, nil, '', CustomerRef, CustomerName, LogDate,
          LogItem, LogDate, Desc]);
        Post;
      end;
    if not TblDetail.Locate('LOG_ITEM', LogItem, []) then
      InsertLogItemSalesInvoiceItems;
    ViewRecord(null, True, [mbOK]);
  end;
  with TblDetail do
  begin
    Edit;
    FieldByName('INVOICED').AsInteger := 1;
    Post;
  end;
  ApplyUpdates(0);
end;

procedure TLKRecordsGlidingLog.DoDetailAfterInsert;
begin
  with TblDetail do
  begin
    FieldByName('PRICING').AsString := 'Club rates';
    FieldByName('INVOICED').AsInteger := 0;
  end;
  inherited doDetailAfterInsert;
end;

procedure TLKRecordsGlidingLog.DoDetailBeforePost;
var
  T: TDateTime;
begin
  with TblDetail do
  begin
    if not (FieldByName('LAUNCH').IsNull or FieldByName('LAND').isNull) then
    begin
      T := FieldByName('LAND').AsDateTime - FieldByName('LAUNCH').AsDateTime;
      if (T > 0.4) then
        Showmessage('WARNING - long flight!');
      if (T < 0) then
        Showmessage('WARNING - Landed before launch!');
      FieldByName('DURATION').AsDateTime := T;
    end;
  end;
  inherited doDetailBeforePost;
end;

procedure TLKRecordsGlidingLog.DoMasterBeforePost;
var
  d: TDateTime;
begin
  d := tmDocumentDate;
  if (d > 0) then
    FieldByName('DAY_NAME').AsString := WeekDayName(d);
  inherited doMasterBeforePost;
end;

function TLKRecordsGlidingLog.GetDaybook: TLKDaybookGlidingLog;
begin
  Result := TLKDaybookGlidingLog(inherited Daybook);
end;

procedure TLKRecordsGlidingLog.InsertLogItemSalesInvoiceItems;
begin
  if Assigned(OnInsertLogItemSalesInvoiceItems) then
    OnInsertLogItemSalesInvoiceItems(Self);
end;

procedure TLKRecordsGlidingLog.Notification(AComponent: TComponent; Operation:
    TOperation);
begin
  if (Operation = opRemove) then
  begin
    if (AComponent = FClubMembers) then
      FClubMembers := nil;
  end;
  inherited Notification(AComponent, Operation);
end;

procedure TLKRecordsGlidingLog.SetClubMembers(Value: TLKClubMembers);
begin
  if FClubMembers <> Value then
  begin
    if Assigned(Value) then
      Value.FreeNotification(Self);
    FClubMembers := Value;
  end;
end;

procedure TLKRecordsGlidingLog.SetDaybook(Value: TLKDaybookGlidingLog);
begin
  if assigned(Daybook) then
    Daybook.InputDocuments := nil;
  inherited Daybook := Value;
  if assigned(Value) {and not (csLoading in ComponentState) } then
  begin
    Value.InputDocuments := self;
    Gliders := Value.LogGliders;
  end;
end;

procedure TLKRecordsGlidingLog.SetPricingTypeItems(const Value: TStrings);
begin
  FPricingTypeItems.Assign(Value);
end;


{
************************* TLKRecordsGlidingLogAerotow **************************
}
constructor TLKRecordsGlidingLogAerotow.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FTugCurrent := '';
  FTugPilotCurrent := '';
end;

procedure TLKRecordsGlidingLogAerotow.CreateViewForm;
begin
  Application.CreateForm(TFormLKGlidingLogAerotowInput, FFormView);
  TFormLKGlidingLogAerotowInput(FormView).Construct(Self);
end;

procedure TLKRecordsGlidingLogAerotow.CreateViewFormRecord;
begin
  Application.CreateForm(TFormLKGlidingLogAerotowInputRecord, FFormViewRecord);
  TFormLKGlidingLogAerotowInputRecord(FormViewRecord).Construct(Self);
end;

procedure TLKRecordsGlidingLogAerotow.DlgImportExportExecute(AOwner: TComponent;
    const AsExportDlg, Autoexecute: boolean);
var
  Okay: Boolean;
  S: string;
  CSVFileName: string;
  CSVFile: TextFile;
  Fs: TFormatSettings;
  RecordType: string;
  Ref: string;
  D1, T1, T2: TDateTime;
  S1,S2,S3,S4,S5,S6,S7,S8,S9: string;
  F1, F2, F3: Double;
  I1, I2, I3: Integer;

  function GetNextField: string;
  var
    I: Integer;
  begin
    if (Pos('"', S) = 1) then
    begin
      Result := Copy(S, 2, Pos('",', S) - 2);
      System.Delete(S, 1, Length(Result) + 3);
    end
    else
    begin
      I := Pos(',', S);
      if I = 0 then
        Result := S
      else
        Result := Copy(S, 1, I - 1);
      System.Delete(S, 1, Length(Result) + 1);
    end;
    Result := Trim(Result);
  end;

  function GetNextFieldAsTime: Double;
  begin
//     Result :=  Copy(GetNextField, 1, 5);
       if (GetNextField > '') then
         Result := 0.0
       else
         Result := 1.0;
  end;

  function GetNextFieldAsDate: string;
  begin
//     Result :=  Copy(GetNextField, 1, 5);
       Result := GetNextField;
  end;

begin
  if AsExportDlg then
    inherited
  else
  begin
    with TOpenDialog.Create(Application) do
    try
      Title := 'Select Import file';
      DefaultExt := '.csv';
      Filter := 'Comma Separated Variable (CSV)|*.csv';
      InitialDir := InstallFolder;
      Options := Options + [ofFileMustExist];
      Okay := Execute;
      CSVFileName := FileName;
    finally Free;
    end;
    if not Okay then
      Exit;
    Fs.Create;
    Fs.DateSeparator := '-';
    Fs.TimeSeparator := ':';
    Fs.ShortDateFormat := 'y/m/d';
    Fs.ShortTimeFormat := 'hh:mm:ss';
    AssignFile(CSVFile, CSVFileName);
    Reset(CSVFile);
    Screen.Cursor := crHourglass;
    RefreshMSumDPairs := False;
    OKToInsert := True;
    ExternalDisableControls;
    try
      repeat
        ReadLn(CSVFile, S);
        RecordType := GetNextField;
        if RecordType = 'm' then
        begin
          GetNextField; //id
          D1 := StrToDate(GetNextField, Fs); // Flight_date
          S1 := GetNextField; // Day_name
          S2 := GetNextField; // Notes
          S3 := GetNextField; // Log_image
          S4 := GetNextField; // Tug_log_image
          F1 := StrToFloat(GetNextField); //  Start_tacho
          F2 := StrToFloat(GetNextField); //  Stop_tacho
          F3 := StrToFloat(GetNextField); //  Tach_time
          T1 := StrToTime(GetNextField, Fs); //  Tug_time
          I1 := StrToInt(GetNextField); // Height
          I2 := StrToInt(GetNextField); // Launches
          FTugPilotCurrent := GetNextField; //  Tug_pilot
          T2 := StrToTime(GetNextField, Fs); // Soaring_time
          I3 := StrToInt(GetNextField); // Kms_flown
          Ref := MakeDocumentReference(DocumentRefPrefix, SequenceNumberFillChar,
            IntToStr(SequenceNumberNext), DocumentRefLength);
          TblMaster.InsertRecord(
            [ Ref,
              D1,    //  StrToDate(GetNextFieldAsDate, Fs), // Flight_date
              S1,    //  GetNextField, // Day_name
              S2,    //  GetNextField, // Notes
              S3,    //  GetNextField, // Log_image
              S4,    //  GetNextField, // Tug_log_image
              F1,    //  StrToFloat(GetNextField), //  Start_tacho
              F2,    //  StrToFloat(GetNextField), //  Stop_tacho
              F3,    //  StrToFloat(GetNextField), //  Tach_time
              T1,    //  StrToTime(GetNextFieldAsTime, Fs), //  Tug_time
              I1,    //  StrToInt(GetNextField), // Height
              I2,    //  StrToInt(GetNextField), // Launches
              FTugPilotCurrent,    //  GetNextField, //  Tug_pilot
              T2,    //  StrToTime(GetNextField, Fs), // Soaring_time
              I3    //  StrToInt(GetNextField) // Kms_flown
            ]);
        end else
        if RecordType = 'd' then
        begin
          GetNextField; //  id
          GetNextField; //  flight_logs_id
          GetNextField; //  Flight_date
          I1 := StrToInt(GetNextField); // Log_item
          S1 := GetNextField; // Member_reference
          S2 := GetNextField; // Pilot_name
          S3 := GetNextField; // Pricing
          S4 := GetNextField; // Instructor
          S5 := GetNextField; // Instructor_name
          S6 := GetNextField; // Glider_callsign
          S7 := GetNextField; // Glider_type
          S8 := GetNextField; // Tug
          T1 := StrToTime(GetNextField, Fs); // Launch
          T2 := StrToTime(GetNextField, Fs); // Land
          D1 := StrToTime(GetNextField, Fs); // Duration
          I2 := StrToInt(GetNextField); //  Release_height
          I3 := StrToInt(GetNextField); //  Kms_flown
          S9 := GetNextField; // Notes
          try
            TblDetail.InsertRecord(
              [ Ref,
                I1,  // Log_item
                S1,  // Member_reference
                S2,  // Pilot_name
                S3,  // Pricing
                S4,  // Instructor
                S5,  // Instructor_name
                S6,  // Glider_callsign
                S7,  // Glider_type
                S8,  // Tug
                T1,  // Launch
                T2,  // Land
                D1,  // Duration
                I2,  //  Release_height
                FTugPilotCurrent, '', StrToTime('00:09:00', Fs) , '',
                I3,  //  Kms_flown
                S9  // Notes
              ]);
            except on E:Exception do
              Showmessage(E.Message);
            end;
        end;
      until Eof(CSVFile);
      ApplyUpdates(0);
    finally
      CloseFile(CSVFile);
      Screen.Cursor := crDefault;
      RefreshMSumDPairs := True;
      OKToInsert := False;
      ExternalEnableControls;
    end;
  end;
end;

procedure TLKRecordsGlidingLogAerotow.DoDetailAfterInsert;
begin
  with TblDetail do
  begin
    FieldByName('TUG').AsString := TugCurrent;
    FieldByName('TUG_PILOT').AsString := TugPilotCurrent;
  end;
  inherited doDetailAfterInsert;
end;

procedure TLKRecordsGlidingLogAerotow.DoDetailAfterPost;
begin
  with TblDetail do
  begin
    if not FieldByName('TUG').IsNull then
      FTugCurrent := FieldByName('TUG').AsString;
    if not FieldByName('TUG_PILOT').IsNull then
      FTugPilotCurrent := FieldByName('TUG_PILOT').AsString;
  end;
  inherited doDetailAfterPost;
end;

procedure TLKRecordsGlidingLogAerotow.DoDetailBeforePost;
var
  H: Integer;
begin
  with TblDetail do
  begin
    if not (FieldByName('RELEASE_HEIGHT').IsNull) then
    begin
      H := FieldByName('RELEASE_HEIGHT').AsInteger;
      if (H > 5000) then
        Showmessage('QUERY - Tow height over 5,000?');
      if (H < 700) then
        Showmessage('WARNING - Tow height under 700?');
    end;
  end;
  inherited DoDetailBeforePost;
end;

procedure TLKRecordsGlidingLogAerotow.DoMasterBeforePost;
begin
  if not FieldByName('TUG_PILOT').IsNull then
    FTugPilotCurrent := FieldByName('TUG_PILOT').AsString;
  FieldByName('TACH_TIME').AsFloat :=
    FieldByName('CLOSE_TACHO').AsFloat - FieldByName('OPEN_TACHO').AsFloat;
  inherited doMasterBeforePost;
end;

function TLKRecordsGlidingLogAerotow.GetDaybook: TLKDaybookGlidingLogAerotow;
begin
  Result := TLKDaybookGlidingLogAerotow(inherited Daybook);
end;

procedure TLKRecordsGlidingLogAerotow.SetDaybook(Value:
    TLKDaybookGlidingLogAerotow);
begin
  inherited Daybook := Value;
  if Assigned(Value) then
    Tugs := Value.LogTugs;
end;


{
************************** TLKRecordsGlidingLogWinch ***************************
}
function TLKRecordsGlidingLogWinch.GetDaybook: TLKDaybookGlidingLogWinch;
begin
  Result := TLKDaybookGlidingLogWinch(inherited Daybook);
end;

procedure TLKRecordsGlidingLogWinch.SetDaybook(Value:
    TLKDaybookGlidingLogWinch);
begin
  inherited Daybook := Value;
  //if Assigned(Value) then Tugs := Value.TugLogs;
end;

initialization

  RegisterClasses([TLKClubMembers,
    TLKLedgerLogGlider, TLKLedgerLogTug,
      TLKLedgerLogPilotGlider, TLKLedgerLogPilotTug,
      TLKDaybookGlidingLogAerotow, TLKDaybookGlidingLogWinch,
      TLKRecordsGlidingLogAerotow, TLKRecordsGlidingLogWinch]);

end.
