{*****************************************************************************}
{                                                                             }
{   This file is part of the LKGliding components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form to show a TLKRecordsGlidingLogAerotow record                      }
{                                                                             }
{*****************************************************************************}

unit LKGlidingLogAerotowInputRecordDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, LKGlidingLogInputRecordDlg, Menus, ImgList, ActnList, DB,
  StdCtrls, DBCtrls, ExtCtrls, ComCtrls, Buttons, ToolWin,
  LKGlidingComponents, LKdlgs, LKUtilComponents, StdActns,
  LKDBControls, Grids, DBGrids, JvExDBGrids, JvDBGrid, JvDBUltimGrid;

type
  TFormLKGlidingLogAerotowInputRecord = class(TFormLKGlidingLogInputRecord)
    ActionViewTugs: TAction;
    ActionTugRefExit: TAction;
    ActionTugPilotRefExit: TAction;
    sbTugs: TToolButton;
    procedure ActionViewTugsExecute(Sender: TObject);
    procedure ActionTugRefExitExecute(Sender: TObject);
    procedure ActionTugPilotRefExitExecute(Sender: TObject);
  private
    { Private declarations }
    FMyAccountsComponent: TLKRecordsGlidingLogAerotow;
  protected
    DBETugRef, DBETugPilotRef, DBETugPilotName: TDBEdit;
    procedure CustomiseMasterEdits; override;
    procedure SetDetailEditsActions; override;
  public
    { Public declarations }
    procedure Construct(Sender: TLKRecordsGlidingLogAerotow);
  end;

var
  FormLKGlidingLogAerotowInputRecord: TFormLKGlidingLogAerotowInputRecord;

implementation

uses LKDefine, LKUtils;

{$R *.dfm}

{ TFormLKGlidingLogAerotowInputRecord }

procedure TFormLKGlidingLogAerotowInputRecord.Construct(
  Sender: TLKRecordsGlidingLogAerotow);
begin
  FMyAccountsComponent := Sender;
  inherited Construct(Sender);
end;

procedure TFormLKGlidingLogAerotowInputRecord.SetDetailEditsActions;
var
  I: Integer;
begin
  if Assigned(DetailEditList) then
  with DetailEditList do
    for I := 0 to Count - 1 do
      with Items[I] do
      begin
        if (FieldName = 'TUG') then
        begin
          DBETugRef := TDBEdit(EditControl);
          DBETugRef.OnExit := ActionTugRefExit.OnExecute;
        end;
        if (FieldName = 'TUG_PILOT') then
        begin
          DBETugPilotRef := TDBEdit(EditControl);
          DBETugPilotRef.OnExit := ActionTugPilotRefExit.OnExecute;
        end;
        if (FieldName = 'TUG_PILOT_NAME') then
        begin
          DBETugPilotName := TDBEdit(EditControl);
          ReadOnly := True;
        end;
      end;
  inherited;
end;

procedure TFormLKGlidingLogAerotowInputRecord.CustomiseMasterEdits;
var
  I: Integer;
begin
  inherited;  
  with MasterEditList do
    for I := 0 to Count - 1 do
      with Items[I] do
      begin
        if (FieldName = 'TACH_TIME') then
          ReadOnly := True
        else
        if (FieldName = 'TUG_PILOT') then
        begin
          ControlType := dbcDCLookup;
          with TLKDBDcLookupEdit(EditControl) do
          begin
            LookupDataComponent := FMyAccountsComponent.ClubMembers;
            LookupDataset := tbMaster;
            LookupFieldName := 'MEMBER_REFERENCE';
          end;
        end;
      end;
end;

procedure TFormLKGlidingLogAerotowInputRecord.ActionViewTugsExecute(
  Sender: TObject);
begin
  inherited;
  FMyAccountsComponent.Tugs.View(null, true, [mbOK]);
end;

procedure TFormLKGlidingLogAerotowInputRecord.ActionTugRefExitExecute(
  Sender: TObject);
var
  S: string;
begin
  inherited;
  if (FMyAccountsComponent.TblDetail.State = dsBrowse) then
    Exit;
  S := DBETugRef.Field.AsString;
  if (S = '') then
    DBETugRef.Field.AsString := ''
  else
    with FMyAccountsComponent.Tugs do
      if not Locate('TUG_CALLSIGN', S, []) then
      begin
        View(S, True, [mbOK]);
        DBETugRef.Field.AsString :=
          TblMaster.FieldByname('TUG_CALLSIGN').AsString;
      end;
end;

procedure TFormLKGlidingLogAerotowInputRecord.ActionTugPilotRefExitExecute(
  Sender: TObject);
var
  S: string;
begin
  if (FMyAccountsComponent.TblDetail.State = dsBrowse) then
    exit;
  S := DBETugPilotRef.Field.AsString;
  if (S = '') then
    DBETugPilotName.Field.AsString := ''
  else
    with FMyAccountsComponent.ClubMembers do
    begin
      if not Locate('MEMBER_REFERENCE', S, []) then
      begin
        View(S, True, [mbOK]);
        DBETugPilotRef.Field.AsString :=
          TblMaster.FieldByname('MEMBER_REFERENCE').AsString;
      end;
      DBETugPilotName.Field.AsString :=
        TblMaster.FieldByName('INITIALS').AsString + ' '
        + TblMaster.FieldByName('SURNAME').AsString;
    end;
end;

end.

