{*****************************************************************************}
{                                                                             }
{   This file is part of the LKGliding components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form to show a TLKRecordsGlidingLog record                             }
{                                                                             }
{*****************************************************************************}

unit LKGlidingLogInputRecordDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, LKDocumentsOrderEntryRecordDlg, Menus, ImgList, ActnList, DB,
  StdCtrls, DBCtrls, ExtCtrls, ComCtrls, Buttons, ToolWin, JvDBControls,
  LKGlidingComponents, LKdlgs, LKUtilComponents, StdActns, Grids, DBGrids,
  LKDBControls, JvExDBGrids, JvDBGrid, JvExStdCtrls, JvDBUltimGrid;

type
  TFormLKGlidingLogInputRecord = class(TFormLKDocumentsOrderEntryRecord)
    ActionViewMembers: TAction;
    ActionViewGliders: TAction;
    ActionCustRefExit: TAction;
    ActionInstructorRefExit: TAction;
    ActionGliderRefExit: TAction;
    PopupBreak2: TMenuItem;
    ActionInvoiceItem: TAction;
    Invoice1: TMenuItem;
    ActionJournalAnalysisAdjustments: TAction;
    sbMembers: TToolButton;
    ToolButton2: TToolButton;
    sbCostAdj: TToolButton;
    procedure ActionViewMembersExecute(Sender: TObject);
    procedure ActionViewGlidersExecute(Sender: TObject);
    procedure ActionCustRefExitExecute(Sender: TObject);
    procedure ActionInstructorRefExitExecute(Sender: TObject);
    procedure ActionGliderRefExitExecute(Sender: TObject);
    procedure ActionInvoiceItemExecute(Sender: TObject);
    procedure ActionJournalAnalysisAdjustmentsExecute(Sender: TObject);
  private
    { Private declarations }
    FMyAccountsComponent: TLKRecordsGlidingLog;
  protected
    DBECustRef, DBEPilotName,
    DBEInstructorRef, DBEInstructorName,
    DBEGliderRef, DBEGliderType: TDBEdit;
    CBPricing: TDBComboBox;
    procedure CustomiseMasterEdits; override;
    procedure CustomiseDetailEdits; override;
    procedure SetDetailEditsActions; override;
  public
    { Public declarations }
    procedure Construct(Sender: TLKRecordsGlidingLog);
  end;

var
  FormLKGlidingLogInputRecord: TFormLKGlidingLogInputRecord;

implementation

{$R *.dfm}

uses LKDefine, LKUtils;

{ TFormLKGlidingLogInputRecord }

procedure TFormLKGlidingLogInputRecord.Construct(Sender: TLKRecordsGlidingLog);
begin
  FMyAccountsComponent := Sender;
  ActionJournalAnalysisAdjustments.Visible :=
    Assigned(FMyAccountsComponent.OnJournalAnalysisAdjustments);
  inherited Construct(Sender);
end;

procedure TFormLKGlidingLogInputRecord.CustomiseMasterEdits;
var
  I: Integer;
begin
  inherited;
  with MasterEditList do
    for I := 0 to Count - 1 do
      with Items[I] do
      begin
        if (FieldName = 'DAY_NAME') then
          ReadOnly := True;
      end;
end;

procedure TFormLKGlidingLogInputRecord.CustomiseDetailEdits;
var
  I: integer;
begin
  if Assigned(DetailEditList) then
  with DetailEditList do
    for I := 0 to Count - 1 do
      with TLKDcDBEditControl(Items[I]) do
      begin
        if (FieldName = 'PRICING') then
        begin
          ControlType := dbcComboBox;
          CBPricing := TDBComboBox(EditControl);
          CBPricing.Items.Assign(FMyAccountsComponent.PricingTypeItems);
        end;
      end;
  inherited;
end;

procedure TFormLKGlidingLogInputRecord.SetDetailEditsActions;
var
  I: Integer;
begin
  if Assigned(DetailEditList) then
  with DetailEditList do
    for I := 0 to Count - 1 do
      with TLKDcDBEditControl(items[I]) do
      begin
        if (FieldName = 'CUSTOMER_ACCOUNT') then
        begin
          DBECustRef := TDBEdit(EditControl);
          DBECustRef.OnExit := ActionCustRefExit.OnExecute;
        end;
        if (FieldName = 'PILOT_NAME') then
          DBEPilotName := TDBEdit(EditControl);
        if (FieldName = 'INSTRUCTOR') then
        begin
          DBEInstructorRef := TDBEdit(EditControl);
          DBEInstructorRef.OnExit := ActionInstructorRefExit.OnExecute;
        end;
        if (FieldName = 'INSTRUCTOR_NAME') then
        begin
          DBEInstructorName := TDBEdit(EditControl);
          ReadOnly := True;
        end;
        if (FieldName = 'GLIDER_CALLSIGN') then
        begin
          DBEGliderRef := TDBEdit(EditControl);
          DBEGliderRef.OnExit := ActionGliderRefExit.OnExecute;
        end;
        if (FieldName = 'GLIDER_TYPE') then
          DBEGliderType := TDBEdit(EditControl);
        if (FieldName = 'LAND') then
          TDBEdit(EditControl).OnExit := ActionPostEditDetailDBExecute;
        if (FieldName = 'DURATION') then
          ReadOnly := True;
        if (FieldName = 'INVOICED') then
          ReadOnly := True;
      end;
  inherited;
end;

procedure TFormLKGlidingLogInputRecord.ActionViewMembersExecute(
  Sender: TObject);
begin
  inherited;
  FMyAccountsComponent.ClubMembers.View(null, True, [mbOK]);
end;

procedure TFormLKGlidingLogInputRecord.ActionViewGlidersExecute(
  Sender: TObject);
begin
  inherited;
  FMyAccountsComponent.Gliders.View(null, True, [mbOK]);
end;

procedure TFormLKGlidingLogInputRecord.ActionCustRefExitExecute(
  Sender: TObject);
var
  S: string;
begin
  inherited;
  if (FMyAccountsComponent.TblDetail.State = dsBrowse) then
    exit;
  S := DBECustRef.Field.AsString;
  if (S = '') then
    DBEPilotName.Field.Value := null
  else
    with FMyAccountsComponent.ClubMembers do
    begin
      if not Locate('MEMBER_REFERENCE', S, []) then
      begin
        View(null, True, [mbOK]);
        DBECustRef.Field.AsString :=
          TblMaster.FieldByname('MEMBER_REFERENCE').AsString;
      end;
      if (DBEPilotName.Field.AsString = '') then
        DBEPilotName.Field.AsString := TblMaster.FieldByName('INITIALS').AsString
          + ' '
          + TblMaster.FieldByName('SURNAME').AsString;
    end;
end;

procedure TFormLKGlidingLogInputRecord.ActionInstructorRefExitExecute(Sender:
  TObject);
var
  S: string;
begin
  inherited;
  if (FMyAccountsComponent.TblDetail.State = dsBrowse) then
    exit;
  S := DBEInstructorRef.Field.AsString;
  if (S = '') then
    DBEInstructorName.Field.AsString := ''
  else
    with FMyAccountsComponent.ClubMembers do
    begin
      if not Locate('MEMBER_REFERENCE', S, []) then
      begin
        View(S, True, [mbOK]);
        DBEInstructorRef.Field.AsString :=
          TblMaster.FieldByname('MEMBER_REFERENCE').AsString;
      end;
      DBEInstructorName.Field.AsString :=
        TblMaster.FieldByName('INITIALS').AsString + ' '
        + TblMaster.FieldByName('SURNAME').AsString;
    end;
end;

procedure TFormLKGlidingLogInputRecord.ActionGliderRefExitExecute(
  Sender: TObject);
var
  S: string;
begin
  inherited;
  if (FMyAccountsComponent.TblDetail.State = dsBrowse) then
    exit;
  S := DBEGliderRef.Field.AsString;
  if (S = '') then
    DBEGliderType.Field.AsString := ''
  else
    with FMyAccountsComponent.Gliders do
    begin
      if not Locate('GLIDER_CALLSIGN', S, []) then
      begin
        View(S, True, [mbOK]);
        DBEGliderRef.Field.AsString :=
          TblMaster.FieldByname('GLIDER_CALLSIGN').AsString;
      end;
      DBEGliderType.Field.AsString :=
        TblMaster.FieldByName('GLIDER_TYPE').AsString;
    end;
end;

procedure TFormLKGlidingLogInputRecord.ActionInvoiceItemExecute(
  Sender: TObject);
begin
  inherited;
  FMyAccountsComponent.DlgNewInvoiceWizardExecute;
end;

procedure TFormLKGlidingLogInputRecord.ActionJournalAnalysisAdjustmentsExecute(
  Sender: TObject);
begin
  inherited;
  FMyAccountsComponent.OnJournalAnalysisAdjustments(self);
end;

end.

