unit LKServiceComponents;

interface

uses
  Windows, Messages, SysUtils, Classes, ExtCtrls, Graphics, Controls, Forms,
  Dialogs,
  DBCommon, DB, SqlExpr, DbConsts, LKDefine, StdCtrls, DBCtrls, DBGrids,
  ActnList, ImgList,
  IniFiles, ComCtrls, Variants,
  LKdlgs, LKCoreComponents, LKAccountsComponents;

type
  TLKServiceJobCards = class;

  TLKServiceCustomers = class(TLKRecords)
  end;

  TLKServiceLedgerSales = class(TLKLedgerSales)
  private
    function GetCustomers: TLKServiceCustomers;
    procedure SetCustomers(Value: TLKServiceCustomers);
  protected
    procedure AssignTo(Dest: TPersistent); override;
    procedure SetActive(Value: Boolean); override;
  public
    function RecordNew: TModalResult; override;
  published
    property Customers: TLKServiceCustomers read GetCustomers write
            SetCustomers;
  end;

  TLKServiceDaybookSales = class(TLKDaybookSales)
  private
    function GetLedger: TLKServiceLedgerSales;
    procedure SetLedger(Value: TLKServiceLedgerSales);
  published
    property Ledger: TLKServiceLedgerSales read GetLedger write SetLedger;
  end;

  TLKServiceQuotations = class(TLKPricesBase)
  private
    FJobCards: TLKServiceJobCards;
    function GettmDocumentDate: TDateTime;
    procedure SetJobCards(Value: TLKServiceJobCards);
    procedure SettmDocumentDate(Value: TDateTime);
  protected
    procedure CreateViewFormRecord; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation);
            override;
  public
    procedure CreateViewForm; override;
    procedure DlgCopyPriceItemsFromExecute(ParentForm: TForm); override;
    procedure DlgUpdateJobPricesExecute;
    property JobCards: TLKServiceJobCards read FJobCards write SetJobCards;
    property tmDocumentDate: TDateTime read GettmDocumentDate write
            SettmDocumentDate;
  end;

  TLKServiceJobCardsJobInvoices = class(TLKContainedDocuments)
  private
    function GetContainerComponent: TLKServiceJobCards;
    procedure SetContainerComponent(Value: TLKServiceJobCards);
  public
    property ContainerComponent: TLKServiceJobCards read GetContainerComponent
            write SetContainerComponent;
  end;

  TLKServiceJobCards = class(TLKDocumentsEntryTrade)
  private
    FJobInvoices: TLKServiceJobCardsJobInvoices;
    function GetMasterComponent: TLKServiceQuotations;
    function GetSalesInvoices: TLKDocumentsSalesInvoices;
    procedure SetJobInvoices(Value: TLKServiceJobCardsJobInvoices);
    procedure SetMasterComponent(Value: TLKServiceQuotations);
    procedure SetSalesInvoices(Value: TLKDocumentsSalesInvoices);
  protected
    procedure AssignTo(Dest: TPersistent); override;
    procedure CreateViewFormRecord; override;
    procedure DoMasterBeforePost; override;
    function EndOfMonth: TModalResult; override;
    procedure Loaded; override;
    procedure SetName(const NewName: TComponentName); override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure CreateViewForm; override;
    procedure DlgPrintWorkInProgressReport; virtual;
    procedure DlgUpdatePricesExecute;
    procedure Refresh; override;
  published
    property JobInvoices: TLKServiceJobCardsJobInvoices read FJobInvoices
            write SetJobInvoices;
    property MasterComponent: TLKServiceQuotations read GetMasterComponent
            write SetMasterComponent;
    property SalesInvoices: TLKDocumentsSalesInvoices read GetSalesInvoices
            write SetSalesInvoices;
  end;

procedure Register;

implementation

uses LKUtils, LKUtilsDB,
  LKServiceQuotationDlg, LKServiceQuotationInputRecordDlg,
  LKServiceQuotationCopyFromDlg,
  LKServiceJobCardsDlg, LKServiceJobCardsInputRecordDlg;

procedure Register;
begin
  RegisterComponents('LKSrvce', [TLKServiceCustomers]);
  RegisterComponents('LKSrvce', [TLKServiceLedgerSales]);
  RegisterComponents('LKSrvce', [TLKServiceDaybookSales]);
  RegisterComponents('LKSrvce', [TLKServiceJobCards]);
  RegisterComponents('LKSrvce', [TLKServiceQuotations]);
end;


{
**************************** TLKServiceLedgerSales ****************************
}
procedure TLKServiceLedgerSales.AssignTo(Dest: TPersistent);
var
  DestSLSales: TLKServiceLedgerSales;
begin
  inherited AssignTo(Dest);
  if not (Dest is TLKServiceLedgerSales) then
    exit;
  DestSLSales := TLKServiceLedgerSales(Dest);
  DestSLSales.Customers := Customers;
end;

function TLKServiceLedgerSales.GetCustomers: TLKServiceCustomers;
begin
  if MasterComponent is TLKServiceCustomers then
    Result := TLKServiceCustomers(MasterComponent)
  else
    Result := nil;
end;

function TLKServiceLedgerSales.RecordNew: TModalResult;
var
  c, n: string;
begin
  Result := inherited RecordNew;
  if (Result = mrOK) then
  begin
    c := tmAccountReference;
    n := TblMaster.FieldByName('CONTRACT_NAME').AsString;
    with NominalLedger.CostLedger.CostCentres do
      if not FindKey([c]) then
      begin
        InsertRecord([c, 'JOBS', null, null, null, n]);
        ApplyUpdates(0);
      end;
  end;
end;

procedure TLKServiceLedgerSales.SetActive(Value: Boolean);
begin
  if (csLoading in ComponentState) then
    exit;
  if assigned(Customers) then
    Customers.Active := Value;
  inherited SetActive(Value);
end;

procedure TLKServiceLedgerSales.SetCustomers(Value: TLKServiceCustomers);
begin
  MasterComponent := Value;
  MasterTable := tbMaster;
  MasterFields := 'CUSTOMER_REFERENCE';
  //if assigned(Value) then Value.FreeNotification(Self);
end;


{
*************************** TLKServiceDaybookSales ****************************
}
function TLKServiceDaybookSales.GetLedger: TLKServiceLedgerSales;
begin
  Result := TLKServiceLedgerSales(inherited Ledger);
end;

procedure TLKServiceDaybookSales.SetLedger(Value: TLKServiceLedgerSales);
begin
  inherited Ledger := Value;
end;


{
**************************** TLKServiceQuotations *****************************
}
procedure TLKServiceQuotations.CreateViewForm;
begin
  FormView := TFormLKServiceQuotation.Create(Application);
  TFormLKServiceQuotation(FormView).Construct(Self);
end;

procedure TLKServiceQuotations.CreateViewFormRecord;
begin
  FormViewRecord := TFormLKServiceQuotationInputRecord.Create(Application);
  TFormLKServiceQuotationInputRecord(FormViewRecord).Construct(Self);
end;

procedure TLKServiceQuotations.DlgCopyPriceItemsFromExecute(ParentForm: TForm);
begin
  //check Quotes detail is empty
  if TblDetail.FindFirst then
    raise
      LKError.create('You cannot copy to this Quotation because it already has priced items');
  with TDlgLKServiceQuotationCopyFrom.Create(Application) do
  try
    PopUpParent := ParentForm;
    Construct(Self);
    ShowModal;
  finally
    Free;
  end;
end;

procedure TLKServiceQuotations.DlgUpdateJobPricesExecute;
begin
  JobCards.DlgUpdatePricesExecute;
end;

function TLKServiceQuotations.GettmDocumentDate: TDateTime;
begin
  Result := GetDate(TblMaster.FieldByName('DATE_'));
end;

procedure TLKServiceQuotations.Notification(AComponent: TComponent; Operation:
        TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) then
  begin
    if (AComponent = FJobCards) then
      FJobCards := nil;
  end;
end;

procedure TLKServiceQuotations.SetJobCards(Value: TLKServiceJobCards);
begin
    FJobCards := Value;
  if assigned(Value) then
    Value.FreeNotification(Self);
end;

procedure TLKServiceQuotations.SettmDocumentDate(Value: TDateTime);
begin
  TblMaster.FieldByName('DATE_').AsDateTime := Value;
end;


{
************************ TLKServiceJobCardsJobInvoices ************************
}
function TLKServiceJobCardsJobInvoices.GetContainerComponent:
        TLKServiceJobCards;
begin
  Result := TLKServiceJobCards(inherited ContainerComponent);
end;

procedure TLKServiceJobCardsJobInvoices.SetContainerComponent(Value:
        TLKServiceJobCards);
begin
  inherited ContainerComponent := Value;
  if assigned(Value) then
    Value.JobInvoices := self;
end;


{
***************************** TLKServiceJobCards ******************************
}
constructor TLKServiceJobCards.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  DocumentRefLength := 8;
  with TLKServiceJobCardsJobInvoices.Create(Self) do
    ContainerComponent := Self;
  with JobInvoices do
  begin
    IndexFieldNames := 'JOB_NUMBER;INVOICE_REFERENCE';
    tDetailIndexFieldNames := 'INVOICE_REFERENCE';
    tDetailMasterFields := 'INVOICE_REFERENCE';
    MasterComponent := self;
    MasterTable := tbMaster;
    MasterFields := 'JOB_NUMBER';
  end;
  OnEndOfMonth := doEndOfMonth;
end;

procedure TLKServiceJobCards.AssignTo(Dest: TPersistent);
begin
  inherited;
  TLKServiceJobCards(Dest).JobInvoices.Assign(JobInvoices);
end;

procedure TLKServiceJobCards.CreateViewForm;
begin
  FormView := TFormLKServiceJobCards.Create(Application);
  TFormLKServiceJobCards(FormView).Construct(Self);
end;

procedure TLKServiceJobCards.CreateViewFormRecord;
begin
  FormViewRecord := TFormLKServiceJobCardsInputRecord.Create(Application);
  TFormLKServiceJobCardsInputRecord(FormViewRecord).Construct(Self);
end;

procedure TLKServiceJobCards.DlgPrintWorkInProgressReport;
begin
  Showmessage('Work in Progress report not implemented');
end;

procedure TLKServiceJobCards.DlgUpdatePricesExecute;
var
  i: Integer;
begin
  if
    (MessageDlg('Confirm that you want to recalculate all job cards for price changes?', mtConfirmation,
    [mbYes, mbCancel], 0) = mrCancel) then
    exit;
  screen.cursor := crHourglass;
  try
    Filtered := false;
    i := 0;
    if FindFirst then
      repeat
        Application.ProcessMessages;
        if (i = 100) then
        begin
          ApplyUpdates(0);
          i := 0;
        end
        else
          inc(i);
        with TblDetail do
          if FindFirst then
            repeat Edit;
              Post;
            until not FindNext;
      until not FindNext;
    ApplyUpdates(0);
  finally screen.cursor := crDefault;
  end;
end;

procedure TLKServiceJobCards.DoMasterBeforePost;
begin
  inherited DoMasterBeforePost;
  TblMaster.FieldByName('BATCH').Value := null;
end;

function TLKServiceJobCards.EndOfMonth: TModalResult;
begin
  DlgPrintWorkInProgressReport;
  Result := inherited EndOfMonth;
end;

function TLKServiceJobCards.GetMasterComponent: TLKServiceQuotations;
begin
  Result := TLKServiceQuotations(inherited MasterComponent);
end;

function TLKServiceJobCards.GetSalesInvoices: TLKDocumentsSalesInvoices;
begin
  Result := inherited SalesInvoices;
end;

procedure TLKServiceJobCards.Loaded;
begin
  inherited Loaded;
  try
    with JobInvoices do
    begin
      {
      if assigned(FSalesInvoices) then
       begin
       tMasterTableName := FSalesInvoices.tDetailTableName;
       tDetailTableName := FSalesInvoices.tMasterTableName;
       end;
      IndexFieldNames := 'JOB_NUMBER;Invoice_reference';
      tDetailIndexFieldNames := 'Invoice_reference';
      tDetailMasterFields :='Invoice_reference';
      }
      MasterComponent := self;
      MasterTable := tbMaster;
      MasterFields := 'JOB_NUMBER';
    end;
  except
    if csDesigning in ComponentState then
      Application.HandleException(Self)
    else
      raise;
  end;
  //showmessage('Loaded finish');
end;

procedure TLKServiceJobCards.Refresh;
begin
  JobInvoices.Refresh;
  inherited Refresh;
end;

procedure TLKServiceJobCards.SetJobInvoices(Value:
        TLKServiceJobCardsJobInvoices);
begin
  FJobInvoices.Free;
    FJobInvoices := Value;
end;

procedure TLKServiceJobCards.SetMasterComponent(Value: TLKServiceQuotations);
begin
  inherited MasterComponent := Value;
  if assigned(Value) then
    Value.JobCards := Self;
end;

procedure TLKServiceJobCards.SetName(const NewName: TComponentName);
begin
  inherited SetName(NewName);
  if (csLoading in ComponentState) then
    exit;
  JobInvoices.Name := NewName + loadingTmpName + 'JobInvoices';
end;

procedure TLKServiceJobCards.SetSalesInvoices(Value:
        TLKDocumentsSalesInvoices);
begin
  inherited SalesInvoices := Value;
  if assigned(Value) then
    with JobInvoices do
    begin
      //Value.JobCards := self;
      tMasterTableName := Value.tDetailTableName;
      tDetailTableName := Value.tMasterTableName;
    end;
end;

initialization

  RegisterClasses([TLKServiceCustomers, TLKServiceLedgerSales,
    TLKServiceDaybookSales,
      TLKServiceQuotations, TLKServiceJobCardsJobInvoices,
      TLKServiceJobCards]);

end.

