{*****************************************************************************}
{                                                                             }
{   This file is part of the installer program executable                     }
{                                                                             }
{   See the file COPYING.GPL.txt included in this distribution,               }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Form to do the zip file extraction and make shortcuts                  }
{                                                                             }
{*****************************************************************************}

unit InstallDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  ShlObj, ActiveX, ComObj, Registry, Dialogs, StdCtrls,
  JvExStdCtrls, JvCombobox, JvDriveCtrls, LKBaseComponent;

type
  TTarget = (msDesktop, msProgramMenu, msStartup, msStartMenu);

type
  TLKShortCut = class(TComponent)
  private
    class function Directory(const Value: TTarget): string;
  public
    { Public declarations }
    class procedure CreateShortCut(
      Target: TTarget; AppPath: TFileName; CaptionName: string);
    class procedure Delete(Target: TTarget; CaptionName: string);
  end;

type
  TFormInstall = class(TForm)
    BtnInstall: TButton;
    LKZipper1: TLKZipper;
    CbDesktop: TCheckBox;
    CbStartMenu: TCheckBox;
    JvDriveCombo1: TJvDriveCombo;
    StaticText1: TStaticText;
    procedure BtnInstallClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    InstallApSetupZip: string;
    InstallApExe: string;
    InstallApName: string;
  end;

var
  FormInstall: TFormInstall;

implementation

{$R *.dfm}

uses LKUtils, LKDefine;

procedure TFormInstall.FormCreate(Sender: TObject);
begin
  JvDriveCombo1.Drive := 'C';
end;

procedure TFormInstall.BtnInstallClick(Sender: TObject);
var P: string;
begin
//   if (DirectoryCopy(ExtractFilePath(Application.ExeName) + 'Accounts',
//       JvDriveCombo1.Drive + ':'  + PathDelim  + 'Accounts', False) > 0) then
  if LKZipper1.UnZip(JvDriveCombo1.Drive + ':'  + PathDelim,
    ExtractFilePath(Application.ExeName) + InstallApSetupZip) then
  begin
    P := JvDriveCombo1.Drive + ':' + InstallApExe;
    if cbDesktop.Checked then
      TLKShortcut.CreateShortCut(msDesktop, P, InstallApName);
    if cbStartMenu.Checked then
      TLKShortcut.CreateShortCut(msProgramMenu, P, InstallApName);
    Showmessage(InstallApName + ' Installation complete!');
    ModalResult := mrOK;
  end
  else
    ModalResult := mrCancel;
end;

class function TLKShortCut.Directory(const Value: TTarget): string;
var
  MyReg: TRegIniFile;
  WFileName: WideString;
begin
  MyReg := TRegIniFile.Create(
    'Software\MicroSoft\Windows\CurrentVersion\Explorer');
  { TRY to read the key }
  try
    case Value of
      msDesktop:
        Result := MyReg.ReadString('Shell Folders', 'Desktop', '');
      msProgramMenu:
        Result := MyReg.ReadString('Shell Folders', 'Programs', '');
      msStartMenu:
        Result := MyReg.ReadString('Shell Folders', 'Start Menu', '');
      msStartup:
        Result := MyReg.ReadString('Shell Folders', 'Startup', '');
    end;
  finally
    MyReg.Free;
  end;
end;

class procedure TLKShortCut.CreateShortCut(
      Target: TTarget; AppPath: TFileName; CaptionName: string);
var
  MyObject: IUnknown;
  MySLink: IShellLink;
  MyPFile: IPersistFile;
  WFileName: WideString;
begin
  MyObject := CreateComObject(CLSID_ShellLink);
  MySLink := MyObject as IShellLink;
  MyPFile := MyObject as IPersistFile;
  with MySLink do
  begin
    SetPath(PChar(AppPath));
    SetWorkingDirectory(PChar(ExtractFilePath(AppPath)));
  end;
  WFileName := Directory(Target) + '\' + CaptionName + '.lnk';
  MyPFile.Save(PWChar(WFileName), False);
  Application.ProcessMessages;
end;

class procedure TLKShortCut.Delete(Target: TTarget; CaptionName: string);
begin
  DeleteFile(PWChar(Directory(Target) + '\' + CaptionName + '.lnk'));
end;

end.
