{ ***************************************************************************** }
{ }
{ This file is part of the installer program executable }
{ }
{ See the file COPYING.GPL.txt included in this distribution, }
{ for details about the Licence and Copyright. }
{ }
{ This code is distributed in the hope that it will be useful, }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. }
{ }
{ Author: Leslie John Kaye :    www.leskaye.com }
{ }
{ Abstract: }
{ The Installer program simply unzips a zip archive to the root }
{ of the selected disc drive and creates program shortcuts }
{ See README.txt for more information }
{ }
{ ***************************************************************************** }

unit InstallMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, jpeg, ExtCtrls, StdCtrls, IniFiles, Buttons, ShellAPI;

type
  TLKInstallPanel = class(TPanel)
  private
    ButtonGettingStarted: TBitBtn;
    ButtonInstall: TButton;
    FApplicationExeName: string;
    FFileGettingStarted: string;
    FFileSetupZip: string;
    LabelTitle: TLabel;
    procedure ButtonGettingStartedClick(Sender: TObject);
    procedure ButtonInstallClick(Sender: TObject);
    function GetApplicationTitle: string;
    procedure SetApplicationTitle(const Value: string);
    procedure SetFileGettingStarted(const Value: string);
    procedure SetFileSetupZip(const Value: string);
  public
    constructor Create(AOwner: TComponent); override;
    property ApplicationExeName
      : string read FApplicationExeName write FApplicationExeName;
    property ApplicationTitle: string read GetApplicationTitle write
      SetApplicationTitle;
    property FileGettingStarted: string read FFileGettingStarted write
      SetFileGettingStarted;
    property FileSetupZip: string read FFileSetupZip write SetFileSetupZip;
  end;

  TFormInstallMain = class(TForm)
    btnClose: TButton;
    Image2: TImage;
    Panel2: TPanel;
    procedure btnCloseClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
  end;

var
  FormInstallMain: TFormInstallMain;

implementation

uses LKUtils, LKDefine, EULADlg, InstallDlg;
{$R *.dfm}

{
  ******************************* TMtMInstallPanel *******************************
}
constructor TLKInstallPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Align := alTop;
  Color := clBlack;
  Height := 104;
  LabelTitle := TLabel.Create(AOwner);
  with LabelTitle do
  begin
    Left := 16;
    Top := 4;
    Height := 45;
    Parent := Self;
    Font.Name := 'Comic Sans MS';
    Font.Color := clYellow;
    Font.Size := 24;
  end;
  ButtonInstall := TButton.Create(AOwner);
  with ButtonInstall do
  begin
    Left := 16;
    Top := 55;
    Height := 41;
    Width := 409;
    Parent := Self;
    Font.Name := 'Benguiat Bk BT';
    Font.Style := [fsBold];
    Font.Size := 18;
    Caption := 'Install >>>';
    OnClick := ButtonInstallClick;
  end;
  ButtonGettingStarted := TBitBtn.Create(AOwner);
  with ButtonGettingStarted do
  begin
    Left := 440;
    Top := 55;
    Height := 41;
    Width := 217;
    Parent := Self;
    Font.Name := 'Benguiat Bk BT';
    Font.Style := [fsBold];
    Font.Size := 14;
    Caption := 'Getting Started';
    Glyph.LoadFromResourceName(MainInstance, 'PDF');
    OnClick := ButtonGettingStartedClick;
  end;
end;

procedure TLKInstallPanel.ButtonGettingStartedClick(Sender: TObject);
begin
  ShellExecute(MainInstance, 'open',
    PChar('file:///' + ExtractFilePath(Application.ExeName)
        + FileGettingStarted), nil, nil, 1);
end;

procedure TLKInstallPanel.ButtonInstallClick(Sender: TObject);
begin
  if FileExists('.\EULA.rtf') and not(FormEula.ShowModal = mrOK) then
    Exit;
  with FormInstall do
  begin
    InstallApSetupZip := FFileSetupZip;
    InstallApExe := FApplicationExeName;
    InstallApName := ApplicationTitle;
    ShowModal;
  end;
end;

function TLKInstallPanel.GetApplicationTitle: string;
begin
  Result := LabelTitle.Caption;
end;

procedure TLKInstallPanel.SetApplicationTitle(const Value: string);
begin
  LabelTitle.Caption := Value;
end;

procedure TLKInstallPanel.SetFileGettingStarted(const Value: string);
begin
  ButtonGettingStarted.Visible := (Value > '');
  FFileGettingStarted := Value;
end;

procedure TLKInstallPanel.SetFileSetupZip(const Value: string);
begin
  ButtonInstall.Enabled := (Value > '');
  FFileSetupZip := Value;
end;

{
  ******************************* TFormInstallMain *******************************
}
procedure TFormInstallMain.btnCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TFormInstallMain.FormShow(Sender: TObject);
var
  Apps: TStringList;
  i: Integer;
begin
  Apps := TStringList.Create;
  try
    with TIniFile.Create(ChangeFileExt(Application.ExeName,'.ini')) do
      try
        ReadSection('INSTALLS', Apps);
        Height := 188 + Apps.Count * 104;
        Panel2.Height := 4 + Apps.Count * 104;
        for i := 0 to Apps.Count - 1 do
          with TLKInstallPanel.Create(Self) do
          begin
            Parent := Panel2;
            ApplicationExeName := ReadString(Apps[i], 'ApplicationExeName', '');
            ApplicationTitle := ReadString(Apps[i], 'ApplicationTitle', '');
            FileGettingStarted := ReadString(Apps[i], 'FileGettingStarted', '');
            FileSetupZip := ReadString(Apps[i], 'FileSetupZip', '');
          end;
      finally
        Free;
      end;
  finally
    Apps.Free;
  end;
end;

end.
