unit FileUtilsMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, Menus, ToolWin, ComCtrls, Mask, ExtCtrls, IniFiles,
  AbBase, AbSelfEx, LKDlgs, LKDefine, LKBaseComponent;

type
  TFormMain = class(TForm)
    LKZipper1: TLKZipper;
    Timer1: TTimer;
    procedure FormShow(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    { Private declarations }
    FileNames: TStringList;
    BackupAction, File3, File1, File2: string;
    function DoFixedBackup: boolean;
    function DoFloppyBackup: boolean;
    function DoFixedRestore: boolean;
    function DoFloppyRestore: boolean;
    procedure DoFileCopy;
  public
    { Public declarations }
  end;

var
  FormMain: TFormMain;

implementation

uses LKUtils, LKZipProgressDlg;
{$R *.DFM}

procedure TFormMain.FormShow(Sender: TObject);
begin
  Timer1.Enabled := True;
end;

procedure TFormMain.Timer1Timer(Sender: TObject);
begin
  try
    Timer1.Enabled := false;
    FileNames := TStringList.Create;
    try
      with TIniFile.Create(
        ExtractFilePath(Application.ExeName)+ (LKFileUtilsExe + '.ini')) do
        try
          BackupAction := ReadString('Utils', 'Action', 'no-op');
          File1 := ReadString('Utils', 'File1', 'A:');
          File2 := ReadString('Utils', 'File2', 'C:\Backup');
          File3 := ReadString('Utils', 'File3', '');
          LKZipper1.Password := ReadString('Utils', 'Password', '');
          WriteString('Utils', 'Action', 'no-op');
        finally
          Free;
        end;
      if BackupAction = 'no-op' then
        ShowMessage('This program is intended to be' +
            ' called from LK database programs and should not be run by itself.'
          );
      FileNames.Add(File3 + '.FDB');
      if BackupAction = 'BackupRemoveable' then
        DoFloppyBackup;
      if BackupAction = 'BackupFixed' then
        DoFixedBackup;
      if BackupAction = 'BackupBoth' then
      begin
        DoFixedBackup;
        DoFloppyBackup;
      end;
      if BackupAction = 'RestoreRemovable' then
        DoFloppyRestore;
      if BackupAction = 'RestoreFixed' then
        DoFixedRestore;
      if BackupAction = 'FileCopy' then
        DoFileCopy;
    finally
      FileNames.Free;
    end;
  finally
    Close;
  end;
end;

// GetEnvironmentVariable('TEMP');

function TFormMain.DoFixedBackup: boolean;
var
  ZipFullName: string;
begin
  if not DirectoryExists(File2) then
    ForceDirectories(File2);
  ChDir(File2);
  ZipFullName := File2 + PathDelim + File3;
  if FileExists(ZipFullName + '.zij') then
    DeleteFile(pchar(ZipFullName + '.zij'));
  if FileExists(ZipFullName + '.zik') then
    RenameFile(ZipFullName + '.zik', ZipFullName + '.zij');
  if FileExists(ZipFullName + '.zil') then
    RenameFile(ZipFullName + '.zil', ZipFullName + '.zik');
  if FileExists(ZipFullName + '.zim') then
    RenameFile(ZipFullName + '.zim', ZipFullName + '.zil');
  if FileExists(ZipFullName + '.zin') then
    RenameFile(ZipFullName + '.zin', ZipFullName + '.zim');
  if FileExists(ZipFullName + '.zio') then
    RenameFile(ZipFullName + '.zio', ZipFullName + '.zin');
  if FileExists(ZipFullName + '.zip') then
    RenameFile(ZipFullName + '.zip', ZipFullName + '.zio');
  Result := LKZipper1.ZipFiles(FileNames, DataFolder, File3, File2, False);
end;

function TFormMain.DoFixedRestore: boolean;
begin
  Result := LKZipper1.UnZip(DataFolder, File2 + PathDelim + File3 + '.zip');
end;

function TFormMain.DoFloppyBackup: boolean;
begin
  Result := True;
  if (MessageDlg('Backup to removeable drive?', mtInformation,
      [mbOK, mbCancel], 0) = mrOK) then
    Result := LKZipper1.ZipFiles(FileNames, DataFolder,
      File3, File1, True);
end;

function TFormMain.DoFloppyRestore: boolean;
begin
  Result := LKZipper1.UnZip(DataFolder, File1 + PathDelim + File3);
end;

procedure TFormMain.DoFileCopy;
begin
  FileCopy(File1, File2);
end;

end.
