unit ConvertAllDlg;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls;

type
  TFormConvertAll = class(TForm)
    MemoFilesIn: TMemo;
    Panel1: TPanel;
    ButtonFinish: TButton;
    ButtonBack: TButton;
    Splitter1: TSplitter;
    MemoFilesOut: TMemo;
    procedure FormShow(Sender: TObject);
    procedure ButtonFinishClick(Sender: TObject);
  private
    Fn, FnOut: TFileName;
    FConvertDirectory: string;
    FAlias: string;
    FAccountsYear: string;
    FProjectDirectory: string;
    FAccountsDirectory: string;
    procedure Convert;
    { Private declarations }
  public
    { Public declarations }
    property Alias: string read FAlias write FAlias;
    property AccountsYear: string read FAccountsYear write FAccountsYear;
    property ConvertDirectory: string read FConvertDirectory write FConvertDirectory;
  end;

var
  FormConvertAll: TFormConvertAll;

implementation

uses FinishDlg;

{$R *.dfm}

{ TFormConvertAll }

procedure TFormConvertAll.ButtonFinishClick(Sender: TObject);
begin
  if ButtonFinish.Caption = '&Abort' then
    ButtonBack.Enabled := True
  else
  begin
    FormFinish.ProjectDirectory := FProjectDirectory;
    FormFinish.AccountsDirectory := FAccountsDirectory;
    FormFinish.Alias := FAlias;
    FormFinish.AccountsYear := FAccountsYear;
    FormFinish.ShowModal;
    ModalResult := mrOK;
  end;
end;

procedure TFormConvertAll.Convert;
var
  I, J: Integer;
  S: string;
  DataFile : string;
  AccountsProgramFolder, AccountsDataFolder: string;
  SearchRec: TSearchRec;
  SLIn, SLOut: TStringList;
  MIn: TMemoryStream;
  FIn: TFileStream;
  BufProcess, BufOut: array[0..4049] of Char;

  procedure CopyBufOut(StartPos, CopyLength: Integer);
  var P: Integer;
  begin
    for P := 0 to CopyLength - 1 do
      BufOut[P] := BufProcess[StartPos + P];
  end;

  function DeBadge(const ZoomStr: string): string;
  var
    I: Integer;
  begin
    Result:= ZoomStr;
    I:= Pos('Zoom', Result);
    while I > 0 do
    begin
      Result := Copy(Result, 1, I - 1) + FAlias
              + Copy(Result, I + 4, Length(Result) - I + 3);
      I:= Pos('Zoom', Result);
    end;
  end;

begin
  ButtonFinish.Caption := '&Abort';
  ButtonBack.Enabled := False;
  FProjectDirectory := '..' + Pathdelim + '..' + Pathdelim
    + 'Projects' + Pathdelim + FAlias;
  if DirectoryExists(FProjectDirectory) and (MessageDlg(FProjectDirectory
    +' already exists. If you continue files in the folder will be overwritten',
    mtWarning, [mbOK, mbAbort], 0) = mrAbort) then
  begin
    ButtonFinishClick(nil);
    Exit;
  end;
  ForceDirectories(FProjectDirectory);
  FAccountsDirectory := Copy(Application.ExeName, 1, 3) + FAlias + 'Accounts';
  AccountsProgramFolder :=  FAccountsDirectory + Pathdelim + 'program';
  AccountsDataFolder := FAccountsDirectory + Pathdelim + 'data';
  ForceDirectories(AccountsProgramFolder);
  ForceDirectories(AccountsDataFolder);
  DataFile := AccountsDataFolder + Pathdelim + Uppercase(FAlias)
    + AccountsYear + '.FDB';
  if FileExists(DataFile) and (MessageDlg(DataFile
    +' already exists. If you continue it will be overwritten',
      mtWarning, [mbOK, mbAbort], 0) = mrAbort) then
  begin
    ButtonFinishClick(nil);
    Exit;
  end;
  Fn := ExtractFilePath(Application.ExeName) + 'dbxdrivers.ini';
  FnOut := AccountsProgramFolder+pathdelim+'dbxdrivers.ini';
  if FileExists(Fn) then
    CopyFile(PWideChar(Fn), PWideChar(FnOut), True);
  MemoFilesIn.Lines.Clear;
  MemoFilesOut.Lines.Clear;
  I := FindFirst(FConvertDirectory + PathDelim + '*.*',faArchive,SearchRec);
  try
    while (I = 0) do
    begin
      if not (ExtractFileName(SearchRec.Name) = 'SYNOPSIS.TXT') then
        MemoFilesIn.Lines.Add(SearchRec.Name);
      I := FindNext(SearchRec);
    end;
  finally
    FindClose(SearchRec);
  end;
  SLIn:= TStringList.Create;
  SLOut:= TStringList.Create;
  try
    for I := 0 to MemoFilesIn.Lines.Count - 1 do
    begin
      if ButtonBack.Enabled then
        Abort;
      Fn := MemoFilesIn.Lines[I];
      S := ExtractFileName(Fn);
      if (S = 'SAMPLE.FDB') or (S = 'SAMPLE.SQL') then
      begin
        if (S = 'SAMPLE.FDB') then
          FnOut := DataFile
        else FnOut := FAccountsDirectory + Pathdelim + 'data' + Pathdelim + S;
        CopyFile(PWideChar(Fn), PWideChar(FnOut), False);
      end
      else
      begin
        FnOut := FProjectDirectory + PathDelim + Debadge(S);
        DeleteFile(FnOut); // remove any previous attempt
        S := ExtractFileExt(Fn);
        if (S = '.ico') or (S = '.identcache') or (S = '.res') then
          CopyFile(PWideChar(Fn), PWideChar(FnOut), False)
        else
        begin
          SLIn.Clear;
          SLOut.Clear;
          if (S = '.dfm') then
          begin
            FIn := TFileStream.Create(Fn, fmOpenRead or fmShareDenyNone);
            MIn := TMemoryStream.Create;
            try
              try
                ObjectResourceToText(FIn, MIn); // throws error if already text
                MIn.Position := 0;
                SLIn.LoadFromStream(MIn);
              except
                SLIn.LoadFromFile(Fn);
              end;
            finally
              Min.Free;
              FIn.Free;
            end;
          end
          else SLIn.LoadFromFile(Fn);
          for J := 0 to SLIn.Count - 1 do
            SLOut.Add(DeBadge(SLIn[J]));
          FileClose(FileCreate(FnOut)); // create an empty file
          SLOut.SaveToFile(FnOut);
        end;
      end;
      MemoFilesOut.Lines.Add(FnOut + ' saved OK');
      Application.ProcessMessages;
    end;
  finally
    SLIn.Free;
    SLOut.Free;
  end;
  ButtonFinish.Caption := '&Finish';
end;

procedure TFormConvertAll.FormShow(Sender: TObject);
begin
  Convert;
end;


end.
