unit MainForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, JvBaseDlg, JvSelectDirectory, JvExStdCtrls,
  JvListBox, JvDriveCtrls, LKBaseComponent;

type
  TFormMain = class(TForm)
    Panel1: TPanel;
    Splitter1: TSplitter;
    ButtonNext: TButton;
    ButtonClose: TButton;
    LKHeaderPanel1: TLKHeaderPanel;
    JvDirectoryListBox1: TJvDirectoryListBox;
    procedure ButtonNextClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure JvDirectoryListBox1Change(Sender: TObject);
    procedure ButtonCloseClick(Sender: TObject);
  private
    FSelectedPath: string;
    FBasePath: string;
    procedure SetSelectedPath(const Value: string);
    property SelectedPath: string read FSelectedPath write SetSelectedPath;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  FormMain: TFormMain;

const DefaultComment = 'Please select one of the Zoom project folders on which '
  + 'to base your Accounts program. ZoomAccounts is a core accounting system.'
  + ' Other folders contain industry specific systems.';

implementation

uses ConvertAllDlg, TargetAccountsDlg;

{$R *.dfm}

procedure TFormMain.ButtonCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TFormMain.ButtonNextClick(Sender: TObject);
label again;
begin
  ButtonNext.Enabled := False;
  again:
  if (FormTargetAccounts.ShowModal = mrOK) then
  with FormConvertAll do
  begin
    Alias := FormTargetAccounts.EditAlias.Text;
    AccountsYear := FormTargetAccounts.EditYear.Text;
    ConvertDirectory := JvDirectoryListBox1.Directory;
    case ShowModal of
      mrRetry : goto again;
    end;
  end;
end;

procedure TFormMain.FormShow(Sender: TObject);
begin
  FBasePath := ExtractFilePath(Application.ExeName);
  JvDirectoryListBox1.Directory := FBasePath;
end;

procedure TFormMain.JvDirectoryListBox1Change(Sender: TObject);
begin
  SelectedPath := JvDirectoryListBox1.Directory;
end;

procedure TFormMain.SetSelectedPath(const Value: string);
begin
  if (Value = FSelectedPath) then
    Exit;
  ButtonNext.Enabled := (Pos(FBasePath+'Zoom', Value) = 1);
  if not ButtonNext.Enabled then
  begin
    LKHeaderPanel1.Comments.Text := DefaultComment;
    Exit;
  end;
  FSelectedPath := Value;
  if FileExists(FSelectedPath + PathDelim + 'SYNOPSIS.TXT') then
    LKHeaderPanel1.Comments.LoadfromFile(
      FSelectedPath + PathDelim + 'SYNOPSIS.TXT')
  else
    LKHeaderPanel1.Comments.Text := DefaultComment;
end;

end.
