{*****************************************************************************}
{                                                                             }
{   This file is part of the ZoomGliding program executable                   }
{                                                                             }
{   See the file COPYING.GPL.txt included in this distribution,               }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Main Form                                                              }
{                                                                             }
{*****************************************************************************}

unit MainForm;

interface

uses
  Windows, Messages, SysUtils, Classes, System.UITypes,
  Graphics, Controls, Forms, Dialogs,
  Buttons, ToolWin, ComCtrls, StdCtrls, ActnList, Menus, ImgList, iniFiles,
    variants,
  DBTables, registry, ExtCtrls, 
  ActnMan, ActnCtrls, CustomizeDlg, DBGrids, DBCtrls, Grids,
  Calendar, DB, XPMan, XPStyleActnCtrls, WinHelpViewer,
  JvExControls, JvComponent,
  JvOutlookBar, JvExExtCtrls, JvNetscapeSplitter,
  LKRecordsComponents, System.Actions;

type
  TFormZoomAccountsMain = class(TForm)
    MainImageList: TImageList;
    MainMenu: TMainMenu;
    Tools1: TMenuItem;
    miOptions: TMenuItem;
    Folders1: TMenuItem;
    CustomizeDlg1: TCustomizeDlg;
    PopupMenu1: TPopupMenu;
    puEdit: TMenuItem;
    EditToolBar1: TMenuItem;
    N4: TMenuItem;
    DrilldownonDiary1: TMenuItem;
    XPManifest1: TXPManifest;
    ActionManagerTools: TActionManager;
    PopupMenuTools: TPopupMenu;
    MenuItem1: TMenuItem;
    CustomizeDlgTools: TCustomizeDlg;
    ActionExit: TAction;
    ActionBackup: TAction;
    ActionEndOfMonth: TAction;
    ActionDiary: TAction;
    ActionHubEnterpriseSettings: TAction;
    ActionHubDatabaseSettings: TAction;
    ActionFullSetup: TAction;
    ActionViewSystemSettings: TAction;
    ActionOnScreen: TAction;
    ActionOffScreen: TAction;
    ActionHelpContents: TAction;
    ActionEndOfYear: TAction;
    ActionRestoreData: TAction;
    ActionRestoreRemovable: TAction;
    Panel1: TPanel;
    miFile: TMenuItem;
    Backup1: TMenuItem;
    miRestore: TMenuItem;
    N2: TMenuItem;
    Exit1: TMenuItem;
    Panel3: TPanel;
    ActionToolBarTools: TActionToolBar;
    pShow: TPanel;
    pACTools: TPanel;
    ToolBarQuickLaunch: TActionToolBar;
    pmOutlookBar: TPopupMenu;
    JvNetscapeSplitter1: TJvNetscapeSplitter;
    miWindow: TMenuItem;
    Help1: TMenuItem;
    Movetoside1: TMenuItem;
    N3: TMenuItem;
    Enterprisesettings1: TMenuItem;
    miView: TMenuItem;
    miSystem: TMenuItem;
    aHelpContents1: TMenuItem;
    EndofYear1: TMenuItem;
    miSettings: TMenuItem;
    ActionUsersPass: TAction;
    Usersandpasswords1: TMenuItem;
    ActionAdministratorPass: TAction;
    Administrator1: TMenuItem;
    Panel2: TPanel;
    lEnterprise: TLabel;
    Label2: TLabel;
    cbEnterprise: TComboBox;
    cbACYears: TComboBox;
    Restore2: TMenuItem;
    RestoreRemovabledisk1: TMenuItem;
    ActionManager1: TActionManager;
    JvOutlookBar1: TJvOutlookBar;
    ActionSQLEditor: TAction;
    SQLEditor1: TMenuItem;
    N1: TMenuItem;
    About1: TMenuItem;
    ActionHelpAbout: TAction;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure ActionCloseExecute(Sender: TObject);
    procedure ActionEndOfMonthExecute(Sender: TObject);
    procedure ActionBackupExecute(Sender: TObject);
    procedure ActionRestoreDataExecute(Sender: TObject);
    procedure ActionToolBarEditExecute(Sender: TObject);
    procedure ActionOnScreenExecute(Sender: TObject);
    procedure ActionOffScreenExecute(Sender: TObject);
    procedure ActionDiaryExecute(Sender: TObject);
    procedure ActionHubDatabaseSettingsExecute(Sender: TObject);
    procedure ActionHubEnterpriseSettingsExecute(Sender: TObject);
    procedure ActionFullSetupExecute(Sender: TObject);
    procedure ActionHelpContentsExecute(Sender: TObject);
    procedure ActionEndOfYearExecute(Sender: TObject);
    procedure ActionUsersPassExecute(Sender: TObject);
    procedure ActionAdministratorPassExecute(Sender: TObject);
    procedure ActionViewSystemSettingsExecute(Sender: TObject);
    procedure CustomizeDlg1Close(Sender: TObject);
    procedure pShowClick(Sender: TObject);
    procedure JvOutlookBar1ButtonClick(Sender: TObject; Index: Integer);
    procedure cbEnterpriseChange(Sender: TObject);
    procedure cbACYearsChange(Sender: TObject);
    procedure ActionRestoreRemovableExecute(Sender: TObject);
    procedure ActionSQLEditorExecute(Sender: TObject);
    procedure ActionHelpAboutExecute(Sender: TObject);
  private
    { Private declarations }
  protected
//    procedure WMSyscommand(var Message: TWmSysCommand); message WM_SYSCOMMAND;
  public
    { Public declarations }
//    procedure CreateParams(var Params: TCreateParams); override;
  end;

var
  FormZoomAccountsMain: TFormZoomAccountsMain;

implementation

{$R *.DFM}
uses DatamodZoom, AboutDlg, LKDefine;

procedure TFormZoomAccountsMain.FormShow(Sender: TObject);
begin
  if Screen.Width < 799 then
    if (MessageDlg('You are recommended to use a screen resolution' + #10 +
      'of not less than 800 x 600 with this program.' + #10 + #10 +
      'Do you want to exit to change (right-click desktop' + #10 +
      'and select properties|settings|desktop area)?',
      mtWarning, [mbYes, mbNo], 0) = mrYes) then
        Halt;
  with DatamoduleZoom do
  begin
    ActionManager1.Images := DMImageList;
    JvOutlookBar1.LargeImages := DMImageList;
    //JvOutlookBar1.PageImages := DMImageList; looks a bit silly
    Hub.AddToActionList(ActionManager1);
    ToolBarQuickLaunch.ActionManager := ActionManager1;
    CustomizeDlg1.ActionManager := ActionManager1;
    with ActionManager1 do
    begin
      ActionBars[0].ActionBar := ToolBarQuickLaunch;
      FileName := ChangeFileExt(Application.ExeName, '.cfg');
      if FileExists(FileName) then
        LoadFromFile(FileName);
    end;
    with Hub do
    begin
      // WindowMenu := miWindow;
      AddToOutlookBar(JvOutlookBar1);
      JvOutlookBar1.ActivePageIndex := 0;
      Setup(False); //Call GetSettings and open
      if (DlgPasswordExecute <> mrOK) then
      begin
        Application.Terminate;
        Exit;
      end;
      Caption := 'Year ending ' + EnterpriseAccountsYearCurrent;
      ActionAdministratorPass.Visible := not (DBMS = sdbLocalFirebird);
      miSettings.Visible := (DBMS = sdbLocalFirebird);
      ActionViewSystemSettings.Visible := (DBMS = sdbLocalFirebird);
      ActionSQLEditor.Visible := (DBMS = sdbLocalFirebird);
      ActionEndOfMonth.Visible := HasPeriodEndProcedure;
      ActionEndOfYear.Visible := HasPeriodEndProcedure;
      miRestore.Visible := (DBMS = sdbLocalFirebird);
      ActionBackup.Visible := (DBMS = sdbLocalFirebird);
      ActionRestoreData.Visible := (DBMS = sdbLocalFirebird);
      ActionRestoreRemovable.Visible := (DBMS = sdbLocalFirebird);
      with cbEnterprise do
      begin
        Items.AddStrings(Enterprises);
        ItemIndex := Items.IndexOf(EnterpriseCurrent);
        Visible := True;
      end;
      with cbACYears do
      begin
        Items.AddStrings(Hub.EnterpriseYears);
        ItemIndex := Items.IndexOf(EnterpriseAccountsYearCurrent);
      end;
    end;
    acgMain.Open;
  end;
  Height := Screen.WorkAreaHeight;
  Top := 0;
  Left := 0;
end;

procedure TFormZoomAccountsMain.FormActivate(Sender: TObject);
begin
  Height := Screen.WorkAreaHeight;
  Top := 0;
  Left := 0;
end;

procedure TFormZoomAccountsMain.ActionCloseExecute(Sender: TObject);
begin
  Close;
end;

procedure TFormZoomAccountsMain.ActionHelpAboutExecute(Sender: TObject);
begin
  with TAboutBox.Create(Self) do
  try
    ShowModal;
  finally Free;
  end;
end;

// ******** Analysis **********

procedure TFormZoomAccountsMain.ActionToolBarEditExecute(Sender: TObject);
begin
  CustomizeDlg1.Show;
end;
{
procedure TFormZoomAccountsMain.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.ExStyle := Params.ExStyle and not WS_EX_TOOLWINDOW or
    WS_EX_APPWINDOW;
end;
}

procedure TFormZoomAccountsMain.CustomizeDlg1Close(Sender: TObject);
begin
  with ActionManager1 do
  begin
    FileName := ChangeFileExt(Application.ExeName, '.cfg');
    SaveToFile(FileName);
  end;
end;

procedure TFormZoomAccountsMain.ActionEndOfMonthExecute(Sender: TObject);
begin
  DatamoduleZoom.Hub.EndOfMonth;
end;

procedure TFormZoomAccountsMain.ActionBackupExecute(Sender: TObject);
begin
  DatamoduleZoom.Hub.FilesBackup(True);
  DatamoduleZoom.Hub.Close;
  Close;
end;

procedure TFormZoomAccountsMain.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  DatamoduleZoom.Hub.FilesCheckForBackup;
  DatamoduleZoom.Hub.Close;
  Application.ProcessMessages;
end;

procedure TFormZoomAccountsMain.ActionRestoreDataExecute(Sender: TObject);
begin
  DatamoduleZoom.Hub.FilesRestore;
end;

procedure TFormZoomAccountsMain.ActionRestoreRemovableExecute(Sender: TObject);
begin
  DatamoduleZoom.Hub.FilesRestoreRemovable;
end;

procedure TFormZoomAccountsMain.pShowClick(Sender: TObject);
begin
  if (pShow.Caption = '>') then
    ActionOnScreenExecute(Sender)
  else
    ActionOffScreenExecute(Sender);
end;
{
procedure TFormZoomAccountsMain.WMSyscommand(var Message: TWmSysCommand);
begin
  case (Message.CmdType and $FFF0) of
    SC_MINIMIZE:
      begin
        ShowWindow(Handle, SW_MINIMIZE);
        Message.Result := 0;
      end;
    SC_RESTORE:
      begin
        ShowWindow(Handle, SW_RESTORE);
        Message.Result := 0;
      end;
  else
    inherited;
  end;
end;
}
procedure TFormZoomAccountsMain.ActionOnScreenExecute(Sender: TObject);
var
  i: integer;
begin
  pShow.Enabled := false;
  Top := 0;
  Height := Screen.Height;
  for i := 12 - Width to 0 do
    Left := i;
  pShow.Caption := '<';
  pShow.Enabled := True;
end;

procedure TFormZoomAccountsMain.ActionOffScreenExecute(Sender: TObject);
var
  I: integer;
begin
  pShow.Enabled := False;
  Top := 0;
  Height := Screen.Height;
  for I := 0 downto - (Margins.Left + pShow.Left) do
    Left := I;
  pShow.Caption := '>';
  pShow.Enabled := True;
end;

procedure TFormZoomAccountsMain.ActionDiaryExecute(Sender: TObject);
begin
  DatamoduleZoom.Hub.Diary.View(null, True, [mbOK]);
end;

procedure TFormZoomAccountsMain.JvOutlookBar1ButtonClick(Sender: TObject; Index:
  Integer);
var
  g, i: integer;
  myGroup: TLKGroup;
begin
  with JvOutlookBar1 do
  begin
    g := ActivePageIndex;
    Pages[g].Buttons[Index].Enabled := false; // avoid double click
    i := Pages[ActivePageIndex].Buttons[Index].Tag;
  end;
  try
    myGroup := TLKGroup(DatamoduleZoom.Hub.Groups[g]);
    with TLKSystemComponent(myGroup.GroupComponents[i]) do
    begin
      DlgViewExecute;
//      FormView.WindowState := wsMaximized;
    end;
  finally JvOutlookBar1.Pages[g].Buttons[Index].Enabled := True;
  end;
end;

procedure TFormZoomAccountsMain.ActionHubDatabaseSettingsExecute(
  Sender: TObject);
begin
  with DatamoduleZoom.Hub do
  begin
    DlgDatabaseExecute;
    with cbEnterprise do
    begin
      Items.Clear;
      Items.AddStrings(Enterprises);
      ItemIndex := Items.IndexOf(EnterpriseCurrent);
    end;
    with DatamoduleZoom.Hub, cbACYears do
    begin
      Items.Clear;
      Items.AddStrings(EnterpriseYears);
      ItemIndex := Items.IndexOf(EnterpriseAccountsYearCurrent);
    end;
  end;
end;

procedure TFormZoomAccountsMain.ActionFullSetupExecute(Sender: TObject);
begin
  DatamoduleZoom.Hub.DlgSetupWizardExecute;
end;

procedure TFormZoomAccountsMain.ActionHubEnterpriseSettingsExecute(
  Sender: TObject);
begin
  DatamoduleZoom.Hub.DlgSetupExecute;
end;

procedure TFormZoomAccountsMain.ActionViewSystemSettingsExecute(Sender: TObject);
begin
  DatamoduleZoom.Hub.SystemSettings.DlgViewExecute;
end;

procedure TFormZoomAccountsMain.ActionSQLEditorExecute(Sender: TObject);
begin
  DatamoduleZoom.Hub.DlgSQLEditorExecute;
end;

procedure TFormZoomAccountsMain.FormCreate(Sender: TObject);
begin
//  SetVistaFonts(Self);
{
  ShowWindow(Application.Handle, SW_HIDE);
  SetWindowLong(Application.Handle, GWL_EXSTYLE,
    GetWindowLong(Application.Handle, GWL_EXSTYLE) and not WS_EX_APPWINDOW
    or WS_EX_TOOLWINDOW);
  ShowWindow(Application.Handle, SW_SHOW);
  }

//  Application.HelpFile := ChangeFileExt(Application.ExeName, '.hlp');
//  HelpFile := Application.HelpFile;
end;

procedure TFormZoomAccountsMain.ActionHelpContentsExecute(Sender: TObject);
begin
  Application.HelpCommand(HELP_FINDER, 0);
end;

procedure TFormZoomAccountsMain.ActionEndOfYearExecute(Sender: TObject);
var S: string;
begin
  with DatamoduleZoom.Hub do
  begin
    EndOfYear(S);
    with cbEnterprise do
    begin
      Items.Clear;
      Items.AddStrings(Enterprises);
      ItemIndex := Items.IndexOf(EnterpriseCurrent);
    end;
    with cbACYears do
    begin
      Items.Clear;
      Items.AddStrings(EnterpriseYears);
      ItemIndex := Items.IndexOf(EnterpriseAccountsYearCurrent);
    end;
  end;
end;

procedure TFormZoomAccountsMain.ActionUsersPassExecute(Sender: TObject);
begin
  DatamoduleZoom.Hub.DlgUsersSetupExecute;
end;

procedure TFormZoomAccountsMain.ActionAdministratorPassExecute(Sender: TObject);
begin
  if (DatamoduleZoom.Hub.DlgAdminPasswordExecute = mrOK) then
  begin
    miSettings.Visible := True;
    ActionViewSystemSettings.Visible := True;
    ActionSQLEditor.Visible := True;
    ActionAdministratorPass.Visible := false;
    Showmessage('Administrator menu options are now available');
  end;
end;

procedure TFormZoomAccountsMain.cbEnterpriseChange(Sender: TObject);
begin
  Screen.Cursor := crHourglass;
  {
   EnterpriseCurrent := cbxEnterpriseCodes.Text;
   if cbEnterpriseDefault.Checked then EnterpriseDefault := EnterpriseCurrent;
   with cbxEnterpriseYears do
    begin
    if (Length(Text) = 4) and (Items.IndexOf(Text) < 0) then Items.Add(Text);
    EnterpriseYears := Items;
    end;
   EnterpriseAccountsYearCurrent := cbxEnterpriseYears.Text;
   if cbEnterpriseYearDefault.Checked then
    EnterpriseAccountsYearDefault := EnterpriseAccountsYearCurrent;
   SingleUser := cbSingleUser.Checked;
   DatabaseHost := eDatabaseHost.Text;
   DBUserName := eUsername.Text;
   DBUserPassword := eDatabasePassword.Text;
   CheckDatabaseExists(True);
     }
  try
    with DatamoduleZoom.Hub do
    begin
      with cbEnterprise do
      begin
        // if (ItemIndex < 0) then Exit;
        if (Text = EnterpriseCurrent) then
          exit;
        EnterpriseCurrent := Text;
      end;
      with cbACYears do
      begin
        Items.Clear;
        Items.AddStrings(EnterpriseYears);
        ItemIndex :=
          Items.IndexOf(EnterpriseAccountsYearCurrent);
      end;
    end;
  finally Screen.Cursor := crDefault;
  end;
  cbACYearsChange(Sender);
end;

procedure TFormZoomAccountsMain.cbACYearsChange(Sender: TObject);
begin
  Screen.Cursor := crHourglass;
  try
    DatamoduleZoom.Hub.Close;
    with cbACYears do
    begin
      // if (ItemIndex < 0) then Exit;
      DatamoduleZoom.Hub.EnterpriseAccountsYearCurrent := Text;
    end;
  finally Screen.Cursor := crDefault;
  end;
  Caption := 'Year ending '
    + DatamoduleZoom.Hub.EnterpriseAccountsYearCurrent;
  with DatamoduleZoom.Hub do
  begin
    DatabaseHost := DatabaseHost;
    DBUserName := DBUserName;
    DBUserPassword := DBUserPassword;
    CheckDatabaseExists;
    Close;
    Open;
  end;
  DatamoduleZoom.acgMain.Open;
end;

initialization
{$WARN SYMBOL_PLATFORM OFF}
  ReportMemoryLeaksOnShutdown := False {DebugHook <> 0};
 //  ReportMemoryLeaksOnShutdown := True {DebugHook <> 0};
{$WARN SYMBOL_PLATFORM ON}

end.

