{*****************************************************************************}
{                                                                             }
{   This file is part of the ZoomGliding program executable                   }
{                                                                             }
{   See the file COPYING.GPL.txt included in this distribution,               }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      Data module                                                            }
{                                                                             }
{*****************************************************************************}

unit DataModZoom;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  variants,
  LKDefine, LKRecordsComponents, ZoomGlidingComponents,
  LKBaseComponent, DBClient, WideStrings, LKAccountsComponents,
  LKCRMBaseComponents, DB, SqlExpr, LKGlidingComponents, FMTBcd, Provider,
  DBXFirebird, ImgList, ActnList;

type
  TDatamoduleZoom = class(TDataModule)
    FDBConnection: TSQLConnection;
    Hub: TLKHubAccounts;
    acgMain: TLKGroup;
    LedgerNominal: TZoomGlidingLedgerNominal;
    LedgerCost: TZoomGlidingLedgerCost;
    JournalDayBook: TZoomGlidingDayBookJournal;
    Journal: TZoomGlidingJournal;
    VATReturns: TZoomGlidingSalesTaxReturns;
    acgSales: TLKGroup;
    LedgerSales: TZoomGlidingLedgerSales;
    acgPurchases: TLKGroup;
    LedgerPurchase: TZoomGlidingLedgerPurchase;
    DaybookPurchase: TZoomGlidingDaybookPurchase;
    acgBank: TLKGroup;
    LedgerBank: TZoomGlidingLedgerBank;
    DaybookBankings: TZoomGlidingDaybookBank;
    DaybookBankPayments: TZoomGlidingDaybookBank;
    PurchInvoices: TZoomGlidingDocumentsPurchaseInvoices;
    Bankings: TZoomGlidingDocumentsBankBook;
    BankPayments: TZoomGlidingDocumentsBankBook;
    DaybookCheques: TZoomGlidingDaybookBank;
    Cheques: TZoomGlidingDocumentsBankBook;
    PriceLists: TZoomGlidingPriceLists;
    acgMembers: TLKGroup;
    ClubMembers: TZoomGlidingClubMembers;
    SalesInvoices: TZoomGlidingDocumentsSalesInvoices;
    GliderLogs: TZoomGlidingLedgerLogGlider;
    TugLogs: TZoomGlidingLedgerLogTug;
    GliderPilotLogs: TZoomGlidingLedgerLogPilotGlider;
    TugPilotLogs: TZoomGlidingLedgerLogPilotTug;
    GlidingLogDaybookAerotow: TZoomGlidingDaybookLogAerotow;
    GlidingLogAerotow: TZoomGlidingLogAerotow;
    DaybookSales: TZoomGlidingDaybookSales;
    DMImageList: TImageList;
    procedure GlidingLogAerotowInsertLogItemSalesInvoiceItems(
      Sender: TObject);
    procedure GlidingLogAerotowJournalAnalysisAdjustments(Sender: TObject);
    procedure DataModuleCreate(Sender: TObject);
  private
    { Private declarations }
    CostAdjPriceListSelected: boolean;
  public
    { Public declarations }
  end;

var
  DatamoduleZoom: TDatamoduleZoom;

implementation

uses MainForm, LKUtils, LKUtilsDB;

{$R *.DFM}

procedure TDatamoduleZoom.DataModuleCreate(Sender: TObject);
var
  Recent, W: string; //, current
  D,M,Y: Word;
begin

  DecodeDate((Date - 760), Y, M, D);
  Recent := Chr(39)+IntToStr(M)+'/'+IntToStr(D)+'/'+IntToStr(Y)+Chr(39);
 // Recent := Chr(39) + DateToStr(Date - 760) + Chr(39);

//    Chr(39) + StringReplace(DateToStr(Date - 760), '/', '.', [rfReplaceAll]) +
//      Chr(39);
  //current := chr(39) + DateToStr(Date-30) + chr(39);
  // if (DateDelim <> '/') then begin
  // while (pos('/',current) > 0) do current[pos('/',current)] :=  char('.');//DateDelim;
  // end;
  W := 'BATCH IS NULL OR DATE_ > ' + Recent;
  Journal.TblMaster.WhereClause := W;
  GlidingLogAerotow.TblMaster.WhereClause := W;
  SalesInvoices.TblMaster.WhereClause := W;
  PurchInvoices.TblMaster.WhereClause := W;
  Bankings.TblMaster.WhereClause := W;
  Cheques.TblMaster.WhereClause := W;

  //Journal.Filtered := True;
  CostAdjPriceListSelected := False;

end;

procedure TDatamoduleZoom.GlidingLogAerotowInsertLogItemSalesInvoiceItems(
  Sender: TObject);
var
  CusRef, Doc, Pilot, Glider, Tug, Voucher, Pricing: string;
  Log, Item, Minutes, Height, HeightExtra: Integer;

  procedure AddPrice(const APricecode, ACostCentre: string; AQty: double);
  var
    Plist: string;
  begin
    if (AQty = 0) then
      Exit;
    with PriceLists.TblDetail do
    begin
      if not Locate('PRICE_CODE', APricecode, []) then
        Exit;
      Plist := FieldByName('PRICE_LIST').AsString;
    end;
    with SalesInvoices.TblDetail do
    begin
      AppendRecord([nil, Item, Plist, APricecode, nil, ACostCentre, Doc, Log,
        nil, nil, AQty]);
      Edit;
      FieldByName('DESCRIPTION').AsString := FieldByName('DESCRIPTION').asString
        + ' ' + Pilot + ' ' + Voucher;
      Post;
    end;
    Inc(Item);
  end;

begin
  with SalesInvoices.TblDetail do
    if FindLast then
      Item := GetInteger(FieldByName('ITEM_NUMBER')) + 1
    else
      Item := 1;
  with GlidingLogAerotow.TblDetail do
  begin
    Doc := FieldByName('LOG_REFERENCE').AsString;
    Log := GetInteger(FieldByName('ITEM_NUMBER'));
    CusRef := FieldByName('CUSTOMER_ACCOUNT').AsString;
    Pilot := FieldByName('PILOT_NAME').AsString;
    Glider := FieldByName('GLIDER_CALLSIGN').AsString;
    Tug := FieldByName('TUG').AsString;
    Voucher := FieldByName('VOUCHER_NO').AsString;
    Pricing := FieldByName('PRICING').AsString;
    Minutes := Round(GetFloat(FieldByName('DURATION')) * 1440);
      // minutes in a day
    Height := GetInteger(FieldByName('RELEASE_HEIGHT'));
    HeightExtra := Height - 2000;
    if (HeightExtra < 0) then
      HeightExtra := 0
    else
      Height := 2000;
  end;
  if (Pricing = 'No charge') then
  begin
    SalesInvoices.Delete;
    Exit;
  end;
  with SalesInvoices.TblMaster do
  begin
    Edit;
    FieldByName('INVOICE_IMAGE').AsString :=
      GlidingLogAerotow.TblMaster.FieldByName('LOG_IMAGE').asString;
    Post;
  end;
  if (Pricing = 'Club rates') then
  begin
    if (Height > 0) then
    begin
      AddPrice('AT', Tug, 1);
      AddPrice('AB', Tug, Height / 100);
      AddPrice('AH', Tug, HeightExtra / 100);
    end;
    if (Glider > '') then
      AddPrice('G' + Glider, Glider, Minutes);
  end;
  if (Pricing = 'Training circuit') then
    AddPrice('AL', Glider, 1);
  if (Pricing = 'Trial lesson') then
    AddPrice('LO', Glider, 1);
  if (Pricing = 'Trial lesson BGA') then
    AddPrice('BGA', Glider, 1);
  if (Pricing = 'Trial lesson Agent') then
    AddPrice('LT', Glider, 1);
  if (Pricing = 'Trial lesson 5000') then
    AddPrice('LM', Glider, 1);
  if (Pricing = '2 flight T/Lesson') then
    AddPrice('LE', Glider, 1);
  if (Pricing = '2 flight T/L Agent') then
    AddPrice('LV', Glider, 1);
  if (Pricing = '1 Day course') then
    AddPrice('LC', Glider, 1);
  if (Pricing = '1 Day course Agent') then
    AddPrice('LU', Glider, 1);
end;

procedure TDatamoduleZoom.GlidingLogAerotowJournalAnalysisAdjustments(
  Sender: TObject);
var
  JnlRef, cusRef, Doc, Log, Pilot, Glider, Tug, Voucher, Pricing: string;
  Item, Minutes, Height: Integer;
  JnlDate: TDateTime;
  IsNew: boolean;

  procedure CostAdjust;
  var
    N, D: string;
    L: currency;
  begin
    if (Height > 0) then
    begin
      L := 0;
      with PriceLists.TblDetail do
      begin
        if Locate('PRICE_CODE', 'AT', []) then
        begin
          L := GetCurrency(FieldByName('UNIT_PRICE'));
          N := FieldByName('NOMINAL_CODE').asString;
          D := FieldByName('DESCRIPTION').asString;
          if Locate('PRICE_CODE', 'AH', []) then
          begin
            L := L + (GetCurrency(FieldByName('UNIT_PRICE')) * Height / 100);
            D := D + ' ' + FieldByName('DESCRIPTION').asString;
          end;
        end;
      end;
      D := Doc + '/' + Log + ' ' + Pilot + ' ' + D;
      RoundCents(L);
      if not CurrencyIsZero(L) then
        with Journal do
        begin
          OkToInsert := true;
          try
            TblDetail.AppendRecord([JnlRef, Item, N, '', null, null, D,
              DebitOrNull(L), CreditOrNull(L)]);
            inc(Item);
            L := -L;
            TblDetail.AppendRecord([JnlRef, Item, N, Tug, null, null, D,
              DebitOrNull(L), CreditOrNull(L)]);
            Inc(Item);
          finally OkToInsert := false;
          end;
        end;
    end;
    if (Glider > '') and (Minutes > 0) then
    begin
      with PriceLists.TblDetail do
      begin
        if Locate('PRICE_CODE', 'G' + Glider, []) then
        begin
          L := GetCurrency(FieldByName('UNIT_PRICE'));
          N := FieldByName('NOMINAL_CODE').asString;
          D := FieldByName('DESCRIPTION').asString;
        end
        else
          L := 0;
      end;
      L := RoundCents(L * Minutes);
      D := Doc + '/' + Log + ' ' + Pilot + ' ' + D;
      if not CurrencyIsZero(L) then
        with Journal do
        begin
          OkToInsert := true;
          try
            TblDetail.AppendRecord([JnlRef, Item, N, '', null, null, D,
              DebitOrNull(L), CreditOrNull(L)]);
            inc(Item);
            L := -L;
            TblDetail.AppendRecord([JnlRef, Item, N, Glider, null, null, D,
              DebitOrNull(L), CreditOrNull(L)]);
            inc(Item);
          finally OkToInsert := false;
          end;
        end;
    end;
  end;

begin
  if not CostAdjPriceListSelected then
  begin
    with PriceLists do
      case RecordCount of
        0:
          begin
            Showmessage('No prices entered in Price Lists!');
            Exit;
          end;
        1: ;
      else
        begin
          Showmessage('Select price list');
          View(null, true, [mbOK]);
        end;
      end;
    CostAdjPriceListSelected := true;
  end;
  JnlDate := GlidingLogAerotow.tmDocumentDate;
  with Journal do
  begin
    Filter := '';
    Filtered := false;
    isNew := not Locate('DATE_;DESCRIPTION',
      VarArrayOf([jnldate, 'TRIAL FLIGHT COSTING ADJUSTMENTS']), []);
    if isNew then
    begin
      JnlRef := IntToStr(SequenceNumberNext);
      InsertRecord([JnlRef, null, JnlDate, 'TRIAL FLIGHT COSTING ADJUSTMENTS',
        'AUTO']);
    end;
    JnlRef := TblMaster.Fields[0].AsString;
    with TblDetail do
      if FindLast then
        item := GetInteger(FieldByName('ITEM_NUMBER')) + 1
      else
        item := 1;
  end;
  if IsNew then
    with GlidingLogAerotow.TblDetail do
      if FindFirst then
        repeat
          Pricing := FieldByName('PRICING').AsString;
          if not ((Pricing = 'Club rates') or (Pricing = 'No charge')) then
          begin
            Doc := FieldByName('LOG_REFERENCE').AsString;
            Log := FieldByName('ITEM_NUMBER').AsString;
            cusRef := FieldByName('CUSTOMER_ACCOUNT').AsString;
            Pilot := FieldByName('PILOT_NAME').AsString;
            Glider := FieldByName('GLIDER_CALLSIGN').AsString;
            Tug := FieldByName('TUG').AsString;
            Voucher := FieldByName('VOUCHER_NO').AsString;
            Minutes := Round(GetFloat(FieldByName('DURATION')) * 1440);
              // minutes in a day
            Height := GetInteger(FieldByName('RELEASE_HEIGHT'));
            CostAdjust;
          end;
        until not findNext;
  with Hub.Journal do
  begin
    View(Null, False, []);
    ViewRecord(Null, True, [mbOK]);
  end;
end;

end.

