DECLARE EXTERNAL FUNCTION RDB$GET_CONTEXT
    VARCHAR(80) NULL,
    VARCHAR(80) NULL
    RETURNS VARCHAR(255) FREE_IT
    ENTRY_POINT 'get_context' MODULE_NAME 'system_module';

DECLARE EXTERNAL FUNCTION RDB$SET_CONTEXT
    VARCHAR(80) NULL,
    VARCHAR(80) NULL,
    VARCHAR(255) NULL
    RETURNS INTEGER BY VALUE
    ENTRY_POINT 'set_context' MODULE_NAME 'system_module';

CREATE GENERATOR ANALYSIS_LEDGER_DEFERRED_GEN;

SET GENERATOR ANALYSIS_LEDGER_DEFERRED_GEN TO 0;

CREATE GENERATOR ANALYSIS_LEDGER_DTL_GEN;

SET GENERATOR ANALYSIS_LEDGER_DTL_GEN TO 0;

CREATE GENERATOR BANK_LEDGER_DTL_GEN;

SET GENERATOR BANK_LEDGER_DTL_GEN TO 0;

CREATE GENERATOR GLIDER_LOGS_GEN;

SET GENERATOR GLIDER_LOGS_GEN TO 0;

CREATE GENERATOR PILOT_LOG_GLIDERS_GEN;

SET GENERATOR PILOT_LOG_GLIDERS_GEN TO 0;

CREATE GENERATOR PILOT_LOG_TUGS_GEN;

SET GENERATOR PILOT_LOG_TUGS_GEN TO 0;

CREATE GENERATOR PURCHASE_LEDGER_DTL_GEN;

SET GENERATOR PURCHASE_LEDGER_DTL_GEN TO 0;

CREATE GENERATOR SALES_LEDGER_DTL_GEN;

SET GENERATOR SALES_LEDGER_DTL_GEN TO 0;

CREATE GENERATOR TUG_LOGS_GEN;

SET GENERATOR TUG_LOGS_GEN TO 0;

CREATE TABLE ACCOUNTS_PERIODS 
(
  PERIOD_END                 DATE         DEFAULT 'TODAY' NOT NULL,
 CONSTRAINT PK_ACCOUNTS_PERIODS PRIMARY KEY (PERIOD_END)
);

CREATE TABLE ANALYSIS_LEDGER_DEFERRED 
(
  URN                           BIGINT         NOT NULL,
  NOMINAL_CODE                 VARCHAR(     8)  COLLATE ISO8859_1,
  COST_CENTRE                  VARCHAR(     8)  COLLATE ISO8859_1,
  DATE_                           DATE         DEFAULT NULL,
  DAYBOOK_ID                   VARCHAR(     3) DEFAULT NULL COLLATE ISO8859_1,
  LEDGER_ACCOUNT               VARCHAR(     8)  COLLATE ISO8859_1,
  DOCUMENT_NUMBER              VARCHAR(    10) DEFAULT NULL COLLATE ISO8859_1,
  DESCRIPTION                  VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  DEBIT                        DECIMAL( 18, 4) DEFAULT NULL,
  CREDIT                       DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_ANALYSIS_LEDGER_DEFERRED PRIMARY KEY (URN)
);

CREATE ASC INDEX I_ANALYSIS_LEDGER_DEFERRED_CSNM ON ANALYSIS_LEDGER_DEFERRED (COST_CENTRE, NOMINAL_CODE, URN);

SET TERM ^^ ;
CREATE TRIGGER ANALYSIS_LEDGER_DEFERRED_ID FOR ANALYSIS_LEDGER_DEFERRED ACTIVE BEFORE INSERT POSITION 0 AS
begin
 new.URN = gen_id(ANALYSIS_LEDGER_DEFERRED_GEN, 1);
end ^^
SET TERM ; ^^

CREATE TABLE ANALYSIS_LEDGER_DTL 
(
  URN                           BIGINT         DEFAULT 0 NOT NULL,
  NOMINAL_CODE                 VARCHAR(     8)  COLLATE ISO8859_1,
  COST_CENTRE                  VARCHAR(     8)  COLLATE ISO8859_1,
  DATE_                           DATE         DEFAULT NULL,
  DAYBOOK_ID                   VARCHAR(     3) DEFAULT NULL COLLATE ISO8859_1,
  LEDGER_ACCOUNT               VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  DOCUMENT_NUMBER              VARCHAR(    10) DEFAULT NULL COLLATE ISO8859_1,
  DESCRIPTION                  VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  DEBIT                        DECIMAL( 18, 4) DEFAULT NULL,
  CREDIT                       DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_ANALYSIS_LEDGER_DTL PRIMARY KEY (URN)
);

CREATE ASC INDEX I_ANALYSIS_LEDGER_DTL_COSTNOM ON ANALYSIS_LEDGER_DTL (COST_CENTRE, NOMINAL_CODE, DATE_, URN);

CREATE ASC INDEX I_ANALYSIS_LEDGER_DTL_NOMDATE ON ANALYSIS_LEDGER_DTL (NOMINAL_CODE, DATE_, URN);

SET TERM ^^ ;
CREATE TRIGGER ANALYSIS_LEDGER_DTL_ID FOR ANALYSIS_LEDGER_DTL ACTIVE BEFORE INSERT POSITION 0 AS
begin
 new.URN = gen_id(ANALYSIS_LEDGER_DTL_GEN, 1);
end ^^
SET TERM ; ^^

CREATE TABLE BACS_PAYMENTS 
(
  BACS_PAYMENT_REFERENCE              VARCHAR(    10) NOT NULL COLLATE ISO8859_1,
  BATCH                               NUMERIC( 18, 0) DEFAULT NULL,
  BANK_ACCOUNT_REFERENCE              VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  DATE_                                  DATE         DEFAULT NULL,
  GOODS                               DECIMAL( 18, 4) DEFAULT NULL,
  VAT                                 DECIMAL( 18, 4) DEFAULT NULL,
  TOTAL                               DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_BACS_PAYMENTS PRIMARY KEY (BACS_PAYMENT_REFERENCE)
);

CREATE TABLE BACS_PAYMENTS_DTL 
(
  BACS_PAYMENT_REFERENCE              VARCHAR(    10) NOT NULL COLLATE ISO8859_1,
  ITEM_NUMBER                         INTEGER         DEFAULT 0 NOT NULL,
  NOMINAL_CODE                        VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  COST_CENTRE                         VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  LEDGER_ID                           VARCHAR(     3) DEFAULT NULL COLLATE ISO8859_1,
  LEDGER_REFERENCE                    VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  LEDGER_DOCUMENT                     VARCHAR(    10) DEFAULT NULL COLLATE ISO8859_1,
  DESCRIPTION                         VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  VAT_CODE                            INTEGER         DEFAULT 0,
  GOODS                               DECIMAL( 18, 4) DEFAULT NULL,
  VAT                                 DECIMAL( 18, 4) DEFAULT NULL,
  TOTAL                               DECIMAL( 18, 4) DEFAULT NULL,
  BANK_SORT_CODE                      VARCHAR(     6) DEFAULT NULL COLLATE ISO8859_1,
  BANK_ACCOUNT_NUMBER                 VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  ACCOUNT_NAME                        VARCHAR(    25) DEFAULT NULL COLLATE ISO8859_1,
  BUILDING_SOCIETY                    VARCHAR(    25) DEFAULT NULL COLLATE ISO8859_1,
  BS_ROLL_NUMBER                      VARCHAR(    25) DEFAULT NULL COLLATE ISO8859_1,
 CONSTRAINT PK_BACS_PAYMENTS_DTL PRIMARY KEY (BACS_PAYMENT_REFERENCE, ITEM_NUMBER)
);

CREATE TABLE BANKINGS 
(
  BANKING_REFERENCE                   VARCHAR(    10) NOT NULL COLLATE ISO8859_1,
  BATCH                               NUMERIC( 18, 0) DEFAULT NULL,
  VAT_PERIOD                          VARCHAR(     4) DEFAULT NULL COLLATE ISO8859_1,
  BANK_ACCOUNT_REFERENCE              VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  DATE_                                  DATE         DEFAULT NULL,
  DESCRIPTION                         VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  GOODS                               DECIMAL( 18, 4) DEFAULT NULL,
  VAT                                 DECIMAL( 18, 4) DEFAULT NULL,
  TOTAL                               DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_BANKINGS PRIMARY KEY (BANKING_REFERENCE)
);

CREATE ASC INDEX I_BANKINGS_BANKACREF ON BANKINGS (BANK_ACCOUNT_REFERENCE, DATE_, BANKING_REFERENCE);

CREATE TABLE BANKINGS_DTL 
(
  BANKING_REFERENCE              VARCHAR(    10) NOT NULL COLLATE ISO8859_1,
  ITEM_NUMBER                    INTEGER         DEFAULT 1 NOT NULL,
  NOMINAL_CODE                   VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  COST_CENTRE                    VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  LEDGER_ID                      VARCHAR(     3) DEFAULT NULL COLLATE ISO8859_1,
  LEDGER_REFERENCE               VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  LEDGER_DOCUMENT                VARCHAR(    10) DEFAULT NULL COLLATE ISO8859_1,
  DESCRIPTION                    VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  VAT_CODE                       INTEGER         DEFAULT 0,
  GOODS                          DECIMAL( 18, 4) DEFAULT NULL,
  VAT                            DECIMAL( 18, 4) DEFAULT NULL,
  TOTAL                          DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_BANKINGS_DTL PRIMARY KEY (BANKING_REFERENCE, ITEM_NUMBER)
);

CREATE TABLE BANK_LEDGER 
(
  BANK_ACCOUNT_REFERENCE              VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  BANK_ACCOUNT_NAME                   VARCHAR(    55) DEFAULT NULL COLLATE ISO8859_1,
  INVOICE_ADDRESS1                    VARCHAR(    25) DEFAULT NULL COLLATE ISO8859_1,
  INVOICE_ADDRESS2                    VARCHAR(    25) DEFAULT NULL COLLATE ISO8859_1,
  INVOICE_ADDRESS3                    VARCHAR(    25) DEFAULT NULL COLLATE ISO8859_1,
  POSTCODE                               CHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  MANAGER_CONTACT                     VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  MANAGER_TELEPHONE                   VARCHAR(    12) DEFAULT NULL COLLATE ISO8859_1,
  MANAGER_TELEPHONE_DATA              VARCHAR(    12) DEFAULT NULL COLLATE ISO8859_1,
  MANAGER_FAX                         VARCHAR(    12) DEFAULT NULL COLLATE ISO8859_1,
  BRANCH_CONTACT                      VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  BRANCH_TELEPHONE                    VARCHAR(    12) DEFAULT NULL COLLATE ISO8859_1,
  BRANCH_FAX                          VARCHAR(    12) DEFAULT NULL COLLATE ISO8859_1,
  BANK_SORT_CODE                         CHAR(     6) DEFAULT NULL COLLATE ISO8859_1,
  BANK_ACCOUNT_TYPE                   VARCHAR(    15) DEFAULT NULL COLLATE ISO8859_1,
  BANK_ACCOUNT_NUMBER                    CHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  ACCOUNT_NAME                        VARCHAR(    25) DEFAULT NULL COLLATE ISO8859_1,
  INTEREST_RATE              DOUBLE PRECISION         DEFAULT NULL,
  OVERDRAFT_LIMIT                     DECIMAL( 18, 4) DEFAULT NULL,
  ACCOUNT_BALANCE                     DECIMAL( 18, 4) DEFAULT NULL,
  TURNOVER_THIS_YEAR                  DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_BANK_LEDGER PRIMARY KEY (BANK_ACCOUNT_REFERENCE)
);

CREATE TABLE BANK_LEDGER_DTL 
(
  URN                                  BIGINT         DEFAULT 0 NOT NULL,
  BANK_ACCOUNT_REFERENCE              VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  DATE_                                  DATE         DEFAULT NULL,
  DAYBOOK_ID                          VARCHAR(     3) DEFAULT NULL COLLATE ISO8859_1,
  DOCUMENT_NUMBER                     VARCHAR(    10) DEFAULT NULL COLLATE ISO8859_1,
  DESCRIPTION                         VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  DEBIT                               DECIMAL( 18, 4) DEFAULT NULL,
  CREDIT                              DECIMAL( 18, 4) DEFAULT NULL,
  BALANCE                             DECIMAL( 18, 4) DEFAULT NULL,
  CLEARED                                DATE         DEFAULT NULL,
  CURRENT_                           SMALLINT         DEFAULT NULL,
 CONSTRAINT PK_BANK_LEDGER_DTL PRIMARY KEY (URN)
);

CREATE ASC INDEX I_BANK_LEDGER_DTL_ACREF ON BANK_LEDGER_DTL (BANK_ACCOUNT_REFERENCE, DATE_, URN);

SET TERM ^^ ;
CREATE TRIGGER BANK_LEDGER_DTL_ID FOR BANK_LEDGER_DTL ACTIVE BEFORE INSERT POSITION 0 AS
begin
 new.URN = gen_id(BANK_LEDGER_DTL_GEN, 1);
end ^^
SET TERM ; ^^

CREATE TABLE BANK_PAYMENTS 
(
  PAYMENT_REFERENCE                   VARCHAR(    10) NOT NULL COLLATE ISO8859_1,
  BATCH                               NUMERIC( 18, 0) DEFAULT NULL,
  VAT_PERIOD                          VARCHAR(     4)  COLLATE ISO8859_1,
  BANK_ACCOUNT_REFERENCE              VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  DATE_                                  DATE         DEFAULT NULL,
  PAYEE                               VARCHAR(    55) DEFAULT NULL COLLATE ISO8859_1,
  INVOICE_ADDRESS1                    VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  INVOICE_ADDRESS2                    VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  INVOICE_ADDRESS3                    VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  POSTCODE                               CHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  GOODS                               DECIMAL( 18, 4) DEFAULT NULL,
  VAT                                 DECIMAL( 18, 4) DEFAULT NULL,
  TOTAL                               DECIMAL( 18, 4) DEFAULT NULL,
  BANK_ACCOUNT_NUMBER                    CHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  BANK_SORT_CODE                         CHAR(     6) DEFAULT NULL COLLATE ISO8859_1,
 CONSTRAINT PK_BANK_PAYMENTS PRIMARY KEY (PAYMENT_REFERENCE)
);

CREATE ASC INDEX I_BANK_PAYMENTS_BANKACREF ON BANK_PAYMENTS (BANK_ACCOUNT_REFERENCE, DATE_, PAYMENT_REFERENCE);

CREATE TABLE BANK_PAYMENTS_DTL 
(
  PAYMENT_REFERENCE              VARCHAR(    10) NOT NULL COLLATE ISO8859_1,
  ITEM_NUMBER                    INTEGER         DEFAULT 0 NOT NULL,
  NOMINAL_CODE                   VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  COST_CENTRE                    VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  LEDGER_ID                      VARCHAR(     3) DEFAULT NULL COLLATE ISO8859_1,
  LEDGER_REFERENCE               VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  LEDGER_DOCUMENT                VARCHAR(    10) DEFAULT NULL COLLATE ISO8859_1,
  DESCRIPTION                    VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  VAT_CODE                       INTEGER         DEFAULT 0,
  GOODS                          DECIMAL( 18, 4) DEFAULT NULL,
  VAT                            DECIMAL( 18, 4) DEFAULT NULL,
  TOTAL                          DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_BANK_PAYMENTS_DTL PRIMARY KEY (PAYMENT_REFERENCE, ITEM_NUMBER)
);

CREATE TABLE BANK_STATEMENTS 
(
  ENTRY_DATE                      DATE         DEFAULT 'TODAY' NOT NULL,
  TRANSACTION_REF              INTEGER         DEFAULT 0 NOT NULL,
  "TYPE"                          CHAR(     4) DEFAULT NULL COLLATE ISO8859_1,
  STATUS                          CHAR(     1) DEFAULT NULL COLLATE ISO8859_1,
  DESCRIPTION                  VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  CASHBOOK                        DATE         DEFAULT NULL,
  AMOUNT                       DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_BANK_STATEMENTS PRIMARY KEY (ENTRY_DATE, TRANSACTION_REF)
);

CREATE TABLE CHEQUE_PAYMENTS 
(
  CHEQUE_NUMBER                       VARCHAR(    10) NOT NULL COLLATE ISO8859_1,
  BATCH                               NUMERIC( 18, 0) DEFAULT NULL,
  VAT_PERIOD                          VARCHAR(     4) DEFAULT NULL COLLATE ISO8859_1,
  BANK_ACCOUNT_REFERENCE              VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  DATE_                                  DATE         DEFAULT NULL,
  PAYEE                               VARCHAR(    55) DEFAULT NULL COLLATE ISO8859_1,
  INVOICE_ADDRESS1                    VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  INVOICE_ADDRESS2                    VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  INVOICE_ADDRESS3                    VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  POSTCODE                               CHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  GOODS                               DECIMAL( 18, 4) DEFAULT NULL,
  VAT                                 DECIMAL( 18, 4) DEFAULT NULL,
  TOTAL                               DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_CHEQUE_PAYMENTS PRIMARY KEY (CHEQUE_NUMBER)
);

CREATE ASC INDEX I_CHEQUE_PAYMENTS_BANKACREF ON CHEQUE_PAYMENTS (BANK_ACCOUNT_REFERENCE, DATE_, CHEQUE_NUMBER);

CREATE TABLE CHEQUE_PAYMENTS_DTL 
(
  CHEQUE_NUMBER                 VARCHAR(    10) NOT NULL COLLATE ISO8859_1,
  ITEM_NUMBER                   INTEGER         DEFAULT 0 NOT NULL,
  NOMINAL_CODE                  VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  COST_CENTRE                   VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  LEDGER_ID                     VARCHAR(     3) DEFAULT NULL COLLATE ISO8859_1,
  LEDGER_REFERENCE              VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  LEDGER_DOCUMENT               VARCHAR(    10) DEFAULT NULL COLLATE ISO8859_1,
  DESCRIPTION                   VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  VAT_CODE                      INTEGER         DEFAULT 0,
  GOODS                         DECIMAL( 18, 4) DEFAULT NULL,
  VAT                           DECIMAL( 18, 4) DEFAULT NULL,
  TOTAL                         DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_CHEQUE_PAYMENTS_DTL PRIMARY KEY (CHEQUE_NUMBER, ITEM_NUMBER)
);

CREATE TABLE COST_ACCOUNTS 
(
  COST_CENTRE               VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  NOMINAL_CODE              VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  PERIOD_END                   DATE         DEFAULT '01/01/1900' NOT NULL,
  THIS_MONTH                DECIMAL( 18, 4) DEFAULT NULL,
  YEAR_TO_DATE              DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_COST_ACCOUNTS PRIMARY KEY (COST_CENTRE, NOMINAL_CODE, PERIOD_END)
);

CREATE TABLE COST_CENTRES 
(
  COST_CENTRE                VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  COST_CATEGORY              VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  ANALYSIS1                  VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  ANALYSIS2                  VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  ANALYSIS3                  VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  DESCRIPTION                VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
 CONSTRAINT PK_COST_CENTRES PRIMARY KEY (COST_CENTRE)
);

CREATE ASC INDEX I_COST_CENTRES_CATEGORY ON COST_CENTRES (COST_CATEGORY, COST_CENTRE);

CREATE TABLE COST_CENTRE_CATEGORIES 
(
  COST_CENTRE_TYPE              VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  COST_CATEGORY                 VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  DESCRIPTION                   VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  ALLOW_ENTRIES                SMALLINT         DEFAULT 1,
 CONSTRAINT PK_COST_CENTRE_CATEGORIES PRIMARY KEY (COST_CENTRE_TYPE, COST_CATEGORY)
);

CREATE TABLE COST_CENTRE_TYPES 
(
  COST_CENTRE_TYPE              VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  DESCRIPTION                   VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
 CONSTRAINT PK_COST_CENTRE_TYPES PRIMARY KEY (COST_CENTRE_TYPE)
);

CREATE TABLE COST_LEDGER 
(
  COST_CENTRE                   VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  NOMINAL_CODE                  VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  DESCRIPTION                   VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  OPENING_RESERVES              DECIMAL( 18, 4) DEFAULT NULL,
  OPENING_BALANCE               DECIMAL( 18, 4) DEFAULT NULL,
  ACCOUNT_BALANCE               DECIMAL( 18, 4) DEFAULT NULL,
  RESERVES                      DECIMAL( 18, 4) DEFAULT NULL,
  PROFIT_AND_LOSS               DECIMAL( 18, 4) DEFAULT NULL,
  BALANCE_SHEET                 DECIMAL( 18, 4) DEFAULT NULL,
  PREVIOUS_PERIOD               DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_COST_LEDGER PRIMARY KEY (COST_CENTRE, NOMINAL_CODE)
);

CREATE TABLE DIARY 
(
  DIARY_DATE                   DATE         DEFAULT '01/01/1900' NOT NULL,
  THINGS_TO_DO                 BLOB SUB_TYPE 1 SEGMENT SIZE 80,
 CONSTRAINT PK_DIARY PRIMARY KEY (DIARY_DATE)
);

CREATE TABLE DIARY_DTL 
(
  DIARY_DATE                       DATE         DEFAULT '01/01/1900' NOT NULL,
  LEDGER_ID                     VARCHAR(     3) NOT NULL COLLATE ISO8859_1,
  LEDGER_REFERENCE              VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  NOTES                         VARCHAR(    60) DEFAULT NULL COLLATE ISO8859_1,
  ACTION_BY                     VARCHAR(    20) DEFAULT NULL COLLATE ISO8859_1,
 CONSTRAINT PK_DIARY_DTL PRIMARY KEY (DIARY_DATE, LEDGER_ID, LEDGER_REFERENCE)
);

CREATE TABLE FIXED_ASSETS 
(
  ASSET_REFERENCE                           VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  ASSET_CATEGORY                            VARCHAR(     8)  COLLATE ISO8859_1,
  COST_CENTRE                               VARCHAR(     8)  COLLATE ISO8859_1,
  DESCRIPTION                               VARCHAR(    40)  COLLATE ISO8859_1,
  DATE_ACQUIRED                                DATE,
  COST_OR_VALUATION                         DECIMAL( 18, 4),
  ANTICIPATED_DISPOSAL_DATE                    DATE,
  ANTICIPATED_DISPOSAL_VALUE                DECIMAL( 18, 4),
  DEPRECIATION_LAST_PERIOD_END              DECIMAL( 18, 4),
 CONSTRAINT PK_FIXED_ASSETS PRIMARY KEY (ASSET_REFERENCE)
);

CREATE ASC INDEX I_FIXED_ASSETS_CATEGORY ON FIXED_ASSETS (ASSET_CATEGORY, ASSET_REFERENCE);

CREATE TABLE FIXED_ASSETS_DEPRECIATION 
(
  ASSET_REFERENCE                   VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  DATE_                                DATE         NOT NULL,
  DEPRECIATION_TO_DATE              DECIMAL( 18, 4),
  WRITTEN_DOWN_VALUE                DECIMAL( 18, 4),
 CONSTRAINT PK_FIXED_ASSETS_DEPRECIATIO PRIMARY KEY (ASSET_REFERENCE, DATE_)
);

CREATE TABLE FIXED_ASSET_CATEGORIES 
(
  ASSET_CATEGORY                            VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  DESCRIPTION                               VARCHAR(    40)  COLLATE ISO8859_1,
  DEPRECIATION_NOMINAL_CODE_PL              VARCHAR(     8)  COLLATE ISO8859_1,
  DEPRECIATION_NOMINAL_CODE_BS              VARCHAR(     8)  COLLATE ISO8859_1,
  DEPRECIATION_METHOD                       INTEGER,
  DEPRECIATION_RATE_PERCENT_PA     DOUBLE PRECISION,
  ASSET_LIFE_IN_YEARS              DOUBLE PRECISION,
 CONSTRAINT PK_FIXED_ASSET_CATEGORIES PRIMARY KEY (ASSET_CATEGORY)
);

CREATE TABLE FLIGHT_LOG 
(
  LOG_REFERENCE              VARCHAR(    10) NOT NULL COLLATE ISO8859_1,
  DATE_                         DATE         DEFAULT NULL,
  DAY_NAME                   VARCHAR(    10) DEFAULT NULL COLLATE ISO8859_1,
  NOTES                      VARCHAR(    50) DEFAULT NULL COLLATE ISO8859_1,
  LOG_IMAGE                  VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  TUG_LOG_IMAGE              VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  OPEN_TACHO                 DECIMAL( 12, 2) DEFAULT NULL,
  CLOSE_TACHO                DECIMAL( 12, 2) DEFAULT NULL,
  TACH_TIME                  DECIMAL( 12, 2) DEFAULT NULL,
  TUG_TIME                      TIME         DEFAULT NULL,
  HEIGHT                     INTEGER         DEFAULT NULL,
  LAUNCHES                   INTEGER         DEFAULT NULL,
  TUG_PILOT                  VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  SOARING_TIME                  TIME         DEFAULT NULL,
  KMS_FLOWN                  INTEGER         DEFAULT NULL,
  BATCH                      INTEGER         DEFAULT NULL,
 CONSTRAINT PK_FLIGHT_LOG PRIMARY KEY (LOG_REFERENCE)
);

CREATE TABLE FLIGHT_LOG_DTL 
(
  LOG_REFERENCE                 VARCHAR(    10) NOT NULL COLLATE ISO8859_1,
  ITEM_NUMBER                   INTEGER         DEFAULT 0 NOT NULL,
  CUSTOMER_ACCOUNT              VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  PILOT_NAME                    VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  PRICING                       VARCHAR(    20) DEFAULT 'Club rates' COLLATE ISO8859_1,
  INSTRUCTOR                    VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  INSTRUCTOR_NAME               VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  GLIDER_CALLSIGN               VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  GLIDER_TYPE                   VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  TUG                           VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  LAUNCH                           TIME         DEFAULT NULL,
  LAND                             TIME         DEFAULT NULL,
  DURATION                         TIME         DEFAULT NULL,
  RELEASE_HEIGHT                INTEGER         DEFAULT NULL,
  TUG_PILOT                     VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  TUG_PILOT_NAME                VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  TUG_TIME                         TIME         DEFAULT NULL,
  VOUCHER_NO                    VARCHAR(    20) DEFAULT NULL COLLATE ISO8859_1,
  KMS_FLOWN                     INTEGER         DEFAULT NULL,
  NOTES                         VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  INVOICED                     SMALLINT         DEFAULT 0,
 CONSTRAINT PK_FLIGHT_LOG_DTL PRIMARY KEY (LOG_REFERENCE, ITEM_NUMBER)
);

CREATE TABLE GLIDERS 
(
  GLIDER_CALLSIGN              VARCHAR(     8) DEFAULT 'G' NOT NULL COLLATE ISO8859_1,
  GLIDER_TYPE                  VARCHAR(    30)  COLLATE ISO8859_1,
  C_OF_A                     TIMESTAMP,
  BATCH                        INTEGER,
 CONSTRAINT PK_GLIDERS PRIMARY KEY (GLIDER_CALLSIGN)
);

CREATE TABLE GLIDER_LOGS 
(
  URN                           BIGINT         DEFAULT 0 NOT NULL,
  GLIDER_CALLSIGN              VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  DATE_                           DATE         DEFAULT NULL,
  DOCUMENT_NUMBER              VARCHAR(    10)  COLLATE ISO8859_1,
  ITEM_NUMBER                  INTEGER         DEFAULT NULL,
  PILOT                        VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  PILOT_NAME                   VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  INSTRUCTOR                   VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  LAUNCH                          TIME         DEFAULT NULL,
  LAND                            TIME         DEFAULT NULL,
  DURATION                        TIME         DEFAULT NULL,
  RELEASE_HEIGHT               INTEGER         DEFAULT NULL,
  KMS_FLOWN                    INTEGER         DEFAULT NULL,
  NOTES                        VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  LOG_IMAGE                    VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
 CONSTRAINT PK_GLIDER_LOGS PRIMARY KEY (URN)
);

CREATE ASC INDEX I_GLIDER_LOGS_GLIDER ON GLIDER_LOGS (GLIDER_CALLSIGN, DATE_, URN);

SET TERM ^^ ;
CREATE TRIGGER GLIDER_LOGS_ID FOR GLIDER_LOGS ACTIVE BEFORE INSERT POSITION 0 AS
begin
 new.URN = gen_id(GLIDER_LOGS_GEN, 1);
end ^^
SET TERM ; ^^

CREATE TABLE JOURNAL 
(
  JOURNAL_REFERENCE              VARCHAR(    10) NOT NULL COLLATE ISO8859_1,
  BATCH                          NUMERIC( 18, 0) DEFAULT NULL,
  DATE_                             DATE         DEFAULT NULL,
  DESCRIPTION                    VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  ENTERED_BY                     VARCHAR(    20) DEFAULT NULL COLLATE ISO8859_1,
 CONSTRAINT PK_JOURNAL PRIMARY KEY (JOURNAL_REFERENCE)
);

CREATE TABLE JOURNAL_DTL 
(
  JOURNAL_REFERENCE              VARCHAR(    10) NOT NULL COLLATE ISO8859_1,
  ITEM_NUMBER                    INTEGER         DEFAULT 0 NOT NULL,
  NOMINAL_CODE                   VARCHAR(     8)  COLLATE ISO8859_1,
  COST_CENTRE                    VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  LEDGER_ID                      VARCHAR(     3) DEFAULT NULL COLLATE ISO8859_1,
  LEDGER_REFERENCE               VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  DESCRIPTION                    VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  DEBIT                          DECIMAL( 18, 4) DEFAULT NULL,
  CREDIT                         DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_JOURNAL_DTL PRIMARY KEY (JOURNAL_REFERENCE, ITEM_NUMBER)
);

CREATE TABLE JOURNAL_SKELETON 
(
  SKELETON_REFERENCE              VARCHAR(    10) NOT NULL COLLATE ISO8859_1,
  DESCRIPTION                     VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
 CONSTRAINT PK_JOURNAL_SKELETON PRIMARY KEY (SKELETON_REFERENCE)
);

CREATE TABLE JOURNAL_SKELETON_DTL 
(
  SKELETON_REFERENCE              VARCHAR(    10) NOT NULL COLLATE ISO8859_1,
  ITEM_NUMBER                     INTEGER         DEFAULT 0 NOT NULL,
  NOMINAL_CODE                    VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  COST_CENTRE                     VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  DESCRIPTION                     VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  DEBIT                           DECIMAL( 18, 4) DEFAULT NULL,
  CREDIT                          DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_JOURNAL_SKELETON_DTL PRIMARY KEY (SKELETON_REFERENCE, ITEM_NUMBER)
);

CREATE TABLE MEMBERS 
(
  MEMBER_REFERENCE              VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  SURNAME                       VARCHAR(    25)  COLLATE ISO8859_1,
  FORENAME                      VARCHAR(    25)  COLLATE ISO8859_1,
  DISPLAY_NAME                  VARCHAR(    40)  COLLATE ISO8859_1,
  INITIALS                      VARCHAR(    10)  COLLATE ISO8859_1,
  MEMBERSHIP                       CHAR(     8) DEFAULT 'FULL' COLLATE ISO8859_1,
  ADDRESS1                      VARCHAR(    30)  COLLATE ISO8859_1,
  ADDRESS2                      VARCHAR(    30)  COLLATE ISO8859_1,
  ADDRESS3                      VARCHAR(    30)  COLLATE ISO8859_1,
  ADDRESS4                      VARCHAR(    30)  COLLATE ISO8859_1,
  POSTCODE                      VARCHAR(     8)  COLLATE ISO8859_1,
  TELEPHONE                     VARCHAR(    12)  COLLATE ISO8859_1,
  EMAIL                         VARCHAR(    40)  COLLATE ISO8859_1,
  BATCH                         INTEGER,
 CONSTRAINT PK_MEMBERS PRIMARY KEY (MEMBER_REFERENCE)
);

CREATE TABLE NOMINAL_ACCOUNTS 
(
  NOMINAL_CODE              VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  PERIOD_END                   DATE         DEFAULT '01/01/1900' NOT NULL,
  THIS_MONTH                DECIMAL( 18, 4) DEFAULT NULL,
  YEAR_TO_DATE              DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_NOMINAL_ACCOUNTS PRIMARY KEY (NOMINAL_CODE, PERIOD_END)
);

CREATE TABLE NOMINAL_ACCRUALS 
(
  ITEM_NUMBER               INTEGER         NOT NULL,
  NOMINAL_CODE              VARCHAR(     8)  COLLATE ISO8859_1,
  COST_CENTRE               VARCHAR(     8)  COLLATE ISO8859_1,
  DESCRIPTION               VARCHAR(    40)  COLLATE ISO8859_1,
  DATE_FROM                    DATE,
  DATE_TO                      DATE,
  AMOUNT                    DECIMAL( 18, 4),
 CONSTRAINT PK_NOMINAL_ACCRUALS PRIMARY KEY (ITEM_NUMBER)
);

CREATE TABLE NOMINAL_ACCRUALS_DTL 
(
  ITEM_NUMBER              INTEGER         NOT NULL,
  DATE_                       DATE         NOT NULL,
  BALANCE                  DECIMAL( 18, 4),
 CONSTRAINT PK_NOMINAL_ACCRUALS_DTL PRIMARY KEY (ITEM_NUMBER, DATE_)
);

CREATE TABLE NOMINAL_BUDGETS 
(
  NOMINAL_CODE              VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  PERIOD_END                   DATE         DEFAULT '01/01/1900' NOT NULL,
  BUDGET                    DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_NOMINAL_BUDGETS PRIMARY KEY (NOMINAL_CODE, PERIOD_END)
);

CREATE TABLE NOMINAL_CODES 
(
  NOMINAL_CODE                       VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  LINE_NUMBER                        VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  DESCRIPTION                        VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  DEFAULT_VAT_CODE                   INTEGER         DEFAULT NULL,
  VARIABLE_COST_PERCENT                FLOAT,
 CONSTRAINT PK_NOMINAL_CODES PRIMARY KEY (NOMINAL_CODE)
);

CREATE ASC INDEX I_NOMINAL_CODES_LINENUMBER ON NOMINAL_CODES (LINE_NUMBER, NOMINAL_CODE);

CREATE TABLE NOMINAL_LEDGER 
(
  NOMINAL_CODE                  VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  DESCRIPTION                   VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  OPENING_RESERVES              DECIMAL( 18, 4) DEFAULT NULL,
  OPENING_BALANCE               DECIMAL( 18, 4) DEFAULT NULL,
  ACCOUNT_BALANCE               DECIMAL( 18, 4) DEFAULT NULL,
  RESERVES                      DECIMAL( 18, 4) DEFAULT NULL,
  PROFIT_AND_LOSS               DECIMAL( 18, 4) DEFAULT NULL,
  BALANCE_SHEET                 DECIMAL( 18, 4) DEFAULT NULL,
  PREVIOUS_PERIOD               DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_NOMINAL_LEDGER PRIMARY KEY (NOMINAL_CODE)
);

CREATE TABLE NOMINAL_OPENING_BALANCES 
(
  RESERVE_REFERENCE              VARCHAR(    10) NOT NULL COLLATE ISO8859_1,
  DATE_                             DATE         DEFAULT NULL,
  NOMINAL_CODE                   VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  COST_CENTRE                    VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  DESCRIPTION                    VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  TOTAL                          DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_NOMINAL_OPENING_BALANCES PRIMARY KEY (RESERVE_REFERENCE)
);

CREATE TABLE NOMINAL_OPENING_BALANCE_DTL 
(
  RESERVE_REFERENCE              VARCHAR(    10) NOT NULL COLLATE ISO8859_1,
  ITEM_NUMBER                    INTEGER         NOT NULL,
  NOMINAL_CODE                   VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  COST_CENTRE                    VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  DESCRIPTION                    VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  DEBIT                          DECIMAL( 18, 4) DEFAULT NULL,
  CREDIT                         DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_NOMINAL_OPENING_BALANCE_ PRIMARY KEY (RESERVE_REFERENCE, ITEM_NUMBER)
);

CREATE TABLE NOMINAL_PREPAYMENTS 
(
  ITEM_NUMBER               INTEGER         NOT NULL,
  NOMINAL_CODE              VARCHAR(     8)  COLLATE ISO8859_1,
  COST_CENTRE               VARCHAR(     8)  COLLATE ISO8859_1,
  DESCRIPTION               VARCHAR(    40)  COLLATE ISO8859_1,
  DATE_FROM                    DATE,
  DATE_TO                      DATE,
  AMOUNT                    DECIMAL( 18, 4),
 CONSTRAINT PK_NOMINAL_PREPAYMENTS PRIMARY KEY (ITEM_NUMBER)
);

CREATE TABLE NOMINAL_PREPAYMENTS_DTL 
(
  ITEM_NUMBER              INTEGER         NOT NULL,
  DATE_                       DATE         NOT NULL,
  BALANCE                  DECIMAL( 18, 4),
 CONSTRAINT PK_NOMINAL_PREPAYMENTS_DTL PRIMARY KEY (ITEM_NUMBER, DATE_)
);

CREATE TABLE NOMINAL_REPORTS 
(
  LAYOUT_TYPE                     VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  TITLE                           VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  IS_PROFIT_AND_LOSS             SMALLINT         DEFAULT 1,
 CONSTRAINT PK_NOMINAL_REPORTS PRIMARY KEY (LAYOUT_TYPE)
);

CREATE TABLE NOMINAL_REPORT_HEADS 
(
  LAYOUT_TYPE                          VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  LAYOUT_HEAD                          VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  DESCRIPTION                          VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  NORMALLY_A_DEBIT                    SMALLINT         DEFAULT 1 NOT NULL,
  SUBTOTAL_PRE_ZERO                   SMALLINT         DEFAULT 0 NOT NULL,
  SUBTOTAL_TITLE                       VARCHAR(    40)  COLLATE ISO8859_1,
  SUBTOTAL_NORMALLY_DEBIT             SMALLINT         DEFAULT 1 NOT NULL,
 CONSTRAINT PK_NOMINAL_REPORT_HEADS PRIMARY KEY (LAYOUT_TYPE, LAYOUT_HEAD)
);

CREATE TABLE NOMINAL_REPORT_SUMMARY 
(
  LAYOUT_TYPE              VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  LAYOUT_HEAD              VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  LINE_NUMBER              VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  DESCRIPTION              VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
 CONSTRAINT PK_NOMINAL_REPORT_SUMMARY PRIMARY KEY (LAYOUT_TYPE, LAYOUT_HEAD, LINE_NUMBER)
);

CREATE TABLE NOMINAL_RESERVES 
(
  RESERVE_REFERENCE              VARCHAR(    10) NOT NULL COLLATE ISO8859_1,
  DATE_                             DATE         DEFAULT NULL,
  NOMINAL_CODE                   VARCHAR(     8)  COLLATE ISO8859_1,
  COST_CENTRE                    VARCHAR(     8)  COLLATE ISO8859_1,
  DESCRIPTION                    VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  TOTAL                          DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_NOMINAL_RESERVES PRIMARY KEY (RESERVE_REFERENCE)
);

CREATE TABLE NOMINAL_RESERVES_DTL 
(
  RESERVE_REFERENCE              VARCHAR(    10) NOT NULL COLLATE ISO8859_1,
  ITEM_NUMBER                    INTEGER         NOT NULL,
  NOMINAL_CODE                   VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  COST_CENTRE                    VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  DESCRIPTION                    VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  DEBIT                          DECIMAL( 18, 4) DEFAULT NULL,
  CREDIT                         DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_NOMINAL_RESERVES_DTL PRIMARY KEY (RESERVE_REFERENCE, ITEM_NUMBER)
);

CREATE GENERATOR PETTY_CASH_LEDGER_DTL_GEN;

SET GENERATOR PETTY_CASH_LEDGER_DTL_GEN TO 0;

CREATE TABLE PETTY_CASH_LEDGER 
(
  CASH_ACCOUNT_REFERENCE              VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  CASH_ACCOUNT_NAME                   VARCHAR(    55) DEFAULT NULL COLLATE ISO8859_1,
  ACCOUNT_BALANCE                     DECIMAL( 18, 4) DEFAULT NULL,
  TURNOVER_THIS_YEAR                  DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_PETTY_CASH_LEDGER PRIMARY KEY (CASH_ACCOUNT_REFERENCE)
);

CREATE TABLE PETTY_CASH_LEDGER_DTL 
(
  URN                                  BIGINT         DEFAULT 0 NOT NULL,
  CASH_ACCOUNT_REFERENCE              VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  DATE_                                  DATE         DEFAULT NULL,
  DAYBOOK_ID                          VARCHAR(     3) DEFAULT NULL COLLATE ISO8859_1,
  DOCUMENT_NUMBER                     VARCHAR(    10) DEFAULT NULL COLLATE ISO8859_1,
  DESCRIPTION                         VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  DEBIT                               DECIMAL( 18, 4) DEFAULT NULL,
  CREDIT                              DECIMAL( 18, 4) DEFAULT NULL,
  BALANCE                             DECIMAL( 18, 4) DEFAULT NULL,
  CLEARED                                DATE         DEFAULT NULL,
  CURRENT_                           SMALLINT         DEFAULT NULL,
 CONSTRAINT PK_PETTY_CASH_LEDGER_DTL PRIMARY KEY (URN)
);

CREATE ASC INDEX I_PETTY_CASH_LEDGER_DTL_ ON PETTY_CASH_LEDGER_DTL (CASH_ACCOUNT_REFERENCE, DATE_, URN);

SET TERM ^^ ;
CREATE TRIGGER PETTY_CASH_LEDGER_DTL_ FOR PETTY_CASH_LEDGER_DTL ACTIVE BEFORE INSERT POSITION 0 AS
/*
  You can change this template in the template editor:
  File | Preferences | Object Templates

  Trigger:

  Author   : , 
  Date     : 30/07/2010 19:08:50
  Purpose  :
*/
begin
 new.URN = gen_id(PETTY_CASH_LEDGER_DTL_GEN, 1);
end ^^
SET TERM ; ^^

CREATE TABLE PETTY_CASH_VOUCHERS 
(
  PAYMENT_REFERENCE              VARCHAR(    10) NOT NULL COLLATE ISO8859_1,
  BATCH                          NUMERIC( 18, 0) DEFAULT NULL,
  VAT_PERIOD                     VARCHAR(     4)  COLLATE ISO8859_1,
  DATE_                             DATE         DEFAULT NULL,
  PAYEE                          VARCHAR(    55) DEFAULT NULL COLLATE ISO8859_1,
  GOODS                          DECIMAL( 18, 4) DEFAULT NULL,
  VAT                            DECIMAL( 18, 4) DEFAULT NULL,
  TOTAL                          DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_PETTY_CASH_VOUCHERS PRIMARY KEY (PAYMENT_REFERENCE)
);

CREATE TABLE PETTY_CASH_VOUCHERS_DTL 
(
  PAYMENT_REFERENCE              VARCHAR(    10) NOT NULL COLLATE ISO8859_1,
  ITEM_NUMBER                    INTEGER         DEFAULT 0 NOT NULL,
  NOMINAL_CODE                   VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  COST_CENTRE                    VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  LEDGER_ID                      VARCHAR(     3) DEFAULT NULL COLLATE ISO8859_1,
  LEDGER_REFERENCE               VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  LEDGER_DOCUMENT                VARCHAR(    10) DEFAULT NULL COLLATE ISO8859_1,
  DESCRIPTION                    VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  VAT_CODE                       INTEGER         DEFAULT 0,
  GOODS                          DECIMAL( 18, 4) DEFAULT NULL,
  VAT                            DECIMAL( 18, 4) DEFAULT NULL,
  TOTAL                          DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_PETTY_CASH_VOUCHERS_DTL PRIMARY KEY (PAYMENT_REFERENCE, ITEM_NUMBER)
);


CREATE TABLE PILOT_LOG_GLIDERS 
(
  URN                             BIGINT         DEFAULT 0 NOT NULL,
  MEMBER_REFERENCE               VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  DATE_                             DATE         DEFAULT NULL,
  DOCUMENT_NUMBER                VARCHAR(    10) DEFAULT NULL COLLATE ISO8859_1,
  ITEM_NUMBER                    INTEGER         DEFAULT NULL,
  PILOT_NAME                     VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  GLIDER_CALLSIGN                VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  CREW                           VARCHAR(     4) DEFAULT NULL COLLATE ISO8859_1,
  SECOND_PILOT                   VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  SECOND_PILOT_NAME              VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  LAUNCH                            TIME         DEFAULT NULL,
  LAND                              TIME         DEFAULT NULL,
  DURATION                          TIME         DEFAULT NULL,
  RELEASE_HEIGHT                 INTEGER         DEFAULT NULL,
  KMS_FLOWN                      INTEGER         DEFAULT NULL,
  NOTES                          VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  LOG_IMAGE                      VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
 CONSTRAINT PK_PILOT_LOG_GLIDERS PRIMARY KEY (URN)
);

CREATE ASC INDEX I_PILOT_LOG_GLIDERS_PILOT ON PILOT_LOG_GLIDERS (MEMBER_REFERENCE, DATE_, URN);

SET TERM ^^ ;
CREATE TRIGGER PILOT_LOG_GLIDERS_ID FOR PILOT_LOG_GLIDERS ACTIVE BEFORE INSERT POSITION 0 AS
begin
 new.URN = gen_id(PILOT_LOG_GLIDERS_GEN, 1);
end ^^
SET TERM ; ^^

CREATE TABLE PILOT_LOG_TUGS 
(
  URN                            BIGINT         DEFAULT 0 NOT NULL,
  MEMBER_REFERENCE              VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  DATE_                            DATE         DEFAULT NULL,
  DOCUMENT_NUMBER               VARCHAR(    10) DEFAULT NULL COLLATE ISO8859_1,
  ITEM_NUMBER                   INTEGER         DEFAULT NULL,
  TUG_CALLSIGN                  VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  LAUNCH                           TIME         DEFAULT NULL,
  TUG_TIME                         TIME         DEFAULT NULL,
  RELEASE_HEIGHT                INTEGER         DEFAULT NULL,
  TUG_LOG_IMAGE                 VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
 CONSTRAINT PK_PILOT_LOG_TUGS PRIMARY KEY (URN)
);

CREATE ASC INDEX I_PILOT_LOG_TUGS_PILOT ON PILOT_LOG_TUGS (MEMBER_REFERENCE, DATE_, URN);

SET TERM ^^ ;
CREATE TRIGGER PILOT_LOG_TUGS_ID FOR PILOT_LOG_TUGS ACTIVE BEFORE INSERT POSITION 0 AS
begin
 new.URN = gen_id(PILOT_LOG_TUGS_GEN, 1);
end ^^
SET TERM ; ^^

CREATE TABLE PRICE_LISTS 
(
  PRICE_LIST               VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  DATE_                       DATE,
  DESCRIPTION              VARCHAR(    40)  COLLATE ISO8859_1,
 CONSTRAINT PK_PRICE_LISTS PRIMARY KEY (PRICE_LIST)
);

CREATE TABLE PRICE_LISTS_DTL 
(
  PRICE_LIST                VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  PRICE_CODE                VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  DESCRIPTION               VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  VAT_CODE                  INTEGER         DEFAULT 1,
  UNIT_PRICE                DECIMAL( 18, 4) DEFAULT 0.00,
  NOMINAL_CODE              VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
 CONSTRAINT PK_PRICE_LISTS_DTL PRIMARY KEY (PRICE_LIST, PRICE_CODE)
);

CREATE TABLE PURCHASE_INVOICE 
(
  INVOICE_REFERENCE                    VARCHAR(    10) NOT NULL COLLATE ISO8859_1,
  BATCH                                NUMERIC( 18, 0) DEFAULT NULL,
  VAT_PERIOD                           VARCHAR(     4) DEFAULT NULL COLLATE ISO8859_1,
  SUPPLIER_REFERENCE                   VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  SUPPLIER_NAME                        VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  DATE_                                   DATE         DEFAULT NULL,
  DUE_DATE                                DATE         DEFAULT NULL,
  ORDER_NUMBER                         VARCHAR(    10) DEFAULT NULL COLLATE ISO8859_1,
  SUPPLIER_INVOICE_NUMBER              VARCHAR(    15) DEFAULT NULL COLLATE ISO8859_1,
  ASSESSMENT                           INTEGER         DEFAULT NULL,
  QUERY                                   CHAR(     1) DEFAULT NULL COLLATE ISO8859_1,
  INVOICE_IMAGE                        VARCHAR(    80) DEFAULT NULL COLLATE ISO8859_1,
  GOODS                                DECIMAL( 18, 4) DEFAULT NULL,
  VAT                                  DECIMAL( 18, 4) DEFAULT NULL,
  TOTAL                                DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_PURCHASE_INVOICE PRIMARY KEY (INVOICE_REFERENCE)
);

CREATE ASC INDEX I_PURCHASE_INVOICE_SUPLREF ON PURCHASE_INVOICE (SUPPLIER_REFERENCE, INVOICE_REFERENCE);

CREATE TABLE PURCHASE_INVOICE_DTL 
(
  INVOICE_REFERENCE              VARCHAR(    10) NOT NULL COLLATE ISO8859_1,
  ITEM_NUMBER                    INTEGER         DEFAULT 0 NOT NULL,
  NOMINAL_CODE                   VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  COST_CENTRE                    VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  DESCRIPTION                    VARCHAR(    60) DEFAULT NULL COLLATE ISO8859_1,
  VAT_CODE                       INTEGER         DEFAULT 0,
  QUANTITY              DOUBLE PRECISION         DEFAULT 1,
  RATE                           DECIMAL( 18, 4) DEFAULT 0,
  GOODS                          DECIMAL( 18, 4) DEFAULT NULL,
  VAT                            DECIMAL( 18, 4) DEFAULT NULL,
  TOTAL                          DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_PURCHASE_INVOICE_DTL PRIMARY KEY (INVOICE_REFERENCE, ITEM_NUMBER)
);

CREATE TABLE PURCHASE_LEDGER 
(
  SUPPLIER_REFERENCE               VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  SUPPLIER_NAME                    VARCHAR(    55) DEFAULT NULL COLLATE ISO8859_1,
  INVOICE_ADDRESS1                 VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  INVOICE_ADDRESS2                 VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  INVOICE_ADDRESS3                 VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  INVOICE_ADDRESS4                 VARCHAR(    30)  COLLATE ISO8859_1,
  POSTCODE                            CHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  ACCOUNTS_CONTACT                 VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  ACCOUNTS_TELEPHONE               VARCHAR(    14) DEFAULT NULL COLLATE ISO8859_1,
  ACCOUNTS_FAX                     VARCHAR(    14) DEFAULT NULL COLLATE ISO8859_1,
  EMAIL                            VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  BANK_SORT_CODE                   VARCHAR(     6) DEFAULT NULL COLLATE ISO8859_1,
  BANK_ACCOUNT_NUMBER              VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  ACCOUNT_NAME                     VARCHAR(    25) DEFAULT NULL COLLATE ISO8859_1,
  ACCOUNT_BALANCE                  DECIMAL( 18, 4) DEFAULT NULL,
  CURRENT_                         DECIMAL( 18, 4) DEFAULT NULL,
  ONE_MONTH                        DECIMAL( 18, 4) DEFAULT NULL,
  TWO_MONTHS                       DECIMAL( 18, 4) DEFAULT NULL,
  THREE_MONTHS                     DECIMAL( 18, 4) DEFAULT NULL,
  OVER_3_MONTHS                    DECIMAL( 18, 4) DEFAULT NULL,
  TURNOVER_THIS_YEAR               DECIMAL( 18, 4) DEFAULT NULL,
  CREDIT_TERMS_DAYS                INTEGER         DEFAULT NULL,
 CONSTRAINT PK_PURCHASE_LEDGER PRIMARY KEY (SUPPLIER_REFERENCE)
);

CREATE ASC INDEX I_PURCHASE_LEDGER_NAME ON PURCHASE_LEDGER (SUPPLIER_NAME, SUPPLIER_REFERENCE);

CREATE TABLE PURCHASE_LEDGER_DTL 
(
  URN                              BIGINT         DEFAULT 0 NOT NULL,
  SUPPLIER_REFERENCE              VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  DATE_                              DATE         DEFAULT NULL,
  DAYBOOK_ID                      VARCHAR(     3) DEFAULT NULL COLLATE ISO8859_1,
  DOCUMENT_NUMBER                 VARCHAR(    10) DEFAULT NULL COLLATE ISO8859_1,
  DESCRIPTION                     VARCHAR(    60) DEFAULT NULL COLLATE ISO8859_1,
  DEBIT                           DECIMAL( 18, 4) DEFAULT NULL,
  CREDIT                          DECIMAL( 18, 4) DEFAULT NULL,
  UNPAID                          DECIMAL( 18, 4) DEFAULT NULL,
  DUE_DATE                           DATE         DEFAULT NULL,
  QUERY                              CHAR(     1) DEFAULT NULL COLLATE ISO8859_1,
  CLEARED                            DATE         DEFAULT NULL,
  CURRENT_                       SMALLINT         DEFAULT 1,
 CONSTRAINT PK_PURCHASE_LEDGER_DTL PRIMARY KEY (URN)
);

CREATE ASC INDEX I_PURCHASE_LEDGER_DTL_SUPLREF ON PURCHASE_LEDGER_DTL (SUPPLIER_REFERENCE, DATE_, URN);

SET TERM ^^ ;
CREATE TRIGGER PURCHASE_LEDGER_DTL_ID FOR PURCHASE_LEDGER_DTL ACTIVE BEFORE INSERT POSITION 0 AS
begin
 new.URN = gen_id(PURCHASE_LEDGER_DTL_GEN, 1);
end ^^
SET TERM ; ^^

CREATE TABLE SALES_INVOICE 
(
  INVOICE_REFERENCE               VARCHAR(    10) NOT NULL COLLATE ISO8859_1,
  BATCH                           NUMERIC(  8, 0) DEFAULT NULL,
  VAT_PERIOD                         CHAR(     4) DEFAULT NULL COLLATE ISO8859_1,
  CUSTOMER_REFERENCE              VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  CUSTOMER_NAME                   VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  DATE_                              DATE         DEFAULT NULL,
  LOG_ITEM                        INTEGER         DEFAULT NULL,
  DUE_DATE                           DATE         DEFAULT NULL,
  DESCRIPTION_LINE1               VARCHAR(    60) DEFAULT NULL COLLATE ISO8859_1,
  DESCRIPTION_LINE2               VARCHAR(    60) DEFAULT NULL COLLATE ISO8859_1,
  INVOICE_IMAGE                   VARCHAR(    80) DEFAULT NULL COLLATE ISO8859_1,
  GOODS                           DECIMAL( 18, 4) DEFAULT NULL,
  VAT                             DECIMAL( 18, 4) DEFAULT NULL,
  TOTAL                           DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_SALES_INVOICE PRIMARY KEY (INVOICE_REFERENCE)
);

CREATE ASC INDEX I_SALES_INVOICE_CUSTREF ON SALES_INVOICE (CUSTOMER_REFERENCE, INVOICE_REFERENCE);

CREATE TABLE SALES_INVOICE_DTL 
(
  INVOICE_REFERENCE               VARCHAR(    10) NOT NULL COLLATE ISO8859_1,
  ITEM_NUMBER                     INTEGER         DEFAULT 0 NOT NULL,
  PRICE_LIST                      VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  PRICE_CODE                      VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  NOMINAL_CODE                    VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  COST_CENTRE                     VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  DOCUMENT_REFERENCE              VARCHAR(    10) DEFAULT NULL COLLATE ISO8859_1,
  LOG_ITEM                        INTEGER         DEFAULT NULL,
  DESCRIPTION                     VARCHAR(    60) DEFAULT NULL COLLATE ISO8859_1,
  VAT_CODE                        INTEGER         DEFAULT NULL,
  QUANTITY               DOUBLE PRECISION         DEFAULT 1,
  RATE                            DECIMAL( 18, 4) DEFAULT 0,
  GOODS                           DECIMAL( 18, 4) DEFAULT NULL,
  VAT                             DECIMAL( 18, 4) DEFAULT NULL,
  TOTAL                           DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_SALES_INVOICE_DTL PRIMARY KEY (INVOICE_REFERENCE, ITEM_NUMBER)
);

CREATE TABLE SALES_LEDGER 
(
  CUSTOMER_REFERENCE              VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  CUSTOMER_NAME                   VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  INVOICE_ADDRESS1                VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  INVOICE_ADDRESS2                VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  INVOICE_ADDRESS3                VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  INVOICE_ADDRESS4                VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  POSTCODE                           CHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  ACCOUNTS_CONTACT                VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  ACCOUNTS_TELEPHONE              VARCHAR(    14) DEFAULT NULL COLLATE ISO8859_1,
  ACCOUNTS_FAX                    VARCHAR(    14) DEFAULT NULL COLLATE ISO8859_1,
  EMAIL                           VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
  CREDIT_RATING                   VARCHAR(    30) DEFAULT NULL COLLATE ISO8859_1,
  ACCOUNT_BALANCE                 DECIMAL( 18, 4) DEFAULT NULL,
  CURRENT_                        DECIMAL( 18, 4) DEFAULT NULL,
  ONE_MONTH                       DECIMAL( 18, 4) DEFAULT NULL,
  TWO_MONTHS                      DECIMAL( 18, 4) DEFAULT NULL,
  THREE_MONTHS                    DECIMAL( 18, 4) DEFAULT NULL,
  OVER_3_MONTHS                   DECIMAL( 18, 4) DEFAULT NULL,
  TURNOVER_THIS_YEAR              DECIMAL( 18, 4) DEFAULT NULL,
  CREDIT_LIMIT                    DECIMAL( 18, 4) DEFAULT NULL,
  CREDIT_TERMS_DAYS               INTEGER         DEFAULT NULL,
 CONSTRAINT PK_SALES_LEDGER PRIMARY KEY (CUSTOMER_REFERENCE)
);

CREATE ASC INDEX I_SALES_LEDGER_NAME ON SALES_LEDGER (CUSTOMER_NAME, CUSTOMER_REFERENCE);

CREATE TABLE SALES_LEDGER_DTL 
(
  URN                              BIGINT         DEFAULT 0 NOT NULL,
  CUSTOMER_REFERENCE              VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  DATE_                              DATE         DEFAULT NULL,
  DAYBOOK_ID                      VARCHAR(     3) DEFAULT NULL COLLATE ISO8859_1,
  DOCUMENT_NUMBER                 VARCHAR(    10) DEFAULT NULL COLLATE ISO8859_1,
  DESCRIPTION                     VARCHAR(    60) DEFAULT NULL COLLATE ISO8859_1,
  DEBIT                           DECIMAL( 18, 4) DEFAULT NULL,
  CREDIT                          DECIMAL( 18, 4) DEFAULT NULL,
  UNPAID                          DECIMAL( 18, 4) DEFAULT NULL,
  DUE_DATE                           DATE         DEFAULT NULL,
  QUERY                              CHAR(     1) DEFAULT NULL COLLATE ISO8859_1,
  CLEARED                            DATE         DEFAULT NULL,
  CURRENT_                       SMALLINT         DEFAULT 1,
 CONSTRAINT PK_SALES_LEDGER_DTL PRIMARY KEY (URN)
);

CREATE ASC INDEX I_SALES_LEDGER_DTL_CUSTREF ON SALES_LEDGER_DTL (CUSTOMER_REFERENCE, DATE_, URN);

SET TERM ^^ ;
CREATE TRIGGER SALES_LEDGER_DTL_ID FOR SALES_LEDGER_DTL ACTIVE BEFORE INSERT POSITION 0 AS
begin
  new.URN = gen_id(SALES_LEDGER_DTL_GEN, 1);
end ^^
SET TERM ; ^^

CREATE TABLE SYSTEM_SETTINGS 
(
  USER_ID                VARCHAR(    10) NOT NULL COLLATE ISO8859_1,
  COMPONENT              VARCHAR(    40) NOT NULL COLLATE ISO8859_1,
  SECTION                VARCHAR(    80) NOT NULL COLLATE ISO8859_1,
  ITEM                   VARCHAR(    50) NOT NULL COLLATE ISO8859_1,
  SETTING                VARCHAR(   255) DEFAULT NULL COLLATE ISO8859_1,
 CONSTRAINT PK_SYSTEM_SETTINGS PRIMARY KEY (USER_ID, COMPONENT, SECTION, ITEM)
);

CREATE TABLE TUGS 
(
  TUG_CALLSIGN               VARCHAR(     8) NOT NULL COLLATE ISO8859_1,
  FULL_CALLSIGN              VARCHAR(     8)  COLLATE ISO8859_1,
  C_OF_A                   TIMESTAMP         DEFAULT CURRENT_DATE,
 CONSTRAINT PK_TUGS PRIMARY KEY (TUG_CALLSIGN)
);

CREATE TABLE TUG_LOGS 
(
  URN                           BIGINT         DEFAULT 0 NOT NULL,
  TUG_CALLSIGN                 VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  DATE_                           DATE         DEFAULT NULL,
  DOCUMENT_NUMBER              VARCHAR(    10) DEFAULT NULL COLLATE ISO8859_1,
  ITEM_NUMBER                  INTEGER         DEFAULT NULL,
  GLIDER_CALLSIGN              VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  LAUNCH                          TIME         DEFAULT NULL,
  TUG_TIME                        TIME         DEFAULT NULL,
  RELEASE_HEIGHT               INTEGER         DEFAULT NULL,
  TUG_PILOT                    VARCHAR(     8) DEFAULT NULL COLLATE ISO8859_1,
  TUG_LOG_IMAGE                VARCHAR(    40) DEFAULT NULL COLLATE ISO8859_1,
 CONSTRAINT PK_TUG_LOGS PRIMARY KEY (URN)
);

CREATE ASC INDEX I_TUG_LOGS_TUG ON TUG_LOGS (TUG_CALLSIGN, DATE_, URN);

SET TERM ^^ ;
CREATE TRIGGER TUG_LOGS_ID FOR TUG_LOGS ACTIVE BEFORE INSERT POSITION 0 AS
begin
 new.URN = gen_id(TUG_LOGS_GEN, 1);
end ^^
SET TERM ; ^^

CREATE TABLE VAT_CODES 
(
  VAT_CODE                  INTEGER         DEFAULT 0 NOT NULL,
  RATE_PERCENT                FLOAT         DEFAULT 0.00,
  DESCRIPTION               VARCHAR(    40)  COLLATE ISO8859_1,
 CONSTRAINT PK_VAT_CODES PRIMARY KEY (VAT_CODE)
);

CREATE TABLE VAT_RETURNS 
(
  VAT_PERIOD                  VARCHAR(     4) NOT NULL COLLATE ISO8859_1,
  VAT_PERIOD_END                 DATE,
  OUTPUT_GOODS                DECIMAL( 18, 4) DEFAULT NULL,
  OUTPUT_VAT                  DECIMAL( 18, 4) DEFAULT NULL,
  INPUT_GOODS                 DECIMAL( 18, 4) DEFAULT NULL,
  INPUT_VAT                   DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_VAT_RETURNS PRIMARY KEY (VAT_PERIOD)
);

CREATE ASC INDEX I_VAT_RETURNS_DATE ON VAT_RETURNS (VAT_PERIOD_END, VAT_PERIOD);

CREATE TABLE VAT_RETURN_DTL 
(
  VAT_PERIOD               VARCHAR(     4) DEFAULT '0000' NOT NULL COLLATE ISO8859_1,
  IS_INPUT                SMALLINT         DEFAULT 0 NOT NULL,
  DAYBOOK_ID               VARCHAR(     3) DEFAULT 'AAA' NOT NULL COLLATE ISO8859_1,
  GOODS_0                  DECIMAL( 18, 4) DEFAULT NULL,
  VAT_0                    DECIMAL( 18, 4) DEFAULT NULL,
  TOTAL_GOODS              DECIMAL( 18, 4) DEFAULT NULL,
  TOTAL_VAT                DECIMAL( 18, 4) DEFAULT NULL,
  GOODS_1                  DECIMAL( 18, 4) DEFAULT NULL,
  VAT_1                    DECIMAL( 18, 4) DEFAULT NULL,
  GOODS_2                  DECIMAL( 18, 4) DEFAULT NULL,
  VAT_2                    DECIMAL( 18, 4) DEFAULT NULL,
  GOODS_3                  DECIMAL( 18, 4) DEFAULT NULL,
  VAT_3                    DECIMAL( 18, 4) DEFAULT NULL,
  GOODS_4                  DECIMAL( 18, 4) DEFAULT NULL,
  VAT_4                    DECIMAL( 18, 4) DEFAULT NULL,
  GOODS_5                  DECIMAL( 18, 4) DEFAULT NULL,
  VAT_5                    DECIMAL( 18, 4) DEFAULT NULL,
  GOODS_6                  DECIMAL( 18, 4) DEFAULT NULL,
  VAT_6                    DECIMAL( 18, 4) DEFAULT NULL,
  GOODS_7                  DECIMAL( 18, 4) DEFAULT NULL,
  VAT_7                    DECIMAL( 18, 4) DEFAULT NULL,
  GOODS_8                  DECIMAL( 18, 4) DEFAULT NULL,
  VAT_8                    DECIMAL( 18, 4) DEFAULT NULL,
  GOODS_9                  DECIMAL( 18, 4) DEFAULT NULL,
  VAT_9                    DECIMAL( 18, 4) DEFAULT NULL,
  GOODS_10                 DECIMAL( 18, 4) DEFAULT NULL,
  VAT_10                   DECIMAL( 18, 4) DEFAULT NULL,
 CONSTRAINT PK_VAT_RETURN_DTL PRIMARY KEY (VAT_PERIOD, IS_INPUT, DAYBOOK_ID)
);

