{*****************************************************************************}
{                                                                             }
{   This file is part of the LKRecords components Library                     }
{                                                                             }
{   See the file COPYING.LGPL.txt included in this distribution,              }
{   for details about the Licence and Copyright.                              }
{                                                                             }
{   This code is distributed in the hope that it will be useful,              }
{   but WITHOUT ANY WARRANTY; without even the implied warranty of            }
{   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      }
{                                                                             }
{   Author: Leslie John Kaye :    www.leskaye.com                             }
{                                                                             }
{   Abstract:                                                                 }
{      TLKRecords components                                                  }
{                                                                             }
{*****************************************************************************}

unit LKRecordsComponents;

interface

uses
  Windows, Messages, SysUtils, Classes, System.Types, System.UITypes,
  ExtCtrls, Graphics, Controls, Forms,
  Dialogs, StdCtrls, DBCtrls, DBGrids, ActnList, ActnMan, ImgList, IniFiles,
  ComCtrls, Registry, Variants, DB, DBTables, DBClient, DBXCommon, SqlExpr,
  Provider, xmldom, XMLIntf, msxmldom, XMLDoc,
  Math, RecError, JvExControls, JvComponent, JvOutlookBar, Contnrs, Menus,
    ShellApi, WideStrings,
  LKDefine, LKFormBaseDlg, LKTableBase, LKBaseComponent, LKDaybookBatchErrorsDlg;

type

  TLKDataComponent = class;
  TLKContainedRecords = class;
  TLKGroup = class;
  TLKGroupList = class;
  TLKHubRecords = class;
  TLKDaybookBase = class;
  TLKDaybookList = class;

  TLKDataComponentNotifyEvent = procedure (ALKComponent: TLKDataComponent) of
      object;
  TLKDlgNotifyEvent = procedure (Dlg: TForm) of object;
  TLKPrinterNotifyEvent = procedure (Sender: TObject; ALKComponent:
      TLKDataComponent) of object;
  TLKSystemComponent = class(TLKComponent)
  private
    FAccountsHub: TLKHubRecords;
    FAfterOpen: TNotifyEvent;
    FBeforeOpen: TNotifyEvent;
    FContainerComponent: TLKSystemComponent;
    FEditing: Boolean;
    FOnEndOfMonth: TNotifyEvent;
    FOnEndOfYear: TNotifyEvent;
    FPrintShareLayouts: Boolean;
    function GetSystemSettings: TLKDataComponent;
    procedure SetAccountsHub(Value: TLKHubRecords);
  protected
    FActive: Boolean;
    FFolderStoreDocumentImages: string;
    FHasPeriodEndProcedure: Boolean;
    FSQLConnection: TSQLConnection;
    FUserName: string;
    uao: Boolean;
    ubo: Boolean;
    procedure AssignTo(Dest: TPersistent); override;
    procedure CreateSubComponents; virtual;
    procedure DoAfterOpen; virtual;
    procedure DoAfterSetHub; virtual;
    procedure DoBeforeOpen; virtual;
    procedure DoEndOfMonth(Sender: TObject);
    procedure DoEndOfYear(Sender: TObject);
    function GetActive: Boolean; virtual;
    function GetFolderStoreDocumentImages: string; virtual;
    function GetSubComponentOwner: TLKSystemComponent; virtual;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation);
        override;
    procedure SaveSettings; virtual;
    procedure SetActive(Value: Boolean); virtual;
    procedure SetContainedComponentNames; virtual;
    procedure SetEditing(Value: Boolean); virtual;
    procedure SetFolderStoreDocumentImages(const Value: string); virtual;
    procedure SetImages(Value: TCustomImageList); override;
    procedure SetName(const NewName: TComponentName); override;
    procedure SetPrintShareLayouts(const Value: Boolean); virtual;
    procedure SetSQLConnection(const Value: TSQLConnection); virtual;
    procedure SetUserName(const Value: string); virtual;
    property ContainerComponent: TLKSystemComponent read FContainerComponent;
  public
    constructor Create(AOwner: TComponent); override;
    procedure AddToTreeView(ATreeView: TTreeView; ATreeNode: TTreeNode);
        override;
    procedure Close;
    procedure DlgSQLEditorExecute;
    function EndOfMonth: TModalResult; virtual;
    function EndOfYear(ProgressText: string): TModalResult; virtual;
    procedure Forget(const Section: string; isSharedSetting: boolean);
    function LoadFromFile: Boolean; virtual;
    procedure Open;
    function RecallBool(const Section, Ident: string; Default: Boolean;
        isSharedSetting: boolean = true): Boolean;
    function RecallCrypt(const Section, Ident, PlainDefault: string; const
        EncryptLength: Integer; IsSharedSetting: Boolean = True): string;
    function RecallDate(const Section, Ident: string; Default: TDateTime;
        isSharedSetting: boolean = true): TDateTime;
    function RecallFloat(const Section, Ident: string; Default: double;
        isSharedSetting: boolean = true): Double;
    function RecallInteger(const Section, Ident: string; Default: Longint;
        isSharedSetting: boolean = true): LongInt;
    procedure RecallSection(const Section: string; Strings: TStrings;
        IsSharedSetting: boolean = true);
    function RecallString(const Section, Ident, Default: string;
        isSharedSetting: boolean = true): string;
    procedure RememberBool(const Section, Ident: string; Value: Boolean;
        isSharedSetting: boolean = true);
    procedure RememberCrypt(const Section, Ident, PlainValue: string; const
        EncryptLength: Integer; IsSharedSetting: Boolean = True);
    procedure RememberDate(const Section, Ident: string; Value: TDateTime;
        isSharedSetting: boolean = true);
    procedure RememberFloat(const Section, Ident: string; Value: double;
        isSharedSetting: boolean = true);
    procedure RememberInteger(const Section, Ident: string; Value: Longint;
        isSharedSetting: boolean = true);
    procedure RememberSection(const Section: string; Strings: TStrings;
        isSharedSetting: boolean = true);
    procedure RememberString(const Section, Ident, Value: string;
        isSharedSetting: boolean = True);
    procedure SaveToFile; virtual;
    function Setup(const ForceIfSettingsExist: Boolean): TModalResult;
    function SQLExecute(const SQL: string): Integer;
    procedure SQLStatementToStrings(const SQL: string; SQLStatements: TStrings);
    property AccountsHub: TLKHubRecords read FAccountsHub write SetAccountsHub;
    property Active: Boolean read GetActive write SetActive;
    property FolderStoreDocumentImages: string read
        GetFolderStoreDocumentImages write SetFolderStoreDocumentImages;
    property HasPeriodEndProcedure: Boolean read FHasPeriodEndProcedure default
        False;
    property PrintShareLayouts: Boolean read FPrintShareLayouts write
        SetPrintShareLayouts default True;
    property SQLConnection: TSQLConnection read FSQLConnection write
        SetSQLConnection;
    property SystemSettings: TLKDataComponent read GetSystemSettings;
    property UserName: string read FUserName write SetUserName;
    property AfterOpen: TNotifyEvent read FAfterOpen write FAfterOpen;
    property BeforeOpen: TNotifyEvent read FBeforeOpen write FBeforeOpen;
  published
    property Editing: Boolean read FEditing write SetEditing nodefault;
    property ImageIndex default -1;
    property NameFriendly;
    property OnEndOfMonth: TNotifyEvent read FOnEndOfMonth write FOnEndOfMonth;
    property OnEndOfYear: TNotifyEvent read FOnEndOfYear write FOnEndOfYear;
  end;

  TLKGroupComponent = class(TLKSystemComponent)
  private
    FACGroup: TLKGroup;
    procedure SetACGroup(Value: TLKGroup);
  protected
    procedure DoAfterSetACGroup; virtual;
    procedure Notification(AComponent: TComponent; Operation: TOperation);
        override;
  public
    function EndOfYear(ProgressText: string): TModalResult; override;
    property ACGroup: TLKGroup read FACGroup write SetACGroup;
  end;

  TLKGroupComponentList = class(TLKComponentList)
  protected
    function GetItems(Index: Integer): TLKGroupComponent;
    procedure SetItems(Index: Integer; AComponent: TLKGroupComponent);
  public
    function Add(AComponent: TLKGroupComponent): Integer;
    function Extract(Item: TLKGroupComponent): TLKGroupComponent;
    function First: TLKGroupComponent;
    function IndexOf(AComponent: TLKGroupComponent): Integer;
    procedure Insert(Index: Integer; AComponent: TLKGroupComponent);
    function Last: TLKGroupComponent;
    property Items[Index: Integer]: TLKGroupComponent read GetItems write
        SetItems; default;
  end;

  TLKSystemReport = class(TLKGroupComponent)
  published
    property ACGroup;
  end;

  TLKActionList = class(TActionList)
  private
    FDataComponent: TLKDataComponent;
  protected
    function GetOwner: TPersistent; override;
    procedure ReadState(Reader: TReader); override;
    procedure SetDataComponent(Value: TLKDataComponent);
    procedure SetParentComponent(AParent: TComponent); override;
  public
    constructor Create(AOwner: TComponent); override;
    function GetParentComponent: TComponent; override;
    function HasParent: Boolean; override;
    property DataComponent: TLKDataComponent read FDataComponent write
        SetDataComponent;
  end;

  TLKDataSourceList = class(TComponentList)
  protected
    function GetItems(Index: Integer): TDataSource;
    procedure SetItems(Index: Integer; AComponent: TDataSource);
  public
    function Add(AComponent: TDataSource): Integer;
    function Extract(Item: TDataSource): TDataSource;
    function First: TDataSource;
    function IndexOf(AComponent: TDataSource): Integer;
    procedure Insert(Index: Integer; AComponent: TDataSource);
    function Last: TDataSource;
    function Remove(AComponent: TDataSource): Integer;
    property Items[Index: Integer]: TDataSource read GetItems write SetItems;
        default;
  end;

  TLKTable = class(TLKTableBase)
  private
    FDataComponent: TLKDataComponent;
    FDataSources: TLKDataSourceList;
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation);
        override;
    procedure SetDataComponent(Value: TLKDataComponent); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure CopyFieldValuesFrom(ASource: TDataset; FieldNames: TStringList);
    procedure DlgPrintExecute(const ATitle, AReportName: string; Code: Variant;
        AutoExecute: boolean);
    function DlgTableFilterExecute(const AFilterFieldName: string = ''):
        Boolean;
    procedure ExternalDataSourceAdd(ADataSource: TDataSource);
    procedure ExternalDataSourceRemove(ADataSource: TDataSource);
    procedure ExternalDisableControls;
    procedure ExternalEnableControls;
    function HasParent: Boolean; override;
    property DataComponent: TLKDataComponent read FDataComponent write
        SetDataComponent;
  end;

  TLKTableMaster = class(TLKTable)
  protected
    procedure DoAfterDelete; override;
    procedure DoAfterInsert; override;
    procedure DoAfterPost; override;
    procedure DoBeforeDelete; override;
    procedure DoBeforeEdit; override;
    procedure DoBeforeInsert; override;
    procedure DoBeforePost; override;
    procedure DoOnCalcFields; override;
    procedure SetDataComponent(Value: TLKDataComponent); override;
  end;

  TLKTableDetail = class(TLKTable)
  protected
    procedure DoAfterDelete; override;
    procedure DoAfterInsert; override;
    procedure DoAfterPost; override;
    procedure DoBeforeDelete; override;
    procedure DoBeforeEdit; override;
    procedure DoBeforeInsert; override;
    procedure DoBeforePost; override;
    procedure SetDataComponent(Value: TLKDataComponent); override;
  published
    property Filter;
    property Filtered;
    property IndexDefs;
    property IndexFieldNames;
    property IndexName;
    property MasterFields;
  end;

  TLKTableView = class(TLKTable)
  end;

  TLKDataComponent = class(TLKGroupComponent)
  private
    FASCIIDateIdentifier: string;
    FASCIIDateSeparator: string;
    FASCIIFieldSeparator: string;
    FASCIIFillCharNumber: string;
    FASCIIFillCharString: string;
    FASCIIFixedFieldSize: Boolean;
    FASCIIStringIdentifier: string;
    FChildComponents: TComponentList;
    FDeleteDetailWithMasterRecord: Boolean;
    FDetailAfterDelete: TDataSetNotifyEvent;
    FDetailAfterInsert: TDataSetNotifyEvent;
    FDetailAfterPost: TDataSetNotifyEvent;
    FDetailBeforeDelete: TDataSetNotifyEvent;
    FDetailBeforeEdit: TDataSetNotifyEvent;
    FDetailBeforeInsert: TDataSetNotifyEvent;
    FDetailBeforePost: TDataSetNotifyEvent;
    FFilters: TStrings;
    FFiltersDetail: TStrings;
    FInternalDetailDS: TDataSource;
    FInternalMasterDS: TDataSource;
    FInUserEdit: Boolean;
    FIsBatchNumbering: Boolean;
    FIsRecordNew: Boolean;
    FMasterAfterDelete: TDataSetNotifyEvent;
    FMasterAfterInsert: TDataSetNotifyEvent;
    FMasterAfterPost: TDataSetNotifyEvent;
    FMasterBeforeDelete: TDataSetNotifyEvent;
    FMasterBeforeEdit: TDataSetNotifyEvent;
    FMasterBeforeInsert: TDataSetNotifyEvent;
    FMasterBeforePost: TDataSetNotifyEvent;
    FMasterComponent: TLKDataComponent;
    FMasterTable: TLKACTable;
    FMyViewAll: TLKDataComponent;
    FMyViewAllDetail: TLKDataComponent;
    FOKToInsert: Boolean;
    FOnCopyDetailRecord: TLKDataComponentNotifyEvent;
    FOnCopyMasterRecord: TLKDataComponentNotifyEvent;
    FOnDetailDrillDown: TNotifyEvent;
    FOnDetailReconcileError: TReconcileErrorEvent;
    FOnExportBefore: TLKDataComponentNotifyEvent;
    FOnExportComplete: TLKDataComponentNotifyEvent;
    FOnExportPrintReport: TLKDataComponentNotifyEvent;
    FOnImportBefore: TLKDataComponentNotifyEvent;
    FOnImportComplete: TLKDataComponentNotifyEvent;
    FOnMasterCalcFields: TDataSetNotifyEvent;
    FOnMasterGridDblClick: TNotifyEvent;
    FOnMasterReconcileError: TReconcileErrorEvent;
    FOnPrintComplete: TLKPrinterNotifyEvent;
    FOnPrintHeader: TLKPrinterNotifyEvent;
    FOnView: TLKDataComponentNotifyEvent;
    FOnViewRecord: TLKDataComponentNotifyEvent;
    FRefreshMSumDPairs: Boolean;
    FSequenceNumberNext: Integer;
    FTblDetail: TLKTableDetail;
    FTblMaster: TLKTableMaster;
    FTdDescriptionFieldName: string;
    function GetChangeCount: Integer;
    function GetDisableStringTrim: Boolean;
    function GetFetchOnDemand: Boolean;
    function GetFilter: string;
    function GetFiltered: Boolean;
    function GetIndexDefs: TIndexDefs;
    function GetIndexFieldNames: Widestring;
    function GetIndexName: Widestring;
    function GetMasterFields: Widestring;
    function GetRecordCount: Integer;
    function GetSequenceNumberNext: Integer;
    function GetTblDetailUsed: Boolean;
    function GetTdDescription: string;
    procedure SetActionList(Value: TLKActionList);
    procedure SetDetailDS(const Value: TDataSource);
    procedure SetDisableStringTrim(Value: Boolean);
    procedure SetFetchOnDemand(Value: Boolean);
    procedure SetFilter(const Value: string);
    procedure SetFiltered(const Value: Boolean);
    procedure SetFilters(const Value: TStrings);
    procedure SetFiltersDetail(Value: TStrings);
    procedure SetIndexDefs(Value: TIndexDefs);
    procedure SetIndexFieldNames(const Value: Widestring);
    procedure SetIndexName(const Value: Widestring);
    procedure SetMasterComponent(const Value: TLKDataComponent);
    procedure SetMasterDS(const Value: TDataSource);
    procedure SetMasterFields(const Value: Widestring);
    procedure SetRefreshMSumDPairs(Value: Boolean);
    procedure SetSequenceActive(Value: Boolean);
    procedure SetSequenceNumberNext(Value: Integer);
    procedure SetTdDescription(const Value: string);
  protected
    FActionList: TLKActionList;
    FFormViewRecord: TFormLKBase;
    FMSumDPairFieldNames: TStrings;
    FMyFieldOwner: TComponent;
    FMyFindDialog: TForm;
    FMyFindDialogDetail: TForm;
    FNewDetailRecord: Boolean;
    FOldIndexName: string;
    FOnPrint: TLKDataComponentNotifyEvent;
    FSequenceActive: Boolean;
    FTotalDetailFieldNames: TStrings;
    FTotalMasterFieldNames: TStrings;
    FWasActive: Boolean;
    procedure AssignTo(Dest: TPersistent); override;
    procedure ChangeKeyFieldValues(FieldNameList: TStringList; OldValues,
        NewValues: array of variant);
    procedure CreateSubComponents; override;
    procedure CreateViewFormRecord; virtual;
    procedure DoCopyDetailRecord; virtual;
    procedure DoCopyMasterRecord; virtual;
    procedure DoDetailAfterDelete; virtual;
    procedure DoDetailAfterInsert; virtual;
    procedure DoDetailAfterPost; virtual;
    procedure DoDetailBeforeDelete; virtual;
    procedure DoDetailBeforeEdit; virtual;
    procedure DoDetailBeforeInsert; virtual;
    procedure DoDetailBeforePost; virtual;
    procedure DoExportBefore; virtual;
    procedure DoExportComplete; virtual;
    procedure DoExportPrintReport; virtual;
    procedure DoImportBefore; virtual;
    procedure DoImportComplete; virtual;
    procedure DoMasterAfterDelete; virtual;
    procedure DoMasterAfterInsert; virtual;
    procedure DoMasterAfterPost; virtual;
    procedure DoMasterBeforeDelete; virtual;
    procedure DoMasterBeforeEdit; virtual;
    procedure DoMasterBeforeInsert; virtual;
    procedure DoMasterBeforePost; virtual;
    procedure DoMasterCalcFields; virtual;
    function DoView: Boolean;
    function DoViewRecord: Boolean;
    function GetActive: Boolean; override;
    function GetFolderStoreDocumentImages: string; override;
    function GetSettings: TModalResult; override;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation);
        override;
    procedure SetActive(Value: Boolean); override;
    procedure SetASCIIDateIdentifier(const Value: string); virtual;
    procedure SetASCIIFillCharNumber(const Value: string); virtual;
    procedure SetASCIIFillCharString(const Value: string); virtual;
    procedure SetASCIIStringIdentifier(const Value: string); virtual;
    procedure SetContainedComponentNames; override;
    procedure SetFolderStoreDocumentImages(const Value: string); override;
    procedure SetImages(AImageList: TCustomImageList); override;
    procedure SetMasterTable(const Value: TLKACTable);
    procedure SetSQLConnection(const Value: TSQLConnection); override;
    function ValidateRecord: string; virtual;
    property TdDescription: string read GetTdDescription write SetTdDescription;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function AccountCheckTotals(const showOK, correctErrors, raiseError:
        boolean): Boolean; virtual;
    procedure Append;
    procedure AppendRecord(const Values: array of const);
    function ApplyUpdates(MaxErrors: Integer): Integer; dynamic;
    procedure Cancel;
    procedure CancelUpdates;
    procedure ChildComponentAdd(Value: TLKDataComponent);
    procedure ChildComponentRemove(Value: TLKDataComponent);
    procedure CopyDetailRecord(ASource: TLKDataComponent; impAutoIncName:
        string; impAutoIncValue: integer; detailFieldNames: TStringList; AMode:
        TBatchMode);
    procedure CopyMasterRecord(ASource: TLKDataComponent; masterFieldNames:
        TStringList; AMode: TBatchMode);
    procedure CopyRecords(ASource: TLKDataComponent; CopyMasterFilter: string;
        CopyTables: TLKACTableSelect; IgnoreExceptions: boolean; AMode:
        TBatchMode); virtual;
    procedure CreateViewForm; override;
    procedure Delete;
    procedure DlgBlockDeleteExecute(AOwner: TComponent; TablesToDelete:
        TLKACTableSelect; masterDeleteFilter, detailDeleteFilter: string);
    procedure DlgChangeKeyFieldValueExecute;
    function DlgFindDetailExecute(AOwner: TComponent; const searchFieldName:
        string): Boolean;
    function DlgFindExecute(const searchFieldName: string): Boolean;
    procedure DlgImportExportExecute(AOwner: TComponent; const AsExportDlg,
        Autoexecute: boolean); virtual;
    function DlgNewExecute: TModalResult; virtual;
    procedure DlgPrintEnvelopeExecute(const Addressee: string; addressFields:
        TLKFieldNumbers; autoExecute: boolean);
    procedure DlgPrintExecute(const ATitle, AReportName: string; key: variant;
        autoExecute: boolean); dynamic;
    function DlgSelectRecordExecute(const aTitle: string; var SearchValues:
        variant): TModalResult;
    function DlgTableFilterExecute(const AFilterTable: TLKACTable;
        AFilterFieldName: string = ''): Boolean;
    function DlgViewExecute: TModalResult; override;
    function DlgViewTotalsExecute(AOwner: TComponent; VTCaption: string;
        AButtons: TMsgDlgButtons): Word;
    procedure DoDetailReconcileError(DataSet: TCustomClientDataSet; E:
        EReconcileError; UpdateKind: TUpdateKind; var Action: TReconcileAction);
        virtual;
    procedure DoMasterReconcileError(DataSet: TCustomClientDataSet; E:
        EReconcileError; UpdateKind: TUpdateKind; var Action: TReconcileAction);
        virtual;
    procedure DoPrintComplete(Sender: TObject);
    procedure DoPrintHeader(Sender: TObject);
    procedure DrillDown; virtual;
    procedure DrillDownRecord; virtual;
    procedure EmptyTables; virtual;
    function ExportData(const masterTextFileName, detailTextFileName: string;
        EMasterfields, EDetailFields: TLKFieldNumbers): Boolean; virtual;
    procedure ExternalDisableControls;
    procedure ExternalEnableControls;
    function FieldByName(const FieldName: string): TField;
    function FindFirst: Boolean;
    function FindKey(const KeyValues: array of const): Boolean;
    function FindLast: Boolean;
    function FindNext: Boolean;
    function FindPrior: Boolean;
    function ImportData(const masterTextFileName, detailTextFileName: string;
        IMasterFields, IDetailFields: TLKFieldNumbers): Boolean; virtual;
    procedure Insert;
    procedure InsertRecord(const Values: array of const); virtual;
    function LoadFromFile: Boolean; override;
    function Locate(const KeyFields: string; const KeyValues: Variant; options:
        TLocateOptions): Boolean;
    function MakeDocumentReference(const PrefixStr, FillChar,
        ReferenceToExpand: string; ResultLength: integer): string;
    procedure MoveFieldsToDatamodule;
    procedure Post;
    procedure RecordDelete; virtual;
    function RecordNew: TModalResult; virtual;
    function RecordNewDetail: Boolean; virtual;
    procedure Refresh;
    procedure RestoreFilter;
    procedure SaveFilter;
    procedure SaveToFile; override;
    function UndoLastChange(FollowChange: Boolean): Boolean;
    procedure Validate(AValMsgList: TStrings); virtual;
    function ValidateExportBatch: string; virtual;
    function View(const KeyValues: Variant; ShowAsModal: Boolean; Buttons:
        TMsgDlgButtons): TModalResult; virtual;
    function ViewAll(const KeyValues: variant; Buttons: TMsgDlgButtons):
        TModalResult;
    function ViewAllDetail(const KeyValues: variant; Buttons: TMsgDlgButtons):
        TModalResult;
    function ViewRecord(const KeyValues: Variant; ShowAsModal: Boolean;
        Buttons: TMsgDlgButtons): TModalResult; virtual;
    property ActionList: TLKActionList read FActionList write SetActionList;
    property ASCIIDateIdentifier: string read FASCIIDateIdentifier write
        FASCIIDateIdentifier;
    property ASCIIDateSeparator: string read FASCIIDateSeparator write
        FASCIIDateSeparator;
    property ASCIIFieldSeparator: string read FASCIIFieldSeparator write
        FASCIIFieldSeparator;
    property ASCIIFillCharNumber: string read FASCIIFillCharNumber write
        SetASCIIFillCharNumber;
    property ASCIIFillCharString: string read FASCIIFillCharString write
        SetASCIIFillCharString;
    property ASCIIFixedFieldSize: Boolean read FASCIIFixedFieldSize write
        FASCIIFixedFieldSize default false;
    property ASCIIStringIdentifier: string read FASCIIStringIdentifier write
        SetASCIIStringIdentifier;
    property ChangeCount: Integer read GetChangeCount;
    property ChildComponents: TComponentList read FChildComponents;
    property DisableStringTrim: Boolean read GetDisableStringTrim write
        SetDisableStringTrim;
    property FetchOnDemand: Boolean read GetFetchOnDemand write
        SetFetchOnDemand;
    property Filter: string read GetFilter write SetFilter;
    property Filtered: Boolean read GetFiltered write SetFiltered;
    property Filters: TStrings read FFilters write SetFilters;
    property FiltersDetail: TStrings read FFiltersDetail write SetFiltersDetail;
    property FormViewRecord: TFormLKBase read FFormViewRecord;
    property IndexDefs: TIndexDefs read GetIndexDefs write SetIndexDefs;
    property IndexFieldNames: Widestring read GetIndexFieldNames write
        SetIndexFieldNames;
    property IndexName: Widestring read GetIndexName write SetIndexName;
    property InternalDetailDS: TDataSource read FInternalDetailDS;
    property InternalMasterDS: TDataSource read FInternalMasterDS;
    property InUserEdit: Boolean read FInUserEdit write FInUserEdit default
        True;
    property IsBatchNumbering: Boolean read FIsBatchNumbering write
        FIsBatchNumbering default False;
    property IsRecordNew: Boolean read FIsRecordNew write FIsRecordNew default
        False;
    property MasterComponent: TLKDataComponent read FMasterComponent write
        SetMasterComponent;
    property MasterFields: Widestring read GetMasterFields write
        SetMasterFields;
    property MasterTable: TLKACTable read FMasterTable write SetMasterTable;
    property OKToInsert: Boolean read FOKToInsert write FOKToInsert default
        False;
    property RecordCount: Integer read GetRecordCount;
    property RefreshMSumDPairs: Boolean read FRefreshMSumDPairs write
        SetRefreshMSumDPairs nodefault;
    property SequenceActive: Boolean read FSequenceActive write
        SetSequenceActive default False;
    property SequenceNumberNext: Integer read GetSequenceNumberNext write
        SetSequenceNumberNext;
    property TblDetail: TLKTableDetail read FTblDetail;
    property TblDetailDataSource: TDataSource write SetDetailDS;
    property TblDetailUsed: Boolean read GetTblDetailUsed;
    property TblMaster: TLKTableMaster read FTblMaster;
    property TblMasterDataSource: TDataSource write SetMasterDS;
    property TdDescriptionFieldName: string read FTdDescriptionFieldName write
        FTdDescriptionFieldName;
    property OnExportBefore: TLKDataComponentNotifyEvent read FOnExportBefore
        write FOnExportBefore;
    property OnExportComplete: TLKDataComponentNotifyEvent read
        FOnExportComplete write FOnExportComplete;
    property OnExportPrintReport: TLKDataComponentNotifyEvent read
        FOnExportPrintReport write FOnExportPrintReport;
    property OnImportBefore: TLKDataComponentNotifyEvent read FOnImportBefore
        write FOnImportBefore;
    property OnImportComplete: TLKDataComponentNotifyEvent read
        FOnImportComplete write FOnImportComplete;
  published
    property Active;
    property AfterOpen;
    property BeforeOpen;
    property DeleteDetailWithMasterRecord: Boolean read
        FDeleteDetailWithMasterRecord write FDeleteDetailWithMasterRecord
        nodefault;
    property DetailAfterDelete: TDataSetNotifyEvent read FDetailAfterDelete
        write FDetailAfterDelete;
    property DetailAfterInsert: TDataSetNotifyEvent read FDetailAfterInsert
        write FDetailAfterInsert;
    property DetailAfterPost: TDataSetNotifyEvent read FDetailAfterPost write
        FDetailAfterPost;
    property DetailBeforeDelete: TDataSetNotifyEvent read FDetailBeforeDelete
        write FDetailBeforeDelete;
    property DetailBeforeEdit: TDataSetNotifyEvent read FDetailBeforeEdit write
        FDetailBeforeEdit;
    property DetailBeforeInsert: TDataSetNotifyEvent read FDetailBeforeInsert
        write FDetailBeforeInsert;
    property DetailBeforePost: TDataSetNotifyEvent read FDetailBeforePost write
        FDetailBeforePost;
    property MasterAfterDelete: TDataSetNotifyEvent read FMasterAfterDelete
        write FMasterAfterDelete;
    property MasterAfterInsert: TDataSetNotifyEvent read FMasterAfterInsert
        write FMasterAfterInsert;
    property MasterAfterPost: TDataSetNotifyEvent read FMasterAfterPost write
        FMasterAfterPost;
    property MasterBeforeDelete: TDataSetNotifyEvent read FMasterBeforeDelete
        write FMasterBeforeDelete;
    property MasterBeforeEdit: TDataSetNotifyEvent read FMasterBeforeEdit write
        FMasterBeforeEdit;
    property MasterBeforeInsert: TDataSetNotifyEvent read FMasterBeforeInsert
        write FMasterBeforeInsert;
    property MasterBeforePost: TDataSetNotifyEvent read FMasterBeforePost write
        FMasterBeforePost;
    property MSumDPairFieldNames: TStrings read FMSumDPairFieldNames write
        FMSumDPairFieldNames;
    property TotalDetailFieldNames: TStrings read FTotalDetailFieldNames
        write FTotalDetailFieldNames;
    property TotalMasterFieldNames: TStrings read FTotalMasterFieldNames
        write FTotalMasterFieldNames;
    property OnCopyDetailRecord: TLKDataComponentNotifyEvent read
        FOnCopyDetailRecord write FOnCopyDetailRecord;
    property OnCopyMasterRecord: TLKDataComponentNotifyEvent read
        FOnCopyMasterRecord write FOnCopyMasterRecord;
    property OnDetailDrillDown: TNotifyEvent read FOnDetailDrillDown write
        FOnDetailDrillDown;
    property OnDetailReconcileError: TReconcileErrorEvent read
        FOnDetailReconcileError write FOnDetailReconcileError;
    property OnMasterCalcFields: TDataSetNotifyEvent read FOnMasterCalcFields
        write FOnMasterCalcFields;
    property OnMasterGridDblClick: TNotifyEvent read FOnMasterGridDblClick
        write FOnMasterGridDblClick;
    property OnMasterReconcileError: TReconcileErrorEvent read
        FOnMasterReconcileError write FOnMasterReconcileError;
    property OnPrint: TLKDataComponentNotifyEvent read FOnPrint write FOnPrint;
    property OnPrintComplete: TLKPrinterNotifyEvent read FOnPrintComplete write
        FOnPrintComplete;
    property OnPrintHeader: TLKPrinterNotifyEvent read FOnPrintHeader write
        FOnPrintHeader;
    property OnView: TLKDataComponentNotifyEvent read FOnView write FOnView;
    property OnViewRecord: TLKDataComponentNotifyEvent read FOnViewRecord write
        FOnViewRecord;
  end;

  TLKRecordsBase = class(TLKDataComponent)
  private
    function GetTdDocumentItemNumber: Integer;
    function GetTmDocumentReference: string;
    procedure SetTdDocumentItemNumber(const Value: Integer);
    procedure SetTmDocumentReference(const Value: string);
  protected
    procedure CreateViewFormRecord; override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure CreateViewForm; override;
    function GetSettings: TModalResult; override;
    property TdDocumentItemNumber: Integer read GetTdDocumentItemNumber write
        SetTdDocumentItemNumber;
    property TmDocumentReference: string read GetTmDocumentReference write
        SetTmDocumentReference;
  published
    property DisableStringTrim;
    property FetchOnDemand;
    property Filter;
    property Filtered;
    property Filters;
    property FiltersDetail;
    property IndexDefs;
    property IndexFieldNames;
    property IndexName;
    property MasterFields;
    property MasterTable;
    property TblDetail;
    property TblMaster;
  end;

  TLKRecordsList = class(TLKComponentList)
  protected
    function GetItems(Index: Integer): TLKRecordsBase;
    procedure SetItems(Index: Integer; AComponent: TLKRecordsBase);
  public
    function Add(AComponent: TLKRecordsBase): Integer;
    function Extract(Item: TLKRecordsBase): TLKRecordsBase;
    function First: TLKRecordsBase;
    function IndexOf(AComponent: TLKRecordsBase): Integer;
    procedure Insert(Index: Integer; AComponent: TLKRecordsBase);
    function Last: TLKRecordsBase;
    property Items[Index: Integer]: TLKRecordsBase read GetItems write SetItems;
        default;
  end;

  TLKRecords = class(TLKRecordsBase)
  protected
    procedure DoAfterSetHub; override;
  published
    property ACGroup;
    property MasterComponent;
  end;

  TLKRecordsASCII = class(TLKRecords)
  public
    constructor Create(AOwner: TComponent); override;
    procedure CreateViewForm; override;
  published
    property ASCIIDateIdentifier;
    property ASCIIDateSeparator;
    property ASCIIFieldSeparator;
    property ASCIIFillCharNumber;
    property ASCIIFillCharString;
    property ASCIIFixedFieldSize;
    property ASCIIStringIdentifier;
    property OnExportBefore;
    property OnExportComplete;
    property OnExportPrintReport;
    property OnImportBefore;
    property OnImportComplete;
  end;

  TLKRecordsInput = class(TLKRecordsBase)
  protected
    FCopyFromExcludeFieldsMaster: TStrings;
    procedure CreateViewFormRecord; override;
    procedure DoAfterSetHub; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure CreateViewForm; override;
    procedure DlgNewRecordCopyFromExecute;
  published
    property ACGroup;
    property MasterComponent;
  end;

  TLKContainedRecords = class(TLKRecordsBase)
  private
    function GetMasterComponentName: string;
  protected
    procedure DoAfterSetACGroup; override;
    function GetSubComponentOwner: TLKSystemComponent; override;
  public
    constructor Create(AOwner: TComponent); override;
    property ContainerComponent;
  published
    property MasterComponentName: string read GetMasterComponentName;
  end;

  TLKRecordsSequenced = class(TLKRecordsInput)
  private
    FDocumentRefPrefix: string;
    FSequenceNumberFillChar: string;
    function GetDocumentRefPrefix: string;
    function GetSequenceNumberFillChar: string;
    function GetTdItemNumber: Integer;
    function GetTmDocumentDate: TDateTime;
    procedure SetDocumentRefPrefix(const Value: string);
    procedure SetSequenceNumberFillChar(const Value: string);
    procedure SetTdItemNumber(Value: Integer);
    procedure SetTmDocumentDate(Value: TDateTime);
  protected
    FDocumentRefLength: Integer;
    procedure CreateViewFormRecord; override;
    procedure DoMasterAfterInsert; override;
    procedure DoMasterBeforeDelete; override;
    procedure DoMasterBeforePost; override;
    function DoSetup(const ForceIfSettingsExist: Boolean): TModalResult;
        override;
    function GetDocumentReferenceFieldNumber: Integer; virtual;
    procedure SetActive(Value: Boolean); override;
    property DocumentReferenceFieldNumber: Integer read
        GetDocumentReferenceFieldNumber;
  public
    constructor Create(AOwner: TComponent); override;
    procedure CreateViewForm; override;
    function DlgBlockInsertRecordsExecute: Boolean; virtual;
    function DlgSetDocumentsSequenceExecute: TModalResult; virtual;
    property DocumentRefPrefix: string read GetDocumentRefPrefix write
        SetDocumentRefPrefix;
    property SequenceNumberFillChar: string read GetSequenceNumberFillChar
        write SetSequenceNumberFillChar;
    property TdItemNumber: Integer read GetTdItemNumber write SetTdItemNumber;
    property TmDocumentDate: TDateTime read GetTmDocumentDate write
        SetTmDocumentDate;
  published
    property DocumentRefLength: Integer read FDocumentRefLength write
        FDocumentRefLength;
  end;

  TLKLedgerBase = class(TLKDataComponent)
  private
    FBalanceFieldName: string;
    FLedgerDate: TDateTime;
    FLedgerID: string;
    FOKToEdit: Boolean;
    FStatus: TLKACStatus;
    function GetStatus: TLKACStatus;
    function GetSuspensePostingsCode: string;
    procedure SetStatus(Value: TLKACStatus);
    procedure SetSuspensePostingsCode(const Value: string);
  protected
    FDaybooks: TLKDaybookList;
    procedure AssignTo(Dest: TPersistent); override;
    procedure CreateViewFormRecord; override;
    procedure DeleteInputDocument; virtual;
    procedure DoAfterSetHub; override;
    procedure DoDetailBeforeDelete; override;
    procedure DoDetailBeforeInsert; override;
    procedure DoDetailBeforePost; override;
    procedure DoMasterBeforeDelete; override;
    function EndOfMonthNext: TDateTime; virtual;
    function EndOfYearFilter(TransactionMonthsToRetain: Integer): string;
        virtual;
    function GetLedgerDate: TDateTime; virtual;
    function GettmAccountName: string; virtual;
    function GettmAccountReference: string; virtual;
    procedure Loaded; override;
    procedure SetBalanceFieldName(const Value: string); virtual;
    procedure SetBalanceFieldPairs; virtual;
    procedure SetLedgerDate(Value: TDateTime); virtual;
    procedure SetLedgerID(const Value: string); virtual;
    procedure SettmAccountName(const Value: string); virtual;
    procedure SettmAccountReference(const Value: string); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function AccountCheckTotals(const showOK, correctErrors, raiseError:
        boolean): Boolean; override;
    procedure AccountClear; virtual;
    function AccountIsValid(const KeyValues: Variant; RaiseError: Boolean =
        False): Boolean; virtual;
    procedure AccountMerge(AOwner: TComponent); virtual;
    procedure AccountPost(const DetailRecord: array of const); virtual;
    procedure CreateViewForm; override;
    procedure DaybookAdd(ADaybook: TLKDaybookBase);
    procedure DaybookRemove(ADaybook: TLKDaybookBase);
    procedure DlgLedgerAllocateExecute; virtual;
    procedure DoDetailReconcileError(DataSet: TCustomClientDataSet; E:
        EReconcileError; UpdateKind: TUpdateKind; var Action: TReconcileAction);
        override;
    procedure DoMasterReconcileError(DataSet: TCustomClientDataSet; E:
        EReconcileError; UpdateKind: TUpdateKind; var Action: TReconcileAction);
        override;
    procedure DrillDownRecord; override;
    procedure EmptyTables; override;
    function EndOfMonth: TModalResult; override;
    procedure InsertRecord(const Values: array of const); override;
    procedure RecordDelete; override;
    function RecordNew: TModalResult; override;
    function SumLedgerBalances: Currency;
    property Daybooks: TLKDaybookList read FDaybooks;
    property LedgerDate: TDateTime read GetLedgerDate write SetLedgerDate;
    property OKToEdit: Boolean read FOKToEdit write FOKToEdit;
    property Status: TLKACStatus read GetStatus write SetStatus;
    property SuspensePostingsCode: string read GetSuspensePostingsCode write
        SetSuspensePostingsCode;
    property tmAccountName: string read GettmAccountName write SettmAccountName;
    property tmAccountReference: string read GettmAccountReference write
        SettmAccountReference;
  published
    property ACGroup;
    property BalanceFieldName: string read FBalanceFieldName write
        SetBalanceFieldName;
    property DisableStringTrim;
    property FetchOnDemand;
    property Filter;
    property Filtered;
    property Filters;
    property FiltersDetail;
    property IndexDefs;
    property IndexFieldNames;
    property IndexName;
    property LedgerID: string read FLedgerID write SetLedgerID;
    property TblDetail;
    property TblMaster;
  end;

  TLKLedgerList = class(TLKComponentList)
  protected
    function GetItems(Index: Integer): TLKLedgerBase;
    procedure SetItems(Index: Integer; AComponent: TLKLedgerBase);
  public
    function Add(AComponent: TLKLedgerBase): Integer;
    function ComponentByID(const ALedgerID: string): TLKLedgerBase;
    function Extract(Item: TLKLedgerBase): TLKLedgerBase;
    function First: TLKLedgerBase;
    function IndexOf(AComponent: TLKLedgerBase): Integer;
    procedure Insert(Index: Integer; AComponent: TLKLedgerBase);
    function Last: TLKLedgerBase;
    property Items[Index: Integer]: TLKLedgerBase read GetItems write SetItems;
        default;
  end;

  TLKDaybookBase = class(TLKDataComponent)
  private
    FBatchNumber: Integer;
    FDaybookID: string;
    FInputDocuments: TLKRecordsBase;
    FLedger: TLKDataComponent;
    FLedgerAccountNoFieldName: string;
    function GetBatchNumber: Integer;
    function GetDocumentDescription: string;
    function GetLedgerAccountNoFieldName: string;
    function GetTblDDocumentItemNumber: Integer;
    function GetTblMDocumentDate: TDateTime;
    function GetTblMDocumentReference: string;
    procedure RemoveBatchNumber(const BatchNumber: integer);
    procedure SetBatchNumber(Value: Integer);
    procedure SetDaybookID(const Value: string);
    procedure SetInputDocuments(Value: TLKRecordsBase);
    procedure SetLedger(Value: TLKDataComponent);
    procedure SetTblDDocumentItemNumber(const Value: Integer);
    procedure SetTblMDocumentDate(Value: TDateTime);
    procedure SetTblMDocumentReference(const Value: string);
  protected
    FBatchDate: TDateTime;
    FDlgDaybookErrors: TDlgLKDaybookBatchErrors;
    FDocumentDescriptionlFieldNames: TStrings;
    LedgerTempURN: Integer;
    procedure AssignTo(Dest: TPersistent); override;
    procedure DoAfterSetHub; override;
    function GetBatchDate: TDateTime; virtual;
    function GetBatchDescription: string; virtual;
    function GetSettings: TModalResult; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation);
        override;
    procedure PbAfterProcessBatch; virtual;
    procedure PbBeforePostDocument; virtual;
    procedure PbBeforeProcessBatch; virtual;
    function PbCheckBatchIsValid: TModalResult; virtual;
    procedure PbCheckBatchItemDetailIsValid; virtual;
    procedure PbCheckBatchItemMasterIsValid; virtual;
    function PbDlgDaybookErrorsCorrectError(ErrorInfo: TStrings): TModalResult;
        virtual;
    function PbDlgDaybookErrorsCorrectErrors: TModalResult; virtual;
    procedure PbDlgDaybookErrorsCorrectWarnings; virtual;
    procedure PbDlgDaybookErrorsCreate; virtual;
    procedure PbFinalisePostings; virtual;
    procedure PbPostDocumentToLedger; virtual;
    procedure PbPrintPostingsReport; virtual;
    procedure PbProcessDocument; virtual;
    procedure PbProcessedComplete; virtual;
    procedure PbRemoveFilter; virtual;
    function PbSetFilter: Boolean; virtual;
    procedure PbTablesApplyUpdates; virtual;
    procedure PbTablesCancelUpdates; virtual;
    procedure PbTablesRefresh; virtual;
    procedure SetActive(Value: Boolean); override;
    procedure SetBatchDate(Value: TDateTime); virtual;
    procedure SetDocumentDescriptionlFieldNames(const Value: TStrings);
    property BatchDescription: string read GetBatchDescription;
    property DocumentDescription: string read GetDocumentDescription;
    property LedgerAccountNoFieldName: string read GetLedgerAccountNoFieldName;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure CreateViewForm; override;
    procedure DlgRemoveBatchNumberExecute;
    procedure DlgSetBatchNumberExecute;
    procedure DrillDown; override;
    procedure ProcessBatch(AOwner: TComponent);
    function View(const KeyValues: variant; ShowAsModal: boolean; Buttons:
        TMsgDlgButtons): TModalResult; override;
    property BatchDate: TDateTime read GetBatchDate write SetBatchDate;
    property BatchNumber: Integer read GetBatchNumber write SetBatchNumber
        default 0;
    property DlgDaybookErrors: TDlgLKDaybookBatchErrors read FDlgDaybookErrors
        write FDlgDaybookErrors;
    property InputDocuments: TLKRecordsBase read FInputDocuments write
        SetInputDocuments;
    property Ledger: TLKDataComponent read FLedger write SetLedger;
    property TblDDocumentItemNumber: Integer read GetTblDDocumentItemNumber
        write SetTblDDocumentItemNumber;
    property TblMDocumentDate: TDateTime read GetTblMDocumentDate write
        SetTblMDocumentDate;
    property TblMDocumentReference: string read GetTblMDocumentReference write
        SetTblMDocumentReference;
  published
    property ACGroup;
    property DaybookID: string read FDaybookID write SetDaybookID;
    property DocumentDescriptionlFieldNames: TStrings read
        FDocumentDescriptionlFieldNames write SetDocumentDescriptionlFieldNames;
  end;

  TLKDaybookList = class(TLKComponentList)
  protected
    function GetItems(Index: Integer): TLKDaybookBase;
    procedure SetItems(Index: Integer; AComponent: TLKDaybookBase);
  public
    function Add(AComponent: TLKDaybookBase): Integer;
    function ComponentByID(const ADaybookID: string): TLKDaybookBase;
    function Extract(Item: TLKDaybookBase): TLKDaybookBase;
    function First: TLKDaybookBase;
    function IndexOf(AComponent: TLKDaybookBase): Integer;
    procedure Insert(Index: Integer; AComponent: TLKDaybookBase);
    function Last: TLKDaybookBase;
    property Items[Index: Integer]: TLKDaybookBase read GetItems write SetItems;
        default;
  end;

  TLKDaybookSimple = class(TLKDaybookBase)
  private
    FOnDaybookApplyUpdates: TLKDataComponentNotifyEvent;
    FOnDaybookBeforeProcessBatch: TLKDataComponentNotifyEvent;
    FOnDaybookCancelUpdates: TLKDataComponentNotifyEvent;
    FOnDaybookCheckBatchItemDetailIsValid: TLKDataComponentNotifyEvent;
    FOnDaybookCheckBatchItemMasterIsValid: TLKDataComponentNotifyEvent;
    FOnDaybookFinalisePostings: TLKDataComponentNotifyEvent;
    FOnDaybookProcessDocument: TLKDataComponentNotifyEvent;
    FOnDaybookRefresh: TLKDataComponentNotifyEvent;
    FOnDaybookSetFilter: TLKDataComponentNotifyEvent;
    FOnLedgerDrillDown: TLKDataComponentNotifyEvent;
  protected
    DocsHaveSalesTax: Boolean;
    NominalBatchTotal: Currency;
    TotalSalesTax: Currency;
    procedure PbBeforeProcessBatch; override;
    procedure PbCheckBatchItemDetailIsValid; override;
    procedure PbCheckBatchItemMasterIsValid; override;
    procedure PbFinalisePostings; override;
    procedure PbProcessDocument; override;
    function PbSetFilter: Boolean; override;
    procedure PbTablesApplyUpdates; override;
    procedure PbTablesCancelUpdates; override;
    procedure PbTablesRefresh; override;
  public
    procedure DrillDown; override;
  published
    property Ledger;
    property OnDaybookApplyUpdates: TLKDataComponentNotifyEvent read
        FOnDaybookApplyUpdates write FOnDaybookApplyUpdates;
    property OnDaybookBeforeProcessBatch: TLKDataComponentNotifyEvent read
        FOnDaybookBeforeProcessBatch write FOnDaybookBeforeProcessBatch;
    property OnDaybookCancelUpdates: TLKDataComponentNotifyEvent read
        FOnDaybookCancelUpdates write FOnDaybookCancelUpdates;
    property OnDaybookCheckBatchItemDetailIsValid: TLKDataComponentNotifyEvent
        read FOnDaybookCheckBatchItemDetailIsValid write
        FOnDaybookCheckBatchItemDetailIsValid;
    property OnDaybookCheckBatchItemMasterIsValid: TLKDataComponentNotifyEvent
        read FOnDaybookCheckBatchItemMasterIsValid write
        FOnDaybookCheckBatchItemMasterIsValid;
    property OnDaybookFinalisePostings: TLKDataComponentNotifyEvent read
        FOnDaybookFinalisePostings write FOnDaybookFinalisePostings;
    property OnDaybookProcessDocument: TLKDataComponentNotifyEvent read
        FOnDaybookProcessDocument write FOnDaybookProcessDocument;
    property OnDaybookRefresh: TLKDataComponentNotifyEvent read
        FOnDaybookRefresh write FOnDaybookRefresh;
    property OnDaybookSetFilter: TLKDataComponentNotifyEvent read
        FOnDaybookSetFilter write FOnDaybookSetFilter;
    property OnLedgerDrillDown: TLKDataComponentNotifyEvent read
        FOnLedgerDrillDown write FOnLedgerDrillDown;
  end;

  TLKRecordsPosting = class(TLKRecordsSequenced)
  private
    FDaybook: TLKDaybookBase;
    procedure SetDaybook(Value: TLKDaybookBase);
  protected
    procedure AssignTo(Dest: TPersistent); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation);
        override;
  public
    procedure CreateViewForm; override;
    procedure DaybookProcessBatch(AOwner: TComponent);
    property Daybook: TLKDaybookBase read FDaybook write SetDaybook;
  end;

  TLKGroup = class(TLKSystemComponent)
  private
    FGroupComponents: TLKGroupComponentList;
    FMasterGroup: TLKGroup;
    FPassword: string;
    FPasswordOK: Boolean;
    procedure SetMasterGroup(Value: TLKGroup);
    procedure SetPassword(const Value: string);
  protected
    FSubGroups: TLKGroupList;
    procedure DoAfterSetHub; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation);
        override;
    procedure SetActive(Value: Boolean); override;
    procedure SetEditing(Value: Boolean); override;
    procedure SetFolderLocalSettings(const Value: string); override;
    procedure SetFolderSharedSettings(const Value: string); override;
    procedure SetImages(Value: TCustomImageList); override;
    procedure SetPrintShareLayouts(const Value: Boolean); override;
    procedure SetSQLConnection(const Value: TSQLConnection); override;
    procedure SetUserName(const Value: string); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure AddToActionList(AActionList: TCustomActionList); override;
    procedure AddToOutlookBar(AnOutlookBar: TJvOutlookBar); override;
    procedure AddToTreeView(ATreeView: TTreeView; ATreeNode: TTreeNode);
        override;
    procedure CreateViewForm; override;
    function FindComponent(AComponentName: string): TLKGroupComponent;
    function GetSettings: TModalResult; override;
    procedure GroupComponentAdd(AGroupComponent: TLKGroupComponent); dynamic;
    procedure GroupComponentRemove(AGroupComponent: TLKGroupComponent); dynamic;
    procedure SubGroupAdd(AACgroup: TLKGroup); virtual;
    procedure SubGroupRemove(AACGroup: TLKGroup); virtual;
    property GroupComponents: TLKGroupComponentList read FGroupComponents;
  published
    property AccountsHub;
    property Active;
    property AfterOpen;
    property BeforeOpen;
    property MasterGroup: TLKGroup read FMasterGroup write SetMasterGroup;
    property Password: string read FPassword write SetPassword;
  end;

  TLKGroupList = class(TLKComponentList)
  protected
    function GetItems(Index: Integer): TLKGroup;
    procedure SetItems(Index: Integer; AComponent: TLKGroup);
  public
    function Add(AComponent: TLKGroup): Integer;
    function Extract(Item: TLKGroup): TLKGroup;
    function First: TLKGroup;
    function IndexOf(AComponent: TLKGroup): Integer;
    procedure Insert(Index: Integer; AComponent: TLKGroup);
    function Last: TLKGroup;
    property Items[Index: Integer]: TLKGroup read GetItems write SetItems;
        default;
  end;

  TLKHubContainedDocuments = class(TLKContainedRecords)
  private
    function GetContainerComponent: TLKHubRecords;
  public
    constructor Create(AOwner: TComponent); override;
    property ContainerComponent: TLKHubRecords read GetContainerComponent;
  end;

  TLKHubSystemSettings = class(TLKHubContainedDocuments)
  public
    constructor Create(AOwner: TComponent); override;
  end;

  TLKHubRecordsDiary = class(TLKHubContainedDocuments)
  public
    constructor Create(AOwner: TComponent); override;
    procedure CreateViewForm; override;
  end;

  TLKHubRecords = class(TLKSystemComponent)
  private
    FDataTables: TStrings;
    FDBMS: TLKSupportedDatabaseType;
    FDBUserName: string;
    FDBUserPassword: string;
    FEnterpriseAccountsYearCurrent: string;
    FEnterpriseCurrent: string;
    FEnterpriseDefault: string;
    FEnterprises: TStrings;
    FEnterpriseYears: TStrings;
    FFolderBackup: string;
    FFolderBackupRemovable: string;
    FGroups: TLKGroupList;
    FSystemSettings: TLKDataComponent;
    FUseEmbeddedFirebird: string;
    FUserEmailAddr: string;
    FUsers: TStrings;
    FUserSupportURL: string;
    function GetAdministratorPassword: string;
    function GetDatabaseHost: string;
    function GetDBMS: TLKSupportedDatabaseType;
    function GetDBUserName: string;
    function GetDBUserPassword: string;
    function GetDivision: string;
    function GetDoBackup: Boolean;
    function GetDoBackupRemovable: Boolean;
    function GetEnterpriseAccountsEmail: string;
    function GetEnterpriseAccountsFax: string;
    function GetEnterpriseAccountsTelephone: string;
    function GetEnterpriseAccountsYearCurrent: string;
    function GetEnterpriseAccountsYearDefault: string;
    function GetEnterpriseAddress1: string;
    function GetEnterpriseAddress2: string;
    function GetEnterpriseAddress3: string;
    function GetEnterpriseAddress4: string;
    function GetEnterpriseAddressPostcode: string;
    function GetEnterpriseCurrent: string;
    function GetEnterpriseDefault: string;
    function GetEnterpriseName: string;
    function GetEnterprises: TStrings;
    function GetEnterpriseYears: TStrings;
    function GetFilesBackupDate: TDateTime;
    function GetFolderBackup: string;
    function GetFolderBackupRemovable: string;
    function GetUserEmailAddr: string;
    function GetUserShowLogon: Boolean;
    procedure SetAdministratorPassword(const Value: string);
    procedure SetDatabaseHost(const Value: string);
    procedure SetDBMS(Value: TLKSupportedDatabaseType);
    procedure SetDBUserName(const Value: string);
    procedure SetDBUserPassword(const Value: string);
    procedure SetDoBackup(Value: Boolean);
    procedure SetDoBackupRemovable(Value: Boolean);
    procedure SetEnterpriseAccountsEmail(const Value: string);
    procedure SetEnterpriseAccountsFax(const Value: string);
    procedure SetEnterpriseAccountsTelephone(const Value: string);
    procedure SetEnterpriseAccountsYearCurrent(const Value: string);
    procedure SetEnterpriseAccountsYearDefault(const Value: string);
    procedure SetEnterpriseAddress1(const Value: string);
    procedure SetEnterpriseAddress2(const Value: string);
    procedure SetEnterpriseAddress3(const Value: string);
    procedure SetEnterpriseAddress4(const Value: string);
    procedure SetEnterpriseAddressPostcode(const Value: string);
    procedure SetEnterpriseCurrent(const Value: string);
    procedure SetEnterpriseDefault(const Value: string);
    procedure SetEnterpriseName(const Value: string);
    procedure SetEnterprises(Value: TStrings);
    procedure SetEnterpriseYears(Value: TStrings);
    procedure SetFilesBackupDate(Value: TDateTime);
    procedure SetFolderBackup(const Value: string);
    procedure SetFolderBackupRemovable(const Value: string);
    procedure SetUserEmailAddr(const Value: string);
    procedure SetUsers(Value: TStrings);
    procedure SetUserShowLogon(Value: Boolean);
  protected
    FDatabaseHost: string;
    FDaybooks: TLKDaybookList;
    FDiary: TLKHubRecordsDiary;
    FLedgers: TLKLedgerList;
    FRecords: TLKRecordsList;
    procedure CreateDiary; virtual;
    procedure CreateSubComponents; override;
    function DoSetup(const ForceIfSettingsExist: Boolean): TModalResult;
        override;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation);
        override;
    procedure SetActive(Value: Boolean); override;
    procedure SetContainedComponentNames; override;
    procedure SetFolderLocalSettings(const Value: string); override;
    procedure SetFolderSharedSettings(const Value: string); override;
    procedure SetImages(Value: TCustomImageList); override;
    procedure SetPrintShareLayouts(const Value: Boolean); override;
    procedure SetSQLConnection(const Value: TSQLConnection); override;
    procedure SetSQLConnectionParams;
    procedure SetUserName(const Value: string); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure AddToActionList(AActionList: TCustomActionList); override;
    procedure AddToOutlookBar(AnOutlookBar: TJvOutlookBar); override;
    procedure AddToTreeView(ATreeView: TTreeView; ATreeNode: TTreeNode);
        override;
    function CheckDatabaseExists: Boolean;
    procedure DaybookAdd(ADaybook: TLKDaybookBase);
    procedure DaybookRemove(ADaybook: TLKDaybookBase);
    function DlgAdminPasswordExecute: TModalResult;
    function DlgAdminPasswordSetExecute: TModalResult;
    function DlgDatabaseExecute: TModalResult; virtual;
    function DlgEnterpriseExecute: TModalResult; virtual;
    function DlgPasswordExecute: TModalResult;
    procedure DlgSetupExecute;
    function DlgSetupWizardExecute: TModalResult;
    procedure DlgSupportExecute; virtual;
    procedure DlgUsersSetupExecute;
    procedure DrillDownDaybook(const DaybookID, DocumentReference: string);
    procedure DrillDownDiary; virtual;
    function EndOfMonth: TModalResult; override;
    function EndOfYear(ProgressText: string): TModalResult; override;
    procedure FilesBackup(const BackupRemovable: Boolean); virtual;
    procedure FilesCheckForBackup;
    procedure FilesRestore;
    procedure FilesRestoreRemovable;
    function FindComponent(AComponentName: string): TLKGroupComponent;
    function FindDaybook(const ADaybookID: string): TLKDaybookBase;
    function FindGroup(AGroupName: string): TLKGroup;
    function FindLedger(const ALedgerID: string): TLKLedgerBase;
    function GetSettings: TModalResult; override;
    procedure GroupAdd(AGroup: TLKGroup);
    procedure GroupRemove(AGroup: TLKGroup);
    procedure LedgerAdd(ALedger: TLKLedgerBase);
    procedure LedgerRemove(ALedger: TLKLedgerBase);
    procedure RecordsAdd(ARecords: TLKRecordsBase);
    procedure RecordsRemove(ARecords: TLKRecordsBase);
    procedure SaveTables(TableList: TStringList; const FullSQLFileName: string;
        const SaveDatabase, SaveTableDescr, SaveTables: boolean);
    function SequenceGet(const Description: string): Integer;
    procedure SequenceSet(const Description: string; Value: integer);
    procedure UserAdd(const AUserName, APassword: string);
    procedure UserDelete(const AUserName: string);
    procedure Validate(AValMsgList: TStrings); virtual;
    property AdministratorPassword: string read GetAdministratorPassword write
        SetAdministratorPassword;
    property DatabaseHost: string read GetDatabaseHost write SetDatabaseHost;
    property DataTables: TStrings read FDataTables;
    property Daybooks: TLKDaybookList read FDaybooks;
    property DBMS: TLKSupportedDatabaseType read GetDBMS write SetDBMS;
    property DBUserName: string read GetDBUserName write SetDBUserName;
    property DBUserPassword: string read GetDBUserPassword write
        SetDBUserPassword;
    property Division: string read GetDivision;
    property DoBackup: Boolean read GetDoBackup write SetDoBackup;
    property DoBackupRemovable: Boolean read GetDoBackupRemovable write
        SetDoBackupRemovable;
    property EnterpriseAccountsEmail: string read GetEnterpriseAccountsEmail
        write SetEnterpriseAccountsEmail;
    property EnterpriseAccountsFax: string read GetEnterpriseAccountsFax write
        SetEnterpriseAccountsFax;
    property EnterpriseAccountsTelephone: string read
        GetEnterpriseAccountsTelephone write SetEnterpriseAccountsTelephone;
    property EnterpriseAccountsYearCurrent: string read
        GetEnterpriseAccountsYearCurrent write SetEnterpriseAccountsYearCurrent;
    property EnterpriseAccountsYearDefault: string read
        GetEnterpriseAccountsYearDefault write SetEnterpriseAccountsYearDefault;
    property EnterpriseAddress1: string read GetEnterpriseAddress1 write
        SetEnterpriseAddress1;
    property EnterpriseAddress2: string read GetEnterpriseAddress2 write
        SetEnterpriseAddress2;
    property EnterpriseAddress3: string read GetEnterpriseAddress3 write
        SetEnterpriseAddress3;
    property EnterpriseAddress4: string read GetEnterpriseAddress4 write
        SetEnterpriseAddress4;
    property EnterpriseAddressPostcode: string read
        GetEnterpriseAddressPostcode write SetEnterpriseAddressPostcode;
    property EnterpriseCurrent: string read GetEnterpriseCurrent write
        SetEnterpriseCurrent;
    property EnterpriseDefault: string read GetEnterpriseDefault write
        SetEnterpriseDefault;
    property EnterpriseName: string read GetEnterpriseName write
        SetEnterpriseName;
    property Enterprises: TStrings read GetEnterprises write SetEnterprises;
    property EnterpriseYears: TStrings read GetEnterpriseYears write
        SetEnterpriseYears;
    property FilesBackupDate: TDateTime read GetFilesBackupDate write
        SetFilesBackupDate;
    property FolderBackup: string read GetFolderBackup write SetFolderBackup;
    property FolderBackupRemovable: string read GetFolderBackupRemovable write
        SetFolderBackupRemovable;
    property Groups: TLKGroupList read FGroups;
    property Ledgers: TLKLedgerList read FLedgers;
    property Records: TLKRecordsList read FRecords;
    property UserEmailAddr: string read GetUserEmailAddr write SetUserEmailAddr;
    property Users: TStrings read FUsers write SetUsers;
    property UserShowLogon: Boolean read GetUserShowLogon write
        SetUserShowLogon;
  published
    property Active;
    property AfterOpen;
    property BeforeOpen;
    property Diary: TLKHubRecordsDiary read FDiary;
    property Images;
    property SQLConnection;
    property SystemSettings: TLKDataComponent read FSystemSettings;
    property UserSupportURL: string read FUserSupportURL write FUserSupportURL;
  end;

procedure Register;

implementation

uses
  LKUtils, LKUtilsDB, LKUtilComponents, LKSystemComponentSQLWizardDlg,
  LKdlgs, LKWrZipDlg, LKZipProgressDlg,
  LKHubSetupWizardDlg, LKHubEnterPasswdDlg, LKHubEditUsersDlg, LKHubDatabaseDlg,
  LKHubBackupNagDlg, LKHubEnterpriseDlg, LKHubDiaryDlg,
  LKHubEndOfMonthWizDlg, LKHubEndOfYearWizDlg,
  LKGroupDlg, LKGroupEnterPasswdDlg,
  LKDataCompChangeKeyFieldValueDlg,
  LKDataComponentDlg, LKDataComponentRecordDlg,
  LKRecordsBaseDlg, LKRecordsBaseRecordDlg,
  LKRecordsInputDlg, LKRecordsInputRecordDlg,
  LKRecordsInputCopyFromDlg,
  LKRecordsSequencedRecordDlg,
  LKRecordsASCIIDlg, LKRecordsSequenceSetDlg, LKRecordsSequencedDlg,
  LKRecordsPostingDlg,
  LKDataCompBlockDelDlg, LKDataCompTotalsDlg, LKFindDg,
  LKDataCompImpExpSetDlg,
  LKDaybookBaseDlg,
  LKLedgerBaseDlg, LKLedgerBaseRecordDlg,
  LKTblFilterDg,
  prLKTableBasePrintRecordDlg, prLKRecordDlg, prLKEnvelopeDlg,
  LKDataCompSelectRecordWiz;

procedure Register;
begin
  // RegisterComponents('LKRecords', [TLKHub]);
  RegisterComponents('LKRecords', [TLKHubRecords]);
  RegisterComponents('LKRecords', [TLKTableView]);
  RegisterComponents('LKRecords', [TLKGroup]);
  RegisterComponents('LKRecords', [TLKRecords]);
  RegisterComponents('LKRecords', [TLKRecordsSequenced]);
  RegisterComponents('LKRecords', [TLKDaybookSimple]);
end;
{
****************************** TLKSystemComponent ******************************
}
constructor TLKSystemComponent.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FEditing := True;
  FUserName := 'DEFAULT';
  FContainerComponent := GetSubComponentOwner;
  CreateSubComponents;
//  SetContainedComponentNames;
end;

procedure TLKSystemComponent.AddToTreeView(ATreeView: TTreeView; ATreeNode:
    TTreeNode);
var
  I: Integer;
begin
  inherited AddToTreeView(ATreeView, ATreeNode);
  if FTreeNodeVisible then
  begin
    for i := 0 to ComponentCount - 1 do
      if Components[i] is TLKContainedRecords then
        TLKContainedRecords(Components[i]).AddToTreeView(ATreeView, TreeNode);
  end;
end;

procedure TLKSystemComponent.AssignTo(Dest: TPersistent);
var
  DestACB: TLKSystemComponent;
begin
  inherited;
  DestACB := TLKSystemComponent(Dest);
  DestACB.Editing := Editing;
  DestACB.Images := Images;
  DestACB.SQLConnection := SQLConnection;
end;

procedure TLKSystemComponent.Close;
begin
  SetActive(False);
end;

procedure TLKSystemComponent.CreateSubComponents;
begin
  // override as required
end;

procedure TLKSystemComponent.DlgSQLEditorExecute;
begin
  with TDlgLKSystemComponentSQLWizard.Create(Owner) do
  try
    Construct(Self);
    ShowModal;
  finally Free;
  end;
end;

procedure TLKSystemComponent.DoAfterOpen;
begin
  if Assigned(FAfterOpen) then
    FAfterOpen(Self);
end;

procedure TLKSystemComponent.DoAfterSetHub;
begin
  //
end;

procedure TLKSystemComponent.DoBeforeOpen;
begin
  if Assigned(FBeforeOpen) then
    FBeforeOpen(Self);
end;

procedure TLKSystemComponent.DoEndOfMonth(Sender: TObject);
begin
  if Assigned(FOnEndOfMonth) then
    FOnEndOfMonth(Self);
end;

procedure TLKSystemComponent.DoEndOfYear(Sender: TObject);
begin
  if Assigned(FOnEndOfYear) then
    FOnEndOfYear(Self);
end;

function TLKSystemComponent.EndOfMonth: TModalResult;
begin
  DoEndOfMonth(Self);
  Result := mrOK;
end;

function TLKSystemComponent.EndOfYear(ProgressText: string): TModalResult;
begin
  DoEndOfYear(Self);
  Result := mrOK;
end;

procedure TLKSystemComponent.Forget(const Section: string; isSharedSetting:
    boolean);
var
  usr: string;
begin
  if isSharedSetting then
    usr := '~'
  else
    usr := UserName;
  with SystemSettings.TblMaster do
  begin
    Filter := 'USER_ID = ' + LKSQLStrDelimiter + usr + LKSQLStrDelimiter
      + ' AND COMPONENT = ' + LKSQLStrDelimiter + NameFriendly +
        LKSQLStrDelimiter
      + ' AND SECTION = ' + LKSQLStrDelimiter + Section + LKSQLStrDelimiter;
    try
      Filtered := True;
      while FindLast do
        Delete;
      ApplyUpdates(0);
    finally Filter := '';
      Filtered := False;
    end;
  end;
end;

function TLKSystemComponent.GetActive: Boolean;
begin
  Result := FActive;
end;

function TLKSystemComponent.GetFolderStoreDocumentImages: string;
begin
  if (FFolderStoreDocumentImages = '') and not (csLoading in ComponentState)
    then
    FFolderStoreDocumentImages := RecallString('Settings',
      'FolderStoreDocumentImages', '', True);
  Result := FFolderStoreDocumentImages;
end;

function TLKSystemComponent.GetSubComponentOwner: TLKSystemComponent;
begin
  Result := Self;
end;

function TLKSystemComponent.GetSystemSettings: TLKDataComponent;
begin
  if Assigned(AccountsHub) then
    Result := AccountsHub.SystemSettings
  else Result := nil;
end;

procedure TLKSystemComponent.Loaded;
var
  I: Integer;
begin
  inherited Loaded;
  //AccountsHub := FAccountsHub;
  for i := 0 to ComponentCount - 1 do
    if Components[i] is TLKContainedRecords then
      TLKContainedRecords(Components[i]).AccountsHub := AccountsHub;
end;

function TLKSystemComponent.LoadFromFile: Boolean;
var
  i: Integer;
begin
  Result := True;
  for i := 0 to ComponentCount - 1 do
  begin
    Result := TLKSystemComponent(Components[i]).LoadFromFile;
    if not Result then
      Exit;
  end;
end;

procedure TLKSystemComponent.Notification(AComponent: TComponent; Operation:
    TOperation);
begin
  if (Operation = opRemove) then
  begin
    if (AComponent = FImages) then
      FImages := nil;
    if (AComponent = FAccountsHub) then
      FAccountsHub := nil;
    if (AComponent = FSQLConnection) then
      SQLConnection := nil;
  end;
  inherited Notification(AComponent, Operation);
end;

procedure TLKSystemComponent.Open;
begin
  SetActive(True);
end;

function TLKSystemComponent.RecallBool(const Section, Ident: string; Default:
    Boolean; isSharedSetting: boolean = true): Boolean;
begin
  Result := (RecallString(Section, Ident, BooleanToYesNo(Default),
    isSharedSetting) = 'Yes');
end;

function TLKSystemComponent.RecallCrypt(const Section, Ident, PlainDefault:
    string; const EncryptLength: Integer; IsSharedSetting: Boolean = True):
    string;
begin
  Result := RecallString(Section, Ident, '', IsSharedSetting);
  if (Result = '') then
    Result := PlainDefault
  else
    Result := Crypt(Result, EncryptLength, True);
end;

function TLKSystemComponent.RecallDate(const Section, Ident: string; Default:
    TDateTime; isSharedSetting: boolean = true): TDateTime;
var
  d: Double;
begin
  d := RecallInteger(Section, Ident, Round(Default), isSharedSetting);
  Result := d;
end;

function TLKSystemComponent.RecallFloat(const Section, Ident: string; Default:
    double; isSharedSetting: boolean = true): Double;
var
  s: string;
begin
  s := RecallString(Section, Ident, FloatToStr(Default), isSharedSetting);
  if not TryStrToFloat(s, Result) then
    Result := Default;
end;

function TLKSystemComponent.RecallInteger(const Section, Ident: string;
    Default: Longint; isSharedSetting: boolean = true): LongInt;
var
  s: string;
begin
  s := RecallString(Section, Ident, IntToStr(Default), isSharedSetting);
  if not TryStrToInt(s, Result) then
    Result := Default;
end;

procedure TLKSystemComponent.RecallSection(const Section: string; Strings:
    TStrings; IsSharedSetting: boolean = true);
var
  Usr: string;
begin
  Strings.Clear;
  if not (Assigned(SystemSettings) and SystemSettings.Active) then
    Exit;
  if IsSharedSetting then
    Usr := '~'
  else
    Usr := UserName;
  with SystemSettings.TblMaster do
    if Locate('USER_ID;COMPONENT;SECTION',
      VarArrayOf([Usr, NameFriendly, Section]),[]) then
    repeat
      if ((FieldByName('USER_ID').AsString = Usr)
        and  (FieldByName('COMPONENT').AsString = NameFriendly)
        and  (FieldByName('SECTION').AsString = Section)) then
        Strings.Add(FieldByName('ITEM').AsString)
      else Break;
    until not FindNext;
end;

function TLKSystemComponent.RecallString(const Section, Ident, Default: string;
    isSharedSetting: boolean = true): string;
var
  Usr: string;
begin
  Result := Default;
  if not (Assigned(SystemSettings) and SystemSettings.Active) then
    Exit;
  if IsSharedSetting then
    Usr := '~'
  else
    Usr := UserName;
  with SystemSettings.TblMaster do
    if Locate('USER_ID;COMPONENT;SECTION;ITEM',
      VarArrayOf([Usr, NameFriendly, Section, Ident]),[]) then
      Result := FieldByName('SETTING').AsString;
end;

procedure TLKSystemComponent.RememberBool(const Section, Ident: string; Value:
    Boolean; isSharedSetting: boolean = true);
begin
  RememberString(Section, Ident, BooleanToYesNo(Value), isSharedSetting);
end;

procedure TLKSystemComponent.RememberCrypt(const Section, Ident, PlainValue:
    string; const EncryptLength: Integer; IsSharedSetting: Boolean = True);
begin
  RememberString(Section, Ident, Crypt(PlainValue, EncryptLength, False), IsSharedSetting);
end;

procedure TLKSystemComponent.RememberDate(const Section, Ident: string; Value:
    TDateTime; isSharedSetting: boolean = true);
begin
  RememberInteger(Section, Ident, Round(Value), isSharedSetting);
end;

procedure TLKSystemComponent.RememberFloat(const Section, Ident: string; Value:
    double; isSharedSetting: boolean = true);
begin
  RememberString(Section, Ident, FloatToStr(Value), isSharedSetting);
end;

procedure TLKSystemComponent.RememberInteger(const Section, Ident: string;
    Value: Longint; isSharedSetting: boolean = true);
begin
  RememberString(Section, Ident, IntToStr(Value), isSharedSetting);
end;

procedure TLKSystemComponent.RememberSection(const Section: string; Strings:
    TStrings; isSharedSetting: boolean = true);
var
  i: Integer;
  usr: string;
  s: string;
  me: string;
begin
  me := NameFriendly;
  if not (Assigned(SystemSettings) and SystemSettings.Active) then
    Exit;
  if isSharedSetting then
    usr := '~'
  else
    usr := UserName;
  with SystemSettings do
  begin
    Filter :=
      'USER_ID = ' + LKSQLStrDelimiter + usr + LKSQLStrDelimiter +
      ' AND COMPONENT = ' + LKSQLStrDelimiter + me + LKSQLStrDelimiter +
      ' AND SECTION = ' + LKSQLStrDelimiter + Section + LKSQLStrDelimiter;
    try
      Filtered := True;
      try
        with TblMaster do
          while FindFirst do
            Delete;
        for i := 0 to Strings.Count - 1 do
        begin
          s := Trim(Strings[i]);
          if (s > '') then
            AppendRecord([usr, me, Section, s, null]);
        end;
        ApplyUpdates(0);
      except CancelUpdates;
      end;
    finally Filter := '';
      Filtered := False;
    end;
  end;
end;

procedure TLKSystemComponent.RememberString(const Section, Ident, Value: string;
    isSharedSetting: boolean = True);
var
  Usr: string;
begin
  if not (Assigned(SystemSettings) and SystemSettings.Active) then
    Exit;
  if IsSharedSetting then
    Usr := '~'
  else
    Usr := UserName;
  with SystemSettings do
  begin
    if Locate('USER_ID;COMPONENT;SECTION;ITEM',
      VarArrayOf([Usr, Self.NameFriendly, Section, Ident]),[]) then
    begin
      with TblMaster do
      begin
        if (FieldByName('SETTING').AsString = Value) then
          Exit;
        Edit;
        FieldByName('SETTING').AsString := Value;
        Post;
      end;
    end
    else
      InsertRecord([Usr, Self.NameFriendly, Section, Ident, Value]);
    ApplyUpdates(0);
  end;
end;

procedure TLKSystemComponent.SaveSettings;
begin
  // overwrite in decendants
end;

procedure TLKSystemComponent.SaveToFile;
var
  i: Integer;
begin
  for i := 0 to ComponentCount - 1 do
    TLKSystemComponent(Components[i]).SaveToFile;
end;

procedure TLKSystemComponent.SetAccountsHub(Value: TLKHubRecords);
var
  I: Integer;
begin
  FAccountsHub := Value;
  if Assigned(Value) and (Value <> Self) then
  begin
    Value.FreeNotification(Self);
    Images := Value.Images;
    SQLConnection := Value.SQLConnection;
    PrintShareLayouts := Value.PrintShareLayouts;
  end;
  for I := 0 to ComponentCount - 1 do
    if Components[i] is TLKContainedRecords then
      TLKContainedRecords(Components[I]).AccountsHub := Value;
  DoAfterSetHub;
end;

procedure TLKSystemComponent.SetActive(Value: Boolean);
var
  I: Integer;
begin
  if (csLoading in ComponentState) then
    Exit;
  FActive := Value;
  for I := 0 to ComponentCount - 1 do
    if Components[I] is TLKContainedRecords then
    try
      TLKContainedRecords(Components[I]).Active := Value;
    except on E: Exception do
      if Value then Showmessage('Error opening data for '
      + TLKContainedRecords(Components[I]).NameFriendly + ': ' + E.Message);
    end;
end;

procedure TLKSystemComponent.SetContainedComponentNames;
begin
  // streamed components must be named:
  // override to synchronise any contained component names
end;

procedure TLKSystemComponent.SetEditing(Value: Boolean);
var
  I: Integer;
begin
  FEditing := Value;
  for I := 0 to ComponentCount - 1 do
    if Components[I] is TLKContainedRecords then
      TLKContainedRecords(Components[i]).Editing := Value;
end;

procedure TLKSystemComponent.SetFolderStoreDocumentImages(const Value: string);
begin
  if (Value = '') or (Value = FFolderStoreDocumentImages) then
    Exit;
  if not (csLoading in ComponentState) then
    RememberString('Settings', 'FolderStoreDocumentImages', Value, True);
  FFolderStoreDocumentImages := Value;
end;

procedure TLKSystemComponent.SetImages(Value: TCustomImageList);
var
  I: Integer;
begin
  inherited SetImages(Value);
  for i := 0 to ComponentCount - 1 do
    if Components[i] is TLKContainedRecords then
      TLKContainedRecords(Components[i]).Images := Value;
end;

procedure TLKSystemComponent.SetName(const NewName: TComponentName);
begin
  inherited SetName(NewName);
  SetContainedComponentNames;
end;

procedure TLKSystemComponent.SetPrintShareLayouts(const Value: Boolean);
var
  I: Integer;
begin
  FPrintShareLayouts := Value;
  for I := 0 to ComponentCount - 1 do
    if Components[I] is TLKContainedRecords then
      TLKContainedRecords(Components[I]).PrintShareLayouts := Value;
end;

procedure TLKSystemComponent.SetSQLConnection(const Value: TSQLConnection);
var
  I: Integer;
begin
  if FSQLConnection <> Value then
  begin
  if Assigned(FSQLConnection) then
    FSQLConnection.RemoveFreeNotification(Self);
  FSQLConnection := Value;
  if Assigned(Value) then
    Value.FreeNotification(Self);
  for I := 0 to ComponentCount - 1 do
    if Components[I] is TLKContainedRecords then
      TLKContainedRecords(Components[i]).SQLConnection := Value;
    FSQLConnection := Value;
  end;
end;

function TLKSystemComponent.Setup(const ForceIfSettingsExist: Boolean):
    TModalResult;
begin
  Result := GetSettings;
  if (Result <> mrOK) or ForceIfSettingsExist then
  begin
    Result := DoSetup(ForceIfSettingsExist);
    if (Result = mrOK) then
      SaveSettings;
  end;
end;

procedure TLKSystemComponent.SetUserName(const Value: string);
var
  I: Integer;
begin
  FUserName := Value;
  for I := 0 to ComponentCount - 1 do
    if Components[I] is TLKContainedRecords then
      TLKContainedRecords(Components[I]).UserName := Value;
end;

function TLKSystemComponent.SQLExecute(const SQL: string): Integer;
var
  i: Integer;
  SQLStatements: TStringList;
begin
  if not Assigned(SQLConnection) then
  begin
    Result := -1;
    Exit;
  end;
  if SQLConnection.Connected then
    Result := 0
  else
  begin
    Result := -1;
    Exit;
  end;
  SQLStatements := TStringList.Create;
  try
    SQLStatementToStrings(SQL, SQLStatements);
    with SQLStatements do
      for i := 0 to Count - 1 do
      begin
        Result := SQLConnection.ExecuteDirect(Strings[i]);
        // if (Result <> 0) then break;
      end;
  finally SQLStatements.Free;
  end;
  //if (Result = 0) then Result := SQLConnection.ExecuteDirect('COMMIT');
end;

procedure TLKSystemComponent.SQLStatementToStrings(const SQL: string;
    SQLStatements: TStrings);
var
  SQLDelimiter: string;
  statement: string;
  i: Integer;
  j: Integer;
  endPos: Integer;
  startPos: Integer;
  finished: Boolean;
  SQLDelimiterFound: Boolean;
begin
  // initialise the SQLDelimiter which may be changed by SET TERM
  SQLDelimiter := ';';
  //parse the SQL to extract single statements
  finished := False;
  endPos := 0;
  startPos := 1;
  while not finished do
  begin
    finished := (startPos > Length(SQL));
    if finished then
      Exit;
    for i := startPos to Length(SQL) do
    begin
      SQLDelimiterFound := False;
      if SQL[i] = SQLDelimiter[1] then
        for j := 1 to Length(SQLDelimiter) do
        begin
          SQLDelimiterFound := SQL[i - 1 + j] = SQLDelimiter[j];
          if not SQLDelimiterFound then
            break;
        end;
      finished := (i = Length(SQL));
      if SQLDelimiterFound or finished then
      begin
        endPos := i;
        if finished and not SQLDelimiterFound then
          inc(endPos);
        break;
      end;
    end;
    statement := Trim(Copy(SQL, startPos, endPos - startPos));
    // for i := 1 to Length(statement) do if (statement[i] < ' ')  then statement[i] := ' ';
    if (statement > '') then
      SQLStatements.Add(statement);
    startPos := endPos + length(SQLDelimiter);
    i := pos('SET TERM ', statement);
    if (i > 0) then
    begin
      inc(i, 8);
      SQLDelimiter := Trim(Copy(statement, i, 1 + Length(statement) - i));
    end;
  end;
end;

{
****************************** TLKGroupComponent *******************************
}
procedure TLKGroupComponent.DoAfterSetACGroup;
begin
  AccountsHub := ACGroup.AccountsHub;
end;

function TLKGroupComponent.EndOfYear(ProgressText: string): TModalResult;
begin
  if not Active then
    if Assigned(ACGroup) then
      ACGroup.Open
    else
      Open;
  Result := inherited EndOfYear(ProgressText);
end;

procedure TLKGroupComponent.Notification(AComponent: TComponent; Operation:
    TOperation);
begin
  if (Operation = opRemove) then
  begin
    if (AComponent = FACGroup) then
      FACGroup := nil;
  end;
  inherited Notification(AComponent, Operation);
end;

procedure TLKGroupComponent.SetACGroup(Value: TLKGroup);
begin
  if FACGroup <> Value then
  begin
  if Assigned(FACGroup) then
  begin
    FACGroup.GroupComponentRemove(self);
    AccountsHub := nil;
  end;
  FACGroup := Value;
  if Assigned(FACGroup) then
  begin
    FACGroup.FreeNotification(Self);
    FACGroup.GroupComponentAdd(Self);
    DoAfterSetACGroup;
  end;
  end;
end;

{
**************************** TLKGroupComponentList *****************************
}
function TLKGroupComponentList.Add(AComponent: TLKGroupComponent): Integer;
begin
  Result := inherited Add(AComponent);
end;

function TLKGroupComponentList.Extract(Item: TLKGroupComponent):
    TLKGroupComponent;
begin
  Result := TLKGroupComponent(inherited Extract(Item));
end;

function TLKGroupComponentList.First: TLKGroupComponent;
begin
  Result := TLKGroupComponent(inherited First);
end;

function TLKGroupComponentList.GetItems(Index: Integer): TLKGroupComponent;
begin
  Result := TLKGroupComponent(inherited Items[Index]);
end;

function TLKGroupComponentList.IndexOf(AComponent: TLKGroupComponent): Integer;
begin
  Result := inherited IndexOf(AComponent);
end;

procedure TLKGroupComponentList.Insert(Index: Integer; AComponent:
    TLKGroupComponent);
begin
  inherited Insert(Index, AComponent);
end;

function TLKGroupComponentList.Last: TLKGroupComponent;
begin
  Result := TLKGroupComponent(inherited Last);
end;

procedure TLKGroupComponentList.SetItems(Index: Integer; AComponent:
    TLKGroupComponent);
begin
  inherited Items[Index] := AComponent;
end;

{
******************************** TLKActionList *********************************
}
constructor TLKActionList.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  SetSubComponent(True);
  {
  if (AOwner is TLKDataComponent) then
  begin
    DataComponent :=  TLKDataComponent(AOwner);
    SetSubComponent(True);
  end
  else raise LKError.Create(
    'Program error: TLKActionList Owner should ba a TLKDataComponent');
    }
end;

function TLKActionList.GetOwner: TPersistent;
begin
  if FDataComponent <> nil then
    Result := FDataComponent
  else
    Result := inherited GetOwner;
end;

function TLKActionList.GetParentComponent: TComponent;
begin
  if Assigned(FDataComponent) then
    Result := FDataComponent
  else
    Result := inherited GetParentComponent;
end;

function TLKActionList.HasParent: Boolean;
begin
  if FDataComponent <> nil then
    Result := True
  else
    Result := inherited HasParent;
end;

procedure TLKActionList.ReadState(Reader: TReader);
begin
  inherited ReadState(Reader);
  if Reader.Parent is TLKDataComponent then
    DataComponent := TLKDataComponent(Reader.Parent);
end;

procedure TLKActionList.SetDataComponent(Value: TLKDataComponent);
begin
  if Assigned(Value) then
  begin
    Value.Actionlist := Self;
//    Name := Value.Name + 'Actions';
  end;
  FDataComponent := Value;
end;

procedure TLKActionList.SetParentComponent(AParent: TComponent);
begin
  if not (csLoading in ComponentState) and (AParent is TLKDataComponent) then
    FDataComponent := TLKDataComponent(AParent);
end;

{
****************************** TLKDataSourceList *******************************
}
function TLKDataSourceList.Add(AComponent: TDataSource): Integer;
begin
  Result := inherited Add(AComponent);
end;

function TLKDataSourceList.Extract(Item: TDataSource): TDataSource;
begin
  Result := TDataSource(inherited Extract(Item));
end;

function TLKDataSourceList.First: TDataSource;
begin
  Result := TDataSource(inherited First);
end;

function TLKDataSourceList.GetItems(Index: Integer): TDataSource;
begin
  Result := TDataSource(inherited Items[Index]);
end;

function TLKDataSourceList.IndexOf(AComponent: TDataSource): Integer;
begin
  Result := inherited IndexOf(AComponent);
end;

procedure TLKDataSourceList.Insert(Index: Integer; AComponent: TDataSource);
begin
  inherited Insert(Index, AComponent);
end;

function TLKDataSourceList.Last: TDataSource;
begin
  Result := TDataSource(inherited Last);
end;

function TLKDataSourceList.Remove(AComponent: TDataSource): Integer;
begin
  Result := inherited Remove(AComponent);
end;

procedure TLKDataSourceList.SetItems(Index: Integer; AComponent: TDataSource);
begin
  inherited Items[Index] := AComponent;
end;

{
*********************************** TLKTable ***********************************
}
constructor TLKTable.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  SetSubComponent(True);
  FDataSources := TLKDataSourceList.Create(False);
end;

destructor TLKTable.Destroy;
begin
  FDataSources.Free;
  inherited Destroy;
end;

procedure TLKTable.CopyFieldValuesFrom(ASource: TDataset; FieldNames:
    TStringList);
var
  I: Integer;
  F: TField;
begin
  if (FieldNames = nil) then
    for I := 0 to (FieldCount - 1) do
    begin
      F := ASource.FindField(Fields[I].Fieldname);
      if Assigned(F) then
        Fields[I].asVariant := F.asVariant;
    end
  else
    for I := 0 to (FieldNames.Count - 1) do
      if (FieldByName(FieldNames[I]).DataType <> ftAutoInc) then
      begin
        F := ASource.FindField(FieldNames[I]);
        if Assigned(F) then
          FieldByName(FieldNames[I]).asVariant := F.AsVariant;
      end;
end;

procedure TLKTable.DlgPrintExecute(const ATitle, AReportName: string; Code:
    Variant; AutoExecute: boolean);
begin
  if (Code <> null) then
  try
    FindKey([Code]);
  except on E: Exception do
    Showmessage(E.Message);
  end;
  with TDlgLKTableBasePrintRecord.Create(Owner) do
  try
    Construct(Self, ATitle);
    Execute(AReportName, AutoExecute);
  finally Free;
  end;
end;

function TLKTable.DlgTableFilterExecute(const AFilterFieldName: string = ''):
    Boolean;
var
  NewFilter, ThisFieldType: string;
  I: Integer;
begin
  Result := False;
  FormLKTableFilter := TFormLKTableFilter.Create(Owner);
  try
    with FormLKTableFilter do
    begin
      with cbxFieldName do
      begin
        Items.clear;
        for I := 0 to (FieldDefs.count - 1) do
        begin
          case FieldDefs.Items[I].DataType of
            ftString: ThisFieldType := 'Text';
            ftSmallint, ftBoolean: ThisFieldType := 'TRUE or FALSE';
            ftInteger, ftWord, ftFloat, ftCurrency, ftFmtBCD, ftBCD, ftAutoInc:
              ThisFieldType := 'Number';
            ftDate, ftTime, ftDateTime: ThisFieldType := 'Date';
          else
            ThisFieldType := 'unknown';
          end;
          if ThisFieldType <> 'unknown' then
          begin
            Items.add(FieldDefs.items[I].name);
            stringFields.add(ThisFieldType);
            if (items[I] = AFilterFieldName) then
              text := AFilterFieldName;
          end;
        end;
        if (Text = '') and (Items.Count > 0) then
          Text := items[0];
      end;
      caption := 'Set table filter ' + TableName;
      if (ShowModal = mrOK) then
      begin
        NewFilter := '';
        for I := 0 to mLink.lines.Count - 1 do
          NewFilter := NewFilter + ' ' + mLink.lines[I]
            + ' ' + mFieldName.lines[I] + ' ' + mTest.lines[I] + ' ' +
              mData.lines[I];
        Filter := NewFilter;
        Result := True;
      end;
    end;
  finally
    FormLKTableFilter.free;
  end;
end;

procedure TLKTable.ExternalDataSourceAdd(ADataSource: TDataSource);
begin
  if Assigned(ADataSource) then
    with FDataSources do if (IndexOf(ADataSource) < 0) then
    begin
      Add(ADataSource);
      ADataSource.FreeNotification(Self);
      ADataSource.DataSet := Self;
    end;
end;

procedure TLKTable.ExternalDataSourceRemove(ADataSource: TDataSource);
begin
  FDataSources.Extract(ADataSource);
end;

procedure TLKTable.ExternalDisableControls;
var
  I: Integer;
begin
  with FDataSources do for I := 0 to Count - 1 do
    Items[I].DataSet := nil;
end;

procedure TLKTable.ExternalEnableControls;
var
  I: Integer;
begin
  with FDataSources do for I := 0 to Count - 1 do
    Items[I].DataSet := Self;
end;

function TLKTable.HasParent: Boolean;
begin
  Result := Assigned(FDataComponent);
end;

procedure TLKTable.Notification(AComponent: TComponent; Operation: TOperation);
begin
  if (Operation = opRemove) then
  begin
    if (AComponent is TDataSource) then
      FDataSources.Extract(TDataSource(AComponent));
  end;
  inherited Notification(AComponent, Operation);
end;

procedure TLKTable.SetDataComponent(Value: TLKDataComponent);
begin
  FDataComponent := Value;
end;


{
******************************** TLKTableMaster ********************************
}
procedure TLKTableMaster.DoAfterDelete;
begin
  inherited DoAfterDelete;
  DataComponent.DoMasterAfterDelete;
end;

procedure TLKTableMaster.DoAfterInsert;
begin
  inherited DoAfterInsert;
  DataComponent.DoMasterAfterInsert;
end;

procedure TLKTableMaster.DoAfterPost;
begin
  inherited DoAfterPost;
  DataComponent.DoMasterAfterPost;
end;

procedure TLKTableMaster.DoBeforeDelete;
begin
  inherited DoBeforeDelete;
  DataComponent.DoMasterBeforeDelete;
end;

procedure TLKTableMaster.DoBeforeEdit;
begin
  inherited DoBeforeEdit;
  DataComponent.DoMasterBeforeEdit;
end;

procedure TLKTableMaster.DoBeforeInsert;
begin
  inherited DoBeforeInsert;
  if not DataComponent.OKToInsert then
    sysUtils.abort;
  DataComponent.DoMasterBeforeInsert;
end;

procedure TLKTableMaster.DoBeforePost;
begin
  inherited DoBeforePost;
  DataComponent.DoMasterBeforePost;
end;

procedure TLKTableMaster.DoOnCalcFields;
begin
  inherited DoOnCalcFields;
  DataComponent.DoMasterCalcFields;
end;

procedure TLKTableMaster.SetDataComponent(Value: TLKDataComponent);
begin
  inherited SetDataComponent(Value);
  if Assigned(DataComponent) then
    OnReconcileError := DataComponent.DoMasterReconcileError
  else OnReconcileError := nil;
end;

{
******************************** TLKTableDetail ********************************
}
procedure TLKTableDetail.DoAfterDelete;
begin
  inherited DoAfterDelete;
  DataComponent.DoDetailAfterDelete;
end;

procedure TLKTableDetail.DoAfterInsert;
begin
  inherited DoAfterInsert;
  DataComponent.DoDetailAfterInsert;
end;

procedure TLKTableDetail.DoAfterPost;
begin
  inherited DoAfterPost;
  DataComponent.DoDetailAfterPost;
end;

procedure TLKTableDetail.DoBeforeDelete;
begin
  inherited DoBeforeDelete;
  DataComponent.DoDetailBeforeDelete;
end;

procedure TLKTableDetail.DoBeforeEdit;
begin
  inherited DoBeforeEdit;
  DataComponent.DoDetailBeforeEdit;
end;

procedure TLKTableDetail.DoBeforeInsert;
begin
  inherited DoBeforeInsert;
  DataComponent.DoDetailBeforeInsert;
end;

procedure TLKTableDetail.DoBeforePost;
begin
  inherited DoBeforePost;
  DataComponent.DoDetailBeforePost;
end;

procedure TLKTableDetail.SetDataComponent(Value: TLKDataComponent);
begin
  inherited SetDataComponent(Value);
  if Assigned(DataComponent) then
    OnReconcileError := DataComponent.DoDetailReconcileError
  else OnReconcileError := nil;
end;

{
******************************* TLKDataComponent *******************************
}
constructor TLKDataComponent.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FMyFieldOwner := AOwner;
  FASCIIDateSeparator := '';
  FASCIIDateIdentifier := '"';
  FASCIIFillCharNumber := '0';
  FASCIIFillCharString := ' ';
  FASCIIFieldSeparator := ',';
  FASCIIStringIdentifier := '"';
  FDeleteDetailWithMasterRecord := True;
  FRefreshMSumDPairs := True;
  FTdDescriptionFieldName := 'DESCRIPTION';
  FChildComponents := TComponentList.Create(False);
  FFilters := TStringList.Create;
  FFiltersDetail := TStringList.Create;
  FMSumDPairFieldNames := TStringList.Create;
  FTotalMasterFieldNames := TStringList.Create;
  FTotalDetailFieldNames := TStringList.Create;
end;

destructor TLKDataComponent.Destroy;
begin
  //FreeAndNil(FFormViewRecord);
  FMSumDPairFieldNames.Free;
  FChildComponents.Free;
  FTotalMasterFieldNames.Free;
  FTotalDetailFieldNames.Free;
  FFilters.Free;
  FFiltersDetail.Free;
  inherited Destroy;
end;

function TLKDataComponent.AccountCheckTotals(const showOK, correctErrors,
    raiseError: boolean): Boolean;
var
  I, X: Integer;
  Amount, Total: Currency;
  IsMinus: Boolean;
  MasterFieldName, DetailFieldName, ErrorMsg: string;
  MasterTotals: array[0..50] of Currency;
  MasterTotalFields: TStringList;
begin
  Result := True;
  MasterTotalFields := TStringList.create;
  try
    if Assigned(FMSumDPairFieldNames) then
    begin
      // construct list of Master total fields and zero values of totals array
      with FMSumDPairFieldNames do
      begin
        for I := 0 to round(count / 2) - 1 do
        begin
          MasterFieldName := strings[I * 2];
          with MasterTotalFields do
            if (indexOf(MasterFieldName) < 0) then
              add(MasterFieldName);
          MasterTotals[I] := 0;
        end;
        // sum detail table values for master fields
        with TblDetail do
        begin
          SaveFilter;
          Filtered := False;
          Filter := '';
          if FindFirst then
            repeat
              for I := 0 to round(count / 2) - 1 do
              begin
                MasterFieldName := strings[I * 2];
                DetailFieldName := strings[1 + I * 2];
                if (pos('+', DetailFieldName) = 1) then
                  system.Delete(DetailFieldName, 1, 1);
                IsMinus := (pos('-', DetailFieldName) = 1);
                if IsMinus then
                  system.Delete(DetailFieldName, 1, 1);
                Amount := GetCurrency(FieldByName(DetailFieldName));
                if IsMinus then
                  Amount := -Amount;
                X := MasterTotalFields.IndexOf(MasterFieldName);
                MasterTotals[X] := MasterTotals[X] + Amount;
              end;
            until not FindNext;
          RestoreFilter;
        end;
      end;
      // confirm detail totals equal master values and fix if required
      for I := 0 to (MasterTotalFields.Count - 1) do
      begin
        MasterFieldName := MasterTotalFields.strings[I];
        Amount := MasterTotals[I];
        total := GetCurrency(FTblMaster.FieldByName(MasterFieldName));
        Result := equalCurrency(total, Amount);
        if not Result then
        begin
          ErrorMsg := nameFriendly + ' Total for ' + masterFieldName +
            ' adds up to ' + FloatToStrF(Amount, ffFixed, 15, 2) +
            ' but is shown as ' + FloatToStrF(total, ffFixed, 15, 2) +
              ' difference ' + FloatToStrF(total - Amount, ffFixed, 15, 2);
          if correctErrors then
            with FTblMaster do
            begin
              Edit;
              FieldByName(MasterFieldName).AsCurrency := Amount;
              Post;
              ApplyUpdates(-1);
            end;
          if raiseError then
            raise LKError.create(ErrorMsg)
          else if ShowOK then
            ShowMessage(ErrorMsg);
        end;
      end;
      if showOK and Result then
        ShowMessage('Account totals are OK');
    end;
  finally
    MasterTotalFields.free;
  end;
end;

procedure TLKDataComponent.Append;
begin
  FOKToInsert := True;
  try
    FTblMaster.Append;
  finally FOKToInsert := False;
  end;
end;

procedure TLKDataComponent.AppendRecord(const Values: array of const);
begin
  FOKToInsert := True;
  try
    FTblMaster.AppendRecord(Values);
  finally FOKToInsert := False;
  end;
end;

function TLKDataComponent.ApplyUpdates(MaxErrors: Integer): Integer;
begin
  Result := 0;
  //if (csDestroying in ComponentState) or not Active then Exit;
  //if Assigned(FTblMaster.Clone) then Result := FTblMaster.Clone.ApplyUpdates(MaxErrors) else
  if (FTblMaster.ChangeCount > 0) then
    Result := FTblMaster.ApplyUpdates(MaxErrors);
  //if Assigned(TblDetail.Clone) then Result := Result + TblDetail.Clone.ApplyUpdates(MaxErrors) else
  if TblDetailUsed then
    if (TblDetail.ChangeCount > 0) then
      Result := Result + TblDetail.ApplyUpdates(MaxErrors);
end;

procedure TLKDataComponent.AssignTo(Dest: TPersistent);
var
  DestACB: TLKDataComponent;
begin
  inherited AssignTo(Dest);
  DestACB := TLKDataComponent(Dest);
    //DestACB.Filter := Filter;
    //DestACB.Filtered := Filtered;
    //DestACB.Filters.AddStrings(Filters);DestACB := TLKDataComponent(Dest);
  DestACB.ACGroup := ACGroup;
  DestACB.FTblMaster.Assign(FTblMaster);
  DestACB.TblDetail.Assign(TblDetail);
  //  DestACB.IndexDefs.Assign(IndexDefs);
    //DestACB.FTblMaster.FieldDefs.Assign(FTblMaster.FieldDefs);
  //  if (IndexFieldNames > '') then
  //    DestACB.IndexFieldNames := IndexFieldNames
  //  else
  //    DestACB.IndexName := IndexName;
  //  DestACB.FTblMaster.TableName := FTblMaster.TableName;
  //  DestACB.TblDetail.TableName := TblDetail.TableName;
  //  DestACB.TblDetail.IndexDefs.Assign(TblDetail.IndexDefs);
    //DestACB.tblDetail.FieldDefs.Assign(tblDetail.FieldDefs);
    //DestACB.tDetailConstraints.AddStrings(tDetailConstraints);
    // DestACB.TblDetail.MasterFields := TblDetail.MasterFields;
  //  if (TblDetail.IndexFieldNames > '') then
  //    DestACB.TblDetail.IndexFieldNames := TblDetail.IndexFieldNames
  //  else
  //    DestACB.TblDetail.IndexName := TblDetail.IndexName;
    //DestACB.TblMasterConstraints.AddStrings(tMasterConstraints);
    //DestACB.TblMasterDataSource := nil; // sets to internal
    //DestACB.TblDetailDataSource := nil; // sets to internal
  //  DestACB.MasterComponent := MasterComponent;
  //  DestACB.MasterTable := MasterTable;
  //  DestACB.MasterFields := MasterFields;
  DestACB.MSumDPairFieldNames.Assign(MSumDPairFieldNames);
  DestACB.TotalMasterFieldNames.AddStrings(TotalMasterFieldNames);
  DestACB.TotalDetailFieldNames.AddStrings(TotalDetailFieldNames);
  DestACB.DetailAfterDelete := DetailAfterDelete;
  DestACB.DetailAfterInsert := DetailAfterInsert;
  DestACB.DetailAfterPost := DetailAfterPost;
  DestACB.DetailBeforeDelete := DetailBeforeDelete;
  DestACB.DetailBeforeInsert := DetailBeforeInsert;
  DestACB.DetailBeforePost := DetailBeforePost;
  DestACB.OnDetailDrillDown := OnDetailDrillDown;
  DestACB.OnCopyMasterRecord := OnCopyMasterRecord;
  DestACB.MasterAfterDelete := MasterAfterDelete;
  DestACB.MasterAfterInsert := MasterAfterInsert;
  DestACB.MasterAfterPost := MasterAfterPost;
  DestACB.MasterBeforeDelete := MasterBeforeDelete;
  DestACB.MasterBeforeInsert := MasterBeforeInsert;
  DestACB.MasterBeforePost := MasterBeforePost;
  DestACB.OnMasterCalcFields := OnMasterCalcFields;
  DestACB.OnMasterGridDblClick := OnMasterGridDblClick;
    //DestACB.OnBlobTableBeforePost := OnBlobTableBeforePost;
  DestACB.OnCopyDetailRecord := OnCopyDetailRecord;
  DestACB.OnPrint := OnPrint;
  DestACB.OnPrintComplete := OnPrintComplete;
  DestACB.OnPrintHeader := OnPrintHeader;
  DestACB.OnView := OnView;
    {
    with FTblMaster.BlobTables do for i := 0 to Count -1 do with TLKTableBlob(items[i]) do
     begin
     myBlobTable := TLKTableBlob.Create(DestACB);
     myBlobTable.FieldDefs.assign(FieldDefs);
     myBlobTable.Owner := DestACB;
     myBlobTable.ContainerTable := ContainerTable;
     myBlobTable.BlobFieldName := BlobFieldName;
     end;
    with TblDetail.BlobTables do for i := 0 to Count -1 do with TLKTableBlob(items[i]) do
     begin
     myBlobTable := TLKTableBlob.Create(DestACB);
     myBlobTable.FieldDefs.assign(FieldDefs);
     myBlobTable.Owner := DestACB;
     myBlobTable.ContainerTable := ContainerTable;
     myBlobTable.BlobFieldName := BlobFieldName;
     end;
    }
end;

procedure TLKDataComponent.Cancel;
begin
  FTblMaster.Cancel;
end;

procedure TLKDataComponent.CancelUpdates;
begin
  FTblMaster.CancelUpdates;
  if TblDetailUsed then
    TblDetail.CancelUpdates;
end;

procedure TLKDataComponent.ChangeKeyFieldValues(FieldNameList: TStringList;
    OldValues, NewValues: array of variant);
var
  MFieldNames, DFieldNames: string;
  MFieldNameList, DFieldNameList: TStringList;
  myNewMValues, myOldMValues, myNewDValues, myOldDValues: Variant;
  I, a: Integer;
begin
  MFieldNameList := TStringList.Create;
  DFieldNameList := TStringList.Create;
  try
    with FieldNameList do
      for i := 0 to Count - 1 do
        if Assigned(FTblMaster.FindField(Strings[i])) and not (OldValues[i] =
          NewValues[i]) then
        begin
          MFieldNameList.Add(Strings[i]);
          myOldMValues := OldValues[i];
          myNewMValues := NewValues[i];
        end;
    case MFieldNameList.Count of
      0: ;
      1:
        begin
          with FieldNameList do
            for i := 0 to Count - 1 do
              if Assigned(FTblMaster.FindField(Strings[i])) and not (OldValues[i]
                = NewValues[i]) then
              begin
                MFieldNames := Strings[i];
                myOldMValues := OldValues[i];
                myNewMValues := NewValues[i];
                break;
              end;
        end;
    else
      begin
        MFieldNames := '';
        myOldMValues := VarArrayCreate([0, MFieldNameList.Count - 1],
          varVariant);
        myNewMValues := VarArrayCreate([0, MFieldNameList.Count - 1],
          varVariant);
        a := 0;
        with FieldNameList do
          for i := 0 to Count - 1 do
            if Assigned(FTblMaster.FindField(Strings[i])) and not (OldValues[i] =
              NewValues[i]) then
            begin
              if (a > 0) then
                MFieldNames := MFieldNames + ';';
              MFieldNames := MFieldNames + Strings[i];
              myOldMValues[a] := OldValues[i];
              myNewMValues[a] := NewValues[i];
              inc(a);
            end;
      end;
    end;
    if TblDetailUsed then
    begin
      with FieldNameList do
        for i := 0 to Count - 1 do
          if Assigned(TblDetail.FindField(Strings[i])) and not (OldValues[i] =
            NewValues[i]) then
            DFieldNameList.Add(Strings[i]);
      case DFieldNameList.Count of
        0: ;
        1:
          begin
            with FieldNameList do
              for i := 0 to Count - 1 do
                if Assigned(TblDetail.FindField(Strings[i])) and not
                  (OldValues[i] = NewValues[i]) then
                begin
                  DFieldNames := Strings[i];
                  myOldDValues := OldValues[i];
                  myNewDValues := NewValues[i];
                  break;
                end;
          end;
      else
        begin
          DFieldNames := '';
          myOldDValues := VarArrayCreate([0, DFieldNameList.Count - 1],
            varVariant);
          myNewDValues := VarArrayCreate([0, DFieldNameList.Count - 1],
            varVariant);
          a := 0;
          with FieldNameList do
            for i := 0 to Count - 1 do
              if Assigned(TblDetail.FindField(Strings[i])) and not (OldValues[i]
                = NewValues[i]) then
              begin
                if (a > 0) then
                  DFieldNames := DFieldNames + ';';
                DFieldNames := DFieldNames + Strings[i];
                myOldDValues[a] := OldValues[i];
                myNewDValues[a] := NewValues[i];
                inc(a);
              end;
        end;
      end;
    end;
    if (MFieldNameList.Count > 0) then
      with FTblMaster do
        while Locate(MFieldNames, myOldMValues, []) do
        begin
          with FChildComponents do
            for i := 0 to count - 1 do
              with TLKDataComponent(items[i]) do
                if (MasterTable = tbMaster) then
                  ChangeKeyFieldValues(FieldNameList, OldValues, NewValues);
          if (DFieldNameList.Count > 0) then
            with TblDetail do
              while Locate(DFieldNames, myOldDValues, []) do
              begin
                with FChildComponents do
                  for i := 0 to count - 1 do
                    with TLKDataComponent(items[i]) do
                      if (MasterTable = tbDetail) then
                        ChangeKeyFieldValues(FieldNameList, OldValues,
                          NewValues);
                Edit; //TblDetail
                with DFieldNameList do
                  if (Count = 1) then
                    FieldByName(Strings[0]).Value := myNewDValues
                  else
                    for i := 0 to Count do
                      FieldByName(Strings[i]).Value := myNewDValues[i];
                try
                  Post;
                except on E: Exception do
                  begin
                    showmessage(E.Message);
                    Cancel;
                    Delete;
                  end;
                end;
              end;
          Edit; //FTblMaster
          with MFieldNameList do
            if (Count = 1) then
              FieldByName(Strings[0]).Value := myNewMValues
            else
              for i := 0 to Count do
                FieldByName(Strings[i]).Value := myNewMValues[i];
          Post;
        end;
    ApplyUpdates(0);
  finally
    MFieldNameList.Free;
    DFieldNameList.Free;
  end;
end;

procedure TLKDataComponent.ChildComponentAdd(Value: TLKDataComponent);
begin
  with FChildComponents do
    if (indexOf(Value) = -1) then
      add(Value);
end;

procedure TLKDataComponent.ChildComponentRemove(Value: TLKDataComponent);
var
  i: Integer;
begin
  if Assigned(Value) then
    with FChildComponents do
    begin
      i := indexOf(Value);
      if i > -1 then
        Delete(i);
    end;
end;

procedure TLKDataComponent.CopyDetailRecord(ASource: TLKDataComponent;
    impAutoIncName: string; impAutoIncValue: integer; detailFieldNames:
    TStringList; AMode: TBatchMode);
var
  i, x: Integer;
  myKeyValues: Variant;
  myIndexNames: string;
begin
  with TblDetail do
  begin
    // avoid key violation by locating any existing record else insert new record;
    if (IndexFieldCount = 0) or (AMode in [batAppend, batCopy]) then
      Append
    else
    begin
      if IndexFieldCount = 1 then
      begin
        myIndexNames := IndexFields[0].FieldName;
        for i := 0 to detailFieldNames.count - 1 do
          if (detailFieldNames[i] = myIndexNames) then
          begin
            myKeyValues :=
              ASource.TblDetail.FindField(detailFieldNames[i]).Value;
            Break;
          end;
      end
      else
      begin
        myKeyValues := VarArrayCreate([0, IndexFieldCount - 1], varVariant);
        myIndexNames := '';
        for x := 0 to IndexFieldCount - 1 do
        begin
          myIndexNames := myIndexNames + IndexFields[x].FieldName;
          if (x < (IndexFieldCount - 1)) then
            myIndexNames := myIndexNames + ';';
          for i := 0 to detailFieldNames.Count - 1 do
            if (detailFieldNames[i] = IndexFields[x].FieldName) then
            begin
              myKeyValues[x] :=
                ASource.TblDetail.FindField(detailFieldNames[i]).asVariant;
              break;
            end;
        end;
      end;
      if Locate(myIndexNames, myKeyValues, []) then
        case AMode of
          batAppend: raise LKError.Create(NameFriendly
              + ' Detail table record already exists. (Key violation) ' +
                Fields[0].AsString);
          batUpdate, batAppendUpdate: Edit;
          batDelete, batCopy:
            begin
              Delete;
              Insert;
            end;
        end
      else
        Append;
    end;
    try
      CopyFieldValuesFrom(ASource.TblDetail, detailFieldNames);
      // if auto-increment field im FTblMaster set same field in TblDetail
      if (impAutoIncName > '') then
        FieldByName(impAutoIncName).asInteger := impAutoIncValue;
      Post;
      doCopyDetailRecord;
    except Delete;
    end;
  end;
end;

procedure TLKDataComponent.CopyMasterRecord(ASource: TLKDataComponent;
    masterFieldNames: TStringList; AMode: TBatchMode);
var
  i, x: Integer;
  myIndexNames: string;
  myKeyValues: Variant;
begin
  FOKToInsert := True;
  isBatchNumbering := True;
  try
    with FTblMaster do
    begin
      // avoid key violation by locating any existing record else insert new record;
      if (IndexFieldCount = 0) or (AMode in [batAppend, batCopy]) then
        Append
      else
      begin
        if IndexFieldCount = 1 then
        begin
          myIndexNames := IndexFields[0].FieldName;
          for i := 0 to masterFieldNames.count - 1 do
            if (masterFieldNames[i] = myIndexNames) then
            begin
              myKeyValues :=
                ASource.FTblMaster.FindField(masterFieldNames[i]).asVariant;
              Break;
            end;
        end
        else
        begin
          myKeyValues := VarArrayCreate([0, IndexFieldCount - 1], varVariant);
          myIndexNames := '';
          for x := 0 to IndexFieldCount - 1 do
          begin
            myIndexNames := myIndexNames + IndexFields[x].FieldName;
            if (x < (IndexFieldCount - 1)) then
              myIndexNames := myIndexNames + ';';
            for i := 0 to masterFieldNames.Count - 1 do
              if (masterFieldNames[i] = IndexFields[x].FieldName) then
              begin
                myKeyValues[x] :=
                  ASource.FTblMaster.FindField(masterFieldNames[i]).asVariant;
                break;
              end;
          end;
        end;
        if Locate(myIndexNames, myKeyValues, []) then
          case AMode of
            batAppend: raise LKError.Create(NameFriendly
                + ' Master table record already exists. (Key violation) ' +
                  Fields[0].AsString);
            batUpdate, batAppendUpdate: Edit;
            batDelete, batCopy:
              begin
                Delete;
                Insert;
              end;
          end
        else
          Append;
      end;
      try
        CopyFieldValuesFrom(ASource.FTblMaster, masterFieldNames);
        Post;
        doCopyMasterRecord;
      except Delete;
      end;
    end;
  finally FOKToInsert := False;
    isBatchNumbering := False;
  end;
  Application.ProcessMessages;
end;

procedure TLKDataComponent.CopyRecords(ASource: TLKDataComponent;
    CopyMasterFilter: string; CopyTables: TLKACTableSelect; IgnoreExceptions:
    boolean; AMode: TBatchMode);
var
  impField: TField;
  masterFieldNames, detailFieldNames: TStringList;
  impAutoIncName, oldSourceFilter: string;
  impAutoIncValue, i: Integer;
  wasOpen, sourceWasFiltered: Boolean;
begin
  wasOpen := active;
  open;
  // check success of open
  if not active then
    raise LKError.create('Could not open ' + nameFriendly +
      ' to receive Copy data');
  // apply any previous changes to avoid loss on error
  if (ChangeCount > 0) then
    ApplyUpdates(0);
  // set filter of source component
  with ASource do
  begin
    sourceWasFiltered := Filtered;
    oldSourceFilter := Filter;
    open;
    filter := CopyMasterFilter;
    filtered := (CopyMasterFilter > '');
    if not Active then
      raise LKError.create('Could not open Copy tables');
    if not FindFirst then
      if IgnoreExceptions then
        SysUtils.Abort
      else
        raise LKError.create('No records found to copy from ' +
          ASource.NameFriendly);
  end;
  try
    // create lists to hold names of fields (to copy values) with identical name and dataType
    masterFieldNames := TStringList.create;
    detailFieldNames := TStringList.create;
    try
      impAutoIncName := '';
      // identify valid Copy fields with same name and type and save in list
      if CopyTables in [tbsMaster, tbsBoth] then
        with FTblMaster.fieldDefs do
          for i := 0 to Count - 1 do
          begin
            impField := ASource.FTblMaster.FindField(items[i].Name);
            if Assigned(impField) then
              if (impField.dataType = ftAutoInc) then
              begin
                // get name of first auto-increment field (may be detail key)
                if (items[i].dataType = ftInteger) then
                  masterFieldNames.add(items[i].Name)
                else if (impAutoIncName = '') then
                  impAutoIncName := impField.Fieldname;
              end
              else if (impField.dataType = items[i].dataType) then
                masterFieldNames.add(items[i].Name);
          end;
      if TblDetailUsed and (CopyTables in [tbsDetail, tbsBoth]) then
      begin
        with TblDetail.fieldDefs do
          for i := 0 to Count - 1 do
          begin
            impField := ASource.TblDetail.FindField(items[i].Name);
            if Assigned(impField) then
              if (impField.DataType = items[i].DataType) then
                DetailFieldNames.add(items[i].Name);
          end;
        // ensure that auto increment field exists in TblDetail
        if (TblDetail.FindField(impAutoIncName) = nil) then
          impAutoIncName := '';
      end;
      // copy records across
      if ASource.FTblMaster.findFirst then
        repeat
          Application.ProcessMessages;
          try
            if CopyTables in [tbsMaster, tbsBoth] then
              CopyMasterRecord(ASource, masterFieldNames, AMode);
            if (impAutoIncName = '') then
              impAutoIncValue := 0
            else
              impAutoIncValue :=
                FTblMaster.FieldByName(impAutoIncName).asInteger;
            if CopyTables in [tbsDetail, tbsBoth] then
              if ASource.TblDetail.FindFirst then
                repeat
                  CopyDetailRecord(ASource, impAutoIncName, impAutoIncValue,
                    detailFieldNames, AMode);
                until not ASource.TblDetail.FindNext;
          except on E: Exception do
              if not IgnoreExceptions then
              begin
                CancelUpdates;
                raise;
              end;
          end; //showMessage(E.Message); end;
        until not ASource.FTblMaster.FindNext;
      if (ChangeCount > 0) then
        ApplyUpdates(0);
    finally
      masterFieldNames.free;
      detailFieldNames.free;
      active := wasOpen;
    end;
  finally
    with ASource do
    begin
      Filter := oldSourceFilter;
      Filtered := sourceWasFiltered;
    end;
  end;
end;

procedure TLKDataComponent.CreateSubComponents;
begin
  inherited CreateSubComponents;
  FInternalMasterDS := TDataSource.Create(ContainerComponent);
  FInternalDetailDS := TDataSource.Create(ContainerComponent);
  FActionList := TLKActionList.Create(ContainerComponent);
  FActionList.Name := ClassName + 'ActionList';
  FTblMaster := TLKTableMaster.Create(ContainerComponent);
  FTblMaster.Name := ClassName + 'TblMaster';
  FTblMaster.DataComponent := Self;
  FInternalMasterDS.DataSet := TblMaster;
  FTblDetail := TLKTableDetail.Create(ContainerComponent);
  FTblDetail.Name := ClassName + 'TblDetail';
  FTblDetail.DataComponent := Self;
  FTblDetail.MasterSource := FInternalMasterDS;
  FInternalDetailDS.DataSet := TblDetail;
end;

procedure TLKDataComponent.CreateViewForm;
begin
  Application.CreateForm(TFormLKDataComponent, FFormView);
  TFormLKDataComponent(FormView).Construct(Self);
end;

procedure TLKDataComponent.CreateViewFormRecord;
begin
  Application.CreateForm(TFormLKDataComponentRecord, FFormViewRecord);
  TFormLKDataComponentRecord(FormViewRecord).Construct(Self);
end;

procedure TLKDataComponent.Delete;
begin
  FTblMaster.Delete;
end;

procedure TLKDataComponent.DlgBlockDeleteExecute(AOwner: TComponent;
    TablesToDelete: TLKACTableSelect; masterDeleteFilter, detailDeleteFilter:
    string);
var
  HoldMaster: TLKDataComponent;
begin
  HoldMaster := MasterComponent;
  try
    with TFormLKDataCompBlockDelete.Create(Self) do
    try
      Construct(Self);
      Execute(TablesToDelete, masterDeleteFilter, detailDeleteFilter);
    finally Free;
    end;
  finally MasterComponent := HoldMaster;
  end;
end;

procedure TLKDataComponent.DlgChangeKeyFieldValueExecute;
begin
  with TDlgLKChangeKeyFieldValue.Create(Self) do
  try
    if (Execute(Self) = mrOK) then
      ChangeKeyFieldValues(ChangeFieldNames, FieldOldValues, FieldNewValues);
  finally Free;
  end;
end;

function TLKDataComponent.DlgFindDetailExecute(AOwner: TComponent; const
    searchFieldName: string): Boolean;
var
  options: TLocateOptions;
  finished: Boolean;
begin
  if (FMyFindDialogDetail = nil) then
  begin
    FMyFindDialogDetail := TFormAcFindDialog.Create(Self);
    TFormACFindDialog(FMyFindDialogDetail).Construct(TblDetail);
  end;
  with TFormAcFindDialog(FMyFindDialogDetail) do
  begin
    with cbxFields do
      if (TblDetail.FindField(searchFieldName) <> nil) then
        text := searchFieldName
      else if (text = '') then
        text := items[0];
    repeat
      finished := True;
      Result := (showModal = mrOK);
      if Result and (eDetails.text > '') then
      begin
        screen.cursor := crHourglass;
        try
          options := [];
          if xBxcase.checked then
            options := options + [loCaseInsensitive];
          if xBxPart.checked then
            options := options + [loPartialKey];
          finished := TblDetail.locate(cbxFields.text, variant(eDetails.text),
            options);
        finally screen.cursor := crDefault;
        end;
        if not finished then
          messageDlg('Cannot find ' + eDetails.text, mtInformation, [mbOK], 0);
      end;
    until finished;
  end;
end;

function TLKDataComponent.DlgFindExecute(const searchFieldName: string):
    Boolean;
var
  thisDocs, masterDocs: TLKDataComponent;
  i: Integer;
  Docs: TComponentList;
  myidxValues: Variant;
  options: TLocateOptions;
  finished: Boolean;
  myidxValuesCount: Integer;

  procedure locateIn(const myTable: TLKTableBase);
  var
    i, x: integer;
    LocateValues: variant;
  begin
    LocateValues := null;
    with myTable do
    begin
      i := IndexFieldCount;
      case i of
        0: ;
        1: case myidxValuesCount of
            0: ;
            1: LocateValues := myidxValues;
          else
            LocateValues := myidxValues[0];
          end;
      else
        begin
          LocateValues := VarArrayCreate([0, i - 1], varVariant);
          x := i;
          if (x > myidxValuesCount) then
            x := myidxValuesCount;
          case x of
            0: ;
            1: LocateValues[0] := myidxValues;
          else
            for i := 0 to x - 1 do
              LocateValues[i] := myidxValues[i];
          end;
        end;
      end;
      Locate(CurrentIndexFieldNames, LocateValues, []);
    end;
  end;

begin
  // identify the master component links
  Docs := TComponentList.Create;
  Docs.OwnsObjects := False;
  try
    thisDocs := self;
    repeat
      Docs.Add(thisDocs);
      thisDocs := thisDocs.MasterComponent;
    until not Assigned(thisDocs);
    masterDocs := MasterComponent;
    MasterComponent := nil;
    if (FMyFindDialog = nil) then
    begin
      FMyFindDialog := TFormAcFindDialog.create(Self);
      TFormACFindDialog(FMyFindDialog).Construct(FTblMaster);
    end;
    with TFormAcFindDialog(FMyFindDialog) do
    begin
      with cbxFields do
        if Assigned(FTblMaster.FindField(searchFieldName)) then
          text := searchFieldName
        else if (text = '') then
          text := items[0];
      repeat
        finished := True;
        Result := (showModal = mrOK);
        if Result and (eDetails.text > '') then
        begin
          screen.cursor := crHourglass;
          try
            options := [];
            if xBxcase.checked then
              options := options + [loCaseInsensitive];
            if xBxPart.checked then
              options := options + [loPartialKey];
            finished := Locate(cbxFields.text, variant(eDetails.text), options);
          finally screen.cursor := crDefault;
          end;
          if not finished then
            messageDlg('Cannot find ' + eDetails.text, mtInformation, [mbOK],
              0);
        end;
      until finished;
    end;
    if (Docs.Count = 1) or not Result then
      MasterComponent := masterDocs
    else
    begin
      // mark it and extract the index field values
      screen.cursor := crHourglass;
      try
        myidxValues := FTblMaster.CurrentIndexFieldValues;
        myidxValuesCount := FTblMaster.IndexFieldCount;
        MasterComponent := masterDocs;
        //to locate the master records in the master tables
        for i := Docs.Count - 1 downto 1 do
        begin
          masterDocs := TLKDataComponent(Docs[i]);
          thisDocs := TLKDataComponent(Docs[i - 1]);
          locateIn(masterDocs.FTblMaster);
          case thisDocs.MasterTable of
            tbDetail: locateIn(masterDocs.TblDetail);
          end;
        end;
        locateIn(FTblMaster);
      finally screen.cursor := crDefault;
      end;
    end;
  finally Docs.Free;
  end;
end;

procedure TLKDataComponent.DlgImportExportExecute(AOwner: TComponent; const
    AsExportDlg, Autoexecute: boolean);
begin
  with TDlgLKDataComponentImportExport.Create(Self) do
  try
    Execute(self, AsExportDlg, Autoexecute);
  finally Free;
  end;
end;

function TLKDataComponent.DlgNewExecute: TModalResult;
begin
  Result := RecordNew;
end;

procedure TLKDataComponent.DlgPrintEnvelopeExecute(const Addressee: string;
    addressFields: TLKFieldNumbers; autoExecute: boolean);
var
  n, x: Integer;
  address: array[0..5] of string;
begin
  address[0] := addressee;
  for n := 1 to 5 do
    address[n] := '';
  x := 1;
  with FTblMaster do
    for n := 0 to fieldCount - 1 do
    begin
      if (n in addressFields) then
      begin
        address[x] := fields[n].AsString;
        inc(x);
        if (x = 6) then
          break;
      end;
    end;
  with TLKPrintEnvelopeDlg.create(Owner) do
  begin
    try
      Construct(Self, '');
      edit0.text := address[0];
      edit1.text := address[1];
      edit2.text := address[2];
      edit3.text := address[3];
      edit4.text := address[4];
      edit5.text := address[5];
      Execute('Default', autoExecute);
    finally Free;
    end;
  end;
end;

procedure TLKDataComponent.DlgPrintExecute(const ATitle, AReportName: string;
    key: variant; autoExecute: boolean);
begin
  if Assigned(FOnPrint) then
    FOnPrint(self);
  if (key <> null) then
  try
    FTblMaster.Locate(FTblMaster.Fields[0].FieldName,Key,[]);
  except
   //
  end;
  with TDlgLKPrintRecord.Create(Owner) do
  begin
    try
      Construct(Self, ATitle);
      Execute(AReportName, autoExecute);
    finally Free;
    end;
  end;
end;

function TLKDataComponent.DlgSelectRecordExecute(const aTitle: string; var
    SearchValues: variant): TModalResult;
var
  thisDocs, masterDocs: TLKDataComponent;
  i: Integer;
  Docs: TComponentList;
  myidxValues: Variant;
  myidxValuesCount: Integer;

  procedure locateIn(const myTable: TLKTableBase);
  var
    i, x: integer;
    LocateValues: variant;
  begin
    LocateValues := null;
    with myTable do
    begin
      i := IndexFieldCount;
      case i of
        0: ;
        1: case myidxValuesCount of
            0: ;
            1: LocateValues := myidxValues;
          else
            LocateValues := myidxValues[0];
          end;
      else
        begin
          LocateValues := VarArrayCreate([0, i - 1], varVariant);
          x := i;
          if (x > myidxValuesCount) then
            x := myidxValuesCount;
          case x of
            0: ;
            1: LocateValues[0] := myidxValues;
          else
            for i := 0 to x - 1 do
              LocateValues[i] := myidxValues[i];
          end;
        end;
      end;
      Locate(CurrentIndexFieldNames, LocateValues, []);
    end;
  end;

begin
  // identify the master component links
  Docs := TComponentList.Create;
  Docs.OwnsObjects := False;
  try
    thisDocs := self;
    repeat
      Docs.Add(thisDocs);
      thisDocs := thisDocs.MasterComponent;
    until not Assigned(thisDocs);
    masterDocs := MasterComponent;
    MasterComponent := nil;
    {
    try
    if (SearchValues <> null) then with FTblMaster do
     Locate(CurrentIndexFieldNames,SearchValues,[loCaseInsensitive,loPartialKey]);
    except ; end;
    }
    with TLKDataCompSelectRecordDlg.Create(Owner) do
    begin
      try
        Construct(Self);
        Caption := aTitle;
        Result := showModal;
      finally Free;
      end;
    end;
    case Result of
      mrCancel, mrRetry:
        begin
          SearchValues := null;
          MasterComponent := masterDocs;
          Exit;
        end;
    end;
    SearchValues := FTblMaster.CurrentIndexFieldValues;
    if (Docs.Count > 1) then
    begin
      // mark it and extract the index field Values
      screen.cursor := crHourglass;
      try
        myidxValues := FTblMaster.CurrentIndexFieldValues;
        myidxValuesCount := FTblMaster.IndexFieldCount;
        MasterComponent := masterDocs;
        //to locate the master records in the master tables
        for i := Docs.Count - 1 downto 1 do
        begin
          masterDocs := TLKDataComponent(Docs[i]);
          thisDocs := TLKDataComponent(Docs[i - 1]);
          locateIn(masterDocs.FTblMaster);
          case thisDocs.MasterTable of
            tbDetail: locateIn(masterDocs.TblDetail);
          end;
        end;
        locateIn(FTblMaster);
      finally screen.cursor := crDefault;
      end;
    end;
  finally Docs.Free;
  end;
end;

function TLKDataComponent.DlgTableFilterExecute(const AFilterTable: TLKACTable;
    AFilterFieldName: string = ''): Boolean;
begin
  case AFilterTable of
    tbMaster:
      Result := FTblMaster.DlgTableFilterExecute(AFilterFieldName);
    else
      Result := TblDetail.DlgTableFilterExecute(AFilterFieldName);
  end;
end;

function TLKDataComponent.DlgViewExecute: TModalResult;
begin
  Result := View(null, False, []);
//  FormView.WindowState := wsMaximized;
end;

function TLKDataComponent.DlgViewTotalsExecute(AOwner: TComponent; VTCaption:
    string; AButtons: TMsgDlgButtons): Word;
begin
  //Result := mrCancel;
  with TDlgLKDataCompTotals.Create(Owner) do
  try
    Caption := VTCaption;
    Construct(Self);
    Result := ShowModal;
  finally Free;
  end;
end;

procedure TLKDataComponent.DoCopyDetailRecord;
begin
  if Assigned(FOnCopyDetailRecord) then
    FOnCopyDetailRecord(Self);
end;

procedure TLKDataComponent.DoCopyMasterRecord;
begin
  if Assigned(FOnCopyMasterRecord) then
    FOnCopyMasterRecord(Self);
end;

procedure TLKDataComponent.DoDetailAfterDelete;
begin
  if Assigned(FDetailAfterDelete) then
    FDetailAfterDelete(TblDetail);
  if Assigned(FormView) and FormView.Visible then
    TFormLKDataComponent(FormView).ActionUndo.Enabled := (ChangeCount >
        0);
  if Assigned(FFormViewRecord) and FFormViewRecord.Visible then
      TFormLKDataComponent(FFormViewRecord).ActionUndo.Enabled :=
        (ChangeCount > 0);
end;

procedure TLKDataComponent.DoDetailAfterInsert;
begin
  if Assigned(FDetailAfterInsert) then
    FDetailAfterInsert(TblDetail);
end;

procedure TLKDataComponent.DoDetailAfterPost;
var
  I: Integer;
  X: Integer;
  Values: Array[0..500] of Currency;
  SubtractValue: Array[0..500] of Boolean;
  MasterTotalFieldNames, DetailSumFieldNames: TStringList;
  DetailFieldName: string;
  MyBookmark: TBookmark;

  procedure AddDetailValues;
  var
    I: Integer;
    Value: Currency;
  begin
    for I := 0 to DetailSumFieldNames.Count - 1 do
    begin
      Value := GetCurrency(TblDetail.FieldByName(DetailSumFieldNames[I]));
      if SubtractValue[I] then
        Values[I] := Values[I] - Value
      else
        Values[I] := Values[I] + Value;
    end;
  end;

  procedure PostMasterFields;
  var
    I: Integer;
  begin
    FTblMaster.Edit;
    for I := 0 to MasterTotalFieldNames.Count - 1 do
      AddCurrencyToField(Values[I],
        FTblMaster.FieldByName(MasterTotalFieldNames[I]));
    FTblMaster.Post;
  end;

begin
  if FRefreshMSumDPairs and (FMSumDPairFieldNames.Count > 0) then
  try
    MasterTotalFieldNames := TStringList.Create;
    DetailSumFieldNames := TStringList.Create;
    try
      I := 0;
      for X := 0 to Trunc(FMSumDPairFieldNames.Count / 2) - 1 do
      begin
        MasterTotalFieldNames.Add(FMSumDPairFieldNames[I]);
        Inc(I);
        Values[X] := 0;
        DetailFieldName := FMSumDPairFieldNames[I];
        SubtractValue[X] := (Pos('-', DetailFieldName) = 1);
        if (Pos('+',DetailFieldName) = 1) or (Pos('-',DetailFieldName) = 1) then
          System.Delete(DetailFieldName, 1, 1);
        DetailSumFieldNames.Add(DetailFieldName);
        Inc(I);
      end;
      if FNewDetailRecord then
      begin
        AddDetailValues;
        PostMasterFields;
      end
      else
      begin
        FTblMaster.Edit;
        for I := 0 to MasterTotalFieldNames.Count - 1 do
          FTblMaster.FieldByName(MasterTotalFieldNames[I]).AsCurrency := 0;
        FTblMaster.Post;
        MyBookmark := TblDetail.GetBookMark;
        ExternalDisableControls;
        try
          if TblDetail.FindFirst then
          repeat
            AddDetailValues;
          until not TblDetail.FindNext;
          PostMasterFields;
        finally
          ExternalEnableControls;
          TblDetail.GotoBookMark(MyBookmark);
          TblDetail.FreeBookmark(MyBookmark);
        end;
      end;
    finally
      MasterTotalFieldNames.Free;
      DetailSumFieldNames.Free;
    end;
  except
    Cancel;
    TblDetail.Delete;
    raise;
  end;
  if Assigned(FDetailAfterPost) then
    FDetailAfterPost(TblDetail);
  if Assigned(FormView) then
    if FormView.Visible then
      TFormLKDataComponent(FormView).ActionUndo.Enabled :=
        (ChangeCount > 0);
  if Assigned(FFormViewRecord) then
    if FFormViewRecord.Visible then
      TFormLKDataComponent(FFormViewRecord).ActionUndo.Enabled :=
        (ChangeCount > 0);
end;

procedure TLKDataComponent.DoDetailBeforeDelete;
var
  I, X: Integer;
  MasterFieldName, DetailFieldName: string;
  SubtractValue: Boolean;
  Value: Double;
begin
  try
    if RefreshMSumDPairs then
      with FMSumDPairFieldNames do
      begin
        I := 0;
        for X := 1 to Trunc(Count / 2) do
        begin
          MasterFieldName := Strings[I];
          Inc(I);
          DetailFieldName := Strings[I];
          Inc(I);
          SubtractValue := (pos('-', DetailFieldName) = 1);
          if (Pos('+', DetailFieldName) = 1)
            or (Pos('-', DetailFieldName) = 1) then
              DetailFieldName :=
                Copy(DetailFieldName, 2, Length(DetailFieldName) - 1);
          Value := GetFloat(TblDetail.FieldByName(DetailFieldName));
          if SubtractValue then
            Value := -Value;
          with FTblMaster do
          begin
            Edit;
            AddFloatToField(-Value, FieldByName(MasterFieldName));
            Post;
          end;
        end;
      end;
    if Assigned(FDetailBeforeDelete) then
      FDetailBeforeDelete(TblDetail);
  except Cancel;
    raise;
  end;
end;

procedure TLKDataComponent.DoDetailBeforeEdit;
begin
  with FTblMaster do
    if (State in [dsEdit, dsInsert]) then
      Post;
  if Assigned(FDetailBeforeEdit) then
    FDetailBeforeEdit(TblDetail);
end;

procedure TLKDataComponent.DoDetailBeforeInsert;
begin
  if FTblMaster.State in [dsEdit, dsInsert] then
    FTblMaster.Post;
  if (FTblMaster.RecordCount = 0) then
    raise LKError.create('You cannot insert a record into ' + nameFriendly +
      ' details until you have entered a master record');
  if Assigned(FDetailBeforeInsert) then
    FDetailBeforeInsert(TblDetail);
end;

procedure TLKDataComponent.DoDetailBeforePost;
begin
  if FTblMaster.State in [dsEdit, dsInsert] then
    FTblMaster.Post;
  FNewDetailRecord := (TblDetail.State = dsInsert);
  if Assigned(FDetailBeforePost) then
    FDetailBeforePost(TblDetail);
end;

procedure TLKDataComponent.DoDetailReconcileError(DataSet: TCustomClientDataSet;
    E: EReconcileError; UpdateKind: TUpdateKind; var Action: TReconcileAction);
begin
  if Assigned(FOnDetailReconcileError) then
    FOnDetailReconcileError(DataSet, E, UpdateKind, Action)
  else
    Action := HandleReconcileError(DataSet, UpdateKind, E);
end;

procedure TLKDataComponent.DoExportBefore;
begin
  if Assigned(FOnExportBefore) then
    FOnExportBefore(Self);
end;

procedure TLKDataComponent.DoExportComplete;
begin
  if Assigned(FOnExportComplete) then
    FOnExportComplete(Self);
end;

procedure TLKDataComponent.DoExportPrintReport;
begin
  if Assigned(FOnExportPrintReport) then
    FOnExportPrintReport(self);
end;

procedure TLKDataComponent.DoImportBefore;
begin
  if Assigned(FOnImportBefore) then
    FOnImportBefore(Self);
end;

procedure TLKDataComponent.DoImportComplete;
begin
  if Assigned(FOnImportComplete) then
    FOnImportComplete(Self);
end;

procedure TLKDataComponent.DoMasterAfterDelete;
begin
  if Assigned(FMasterAfterDelete) then
    FMasterAfterDelete(FTblMaster);
end;

procedure TLKDataComponent.DoMasterAfterInsert;
begin
  if Assigned(FMasterAfterInsert) then
    FMasterAfterInsert(FTblMaster);
end;

procedure TLKDataComponent.DoMasterAfterPost;
begin
  if Assigned(FMasterAfterPost) then
    FMasterAfterPost(FTblMaster);
  if Assigned(FormView) then
    if FormView.Visible then
      TFormLKDataComponent(FormView).ActionUndo.Enabled := (ChangeCount >
        0);
  if Assigned(FFormViewRecord) then
    if FFormViewRecord.Visible then
      TFormLKDataComponent(FFormViewRecord).ActionUndo.Enabled :=
        (ChangeCount > 0);
end;

procedure TLKDataComponent.DoMasterBeforeDelete;
var
  WasFiltered: Boolean;
begin
  if Assigned(FMasterBeforeDelete) then
    FMasterBeforeDelete(FTblMaster);
  if TblDetailUsed then
    if (TblDetail.MasterFields > '') and FDeleteDetailWithMasterRecord then
     begin
       if (TblDetail.State in [dsEdit, dsInsert]) then
         TblDetail.Cancel;
      WasFiltered := TblDetail.Filtered;
      TblDetail.Filtered := False;
      try
        while TblDetail.FindLast do
          TblDetail.Delete;
      finally TblDetail.Filtered := WasFiltered;
      end;
    end;
end;

procedure TLKDataComponent.DoMasterBeforeEdit;
begin
  if TblDetailUsed then
    if (TblDetail.State in [dsEdit, dsInsert]) then
      TblDetail.Post;
  if Assigned(FMasterBeforeEdit) then
    FMasterBeforeEdit(FTblMaster);
end;

procedure TLKDataComponent.DoMasterBeforeInsert;
var
  MasterComponentTable: TLKTableBase;
begin
  if not FOKToInsert then
    SysUtils.Abort;
  if Assigned(FMasterComponent) and (Length(MasterFields) > 0) then
  begin
    if (FMasterTable = tbMaster) then
      MasterComponentTable := MasterComponent.FTblMaster
    else
      MasterComponentTable := MasterComponent.TblDetail;
    if MasterComponentTable.State in [dsEdit, dsInsert] then
      MasterComponentTable.Post;
    if (MasterComponentTable.RecordCount = 0) then
      raise LKError.Create('You cannot enter a record into ' + NameFriendly +
        ' until you have entered linked information into ' +
          MasterComponent.NameFriendly);
  end;
  if Assigned(FMasterBeforeInsert) then
    FMasterBeforeInsert(FTblMaster);
end;

procedure TLKDataComponent.DoMasterBeforePost;
begin
  if Assigned(FMasterBeforePost) then
    FMasterBeforePost(FTblMaster);
end;

procedure TLKDataComponent.DoMasterCalcFields;
begin
  if Assigned(FOnMasterCalcFields) then
    FOnMasterCalcFields(FTblMaster);
end;

procedure TLKDataComponent.DoMasterReconcileError(DataSet: TCustomClientDataSet;
    E: EReconcileError; UpdateKind: TUpdateKind; var Action: TReconcileAction);
var
  wasOKToInsert: Boolean;
begin
  if Assigned(FOnMasterReconcileError) then
    FOnMasterReconcileError(DataSet, E, UpdateKind, Action)
  else
  begin
    wasOKToInsert := FOKToInsert;
    FOKToInsert := True;
    try
      Action := HandleReconcileError(DataSet, UpdateKind, E);
    finally FOKToInsert := wasOKToInsert;
    end;
  end;
end;

procedure TLKDataComponent.DoPrintComplete(Sender: TObject);
begin
  if Assigned(FOnPrintComplete) then
    FOnPrintComplete(sender, self);
end;

procedure TLKDataComponent.DoPrintHeader(Sender: TObject);
begin
  if Assigned(FOnPrintHeader) then
    FOnPrintHeader(sender, self);
end;

function TLKDataComponent.DoView: Boolean;
begin
  Result := Assigned(FOnView);
  if Result then
    FOnView(self);
end;

function TLKDataComponent.DoViewRecord: Boolean;
begin
  Result := Assigned(FOnViewRecord);
  if Result then
    FOnViewRecord(self);
end;

procedure TLKDataComponent.DrillDown;
begin
  ViewRecord(null, True, [mbOK])
end;

procedure TLKDataComponent.DrillDownRecord;
begin
  if Assigned(FOnDetailDrillDown) then
    FOnDetailDrillDown(Self);
end;

procedure TLKDataComponent.EmptyTables;
begin
  FTblMaster.EmptyTable;
  if TblDetailUsed then
    TblDetail.EmptyTable;
end;

function TLKDataComponent.ExportData(const masterTextFileName,
    detailTextFileName: string; EMasterfields, EDetailFields: TLKFieldNumbers):
    Boolean;
var
  exportMaster, exportDetail: Boolean;
  MasterExportFile, DetailExportFile: TextFile;
  i, x: Integer;
  textRecord: string;
  err: string;

  function FieldAsString(theField: TField): string;
  var
    ds, ms, ys, fillChar: string;
    d, m, y: word;
    a, x, l: integer;
  begin
    // load field Value into Result
    case theField.DataType of
      ftBoolean:
        if (theField.Value = True) then
          Result := 'True'
        else
          Result := 'False';
      ftDate, ftDateTime:
        begin
          if theField.IsNull then
            Result := ''
          else
          begin
            DecodeDate(GetDate(theField), y, m, d);
            ds := intToStr(d);
            if (d < 10) then
              ds := '0' + ds;
            ms := intToStr(m);
            if (m < 10) then
              ms := '0' + ms;
            ys := intToStr(y);
            if (y < 100) then
              ys := '20' + ds;
            Result := ds + ASCIIDateSeparator + ms + ASCIIDateSeparator + ys;
          end;
        end;
      ftCurrency, ftFloat:
        case theField.Tag of
          1: Result := floatToStrF(GetCurrency(theField) + 0.49, ffFixed, 15, 0);
            // integer value only
          2: Result := floatToStrF(GetCurrency(theField) * 100, ffFixed, 15, 0);
            // in pence
        else
          Result := floatToStrF(GetCurrency(theField), ffFixed, 15, 2);
            // normal N.NN format
        end;
      ftSmallInt, ftInteger, ftLargeInt: Result :=
        intToStr(getInteger(theField));
    else
      Result := theField.AsString;
    end;
    // if fixed field size (ref displayWidth) then crop or pad out Result with fill char
    if FASCIIFixedFieldSize then
    begin
      x := theField.DisplayWidth;
      l := length(Result);
      case theField.DataType of //remove century if required
        ftDate, ftDateTime:
          begin
            if (x = 6) and (l = 8) then
              System.Delete(Result, 5, 2);
            if (x = 8) and (l = 10) then
              System.Delete(Result, 7, 2);
          end;
      else
        begin
          if (l > x) then // crop Result because it is too big else pad out
            Result := copy(Result, 1, x)
          else
          begin
            case theField.DataType of
              ftCurrency, ftFloat, ftSmallInt, ftInteger, ftLargeInt: fillChar
                := FASCIIFillCharNumber;
            else
              fillChar := FASCIIFillCharString;
            end;
            for a := l + 1 to x do
              Result := fillChar + Result;
          end;
        end;
      end; // case
    end; // fixed size
    case theField.DataType of
      ftString: Result := ASCIIStringIdentifier + Result +
        ASCIIStringIdentifier;
      ftDate, ftDateTime: Result := ASCIIDateIdentifier + Result +
        ASCIIDateIdentifier;
    end;
  end;

begin
  FWasActive := Active;
  open;
  try
    doExportBefore;
    exportMaster := ((masterTextFileName > '') and FTblMaster.FindFirst);
    exportDetail := ((detailTextFileName > '') and TblDetailUsed);
    Result := (exportMaster or exportDetail);
    if Result then
    begin
      err := ValidateExportBatch;
      Result := (err = '');
    end
    else
      err := 'No tables selected for export';
    if not Result then
    begin
      Showmessage(NameFriendly + ' export error : ' + err);
      Exit;
    end;
    try
      doExportPrintReport;
      if exportMaster then
      begin
        assignFile(MasterExportFile, masterTextFileName);
        if (fileExists(masterTextFileName)) then
        begin
          if (MessageDlg('File name ' + masterTextFileName +
            ' already exists. Do you want to append data to this file?',
            mtInformation, [mbYes, mbAbort], 0) <> mrYes) then
            sysutils.abort;
          System.Append(MasterExportFile);
        end
        else
          rewrite(MasterExportFile);
      end;
      if exportDetail then
      begin
        assignFile(DetailExportFile, detailTextFileName);
        if (fileExists(detailTextFileName)) then
        begin
          if (MessageDlg('File name ' + detailTextFileName +
            ' already exists. Do you want to append data to this file?',
            mtInformation, [mbYes, mbAbort], 0) <> mrYes) then
            sysutils.abort;
          System.Append(DetailExportFile);
        end
        else
          rewrite(DetailExportFile);
      end;
      with FTblMaster do
      begin
        findFirst;
        repeat
          if exportMaster then
          begin
            x := 0;
            for i := 0 to fieldCount - 1 do
            begin
              if i in EMasterFields then
                break;
              inc(x);
            end;
            textRecord := FieldAsString(fields[x]);
            for i := x + 1 to fieldCount - 1 do
              if i in EMasterFields then
                textRecord := textRecord + FASCIIFieldSeparator +
                  FieldAsString(fields[i]);
            writeLn(MasterExportFile, textRecord);
          end;
          if exportDetail then
            with TblDetail do
              if findFirst then
                repeat
                  x := 0;
                  for i := 0 to fieldCount - 1 do
                  begin
                    if i in EDetailFields then
                      break;
                    inc(x);
                  end;
                  textRecord := FieldAsString(fields[x]);
                  for i := x + 1 to fieldCount - 1 do
                    if i in EDetailFields then
                      textRecord := textRecord + FASCIIFieldSeparator +
                        FieldAsString(fields[i]);
                  writeLn(DetailExportFile, textRecord);
                until not FindNext;
        until not FindNext;
      end;
      if exportMaster then
        CloseFile(MasterExportFile);
      if exportDetail then
        CloseFile(DetailExportFile);
      doExportComplete;
    except Result := False;
    end;
  finally Active := FWasActive;
  end;
end;

procedure TLKDataComponent.ExternalDisableControls;
begin
  FTblMaster.ExternalDisableControls;
  if Assigned(TblDetail) then
    TblDetail.ExternalDisableControls;
end;

procedure TLKDataComponent.ExternalEnableControls;
begin
  FTblMaster.ExternalEnableControls;
  if Assigned(TblDetail) then
    TblDetail.ExternalEnableControls;
end;

function TLKDataComponent.FieldByName(const FieldName: string): TField;
begin
  Result := FTblMaster.FieldByName(FieldName);
end;

function TLKDataComponent.FindFirst: Boolean;
begin
  Result := FTblMaster.FindFirst;
end;

function TLKDataComponent.FindKey(const KeyValues: array of const): Boolean;
begin
  Result := FTblMaster.FindKey(KeyValues);
end;

function TLKDataComponent.FindLast: Boolean;
begin
  Result := FTblMaster.FindLast;
end;

function TLKDataComponent.FindNext: Boolean;
begin
  Result := FTblMaster.FindNext;
end;

function TLKDataComponent.FindPrior: Boolean;
begin
  Result := FTblMaster.FindPrior;
end;

function TLKDataComponent.GetActive: Boolean;
begin
  Result := inherited GetActive;
  if Assigned(FTblMaster) then
    Result := FTblMaster.Active;
end;

function TLKDataComponent.GetChangeCount: Integer;
begin
  Result := FTblMaster.ChangeCount;
  if TblDetailUsed then
    Result := Result + FTblDetail.ChangeCount;
end;

function TLKDataComponent.GetDisableStringTrim: Boolean;
begin
  Result := FTblMaster.DisableStringTrim;
end;

function TLKDataComponent.GetFetchOnDemand: Boolean;
begin
  Result := FTblMaster.FetchOnDemand;
end;

function TLKDataComponent.GetFilter: string;
begin
  Result := FTblMaster.Filter;
end;

function TLKDataComponent.GetFiltered: Boolean;
begin
  Result := FTblMaster.Filtered;
end;

function TLKDataComponent.GetFolderStoreDocumentImages: string;
begin
  Result := inherited GetFolderStoreDocumentImages;
  if (Result = '') and Assigned(AccountsHub) then
    Result := AccountsHub.FolderStoreDocumentImages;
end;

function TLKDataComponent.GetIndexDefs: TIndexDefs;
begin
  Result := FTblMaster.IndexDefs;
end;

function TLKDataComponent.GetIndexFieldNames: Widestring;
begin
  Result := FTblMaster.IndexFieldNames;
end;

function TLKDataComponent.GetIndexName: Widestring;
begin
  Result := FTblMaster.IndexName;
end;

function TLKDataComponent.GetMasterFields: Widestring;
begin
  Result := FTblMaster.MasterFields;
end;

function TLKDataComponent.GetRecordCount: Integer;
begin
  Result := FTblMaster.RecordCount;
end;

function TLKDataComponent.GetSequenceNumberNext: Integer;
begin
  FSequenceNumberNext := AccountsHub.SequenceGet(NameFriendly);
  Result := FSequenceNumberNext;
end;

function TLKDataComponent.GetSettings: TModalResult;
begin
  Result := inherited GetSettings;
  FSequenceActive := RecallBool('Settings', 'SequenceActive', FSequenceActive,
    True);
end;

function TLKDataComponent.GetTblDetailUsed: Boolean;
begin
  Result := (TblDetail.TableName > ' ') or Assigned(TblDetail.Clone);
end;

function TLKDataComponent.GetTdDescription: string;
begin
  Result := TblDetail.FieldByName(FTdDescriptionFieldName).AsString;
end;

function TLKDataComponent.ImportData(const masterTextFileName,
    detailTextFileName: string; IMasterFields, IDetailFields: TLKFieldNumbers):
    Boolean;

  procedure LoadFromCSVFile(ATable: TLKTableBase; AFileName: string;
    fieldsToLoad: TLKFieldNumbers);
  var
    F: TextFile;
    i, x: integer;
    textRecord, myData: string;
    d, m, y: word;
  begin
    AssignFile(F, AFileName);
    Reset(F);
    try
      repeat
        ReadLn(F, textRecord);
        FOKToInsert := True;
        try
          ATable.Append;
        finally FOKToInsert := False;
        end;
        for i := 0 to (ATable.FieldCount - 1) do
          if i in fieldsToLoad then
          begin
            try
              if (textRecord = '') then
                break;
              if (pos(FASCIIStringIdentifier, textRecord) = 1) then
              begin
                System.Delete(textRecord, 1, 1);
                myData := Copy(textRecord, 1, pos(FASCIIStringIdentifier +
                  FASCIIFieldSeparator, textRecord) - 1);
                System.Delete(textRecord, 1, length(myData) + 2);
              end
              else
              begin
                x := pos(FASCIIFieldSeparator, textRecord);
                if x = 0 then
                  mydata := textRecord
                else
                  myData := copy(textRecord, 1, x - 1);
                System.Delete(textRecord, 1, length(myData) + 1);
              end;
              if (myData <> '') then
                with ATable do
                  case Fields[i].DataType of
                    ftAutoInc: ;
                    ftTime: Fields[i].Value := myData;
                    ftDate,
                      ftDateTime: if (length(myData) > 5) then
                      begin
                        d := strToInt(copy(myData, 1, 2));
                        system.Delete(myData, 1, 2);
                        if (FASCIIDateSeparator <> '') then
                          system.Delete(myData, 1, 1);
                        m := strToInt(copy(myData, 1, 2));
                        system.Delete(myData, 1, 2);
                        if (FASCIIDateSeparator <> '') then
                          system.Delete(myData, 1, 1);
                        y := strToInt(myData);
                        if (y < 100) then
                          y := y + 2000;
                        Fields[i].AsDateTime := encodeDate(y, m, d);
                      end;
                    ftFloat,
                      ftBCD,
                      ftFmtBCD,
                      ftCurrency: Fields[i].AsFloat := strToFloat(myData);
                    ftString,
                      ftFixedChar,
                      ftWideString,
                      ftMemo, ftFmtMemo: Fields[i].AsString := myData;
                    ftInteger,
                      ftSmallint,
                      ftLargeInt: Fields[i].asInteger := strToInt(myData);
                    ftBoolean: Fields[i].Value := (uppercase(myData[1]) = 'T');
                  end;
            except on E: Exception do
              begin
                ShowMessage(E.Message);
              end;
            end;
          end;
        ATable.Post;
      until Eof(F);
      ATable.ApplyUpdates(0);
    finally
      //with myTable do begin close; free; end;
      Closefile(F);
    end;
  end;

begin
  Result := False;
  FWasActive := Active;
  Open;
  FOKToInsert := True;
  try
    DoImportBefore;
    if (masterTextFileName <> '') then
    begin
      if not (fileExists(masterTextFileName)) then
        raise LKError.Create('File ' + masterTextFileName +
          ' does not exist.');

      LoadFromCSVFile(FTblMaster, masterTextFileName, IMasterFields);
    end;
    if (detailTextFileName > '') and TblDetailUsed then
    begin
      if not (fileExists(detailTextFileName)) then
        raise LKError.Create('File ' + detailTextFileName +
          ' does not exist.');
      LoadFromCSVFile(TblDetail, DetailTextFileName, IDetailFields);
    end;
    DoImportComplete;
    Result := True;
  finally FOKToInsert := False;
    Active := FWasActive;
  end;
end;

procedure TLKDataComponent.Insert;
begin
  FOKToInsert := True;
  try
    FTblMaster.Insert;
  finally FOKToInsert := False;
  end;
end;

procedure TLKDataComponent.InsertRecord(const Values: array of const);
begin
  FOKToInsert := True;
  try
    FTblMaster.InsertRecord(Values);
  finally FOKToInsert := False;
  end;
end;

procedure TLKDataComponent.Loaded;
begin
  inherited Loaded;
  SetContainedComponentNames;
  if Assigned(ACGroup) then
    AccountsHub := ACGroup.AccountsHub;
end;

function TLKDataComponent.LoadFromFile: Boolean;
var
  P: string;
  F: string;
begin
  Result := inherited LoadFromFile;
  if not Result then
    Exit;
  P := DataFolder + PathDelim;
  F := P + FTblMaster.TableName + '.cds';
  Result := FileExists(F);
  if Result then
    FTblMaster.LoadFromFile(F);
  if Result and (TblDetail.TableName > '') then
  begin
    F := P + TblDetail.TableName + '.cds';
    Result := FileExists(F);
    if Result then
      TblDetail.LoadFromFile(F);
  end;
end;

function TLKDataComponent.Locate(const KeyFields: string; const KeyValues:
    Variant; options: TLocateOptions): Boolean;
var
  MasterDocs, GreatMasterDocs: TLKDataComponent;
  I: Integer;
  MyIdxValues: array[0..20] of Variant;
  MDMasterTable, GMDMasterTable: TLKACTable;
  MyBookMark: TBookmark;

  procedure LocateIn(const MyTable: TLKTableBase; LocateFields: string);
  var
    LF: string;
    J, K: integer;
    KeyFields: Variant;
  begin
    // count the field names separated by ';'
    LF := LocateFields;
    if (LF = '') then
      Exit;
    J := 0;
    repeat
      K := pos(';', lf);
      if (K > 0) then
      begin
        System.Delete(lf, 1, K);
        Inc(J);
      end;
    until (K = 0);
    KeyFields := VarArrayCreate([0, J], varVariant);
    for K := 0 to J do
      KeyFields[K] := MyIdxValues[K];
    MyTable.Locate(LocateFields, KeyFields, []);
    {
    case i of
     1 : myTable.Locate(locateFields,myidxValues[0],[]);
     2 : myTable.Locate(locateFields,varArrayOf([myidxValues[0],myidxValues[1]]),[]);
     3 : myTable.Locate(locateFields,varArrayOf([myidxValues[0],myidxValues[1],
      myidxValues[2]]),[]);
     4 : myTable.Locate(locateFields,varArrayOf([myidxValues[0],myidxValues[1],
      myidxValues[2],myidxValues[3]]),[]);
     5 : myTable.Locate(locateFields,varArrayOf([myidxValues[0],myidxValues[1],
      myidxValues[2],myidxValues[3],myidxValues[4]]),[]);
     else myTable.Locate(locateFields,varArrayOf([myidxValues[0],myidxValues[1],
      myidxValues[2],myidxValues[3],myidxValues[4],myidxValues[5]]),[]);
     end;
    }
  end;

begin
  // identify the master component links
  MasterDocs := MasterComponent;
  MDMasterTable := MasterTable;
  GreatMasterDocs := nil;
  GMDMasterTable := tbMaster;
  if Assigned(MasterDocs) then
  begin
    GreatMasterDocs := MasterDocs.MasterComponent;
    GMDMasterTable := MasterDocs.MasterTable;
  end;
  // call inherited to find the required record
  MasterComponent := nil;
  Result := FTblMaster.Locate(Keyfields, KeyValues, options);
  MyBookMark := FTblMaster.GetBookmark;
  Screen.Cursor := crHourglass;
  try
    with FTblMaster do
      for I := 0 to IndexFieldCount - 1 do
        MyIdxValues[I] := FTblMaster.IndexFields[I].Value;
    try //to locate the master records in the master tables
      if Assigned(GreatMasterDocs) then
      begin
        case GMDMasterTable of
          tbMaster:
            LocateIn(GreatMasterDocs.FTblMaster, MasterDocs.MasterFields);
          tbDetail:
            begin
              LocateIn(GreatMasterDocs.FTblMaster,
                GreatMasterDocs.TblDetail.MasterFields);
              LocateIn(GreatMasterDocs.TblDetail, MasterDocs.MasterFields);
            end;
        end;
      end;
      if Assigned(MasterDocs) then
      begin
        case MDMasterTable of
          tbMaster:
            LocateIn(MasterDocs.FTblMaster, MasterFields);
          tbDetail:
            begin
              LocateIn(MasterDocs.FTblMaster, MasterDocs.TblDetail.MasterFields);
              LocateIn(MasterDocs.TblDetail, MasterFields);
            end;
        end;
      end;
    except {bin any exceptions}
    end;
    //restore masterComponent and return to located record
    MasterComponent := MasterDocs;
    FTblMaster.GotoBookmark(MyBookMark);
  finally
    begin
      Screen.Cursor := crDefault;
      FTblMaster.FreeBookmark(MyBookMark);
    end;
  end;
end;

function TLKDataComponent.MakeDocumentReference(const PrefixStr, FillChar,
    ReferenceToExpand: string; ResultLength: integer): string;
var
  I: Integer;
begin
  Result := ReferenceToExpand;
  if (ResultLength = 0) then
    Exit;
  // check if no processing required
  if (length(Result) = ResultLength) then
    if (PrefixStr = '') or (pos(PrefixStr, Result) = 1) then
      Exit;
  if (PrefixStr > '') then
    if (pos(PrefixStr, Result) = 1) then
      system.delete(Result, 1, length(PrefixStr));
  i := ResultLength - length(PrefixStr);
  if (length(Result) > i) then
    system.delete(Result, 1, length(Result) - i);
  while (length(Result) < i) do
    Result := FillChar[1] + Result;
  Result := PrefixStr + Result;
end;

procedure TLKDataComponent.MoveFieldsToDatamodule;
var
  I: Integer;
  F: TField;
begin
  if not (csDesigning in ComponentState) then
    Exit;
  // remove external links
  FTblMaster.Mastersource := nil;
  TblDetail.Mastersource := nil;
  // Component editor may have moved fields into AccountsComponent therefore
  // they must be replaced into the datamodule so that
  // TTable.GetChildren(proc,root) can find them
  for I := 0 to FTblMaster.FieldCount - 1 do
  begin
    F := TField(FTblMaster.Fields[I]);
    if Assigned(F) and (F = TField(FindComponent(F.name))) then
    begin
      RemoveComponent(F);
      FMyFieldOwner.InsertComponent(F);
    end;
  end;
  for I := 0 to TblDetail.FieldCount - 1 do
  begin
    F := TField(TblDetail.Fields[I]);
    if Assigned(F) and (F = TField(FindComponent(F.name))) then
    begin
      RemoveComponent(F);
      FMyFieldOwner.InsertComponent(F);
    end;
  end;
end;

procedure TLKDataComponent.Notification(AComponent: TComponent; Operation:
    TOperation);
begin
  if (Operation = opRemove) then
  begin
    if (AComponent = FMasterComponent) then
      FMasterComponent := nil;
//    if (AComponent = FFormViewRecord) then
//      FFormViewRecord := nil;
    if (AComponent = FMyFindDialog) then
      FMyFindDialog := nil;
    if (AComponent = FMyFindDialogDetail) then
      FMyFindDialogDetail := nil;
  end;
  inherited Notification(AComponent, Operation);
end;

procedure TLKDataComponent.Post;
begin
  if TblDetailUsed then
    with TblDetail do
      if State in [dsEdit, dsInsert] then
        Post;
  with FTblMaster do
    if State in [dsEdit, dsInsert] then
      Post;
end;

procedure TLKDataComponent.RecordDelete;
begin
  FTblMaster.Delete;
end;

function TLKDataComponent.RecordNew: TModalResult;
var
  S: string;
begin
  FIsRecordNew := True;
  try
    Append;
    Result := ViewRecord(null, True, [mbOK, mbCancel]);
    if (Result <> mrOK) then
    begin
      S := NameFriendly;
      if (S[length(S)] = 'S') then
        System.Delete(S, length(S), 1);
      if (MessageDlg('Do you want to cancel (delete) this ' + S, mtWarning,
        [mbYes, mbNo], 0)
        = mrYes) then
        if FTblMaster.State = dsInsert then
          FTblMaster.Cancel
        else
          RecordDelete;
    end;
    ApplyUpdates(0);
  finally FIsRecordNew := False;
  end;
end;

function TLKDataComponent.RecordNewDetail: Boolean;
begin
  Result := True;
  TblDetail.InsertRecord([]);
end;

procedure TLKDataComponent.Refresh;
{  procedure DatasetRefresh(ADataset: TDataset);
//  var
//    HoldFilter: string;
//    WasFiltered: Boolean;
  begin
  {
    if ADataset.Filtered then
    begin
      HoldFilter := ADataset.Filter;
      WasFiltered := ADataset.Filtered;
      ADataset.Filtered := False;
      ADataset.Filter := '';
      try
        ADataset.Refresh;
      finally
        ADataset.Filter := HoldFilter;
        ADataset.Filtered := WasFiltered;
      end;
    end
    else
    ADataset.Refresh;
  end;
 }
begin
  if not Active then
    Exit;
  TblMaster.Refresh;
//  DatasetRefresh(TblMaster);
  if TblDetailUsed then
//    DatasetRefresh(TblDetail);
    TblDetail.Refresh;
end;

procedure TLKDataComponent.RestoreFilter;
begin
  FTblMaster.RestoreFilter;
end;

procedure TLKDataComponent.SaveFilter;
begin
  FTblMaster.SaveFilter;
end;

procedure TLKDataComponent.SaveToFile;
var
  P: string;
begin
  inherited SaveToFile;
  P := DataFolder;
  if not DirectoryExists(P) then
    ForceDirectories(P);
  if (FTblMaster.TableName > '') then
    FTblMaster.SaveToFile(P + PathDelim + FTblMaster.TableName + '.cds');
  if (TblDetail.TableName > '') then
    TblDetail.SaveToFile(P + PathDelim + TblDetail.TableName + '.cds');
end;

procedure TLKDataComponent.SetActionList(Value: TLKActionList);
begin
  if (Value = FActionList) then
    Exit;
  FActionList.Free;
  FActionList := Value;
end;

procedure TLKDataComponent.SetActive(Value: Boolean);
var
  AmOpening: Boolean;
begin
  if (csLoading in ComponentState) then
    Exit;
  AmOpening := (Value and not FTblMaster.Active);
  try
    if AmOpening then
    begin
      if Assigned(AccountsHub) then
      begin
        if not AccountsHub.Active then
          AccountsHub.Open;
      end;
      if Assigned(ACGroup) and not ACGroup.Active then
      begin
        ACGroup.Open;
        Exit; // will be opened by the group
      end;
      DoBeforeOpen;
    end;
    inherited SetActive(Value);
    FTblMaster.Active := Value;
    if TblDetailUsed then
      TblDetail.Active := Active;
  except on E: Exception do
      if Value then
      begin
        MessageDlg('Error opening dataset ' + NameFriendly + ' was ' +
          E.Message,
          mtInformation, [mbAbort], 0);
        if not (csDesigning in ComponentState) then
          Halt;
      end
      else
        Showmessage('Error closing dataset ' + NameFriendly + ' was ' +
          E.Message);
  end;
  if AmOpening and Active then
    DoAfterOpen;
end;

procedure TLKDataComponent.SetASCIIDateIdentifier(const Value: string);
begin
  if (Value = '') then
    FASCIIDateIdentifier := ''
  else
    case Value[1] of
      ' ', chr(39), '"': FASCIIDateIdentifier := Value[1];
    else
      FASCIIDateIdentifier := '';
    end;
end;

procedure TLKDataComponent.SetASCIIFillCharNumber(const Value: string);
begin
  if (Value = '') then
    FASCIIFillCharNumber := ' '
  else
    FASCIIFillCharNumber := Value[1];
end;

procedure TLKDataComponent.SetASCIIFillCharString(const Value: string);
begin
  if (Value = '') then
    FASCIIFillCharString := ' '
  else
    FASCIIFillCharString := Value[1];
end;

procedure TLKDataComponent.SetASCIIStringIdentifier(const Value: string);
begin
  if (Value = '') then
    FASCIIStringIdentifier := ''
  else
    case Value[1] of
      ' ', chr(39), '"': FASCIIStringIdentifier := Value[1];
    else
      FASCIIStringIdentifier := '';
    end;
end;

procedure TLKDataComponent.SetContainedComponentNames;
begin
  inherited SetContainedComponentNames;
  FActionList.Name := Name + 'Actions';
  FTblMaster.Name := Name + 'TblMaster';
  FTblDetail.Name := Name  + 'TblDetail';
end;

procedure TLKDataComponent.SetDetailDS(const Value: TDataSource);
begin
  {
  Datasource component property cannot see the ACtables in the object inspector
  so this provides a method to register a datasource with the table
  }
  TblDetail.ExternalDataSourceAdd(Value);
end;

procedure TLKDataComponent.SetDisableStringTrim(Value: Boolean);
begin
  FTblMaster.DisableStringTrim := Value;
  FTblDetail.DisableStringTrim := Value;
end;

procedure TLKDataComponent.SetFetchOnDemand(Value: Boolean);
begin
  FTblMaster.FetchOnDemand := Value;
  FTblDetail.FetchOnDemand := Value;
end;

procedure TLKDataComponent.SetFilter(const Value: string);
begin
  FTblMaster.Filter := Value;
end;

procedure TLKDataComponent.SetFiltered(const Value: Boolean);
begin
  FTblMaster.Filtered := Value;
end;

procedure TLKDataComponent.SetFilters(const Value: TStrings);
begin
  FFilters.Assign(Value);
end;

procedure TLKDataComponent.SetFiltersDetail(Value: TStrings);
begin
  FFiltersDetail.Assign(Value);
end;

procedure TLKDataComponent.SetFolderStoreDocumentImages(const Value: string);
begin
  inherited SetFolderStoreDocumentImages(Value);
  if Assigned(FormView) then
    TFormLKDataComponent(FormView).SetFolderStoreDocumentImages(Value);
  if Assigned(FormViewRecord) then
    TFormLKDataComponentRecord(FormViewRecord).SetFolderStoreDocumentImages(Value);
end;

procedure TLKDataComponent.SetImages(AImageList: TCustomImageList);
begin
  inherited SetImages(AImageList);
  if csLoading in ComponentState then
    Exit;
  FActionList.Images := FImages;
end;

procedure TLKDataComponent.SetIndexDefs(Value: TIndexDefs);
begin
  FTblMaster.IndexDefs := Value;
end;

procedure TLKDataComponent.SetIndexFieldNames(const Value: Widestring);
begin
  FTblMaster.IndexFieldNames := Value;
end;

procedure TLKDataComponent.SetIndexName(const Value: Widestring);
begin
  FTblMaster.IndexName := Value;
end;

procedure TLKDataComponent.SetMasterComponent(const Value: TLKDataComponent);
begin
  if (FMasterComponent = Value) then
    Exit;
  if Assigned(FMasterComponent) then
    FMasterComponent.ChildComponentRemove(Self);
  FMasterComponent := Value;
  if Assigned(FMasterComponent) then
  begin
    FMasterComponent.FreeNotification(self);
    case FMasterTable of
      tbMaster: FTblMaster.MasterSource := FMasterComponent.FInternalMasterDS;
      tbDetail: FTblMaster.MasterSource := FMasterComponent.FInternalDetailDS;
    end;
    FMasterComponent.ChildComponentAdd(Self);
  end
  else
    FTblMaster.MasterSource := nil;
end;

procedure TLKDataComponent.SetMasterDS(const Value: TDataSource);
begin
  {
  Datasource component property cannot see the ACtables in the object inspector
  so this provides a method to register a datasource with the table
  }
  FTblMaster.ExternalDataSourceAdd(Value);
end;

procedure TLKDataComponent.SetMasterFields(const Value: Widestring);
begin
  FTblMaster.MasterFields := Value;
end;

procedure TLKDataComponent.SetMasterTable(const Value: TLKACTable);
begin
  FMasterTable := Value;
  if Assigned(FMasterComponent) then
    case FMasterTable of
      tbMaster: FTblMaster.MasterSource := FMasterComponent.FInternalMasterDS;
      tbDetail: FTblMaster.MasterSource := FMasterComponent.FInternalDetailDS;
    end;
end;

procedure TLKDataComponent.SetRefreshMSumDPairs(Value: Boolean);
begin
  FRefreshMSumDPairs := Value;
  if Active and Value then
  begin
    FNewDetailRecord := False;
    DoDetailAfterPost;
  end;
end;

procedure TLKDataComponent.SetSequenceActive(Value: Boolean);
begin
  RememberBool('Settings', 'SequenceActive', Value, True);
  FSequenceActive := Value;
end;

procedure TLKDataComponent.SetSequenceNumberNext(Value: Integer);
begin
  FSequenceNumberNext := Value;
  AccountsHub.SequenceSet(NameFriendly, Value);
end;

procedure TLKDataComponent.SetSQLConnection(const Value: TSQLConnection);
begin
  inherited SetSQLConnection(Value);
  FTblMaster.SQLConnection := Value;
  TblDetail.SQLConnection := Value;
end;

procedure TLKDataComponent.SetTdDescription(const Value: string);
begin
  TblDetail.FieldByName(FTdDescriptionFieldName).AsString := Value;
end;

function TLKDataComponent.UndoLastChange(FollowChange: Boolean): Boolean;
begin
  Result := False;
  if TblDetailUsed then
    Result := TblDetail.UndoLastChange(FollowChange);
  if not Result then
    Result := FTblMaster.UndoLastChange(FollowChange);
end;

procedure TLKDataComponent.Validate(AValMsgList: TStrings);
var
  MT, DT: TLKTableBase;
  MS, DS: TDataSource;
  CheckedOK, WasOpen: Boolean;
  IM, ID: Integer;
  CheckMsg: string;

  procedure ValMsg(Msg: string);
  begin
    //if boolean(AbortPtr^) then raise
    // LKError.Create('Validation aborted by operator');
    AValMsgList.Append(NameFriendly + ' ' + Msg);
    Application.ProcessMessages;
  end;

begin
  if not FTblMaster.FindFirst then
  begin
    ValMsg('Validated OK. no Master records found');
    Exit;
  end;
  WasOpen := active;
  Close;
  IM := 0;
  ID := 0;
  MT := TLKTableBase.Create(self);
  DT := TLKTableBase.Create(self);
  DS := TDatasource.Create(self);
  try
    try
      MT.SQLConnection := SQLConnection;
      MT.TableName := FTblMaster.TableName;
      DT.SQLConnection := SQLConnection;
      DT.TableName := TblDetail.TableName;
    except on E: exception do
        ValMsg('Error assigning tables ' + E.Message);
    end;
    try
      with MT do
      begin
        Mastersource := nil;
        Open;
      end;
      if TblDetailUsed then
        // reverse the master detail relationship to trace any detail records
        // not having a master record
      begin
        try
          DS.DataSet := DT;
          MT.MasterSource := DS;
          MT.MasterFields := TblDetail.MasterFields;
          with DT do
          begin
            Open;
            if findFirst then
              repeat
                inc(ID);
                if not MT.findFirst then
                  ValMsg('Detail record number ' + intToStr(ID) +
                    ' without master record ref: ' +
                    fields[0].AsString + ',' + fields[1].AsString);
              until not FindNext;
          end;
        except on E: exception do
            ValMsg('Detail table error ' + E.Message);
        end;
      end;
    except on E: exception do
        ValMsg('Master table error ' + E.Message);
    end;
    MT.MasterSource := nil;
    CheckedOK := True;
  finally
    with MT do
    begin
      close;
      free;
    end;
    with DT do
    begin
      close;
      free;
    end;
    DS.free;
  end;
  MS := FTblMaster.masterSource;
  try
    FTblMaster.mastersource := nil;
    open;
    with FTblMaster do
      if findFirst then
        repeat
          CheckMsg := validateRecord;
          inc(IM);
          if (CheckMsg <> 'OK') then
          begin
            CheckedOK := False;
            ValMsg('Record ' + intToStr(IM) + ' ' + CheckMsg);
          end;
        until not FindNext;
  finally FTblMaster.mastersource := MS;
    active := WasOpen;
  end;
  if CheckedOK then
    ValMsg('Validated OK. Master records ' + intToStr(IM) +
      ' Detail records ' + intToStr(ID));
end;

function TLKDataComponent.ValidateExportBatch: string;
begin
  Result := '';
end;

function TLKDataComponent.ValidateRecord: string;
begin
  Result := 'OK';
  try
    AccountCheckTotals(False, False, True);
  except on E: Exception do
      Result := E.Message;
  end;
end;

function TLKDataComponent.View(const KeyValues: Variant; ShowAsModal: Boolean;
    Buttons: TMsgDlgButtons): TModalResult;
begin
  Result := mrOK;
  if not Active then
  begin
    Open;
    if not Active then
    begin
      Result := mrCancel;
      Exit;
    end;
  end;
  if DoView then
    Exit; // custom handling
  if (FormView = nil) then
  begin
    Screen.Cursor := crHourglass;
    try
    CreateViewForm;
    FormView.Name := Self.Name+'FomView';
    finally
      Screen.Cursor := crDefault;
    end;
  end;
  with TFormLKDataComponent(FormView) do
  begin
    if (KeyValues <> null) then
      Locate(TblDetail.MasterFields,KeyValues,[loCaseInsensitive,loPartialKey]);
    Result := ShowForm(showAsModal, Buttons);
  end;
end;

function TLKDataComponent.ViewAll(const KeyValues: variant; Buttons:
    TMsgDlgButtons): TModalResult;
begin
  if not Active then
  begin
    Open;
    if not Active then
    begin
      Result := mrCancel;
      Exit;
    end;
  end;
  if not Assigned(FMyViewAll) then
  begin
    FMyViewAll := TLKDataComponent.Create(Owner);
    FMyViewAll.ACGroup := ACGroup;
    FMyViewAll.Name := 'All' + Self.Name;
    FMyViewAll.NameFriendly := 'All ' + NameFriendly;
    FMyViewAll.FTblMaster.Assign(FTblMaster);
    FMyViewAll.Filters.AddStrings(Filters);
    FMyViewAll.MasterFields := '';
    FMyViewAll.TotalMasterFieldNames.AddStrings(TotalMasterFieldNames);
    FMyViewAll.TblDetail.Assign(TblDetail);
    FMyViewAll.TotalDetailFieldNames.AddStrings(TotalDetailFieldNames);
    FMyViewAll.Open;
  end;
  Result := FMyViewAll.View(KeyValues, True, Buttons);
end;

function TLKDataComponent.ViewAllDetail(const KeyValues: variant; Buttons:
    TMsgDlgButtons): TModalResult;
begin
  if not Active then
  begin
    Open;
    if not Active then
    begin
      Result := mrCancel;
      Exit;
    end;
  end;
  if not Assigned(FMyViewAllDetail) then
  begin
    FMyViewAllDetail := TLKDataComponent.Create(Owner);
    FMyViewAllDetail.ACGroup := ACGroup;
    FMyViewAllDetail.Name := 'AllDetail' + Self.Name;
    FMyViewAllDetail.NameFriendly := 'All Detail Records ' + NameFriendly;
    FMyViewAllDetail.FTblMaster.Assign(TblDetail);
    FMyViewAllDetail.MasterFields := '';
    FMyViewAllDetail.TotalMasterFieldNames.AddStrings(TotalDetailFieldNames);
    FMyViewAllDetail.Filters.AddStrings(FiltersDetail);
    FMyViewAllDetail.Open;
  end;
  Result := FMyViewAllDetail.View(KeyValues, True, Buttons);
end;

function TLKDataComponent.ViewRecord(const KeyValues: Variant; ShowAsModal:
    Boolean; Buttons: TMsgDlgButtons): TModalResult;
begin
  Result := mrOK;
  if not Active then
  begin
    Open;
    if not Active then
    begin
      Result := mrCancel;
      Exit;
    end;
  end;
  if DoView then
    Exit; // custom handling
  if (FFormViewRecord = nil) then
  begin
    Screen.Cursor := crHourglass;
    try
      CreateViewFormRecord;
      FFormViewRecord.Name := Self.Name+'FormViewRecord';
    finally
      Screen.Cursor := crDefault;
    end;
  end;
  with TFormLKDataComponentRecord(FFormViewRecord) do
  begin
    if (KeyValues <> null) then
      Locate(TblDetail.MasterFields,KeyValues,[loCaseInsensitive,loPartialKey]);
    Result := ShowForm(ShowAsModal, Buttons);
  end;
end;

{
******************************** TLKRecordsBase ********************************
}
constructor TLKRecordsBase.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FFolderStoreDocumentImages := '';
end;

procedure TLKRecordsBase.CreateViewForm;
begin
  Application.CreateForm(TFormLKRecordsBase, FFormView);
  TFormLKRecordsBase(FormView).Construct(Self);
end;

procedure TLKRecordsBase.CreateViewFormRecord;
begin
  Application.CreateForm(TFormLKRecordsBaseRecord, FFormViewRecord);
  TFormLKRecordsBaseRecord(FormViewRecord).Construct(Self);
end;

function TLKRecordsBase.GetSettings: TModalResult;
var
  S: string;
begin
  Result := inherited GetSettings;
  if (FFolderStoreDocumentImages = '') then
  begin
    if Assigned(AccountsHub) then
      S := AccountsHub.FolderStoreDocumentImages
    else
      S := DataFolder + PathDelim + 'images';
    FFolderStoreDocumentImages := RecallString('Settings',
      'FolderStoreDocumentImages', S, True);
  end;
end;

function TLKRecordsBase.GetTdDocumentItemNumber: Integer;
begin
  Result := GetInteger(TblDetail.Fields[1]);
end;

function TLKRecordsBase.GetTmDocumentReference: string;
begin
  Result := TblMaster.Fields[0].AsString;
end;

procedure TLKRecordsBase.SetTdDocumentItemNumber(const Value: Integer);
begin
  TblDetail.Fields[1].AsInteger := Value;
end;

procedure TLKRecordsBase.SetTmDocumentReference(const Value: string);
begin
  TblMaster.Fields[0].AsString := Value;
end;

{
******************************** TLKRecordsList ********************************
}
function TLKRecordsList.Add(AComponent: TLKRecordsBase): Integer;
begin
  Result := inherited Add(AComponent);
end;

function TLKRecordsList.Extract(Item: TLKRecordsBase): TLKRecordsBase;
begin
  Result := TLKRecordsBase(inherited Extract(Item));
end;

function TLKRecordsList.First: TLKRecordsBase;
begin
  Result := TLKRecordsBase(inherited First);
end;

function TLKRecordsList.GetItems(Index: Integer): TLKRecordsBase;
begin
  Result := TLKRecordsBase(inherited Items[Index]);
end;

function TLKRecordsList.IndexOf(AComponent: TLKRecordsBase): Integer;
begin
  Result := inherited IndexOf(AComponent);
end;

procedure TLKRecordsList.Insert(Index: Integer; AComponent: TLKRecordsBase);
begin
  inherited Insert(Index, AComponent);
end;

function TLKRecordsList.Last: TLKRecordsBase;
begin
  Result := TLKRecordsBase(inherited Last);
end;

procedure TLKRecordsList.SetItems(Index: Integer; AComponent: TLKRecordsBase);
begin
  inherited Items[Index] := AComponent;
end;

{
********************************** TLKRecords **********************************
}
procedure TLKRecords.DoAfterSetHub;
begin
  inherited DoAfterSetHub;
  if Assigned(AccountsHub) then
    AccountsHub.RecordsAdd(self);
end;

{
******************************* TLKRecordsASCII ********************************
}
constructor TLKRecordsASCII.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Filter := '[Batch] = NULL';
  Filtered := True;
end;

procedure TLKRecordsASCII.CreateViewForm;
begin
  Application.CreateForm(TFormLKRecordsASCII, FFormView);
  TFormLKRecordsASCII(FormView).Construct(Self);
end;

{
******************************* TLKRecordsInput ********************************
}
constructor TLKRecordsInput.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FCopyFromExcludeFieldsMaster := TStringList.Create;
  FCopyFromExcludeFieldsMaster.Add('BATCH');
  FCopyFromExcludeFieldsMaster.Add('DATE_');
end;

destructor TLKRecordsInput.Destroy;
begin
  FCopyFromExcludeFieldsMaster.Free;
  inherited Destroy;
end;

procedure TLKRecordsInput.CreateViewForm;
begin
  Application.CreateForm(TFormLKRecordsInput, FFormView);
  TFormLKRecordsInput(FormView).Construct(Self);
end;

procedure TLKRecordsInput.CreateViewFormRecord;
begin
  Application.CreateForm(TFormLKRecordsInputRecord, FFormViewRecord);
  TFormLKRecordsInputRecord(FormViewRecord).Construct(Self);
end;

procedure TLKRecordsInput.DlgNewRecordCopyFromExecute;
var
  Mr: TModalResult;
  I: Integer;
  J: Integer;
  K: Integer;
  S: string;
  MasterArray: Variant;
  DetailArray: Variant;
begin
  DlgLKRecordsInputCopyFrom := TDlgLKRecordsInputCopyFrom.Create(nil);
  try
    DlgLKRecordsInputCopyFrom.PopupParent := FormView;
    DlgLKRecordsInputCopyFrom.Construct(Self);
    Mr := DlgLKRecordsInputCopyFrom.ShowModal;
    if (Mr = mrOK) then
    begin
      with TblMaster do
      begin
        MasterArray := VarArrayCreate([1,FieldCount-1],varVariant);
        for I := 1 to FieldCount - 1 do
          MasterArray[I] := Fields[I].Value;
      end;
      J := 0;
      with TblDetail do if TblDetailUsed and FindFirst then
      begin
        repeat
          Inc(J);
        until not FindNext;
        if J = 1 then
          DetailArray :=
            VarArrayCreate([1,FieldCount-1],varVariant)
        else
          DetailArray :=
            VarArrayCreate([1,FieldCount-1,1,J],varVariant);
        FindFirst;
        if J = 1 then
          for I := 1 to FieldCount - 1 do
            DetailArray[I] := Fields[I].Value
        else for K := 1 to J do
        begin
          for I := 1 to FieldCount - 1 do
            DetailArray[I,K] := Fields[I].Value;
          FindNext;
        end;
      end;
      MasterComponent := DlgLKRecordsInputCopyFrom.HoldMasterComponent;
      Insert;
      with TblMaster do
      begin
        for I := 1 to FieldCount - 1 do
          if Fields[I].IsNull
          and (FCopyFromExcludeFieldsMaster.IndexOf(Fields[I].FieldName) < 0)
          and not VarIsNull(MasterArray[I]) then
            Fields[I].Value := MasterArray[I];
        Post;
      end;
      RefreshMSumDPairs := False;
      try
        with TblDetail do
        begin
          if J = 1 then
          begin
            Insert;
            for I := 1 to FieldCount - 1 do
            if Fields[I].IsNull and not VarIsNull(DetailArray[I]) then
              Fields[I].Value := DetailArray[I];
            Post;
          end
          else for K := 1 to J do
          begin
            Insert;
            for I := 1 to FieldCount - 1 do
            if Fields[I].IsNull and not VarIsNull(DetailArray[I,K]) then
              Fields[I].Value := DetailArray[I,K];
            Post;
          end;
        end;
      finally
        RefreshMSumDPairs := True;
      end;
      ApplyUpdates(0);
    end;
  finally
    FreeAndNil(DlgLKRecordsInputCopyFrom);
  end;
  if (Mr = mrOK) then
  begin
    IsRecordNew := True;
    try
      Mr := ViewRecord(Null, True, [mbOK, mbCancel]);
    finally
      IsRecordNew := False;
    end;
    if (Mr <> mrOK) then
    begin
      S := NameFriendly;
      if (S[length(S)] = 's') then
        System.Delete(S, Length(S), 1);
      if (MessageDlg('Do you want to cancel (delete) this ' + S, mtWarning,
        [mbYes, mbNo], 0)
        = mrYes) then
        if TblMaster.State = dsInsert then
          Cancel
        else
          RecordDelete;
      ApplyUpdates(0);
    end;
  end;
end;

procedure TLKRecordsInput.DoAfterSetHub;
begin
  inherited DoAfterSetHub;
  if Assigned(AccountsHub) then
    AccountsHub.RecordsAdd(self);
end;

{
***************************** TLKContainedRecords ******************************
}
constructor TLKContainedRecords.Create(AOwner: TComponent);
begin
  if not (AOwner is TLKSystemComponent) then
    raise LKError.Create('Program error: '
      + ClassName + '.Owner should be a TLKSystemComponent');
  inherited Create(AOwner);
  FMyFieldOwner := ContainerComponent.Owner;
  SetSubComponent(True);
end;

procedure TLKContainedRecords.DoAfterSetACGroup;
begin
  inherited DoAfterSetACGroup;
  if Assigned(ACGroup) then
  begin
    ACGroup.GroupComponentRemove(Self);
  //  AccountsHub := nil;
  end;
end;

function TLKContainedRecords.GetMasterComponentName: string;
begin
  if Assigned(MasterComponent) then
    Result := MasterComponent.NameFriendly
  else Result := '';
end;

function TLKContainedRecords.GetSubComponentOwner: TLKSystemComponent;
begin
  Result := TLKSystemComponent(Owner);
end;

{
***************************** TLKRecordsSequenced ******************************
}
constructor TLKRecordsSequenced.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDlgSetupIsImplemented := True;
  FDocumentRefLength := 8;
  FSequenceActive := True;
  FDocumentRefPrefix := '(not Assigned)';
  FSequenceNumberFillChar := '(not Assigned)';
end;

procedure TLKRecordsSequenced.CreateViewForm;
begin
  Application.CreateForm(TFormLKRecordsSequenced, FFormView);
  TFormLKRecordsSequenced(FormView).Construct(Self);
end;

procedure TLKRecordsSequenced.CreateViewFormRecord;
begin
  Application.CreateForm(TFormLKRecordsSequencedRecord, FFormViewRecord);
  TFormLKRecordsSequencedRecord(FormViewRecord).Construct(Self);
end;

function TLKRecordsSequenced.DlgBlockInsertRecordsExecute: Boolean;
var
  I: Integer;
begin
  with TLKViewDialog.Create(nil) do
  begin
    try
      Datatype := vInteger;
      Title := NameFriendly + ' insert new records';
      HeaderCaption := 'Enter number of new records to insert';
      HeaderComments := 'Insert blank records into ' + NameFriendly;
      Value := 10;
      Result := True;
      I := -1;
      while (Result = True) and ((I < 1) or (I > 100)) do
      begin
        Result := Execute;
        I := Value;
        if (I < 1) then
          ShowMessage('You should insert at least one record!');
        if (I > 100) then
          ShowMessage('You cannot insert more than a hundred records.');
      end;
    finally Free;
    end;
  end;
  if Result then
  begin
    while I > 0 do
    begin
      Insert;
      Post;
      Dec(I);
    end;
    ApplyUpdates(0);
  end;
end;

function TLKRecordsSequenced.DlgSetDocumentsSequenceExecute: TModalResult;
begin
  with TDlgLKRecordsSequenceSet.Create(Self) do
  begin
    try
      Caption := NameFriendly + ' sequence numbering';
      ckbxSequenceActive.Checked := SequenceActive;
      eDocumentRefPrefix.Text := DocumentRefPrefix;
      eSequenceNumberFillChar.Text := SequenceNumberFillChar;
      eSequenceNumber.Text := intToStr(AccountsHub.SequenceGet(NameFriendly));
      Result := ShowModal;
      if (Result = mrOK) then
      begin
        SequenceActive := ckbxSequenceActive.Checked;
        DocumentRefPrefix := eDocumentRefPrefix.Text;
        SequenceNumberFillChar := eSequenceNumberFillChar.Text;
        AccountsHub.SequenceSet(NameFriendly, StrToInt(eSequenceNumber.Text));
      end;
    finally Free;
    end;
  end;
end;

procedure TLKRecordsSequenced.DoMasterAfterInsert;
begin
  inherited DoMasterAfterInsert;
  if SequenceActive and
    (TblMaster.Fields[DocumentReferenceFieldNumber].DataType in [ftString,
      ftWidestring]) then
  begin
    TblMaster.Fields[DocumentReferenceFieldNumber].AsString
      := MakeDocumentReference(DocumentRefPrefix, SequenceNumberFillChar,
      IntToStr(SequenceNumberNext), DocumentRefLength);
    SequenceNumberNext := SequenceNumberNext + 1;
  end;
end;

procedure TLKRecordsSequenced.DoMasterBeforeDelete;
begin
  inherited DoMasterBeforeDelete;
  // reset sequence number if current item is to be deleted
  if SequenceActive and
    (TblMaster.Fields[DocumentReferenceFieldNumber].DataType in [ftString,
      ftWidestring]) then
  begin
    if (TblMaster.Fields[DocumentReferenceFieldNumber].AsString =
      MakeDocumentReference(DocumentRefPrefix,
      SequenceNumberFillChar, intToStr(SequenceNumberNext - 1),
        DocumentRefLength)) then
      SequenceNumberNext := SequenceNumberNext - 1;
  end;
end;

procedure TLKRecordsSequenced.DoMasterBeforePost;
begin
  inherited DoMasterBeforePost;
  if (TblMaster.Fields[DocumentReferenceFieldNumber].DataType in [ftString,
    ftWidestring])
    and SequenceActive then
    with TblMaster do
    begin
      Fields[DocumentReferenceFieldNumber].AsString := MakeDocumentReference(
        DocumentRefPrefix, SequenceNumberFillChar,
        Fields[DocumentReferenceFieldNumber].AsString, DocumentRefLength);
    end;
end;

function TLKRecordsSequenced.DoSetup(const ForceIfSettingsExist: Boolean):
    TModalResult;
begin
  Result := inherited DoSetup(ForceIfSettingsExist);
  if ForceIfSettingsExist then
    Result := DlgSetDocumentsSequenceExecute;
end;

function TLKRecordsSequenced.GetDocumentReferenceFieldNumber: Integer;
begin
  Result := 0;
end;

function TLKRecordsSequenced.GetDocumentRefPrefix: string;
begin
  Result := '';
  if csLoading in ComponentState then
    Exit;
  if (FDocumentRefPrefix = '(not Assigned)') then
    FDocumentRefPrefix := RecallString('Settings', 'DocumentRefPrefix', '',
      True);
  Result := FDocumentRefPrefix;
end;

function TLKRecordsSequenced.GetSequenceNumberFillChar: string;
begin
  Result := '0';
  if csLoading in ComponentState then
    Exit;
  if (FSequenceNumberFillChar = '(not Assigned)') then
    FSequenceNumberFillChar := RecallString('Settings', 'SequenceNumberFillChar',
      '0', True);
  Result := FSequenceNumberFillChar;
end;

function TLKRecordsSequenced.GetTdItemNumber: Integer;
begin
  Result := GetInteger(TblDetail.Fields[1]);
end;

function TLKRecordsSequenced.GetTmDocumentDate: TDateTime;
begin
  Result := GetDate(TblMaster.FieldByName('DATE_'));
end;

procedure TLKRecordsSequenced.SetActive(Value: Boolean);
var
  I: Integer;
begin
  if (csLoading in ComponentState) then
    Exit;
  inherited SetActive(Value);
  if Active and SequenceActive then
  begin
    if (TblMaster.Fields[DocumentReferenceFieldNumber].DataType
      in [ftString, ftWidestring]) then
    begin
      I := TblMaster.Fields[DocumentReferenceFieldNumber].Size;
      if (I < DocumentRefLength) then
        DocumentRefLength := I;
    end;
  end;
end;

procedure TLKRecordsSequenced.SetDocumentRefPrefix(const Value: string);
begin
  if (FDocumentRefPrefix <> Value) then
    RememberString('Settings', 'DocumentRefPrefix', Value, True);
  FDocumentRefPrefix := Value;
end;

procedure TLKRecordsSequenced.SetSequenceNumberFillChar(const Value: string);
begin
  if (Value = '') then
    FSequenceNumberFillChar := ' '
  else
    FSequenceNumberFillChar := Value[1];
  RememberString('Settings', 'SequenceNumberFillChar', FSequenceNumberFillChar,
    True);
end;

procedure TLKRecordsSequenced.SetTdItemNumber(Value: Integer);
begin
  TblDetail.Fields[1].AsInteger := Value;
end;

procedure TLKRecordsSequenced.SetTmDocumentDate(Value: TDateTime);
begin
  TblMaster.FieldByName('DATE_').AsDateTime := Value;
end;

{
******************************** TLKLedgerBase *********************************
}
constructor TLKLedgerBase.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FStatus := acsUnknown;
  FDaybooks := TLKDaybookList.Create(False);
  FOKToEdit := False;
  FLedgerDate := 0;
  FHasPeriodEndProcedure := True;
end;

destructor TLKLedgerBase.Destroy;
begin
  FDaybooks.Free;
  inherited Destroy;
end;

function TLKLedgerBase.AccountCheckTotals(const showOK, correctErrors,
    raiseError: boolean): Boolean;
begin
  OKToEdit := correctErrors;
  try
    Result := inherited AccountCheckTotals(showOK, correctErrors, raiseError);
  finally OKToEdit := False;
  end;
end;

procedure TLKLedgerBase.AccountClear;
var
  n: Integer;
begin
  //if not TblMaster.findKey(KeyValues) then raise
  // EACError.Create('Cannot find account reference to clear in' + nameFriendly);
  OKtoEdit := True;
  try
    with TblDetail do
    begin
      if findFirst then
        repeat delete until not findFirst;
    end;
    if Assigned(FMSumDPairFieldNames) then
      with TblMaster do
      begin
        edit;
        for n := 0 to trunc(FMSumDPairFieldNames.count / 2) - 1 do
        begin
          FieldByName(FMSumDPairFieldNames.strings[2 * n]).AsFloat := 0.00;
        end;
        post;
      end;
  finally OKToEdit := False;
  end;
end;

function TLKLedgerBase.AccountIsValid(const KeyValues: Variant; RaiseError:
    Boolean = False): Boolean;
var
  HoldMasterComponent: TLKDataComponent;
  Code: string;
begin
  if not Active then
    Open;
  Code := KeyValues;
  Result := (Code = '');
  if Result then
    Exit
  else
  begin
    HoldMasterComponent := MasterComponent;
    try
      MasterComponent := nil;
      Result := TblMaster.Locate(TblDetail.MasterFields, KeyValues, []);
      if RaiseError and not Result then
        raise LKError.Create(NameFriendly + ' reference ' + KeyValues[0] +
          ' not found');
    finally MasterComponent := HoldMasterComponent;
    end;
  end;
end;

procedure TLKLedgerBase.AccountMerge(AOwner: TComponent);
var
  SrcAcRef, DestAcRef: Variant;
  myFieldList: TStringList;
  i: Integer;
begin
  SrcAcRef := TblMaster.Fields[0].Value;
  DestAcRef := SrcAcRef;
  if (DlgSelectRecordExecute('Select account to merge from', SrcAcRef) <> mrOK)
    then
    abort;
  if (DlgSelectRecordExecute('Select account to merge into', DestAcRef) <> mrOK)
    then
    abort;
  if (DestAcRef = SrcAcRef) then
    Exit;
  Locate(TblDetail.MasterFields, SrcAcRef, []);
  OKToEdit := True;
  myFieldList := TStringList.Create;
  try
    DelimitedStringToList(TblDetail.MasterFields, ';', myFieldList);
    with TblDetail do
      while FindLast do
      begin
        Edit;
        case myFieldList.Count of
          0: ;
          1: FieldByName(myFieldList[0]).Value := DestAcRef;
        else
          for i := 0 to myFieldList.Count - 1 do
            FieldByName(myFieldList[i]).Value := DestAcRef[i];
        end;
        Post;
      end;
    RecordDelete;
  finally
    myFieldList.Free;
    Locate(TblDetail.MasterFields, DestAcRef, []);
    // OKToEdit := False; in AccountCheckTotals
    AccountCheckTotals(False, True, False);
  end;
end;

procedure TLKLedgerBase.AccountPost(const DetailRecord: array of const);
var
  S: string;
  AccountRef: Variant;
  KeyType: Byte;
  HoldMasterComponent: TLKDataComponent;
  WasOKToEdit: Boolean;
begin
  KeyType := DetailRecord[1].vType;
  case KeyType of
    vtInteger:
      begin
        AccountRef := Integer(DetailRecord[1].vInteger);
        S := IntToStr(AccountRef);
      end;
  else
    begin
      S := string(DetailRecord[1].vString);
      AccountRef := S;
    end;
  end;
  HoldMasterComponent := MasterComponent;
  WasOKToEdit := OKToEdit;
  OKToEdit := True;
  try
    MasterComponent := nil;
    if not TblMaster.Locate(TblDetail.MasterFields, AccountRef, []) then
    begin
      AccountRef := SuspensePostingsCode;
      if not TblMaster.Locate(TblDetail.MasterFields, AccountRef, []) then
        raise LKError.Create('Unable to locate ' + S +
          ' or SuspensePostingsCode ' + SuspensePostingsCode + ' in ' +
            NameFriendly);
    end;
    TblDetail.InsertRecord(DetailRecord); //updates account totals
  finally OKToEdit := WasOKToEdit;
    MasterComponent := HoldMasterComponent;
  end;
end;

procedure TLKLedgerBase.AssignTo(Dest: TPersistent);
begin
  inherited AssignTo(Dest);
  if not (Dest is TLKLedgerBase) then
    Exit;
  TLKLedgerBase(Dest).AccountsHub := AccountsHub;
  //TLKLedgerAccounts(Dest).SuspensePostingsCode := SuspensePostingsCode;
end;

procedure TLKLedgerBase.CreateViewForm;
begin
  Application.CreateForm(TFormLKLedgerBase, FFormView);
  TFormLKLedgerBase(FormView).Construct(Self);
end;

procedure TLKLedgerBase.CreateViewFormRecord;
begin
  Application.CreateForm(TFormLKLedgerBaseRecord, FFormViewRecord);
  TFormLKLedgerBaseRecord(FormViewRecord).Construct(Self);
end;

procedure TLKLedgerBase.DaybookAdd(ADaybook: TLKDaybookBase);
begin
  if (FDaybooks.IndexOf(ADaybook) < 0) then
    FDaybooks.Add(ADaybook);
end;

procedure TLKLedgerBase.DaybookRemove(ADaybook: TLKDaybookBase);
var
  I: Integer;
begin
  if (csDestroying in ComponentState) then
    Exit;
  if Assigned(ADaybook) then
    with FDaybooks do
    begin
      I := IndexOf(ADaybook);
      if (I > -1) then
        Delete(I);
    end;
end;

procedure TLKLedgerBase.DeleteInputDocument;
begin
  // override as required
end;

procedure TLKLedgerBase.DlgLedgerAllocateExecute;
begin
  // override as required
end;

procedure TLKLedgerBase.DoAfterSetHub;
begin
  inherited DoAfterSetHub;
  if Assigned(AccountsHub) then
    AccountsHub.LedgerAdd(self);
end;

procedure TLKLedgerBase.DoDetailBeforeDelete;
begin
  if not OKToEdit then
    raise
      LKError.create('You may not delete a Ledger entry : ' + nameFriendly);
  inherited DoDetailBeforeDelete;
end;

procedure TLKLedgerBase.DoDetailBeforeInsert;
begin
  if not OKToEdit then
    Abort;
  inherited DoDetailBeforeInsert;
end;

procedure TLKLedgerBase.DoDetailBeforePost;
begin
  if OKToEdit then
    inherited DoDetailBeforePost
  else Abort;
end;

procedure TLKLedgerBase.DoDetailReconcileError(DataSet: TCustomClientDataSet;
    E: EReconcileError; UpdateKind: TUpdateKind; var Action: TReconcileAction);
var
  wasOKToEdit: Boolean;
begin
  wasOKToEdit := OKToEdit;
  OKToEdit := True;
  try
  inherited DoDetailReconcileError(DataSet, E, UpdateKind, Action);
  finally OKToEdit := wasOKToEdit;
  end;
end;

procedure TLKLedgerBase.DoMasterBeforeDelete;
begin
  if not OKToEdit then
  begin
    with TblDetail do
    begin
      Filtered := False;
    end;
    try
      if TblDetail.FindFirst then
        raise
          LKError.Create(nameFriendly +
          ': You may not delete a Ledger entry with transactions');
    finally TblDetail.Filtered := TblDetail.Filtered;
    end;
    if (BalanceFieldName > '') then
      if not CurrencyIsZero(GetCurrency(TblMaster.FieldByName(BalanceFieldName)))
        then
        raise
          LKError.Create(nameFriendly +
          ': You may not delete a Ledger entry unless the account balance is zero and there are no transactions');
  end;
  inherited DoMasterBeforeDelete;
end;

procedure TLKLedgerBase.DoMasterReconcileError(DataSet: TCustomClientDataSet;
    E: EReconcileError; UpdateKind: TUpdateKind; var Action: TReconcileAction);
var
  wasOKToEdit: Boolean;
begin
  wasOKToEdit := OKToEdit;
  OKToEdit := True;
  try
  inherited DoMasterReconcileError(DataSet, E, UpdateKind, Action);
  finally OKToEdit := wasOKToEdit;
  end;
end;

procedure TLKLedgerBase.DrillDownRecord;
var
  DbkID, DocNo: string;
begin
  if Assigned(AccountsHub) then
  begin
    with TblDetail do
    begin
      DbkID := FieldByName('DAYBOOK_ID').AsString;
      DocNo := FieldByName('DOCUMENT_NUMBER').AsString;
    end;
    AccountsHub.DrillDownDaybook(DbkID, DocNo);
  end;
end;

procedure TLKLedgerBase.EmptyTables;
begin
  OKToEdit := True;
  try
  inherited EmptyTables;
  finally OKToEdit := False;
  end;
end;

function TLKLedgerBase.EndOfMonth: TModalResult;
begin
  Result := inherited EndOfMonth;
  if (Result <> mrOK) then
    Exit;
  if not Active then
    Open;
  with TLKViewDialog.Create(Owner) do
  try
    DataType := vDate;
    Title := nameFriendly + ' Month End';
    HeaderCaption := 'Confirm the new month end date for the ' + nameFriendly;
    HeaderComments :=
      'The date up to which transactions can be posted to the Ledger';
    Value := EndOfMonthNext;
    if Execute then
    begin
      Result := mrOK;
      LedgerDate := Value;
      Showmessage('You may now enter transactions end for ' + nameFriendly
        + ' up to ' + DateToStr(LedgerDate));
    end
    else
    begin
      Result := mrCancel;
      Showmessage('Month end for ' + nameFriendly + ' not changed from ' +
        DateToStr(LedgerDate));
    end;
  finally Free;
  end;
end;

function TLKLedgerBase.EndOfMonthNext: TDateTime;
begin
  Result := MonthEndNext(LedgerDate);
end;

function TLKLedgerBase.EndOfYearFilter(TransactionMonthsToRetain: Integer):
    string;
begin
  Result := 'DATE_ <= ' + Chr(39)
    + DateToStr(MonthEndThis(LedgerDate - 15 - TransactionMonthsToRetain * 30.5))
    + Chr(39);
end;

function TLKLedgerBase.GetLedgerDate: TDateTime;
begin
  if FLedgerDate = 0 then
    FLedgerDate := recallDate('Settings', 'LedgerDate', MonthEndThis(Date),
      True);
  Result := FLedgerDate;
end;

function TLKLedgerBase.GetStatus: TLKACStatus;
var
  s: string;
begin
  if (FStatus = acsUnknown) then
  begin
    s := RecallString('Settings', 'LedgerStatus', 'X', True);
    case s[1] of
      'X': FStatus := acsNotSetUp;
      'P': FStatus := acsPartSetUp;
      'U': FStatus := acsNotUsed;
      'B': FStatus := acsBasic;
      'A': FStatus := acsAdvanced;
    end;
  end;
  Result := FStatus;
end;

function TLKLedgerBase.GetSuspensePostingsCode: string;
begin
  Result := RecallString('Settings', 'SuspensePostingsCode', '', True);
end;

function TLKLedgerBase.GettmAccountName: string;
begin
  Result := TblMaster.Fields[1].AsString;
end;

function TLKLedgerBase.GettmAccountReference: string;
begin
  Result := TblMaster.Fields[0].AsString;
end;

procedure TLKLedgerBase.InsertRecord(const Values: array of const);
begin
  OKToEdit := True;
  try
  inherited InsertRecord(Values);
  finally OKToEdit := False;
  end;
end;

procedure TLKLedgerBase.Loaded;
begin
  inherited Loaded;
  if Assigned(AccountsHub) then
    AccountsHub.LedgerAdd(Self);
end;

procedure TLKLedgerBase.RecordDelete;
var
  wasFiltered: Boolean;
  wasOkToEdit: Boolean;
begin
  if TblDetail.FindFirst then
    raise
      LKError.Create('You may not delete a Ledger Account that has transactions');
  wasOkToEdit := OkToEdit;
  OkToEdit := True;
  with TblDetail do
  begin
    wasFiltered := Filtered;
    Filtered := False;
  end;
  try
  inherited RecordDelete;
  finally
    OkToEdit := wasOkToEdit;
    TblDetail.Filtered := wasFiltered;
  end;
end;

function TLKLedgerBase.RecordNew: TModalResult;
begin
  OKToEdit := True;
  try
  Result := inherited RecordNew;
  finally OKToEdit := False;
  end;
end;

procedure TLKLedgerBase.SetBalanceFieldName(const Value: string);
begin
  if (FBalanceFieldName = Value) then
    Exit;
  FBalanceFieldName := Uppercase(Value);
  SetBalanceFieldPairs;
  // SetBalanceFieldPairs procedure like
   {
    with FMSumDPairFieldNames do
     begin
     Clear;
     if (BalanceFieldName = '') then Exit;
     add(BalanceFieldName);
     add('+DEBIT');
     add(BalanceFieldName);
     add('-CREDIT');
     end;
   }
end;

procedure TLKLedgerBase.SetBalanceFieldPairs;
begin
  //nop
end;

procedure TLKLedgerBase.SetLedgerDate(Value: TDateTime);
begin
  if FLedgerDate <> Value then
  begin
  if (Value > 0) then
      RememberDate('Settings', 'LedgerDate', Value, True);
  FLedgerDate := Value;
  end;
end;

procedure TLKLedgerBase.SetLedgerID(const Value: string);
var
  S: string;
  L: TLKLedgerBase;
begin
  S := Copy(UpperCase(Value), 1, 4);
  if Assigned(AccountsHub) then
    with AccountsHub do
    begin
      L := Ledgers.ComponentByID(S);
      if Assigned(L) and (L <> self) then
        raise LKError.create('Duplicate LedgerID with ' + L.NameFriendly);
    end;
  FLedgerID := S;
end;

procedure TLKLedgerBase.SetStatus(Value: TLKACStatus);
var
  s: string;
begin
  if FStatus <> Value then
  begin
  case Value of
      acsAdvanced: s := 'A';
      acsPartSetUp: s := 'P';
      acsNotUsed: s := 'U';
      acsBasic: s := 'B';
    else
      s := 'X'; // :acsNotSetUp
    end;
    RememberString('Settings', 'LedgerStatus', s, True);
  FStatus := Value;
  end;
end;

procedure TLKLedgerBase.SetSuspensePostingsCode(const Value: string);
begin
  RememberString('Settings', 'SuspensePostingsCode', Value, True);
end;

procedure TLKLedgerBase.SettmAccountName(const Value: string);
begin
  TblMaster.Fields[1].AsString := Value;
end;

procedure TLKLedgerBase.SettmAccountReference(const Value: string);
begin
  TblMaster.Fields[0].AsString := Value;
end;

function TLKLedgerBase.SumLedgerBalances: Currency;
var
  myBookMark: TBookMark;
  HoldMasterComponent: TLKDataComponent;
begin
  Result := 0;
  if (BalanceFieldName = '') then
    Exit;
  with TblMaster do
  begin
    myBookMark := GetBookMark;
    DisableControls;
  end;
  TblDetail.DisableControls;
  HoldMasterComponent := MasterComponent;
  MasterComponent := nil;
  try
    with TblMaster do
      if FindFirst then
        repeat AddFieldToCurr(FieldByName(BalanceFieldName), Result);
        until not FindNext;
  finally
    MasterComponent := HoldMasterComponent;
    with TblMaster do
    begin
      if BookmarkValid(myBookmark) then
        GotoBookmark(myBookmark);
      FreeBookmark(myBookmark);
      EnableControls;
    end;
    TblDetail.EnableControls;
  end;
end;

{
******************************** TLKLedgerList *********************************
}
function TLKLedgerList.Add(AComponent: TLKLedgerBase): Integer;
begin
  Result := inherited Add(AComponent);
end;

function TLKLedgerList.ComponentByID(const ALedgerID: string): TLKLedgerBase;
var
  I: Integer;
begin
  Result := nil;
  for I := 0 to Count - 1 do
    if (Items[I].LedgerID = ALedgerID) then
    begin
      Result := Items[I];
      Break;
    end;
end;

function TLKLedgerList.Extract(Item: TLKLedgerBase): TLKLedgerBase;
begin
  Result := TLKLedgerBase(inherited Extract(Item));
end;

function TLKLedgerList.First: TLKLedgerBase;
begin
  Result := TLKLedgerBase(inherited First);
end;

function TLKLedgerList.GetItems(Index: Integer): TLKLedgerBase;
begin
  Result := TLKLedgerBase(inherited Items[Index]);
end;

function TLKLedgerList.IndexOf(AComponent: TLKLedgerBase): Integer;
begin
  Result := inherited IndexOf(AComponent);
end;

procedure TLKLedgerList.Insert(Index: Integer; AComponent: TLKLedgerBase);
begin
  inherited Insert(Index, AComponent);
end;

function TLKLedgerList.Last: TLKLedgerBase;
begin
  Result := TLKLedgerBase(inherited Last);
end;

procedure TLKLedgerList.SetItems(Index: Integer; AComponent: TLKLedgerBase);
begin
  inherited Items[Index] := AComponent;
end;

{
******************************** TLKDaybookBase ********************************
}
constructor TLKDaybookBase.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FBatchDate := Date;
  FDocumentDescriptionlFieldNames := TStringList.Create;
  FTreeNodeVisible := False;
  FLedgerAccountNoFieldName := '';
  TblMaster.WhereClause := 'BATCH IS NULL';
end;

destructor TLKDaybookBase.Destroy;
begin
  FDocumentDescriptionlFieldNames.Free;
  inherited Destroy;
end;

procedure TLKDaybookBase.AssignTo(Dest: TPersistent);
var
  DestDB: TLKDaybookBase;
begin
  inherited AssignTo(Dest);
  DestDB := TLKDaybookBase(Dest);
  DestDB.Ledger := FLedger;
  // DestDB.DaybookID := DaybookID;
end;

procedure TLKDaybookBase.CreateViewForm;
begin
  Application.CreateForm(TFormLKDaybookBase, FFormView);
  TFormLKDaybookBase(FormView).Construct(Self);
end;

procedure TLKDaybookBase.DlgRemoveBatchNumberExecute;
begin
  with TLKViewDialog.Create(Self) do
  try
    DataType := vInteger;
    Title := NameFriendly + ': clear Batch Number';
    HeaderCaption := 'Enter Batch Number to clear';
    HeaderComments :=
      'Deletes the Batch Number so that transactions can be (re-)posted to the Ledger';
    Value := GetInteger(TblMaster.FieldByName('BATCH'));
    if Execute then
      RemoveBatchNumber(Value);
  finally
    Free;
  end;
end;

procedure TLKDaybookBase.DlgSetBatchNumberExecute;
begin
  with TLKViewDialog.Create(Self) do
  try
    DataType := vInteger;
    Value := BatchNumber;
    Title := NameFriendly + ' next Batch number';
    HeaderCaption := 'Enter Batch Number';
    HeaderComments := 'Set the next Batch number to use: try to ensure that this does'
      + ' not duplicate a number already used';
    if Execute then
      BatchNumber := Value;
  finally
    Free;
  end;
end;

procedure TLKDaybookBase.DoAfterSetHub;
begin
  inherited DoAfterSetHub;
  if Assigned(AccountsHub) then
    AccountsHub.DaybookAdd(self);
end;

procedure TLKDaybookBase.DrillDown;
begin
  if not Assigned(InputDocuments) then
    inherited
  else
  begin
    InputDocuments.TblMaster.GotoCurrent(TblMaster);
    InputDocuments.ViewRecord(Null, True, [mbOK]);
  end;
end;

function TLKDaybookBase.GetBatchDate: TDateTime;
begin
  Result := FBatchDate;
end;

function TLKDaybookBase.GetBatchDescription: string;
begin
  Result := 'Batch ' + IntToStr(BatchNumber);
end;

function TLKDaybookBase.GetBatchNumber: Integer;
begin
  if (FBatchNumber = 0) and not (csLoading in ComponentState) then
    FBatchNumber := RecallInteger('Daybook', 'BatchNumber', 1, True);
  Result := FBatchNumber;
end;

function TLKDaybookBase.GetDocumentDescription: string;
var
  I: Integer;
begin
  Result := '';
  with DocumentDescriptionlFieldNames, TblMaster do
    for I := 0 to Count - 1 do
    begin
      if (I > 0) then
        Result := Result + ' ';
      Result := Result + FieldByName(Strings[I]).AsString;
    end;
end;

function TLKDaybookBase.GetLedgerAccountNoFieldName: string;
begin
  if (FLedgerAccountNoFieldName = '') then
    if Assigned(Ledger) and Ledger.Active then
      FLedgerAccountNoFieldName := Ledger.TblMaster.Fields[0].FieldName;
  Result := FLedgerAccountNoFieldName;
end;

function TLKDaybookBase.GetSettings: TModalResult;
begin
  Result := inherited GetSettings;
  //BatchDate := Date;
end;

function TLKDaybookBase.GetTblDDocumentItemNumber: Integer;
begin
  Result := GetInteger(TblDetail.Fields[1]);
end;

function TLKDaybookBase.GetTblMDocumentDate: TDateTime;
begin
  Result := GetDate(TblMaster.FieldByName('DATE_'));
end;

function TLKDaybookBase.GetTblMDocumentReference: string;
begin
  Result := TblMaster.Fields[0].AsString;
end;

procedure TLKDaybookBase.Notification(AComponent: TComponent; Operation:
    TOperation);
begin
  if (Operation = opRemove) then
  begin
    if (AComponent = FLedger) then
      FLedger := nil;
  end;
  inherited Notification(AComponent, Operation);
end;

procedure TLKDaybookBase.PbAfterProcessBatch;
begin
  with TblMaster do
    while FindLast do
    begin
      Edit;
      FieldByName('BATCH').AsInteger := BatchNumber;
      Post;
    end;
end;

procedure TLKDaybookBase.PbBeforePostDocument;
begin
  // override as required
end;

procedure TLKDaybookBase.PbBeforeProcessBatch;
begin
  if Assigned(Ledger) then
  begin
    if not Ledger.Active then
      Ledger.ACGroup.Open;
    LedgerTempURN := -1;
    Ledger.TblDetail.Filtered := False;
  end;
  Open;
end;

function TLKDaybookBase.PbCheckBatchIsValid: TModalResult;
begin
  FDlgDaybookErrors.ProblemDaybook := Self;
  Result := mrOK;
  if TblMaster.FindFirst then
  begin
    repeat
      PbCheckBatchItemMasterIsValid;
      if TblDetail.FindFirst then
        repeat
          PbCheckBatchItemDetailIsValid;
        until not TblDetail.FindNext;
    until not TblMaster.FindNext;
  end
  else
  // ErrCodeInteger,DocRef,DocItem,DESCRIPTION,CorrectionAction
    FDlgDaybookErrors.AddError(['999', '', '', 'No documents found', 'finish']);
  with FDlgDaybookErrors do
    if AreErrors then
      Result := ShowModal;
  // if correct errors is actioned then re-check the batch
  if Result = mrRetry then
    Result := PbDlgDaybookErrorsCorrectErrors;
end;

procedure TLKDaybookBase.PbCheckBatchItemDetailIsValid;
begin
  // override as required
end;

procedure TLKDaybookBase.PbCheckBatchItemMasterIsValid;
begin
  if not TblDetail.FindFirst then
    FDlgDaybookErrors.AddWarning(['Y', TblMaster.Fields[0].AsString, '',
      'has no detail records']);
end;

function TLKDaybookBase.PbDlgDaybookErrorsCorrectError(ErrorInfo: TStrings):
    TModalResult;
var
  ErrorCode: Integer;
  DocRef: string;
  DocItem: string;
  DetailFound: Boolean;
begin
  Result := mrOK; // revalidate after corrections
  ErrorCode := StrToInt(ErrorInfo[0]);
  DocRef := ErrorInfo[1];
  DocItem := ErrorInfo[2];
  if Locate(TblMaster.Fields[0].FieldName, DocRef, []) then
  begin
    with TblDetail do DetailFound :=
     (DocItem > '') and Locate(Fields[0].FieldName + ';'+Fields[1].FieldName,
        VarArrayOf([DocRef, DocItem]),[]);
    case ErrorCode of
      999:
        begin
          Result := mrAbort;
          Exit;
        end;
      100:
        if (MessageDlg(ErrorInfo.CommaText + ' - confirm Delete?', mtWarning,
          [mbYes, mbNo], 0) = mrYes) then
          RecordDelete
        else
          Result := ViewRecord(null, True, [mbOK, mbAbort]);
      101:
        begin
          Showmessage(ErrorInfo.CommaText);
          Result := ViewRecord(null, True, [mbOK, mbAbort]);
        end;
      102:
        begin
          Showmessage(ErrorInfo.CommaText);
          Result := InputDocuments.ViewRecord(null, True, [mbOK, mbAbort]);
        end;
      200: if DetailFound then
          TblDetail.Delete;
      201: if DetailFound then
        begin
          Showmessage(ErrorInfo.CommaText);
          Result := ViewRecord(null, True, [mbOK, mbAbort]);
        end;
      202: if DetailFound then
        begin
          Showmessage(ErrorInfo.CommaText);
          Result := InputDocuments.ViewRecord(null, True, [mbOK, mbAbort]);
        end;
    end;
  end;
end;

function TLKDaybookBase.PbDlgDaybookErrorsCorrectErrors: TModalResult;
var
  I: Integer;
begin
  Result := mrOK;
  with FDlgDaybookErrors do
  begin
    if AreErrors then
      with sgErrors do
        for I := 1 to RowCount - 1 do
        begin
          Result := PbDlgDaybookErrorsCorrectError(Rows[I]);
          if (Result <> mrOK) then
            break;
        end;
    if AreWarnings then
      if MessageDlg('There are warnings - do you want to edit the entries?',
        mtInformation, [mbYes, mbNo], 0) = mrYes then
        PbDlgDaybookErrorsCorrectWarnings;
  end;
  if (Result = mrOK) then
    Result := mrRetry; // revalidate after corrections
end;

procedure TLKDaybookBase.PbDlgDaybookErrorsCorrectWarnings;
begin
  if Assigned(InputDocuments) then
    InputDocuments.View(FDlgDaybookErrors.sgWarnings.Cells[1, 2], True, [mbOK])
  else
    View(FDlgDaybookErrors.sgWarnings.Cells[1, 2], True, [mbOK]);
end;

procedure TLKDaybookBase.PbDlgDaybookErrorsCreate;
begin
  FDlgDaybookErrors := TDlgLKDaybookBatchErrors.Create(Self);
end;

procedure TLKDaybookBase.PbFinalisePostings;
begin
  //override in decendants
end;

procedure TLKDaybookBase.PbPostDocumentToLedger;
begin
  //override in decendants
end;

procedure TLKDaybookBase.PbPrintPostingsReport;
begin
  DlgPrintExecute(NameFriendly, 'Default', null, False);
end;

procedure TLKDaybookBase.PbProcessDocument;
begin
  PbBeforePostDocument;
  PbPostDocumentToLedger;
end;

procedure TLKDaybookBase.PbProcessedComplete;
begin
  BatchNumber := FBatchNumber + 1;
  Showmessage('Processing complete!');
end;

procedure TLKDaybookBase.PbRemoveFilter;
begin
  Filtered := False;
  Filter := '';
end;

function TLKDaybookBase.PbSetFilter: Boolean;
begin
  Filtered := False;
  Filter := 'BATCH = NULL AND DATE_ <= '
    + Chr(39) + DateToStr(BatchDate) + Chr(39);
  Filtered := True;
  Result := True;
end;

procedure TLKDaybookBase.PbTablesApplyUpdates;
begin
  ApplyUpdates(0);
  if Assigned(FLedger) then
    FLedger.ApplyUpdates(0);
  if Assigned(FInputDocuments) then
    FInputDocuments.ApplyUpdates(0);
end;

procedure TLKDaybookBase.PbTablesCancelUpdates;
begin
  CancelUpdates;
  if Assigned(Ledger) then
    Ledger.CancelUpdates;
  if Assigned(InputDocuments) then
    InputDocuments.CancelUpdates;
end;

procedure TLKDaybookBase.PbTablesRefresh;
begin
//  Refresh;
  if Assigned(FInputDocuments) then
    FInputDocuments.TblMaster.Refresh;
  if Assigned(FLedger) then
    FLedger.TblDetail.Refresh;
end;

procedure TLKDaybookBase.ProcessBatch(AOwner: TComponent);
var
  TX: TDBXTransaction;
  MR: TModalResult;

begin
  PbBeforeProcessBatch;
  //apply any changes prior to processing
  PbTablesApplyUpdates;
//  Refresh;
  if not PbSetFilter then
    Exit;
  PbDlgDaybookErrorsCreate;
  MR := mrRetry;
  try
    while (MR = mrRetry) do
    begin
      FDlgDaybookErrors.ClearMessages;
      MR := PbCheckBatchIsValid;
    end;
  finally FDlgDaybookErrors.Free;
  end;
  case MR of
    mrAbort, mrCancel:
      begin
        PbTablesCancelUpdates;
        PbRemoveFilter;
        Exit;
      end;
  end;
  Application.ProcessMessages;
  //apply any ccorrections prior to processing
  PbTablesApplyUpdates;
  PbPrintPostingsReport;
  Screen.Cursor := crHourglass;
  try
    with TblMaster do
      if FindFirst then
      repeat
        Application.ProcessMessages;
        PbProcessDocument;
      until not FindNext;
    PbFinalisePostings;
    PbAfterProcessBatch;
    MR := mrRetry;
    Application.ProcessMessages;
    if SQLConnection.InTransaction then
      while MR = mrRetry do
      begin
        if SQLConnection.InTransaction then
          MR :=
            MessageDlg('Cannot continue : a database transaction is in progress',
            mtInformation, [mbRetry, mbAbort], 0)
        else
          MR := mrOK;
        if MR = mrAbort then
        begin
          PbTablesCancelUpdates;
          PbRemoveFilter;
          SysUtils.Abort;
        end;
      end;
    try
      TX := SQLConnection.BeginTransaction;
      PbTablesApplyUpdates;
      SQLConnection.CommitFreeAndNil(TX); {on success, commit the changes}
    except
      SQLConnection.RollbackFreeAndNil(TX); {on failure, undo the changes}
      raise;
    end;
  finally Screen.Cursor := crDefault;
  end;
  PbRemoveFilter;
//  ExternalDisableControls;
// Refresh the input documents and where URN fields are determined at the server
  PbTablesRefresh;
//  ExternalEnableControls;
  PbProcessedComplete;
end;

procedure TLKDaybookBase.RemoveBatchNumber(const BatchNumber: integer);
var
  OldFilter: string;
  DetailBatch, IsDetailIndex, WasFiltered: Boolean;
  I: Integer;
begin
  DetailBatch := TblDetailUsed;
  if DetailBatch then
    DetailBatch := Assigned(TblDetail.FindField('BATCH'));
  IsDetailIndex := False;
  if DetailBatch then
    with TblDetail do
      for I := 0 to IndexFieldCount - 1 do
      begin
        IsDetailIndex := IndexFields[I].FieldName = 'BATCH';
        if IsDetailIndex then
          Break;
      end;
  with TblMaster do
  begin
    OldFilter := Filter;
    WasFiltered := Filtered;
    try
      Filter := 'BATCH = ' + IntToStr(BatchNumber);
      Filtered := True;
      while FindFirst do
      begin
        if DetailBatch then
          with TblDetail do
          begin
            if IsDetailIndex then
              while FindFirst do
              begin
                Edit;
                FieldByName('BATCH').Value := null;
                Post;
              end
            else if FindFirst then
              repeat Edit;
                FieldByName('BATCH').Value := null;
                Post;
              until not FindNext;
          end;
        Edit;
        FieldByName('BATCH').Value := Null;
        post;
      end;
    finally
      Filter := OldFilter;
      Filtered := WasFiltered;
    end;
  end;
end;

procedure TLKDaybookBase.SetActive(Value: Boolean);
begin
  if Value and not Active then
  begin
    if Assigned(FInputDocuments) then
    begin
      FInputDocuments.Open;
      inherited SetActive(FInputDocuments.Active);
      Exit;
    end;
  //  TblMaster.InternaldataSet.CommandText :=
  //    'SELECT * FROM ' + TblMaster.TableName + ' WHERE BATCH IS NULL';
  end;
  inherited SetActive(Value);
  // override setting from clonecursor
//  MasterComponent := nil;
//  IndexName := 'DEFAULT_ORDER';
end;

procedure TLKDaybookBase.SetBatchDate(Value: TDateTime);
begin
  FBatchDate := Value;
end;

procedure TLKDaybookBase.SetBatchNumber(Value: Integer);
begin
  RememberInteger('Daybook', 'BatchNumber', Value, True);
  FBatchNumber := Value;
end;

procedure TLKDaybookBase.SetDaybookID(const Value: string);
var
  S: string;
  D: TLKDaybookBase;
begin
  S := Copy(Uppercase(Trim(Value)), 1, 3); // 3 chars only
  if Assigned(AccountsHub) then
    with AccountsHub do
    begin
      D := Daybooks.ComponentByID(S);
      if Assigned(D) and (D <> Self) then
          raise LKError.Create('Duplicate DaybookID with ' + D.NameFriendly);
    end;
  FDaybookID := Value;
end;

procedure TLKDaybookBase.SetDocumentDescriptionlFieldNames(const Value:
    TStrings);
var
  I: Integer;
begin
  with FDocumentDescriptionlFieldNames do
  begin
    Assign(Value);
    for I := 0 to Count - 1 do
      Strings[I] := Uppercase(Strings[I]);
  end;
end;

procedure TLKDaybookBase.SetInputDocuments(Value: TLKRecordsBase);
begin
  FInputDocuments := Value;
  if Assigned(Value) then
  begin

    ACGroup := Value.ACGroup; {
    if (Value.IndexFieldNames > '') then
     IndexFieldNames := Value.IndexFieldNames
    else
     begin
     IndexDefs.Assign(Value.IndexDefs);
     IndexName := Value.IndexName;
     end;
    tMasterTableName := Value.tMasterTableName;
    tMasterWhereClause := 'BATCH IS NULL';}
    TblMaster.Assign(Value.TblMaster);
//    TblMaster.Clone := Value.TblMaster;
//    IndexName := Value.IndexName; //'DEFAULT_ORDER';
//    if Value.TblDetailUsed then
//    begin
    TblDetail.Assign(Value.TblDetail);
// TODO:
//    TblDetail.Clone := Value.TblDetail;
//    TblDetail.IndexName := Value.TblDetail.IndexName; //'PRIMARY_KEY';
//    TblDetail.MasterFields := Value.TblDetail.MasterFields;
      {if (Value.tDetailIndexName > '') then
       begin
       tDetailIndexName := Value.tDetailIndexName;
       if (Value.tDetailIndexFieldNames > '') then
        tDetailIndexFieldNames := Value.tDetailIndexFieldNames
       else
        begin
        TblDetail.IndexDefs.Assign(Value.TblDetail.IndexDefs);
        tDetailIndexName := Value.tDetailIndexName;
        end;
       end;
      tDetailTableName := Value.tDetailTableName;}
//    end;


//    MasterComponent := nil;
    MasterFields := '';
  end;
end;

procedure TLKDaybookBase.SetLedger(Value: TLKDataComponent);
begin
  if (FLedger = Value) then
    Exit;
  if Assigned(FLedger) then
    FLedger.RemoveFreeNotification(Self);
  FLedger := Value;
  if Assigned(Value) then
    Value.FreeNotification(Self);
end;

procedure TLKDaybookBase.SetTblDDocumentItemNumber(const Value: Integer);
begin
  TblDetail.Fields[1].AsInteger := Value;
end;

procedure TLKDaybookBase.SetTblMDocumentDate(Value: TDateTime);
begin
  TblMaster.FieldByName('DATE_').AsDateTime := Value;
end;

procedure TLKDaybookBase.SetTblMDocumentReference(const Value: string);
begin
  TblMaster.Fields[0].AsString := Value;
end;

function TLKDaybookBase.View(const KeyValues: variant; ShowAsModal: boolean;
    Buttons: TMsgDlgButtons): TModalResult;
begin
  Close;
  Result := inherited View(KeyValues, ShowAsModal, Buttons);
end;

{
******************************** TLKDaybookList ********************************
}
function TLKDaybookList.Add(AComponent: TLKDaybookBase): Integer;
begin
  Result := inherited Add(AComponent);
end;

function TLKDaybookList.ComponentByID(const ADaybookID: string): TLKDaybookBase;
var
  I: Integer;
begin
  Result := nil;
  for I := 0 to Count - 1 do
    if (Items[I].DaybookID = ADaybookID) then
    begin
      Result := Items[I];
      Break;
    end;
end;

function TLKDaybookList.Extract(Item: TLKDaybookBase): TLKDaybookBase;
begin
  Result := TLKDaybookBase(inherited Extract(Item));
end;

function TLKDaybookList.First: TLKDaybookBase;
begin
  Result := TLKDaybookBase(inherited First);
end;

function TLKDaybookList.GetItems(Index: Integer): TLKDaybookBase;
begin
  Result := TLKDaybookBase(inherited Items[Index]);
end;

function TLKDaybookList.IndexOf(AComponent: TLKDaybookBase): Integer;
begin
  Result := inherited IndexOf(AComponent);
end;

procedure TLKDaybookList.Insert(Index: Integer; AComponent: TLKDaybookBase);
begin
  inherited Insert(Index, AComponent);
end;

function TLKDaybookList.Last: TLKDaybookBase;
begin
  Result := TLKDaybookBase(inherited Last);
end;

procedure TLKDaybookList.SetItems(Index: Integer; AComponent: TLKDaybookBase);
begin
  inherited Items[Index] := AComponent;
end;

{
******************************* TLKDaybookSimple *******************************
}
procedure TLKDaybookSimple.DrillDown;
begin
  if Assigned(FOnLedgerDrillDown) then
    FOnLedgerDrillDown(Self)
  else
    inherited;
  {
  if not (LedgerAccountNoFieldName > '') and Assigned(InputDocuments) then Exit;
  with TblMaster do
   begin
   KeyValue1 := Fields[0].AsString; // document reference
   KeyValue2 := FieldByName(LedgerAccountNoFieldName).AsString; //ledger reference
   end;
  InputDocuments.View(varArrayOf([KeyValue2,KeyValue1]),True,[mbOK]);
  }
end;

procedure TLKDaybookSimple.PbBeforeProcessBatch;
begin
  {
  NominalBatchTotal := 0;
  NominalPostings.Clear;
  TotalSalesTax := 0;
  }
  if Assigned(FOnDaybookBeforeProcessBatch) then
    FOnDaybookBeforeProcessBatch(Self);
end;

procedure TLKDaybookSimple.PbCheckBatchItemDetailIsValid;
begin
  {with TblDetail do
   begin
   if (nomCode = '') then
    begin
    for i := 2 to FieldCount -1 do if not Fields[i].IsNull then
     begin
     DlgDaybookErrors.AddError(['201',docRef,item,'Nominal code is blank','Display & correct']);
     Exit;
     end;
    DlgDaybookErrors.AddError(['200',docRef,item,'Blank item','Delete']);
    Exit;
    end;
   end;
  }
  if Assigned(FOnDaybookCheckBatchItemDetailIsValid) then
    FOnDaybookCheckBatchItemDetailIsValid(Self);
end;

procedure TLKDaybookSimple.PbCheckBatchItemMasterIsValid;
begin
  if Assigned(FOnDaybookCheckBatchItemMasterIsValid) then
    FOnDaybookCheckBatchItemMasterIsValid(Self);
end;

procedure TLKDaybookSimple.PbFinalisePostings;
begin
  if Assigned(FOnDaybookFinalisePostings) then
    FOnDaybookFinalisePostings(Self);
end;

procedure TLKDaybookSimple.PbProcessDocument;
begin
  if Assigned(FOnDaybookProcessDocument) then
    FOnDaybookProcessDocument(Self);
end;

function TLKDaybookSimple.PbSetFilter: Boolean;
begin
  {
  Filter := 'BATCH = Null';
  Filtered := True;
  }
  Result := True;
  if Assigned(FOnDaybookSetFilter) then
  try
    FOnDaybookSetFilter(Self);
  except Result := False;
  end;
end;

procedure TLKDaybookSimple.PbTablesApplyUpdates;
begin
  if Assigned(FOnDaybookApplyUpdates) then
    FOnDaybookApplyUpdates(Self);
  inherited PbTablesApplyUpdates;
end;

procedure TLKDaybookSimple.PbTablesCancelUpdates;
begin
  if Assigned(FOnDaybookCancelUpdates) then
    FOnDaybookCancelUpdates(Self);
  inherited PbTablesCancelUpdates;
end;

procedure TLKDaybookSimple.PbTablesRefresh;
begin
  if Assigned(FOnDaybookRefresh) then
    FOnDaybookRefresh(Self);
  inherited PbTablesRefresh;
end;

{
****************************** TLKRecordsPosting *******************************
}
procedure TLKRecordsPosting.AssignTo(Dest: TPersistent);
begin
  inherited;
  TLKRecordsPosting(Dest).Daybook := FDaybook;
end;

procedure TLKRecordsPosting.CreateViewForm;
begin
  Application.CreateForm(TFormLKRecordsPosting, FFormView);
  TFormLKRecordsPosting(FormView).Construct(Self);
end;

procedure TLKRecordsPosting.DaybookProcessBatch(AOwner: TComponent);
//var
//  HoldMasterSource: TDataSource;
//  HoldFilter: string;
//  WasFiltered: Boolean;
begin
  if (FDaybook = nil) then
    raise LKError.Create(
      'Program error: Cannot process - a daybook is not Assigned to ' +
        NameFriendly);
  ApplyUpdates(0);
//  HoldMasterSource := TblMaster.MasterSource;
//  HoldFilter := Filter;
//  WasFiltered := Filtered;
//  try
//    TblMaster.MasterSource := nil;
//    Filter := 'BATCH = NULL';
//    Filtered := True;
    with Daybook do
    begin
//      ExternalDisableControls;
 //     if Active then
 //       Refresh
 //     else Open;
//      ExternalEnableControls;
      View(null, True, [mbOK]);
    end;
//  finally
//    TblMaster.MasterSource := HoldMasterSource;
//    Filter := HoldFilter;
//    Filtered := WasFiltered;
//  end;
end;

procedure TLKRecordsPosting.Notification(AComponent: TComponent; Operation:
    TOperation);
begin
  if (Operation = opRemove) then
  begin
    if (AComponent = FDaybook) then
      FDaybook := nil;
  end;
  inherited Notification(AComponent, Operation);
end;

procedure TLKRecordsPosting.SetDaybook(Value: TLKDaybookBase);
begin
  if (FDaybook = Value) then
    Exit;
  if Assigned(FDaybook) then
  begin
    FDaybook.RemoveFreeNotification(Self);
    FDaybook.InputDocuments := nil;
  end;
  FDaybook := Value;
  if Assigned(Value) then
  begin
    Value.FreeNotification(Self);
    Value.InputDocuments := Self;
  end;
end;

{
*********************************** TLKGroup ***********************************
}
constructor TLKGroup.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FGroupComponents := TLKGroupComponentList.Create(False);
  FSubGroups := TLKGroupList.Create(False);
  FPasswordOK := True;
end;

destructor TLKGroup.Destroy;
begin
  FGroupComponents.Free;
  FSubGroups.Free;
  inherited Destroy;
end;

procedure TLKGroup.AddToActionList(AActionList: TCustomActionList);
var
  I: Integer;
begin
  if not FTreeNodeVisible then
    Exit;
  inherited;
  with FGroupComponents do
    for I := 0 to Count - 1 do
      Items[I].AddToActionList(AActionList);
  for I := 0 to FSubGroups.Count - 1 do
    FSubGroups[I].AddToActionList(AActionList);
end;

procedure TLKGroup.AddToOutlookBar(AnOutlookBar: TJvOutlookBar);
var
  I: Integer;
begin
  if TreeNodeVisible then
  begin
  inherited AddToOutlookBar(AnOutlookBar);
  for I := 0 to FGroupComponents.Count - 1 do
      with FGroupComponents[I] do
        if TreeNodeVisible then
        begin
          AddToOutlookBarPage(Self.OutlookBarPage);
          OutlookBarButton.Tag := I;
        end;
  end;
end;

procedure TLKGroup.AddToTreeView(ATreeView: TTreeView; ATreeNode: TTreeNode);
var
  I: Integer;
  TN: TTreeNode;
begin
  inherited AddToTreeView(ATreeView, ATreeNode);
  TN := ATreeNode;
  if TreeNodeVisible then
  begin
    TN := TreeNode;
    with TN do
    begin
      //SelectedIndex := -1;
      StateIndex := 0;
    end;
    with FGroupComponents do
      for I := 0 to Count - 1 do
        Items[I].AddToTreeView(ATreeView, TN);
  end;
  for I := 0 to FSubGroups.Count - 1 do
    FSubGroups[I].AddToTreeView(ATreeView, TN);
end;

procedure TLKGroup.CreateViewForm;
begin
  Application.CreateForm(TDlgLKGroup, FFormView);
  TDlgLKGroup(FormView).Construct(Self);
end;

procedure TLKGroup.DoAfterSetHub;
begin
  inherited DoAfterSetHub;
  if Assigned(AccountsHub) then
    AccountsHub.GroupAdd(Self);
end;

function TLKGroup.FindComponent(AComponentName: string): TLKGroupComponent;
var
  I: Integer;
begin
  Result := nil;
  for i := 0 to FGroupComponents.Count - 1 do
  begin
    if (FGroupComponents[i].Name = AComponentName) then
      Result := FGroupComponents[i];
    if Assigned(Result) then
      break;
  end;
end;

function TLKGroup.GetSettings: TModalResult;
var
  i: Integer;
begin
  Result := inherited GetSettings;
  with GroupComponents do
    for i := 0 to Count - 1 do
      Result := Items[i].GetSettings;
end;

procedure TLKGroup.GroupComponentAdd(AGroupComponent: TLKGroupComponent);
begin
  with FGroupComponents do
    if (indexOf(AGroupComponent) = -1) then
    begin
      Add(AGroupComponent);
      AGroupComponent.FreeNotification(Self);
    end;
end;

procedure TLKGroup.GroupComponentRemove(AGroupComponent: TLKGroupComponent);
var
  I: Integer;
begin
  if (csDestroying in ComponentState) then
    Exit;
  with FGroupComponents do
  begin
    I := IndexOf(AGroupComponent);
    if (I <> -1) then
      Delete(I);
  end;
end;

procedure TLKGroup.Notification(AComponent: TComponent; Operation: TOperation);
begin
  if (Operation = opRemove) then
  begin
    if (AComponent is TLKGroup) then
      if (AComponent = FMasterGroup) then
        FMasterGroup := nil
      else
        SubGroupRemove(TLKGroup(AComponent));
    if (AComponent is TLKGroupComponent) then
      GroupComponentRemove(TLKGroupComponent(AComponent));
  end;
  inherited Notification(AComponent, Operation);
end;

procedure TLKGroup.SetActive(Value: Boolean);
var
  I: Integer;
  PD: TLKProgressDialog;
begin
  if csLoading in ComponentState then
    Exit;
  if Value then
  begin
    if not FPasswordOK then
      with TFormLKGroupEnterPasswd.Create(nil) do
      try
        while (ShowModal = mrOK) and not FPasswordOK do
          FPasswordOK := (Uppercase(Trim(EditPasswd.Text)) = FPassword);
      finally Free;
      end;
    inherited SetActive(FPasswordOK);
    if not FPasswordOK then
      Exit;
    DoBeforeOpen;
    if Assigned(AccountsHub) then
      if not AccountsHub.Active then
        AccountsHub.Open;
    if Assigned(FMasterGroup) then
      FMasterGroup.Open;
    PD := TLKProgressDialog.Create(nil);
    try
      PD.ProgressLabelCaption := 'Group :  ' + NameFriendly;
      PD.StatusPanel0Width := 80;
      PD.StatusPanel0Text := 'Data set...';
      with FGroupComponents do
      begin
        PD.Max := Count;
        PD.Show('Opening...');
        for I := 0 to Count - 1 do
          begin
            if PD.Aborted then
              if (MessageDlg('Confirm that you wish to close the program?',
                mtWarning, [mbAbort, mbCancel], 0) = mrAbort) then
                Halt;
            if not (Items[I] is TLKDaybookBase) then
            begin
              PD.StatusPanel1Text := Items[I].NameFriendly;
              Application.ProcessMessages;
              Items[I].Open;
            end;
            PD.Stepit;
          end;
        PD.Close;
      end;
    finally PD.Free;
    end;
    DoAfterOpen;
  end
  else
  begin
    inherited SetActive(FPasswordOK);
    with FGroupComponents do
      for I := 0 to Count - 1 do
        Items[I].Close;
    end;
end;

procedure TLKGroup.SetEditing(Value: Boolean);
var
  I: Integer;
begin
  inherited SetEditing(Value);
  with GroupComponents do
    for I := 0 to Count - 1 do
      Items[I].Editing := Value;
end;

procedure TLKGroup.SetFolderLocalSettings(const Value: string);
var
  I: Integer;
begin
  inherited SetFolderLocalSettings(Value);
  for I := 0 to FGroupComponents.Count - 1 do
    FGroupComponents[I].FolderLocalSettings := FolderLocalSettings;
end;

procedure TLKGroup.SetFolderSharedSettings(const Value: string);
var
  I: Integer;
begin
  inherited SetFolderSharedSettings(Value);
  for I := 0 to FGroupComponents.Count - 1 do
    FGroupComponents[I].FolderSharedSettings := FolderSharedSettings;
end;

procedure TLKGroup.SetImages(Value: TCustomImageList);
var
  i: Integer;
begin
  inherited SetImages(Value);
  with FGroupComponents do
    for i := 0 to Count - 1 do
      TLKGroupComponent(Items[i]).Images := Value;
end;

procedure TLKGroup.SetMasterGroup(Value: TLKGroup);
begin
  if (FMasterGroup = Value) then
    Exit;
  if Assigned(FMasterGroup) then
    FMasterGroup.SubGroupRemove(Self);
  FMasterGroup := Value;
  if Assigned(FMasterGroup) then
  begin
    FMasterGroup.SubGroupAdd(self);
    FMasterGroup.FreeNotification(self);
  end;
end;

procedure TLKGroup.SetPassword(const Value: string);
begin
  FPassword := Uppercase(Trim(Value));
  FPasswordOK := (Value = '');
end;

procedure TLKGroup.SetPrintShareLayouts(const Value: Boolean);
var
  i: Integer;
begin
  inherited SetPrintShareLayouts(Value);
  with FGroupComponents do
    for i := 0 to count - 1 do
      TLKGroupComponent(Items[i]).PrintShareLayouts := Value;
end;

procedure TLKGroup.SetSQLConnection(const Value: TSQLConnection);
var
  I: Integer;
begin
  inherited SetSQLConnection(Value);
  //if (csLoading in ComponentState) then
  //  Exit;
  with FGroupComponents do
    for I := 0 to Count - 1 do
      TLKGroupComponent(Items[I]).SQLConnection := Value;
end;

procedure TLKGroup.SetUserName(const Value: string);
var
  i: Integer;
begin
  inherited SetUserName(Value);
  with FGroupComponents do
    for i := 0 to count - 1 do
      TLKGroupComponent(Items[i]).UserName := Value;
end;

procedure TLKGroup.SubGroupAdd(AACgroup: TLKGroup);
begin
  with FSubGroups do
    if (indexOf(AACGroup) = -1) then
    begin
      Add(AACGroup);
      AACGroup.FreeNotification(Self);
    end;
end;

procedure TLKGroup.SubGroupRemove(AACGroup: TLKGroup);
var
  i: Integer;
begin
  if (csDestroying in ComponentState) then
    Exit;
  with FSubGroups do
  begin
    i := IndexOf(AACGroup);
    if (i > -1) then
      Delete(i);
  end;
end;

{
********************************* TLKGroupList *********************************
}
function TLKGroupList.Add(AComponent: TLKGroup): Integer;
begin
  Result := inherited Add(AComponent);
end;

function TLKGroupList.Extract(Item: TLKGroup): TLKGroup;
begin
  Result := TLKGroup(inherited Extract(Item));
end;

function TLKGroupList.First: TLKGroup;
begin
  Result := TLKGroup(inherited First);
end;

function TLKGroupList.GetItems(Index: Integer): TLKGroup;
begin
  Result := TLKGroup(inherited Items[Index]);
end;

function TLKGroupList.IndexOf(AComponent: TLKGroup): Integer;
begin
  Result := inherited IndexOf(AComponent);
end;

procedure TLKGroupList.Insert(Index: Integer; AComponent: TLKGroup);
begin
  inherited Insert(Index, AComponent);
end;

function TLKGroupList.Last: TLKGroup;
begin
  Result := TLKGroup(inherited Last);
end;

procedure TLKGroupList.SetItems(Index: Integer; AComponent: TLKGroup);
begin
  inherited Items[Index] := AComponent;
end;

{
*************************** TLKHubContainedDocuments ***************************
}
constructor TLKHubContainedDocuments.Create(AOwner: TComponent);
begin
  if not (AOwner is TLKHubRecords) then
    raise LKError.Create('Program error: '
      + ClassName + '.Owner should be a TLKHubRecords');
  inherited Create(AOwner);
end;

function TLKHubContainedDocuments.GetContainerComponent: TLKHubRecords;
begin
  Result := TLKHubRecords(inherited ContainerComponent);
end;

{
***************************** TLKHubSystemSettings *****************************
}
constructor TLKHubSystemSettings.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  TblMaster.TableName := 'SYSTEM_SETTINGS';
end;

{
****************************** TLKHubRecordsDiary ******************************
}
constructor TLKHubRecordsDiary.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  TblMaster.TableName := 'DIARY';
end;

procedure TLKHubRecordsDiary.CreateViewForm;
begin
  Application.CreateForm(TFormLKHubDiary, FFormView);
  TFormLKHubDiary(FormView).Construct(Self);
end;

{
******************************** TLKHubRecords *********************************
}
constructor TLKHubRecords.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDBMS := sdbUnknown;
  FDatabaseHost := '';
  FDlgSetupIsImplemented := True;
  FFolderBackup := '';
  FFolderBackupRemovable := '';
  FFolderBackup := '';
  FFolderStoreDocumentImages := '';
  FFolderStoreDocumentImages := '';
  FUserEmailAddr := '';
  FEnterpriseCurrent := '';
  FEnterpriseDefault := '';
  FEnterpriseAccountsYearCurrent := '';
  FUseEmbeddedFirebird := '';
  FUserSupportURL := 'http://www.leskaye.com/zoom/';
  FDataTables := TStringList.Create;
  FEnterprises := TStringList.Create;
  FEnterpriseYears := TStringList.Create;
  FUsers := TStringList.Create;
  FGroups := TLKGroupList.Create(False);
  FRecords := TLKRecordsList.Create(False);
  FLedgers := TLKLedgerList.Create(False);
  FDaybooks := TLKDaybookList.Create(False);
  AccountsHub := Self;
end;

destructor TLKHubRecords.Destroy;
begin
  FGroups.Free;
  FLedgers.Free;
  FDaybooks.Free;
  FRecords.Free;
  FDataTables.Free;
  FEnterprises.Free;
  FEnterpriseYears.Free;
  FUsers.Free;
  inherited Destroy;
end;

procedure TLKHubRecords.AddToActionList(AActionList: TCustomActionList);
var
  I: Integer;
begin
  if FTreeNodeVisible then
    for i := 0 to Groups.Count - 1 do
      with Groups[i] do
        if (MasterGroup = nil) then
          AddToActionList(AActionList);
end;

procedure TLKHubRecords.AddToOutlookBar(AnOutlookBar: TJvOutlookBar);
var
  i: Integer;
begin
  if FTreeNodeVisible then
    for i := 0 to Groups.Count - 1 do
      with Groups[i] do
      begin
        AddToOutlookBar(AnOutlookBar);
        OutlookBarPage.Enabled := TreeNodeVisible;
      end;
end;

procedure TLKHubRecords.AddToTreeView(ATreeView: TTreeView; ATreeNode:
    TTreeNode);
var
  i: Integer;
begin
  ATreeView.Images := FImages;
  //inherited; don't show
  for i := 0 to Groups.Count - 1 do
    with Groups[i] do
      if MasterGroup = nil then
        AddToTreeView(ATreeView, ATreeNode);
end;

function TLKHubRecords.CheckDatabaseExists: Boolean;
begin
  Result := Active;
  if not Result then
  try
    Open;
    Close;
  except on E: Exception do
    begin
      Result := False;
      ShowMessage('Unable to open Database '
        + EnterpriseCurrent + EnterpriseAccountsYearCurrent
        + '; ' + E.Message)
    end;
  end;
end;

procedure TLKHubRecords.CreateDiary;
begin
  FDiary := TLKHubRecordsDiary.Create(ContainerComponent);
end;

procedure TLKHubRecords.CreateSubComponents;
begin
  inherited CreateSubComponents;
  FSystemSettings := TLKHubSystemSettings.Create(ContainerComponent);
  CreateDiary;
end;

procedure TLKHubRecords.DaybookAdd(ADaybook: TLKDaybookBase);
begin
  if (FDaybooks.indexOf(ADaybook) < 0) then
    FDaybooks.Add(ADaybook);
  if ADaybook.HasPeriodEndProcedure then
    FHasPeriodEndProcedure := True;
end;

procedure TLKHubRecords.DaybookRemove(ADaybook: TLKDaybookBase);
var
  I: Integer;
begin
  if (csDestroying in ComponentState) then
    Exit;
  I := FDaybooks.IndexOf(ADaybook);
  if (I > -1) then
    FDaybooks.Delete(I);
end;

function TLKHubRecords.DlgAdminPasswordExecute: TModalResult;
begin
  Result := mrCancel;
  if (AdministratorPassword = '') then
  begin
    if
      (MessageDlg('The Administrator password is not set - do you want to set one now?',
      mtInformation, [mbYes, mbNo], 0) = mrYes) then
      DlgAdminPasswordSetExecute;
    Result := mrOK;
  end
  else
    with TLKViewDialog.Create(Self) do
    try
      DataType := vString;
      Title := 'Administrator password for ' + EnterpriseCurrent;
      HeaderCaption := 'Enter the Administrator password';
      HeaderComments := 'This gives access to administration tools on the menu';
      if Execute and (Value = AdministratorPassword) then
        Result := mrOK;
    finally Free;
    end;
end;

function TLKHubRecords.DlgAdminPasswordSetExecute: TModalResult;
begin
  Result := mrCancel;
  with TLKViewDialog.Create(Self) do
  try
    DataType := vString;
    Title := 'Set the Administrator password';
    HeaderCaption := 'Enter an Administrator password';
    HeaderComments := 'You should set a password if other people have access to'
      + ' your data. Ensure that you do not forget it!!';
    if Execute then
    begin
      AdministratorPassword := Value;
      Result := mrOK;
    end;
  finally Free;
  end;
end;

function TLKHubRecords.DlgDatabaseExecute: TModalResult;
begin
  with TDlgHubDatabase.Create(Owner) do
  try
    Construct(Self);
    Result := Showmodal;
  finally
    Free;
  end;
end;

function TLKHubRecords.DlgEnterpriseExecute: TModalResult;
begin
  with TDlgLKHubEnterprise.Create(Owner) do
  try
    Construct(Self);
    Result := Showmodal;
  finally
    Free;
  end;
end;

function TLKHubRecords.DlgPasswordExecute: TModalResult;
var
  Usr: string;
  Pass: string;
begin
  Result := mrOK;
  // Is there only a DEFAULT user set?
  if (FUsers.Count > 1) then
  begin
    Usr := 'unAssigned';
    with TFormLKHubEnterPasswd.Create(nil) do
    try
      while (FUsers.IndexOf(Usr) < 0) and (Result = mrOK) do
      begin
        Result := ShowModal;
        if (Result <> mrOK) then
          break;
        Usr := Uppercase(eUserName.Text);
        Pass := ePasswd.Text;
        if (FUsers.IndexOf(Usr) < 0) then
        begin
          Result := MessageDlg('Invalid user name ' + Usr, mtInformation,
            [mbOK, mbAbort], 0);
          eUserName.Text := '';
        end
        else if (Crypt(SystemSettings.RecallString('Users', Usr, '', True), 25,
          True) <> Pass) then
        begin
          Result := MessageDlg('Invalid password for ' + Usr, mtInformation,
            [mbOK, mbAbort], 0);
          Usr := 'unAssigned'; // loop again
          ePasswd.Text := '';
        end;
      end; //while
    finally Free;
    end;
    if (Result = mrOK) then
      UserName := Usr
    else
    begin
      Application.Terminate;
      Exit;
    end;
  end
  else
    UserName := 'DEFAULT';
end;

procedure TLKHubRecords.DlgSetupExecute;
begin
  DoSetup(True);
end;

function TLKHubRecords.DlgSetupWizardExecute: TModalResult;
begin
  with TFormLKHubSetupWizard.Create(nil) do
  try
    Construct(Self);
    Result := ShowModal;
  finally
    Free;
  end;
end;

procedure TLKHubRecords.DlgSupportExecute;
begin
  ShellExecute(Application.Handle,'open',PWideChar(FUserSupportURL),nil,nil,1);
end;

procedure TLKHubRecords.DlgUsersSetupExecute;
begin
  with TDlgLKHubEditUser.Create(nil) do
  try
    Construct(Self);
    ShowModal;
  finally
    Free;
  end;
end;

function TLKHubRecords.DoSetup(const ForceIfSettingsExist: Boolean):
    TModalResult;
begin
  Result := inherited DoSetup(ForceIfSettingsExist);

  if ForceIfSettingsExist then //eg new install
    Result := DlgEnterpriseExecute;
end;

procedure TLKHubRecords.DrillDownDaybook(const DaybookID, DocumentReference:
    string);
var
  IsBatch: Boolean;
  S: string;
  D: TLKDaybookBase;
begin
  D := Daybooks.ComponentByID(DaybookID);
  if not Assigned(D) then
    Exit;
  S := DocumentReference;
  IsBatch := (pos('Batch ', S) = 1);
  if IsBatch then
  begin
    System.Delete(S, 1, 6);
    S := Trim(S);
    try
      StrToInt(S);
    except
      IsBatch := False;
    end;
  end;
  with D do
  begin
    if Assigned(InputDocuments) and not IsBatch then
    begin
      InputDocuments.TblMaster.SaveFilter;
      InputDocuments.Filter := '';
      InputDocuments.Filtered := False;
      try
        InputDocuments.ViewRecord(DocumentReference, True, [mbOK])
      finally
        InputDocuments.TblMaster.RestoreFilter;
      end;
    end
    else
    begin
      Filter := '';
      Filtered := False;
      if (TblMaster.WhereClause = '') then
      begin
        if not Active then
          Open;
      end else
      begin
        Close;
        TblMaster.WhereClause := '';
        Open;
      end;
      if IsBatch then
      begin
        Filter := 'BATCH = ' + S;
        Filtered := True;
        View(null, True, [mbOK]);
      end
      else
        ViewRecord(DocumentReference, True, [mbOK]);
    end;
  end;
end;

procedure TLKHubRecords.DrillDownDiary;
begin
  Diary.ViewRecord(null, True, [mbOK]);
end;

function TLKHubRecords.EndOfMonth: TModalResult;
begin
  with TFormLKHubEndOfMonthWiz.Create(Owner) do
  try
    Construct(self);
    Result := ShowModal;
  finally
    Free;
  end;
  if (Result = mrOK) then
  Result := inherited EndOfMonth;
end;

function TLKHubRecords.EndOfYear(ProgressText: string): TModalResult;
var
  I: Integer;
  NewYear: string;
  ThisYearFile: string;
  NextYearFile: string;
  Sl: TStringList;
begin
  ThisYearFile := EnterpriseCurrent + EnterpriseAccountsYearCurrent;
  I := StrToInt(EnterpriseAccountsYearCurrent);
  Inc(I);
  NewYear := IntToStr(I);
  ProgressText := 'Create a data file for the financial year ' + NewYear;
  Application.ProcessMessages;
  NextYearFile := EnterpriseCurrent + NewYear;
  Close;
  if (DBMS = sdbLocalFirebird) then
  begin
    ThisYearFile := DataFolder + PathDelim + ThisYearFile + '.FDB';
    NextYearFile := DataFolder + PathDelim + NextYearFile + '.FDB';
    if not FileExists(NextYearFile) then
    begin
      Result:=MessageDlg('File not found: Please confirm that you wish to create '
        + NextYearFile + ' for the new accounting year... ' + NewYear + ' ?',
        mtInformation, [mbOK, mbAbort], 0);
      if (Result <> mrOK) then
        Exit;
      Showmessage(
        'Please restart the application and re-run this Year End procedure');
      with TIniFile.Create(ProgramFolder + PathDelim + LKFileUtilsExe + '.ini') do
      try
        WriteString('Utils', 'Action', 'FileCopy');
        WriteString('Utils', 'File1', ThisYearFile);
        WriteString('Utils', 'File2', NextYearFile);
        WriteString('Utils', 'File3', '');
        // WriteString('Options','Password','');
      finally
        Free;
      end;
      Execute('LKDataFileUtils.exe', ProgramFolder, 0, 0);
      Halt;
    end;
  end
  else
    Showmessage('Before continuing: Your database administrator should copy your '
      + ThisYearFile
      + ' to a new database file and alias ' + NextYearFile +
        ' then restart Firebird on the server');
  Close;
  EnterpriseAccountsYearCurrent := NewYear;
  try
  inherited EndOfYear(ProgressText);
  except
    Showmessage('Problem opening database '
      + ThisYearFile
      + '. Please locate the problem and restart this program.');
    Result := mrCancel;
    Exit;
  end;
  Sl := TStringList.Create;
  try
    Sl.Text := EnterpriseYears.Text;
    Sl.Add(NewYear);
    EnterpriseYears := Sl;
  finally
    Sl.Free;
  end;
  EnterpriseAccountsYearDefault := NewYear;
  with TFormLKHubEndOfYearWizard.Create(Owner) do
  try
    Construct(Self);
    ProgressText := 'Data file successfully open for the financial year '+NewYear;
    Application.ProcessMessages;
    Result := ShowModal;
  finally
    Free;
  end;
end;

procedure TLKHubRecords.FilesBackup(const BackupRemovable: Boolean);
var
  BackupAction: string;

  label
    HDBackup;

begin
  if BackupRemovable and DoBackupRemovable
    and (Messagedlg('Backup to removable disk?', mtConfirmation,
    [mbYes, mbNo], 0) = mrYes) then
    begin
      BackupAction := 'BackupBoth';
    end
  else
    BackupAction := 'BackupFixed';
  with TIniFile.Create(ProgramFolder+Pathdelim+LKFileUtilsExe + '.ini') do
  try
    WriteString('Utils', 'Action', BackupAction);
    WriteBool('Utils', 'ShowMessages', BackupRemovable);
    WriteString('Utils', 'File1', FolderBackupRemovable);
    WriteString('Utils', 'File2', FolderBackup);
    WriteString('Utils', 'File3', EnterpriseCurrent +
      EnterpriseAccountsYearCurrent);
    // WriteString('Options','Password','');
  finally
    Free;
  end;
  Execute('LKDataFileUtils.exe', ProgramFolder, 0, 0);
  FilesBackupDate := Date;
end;

procedure TLKHubRecords.FilesCheckForBackup;
var
  LastBackupDate: Double;
  NagForBackup: Boolean;
begin
  if (FolderBackup = '') or not (DBMS = sdbLocalFirebird) then
    Exit;
  LastBackupDate := FilesBackupDate;
  // nag if unknown backup date
  NagForBackup := LastBackupDate < (date - 3);
  // nag for backup ?
  if NagForBackup then with TDlgBackupNag.Create(Owner) do
  begin
    try
      lEnterpriseName.Caption := EnterpriseAccountsYearCurrent;
      if LastBackupDate = 0 then
        lBackupDate.Caption := 'unknown date'
      else
        lBackupDate.Caption := DateToStr(LastBackupDate);
      NagForBackup := (ShowModal = mrYes);
    finally free;
    end;
  end;
  FilesBackup(NagForBackup);
end;

procedure TLKHubRecords.FilesRestore;
begin
  if (MessageDlg('Confirm that you want to overwrite the current data with your'
    +' last save?', mtWarning, [mbCancel, mbAll], 0) <> mrAll) then
    Exit;
  with TIniFile.Create(ProgramFolder + PathDelim + LKFileUtilsExe + '.ini') do
  try
    WriteString('Utils', 'Action', 'RestoreFixed');
    WriteString('Utils', 'File1', FolderBackupRemovable);
    WriteString('Utils', 'File2', FolderBackup);
    WriteString('Utils', 'File3', EnterpriseCurrent +
      EnterpriseAccountsYearCurrent);
    // WriteString('Utils','Password','');
  finally Free;
  end;
  Execute('LKDataFileUtils.exe', ProgramFolder, 0, INFINITE);
  Application.Terminate;

  {ZipFileName := FolderBackup + PathDelim + EnterpriseCurrent + EnterpriseAccountsYearCurrent+'.ZIP';
  if not fileExists(ZipFileName) then
    begin Showmessage('Backup file ' + ZipFileName + ' not found.'); Exit; end;
  DoDisconnectDatabase;
  with TLKZipper.Create(self) do
   try UnZip(LKDirRoot + PathDelim + 'data',ZipFileName); finally Free; end;
  Open;}
end;

procedure TLKHubRecords.FilesRestoreRemovable;
begin
  if (MessageDlg('Enter the last disk of your backup set in ' +
    FolderBackupRemovable,
    mtWarning, [mbOK, mbCancel], 0) <> mrOK) then
    Exit;
  with TIniFile.Create(ProgramFolder + PathDelim + LKFileUtilsExe + '.ini') do
  try
    WriteString('Utils', 'Action', 'RestoreRemovable');
    WriteString('Utils', 'File1', FolderBackupRemovable);
    WriteString('Utils', 'File2', FolderBackup);
    WriteString('Utils', 'File3', EnterpriseCurrent +
      EnterpriseAccountsYearCurrent);
    //WriteString('Utils','Password','');
  finally Free;
  end;
  Execute('LKDataFileUtils.exe', ProgramFolder, 0, INFINITE);
  Application.Terminate;

  {ZipFileName := FolderBackupRemovable + PathDelim + EnterpriseCurrent + EnterpriseAccountsYearCurrent+'.ZIP';
  if not fileExists(ZipFileName) then
    begin Showmessage('Backup file ' + ZipFileName + ' not found.'); Exit; end;
  DoDisconnectDatabase;
  with TLKZipper.Create(self) do
   try UnZip(LKDirRoot + PathDelim + 'data',ZipFileName); finally Free; end;
  SQLConnection.KeepConnection := True;
  Open;}
end;

function TLKHubRecords.FindComponent(AComponentName: string): TLKGroupComponent;
var
  I: Integer;
begin
  Result := nil;
  for i := 0 to Groups.Count - 1 do
  begin
    Result := TLKGroup(Groups[i]).FindComponent(AComponentName);
    if Assigned(Result) then
      Break;
  end;
end;

function TLKHubRecords.FindDaybook(const ADaybookID: string): TLKDaybookBase;
begin
  Result := Daybooks.ComponentByID(ADaybookID);
end;

function TLKHubRecords.FindGroup(AGroupName: string): TLKGroup;
var
  I: Integer;
begin
  Result := nil;
  if (AGroupName > '') then
    with FGroups do
      for i := 0 to count - 1 do
        if Items[i].name = AGroupName then
        begin
          Result := Items[i];
          break;
        end;
end;

function TLKHubRecords.FindLedger(const ALedgerID: string): TLKLedgerBase;
begin
  Result := Ledgers.ComponentByID(ALedgerID);
end;

function TLKHubRecords.GetAdministratorPassword: string;
begin
  Result := '';
  if not (csLoading in ComponentState) then
    Result := Crypt(RecallString(EnterpriseCurrent, 'AdministratorPassword', '',
      True), 25, True);
end;

function TLKHubRecords.GetDatabaseHost: string;
begin
  if (FDatabaseHost = '') and not (csLoading in ComponentState) then
    if (DBMS = sdbLocalFirebird) then
      FDatabaseHost := 'Localhost'
    else
      FDatabaseHost := RecallStringIni('Settings', 'DatabaseHost',
        'Localhost', True);
  Result := FDatabaseHost;
end;

function TLKHubRecords.GetDBMS: TLKSupportedDatabaseType;
begin
  if (FDBMS = sdbUnknown) and not (csLoading in ComponentState) then
    DBMS := TLKSupportedDatabaseType(RecallIntegerIni('Settings','DBMS',1,True));
  Result := FDBMS;
end;

function TLKHubRecords.GetDBUserName: string;
begin
  if (FDBUserName = '') and not (csLoading in ComponentState) then
    FDBUserName := RecallStringIni('Settings', 'DBUserName', 'SYSDBA', True);
  Result := FDBUserName;
end;

function TLKHubRecords.GetDBUserPassword: string;
begin
  if (FDBUserPassword = '') and not (csLoading in ComponentState) then
    FDBUserPassword := RecallCryptIni('Settings','DBUserPassword','',25,True);
  Result := FDBUserPassword;
end;

function TLKHubRecords.GetDivision: string;
begin
  Result := Copy(EnterpriseCurrent, 3, 2);
end;

function TLKHubRecords.GetDoBackup: Boolean;
begin
  Result := True;
  if not (csLoading in ComponentState) then
    Result := RecallBool(EnterpriseCurrent, 'DoBackup', True, False);
end;

function TLKHubRecords.GetDoBackupRemovable: Boolean;
begin
  Result := True;
  if not (csLoading in ComponentState) then
    Result := RecallBool(EnterpriseCurrent, 'DoBackupRemovable', True, False);
end;

function TLKHubRecords.GetEnterpriseAccountsEmail: string;
begin
  Result := RecallString(EnterpriseCurrent, 'Email', '', True);
end;

function TLKHubRecords.GetEnterpriseAccountsFax: string;
begin
  Result := RecallString(EnterpriseCurrent, 'Fax', '', True);
end;

function TLKHubRecords.GetEnterpriseAccountsTelephone: string;
begin
  Result := RecallString(EnterpriseCurrent, 'Telephone', '', True);
end;

function TLKHubRecords.GetEnterpriseAccountsYearCurrent: string;
begin
  if (FEnterpriseAccountsYearCurrent = '') then
    FEnterpriseAccountsYearCurrent := EnterpriseAccountsYearDefault;
  Result := FEnterpriseAccountsYearCurrent;
end;

function TLKHubRecords.GetEnterpriseAccountsYearDefault: string;
var
  y, m, d: Word;
begin
  if (csLoading in ComponentState) or (EnterpriseCurrent = '') then
    Result := ''
  else
  begin
    DecodeDate(Date, y, m, d);
    Result := RecallStringIni(EnterpriseCurrent,
      'EnterpriseAccountsYearDefault', IntToStr(y + 1), True);
  end;
end;

function TLKHubRecords.GetEnterpriseAddress1: string;
begin
  Result := RecallString(EnterpriseCurrent, 'Address1', '', True);
end;

function TLKHubRecords.GetEnterpriseAddress2: string;
begin
  Result := RecallString(EnterpriseCurrent, 'Address2', '', True);
end;

function TLKHubRecords.GetEnterpriseAddress3: string;
begin
  Result := RecallString(EnterpriseCurrent, 'Address3', '', True);
end;

function TLKHubRecords.GetEnterpriseAddress4: string;
begin
  Result := RecallString(EnterpriseCurrent, 'Address4', '', True);
end;

function TLKHubRecords.GetEnterpriseAddressPostcode: string;
begin
  Result := RecallString(EnterpriseCurrent, 'POSTCODE', '', True);
end;

function TLKHubRecords.GetEnterpriseCurrent: string;
begin
  if (FEnterpriseCurrent = '') then
    FEnterpriseCurrent := EnterpriseDefault;
  Result := FEnterpriseCurrent;
end;

function TLKHubRecords.GetEnterpriseDefault: string;
begin
  if not (csLoading in ComponentState) then
    if (FEnterpriseDefault = '') then
      FEnterpriseDefault := RecallStringIni('Settings', 'EnterpriseDefault',
       '', True);
  Result := FEnterpriseDefault;
end;

function TLKHubRecords.GetEnterpriseName: string;
begin
  Result := '';
  if not (csLoading in ComponentState) then
    Result := RecallString(EnterpriseCurrent, 'EnterpriseName', '', True);
end;

function TLKHubRecords.GetEnterprises: TStrings;
begin
  if (FEnterprises.Count = 0) then
    RecallSectionIni('Enterprises', FEnterprises, True);
  Result := FEnterprises;
end;

function TLKHubRecords.GetEnterpriseYears: TStrings;
begin
  FEnterpriseYears.Clear;
  if (EnterpriseCurrent > '') then
    RecallSectionIni(EnterpriseCurrent + 'EnterpriseYears', FEnterpriseYears,
      True);
  Result := FEnterpriseYears;
end;

function TLKHubRecords.GetFilesBackupDate: TDateTime;
begin
  Result := 0;
  if not (csLoading in ComponentState) then
    Result := RecallDate(EnterpriseCurrent, 'LastBackupDate', 0, True);
end;

function TLKHubRecords.GetFolderBackup: string;
begin
  if not (csLoading in ComponentState) then
    if (FFolderBackup = '') then
      FFolderBackup := RecallString(EnterpriseCurrent, 'FolderBackup'
        , InstallFolder + PathDelim + 'backup', False);
  Result := FFolderBackup;
end;

function TLKHubRecords.GetFolderBackupRemovable: string;
begin
  if not (csLoading in ComponentState) then
    if (FFolderBackupRemovable = '') then
      FFolderBackupRemovable :=
        RecallString(EnterpriseCurrent, 'FolderBackupRemovable', 'E:\',
          False);
  Result := FFolderBackupRemovable;
end;

function TLKHubRecords.GetSettings: TModalResult;
var
  I: Integer;
  IsNewSetup: Boolean;
  FN: string;
  F: TextFile;
  S: string;
  S1: string;

begin
  Result := mrOK;
  try
    FormatSettings.ShortDateFormat := 'dd/mm/yyyy';
    FormatSettings.LongTimeFormat := 'HH:MM';
    GetEnterprises;
    IsNewSetup := (Enterprises.Count = 0);
    if IsNewSetup then
      Result := DlgDatabaseExecute;
    if (Result <> mrOK) then
      Halt;
    EnterpriseCurrent := EnterpriseDefault;
    try
      Open;
    except
      Showmessage('Problem opening database please check the settings');
      { debug
      ShowMessage(SQLConnection.Params.Text);
      ShowMessage('Drivername = '+SQLConnection.DriverName);
      ShowMessage('LibraryName = '+SQLConnection.LibraryName);
      ShowMessage('VendorLib = '+SQLConnection.VendorLib);
      }
      if (DlgDatabaseExecute = mrOK) then
        Open;
    end;
    if not Active then
      Halt;
    FN := ChangeFileExt(Application.ExeName, 'Update.SQL');
    // Run Database Updates;
    if FileExists(FN) then
    begin
      S := '';
      AssignFile(F, FN);
      try
        Reset(F);
        repeat
          ReadLn(F, S1);
          S := S + S1 + ' ';
        until Eof(F);
      finally CloseFile(F);
      end;
      SQLExecute(S);
      DeleteFile(FN);
    end;
    PrintShareLayouts := RecallBool('Settings', 'PrintShareLayouts', True,
      False);
    FFolderStoreDocumentImages := RecallString('Settings',
      'FolderStoreDocumentImages', DataFolder + PathDelim + 'images', True);
    if IsNewSetup then //new install
    begin
      Result := DlgSetupWizardExecute;
      Application.ProcessMessages;
      if (Result <> mrOK) then
      begin
        Showmessage('Closing program for resume later..');
        Close;
        Halt;
      end;
    end;
    RecallSection('Users', FUsers, True);
    if (FUsers.IndexOf('DEFAULT') < 0) then
      UserAdd('DEFAULT', '');
    // to do add user login here
    UserName := 'DEFAULT';
    with Groups do
      for I := 0 to Count - 1 do
        Result := Items[I].GetSettings;
  except on E: Exception do
    begin
      Showmessage('An error occurred: ' + E.Message);
      DlgSupportExecute;
      Halt;
    end;
  end;
end;

function TLKHubRecords.GetUserEmailAddr: string;
begin
  if (FUserEmailAddr = '') and not (csLoading in ComponentState) then
    FUserEmailAddr := RecallString(EnterpriseCurrent, 'UserEmailAddr', '',
      False);
  Result := FUserEmailAddr;
end;

function TLKHubRecords.GetUserShowLogon: Boolean;
begin
  Result := True;
  if not (csLoading in ComponentState) then
    Result := RecallBool(EnterpriseCurrent, 'UserShowLogon', True, False);
end;

procedure TLKHubRecords.GroupAdd(AGroup: TLKGroup);
begin
  with FGroups do
    if (IndexOf(AGroup) < 0) then
    begin
      Add(AGroup);
      AGroup.FreeNotification(Self);
    end;
end;

procedure TLKHubRecords.GroupRemove(AGroup: TLKGroup);
var
  I: Integer;
begin
  if Assigned(AGroup) then
    with FGroups do
    begin
      I := IndexOf(AGroup);
      if (I > -1) then
        Delete(I);
    end;
end;

procedure TLKHubRecords.LedgerAdd(ALedger: TLKLedgerBase);
begin
  if (FLedgers.IndexOf(ALedger) < 0) then
    FLedgers.Add(ALedger);
  if ALedger.HasPeriodEndProcedure then
    FHasPeriodEndProcedure := True;
end;

procedure TLKHubRecords.LedgerRemove(ALedger: TLKLedgerBase);
var
  I: Integer;
begin
  if (csDestroying in ComponentState) then
    Exit;
  I := FLedgers.IndexOf(ALedger);
  if (I > -1) then
    FLedgers.Delete(I);
end;

procedure TLKHubRecords.Loaded;
begin
  inherited Loaded;
  AccountsHub := Self;
//  SetSQLConnectionParams;
end;

procedure TLKHubRecords.Notification(AComponent: TComponent; Operation:
    TOperation);
begin
  if (Operation = opRemove) then
  begin
    if (AComponent is TLKGroup) then
      GroupRemove(TLKGroup(AComponent));
  end;
  inherited Notification(AComponent, Operation);
end;

procedure TLKHubRecords.RecordsAdd(ARecords: TLKRecordsBase);
begin
  if (FRecords.IndexOf(ARecords) < 0) then
    FRecords.Add(ARecords);
  if ARecords.HasPeriodEndProcedure then
    FHasPeriodEndProcedure := True;
end;

procedure TLKHubRecords.RecordsRemove(ARecords: TLKRecordsBase);
var
  I: Integer;
begin
  if (csDestroying in ComponentState) then
    Exit;
  I := FRecords.IndexOf(ARecords);
  if (I > -1) then
    FRecords.Delete(I);
end;

procedure TLKHubRecords.SaveTables(TableList: TStringList; const
    FullSQLFileName: string; const SaveDatabase, SaveTableDescr, SaveTables:
    boolean);
var
  myTableList, indexFieldList: TStringList;
  tbl, s, s1, fn: string;
  i, j: Integer;
  isIndexed: Boolean;
  d, m, y: Word;
  SrcTable: TSQLTable;
  f: TextFile;
begin
  myTableList := TStringList.Create;
  indexFieldList := TStringList.Create;
  Screen.Cursor := crHourGlass;
  SrcTable := TSQLTable.Create(Owner);
  assignfile(f, FullSQLFileName);
  rewrite(f);
  try
    if SaveDatabase then
      WriteLn(f, 'CREATE DATABASE IF NOT EXISTS  `' +
        SQLConnection.Params.Values['Database'] + '`;');
    writeLn(f, 'USE `' + SQLConnection.Params.Values['Database'] + '`;');
    if Assigned(TableList) then
      myTableList.AddStrings(TableList)
    else
      SQLConnection.GetTableNames(myTableList);
    SrcTable.SQLConnection := SQLConnection;
    with myTableList do
      for i := 0 to Count - 1 do
      begin
        tbl := Strings[i];
        // lProgress.Caption := tbl;
        with SrcTable do
        begin
          Close;
          TableName := tbl;
          Open;
          FieldDefs.Update;
          IndexDefs.Update;
          indexFieldList.Clear;
          isIndexed := (IndexDefs.Count > 0);
          if isIndexed then
          begin
            s := IndexDefs.Items[0].Fields;
            repeat
              j := pos(';', s); //find out if more than one link field
              if (j > 0) then
              begin
                fn := copy(s, 1, j - 1);
                System.delete(s, 1, j);
              end
              else
                fn := s;
              while (fn[1] = ' ') do
                System.delete(fn, 1, 1); //chop leading spaces
              while (fn[length(fn)] = ' ') do
                System.delete(fn, length(s), 1); //chop trailing spaces
              if (length(fn) > 0) then
                indexFieldList.Add(fn);
            until (j = 0);
          end;
        end;
        if SaveTableDescr then
        begin
          writeln(f, 'DROP TABLE IF EXISTS `' + tbl + '`;');
          writeLn(f, 'CREATE TABLE `' + tbl + '` ( ');
          with SrcTable.FieldDefs do
            for j := 0 to count - 1 do
              with Items[j] do
              begin
                s := '  `' + Name + '` ';
                case DataType of
                  ftFixedChar: s := s + 'CHAR(' + intToStr(Size) + ') ';
                  ftString, ftWideString:
                    case Size of
                      1..15: s := s + 'CHAR(' + intToStr(Size) + ') ';
                      16..255: s := s + 'VARCHAR(' + intToStr(Size) + ') ';
                    else
                      s := s + 'LONGTEXT';
                    end;
                  ftFmtMemo, //  Formatted text memo field
                  ftMemo: s := s + 'LONGTEXT'; //Text memo field
                  ftWord: s := s + 'SMALLINT(6) UNSIGNED';
                    //16-bit unsigned integer field
                  ftSmallint: s := s + 'SMALLINT(6)'; //16-bit integer field
                  ftInteger: s := s + 'INT(11)'; //32-bit integer field
                  ftLargeint: s := s + 'BIGINT(16)'; //  Large integer field
                  ftAutoInc: s := s + 'INT(11) AUTO_INCREMENT';
                    //  Auto-incrementing 32-bit integer counter field
                  ftBoolean: s := s + 'ENUM(''Y'',''N'')'; //  Boolean field
                  ftFloat, // : s := s + 'DOUBLE'; //Floating-point numeric field
                  ftCurrency: s := s + 'DOUBLE';
                    //'DECIMAL(19,4) '; //  Money field
                  ftBCD, //  Binary-Coded Decimal field that can be converted to Currency type without a loss of precision.
                  ftFMTBcd: s := s + 'DECIMAL(15,4)';
                    //Binary-Coded Decimal field that is too large for ftBCD.
                  ftDateTime, // : s := s + 'datetime'; //    Date and time field
                  ftDate: s := s + 'DATETIME'; //  Date field
                  ftTime: s := s + 'TIME'; //  Time field
                  ftTimeStamp: s := s + 'TIMESTAMP';
                    //  Date and time field accessed through dbExpress
                  ftBytes, //  Fixed number of bytes (binary storage)
                  ftVarBytes, //  Variable number of bytes (binary storage)
                  ftGraphic, //  Bitmap field
                  ftParadoxOle, //  Paradox OLE field
                  ftDBaseOle, //  dBASE OLE field
                  ftTypedBinary, //  Typed binary field
                  ftOraBlob, //  BLOB fields in Oracle 8 tables
                  ftOraClob, //  CLOB fields in Oracle 8 tables
                  ftVariant, //  Data of unknown or undetermined type
                  ftInterface, //  References to interfaces (IUnknown)
                  ftIDispatch, //  References to IDispatch interfaces
                  ftGuid, //  globally unique identifier (GUID) values
                  ftArray: s := s + 'BLOB'; //   Array field
                  //ftCursor  Output cursor from an Oracle stored procedure (TParam only)
                  //ftADT  Abstract Data Type field
                  //ftReference  REF field
                  ftBlob, //  Binary Large OBject field
                  ftDataSet: s := s + 'LONGBLOB'; //DataSet field
                end; //case
                if (indexFieldList.IndexOf(Name) > -1) then
                begin
                  s1 := ' NOT NULL default ';
                  case DataType of
                    ftDateTime: s1 := s1 + chr(39) + '0000-00-00 00:00:00' +
                      chr(39);
                    ftString, ftFixedChar, ftWideString:
                      if Size > 255 then
                        s1 := ''
                      else
                        s1 := s1 + chr(39) + chr(39);
                    ftMemo: s1 := '';
                  else
                    s1 := s1 + chr(39) + '0' + chr(39);
                  end;
                end
                else
                begin
                  s1 := ' default NULL';
                  case DataType of
                    ftString, ftFixedChar, ftWideString:
                      begin
                        if Size > 255 then
                          s1 := '';
                      end;
                    ftMemo: s1 := '';
                  end;
                end;
                writeLn(f, s + s1 + ',');
              end;
          if isIndexed then
          begin
            s := ' PRIMARY KEY (';
            with indexFieldList do
            begin
              s := s + '`' + Strings[0] + '`';
              for j := 1 to count - 1 do
                s := s + ',`' + Strings[j] + '`';
            end;
            writeLn(f, s + ')');
          end;
          s := '  ) TYPE=';
          if (Pos('LEDGER', uppercase(tbl)) > 0) then
            s := s + 'InnoDB ;'
          else
            s := s + 'MyISAM ;';
          writeln(f, s);
        end;
        if SaveTables then
          with SrcTable do
            if FindFirst then
              repeat
                s := 'INSERT INTO `' + tbl + '` VALUES (';
                if Fields[0].IsNull then
                  s1 := 'null'
                else
                begin
                  case Fields[0].DataType of
                    ftDate:
                      begin
                        DecodeDate(Fields[0].AsDateTime, y, m, d);
                        s1 := IntToStr(y) + '-' + IntToStr(m) + '-' +
                          IntToStr(d);
                      end;
                    ftBoolean: if Fields[0].asBoolean then
                        s1 := 'Y'
                      else
                        s1 := 'N';
                  else
                    s1 := Fields[0].AsString;
                  end; // case
                  s1 := '"' + s1 + '"';
                end; //else
                s := s + s1;
                for j := 1 to FieldDefs.Count - 1 do
                begin
                  if Fields[j].IsNull then
                    s1 := 'null'
                  else
                  begin
                    case Fields[j].DataType of
                      ftDate:
                        begin
                          DecodeDate(Fields[j].AsDateTime, y, m, d);
                          s1 := IntToStr(y) + '-' + IntToStr(m) + '-' +
                            IntToStr(d);
                        end;
                      ftBoolean: if Fields[j].asBoolean then
                          s1 := 'Y'
                        else
                          s1 := 'N';
                    else
                      s1 := Fields[j].AsString;
                    end; //case
                    s1 := '"' + s1 + '"';
                  end; // else
                  s := s + ',' + s1;
                end;
                s := s + ') ;';
                writeln(f, s);
                Application.ProcessMessages;
              until not FindNext;
        //ProgressBar1.StepIt;
        Application.ProcessMessages;
      end;
    showmessage('Complete!');
  finally
    myTableList.Free;
    indexFieldList.Free;
    closefile(f);
    // ProgressBar1.Position := 0;
    Screen.Cursor := crDefault;
    with SrcTable do
    begin
      Close;
      Free;
    end;
  end;
end;

function TLKHubRecords.SequenceGet(const Description: string): Integer;
begin
  Result := RecallInteger('Sequence', Description, 1, True);
end;

procedure TLKHubRecords.SequenceSet(const Description: string; Value: integer);
begin
  RememberInteger('Sequence', Description, Value, True);
end;

procedure TLKHubRecords.SetActive(Value: Boolean);
var
  I: Integer;
begin
  if (csLoading in ComponentState) or (Value = FActive) then
    Exit;
  FDataTables.Clear;
  if not Assigned(SQLConnection)then
      Exit;
  if Value then
  begin
    DoBeforeOpen;
    SQLConnection.Connected := True;
    SQLConnection.GetTableNames(FDataTables, False);
  end
  else
  begin
    with FGroups do
      for I := 0 to Count - 1 do
        Items[I].Close;
  end;
  inherited SetActive(SQLConnection.Connected);
  if Value then
    DoAfterOpen
  else
    SQLConnection.Connected := False;
end;

procedure TLKHubRecords.SetAdministratorPassword(const Value: string);
begin
  if (csLoading in ComponentState) or not Active then
    Exit;
  RememberString(EnterpriseCurrent, 'AdministratorPassword', Crypt(Value, 25,
    False), True);
end;

procedure TLKHubRecords.SetContainedComponentNames;
begin
  inherited SetContainedComponentNames;
  FDiary.Name := Name + 'Diary';
  FSystemSettings.Name := Name + 'SystemSettings';
end;

procedure TLKHubRecords.SetDatabaseHost(const Value: string);
begin
  if (FDatabaseHost = Value) or (Value = '') then
    Exit;
  FDatabaseHost := Value;
  if not (csLoading in ComponentState) then
  begin
    RememberStringIni('Settings', 'DatabaseHost', Value, True);
    SetSQLConnectionParams;
  end;
end;

procedure TLKHubRecords.SetDBMS(Value: TLKSupportedDatabaseType);
begin
  if (FDBMS = Value) then
    Exit;
  FDBMS := Value;
  if not (csLoading in ComponentState) then
  begin
    RememberIntegerIni('Settings', 'DBMS', Ord(FDBMS), True);
    SetSQLConnectionParams;
  end;
end;

procedure TLKHubRecords.SetDBUserName(const Value: string);
begin
  if (FDBUserName = Value) or (Value = '') then
    Exit;
  FDBUserName := Value;
  if not (csLoading in ComponentState) then
  begin
    RememberStringIni('Settings', 'DBUserName', FDBUserName, True);
    SetSQLConnectionParams;
  end;
end;

procedure TLKHubRecords.SetDBUserPassword(const Value: string);
begin
  if (FDBUserPassword = Value) or (Value = '') then
    Exit;
  FDBUserPassword := Value;
  if not (csLoading in ComponentState) then
  begin
    RememberCryptIni('Settings', 'DBUserPassword', FDBUserPassword, 25, True);
    SetSQLConnectionParams;
  end;
end;

procedure TLKHubRecords.SetDoBackup(Value: Boolean);
begin
  if not (csLoading in ComponentState) then
    RememberBool(EnterpriseCurrent, 'DoBackup', Value, False);
end;

procedure TLKHubRecords.SetDoBackupRemovable(Value: Boolean);
begin
  if not (csLoading in ComponentState) then
    RememberBool(EnterpriseCurrent, 'DoBackupRemovable', Value, False);
end;

procedure TLKHubRecords.SetEnterpriseAccountsEmail(const Value: string);
begin
  RememberString(EnterpriseCurrent, 'Email', Value, True);
end;

procedure TLKHubRecords.SetEnterpriseAccountsFax(const Value: string);
begin
  RememberString(EnterpriseCurrent, 'Fax', Value, True);
end;

procedure TLKHubRecords.SetEnterpriseAccountsTelephone(const Value: string);
begin
  RememberString(EnterpriseCurrent, 'Telephone', Value, True);
end;

procedure TLKHubRecords.SetEnterpriseAccountsYearCurrent(const Value: string);
begin
  if csLoading in ComponentState then
    Exit;
  if (FEnterpriseAccountsYearCurrent = Value) then
    Exit;
  FEnterpriseAccountsYearCurrent := Value;
  if (Value > '') then
  begin
    FDatabaseHost := '';
    SetSQLConnectionParams;
  end;
end;

procedure TLKHubRecords.SetEnterpriseAccountsYearDefault(const Value: string);
begin
  if not (csLoading in ComponentState) then
    RememberStringIni(EnterpriseCurrent, 'EnterpriseAccountsYearDefault',
      Value, True);
end;

procedure TLKHubRecords.SetEnterpriseAddress1(const Value: string);
begin
  RememberString(EnterpriseCurrent, 'Address1', Value, True);
end;

procedure TLKHubRecords.SetEnterpriseAddress2(const Value: string);
begin
  RememberString(EnterpriseCurrent, 'Address2', Value, True);
end;

procedure TLKHubRecords.SetEnterpriseAddress3(const Value: string);
begin
  RememberString(EnterpriseCurrent, 'Address3', Value, True);
end;

procedure TLKHubRecords.SetEnterpriseAddress4(const Value: string);
begin
  RememberString(EnterpriseCurrent, 'Address4', Value, True);
end;

procedure TLKHubRecords.SetEnterpriseAddressPostcode(const Value: string);
begin
  RememberString(EnterpriseCurrent, 'POSTCODE', Value, True);
end;

procedure TLKHubRecords.SetEnterpriseCurrent(const Value: string);
begin
  if (FEnterpriseCurrent = Value) then
    Exit;
  FEnterpriseCurrent := Value;
  if csLoading in ComponentState then
    Exit;
  if (Value = '') then
    Exit;
  FDatabaseHost := '';
  EnterpriseAccountsYearCurrent := EnterpriseAccountsYearDefault;
end;

procedure TLKHubRecords.SetEnterpriseDefault(const Value: string);
begin
  if not (csLoading in ComponentState) then
    RememberStringIni('Settings', 'EnterpriseDefault', Value, True);
  FEnterpriseDefault := Value;
end;

procedure TLKHubRecords.SetEnterpriseName(const Value: string);
begin
  RememberString(EnterpriseCurrent, 'EnterpriseName', Value, True);
end;

procedure TLKHubRecords.SetEnterprises(Value: TStrings);
var
  I: Integer;
begin
  if csLoading in ComponentState then
    Exit;
  FEnterprises.Assign(Value);
  if Assigned(Value) then
  begin
    ForgetIni('Enterprises', True);
    TStringList(FEnterprises).Sorted := True;
    for I := 0 to FEnterprises.Count - 1 do
      RememberStringIni('Enterprises', FEnterprises[I], '', True);
  end;
end;

procedure TLKHubRecords.SetEnterpriseYears(Value: TStrings);
var
  I: Integer;
begin
  if csLoading in ComponentState then
    Exit;
  FEnterpriseYears.Assign(Value);
  ForgetIni(EnterpriseCurrent + 'EnterpriseYears', True);
  TStringList(FEnterpriseYears).Sorted := True;
  for I := 0 to FEnterpriseYears.Count - 1 do
    RememberStringIni(EnterpriseCurrent + 'EnterpriseYears',FEnterpriseYears[I],
      '', True);
end;

procedure TLKHubRecords.SetFilesBackupDate(Value: TDateTime);
begin
  if not (csLoading in ComponentState) then
    RememberDate(EnterpriseCurrent, 'LastBackupDate', Value, True);
end;

procedure TLKHubRecords.SetFolderBackup(const Value: string);
begin
  if (Value = '') or (Value = FFolderBackup) then
    Exit;
  if not (csLoading in ComponentState) then
    RememberString(EnterpriseCurrent, 'FolderBackup', Value, False);
  FFolderBackup := Value;
end;

procedure TLKHubRecords.SetFolderBackupRemovable(const Value: string);
begin
  if (Value = '') or (Value = FFolderBackupRemovable) then
    Exit;
  if not (csLoading in ComponentState) then
    RememberString(EnterpriseCurrent, 'FolderBackupRemovable', Value, False);
  FFolderBackupRemovable := Value;
end;

procedure TLKHubRecords.SetFolderLocalSettings(const Value: string);
var
  I: Integer;
begin
  inherited SetFolderLocalSettings(Value);
  for I := 0 to FGroups.Count - 1 do
    FGroups[I].FolderLocalSettings := FolderLocalSettings;
end;

procedure TLKHubRecords.SetFolderSharedSettings(const Value: string);
var
  I: Integer;
begin
  inherited SetFolderSharedSettings(Value);
  for I := 0 to FGroups.Count - 1 do
    FGroups[I].FolderSharedSettings := FolderSharedSettings;
end;

procedure TLKHubRecords.SetImages(Value: TCustomImageList);
var
  i: Integer;
begin
  inherited SetImages(Value);
  for i := 0 to FGroups.Count - 1 do
    FGroups[i].Images := Value;
end;

procedure TLKHubRecords.SetPrintShareLayouts(const Value: Boolean);
var
  I: Integer;
begin
  if not (Value = PrintShareLayouts) then
  begin
    RememberBool('Settings', 'PrintShareLayouts', Value, False);
    inherited SetPrintShareLayouts(Value);
  end;
  for I := 0 to FGroups.Count - 1 do
    FGroups[I].PrintShareLayouts := Value;
end;

procedure TLKHubRecords.SetSQLConnection(const Value: TSQLConnection);
var
  I: Integer;
begin
  inherited SetSQLConnection(Value);
  with FGroups do
    for I := 0 to Count - 1 do
      Items[I].SQLConnection := Value;
end;

procedure TLKHubRecords.SetSQLConnectionParams;
var
  WasConnected: Boolean;
begin
  WasConnected := Active;
  Close;
  if Assigned(SQLConnection) then
    with SQLConnection do
    begin
      LoadParamsOnConnect := False;
      Params.Clear;
      case DBMS of
        sdbLocalFirebird:
        begin
          DriverName := 'FirebirdLocal';
          Params.Values['Database'] := DataFolder + PathDelim
            + EnterpriseCurrent + EnterpriseAccountsYearCurrent + '.FDB';
          VendorLib := 'fbembed.dll';
        end;
        sdbFirebird:
        begin
          DriverName := 'Firebird';
          Params.Values['Database'] := DatabaseHost + ':' + EnterpriseCurrent +
            EnterpriseAccountsYearCurrent;
          VendorLib := 'fbclient.dll';
        end;
        sdbMSSQL:
        begin
          DriverName := 'MSSQL';
          VendorLib := 'sqlcli.dll';
        end;
        sdbMSSQL9:
        begin
          DriverName := 'MSSQL9';
          VendorLib := 'sqlcli9.dll';
        end;
      end;
      case DBMS of
        sdbLocalFirebird, sdbFirebird:
        begin
          Params.Values['DriverUnit'] := 'DBXFirebird';
          Params.Values['DriverPackageLoader'] :=
            'TDBXDynalinkDriverLoader,DbxCommonDriver160.bpl';
          Params.Values['DriverAssemblyLoader'] :=
            'Borland.Data.TDBXDynalinkDriverLoader,Borland.Data.DbxCommonDriver,'
            +'Version=16.0.0.0,Culture=neutral,PublicKeyToken=91d62ebb5b0d1b1b';
          Params.Values['MetaDataPackageLoader'] :=
            'TDBXFirebirdMetaDataCommandFactory,DbxFirebirdDriver160.bpl';
          Params.Values['MetaDataAssemblyLoader'] :=
            'Borland.Data.TDBXFirebirdMetaDataCommandFactory,Borland.Data.'
            +'DbxFirebirdDriver,Version=16.0.0.0,Culture=neutral,'
            +'PublicKeyToken=91d62ebb5b0d1b1b';
          Params.Values['GetDriverFunc'] := 'getSQLDriverINTERBASE';
          Params.Values['RoleName'] := 'RoleName';
          Params.Values['SQLDialect'] := '3';
          Params.Values['CommitRetain'] := 'False';
          Params.Values['ServerCharSet'] := 'ISO8859_1';
          Params.Values['WaitOnLocks'] := 'True';
          Params.Values['Firebird TransIsolation'] := 'ReadCommited';
          Params.Values['Trim Char'] := 'False';
          LibraryName := 'dbxfb.dll';
        end;
        sdbMSSQL, sdbMSSQL9:
        begin
          Params.Values['SchemaOverride'] := '%.dbo';
          Params.Values['DriverUnit'] := 'DBXMSSQL';
          Params.Values['DriverPackageLoader'] :=
            'TDBXDynalinkDriverLoader,DBXCommonDriver160.bpl';
          Params.Values['DriverAssemblyLoader'] := 'Borland.Data.TDBXDynalinkDr'
            +'iverLoader,Borland.Data.DbxCommonDriver,Version=16.0.0.0,Culture='
            +'neutral,PublicKeyToken=91d62ebb5b0d1b1b';
          Params.Values['MetaDataPackageLoader'] := 'TDBXMsSqlMetaDataCommandFa'
            +'ctory,DbxMSSQLDriver160.bpl';
          Params.Values['MetaDataAssemblyLoader'] := 'Borland.Data.TDBXMsSqlMet'
            +'aDataCommandFactory,Borland.Data.DbxMSSQLDriver,Version=16.0.0.0,'
            +'Culture=neutral,PublicKeyToken=91d62ebb5b0d1b1b';
          Params.Values['GetDriverFunc'] := 'getSQLDriverMSSQL';
          Params.Values['HostName'] := 'ServerName';
          Params.Values['Database'] :=
            EnterpriseCurrent + EnterpriseAccountsYearCurrent;
          Params.Values['ErrorResourceFile'] := '';
          Params.Values['IsolationLevel'] := 'ReadCommitted';
          Params.Values['OS Authentication'] := 'False';
          Params.Values['Prepare SQL'] := 'False';
        end;
      end;
      Params.Values['User_Name'] := DBUserName;
      Params.Values['Password'] := DBUserPassword;
      Params.Values['LocaleCode'] := '0000';
      Params.Values['BlobSize'] := '-1';
      // Showmessage(Params.Text);
    end;
  Active := WasConnected;
end;

procedure TLKHubRecords.SetUserEmailAddr(const Value: string);
begin
  if FUserEmailAddr <> Value then
  begin
  if not (csLoading in ComponentState) then
      RememberString('Data', 'UserEmailAddr', Value, False);
  FUserEmailAddr := Value;
  end;
end;

procedure TLKHubRecords.SetUserName(const Value: string);
var
  I: Integer;
begin
  inherited SetUserName(Value);
  with SystemSettings do
  begin
    Filter := 'USER_ID = ' + LKSQLStrDelimiter + '~' + LKSQLStrDelimiter +
      ' OR USER_ID = ' + LKSQLStrDelimiter + UserName + LKSQLStrDelimiter;
    Filtered := True;
  end;
  // Set UserName for all system components
  with FGroups do
    for I := 0 to Count - 1 do
      Items[I].UserName := Value;
end;

procedure TLKHubRecords.SetUsers(Value: TStrings);
begin
  FUsers.Assign(Value);
end;

procedure TLKHubRecords.SetUserShowLogon(Value: Boolean);
begin
  if not (csLoading in ComponentState) then
    RememberBool('Settings', 'UserShowLogon', Value, False);
end;

procedure TLKHubRecords.UserAdd(const AUserName, APassword: string);
var
  c, s, i, v: string;
  bm: TBookMark;
begin
  FUsers.Add(AUserName);
  Screen.Cursor := crHourglass;
  try
    RememberString('Users', AUserName, Crypt(APassword, 25, False), True);
    with SystemSettings do
    begin
      s := 'USER_ID = ' + LKSQLStrDelimiter + 'DEFAULT' + LKSQLStrDelimiter;
      Filter := s;
      Filtered := True;
      OKToInsert := True;
      with TblMaster do
        if FindFirst then
          repeat
            bm := GetBookMark;
            try
              c := FieldByName('COMPONENT').AsString;
              s := FieldByName('SECTION').AsString;
              i := FieldByName('ITEM').AsString;
              v := FieldByName('SETTING').AsString;
              try
                InsertRecord([AUserName, c, s, i, v]);
              except;
              end;
              GotoBookmark(bm);
            finally FreeBookMark(bm);
            end;
          until not FindNext;
      ApplyUpdates(-1);
    end;
  finally
    Screen.Cursor := crDefault;
    with SystemSettings do
    begin
      OKToInsert := False;
      Filter := '';
      Filtered := False;
    end;
    UserName := FUserName;
  end;
end;

procedure TLKHubRecords.UserDelete(const AUserName: string);
var
  i: Integer;
begin
  if (AUserName = 'DEFAULT') then
  begin
    Showmessage('You cannot delete the DEFAULT user!');
    Exit;
  end;
  i := FUsers.IndexOf(AUserName);
  if (i < 0) then
  begin
    Showmessage('User name ' + AUserName + ' does not exist');
    Exit;
  end;
  FUsers.Delete(i);
  Screen.Cursor := crHourglass;
  try
    with SystemSettings do
    begin
      with TblMaster do
        if Locate('USER_ID;COMPONENT;SECTION;ITEM',
          VarArrayOf(['~', NameFriendly, 'Users', AUserName]), []) then
          Delete;
      ApplyUpdates(-1);

      Filter := 'USER_ID = ' + LKSQLStrDelimiter + AUserName +
        LKSQLStrDelimiter;
      Filtered := True;
      with TblMaster do
        while FindLast do
          Delete;
      ApplyUpdates(-1);
    end;
  finally
    Screen.Cursor := crDefault;
    UserName := FUserName;
  end;
end;

procedure TLKHubRecords.Validate(AValMsgList: TStrings);
begin
end;

initialization

  RegisterClasses([
    TLKActionList,
      TLKTableMaster,
      TLKTableDetail,
      TLKGroupComponent,
      TLKHubRecordsDiary,
      TLKHubSystemSettings,
      TStringField,
      TNumericField,
      TIntegerField,
      TSmallintField,
      TLargeintField,
      TWordField,
      TAutoIncField,
      TFloatField,
      TCurrencyField,
      TBooleanField,
      TDateTimeField,
      TSQLTimeStampField,
      TDateField,
      TTimeField,
      TBinaryField,
      TBytesField,
      TVarBytesField,
      TBCDField,
      TBlobField,
      TMemoField,
      TGraphicField,
      TObjectField,
      TADTField,
      TArrayField,
      TDataSetField,
      TReferenceField,
      TAggregateField,
      TFMTBCDField
      ]);

end.
